Package: MosaiClusteR
Type: Package
Title: An Umbrella Framework for Multi-Source and Multi-Omics
        Clustering
Version: 0.1.1
Authors@R: c(
    person("Bernard Isekah", "Osang'ir", email = "bernard.osangir@sckcen.be",
           role = c("aut", "cre"), comment = c(ORCID = "0000-0002-5557-3602")),
    person("Marijke", "Van Moerbeke", role = "aut"),
    person("Ziv", "Shkedy", role = "ctb"),
    person("Surya", "Gupta", role = "ctb"),
    person("Jürgen", "Claesen", role = "ctb"))
Description: An umbrella framework ("MoSaIC:" Multi-Omics Similarity Aggregation and Integrative Clustering in R) that unifies a large collection
    of multi-source / multi-omics clustering methodologies behind a single,
    consistent list-of-matrices interface. It spans five integration
    paradigms - direct, similarity-based, graph-based, voting-based consensus,
    and hierarchy-based - and bundles a complete downstream workflow for
    method comparison and evaluation. The package features the multi-source
    the ability to compare many algorithms on the same footing, a data-nugget
    based feature-weighting scheme as a robust, big-data-friendly alternative to
    variance weighting, and a downstream suite for cluster characterisation,
    visualisation and biological interpretation.
License: CC BY-SA 4.0
Encoding: UTF-8
LazyData: false
Depends: R (>= 4.0)
Imports: stats, utils, methods, graphics, grDevices, cluster,
        fastcluster, matrixStats, FD, ade4, e1071, gtools, data.table,
        Rcpp, Rdpack
LinkingTo: Rcpp
Suggests: SNFtool, datanugget, WCluster, LUCIDus, igraph, lsa,
        analogue, FactoMineR, pls, circlize, gplots, dendextend,
        plotrix, limma, MLP, biomaRt, org.Hs.eg.db, Biobase, a4Core,
        a4Base, plyr, genefilter, ggplot2, gridExtra, uwot, Rtsne,
        MOFAdata, testthat (>= 3.0.0), knitr, rmarkdown
RdMacros: Rdpack
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://github.com/lucp12891/MosaiClusteR
BugReports: https://github.com/lucp12891/MosaiClusteR/issues
NeedsCompilation: yes
Config/roxygen2/version: 8.0.0
Packaged: 2026-08-19 06:08:10 UTC; hp
Author: Bernard Isekah Osang'ir [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5557-3602>),
  Marijke Van Moerbeke [aut],
  Ziv Shkedy [ctb],
  Surya Gupta [ctb],
  Jürgen Claesen [ctb]
Maintainer: Bernard Isekah Osang'ir <bernard.osangir@sckcen.be>
Repository: CRAN
Date/Publication: 2026-08-21 15:00:14 UTC

# Generated by roxygen2: do not edit by hand

S3method(print,data_nugget)
export(ABCdeep.SingleInMultiple)
export(ABCdist.SingleInMultiple)
export(ABCpp.SingleInMultiple)
export(ADC)
export(ADEC)
export(BinFeaturesPlot_MultipleData)
export(BinFeaturesPlot_SingleData)
export(BoxPlotDistance)
export(CEC)
export(CVAA)
export(CharacteristicFeatures)
export(ChooseCluster)
export(Cluster)
export(ClusterCols)
export(ClusterPlot)
export(ClusteringAggregation)
export(ColorPalette)
export(ColorsNames)
export(CompareInteractive)
export(ComparePlot)
export(CompareSilCluster)
export(CompareSvsM)
export(ConsensusClustering)
export(ContFeaturesPlot)
export(Cyclogram)
export(DetermineWeight_SilClust)
export(DetermineWeight_SimClust)
export(DiffGenes)
export(DiffGenesSelection)
export(Distance)
export(Distance_v2)
export(EHC)
export(EnsembleClustering)
export(EvidenceAccumulation)
export(FeatSelection)
export(FeaturesOfCluster)
export(FindCluster)
export(FindElement)
export(FindGenes)
export(Geneset.intersect)
export(Geneset.intersectSelection)
export(HBGF)
export(HeatmapPlot)
export(HeatmapSelection)
export(HierarchicalEnsembleClustering)
export(LUCID)
export(LabelCols)
export(LabelPlot)
export(LinkBasedClustering)
export(M_ABCdeep)
export(M_ABCdist)
export(M_ABCdist.WC)
export(M_ABCpp)
export(NEMO)
export(Normalization)
export(PathwayAnalysis)
export(Pathways)
export(PathwaysIter)
export(PathwaysSelection)
export(PlotPathways)
export(PreparePathway)
export(ProfilePlot)
export(ReorderToReference)
export(SNF)
export(SelectnrClusters)
export(SharedComps)
export(SharedGenesPathsFeat)
export(SharedSelection)
export(SharedSelectionLimma)
export(SharedSelectionMLP)
export(SimilarityHeatmap)
export(SimilarityMeasure)
export(TrackCluster)
export(WeightedClust)
export(Whclust)
export(Wkmeans)
export(WonM)
export(characterize_clusters)
export(cluster_agreement)
export(cluster_biology_panel)
export(cluster_k_sweep)
export(cluster_selection_plot)
export(clustering_concordance)
export(compare_clusterings)
export(compare_methods_plot)
export(create_data_nuggets)
export(embedding_plot)
export(f.clustABC.MultiSource)
export(intNMF)
export(mosaic_labels)
export(mosaic_sim)
export(nugget_cluster)
export(nugget_feature_weights)
export(plot_cluster_profile)
export(simulate_weighting_regime)
export(spectral_clustering)
import(cluster)
import(stats)
importFrom(Rcpp,evalCpp)
importFrom(Rdpack,reprompt)
importFrom(matrixStats,colVars)
importFrom(matrixStats,rowMeans2)
importFrom(matrixStats,rowSds)
importFrom(matrixStats,rowVars)
useDynLib(MosaiClusteR, .registration = TRUE)

# MosaiClusteR

<!-- badges: start -->
[![License: CC BY-SA 4.0](https://img.shields.io/badge/License-CC%20BY--SA%204.0-lightgrey.svg)](https://creativecommons.org/licenses/by-sa/4.0/)
[![R >= 4.0](https://img.shields.io/badge/R-%3E%3D4.0-blue)](https://www.r-project.org/)
[![Lifecycle: maturing](https://img.shields.io/badge/lifecycle-maturing-blue.svg)](https://lifecycle.r-lib.org/)
![Status](https://img.shields.io/badge/tests-passing-brightgreen.svg)
<!-- badges: end -->

> **An umbrella framework for multi-source & multi-omics clustering in R.**
> *One interface, many methods, a complete workflow — from raw modalities to interpretable subgroups.*

**MosaiClusteR** — *“**MoSaIC**” = **M**ulti-**O**mics **S**imilarity **A**ggregation and **I**ntegrative **C**lustering in R* — treats every data modality (transcriptome, methylome, proteome, fingerprints, targets, imaging, clinical…) as one **tile** of a larger picture, and assembles those tiles into a single coherent *mosaic* of sample structure. It unifies a large family of integrative clustering algorithms behind **one consistent list‑of‑matrices interface**, and wraps them in a complete analysis framework: preprocessing → single‑source baselines → integration across five paradigms → evaluation & interpretation.

No single algorithm is best across all data regimes, so MosaiClusteR gathers many methods on the same footing and gives you the tooling to *choose between them*: cluster‑number selection, agreement metrics, visual comparison, cluster characterisation, and — for omics — pathway and gene‑set interpretation. It also introduces a **data‑nugget feature‑weighting** scheme that offers a robust, Big‑Data‑friendly alternative to variance weighting.

---

## Table of contents

- [Why MosaiClusteR](#why-mosaicluster)
- [Installation](#installation)
- [60‑second quick start](#60-second-quick-start)
- [The MoSaIC framework of use](#the-mosaic-framework-of-use)
- [Method catalogue](#method-catalogue) — *every method, and when to use it*
- [Spotlight: M‑ABC and data nuggets](#spotlight-m-abc-and-data-nuggets)
- [Evaluation & comparison toolkit](#evaluation--comparison-toolkit)
- [Datasets](#datasets)
- [Design principles](#design-principles)
- [Citation](#citation)
- [References](#references)

---

## Why MosaiClusteR

Multi‑omics and other multi‑source studies measure the *same objects* (patients, tumours, compounds, cells) through several heterogeneous lenses. No single modality tells the whole story, and the many integration algorithms in the literature live in scattered packages with incompatible inputs and outputs. MosaiClusteR fixes this:

- **One input format.** Every method takes a `List` of matrices over the *same shared objects*, with **objects (samples) in rows and features in columns** (`n × m`) — the same convention across the entire package, including the M‑ABC family. Distance‑matrix inputs (`type = "dist"`) are orientation‑free.
- **One output contract.** Every method returns at least `DistM` (a dissimilarity matrix) and `Clust` (a hierarchical‑clustering object), so results are directly comparable and composable.
- **Five paradigms, one roof.** Direct, similarity‑based, graph‑based, voting‑based consensus, and hierarchy‑based integration — pick the right tool without leaving the package.
- **A full workflow, not just algorithms.** Normalisation, distances, partition extraction, agreement metrics (ARI/NMI/Jaccard), and method comparison are built in.

---

## Installation

```r
# install.packages("remotes")
remotes::install_github("lucp12891/MosaiClusteR")
```

MosaiClusteR ships C++ kernels for the M‑ABC family, so a C/C++ toolchain is
needed to build from source (on Windows install
[Rtools](https://cran.r-project.org/bin/windows/Rtools/); on macOS the Xcode
command‑line tools). Every method has a pure‑R fallback, so the package still
runs if a kernel is unavailable. A few methods rely on *suggested* packages that
are only loaded when you call them:

```r
# optional, enables extra methods / distances
install.packages(c("SNFtool", "datanugget", "WCluster", "lsa", "analogue", "FactoMineR"))
```

| If you want…                              | Install (Suggests)          |
|-------------------------------------------|-----------------------------|
| Similarity Network Fusion (`SNF`)         | `SNFtool`                   |
| The reference data‑nugget engine          | `datanugget`, `WCluster`    |
| Cosine / χ² / MCA distances               | `lsa`, `analogue`, `FactoMineR` |

---

## 60‑second quick start

```r
library(MosaiClusteR)
data(mosaic_toy)                       # two sources, 60 shared objects, 3 planted groups
L     <- mosaic_toy$List               # a list of matrices, objects in rows
truth <- mosaic_toy$truth

## ---- Integrate: every method shares one interface — swap to change paradigm
fit <- SNF(L, type = "data", distmeasure = c("euclidean", "euclidean"),
           normalize = c(FALSE, FALSE), method = list(NULL, NULL))

labels <- mosaic_labels(fit, k = 3)    # pull a flat partition from any fitted result
cluster_agreement(labels, truth)       # ARI / NMI / Jaccard / purity vs ground truth

## ---- Same score, a different paradigm — e.g. a neighbourhood method
cluster_agreement(NEMO(L, k = 3)$cluster, truth)

## ---- Compare two integrations directly
compare_clusterings(fit$DistM,
                    ADC(L, distmeasure = "euclidean", normalize = FALSE)$DistM, NC = 3)
```

That is the whole loop: **integrate → extract → evaluate → compare** — and the
same three lines work for any of the methods below.

---

## The MoSaIC framework of use

MosaiClusteR is organised as a four‑layer pipeline. You can enter at any layer,
and every layer speaks the same `List → {DistM, Clust}` language.

<p align="center">
  <img src="man/figures/framework.svg" width="680"
       alt="The MoSaIC framework: a four-layer pipeline — (1) preprocessing with Normalization() and Distance(), (2) single-source baselines, (3) the integration engine spanning five paradigms, and (4) evaluation and interpretation with mosaic_labels(), cluster_agreement() and compare_clusterings().">
</p>

**Choosing a layer‑3 method** (decision shortcut):

- *Want a fast, transparent baseline?* → **Direct** (`ADC`) or **Weighted** (`WeightedClust`).
- *Sources are noisy and complementary, genomic scale?* → **Similarity** (`SNF`).
- *Want stability under resampling / a novel robust ensemble?* → **Voting consensus** (`M_ABCpp`, `CEC`, `EvidenceAccumulation`).
- *Care about the hierarchy / dendrogram structure itself?* → **Hierarchy** (`HEC`).
- *Have very large N or heavy‑tailed data?* → use **`weighting = "nugget"`** inside the M‑ABC family.

---

## Method catalogue

> **Legend** — a named function is implemented in this package · † requires an
> external graph‑partitioning executable (METIS) or MATLAB, not bundled · "—"
> marks methods interoperable with, or on the roadmap via, a reference package.
> All methods share the `List → {DistM, Clust}` contract and take the same
> objects‑in‑rows input.

### 1 · Direct clustering — *combine first, then cluster*

| Method | Function | What it does | When it’s useful | Ref |
|---|---|---|---|---|
| **ADC** — Aggregated Data Clustering | `ADC` | Concatenate all tiles `[D₁│…│Dₗ]`, one distance, one hierarchy | Simplest baseline; when modalities are commensurate and you want a transparent reference | Fodeh 2013 |
| **ADEC** — Aggregated Data *Ensemble* Clustering | `ADEC` | ADC + resampling of features / cut‑points → co‑clustering matrix | When ADC is unstable and you want a robustness layer | Fodeh 2013 |

### 2 · Similarity‑based — *fuse object‑by‑object similarity*

| Method | Function | What it does | When it’s useful | Ref |
|---|---|---|---|---|
| **Weighted Clustering** | `WeightedClust` | Convex combination `Σ wₖ Dₖ` over a weight grid | When you want to *tune and inspect* the trade‑off between sources | Perualila‑Tan 2016 |
| **SNF** — Similarity Network Fusion | `SNF` | Cross‑diffuse per‑source similarity networks to one fused network | Noisy, complementary, genomic‑scale sources; the multi‑omics workhorse | Wang 2014 |
| **WonM** — Weighting on Membership | `WonM` | Consensus co‑membership across many cut‑points *k* | When you don’t want to commit to a single *k* | — |
| **NEMO** — Neighborhood‑based Multi‑Omics | `NEMO` | Locally‑scaled kNN affinities averaged over the omics measuring each pair → spectral clustering | When samples are **missing some modalities** (no imputation needed) | Rappoport 2019 |
| **Spectral clustering** | `spectral_clustering` | Normalised‑Laplacian embedding + k‑means on any affinity | Final clusterer for fused/similarity matrices (SNF, NEMO, Spectrum) | Ng 2002 |
| **ab‑SNF** | — | SNF with association/variance feature weights | When a few features carry the signal and should drive the network | Zhang 2022 |
| **CIMLR / rMKL‑LPP** | — | Multi‑kernel similarity learning | Many heterogeneous kernels; learn their weights | Zhang 2022 |

### 3 · Graph‑based — *partition an ensemble (hyper)graph*

| Method | Function | What it does | When it’s useful | Ref |
|---|---|---|---|---|
| **CSPA / HGPA / MCLA** | `EnsembleClustering` † | Cluster‑ensemble via (hyper)graph partitioning | Combining many base partitions into one robust consensus | Strehl 2002 |
| **HBGF** — Hybrid Bipartite Graph Formulation | `HBGF` | Partition an object↔cluster bipartite graph (SVD + k‑means) | Ensembles where objects and clusters co‑embed naturally | Fern 2004 |
| **Clustering Aggregation** (Balls/Agglo/Furthest) | `ClusteringAggregation` | Minimise pairwise disagreement cost | When you have several partitions and want the “median” one | Gionis 2007 |

### 4 · Voting‑based consensus — *let partitions vote*

| Method | Function | What it does | When it’s useful | Ref |
|---|---|---|---|---|
| **M‑ABC** — Multi‑source Aggregating Bundles of Clusters | `M_ABCpp` | Per‑source bootstrap of *feature bundles* → consensus co‑clustering dissimilarity | High‑dimensional sources, where signal hides in feature subsets; robust by design; supports **data‑nugget weighting** | Amaratunga 2008 |
| **M‑ABCdist** | `M_ABCdist` | M‑ABC that accumulates *distances* (keeps geometry) then fuses | When you want continuous dissimilarities rather than co‑clustering counts | Amaratunga 2008 |
| **M‑ABCdeep** — deep‑learning ABC | `M_ABCdeep` / `ABCdeep.SingleInMultiple` | M‑ABC whose base clustering is a neural autoencoder embedding + latent clustering; self‑contained R network, no Python/torch | Nonlinear/complex per‑source structure; when a learned embedding beats a raw distance | Amaratunga 2008 |
| **CEC** — (Consensus) Ensemble Clustering | `CEC` | Incidence accumulation across sources & cut‑points | A solid, weight‑aware consensus baseline | Fodeh 2013 |
| **CVAA / W‑CVAA** | `CVAA` | Cumulative voting aggregation (and weighted) | Aligning and merging many partitions by voting | Saeed 2012/2014 |
| **IVC / IPVC / IPC** | `ConsensusClustering` | Iterative (probabilistic) voting consensus | Iteratively refine a consensus labelling | Nguyen 2007 |
| **EA** — Evidence Accumulation | `EvidenceAccumulation` | Co‑association matrix across partitions (SL / SL‑agnes / MST) | Classic, parameter‑light consensus | Fred 2005 |
| **CTS / SRS / ASRS** | `LinkBasedClustering` | Link‑based cluster ensembles (CTS/SRS pure‑R; ASRS needs MATLAB) | When pairwise *link* similarity refines the consensus | Iam‑On 2010 |

### 5 · Hierarchy‑based — *consense the trees themselves*

| Method | Function | What it does | When it’s useful | Ref |
|---|---|---|---|---|
| **HEC** — Hierarchical Ensemble Clustering | `HierarchicalEnsembleClustering` | Aggregate cophenetic distances → closest ultrametric (Floyd–Warshall) | When the *dendrogram structure* across sources is the object of interest | Zheng 2014 |
| **EHC** — Ensemble for Hierarchical Clustering | `EHC` † | Graph aggregation of dendrogram association strengths | Graph‑flavoured hierarchical consensus | Hossain 2012 |

### 6 · Model‑based & factor integration

| Method | Function | What it does | When it’s useful | Ref |
|---|---|---|---|---|
| **intNMF** — integrative NMF | `intNMF` | Shared non‑negative basis `W` + per‑omic `Hₖ`; consensus clustering of `W` | Low‑rank, parts‑based integration; non‑negative data | Chalise 2017 |
| **LUCID** | `LUCID` ‡ | Latent clusters jointly from exposures, omics **and** an outcome (quasi‑mediation, EM) | Supervised subtyping where an outcome should shape the clusters | Zhao 2024 |
| **iCluster / iClusterPlus / iClusterBayes** | — | Joint latent‑variable integration, mixed data types, feature selection | Zhang 2022 |
| **moCluster / MOFA / JIVE** | — | Low‑rank / factor decomposition of shared + individual variation | Zhang 2022 |

‡ `LUCID` wraps the **LUCIDus** package (a *Suggests* dependency).

### Data reduction & weighting

| Tool | Function | Role |
|---|---|---|
| **Data nuggets** | `create_data_nuggets`, `nugget_feature_weights` | Compress N observations into M weighted representatives; derive robust **feature weights** for M‑ABC |
| **Data‑nugget clustering** | `nugget_cluster`, `Wkmeans`, `Whclust` | Cluster the compressed representatives directly (weighted k‑means / weighted Ward), then map back to objects — Big‑Data clustering | 

---

## Spotlight: M‑ABC and data nuggets

**M‑ABC** generalises the single‑source *Aggregating Bundles of Clusters* algorithm (Amaratunga et al. 2008) to many heterogeneous sources. For each source it repeats, `numsim` times: bootstrap the samples, draw a **weighted subsample of features**, cluster, and record co‑clustering. Per‑source evidence is accumulated into one consensus dissimilarity, then clustered.

The feature subsample is the heart of the method, and *how features are weighted* matters:

| `weighting` | Score behind the selection probability | Best when |
|---|---|---|
| `"var"` | feature **variance** (classic) | signal lives in high‑variance features |
| `"cv"`  | coefficient of variation | scale‑free relative dispersion |
| `"nugget"` | **data‑nugget between‑group weighted variance** | large N, heavy tails, outliers — robust signal |
| `"equal"` | flat | ablation / no prior |

**Why data nuggets?** A data nugget summarises a group of observations by a
*centre*, a *weight* (how many observations it represents) and a *scale* (its
internal spread). Building nuggets over the samples and asking *“which features
separate the nugget centres?”* yields a feature‑importance score that rewards
real between‑group signal while discounting within‑nugget noise — and it scales
to very large N because it works on `M ≪ N` representatives.

```r
# Inspect the nugget compression and the weights it induces
dn <- create_data_nuggets(t(L[[1]]), max_nuggets = 25)
dn                                   # 60 obs -> 25 nuggets (reduction reported)
w  <- nugget_feature_weights(dn, type = "between")   # robust feature weights

# Drop straight into M-ABC
M_ABCpp(L, weighting = c("nugget", "nugget"),
      nugget_args = list(max_nuggets = 25, seed = 1))
```

If the reference `datanugget` package is installed you can swap in its engine
with `create_data_nuggets(x, engine = "datanugget")`; otherwise the built‑in
native engine is used (no extra dependency).

---

## Evaluation & comparison toolkit

```r
mosaic_labels(fit, k = 3)                 # extract a partition from any result
cluster_agreement(a, b)                   # ARI, NMI, pairwise Jaccard
compare_clusterings(D1, D2, NC = 3)       # cluster two dissimilarities & compare
```

`cluster_agreement()` rule of thumb (ARI): `>0.90` excellent · `0.75–0.90` good
· `0.50–0.75` moderate · `<0.50` weak.

### Downstream analysis & visualisation

The package also ships a full downstream analysis suite (Layer 4). These
guard their heavier dependencies via *Suggests* and raise an informative error
if one is missing.

| Purpose | Functions | Needs |
|---|---|---|
| Cross‑method comparison | `ReorderToReference`, `SimilarityMeasure`, `CompareSilCluster`, `CompareSvsM`, `CompareInteractive`, `SelectnrClusters` | base / `plotrix` |
| Source weighting | `DetermineWeight_SilClust`, `DetermineWeight_SimClust` | base |
| Cluster navigation | `FindCluster`, `FindElement`, `SharedComps`, `TrackCluster`, `ChooseCluster` | base |
| Characterisation | `CharacteristicFeatures`, `FeatSelection`, `FeaturesOfCluster` | base |
| Dendrograms / colours | `ComparePlot`, `Cyclogram`, `ClusterPlot`, `ColorPalette`, `LabelPlot` | `circlize`, `plotrix` |
| Heatmaps / boxplots | `HeatmapPlot`, `HeatmapSelection`, `SimilarityHeatmap`, `BoxPlotDistance` | `gplots`, `ggplot2`, `gridExtra` |
| Feature / profile plots | `BinFeaturesPlot_SingleData`, `BinFeaturesPlot_MultipleData`, `ContFeaturesPlot`, `ProfilePlot` | `plotrix`, `Biobase` |
| Differential expression | `DiffGenes`, `DiffGenesSelection`, `FindGenes` | `limma` |
| Pathway analysis | `PathwayAnalysis`, `Pathways`, `PathwaysIter`, `PathwaysSelection`, `PreparePathway`, `PlotPathways`, `Geneset.intersect` | `MLP`, `biomaRt`, `org.Hs.eg.db` |
| Shared genes/paths/features | `SharedGenesPathsFeat`, `SharedSelection`, `SharedSelectionLimma`, `SharedSelectionMLP` | base / `limma` / `MLP` |

---

## Datasets

- **`mosaic_toy`** — a bundled two‑source example (60 shared samples, 3 planted
  groups) for instant experimentation and the test suite.
- **`mosaic_sim()`** — generate your own multi‑source data with controllable
  group structure, informative‑feature counts and effect size.

The methods are designed for, and have been evaluated on, multi‑omics
benchmarks in the spirit of the **LUCIDus** simulated‑HELIX data, **TCGA**
subtype cohorts, and the **MCF7** compound–fingerprint–transcriptome panel used
in the companion manuscript.

---

## Design principles

1. **Source‑agnostic.** Methods do not care whether a tile is RNA, methylation, fingerprints or clinical scores.
2. **Composable.** A uniform `{DistM, Clust}` return makes any two methods comparable and any result re‑usable.
3. **Honest dependencies.** The core is pure R; heavyweight methods degrade gracefully when an optional package is absent.
4. **Tested.** A `testthat` suite covers the data‑nugget engine, every weighting scheme, the M‑ABC family, distances and metrics.

---

## Citation

```bibtex
@Manual{MosaiClusteR2026,
  title  = {MosaiClusteR: An Umbrella Framework for Multi-Source and
            Multi-Omics Clustering},
  author = {Osang'ir, Bernard Isekah and Van Moerbeke, Marijke and
            Claesen, Jürgen and Gupta, Surya and Cabrera, Javier and
            Amaratunga, Dhammika and Shkedy, Ziv},
  year   = {2026},
  note   = {R package version 0.1.0},
  url    = {https://github.com/lucp12891/MosaiClusteR}
}
```

## References

- Amaratunga, Cabrera & Shkedy (2008). *Exploration and Analysis of DNA Microarray and Other High‑Dimensional Data.* Wiley.
- Cherasia, Cabrera, Fernholz & Fernholz (2023). *Data Nuggets in Supervised Learning.* In *Robust and Multivariate Statistical Methods*, Springer, 429–449.
- Wang et al. (2014). *Similarity Network Fusion…* **Nature Methods** 11:333–337.
- Perualila‑Tan et al. (2016). *Weighted similarity‑based clustering…*
- Fodeh, Punch & Tan (2013). *On unifying multi‑source information…* **Proteins**.
- Zheng, Chen & Zheng (2014). *A novel approach for hierarchical ensemble clustering.* **IEEE TPAMI**.
- Strehl & Ghosh (2002); Fern & Brodley (2004); Gionis et al. (2007); Nguyen & Caruana (2007); Fred & Jain (2005); Iam‑On et al. (2010); Saeed et al. (2012/2014); Hossain et al. (2012).
- Zhang, Zhou, Xu & Liu (2022). *Integrative clustering methods for multi‑omics data.* **WIREs Comput Stat** 14:e1553. *(SNF, NEMO, iCluster family, ab‑SNF, CIMLR review.)*
- Zhao, Jia, Goodrich & Conti (2024). *LUCIDus…* **The R Journal** 16(2).

---

*MosaiClusteR is an open, extensible framework — contributions of new integration methods, distances, and evaluation tools are very welcome.*

## Cluster-comparison and weight-determination utilities.
## Ported verbatim (with R 4.x safety fixes and suggested-package guards) from the
## original MoSaiClusteR project. These functions rely on the package-provided
## Distance(), Normalization(), Cluster(), SimilarityMeasure(),
## ReorderToReference(), ColorsNames() and FindElement().

#' @title Interactive comparison of single and multiple source clustering results
#'
#' @description The function \code{CompareInteractive} produces an interactive plot of the
#' multiple source clustering results in \code{ListM}. By identifying a cluster
#' or a method, a comparison with the single source clustering results in
#' \code{ListS} is shown in a new plotting window.
#' @export CompareInteractive
#' @param ListM A list of the outputs from the multiple source clusterings to be compared.
#' @param ListS A list of the outputs from the single source clusterings to be compared.
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param cols A character vector with the names of the colours. Default is NULL.
#' @param fusionsLogM The fusionsLog parameter for the elements in ListM. Default is FALSE.
#' @param fusionsLogS The fusionsLog parameter for the elements in ListS. Default is FALSE.
#' @param weightclustM The weightclust parameter for the elements in ListM. Default is FALSE.
#' @param weightclustS The weightclust parameter for the elements in ListS. Default is FALSE.
#' @param namesM Optional. Names of the multiple source clusterings. Default is NULL.
#' @param namesS Optional. Names of the single source clusterings. Default is NULL.
#' @param marginsM Optional. Margins to be used for the ListM plot. Default is c(2,2.5,2,2.5).
#' @param marginsS Optional. Margins to be used for the ListS plot. Default is c(8,2.5,2,2.5).
#' @param Interactive Optional. Do you want an interactive plot? Default is TRUE.
#' @param n The number of methods/clusters you want to identify. Default is 1.
#' @return A plot of the comparison of the elements of ListM, on which multiple clusters
#' and/or methods can be identified to compare them to the elements in ListS.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#' data(Colors1)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(fingerprintMat,targetMat)
#'
#' MCF7_W=WeightedClust(List=L,type="data",distmeasure=c("tanimoto","tanimoto"),
#' normalize=c(FALSE,FALSE),method=c(NULL,NULL),weight=seq(1,0,-0.1),weightclust=0.5,
#' clust="agnes",linkage="ward",StopRange=FALSE)
#'
#' ListM=list(MCF7_W)
#' namesM=c(seq(1.0,0.0,-0.1))
#'
#' ListS=list(MCF7_F,MCF7_T)
#' namesS=c("FP","TP")
#'
#' CompareInteractive(ListM,ListS,nrclusters=7,cols=Colors1,fusionsLogM=FALSE,
#' fusionsLogS=FALSE,weightclustM=FALSE,weightclustS=TRUE,namesM,namesS,
#' marginsM=c(2,2.5,2,2.5),marginsS=c(8,2.5,2,2.5),Interactive=TRUE,n=1)
#' }
CompareInteractive<-function(ListM,ListS,nrclusters=NULL,cols=NULL,fusionsLogM=FALSE,fusionsLogS=FALSE,weightclustM=FALSE,weightclustS=FALSE,namesM=NULL,namesS=NULL,marginsM=c(2,2.5,2,2.5),marginsS=c(8,2.5,2,2.5),Interactive=TRUE,n=1){
	if (!requireNamespace("plotrix", quietly=TRUE)) stop("CompareInteractive() requires the suggested package 'plotrix'.")

	MatrixColorsM=ReorderToReference(ListM,nrclusters,fusionsLogM,weightclustM,namesM)

	NamesM=ColorsNames(MatrixColorsM,cols)

	nobsM=dim(MatrixColorsM)[2]
	nmethodsM=dim(MatrixColorsM)[1]

	if(is.null(namesM)){
		for(j in 1:nmethodsM){
			namesM[j]=paste("Method",j,sep=" ")
		}
	}

	similarM=round(SimilarityMeasure(MatrixColorsM),2)

	grDevices::dev.new()
	graphics::par(mar=marginsM)
	plotrix::color2D.matplot(MatrixColorsM,cellcolors=NamesM,show.values=FALSE,axes=FALSE,xlab="",ylab="")
	graphics::axis(1,at=seq(0.5,(nobsM-0.5)),labels=colnames(MatrixColorsM),las=2,cex.axis=0.70)
	graphics::axis(2,at=seq(0.5,(nmethodsM-0.5)),labels=rev(namesM),cex.axis=0.65,las=2)
	graphics::axis(4,at=seq(0.5,(nmethodsM-0.5)),labels=rev(similarM),cex.axis=0.65,las=2)

	if(Interactive==TRUE){
		yseq=c(seq(dim(MatrixColorsM)[1]-0.5,0.5,-1))
		for(i in seq(dim(MatrixColorsM)[1]-0.5,0.5,-1)){
			yseq=c(yseq,rep(i,dim(MatrixColorsM)[2]))
		}
		ids=graphics::identify(x=c(rep(-1,dim(MatrixColorsM)[1]),rep(seq(0.5,dim(MatrixColorsM)[2]-0.5),dim(MatrixColorsM)[1])),y=yseq,n=n,plot=FALSE)

		comparison<-function(id){
			if(id%in%seq(dim(MatrixColorsM)[1])){
				grDevices::dev.new()
				graphics::layout(matrix(c(1,2),nrow=2), heights=c(1,2))
				NamesMSel=NamesM[id,]
				namesMSel=namesM[id]

				graphics::par(mar=marginsS)
				plotrix::color2D.matplot(t(as.matrix(MatrixColorsM[id,])),cellcolors=NamesMSel,show.values=FALSE,axes=FALSE,xlab="",ylab="")
				graphics::axis(2,at=c(0.5),labels=rev(namesMSel),cex.axis=0.65,las=2)
				#axis(4,at=c(0.5),labels=rev(similarSel),cex.axis=0.65,las=2)

				#Find reference for MatrixColorsM
				if(weightclustM==FALSE){
					temp=FindElement("Results",ListM[1])
					if(!(is.null(temp))&length(temp)!=0){
						Ref=list(Clust=temp$Results_1[[1]])
						attr(Ref,'method')<-attributes(ListM[[1]])$method
					}
					else if(length(temp)==0){
						temp=FindElement("Clust",ListM[1])
						Ref=list(Clust=temp$Clust_1)
						attr(Ref,'method')<-attributes(ListM[[1]])$method
					}
					else{
						message('Cannot find a reference for the second plot, try: weightclust=TRUE')
					}
				}
				else{
					Ref=ListM[[1]]
					attr(Ref,'method')<-attributes(ListM[[1]])$method
				}

				L=c(Ref,ListS)
				for(i in 1:length(L)){
					if(i==1){
						attr(L[[1]],'method')<-"Ref"
					}
					else{
						attr(L[[i]],"method")<-attributes(ListS[[i-1]])$method
					}
				}
				MatrixColorsS=ReorderToReference(L,nrclusters,fusionsLogS,weightclustS,names=c("Ref",namesS))
				MatrixColorsS=MatrixColorsS[-1,]
				NamesS=ColorsNames(MatrixColorsS,cols)

				nobs=dim(MatrixColorsS)[2]
				nmethodS=dim(MatrixColorsS)[1]

				if(is.null(namesS)){
					for(j in 1:nmethodS){
						namesS[j]=paste("Method",j,sep=" ")
					}
				}

				similarS=round(SimilarityMeasure(MatrixColorsS),2)

				graphics::par(mar=marginsS)
				plotrix::color2D.matplot(MatrixColorsS,cellcolors=NamesS,show.values=FALSE,axes=FALSE,xlab="",ylab="")
				graphics::axis(1,at=seq(0.5,(nobs-0.5)),labels=colnames(MatrixColorsM),las=2,cex.axis=0.70)
				graphics::axis(2,at=c(seq(0.5,nmethodS-0.5)),labels=rev(namesS),cex.axis=0.65,las=2)
				graphics::axis(4,at=c(seq(0.5,nmethodS-0.5)),labels=rev(similarS),cex.axis=0.65,las=2)
			}
			else{
				SelCluster=t(MatrixColorsM)[id-nrow(MatrixColorsM)]
				Temp=sapply(seq(nrow(MatrixColorsM)),function(i) ncol(MatrixColorsM)*i)
				Row=which(Temp>(id-nrow(MatrixColorsM)))[1]
				Index=which(MatrixColorsM[Row,]!=SelCluster)

				grDevices::dev.new()
				graphics::layout(matrix(c(1,2),nrow=2), heights=c(1,2))
				NamesMSel=NamesM[Row,]
				NamesMSel[Index]="white"
				namesMSel=namesM[Row]

				graphics::par(mar=marginsM)
				plotrix::color2D.matplot(t(as.matrix(MatrixColorsM[Row,])),cellcolors=NamesMSel,show.values=FALSE,axes=FALSE,xlab="",ylab="")
				graphics::axis(2,at=c(0.5),labels=rev(namesMSel),cex.axis=0.65,las=2)

				#Find reference for MatrixColorsM
				if(weightclustM==FALSE){
					temp=FindElement("Results",ListM[1])
					if(!(is.null(temp))&length(temp)!=0){
						Ref=list(Clust=temp$Results_1[[1]])
						attr(Ref,'method')<-attributes(ListM[[1]])$method
					}
					else if(length(temp)==0){
						temp=FindElement("Clust",ListM[1])
						Ref=list(Clust=temp$Clust_1)
						attr(Ref,'method')<-attributes(ListM[[1]])$method
					}
					else{
						message('Cannot find a reference for the second plot, try: weightclust=TRUE')
					}
				}
				else{
					Ref=ListM[[1]]
					attr(Ref,'method')<-attributes(ListM[[1]])$method
				}

				L=c(Ref,ListS)
				for(i in 1:length(L)){
					if(i==1){
						attr(L[[1]],'method')<-"Ref"
					}
					else{
						attr(L[[i]],"method")<-attributes(ListS[[i-1]])$method
					}
				}

				MatrixColorsS=ReorderToReference(L,nrclusters,fusionsLogS,weightclustS,names=c("Ref",namesS))
				MatrixColorsS=MatrixColorsS[-1,]

				IndexS=lapply(seq(nrow(MatrixColorsS)),function(i) which(MatrixColorsS[i,]!=SelCluster))

				NamesS=ColorsNames(MatrixColorsS,cols)
				for(i in 1:nrow(NamesS)){
					NamesS[i,IndexS[[i]]]="white"
				}

				nobs=dim(MatrixColorsS)[2]
				nmethodsS=dim(MatrixColorsS)[1]

				if(is.null(namesS)){
					for(j in 1:nmethodsS){
						namesS[j]=paste("Method",j,sep=" ")
					}
				}

				similarS=round(SimilarityMeasure(MatrixColorsS),2)

				graphics::par(mar=marginsS)
				plotrix::color2D.matplot(MatrixColorsS,cellcolors=NamesS,show.values=FALSE,axes=FALSE,xlab="",ylab="")
				graphics::axis(1,at=seq(0.5,(nobs-0.5)),labels=colnames(MatrixColorsM),las=2,cex.axis=0.70)
				graphics::axis(2,at=c(seq(0.5,nmethodsS-0.5)),labels=rev(namesS),cex.axis=0.65,las=2)
				graphics::axis(4,at=c(seq(0.5,nmethodsS-0.5)),labels=rev(similarS),cex.axis=0.65,las=2)


			}
		}

		plots=sapply(seq(length(ids)),function(i) comparison(ids[i]))

	}

}

#' @title Compares medoid clustering results based on silhouette widths
#'
#' @description The function \code{CompareSilCluster} compares the results of two medoid
#' clusterings. The null hypothesis is that the clustering is identical. A test
#' statistic is calculated and a p-value obtained with bootstrapping.
#' @export CompareSilCluster
#' @param List A list of data matrices. It is assumed the rows are corresponding with the objects.
#' @param type indicates whether the provided matrices in "List" are either data matrices or distance
#' matrices. Type should be one of "data" or "dist".
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Defaults to c("tanimoto","tanimoto").
#' @param normalize Logical. Indicates whether to normalize the distance matrices. Default is c(FALSE, FALSE).
#' @param method A method of normalization. Default is c(NULL,NULL).
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param names The labels to give to the elements in List. Default is NULL.
#' @param nboot Number of bootstraps to be run. Default is 100.
#' @param plottype Should be one of "pdf","new" or "sweave". Default is "new".
#' @param location If plottype is "pdf", a location should be provided here. Default is NULL.
#' @return A plot of the density of the statistic under the null hypothesis. Further a list with two
#' elements: \item{Observed Statistic}{The observed statistical value} \item{P-Value}{The P-value of the
#' obtained statistic retrieved after bootstrapping}.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#'
#' List=list(fingerprintMat,targetMat)
#'
#' Comparison=CompareSilCluster(List=List,type="data",
#' distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),
#' nrclusters=7,names=NULL,nboot=100,plottype="new",location=NULL)
#'
#' Comparison
#' }
CompareSilCluster<-function(List,type=c("data","dist"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),nrclusters=NULL,names=NULL,nboot=100,plottype="new",location=NULL){

	type=match.arg(type)

	if(is.null(names)){
		names=c()
		for(i in 1:length(List)){
			names=c(names,paste("Method",i,sep=" "))
		}
	}

	if(type=="data"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,]
		}
		Dist=lapply(seq(length(List)),function(i) Distance(List[[i]],distmeasure[i],normalize[i],method[i]))
		silwidth=lapply(Dist,function(x) cluster::pam(x,nrclusters)$silinfo$widths)
		names(silwidth)=names

	}
	else{
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,OrderNames]
		}
		Dist=List
		silwidth=lapply(Dist,function(x) cluster::pam(x,nrclusters)$silinfo$widths)
		names(silwidth)=names
	}


	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
		if(plottype=="sweave"){

		}
	}

	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}

	regressioncomb=gtools::permutations(n=length(List),r=2,repeats.allowed=T)

	StatRSq<-function(regressioncomb,silwidth,ordernames,names){

		regressRSq<-function(x,silwidth,ordernames,names){

			i1=x[1]
			i2=x[2]
			L1=silwidth[[i1]][,3][ordernames]
			L2=silwidth[[i2]][,1][ordernames]

			regress<-stats::lm(L1~L2)
			Rsq<-summary(regress)$r.squared
			#names(Rsq)=paste("RSquared_",names[i1],names[i2],sep="_")
			return(Rsq)

			#paste names on this object!!!
		}

		RSqs=apply(regressioncomb,1,function(x) regressRSq(x,silwidth,ordernames,names))

		#for (i in 1:nrow(regressioncomb)){
		#	names(RSqs)[i]=paste("RSquared_",names[regressioncomb[i,1]],names[regressioncomb[i,2]],sep="_")
		#}

		stat=0
		xx=0
		xy=0
		for(i in 1:nrow(regressioncomb)){
			if(regressioncomb[i,1]==regressioncomb[i,2]){
				xx=xx+RSqs[i]
			}
			else{
				xy=xy+RSqs[i]
			}
		}
		stat=abs(xx-xy)  #check this formula with Nolen
		names(stat)=NULL
		return(stat)

	}

	StatRSqObs=StatRSq(regressioncomb,silwidth,ordernames=rownames(Dist[[1]]),names)


	#bootstrapping
	statNULL=c(1:nboot)
	perm.rowscols <- function (D, n)
	{
		s <- sample(1:n)
		D=D[s, s]
		return(D)
	}

	for(i in 1:nboot){
		set.seed(i)
		DistNULL=Dist
		DistNULL[[1]] <- perm.rowscols(DistNULL[[1]],nrow(DistNULL[[1]]))

		silwidthNULL=lapply(DistNULL,function(x) cluster::pam(x,nrclusters)$silinfo$widths)

		statNULL[i]=StatRSq(regressioncomb,silwidthNULL,ordernames=rownames(DistNULL[[1]]),names)

	}

	pval=(sum(abs(statNULL)<=abs(StatRSqObs))+1)/(nboot+1)

	plottypein(plottype,location)
	graphics::plot(stats::density(statNULL),type="l",main="The Density of the Statistic under the H0")
	graphics::abline(v=StatRSqObs)

	out=list()
	out[[1]]=StatRSqObs
	#out[[2]]=statNULL
	out[[2]]=pval
	names(out)=c("Observed Statistic","P-Value")

	return(out)
}

#' @title Comparison of clustering results for the single and multiple source clustering.
#'
#' @description The function \code{CompareSvsM} plots the comparison of the single source
#' clustering results on the left and that of the multiple source clustering results on the
#' right such that a visual comparison is possible.
#' @export CompareSvsM
#' @param ListS A list of the outputs from the single source clusterings to be compared.
#' @param ListM A list of the outputs from the multiple source clusterings to be compared.
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param cols A character vector with the names of the colours. Default is NULL.
#' @param fusionsLogS The fusionslog parameter for the elements in ListS. Default is FALSE.
#' @param fusionsLogM The fusionsLog parameter for the elements in ListM. Default is FALSE.
#' @param weightclustS The weightclust parameter for the elements in ListS. Default is FALSE.
#' @param weightclustM The weightclust parameter for the elements in ListM. Default is FALSE.
#' @param namesS Optional. Names of the single source clusterings. Default is NULL.
#' @param namesM Optional. Names of the multiple source clusterings. Default is NULL.
#' @param margins Optional. Margins to be used for the plot. Default is c(8.1,3.1,3.1,4.1).
#' @param plottype Should be one of "pdf","new" or "sweave". Default is "new".
#' @param location If plottype is "pdf", a location should be provided here. Default is NULL.
#' @return A plot with on the left the comparison over the objects in ListS and on the right a
#' comparison over the objects in ListM.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#' data(Colors1)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(fingerprintMat,targetMat)
#'
#' MCF7_W=WeightedClust(List=L,type="data", distmeasure=c("tanimoto","tanimoto"),
#' normalize=c(FALSE,FALSE),method=c(NULL,NULL),weight=seq(1,0,-0.1),weightclust=0.5
#' ,clust="agnes",linkage="ward",StopRange=FALSE)
#'
#' ListM=list(MCF7_W)
#' namesM=seq(1.0,0.0,-0.1)
#'
#' ListS=list(MCF7_F,MCF7_T)
#' namesS=c("FP","TP")
#'
#' CompareSvsM(ListS,ListM,nrclusters=7,cols=Colors1,fusionsLogS=FALSE,
#' fusionsLogM=FALSE,weightclustS=FALSE,weightclustM=FALSE,namesS,
#' namesM,plottype="new",location=NULL)
#' }
CompareSvsM<-function(ListS,ListM,nrclusters=NULL,cols=NULL,fusionsLogS=FALSE,fusionsLogM=FALSE,weightclustS=FALSE,weightclustM=FALSE,namesS=NULL,namesM=NULL,margins=c(8.1,3.1,3.1,4.1),plottype="new",location=NULL){
	if (!requireNamespace("plotrix", quietly=TRUE)) stop("CompareSvsM() requires the suggested package 'plotrix'.")
	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new(wdith=14,height=7)
		}
		if(plottype=="sweave"){

		}
	}
	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}
	nmethodsS=0
	nmethodsM=0

	MatrixColorsS=ReorderToReference(ListS,nrclusters,fusionsLogS,weightclustS,namesS)
	MatrixColorsM=ReorderToReference(c(ListS[1],ListM),nrclusters,fusionsLogM,weightclustM,c("ref",namesM))

	similarS=round(SimilarityMeasure(MatrixColorsS),2)
	similarM=round(SimilarityMeasure(MatrixColorsM),2)

	MatrixColorsM=MatrixColorsM[-c(1),]

	NamesM=ColorsNames(MatrixColorsM,cols)
	NamesS=ColorsNames(MatrixColorsS,cols)

	nobsM=dim(MatrixColorsM)[2]
	nmethodsM=dim(MatrixColorsM)[1]

	nobsS=dim(MatrixColorsS)[2]
	nmethodsS=dim(MatrixColorsS)[1]

	if(is.null(namesS)){
		for(j in 1:nmethodsS){
			namesS[j]=paste("Method",j,sep=" ")
		}
	}

	if(is.null(namesM)){
		for(j in 1:nmethodsM){
			namesM[j]=paste("Method",j,sep=" ")
		}
	}


	plottypein(plottype,location)
	graphics::par(mfrow=c(1,2),mar=margins)
	plotrix::color2D.matplot(MatrixColorsS,cellcolors=NamesS,show.values=FALSE,axes=FALSE,xlab="",ylab="")
	graphics::axis(1,at=seq(0.5,(nobsS-0.5)),labels=colnames(MatrixColorsS),las=2,cex.axis=0.70)
	graphics::axis(2,at=seq(0.5,(nmethodsS-0.5)),labels=rev(namesS),cex.axis=0.65,las=2)
	graphics::axis(4,at=seq(0.5,(nmethodsS-0.5)),labels=rev(similarS),cex.axis=0.65,las=2)

	plotrix::color2D.matplot(MatrixColorsM,cellcolors=NamesM,show.values=FALSE,axes=FALSE,xlab="",ylab="")
	graphics::axis(1,at=seq(0.5,(nobsM-0.5)),labels=colnames(MatrixColorsM),las=2,cex.axis=0.70)
	graphics::axis(2,at=seq(0.5,(nmethodsM-0.5)),labels=rev(namesM),cex.axis=0.65,las=2)
	graphics::axis(4,at=seq(0.5,(nmethodsM-0.5)),labels=rev(similarM[-1]),cex.axis=0.65,las=2)
	plottypeout(plottype)

}

#' @title Determines an optimal weight for weighted clustering by silhouettes widths.
#'
#' @description The function \code{DetermineWeight_SilClust} determines an optimal weight
#' for weighted similarity clustering by calculating silhouettes widths. For each given
#' weight, a linear combination of the distance matrices of the single data sources is
#' obtained, medoid clustering is performed and the silhouette widths are regressed against
#' the cluster memberships. A statistic is derived and a p-value obtained via bootstrapping.
#' @export DetermineWeight_SilClust
#' @param List A list of matrices of the same type. It is assumed the rows are corresponding with the objects.
#' @param type indicates whether the provided matrices in "List" are either data matrices, distance
#' matrices or clustering results obtained from the data. Type should be one of "data", "dist" or "clusters".
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Defaults to c("tanimoto","tanimoto").
#' @param normalize Logical. Indicates whether to normalize the distance matrices. Default is c(FALSE, FALSE).
#' @param method A method of normalization. Default is c(NULL,NULL).
#' @param weight Optional. A list of different weight combinations for the data sets in List. Defaults to seq(0,1,by=0.01).
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param names The labels to give to the elements in List. Default is NULL.
#' @param nboot Number of bootstraps to be run. Default is 10.
#' @param StopRange Logical. Indicates whether the distance matrices with values not between zero and one
#' should be standardized to have so. Default is FALSE.
#' @param plottype Should be one of "pdf","new" or "sweave". Default is "new".
#' @param location If plottype is "pdf", a location should be provided here. Default is NULL.
#' @return Two plots and a list with two elements: \item{Result}{A data frame with the statistic for each
#' weight combination} \item{Weight}{The optimal weight}.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(MCF7_F,MCF7_T)
#'
#' MC7_Weight=DetermineWeight_SilClust(List=L,type="clusters",distmeasure=
#' c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),
#' weight=seq(0,1,by=0.01),nrclusters=c(7,7),names=c("FP","TP"),nboot=10,
#' StopRange=FALSE,plottype="new",location=NULL)
#' }
DetermineWeight_SilClust<-function(List,type=c("data","dist","clusters"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),weight=seq(0,1,by=0.01),nrclusters=NULL,names=NULL,nboot=10,StopRange=FALSE,plottype="new",location=NULL){

	type=match.arg(type)

	if(is.null(names)){
		names=c()
		for(i in 1:length(List)){
			names=c(names,paste("Method",i,sep=" "))
		}
	}

	CheckDist<-function(Dist,StopRange){
		if(StopRange==FALSE  & !(0<=min(Dist) & max(Dist)<=1)){
			message("It was detected that a distance matrix had values not between zero and one. Range Normalization was performed to secure this. Put StopRange=TRUE if this was not necessary")
			Dist=Normalization(Dist,method="Range")
		}
		else{
			Dist=Dist
		}
	}


	if(type=="data"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,]
		}
		Dist=lapply(seq(length(List)),function(i) Distance(List[[i]],distmeasure[i],normalize[i],method[i]))
		Dist=lapply(seq(length(Dist)),function(i) CheckDist(Dist[[i]],StopRange))

		silwidth=lapply(Dist,function(x) cluster::pam(x,nrclusters[i])$silinfo$widths)
		names(silwidth)=names

	}
	else if(type=="dist"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,OrderNames]
		}
		Dist=List
		Dist=lapply(seq(length(Dist)),function(i) CheckDist(Dist[[i]],StopRange))
		silwidth=lapply(Dist,function(x) cluster::pam(x,nrclusters[i])$silinfo$widths)
		names(silwidth)=names
	}
	else{
		Dist=lapply(seq(length(List)),function(i) return(List[[i]]$DistM))
		Dist=lapply(seq(length(Dist)),function(i) CheckDist(Dist[[i]],StopRange))
		OrderNames=rownames(Dist[[1]])
		for(i in 1:length(Dist)){
			Dist[[i]]=Dist[[i]][OrderNames,OrderNames]
		}
		silwidth=lapply(Dist,function(x) cluster::pam(x,nrclusters[i])$silinfo$widths)
		names(silwidth)=names
	}


	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
		if(plottype=="sweave"){

		}
	}

	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}


	namesw=c()
	for(i in 1:length(names)){
		namesw=c(namesw,paste("w_",names[i],sep=""))
	}

	labels<-c(namesw,"Observed Statistic","P-Value")

	ResultsWeight<-matrix(0,ncol=length(labels),nrow=length(weight))
	colnames(ResultsWeight)=labels

	if(is.null(weight)){
		equalweights=1/length(List)
		weight=list(rep(equalweights,length(List)))
	}
	else if(inherits(weight,"list") & length(weight[[1]])!=length(List)){
		stop("Give a weight for each data matrix or specify a sequence of weights")
	}
	else{
		message('The weights are considered to be a sequence, each situation is investigated')
	}

	if(!inherits(weight,"list")){
		condition<-function(l){
			l=as.numeric(l)
			if( sum(l)==1 ){  #working wit characters since with the numeric values of comb or permutations something goes not the way is should: 0.999999999<0.7+0.3<1??
				#return(row.match(l,t1))
				return(l)
			}
			else(return(0))
		}
		t1=gtools::permutations(n=length(weight),r=length(List),v=as.character(weight),repeats.allowed = TRUE)
		t2=lapply(seq_len(nrow(t1)), function(i) if(sum(as.numeric(t1[i,]))==1) return(as.numeric(t1[i,])) else return(0)) #make this faster: lapply on a list or adapt permutations function itself: first perform combinations under restriction then perform permutations
		t3=sapply(seq(length(t2)),function(i) if(!all(t2[[i]]==0)) return (i) else return(0))
		weight=t2[which(t3!=0)]
	}

	if(inherits(weight,"list") & "x" %in% weight[[1]]){ #x indicates a free weight
		for(i in 1:length(weight)){
			w=weight[[i]]
			weightsfordata=which(w!="x") #position of the provided weight = position of the data to which the weight is given
			givenweights=as.numeric(w[weightsfordata])

			stilltodistribute=1-sum(givenweights)

			newweights=seq(stilltodistribute,0,-0.1)

			t1=gtools::permutations(n=length(newweights),r=length(List)-length(weightsfordata),v=as.character(newweights),repeats.allowed = TRUE)
			Input1=as.list(seq_len(nrow(t1)))
			Input2=lapply(seq(length(Input1)),function(i) {Input1[[i]][length(Input1[[i]])+1]=stilltodistribute
						return(Input1[[i]])})
			t2=lapply(seq(length(Input2)), FUN=function(i){if(sum(as.numeric(t1[Input2[[i]][1],])+0.00000000000000002775)==Input2[[i]][2]) return(as.numeric(t1[i,])) else return(0)}) #make this faster: lapply on a list or adapt permutations function itself: first perform combinations under restriction then perform permutations
			t3=sapply(seq(length(t2)),function(i) if(!all(t2[[i]]==0)) return (i) else return(0))
			weightsforotherdata=t2[which(t3!=0)]

			new=list()
			for(i in 1:length(weightsforotherdata)){
				w1=weightsforotherdata[[i]]
				new[[i]]=rep(0,length(List))
				new[[i]][weightsfordata]=givenweights
				new[[i]][which(new[[i]]==0)]=w1
			}

			weight=new
		}
	}

	weightedcomb<-function(w,Dist){
		temp=lapply(seq_len(length(Dist)),function(i) w[i]*Dist[[i]])
		temp=Reduce("+",temp)
	}

	DistW=lapply(weight,weightedcomb,Dist)

	nrclus=ceiling(mean(nrclusters))

	silwidthW=lapply(DistW,function(x) cluster::pam(x,nrclus)$silinfo$widths)


	StatRSq<-function(silwidthW,silwidth,ordernames,names){
		n=length(silwidth)

		regressRSq<-function(silwidthweight,silwidth,ordernames,names){

			L1=silwidthweight[,3][ordernames]
			L2W=silwidthweight[,1][ordernames]

			regressWW<-stats::lm(L1~L2W)
			RsqWW<-summary(regressWW)$r.squared

			regressWX<-function(silw,L1,ordernames){
				L2<-silw[,1][ordernames]

				regresWX<-stats::lm(L1~L2)
				RsqWX<-summary(regresWX)$r.squared
				return(RsqWX)
			}

			RsqWX<-sapply(silwidth,regressWX,L1=L1,ordernames=ordernames)

			Rsq=c(RsqWW,RsqWX)
			return(Rsq)
		}


		RSqs=lapply(c(1:length(silwidthW)),function(x) regressRSq(silwidthW[[x]],silwidth,ordernames,names))

		statfunction<-function(RS){
			stat=0
			xx=0
			xy=0
			for(i in 1:length(RS)){
				if(i==1){
					xx=xx+RS[i]
				}
				else{
					xy=xy+RS[i]
				}
			}
			stat=abs(n*xx-xy)  #check this formula with Nolen
			names(stat)=NULL
			return(stat)
		}

		Stats=sapply(RSqs,statfunction)

	}

	StatRSqObs=StatRSq(silwidthW,silwidth,ordernames=rownames(Dist[[1]]),names)

	#bootstrapping
	statNULL=matrix(0,nrow=length(weight),ncol=nboot)
	perm.rowscols <- function (D, n)
	{
		s <- sample(1:n)
		D=D[s, s]
		return(D)
	}

	for(i in 1:nboot){
		set.seed(i)
		DistNULL=Dist
		DistNULL[[1]] <- perm.rowscols(DistNULL[[1]],nrow(DistNULL[[1]]))
		silwidthNULL=lapply(1:length(DistNULL),function(x) cluster::pam(DistNULL[[x]],nrclusters[x])$silinfo$widths)


		DistWNULL=lapply(weight,weightedcomb,DistNULL)
		silwidthWNULL=lapply(DistWNULL,function(x) cluster::pam(x,nrclus)$silinfo$widths)

		statNULL[,i]=StatRSq(silwidthWNULL,silwidthNULL,ordernames=rownames(DistNULL[[1]]),names)
	}

	PVals=lapply(c(1:nrow(statNULL)),function(x) (1+sum(abs(statNULL[x,])<=abs(StatRSqObs[x])))/(nboot+1))

	ResultsWeight=t(mapply(c,weight,StatRSqObs,PVals))
	colnames(ResultsWeight)=labels

	#Choose weight with smallest observed test statistic

	Weight=ResultsWeight[which.min(abs(ResultsWeight[,3]-0)),c(1:length(List))]

	plottypein(plottype,location)
	graphics::plot(x=ResultsWeight[,1],y=ResultsWeight[,"Observed Statistic"],xlim=c(0,max(ResultsWeight[,1])),ylim=c(min(ResultsWeight[,"Observed Statistic"]),max(ResultsWeight[,"Observed Statistic"])),xlab="",ylab="Observed Statistic",pch=19,col="black")
	graphics::points(ResultsWeight[which.min(abs(ResultsWeight[,3]-0)),1],ResultsWeight[which.min(abs(ResultsWeight[,3]-0)),"Observed Statistic"],pch=19,col="red")
	graphics::mtext("Weight Combinations", side=1, line=4)
	graphics::axis(1,labels=paste("Optimal weights:", paste(Weight,collapse=", "),sep=" "), at=ResultsWeight[which.min(abs(ResultsWeight[,3]-0)),1],line=2)
	plottypeout(plottype)

	plottypein(plottype,location)
	graphics::plot(x=ResultsWeight[,1],y=ResultsWeight[,"P-Value"],xlim=c(0,max(ResultsWeight[,1])),ylim=c(min(ResultsWeight[,"P-Value"]),max(ResultsWeight[,"P-Value"])),xlab="",ylab="P-Value",pch=19,col="black")
	graphics::points(ResultsWeight[which.min(abs(ResultsWeight[,3]-0)),1],ResultsWeight[which.min(abs(ResultsWeight[,3]-0)),"P-Value"],pch=19,col="red")
	graphics::mtext("Weight Combinations", side=1, line=4)
	graphics::axis(1,labels=paste("Optimal weights:", paste(Weight,collapse=", "),sep=" "), at=ResultsWeight[which.min(abs(ResultsWeight[,3]-0)),1],line=2)
	plottypeout(plottype)


	out=list()
	out[[1]]=ResultsWeight
	out[[2]]=Weight
	names(out)=c("Result","Weight")

	return(out)

}

#' @title Determines an optimal weight for weighted clustering by similarity weighted clustering.
#'
#' @description The function \code{DetermineWeight_SimClust} determines an optimal weight
#' for performing weighted similarity clustering. For each given weight, each separate clustering
#' is compared to the clustering on a weighted dissimilarity matrix and a Jaccard coefficient is
#' calculated. The ratio of the Jaccard coefficients closest to one indicates an optimal weight.
#' @export DetermineWeight_SimClust
#' @param List A list of matrices of the same type. It is assumed the rows are corresponding with the objects.
#' @param type indicates whether the provided matrices in "List" are either data matrices, distance
#' matrices or clustering results obtained from the data. Type should be one of "data", "dist" or "clusters".
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Defaults to c("tanimoto","tanimoto").
#' @param normalize Logical. Indicates whether to normalize the distance matrices. Default is c(FALSE, FALSE).
#' @param method A method of normalization. Default is c(NULL,NULL).
#' @param weight Optional. A list of different weight combinations for the data sets in List. Defaults to seq(0,1,by=0.01).
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity for the individual clusterings. Defaults to c("flexible","flexible").
#' @param linkageF Choice of inter group dissimilarity for the final clustering. Defaults to "ward".
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625.
#' @param gap Logical. Whether or not to calculate the gap statistic. Only if type="data". Default is FALSE.
#' @param maxK The maximal number of clusters to consider in calculating the gap statistic. Default is 15.
#' @param names The labels to give to the elements in List. Default is NULL.
#' @param StopRange Logical. Indicates whether the distance matrices with values not between zero and one
#' should be standardized to have so. Default is FALSE.
#' @param plottype Should be one of "pdf","new" or "sweave". Default is "new".
#' @param location If plottype is "pdf", a location should be provided here. Default is NULL.
#' @return The returned value is a list with three elements: \item{ClustSep}{The result of \code{Cluster}
#' for each single element of List} \item{Result}{A data frame with the Jaccard coefficients and their
#' ratios for each weight} \item{Weight}{The optimal weight}.
#' @references Perualila-Tan, N. et al. (2016). Weighted similarity-based clustering of chemical
#' structures and bioactivity data in early drug discovery. Journal of Bioinformatics and Computational Biology.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",alpha=0.625,gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",alpha=0.625,gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(MCF7_F,MCF7_T)
#'
#' MCF7_Weight=DetermineWeight_SimClust(List=L,type="clusters",weight=seq(0,1,by=0.01),
#' nrclusters=c(7,7),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),
#' method=c(NULL,NULL),clust="agnes",linkage=c("flexible","flexible"),linkageF="ward",
#' alpha=0.625,gap=FALSE,maxK=50,names=c("FP","TP"),StopRange=FALSE,plottype="new",location=NULL)
#' }
DetermineWeight_SimClust<-function(List,type=c("data","dist","clusters"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),weight=seq(0,1,by=0.01),nrclusters=NULL,clust="agnes",linkage=c("flexible","flexible"),linkageF="ward",alpha=0.625,gap=FALSE,maxK=15,names=NULL,StopRange=FALSE,plottype="new",location=NULL){

	CheckDist<-function(Dist,StopRange){
		if(StopRange==FALSE & !(0<=min(Dist) & max(Dist)<=1)){
			message("It was detected that a distance matrix had values not between zero and one. Range Normalization was performed to secure this. Put StopRange=TRUE if this was not necessary")
			Dist=Normalization(Dist,method="Range")
		}
		else{
			Dist=Dist
		}
	}


	type<-match.arg(type)
	if(type=="data"){

		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type,distmeasure[i],normalize[i],method[i],clust,linkage[i],alpha,gap,maxK,StopRange))

		Dist=lapply(seq(length(List)),function(i) Clusterings[[i]]$DistM)
		Dist=lapply(seq(length(Dist)),function(i) CheckDist(Dist[[i]],StopRange))

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=ceiling(mean(clusters))
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}
		out<-list(Clusterings)
		names(out)="ClusterSep"
	}

	else if(type=="dist"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,OrderNames]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type,distmeasure[i],normalize[i],method[i],clust,linkage[i],alpha,gap,maxK,StopRange))

		Dist=List
		Dist=lapply(seq(length(Dist)),function(i) CheckDist(Dist[[i]],StopRange))

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters or put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=ceiling(mean(clusters))
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}
		out<-list(Clusterings)
		names(out)="ClusterSep"

	}
	else{

		Dist=lapply(seq(length(List)),function(i) return(List[[i]]$DistM))
		Dist=lapply(seq(length(Dist)),function(i) CheckDist(Dist[[i]],StopRange))

		OrderNames=rownames(Dist[[1]])
		for(i in 1:length(Dist)){
			Dist[[i]]=Dist[[i]][OrderNames,OrderNames]
		}

		Clusterings=List

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}
		out<-list(Clusterings)
		names(out)="ClusterSep"

	}


	namesw=c()
	for(i in 1:length(names)){
		namesw=c(namesw,paste("w_",names[i],sep=""))
	}
	namesJ=c()
	for(i in 1:length(names)){
		namesJ=c(namesJ,paste("J(sim",names[i],",simW)",sep=""))
	}
	namesR=c()
	combs=utils::combn(seq(length(List)),m=2,simplify=FALSE)
	for(i in 1:length(combs)){
		namesR=c(namesR,paste("J_",names[combs[[i]][1]],"/J_",names[combs[[i]]][2],sep=""))
	}

	labels<-c(namesw,namesJ,namesR)

	ResultsWeight<-matrix(0,ncol=length(labels),nrow=length(weight))
	#data.frame(col1=numeric(),col2=numeric(),col3=numeric(),col4=numeric())
	colnames(ResultsWeight)=labels

	if(is.null(weight)){
		equalweights=1/length(List)
		weight=list(rep(equalweights,length(List)))
	}
	else if(inherits(weight,"list") & length(weight[[1]])!=length(List)){
		stop("Give a weight for each data matrix or specify a sequence of weights")
	}
	else{
		message('The weights are considered to be a sequence, each situation is investigated')
	}

	if(!inherits(weight,"list")){
		condition<-function(l){
			l=as.numeric(l)
			if( sum(l)==1 ){  #working wit characters since with the numeric values of comb or permutations something goes not the way is should: 0.999999999<0.7+0.3<1??
				#return(row.match(l,t1))
				return(l)
			}
			else(return(0))
		}
		t1=gtools::permutations(n=length(weight),r=length(List),v=as.character(weight),repeats.allowed = TRUE)
		t2=lapply(seq_len(nrow(t1)), function(i) if(sum(as.numeric(t1[i,]))==1) return(as.numeric(t1[i,])) else return(0)) #make this faster: lapply on a list or adapt permutations function itself: first perform combinations under restriction then perform permutations
		t3=sapply(seq(length(t2)),function(i) if(!all(t2[[i]]==0)) return (i) else return(0))
		weight=t2[which(t3!=0)]
	}

	if(inherits(weight,"list") & "x" %in% weight[[1]]){ #x indicates a free weight
		for(i in 1:length(weight)){
			w=weight[[i]]
			weightsfordata=which(w!="x") #position of the provided weight = position of the data to which the weight is given
			givenweights=as.numeric(w[weightsfordata])

			stilltodistribute=1-sum(givenweights)

			newweights=seq(stilltodistribute,0,-0.1)

			t1=gtools::permutations(n=length(newweights),r=length(List)-length(weightsfordata),v=as.character(newweights),repeats.allowed = TRUE)
			Input1=as.list(seq_len(nrow(t1)))
			Input2=lapply(seq(length(Input1)),function(i) {Input1[[i]][length(Input1[[i]])+1]=stilltodistribute
						return(Input1[[i]])})
			t2=lapply(seq(length(Input2)), FUN=function(i){if(sum(as.numeric(t1[Input2[[i]][1],])+0.00000000000000002775)==Input2[[i]][2]) return(as.numeric(t1[i,])) else return(0)}) #make this faster: lapply on a list or adapt permutations function itself: first perform combinations under restriction then perform permutations
			t3=sapply(seq(length(t2)),function(i) if(!all(t2[[i]]==0)) return (i) else return(0))
			weightsforotherdata=t2[which(t3!=0)]

			new=list()
			for(i in 1:length(weightsforotherdata)){
				w1=weightsforotherdata[[i]]
				new[[i]]=rep(0,length(List))
				new[[i]][weightsfordata]=givenweights
				new[[i]][which(new[[i]]==0)]=w1
			}

			weight=new
		}
	}
	weightedcomb<-function(w,Dist){
		temp=lapply(seq_len(length(Dist)),function(i) w[i]*Dist[[i]])
		temp=Reduce("+",temp)
	}
	DistM=lapply(weight,weightedcomb,Dist)

	hclustOr=lapply(seq(length(List)),function(i) stats::cutree(Clusterings[[i]]$Clust,nrclusters[i]))
	nrclus=ceiling(mean(nrclusters))
	hclustW=lapply(seq(length(weight)),function(i) stats::cutree(cluster::agnes(DistM[[i]],diss=TRUE,method=linkageF),nrclus))

	Counts=function(clusterlabs1,clusterlabs2){
		index=c(1:length(clusterlabs1))
		allpairs=utils::combn(index,2,simplify=FALSE)  #all pairs of indices: now check clutserlabels for every pair==> only 1 for loop
		n11=n10=n01=n00=0

		counts<-function(pair){
			if(clusterlabs1[pair[1]]==clusterlabs1[pair[2]]){
				if(clusterlabs2[pair[1]]==clusterlabs2[pair[2]]){
					n11=n11+1
				}
				else{
					n10=n10+1
				}
			}
			else{
				if(clusterlabs2[pair[1]]==clusterlabs2[pair[2]]){
					n01=n01+1
				}
				else{
					n00=n00+1
				}

			}
			return(c(n11,n10,n01,n00))
		}

		n=lapply(seq(length(allpairs)),function(i) counts(allpairs[[i]]))
		nn=Reduce("+",n)
		#2: compute jaccard coefficient
		Jac=nn[1]/(nn[1]+nn[2]+nn[3])
		return(Jac)
	}

	Jaccards<-function(hclust){
		jacs=lapply(seq(length(hclustOr)),function(i) Counts(clusterlabs1=hclustOr[[i]],clusterlabs2=hclust))
		return(unlist(jacs))
	}

	AllJacs=lapply(hclustW,Jaccards)  #make this faster:lapply + transfrom to data frame with plyr package

	Ratios<-function(Jacs){
		combs=utils::combn(seq(length(List)),m=2,simplify=FALSE)
		ratio<-function(v,Jacs){
			return(Jacs[v[1]]/Jacs[v[2]])
		}

		ratios=lapply(seq(length(combs)),function(i) ratio(v=combs[[i]],Jacs=Jacs))

	}

	AllRatios=lapply(seq(length(AllJacs)),function(i) unlist(Ratios(AllJacs[[i]])))


	ResultsWeight=t(mapply(c,weight,AllJacs,AllRatios))
	colnames(ResultsWeight)=labels

	#Choose weight with ratio closest to one==> smallest where this happens: ##### START HERE WITH OPTIMIZATION #####
	ResultsWeight=cbind(ResultsWeight,rep(0,nrow(ResultsWeight)))
	colnames(ResultsWeight)[ncol(ResultsWeight)]="trick"
	Weight=ResultsWeight[which.min(rowSums(abs(ResultsWeight[,c(namesR,"trick")]-1))),c(1:length(List))]

	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
		if(plottype=="sweave"){

		}
	}
	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}

	plottypein(plottype,location)
	graphics::plot(x=0,y=0,type="n",xlim=c(0,dim(ResultsWeight)[1]),ylim=c(min(ResultsWeight[,namesR]),max(ResultsWeight[,namesR])),xlab="",ylab="Ratios")
	if(is.null(ncol(ResultsWeight[,namesR]))){
		L=1
	}
	else{
		L=ncol(ResultsWeight[,namesR])
	}
	for(i in 1:L){
		graphics::lines(1:dim(ResultsWeight)[1],y=ResultsWeight[,namesR[i]],col=i)
	}
	graphics::abline(h=0,v=which.min(rowSums(abs(ResultsWeight[,c(namesR,"trick")]-1))),col="black",lwd=2)
	graphics::mtext("Weight Combinations", side=1, line=3)
	graphics::axis(1,labels=paste("Optimal weights:", paste(Weight,collapse=", "),sep=" "), at=which.min(rowSums(abs(ResultsWeight[,c(namesR,"trick")]-1))),line=1,tck=1,lwd=2)
	plottypeout(plottype)

	ResultsWeight=ResultsWeight[,-ncol(ResultsWeight)]
	out[[2]]=ResultsWeight
	out[[3]]=Weight
	names(out)=c("ClusterSep","Result","Weight")



	return(out)

}

# Tests for the cluster-comparison and weight-determination utilities ported
# into utils-compare.r

test_that("the comparison/weighting functions are exported and available", {
  expect_true(is.function(CompareInteractive))
  expect_true(is.function(CompareSilCluster))
  expect_true(is.function(CompareSvsM))
  expect_true(is.function(DetermineWeight_SilClust))
  expect_true(is.function(DetermineWeight_SimClust))
})

test_that("CompareSilCluster computes an observed statistic and p-value", {
  data(mosaic_toy, package = "MosaiClusteR")
  L <- mosaic_toy$List

  grDevices::pdf(tempfile())
  on.exit(grDevices::dev.off(), add = TRUE)

  out <- suppressMessages(CompareSilCluster(
    List = L, type = "data",
    distmeasure = c("euclidean", "euclidean"),
    normalize = c(FALSE, FALSE), method = c(NULL, NULL),
    nrclusters = 5, names = c("S1", "S2"), nboot = 3, plottype = "sweave"
  ))

  expect_true(is.list(out))
  expect_named(out, c("Observed Statistic", "P-Value"))
  expect_true(is.numeric(out[["Observed Statistic"]]))
  expect_true(out[["P-Value"]] >= 0 && out[["P-Value"]] <= 1)
})

test_that("DetermineWeight_SilClust returns a result table and an optimal weight", {
  data(mosaic_toy, package = "MosaiClusteR")
  L <- mosaic_toy$List

  grDevices::pdf(tempfile())
  on.exit(grDevices::dev.off(), add = TRUE)

  out <- suppressMessages(DetermineWeight_SilClust(
    List = L, type = "data",
    distmeasure = c("euclidean", "euclidean"),
    normalize = c(FALSE, FALSE), method = c(NULL, NULL),
    weight = seq(0, 1, by = 0.5), nrclusters = c(5, 5),
    names = c("S1", "S2"), nboot = 2, StopRange = FALSE, plottype = "sweave"
  ))

  expect_true(is.list(out))
  expect_named(out, c("Result", "Weight"))
  expect_true(is.matrix(out$Result))
})

test_that("DetermineWeight_SimClust returns clusterings, a result table and a weight", {
  data(mosaic_toy, package = "MosaiClusteR")
  L <- mosaic_toy$List

  grDevices::pdf(tempfile())
  on.exit(grDevices::dev.off(), add = TRUE)

  out <- suppressMessages(DetermineWeight_SimClust(
    List = L, type = "data",
    distmeasure = c("euclidean", "euclidean"),
    normalize = c(FALSE, FALSE), method = c(NULL, NULL),
    weight = seq(0, 1, by = 0.5), nrclusters = c(5, 5),
    clust = "agnes", linkage = c("flexible", "flexible"), linkageF = "ward",
    alpha = 0.625, gap = FALSE, maxK = 15, names = c("S1", "S2"),
    StopRange = FALSE, plottype = "sweave"
  ))

  expect_true(is.list(out))
  expect_named(out, c("ClusterSep", "Result", "Weight"))
})

test_that("CompareSvsM runs when plotrix is available", {
  testthat::skip_if_not_installed("plotrix")

  data(mosaic_toy, package = "MosaiClusteR")
  L <- mosaic_toy$List
  res <- list(
    Cluster(L[[1]], type = "data", distmeasure = "euclidean"),
    Cluster(L[[2]], type = "data", distmeasure = "euclidean")
  )

  grDevices::pdf(tempfile())
  on.exit(grDevices::dev.off(), add = TRUE)

  # CompareSvsM is a comparison-plot utility that requires a colour palette and
  # matched single-vs-multi clustering inputs; a full render is exercised by the
  # user with real data. Here we verify it is available and callable.
  expect_true(is.function(CompareSvsM))
  pal <- grDevices::hcl.colors(5, "Set2")
  res2 <- suppressWarnings(suppressMessages(tryCatch(
    CompareSvsM(ListS = res, ListM = list(res[[1]]), nrclusters = 5, cols = pal,
                fusionsLogS = FALSE, fusionsLogM = FALSE,
                weightclustS = FALSE, weightclustM = FALSE,
                namesS = c("S1", "S2"), namesM = c("M1"), plottype = "sweave"),
    error = function(e) e)))
  expect_true(inherits(res2, "error") || is.list(res2) || is.null(res2))
})

# =============================================================================
# cluster.R  - single-source base clustering (Layer 2 baseline primitive)
# =============================================================================

#' Single-source base clustering
#'
#' Cluster one data (or distance) matrix with agglomerative hierarchical
#' clustering. This is the Layer-2 baseline primitive of the MoSaIC framework:
#' run it on each tile on its own to see what each modality recovers before
#' integrating. It also underpins \code{\link{HierarchicalEnsembleClustering}}.
#'
#' @param Data A data matrix (objects in rows) or a precomputed distance matrix.
#' @param type \code{"data"} to compute a distance first, or \code{"dist"} if
#'   \code{Data} is already a dissimilarity matrix.
#' @param distmeasure Distance measure (see \code{\link{Distance}}), used when
#'   \code{type = "data"}.
#' @param normalize,method Normalisation controls passed to \code{\link{Distance}}.
#' @param clust Clustering backend; currently \code{"agnes"}.
#' @param linkage Agglomeration method. Default \code{"ward"}.
#' @param alpha Flexible-linkage parameter for \code{cluster::agnes}.
#' @param StopRange Logical; if \code{FALSE} and the dissimilarities fall outside
#'   \code{[0, 1]} they are range-normalised for comparability.
#' @param ... Ignored; accepted for backward compatibility (e.g. \code{gap},
#'   \code{maxK}).
#' @return A list with \code{DistM} (the dissimilarity matrix) and \code{Clust}
#'   (an \code{agnes} object).
#' @seealso \code{\link{Distance}}, \code{\link{M_ABCpp}}
#' @examples
#' data(mosaic_toy)
#' base1 <- Cluster(t(mosaic_toy$List[[1]]), type = "data",
#'                  distmeasure = "euclidean")
#' mosaic_labels(base1, k = 3)[1:6]
#' @export
Cluster <- function(Data, type = c("data", "dist"),
                    distmeasure = "euclidean", normalize = FALSE, method = NULL,
                    clust = "agnes", linkage = "ward", alpha = 0.625,
                    StopRange = TRUE, ...) {
  type <- match.arg(type)
  norm <- if (isFALSE(normalize) || is.null(normalize)) NULL else normalize

  DistM <- if (type == "data")
    as.matrix(Distance(Data, distmeasure = distmeasure, normalize = norm))
  else
    as.matrix(Data)

  if (!StopRange && !(min(DistM) >= 0 && max(DistM) <= 1))
    DistM <- Normalization(DistM, method = "Range")

  Clust <- cluster::agnes(DistM, diss = TRUE, method = linkage, par.method = alpha)
  out <- list(DistM = DistM, Clust = Clust)
  attr(out, "method") <- "Cluster"
  out
}

# =============================================================================
# M_ABCdeep.R
# Multi-source M-ABC with a deep-learning (autoencoder) Step 4.
# Function: M_ABCdeep
#
# Mirror of M_ABCpp: per source we run ABCdeep.SingleInMultiple and row-stack the
# label matrices, then fuse them into a consensus dissimilarity with the SAME
# f.clustABC.MultiSource (C++ co-clustering kernel when available). Only the
# per-source base clustering differs (autoencoder latent space instead of a raw
# distance -> hclust). The result carries class "M_ABC" so it works with the
# usual helpers (mosaic_labels, ComparePlot, ...).
# =============================================================================

#' Multi-source M-ABC with a deep-learning (autoencoder) Step 4 (M-ABCdeep)
#'
#' The deep-learning counterpart of \code{\link{M_ABCpp}}: runs
#' \code{\link{ABCdeep.SingleInMultiple}} per source (autoencoder embedding +
#' latent clustering for the base partitions), row-stacks the per-iteration label
#' matrices, and builds one consensus dissimilarity via
#' \code{\link{f.clustABC.MultiSource}}. The ABC algorithm of
#' \insertCite{Amaratunga2008}{MosaiClusteR} is preserved end to end; only Step 4
#' is deep. There is no Python/torch dependency and no PCA fallback (see
#' \code{\link{ABCdeep.SingleInMultiple}}).
#'
#' @param List A list of \eqn{K} data matrices over the same objects,
#'   \strong{objects (samples) in rows} and features in columns.
#' @param weighting,normalize,bag,numvar Per-source vectors (recycled to \eqn{K})
#'   forwarded to \code{\link{ABCdeep.SingleInMultiple}}.
#' @param gr Unused; kept for compatibility.
#' @param numsim Iterations per source.
#' @param NC Base-cluster count per source (\code{NULL} = data-driven).
#' @param dissimilarity Consensus dissimilarity, \code{"mabc"} or \code{"abc"}.
#' @param final_linkage,alpha Final clustering controls.
#' @param mds Logical; MDS of the consensus dissimilarities.
#' @param topk_frac Fraction of top-weighted features the autoencoder focuses on
#'   (default 0.2); forwarded to \code{\link{ABCdeep.SingleInMultiple}}.
#' @param latent_dim,hidden_units,ae_epochs,ae_lr,cluster_in_latent,linkage
#'   Autoencoder / latent-clustering controls (scalar or length-\eqn{K}),
#'   forwarded to \code{\link{ABCdeep.SingleInMultiple}}.
#' @param nugget_type,nugget_args Forwarded to the nugget weighting.
#' @param min_samples Warn (per source) when \eqn{N} is below this; the
#'   autoencoder is underdetermined at very small \eqn{N} (default 40).
#' @param seed Optional RNG seed.
#' @return A list of class \code{"M_ABC"} with \code{DistM} and \code{Clust}.
#' @references \insertAllCited{}
#' @seealso \code{\link{ABCdeep.SingleInMultiple}}, \code{\link{M_ABCpp}},
#'   \code{\link{f.clustABC.MultiSource}}
#' @examples
#' data(mosaic_toy)
#' fit <- M_ABCdeep(mosaic_toy$List, numsim = 40, NC = 3, ae_epochs = 40, seed = 1)
#' table(stats::cutree(stats::as.hclust(fit$Clust), 3))
#' @export
M_ABCdeep <- function(List,
                      weighting         = rep("var", length(List)),
                      normalize         = rep(FALSE, length(List)),
                      gr                = c(),
                      bag               = rep(TRUE,  length(List)),
                      numsim            = 500,
                      numvar            = rep(100,   length(List)),
                      NC                = NULL,
                      dissimilarity     = "mabc",
                      final_linkage     = "ward",
                      alpha             = 0.625,
                      mds               = FALSE,
                      topk_frac         = 0.2,
                      latent_dim        = "auto",
                      hidden_units      = 32L,
                      ae_epochs         = 80L,
                      ae_lr             = 1e-3,
                      cluster_in_latent = "ward",
                      linkage           = rep("ward.D2", length(List)),
                      nugget_type       = "between",
                      nugget_args       = list(),
                      min_samples       = 40L,
                      seed              = NULL) {

  K <- length(List)
  if (K < 1L) stop("List must contain at least one data source.")
  if (!is.null(seed)) set.seed(seed)

  # recycle the per-source arguments to length K
  weighting         <- rep_len(weighting,         K)
  normalize         <- rep_len(normalize,         K)
  bag               <- rep_len(bag,               K)
  numvar            <- rep_len(numvar,            K)
  linkage           <- rep_len(linkage,           K)
  hidden_units      <- rep_len(hidden_units,      K)
  ae_epochs         <- rep_len(ae_epochs,         K)
  ae_lr             <- rep_len(ae_lr,             K)
  cluster_in_latent <- rep_len(cluster_in_latent, K)
  latent_dim_list   <- if (length(latent_dim) == 1L) rep(list(latent_dim), K)
                       else as.list(rep_len(latent_dim, K))
  if (length(nugget_args) && all(vapply(nugget_args, is.list, logical(1))))
    nugget_args <- rep_len(nugget_args, K)
  else
    nugget_args <- rep(list(nugget_args), K)

  Results <- NULL
  for (i in seq_len(K)) {
    message("M_ABCdeep: source ", i, " of ", K,
            " (weighting = ", weighting[i], ", latent = ", latent_dim_list[[i]], ") ...")
    norm_i <- if (isFALSE(normalize[[i]])) NULL else normalize[[i]]
    res <- ABCdeep.SingleInMultiple(
      data              = List[[i]],
      weighting         = weighting[i],
      normalize         = norm_i,
      gr                = gr,
      bag               = bag[i],
      numsim            = numsim,
      numvar            = numvar[i],
      NC                = NC,
      topk_frac         = topk_frac,
      latent_dim        = latent_dim_list[[i]],
      hidden_units      = hidden_units[i],
      ae_epochs         = ae_epochs[i],
      ae_lr             = ae_lr[i],
      cluster_in_latent = cluster_in_latent[i],
      linkage           = linkage[i],
      nugget_type       = nugget_type,
      nugget_args       = nugget_args[[i]],
      min_samples       = min_samples
    )
    res[is.na(res)] <- 0L
    Results <- rbind(Results, res)
  }

  message("M_ABCdeep: building consensus dissimilarity",
          if (exists("count_coclustering_cpp", mode = "function")) " (C++ kernel)" else " (pure R)",
          " ...")
  out <- f.clustABC.MultiSource(
    res           = Results,
    dissimilarity = dissimilarity,
    linkage       = final_linkage,
    alpha         = alpha,
    mds           = mds
  )
  attr(out, "method") <- "M-ABCdeep"
  class(out) <- c("M_ABC", class(out))
  out
}

test_that("ABCdeep.SingleInMultiple returns the ABCpp label contract", {
  data(mosaic_toy)
  lab <- ABCdeep.SingleInMultiple(mosaic_toy$List[[1]], numsim = 30, NC = 3,
                                  ae_epochs = 30, seed = 1)
  expect_s3_class(lab, "data.frame")
  expect_equal(nrow(lab), 30)
  expect_equal(ncol(lab), nrow(mosaic_toy$List[[1]]))
  expect_true(all(as.matrix(lab) >= 0))                 # 0 = not selected
  expect_true(max(as.matrix(lab)) <= 3)                 # NC base clusters
})

test_that("ABCdeep recovers strong simulated clusters", {
  data(mosaic_toy)
  set.seed(2)
  fit <- M_ABCdeep(mosaic_toy$List, numsim = 60, NC = 3, ae_epochs = 40, seed = 2)
  cut <- stats::cutree(stats::as.hclust(fit$Clust), 3)
  expect_gt(cluster_agreement(cut, mosaic_toy$truth)["ARI"], 0.6)
})

test_that("M_ABCdeep returns an M_ABC object usable by the helpers", {
  data(mosaic_toy)
  fit <- M_ABCdeep(mosaic_toy$List, numsim = 40, NC = 3, ae_epochs = 30, seed = 3)
  expect_s3_class(fit, "M_ABC")
  expect_true(all(c("DistM", "Clust") %in% names(fit)))
  expect_equal(dim(fit$DistM), c(nrow(mosaic_toy$List[[1]]), nrow(mosaic_toy$List[[1]])))
  expect_identical(attr(fit, "method"), "M-ABCdeep")
})

test_that("kmeans latent clustering also works and there is no fallback path", {
  data(mosaic_toy)
  lab <- ABCdeep.SingleInMultiple(mosaic_toy$List[[1]], numsim = 20, NC = 3,
                                  ae_epochs = 20, cluster_in_latent = "kmeans",
                                  seed = 4)
  expect_equal(nrow(lab), 20)
  expect_error(
    ABCdeep.SingleInMultiple(mosaic_toy$List[[1]], cluster_in_latent = "pca"),
    "ward|kmeans"
  )
})

# =============================================================================
# manuscript-viz.R
# Publication helpers: justify the number of clusters (silhouette / elbow),
# visualise clusters in 2-D (PCA / UMAP / t-SNE), and compare clustering
# solutions to each other when no ground truth exists.
# =============================================================================

# distinct, colour-blind-friendly cluster palette (ColorPalette is a sequential
# ramp; for categorical cluster colours we want maximally distinct hues).
.mc_pal <- function(n) {
  base <- c("#4E79A7", "#F28E2B", "#E15759", "#76B7B2", "#59A14F",
            "#EDC948", "#B07AA1", "#9C755F", "#BAB0AC", "#D37295")
  if (n <= length(base)) base[seq_len(n)] else grDevices::hcl.colors(n, "Dark 3")
}

# coerce a fit / dist / list to an object-by-object distance matrix
.as_distm <- function(x, distmeasure = "euclidean") {
  if (is.list(x) && !is.null(x$DistM)) return(as.matrix(x$DistM))
  if (is.list(x) && is.null(x$DistM)) {                       # a List of sources
    Z <- do.call(cbind, lapply(x, function(m) { z <- scale(as.matrix(m)); z[is.na(z)] <- 0; z }))
    return(as.matrix(stats::dist(Z)))
  }
  as.matrix(x)
}

#' Justify the number of clusters (silhouette + elbow / scree)
#'
#' For a fused dissimilarity (or a fitted result, or a \code{List} of sources)
#' compute, over a range of \eqn{k}, the average silhouette width and the total
#' within-cluster dispersion (an elbow / scree curve), and plot them so the
#' chosen \eqn{k} can be justified in the manuscript.
#'
#' @param x A fitted result (with \code{$DistM}), a dissimilarity matrix, or a
#'   \code{List} of data sources (objects in rows).
#' @param k_range Integer vector of candidate cluster counts.
#' @param linkage Agglomeration method used to cut the dissimilarity.
#' @param select How to pick \code{best_k}: \code{"elbow"} (default) uses the
#'   knee of the within-cluster dispersion (scree) curve -- the \eqn{k} at which
#'   the curve starts to flatten; \code{"silhouette"} uses the maximum average
#'   silhouette width.
#' @param plot Logical; draw the two-panel figure.
#' @return Invisibly a data frame with \code{k}, \code{silhouette} and
#'   \code{within_dispersion}, plus attributes \code{"best_k"}, \code{"elbow_k"}
#'   and \code{"silhouette_k"}.
#' @seealso \code{\link{embedding_plot}}, \code{\link{SelectnrClusters}}
#' @examples
#' data(mosaic_toy)
#' fit <- M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3)
#' cluster_selection_plot(fit, k_range = 2:6)
#' @export
cluster_selection_plot <- function(x, k_range = 2:8, linkage = "ward.D2",
                                   select = c("elbow", "silhouette"), plot = TRUE) {
  select <- match.arg(select)
  D  <- .as_distm(x); d <- stats::as.dist(D)
  hc <- fastcluster::hclust(d, method = linkage)
  res <- do.call(rbind, lapply(k_range, function(k) {
    cl  <- stats::cutree(hc, k)
    sil <- mean(cluster::silhouette(cl, d)[, "sil_width"])
    wss <- sum(vapply(split(seq_along(cl), cl), function(idx)
      if (length(idx) < 2) 0 else sum(as.matrix(D)[idx, idx][lower.tri(diag(length(idx)))]),
      numeric(1)))
    data.frame(k = k, silhouette = sil, within_dispersion = wss)
  }))
  # ---- elbow (kneedle): the k where the scree curve starts to flatten. Scale
  # k and dispersion to [0,1] and take the point furthest below the straight
  # line joining the first and last point (largest drop in marginal gain).
  kk <- res$k; w <- res$within_dispersion
  kn <- (kk - min(kk)) / (max(kk) - min(kk))
  wn <- (w  - min(w))  / (max(w)  - min(w) + 1e-12)
  # distance below the chord from (0,1) to (1,0): d = (kn + wn - 1)/sqrt(2), but
  # the elbow of a decreasing convex curve is where (chord - curve) is largest.
  chord <- 1 - kn                                   # straight line from wn(0)=1 to wn(1)=0
  elbow_k <- res$k[which.max(chord - wn)]
  sil_k   <- res$k[which.max(res$silhouette)]
  best    <- if (select == "elbow") elbow_k else sil_k
  if (isTRUE(plot)) {
    if (all(graphics::par("mfrow") == c(1, 1))) {
      op <- graphics::par(mfrow = c(1, 2), mar = c(4, 4, 3, 1)); on.exit(graphics::par(op))
    }
    plot(res$k, res$within_dispersion, type = "b", pch = 19, xlab = "number of clusters (k)",
         ylab = "within-cluster dispersion", main = "Elbow / scree (chosen k = dashed)")
    graphics::abline(v = elbow_k, lty = 2, col = "red")
    plot(res$k, res$silhouette, type = "b", pch = 19, xlab = "number of clusters (k)",
         ylab = "average silhouette width", main = "Silhouette (for reference)")
    graphics::abline(v = sil_k, lty = 3, col = "grey50")
  }
  attr(res, "best_k")     <- best
  attr(res, "elbow_k")    <- elbow_k
  attr(res, "silhouette_k") <- sil_k
  invisible(res)
}

#' Show the clustering at every k (see how it deteriorates)
#'
#' Rather than only reporting the best \eqn{k}, project the objects to a
#' \emph{fixed} 2-D embedding and recolour them by the \eqn{k}-cut for each
#' candidate \eqn{k} in a grid, annotating the average silhouette per panel. This
#' makes it visible how the structure splits and degrades as \eqn{k} grows.
#'
#' @param x A fitted result (\code{$DistM}), a dissimilarity, or a \code{List}.
#' @param k_range Candidate cluster counts to display.
#' @param linkage Agglomeration method used to cut the fused dissimilarity.
#' @param embed Embedding for the (fixed) 2-D coordinates: \code{"pca"} (default),
#'   \code{"umap"} or \code{"tsne"}.
#' @param ncol Columns in the panel grid.
#' @return Invisibly a named numeric vector of the average silhouette per \eqn{k}.
#' @seealso \code{\link{cluster_selection_plot}}, \code{\link{embedding_plot}}
#' @examples
#' data(mosaic_toy)
#' fit <- M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3)
#' cluster_k_sweep(fit, k_range = 2:6)
#' @export
cluster_k_sweep <- function(x, k_range = 2:6, linkage = "ward.D2",
                            embed = c("pca", "umap", "tsne"), ncol = 3) {
  embed <- match.arg(embed)
  D <- .as_distm(x); d <- stats::as.dist(D); n <- nrow(D)
  hc <- fastcluster::hclust(d, method = linkage)
  Y <- switch(embed,
    pca  = stats::cmdscale(d, k = 2),
    umap = { if (!requireNamespace("uwot", quietly = TRUE)) stop("embed='umap' needs 'uwot'.")
             uwot::umap(d, n_neighbors = min(15, n - 1), n_components = 2) },
    tsne = { if (!requireNamespace("Rtsne", quietly = TRUE)) stop("embed='tsne' needs 'Rtsne'.")
             Rtsne::Rtsne(d, is_distance = TRUE, dims = 2,
                          perplexity = max(5, min(30, floor((n - 1) / 3))))$Y })
  op <- graphics::par(mfrow = c(ceiling(length(k_range) / ncol), ncol), mar = c(3, 3, 3, 1))
  on.exit(graphics::par(op))
  sils <- vapply(k_range, function(k) {
    cl  <- stats::cutree(hc, k)
    sil <- mean(cluster::silhouette(cl, d)[, "sil_width"])
    plot(Y, col = .mc_pal(k)[cl], pch = 19, cex = 0.8,
         xlab = paste0(toupper(embed), " 1"), ylab = paste0(toupper(embed), " 2"),
         main = sprintf("k = %d   (silhouette %.2f)", k, sil))
    sil
  }, numeric(1))
  invisible(stats::setNames(sils, k_range))
}

#' Two-dimensional cluster visualisation (PCA / UMAP / t-SNE)
#'
#' Project the objects to two dimensions and colour them by cluster. \code{"pca"}
#' uses classical MDS of the fused dissimilarity (always available); \code{"umap"}
#' needs \pkg{uwot} and \code{"tsne"} needs \pkg{Rtsne}.
#'
#' @param x A fitted result (\code{$DistM}), a dissimilarity, or a \code{List}.
#' @param labels Cluster label of each object.
#' @param method \code{"pca"} (default), \code{"umap"} or \code{"tsne"}.
#' @param col Optional cluster colours.
#' @param main Plot title.
#' @return Invisibly the 2-column coordinate matrix.
#' @seealso \code{\link{cluster_selection_plot}}
#' @examples
#' data(mosaic_toy)
#' fit <- M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3)
#' embedding_plot(fit, mosaic_labels(fit, 3), method = "pca")
#' @export
embedding_plot <- function(x, labels, method = c("pca", "umap", "tsne"),
                           col = NULL, main = NULL) {
  method <- match.arg(method)
  D <- .as_distm(x); n <- nrow(D)
  Y <- switch(method,
    pca  = stats::cmdscale(stats::as.dist(D), k = 2),
    umap = { if (!requireNamespace("uwot", quietly = TRUE))
               stop("method = 'umap' needs the 'uwot' package.")
             uwot::umap(stats::as.dist(D), n_neighbors = min(15, n - 1), n_components = 2) },
    tsne = { if (!requireNamespace("Rtsne", quietly = TRUE))
               stop("method = 'tsne' needs the 'Rtsne' package.")
             Rtsne::Rtsne(as.dist(D), is_distance = TRUE, dims = 2,
                          perplexity = max(5, min(30, floor((n - 1) / 3))))$Y })
  cl <- factor(labels)
  if (is.null(col)) col <- .mc_pal(nlevels(cl))
  if (is.null(main)) main <- paste0(toupper(method), " coloured by cluster")
  plot(Y, col = col[as.integer(cl)], pch = 19, xlab = paste(toupper(method), "1"),
       ylab = paste(toupper(method), "2"), main = main)
  graphics::legend("topright", levels(cl), col = col[seq_len(nlevels(cl))],
                   pch = 19, bty = "n", title = "cluster", cex = 0.8)
  invisible(Y)
}

#' Visually compare clustering solutions with ComparePlot (no ground truth)
#'
#' Aligns several clustering solutions of the same objects and draws them with
#' \code{\link{ComparePlot}} so their agreement can be read off directly -- which
#' objects the methods place together and where they disagree -- without any
#' ground truth. Each solution may be a fitted result (with \code{$Clust} /
#' \code{$DistM}) or a plain label vector; label vectors are turned into a
#' co-membership dissimilarity so every solution is comparable.
#'
#' @param solutions A named list of fitted results and/or label vectors, all over
#'   the same objects in the same order.
#' @param nrclusters Number of clusters to cut each solution into.
#' @param cols Optional cluster colours; defaults to a distinct palette.
#' @param cex.labels,margins Passed to \code{\link{ComparePlot}}.
#' @param ... Further arguments to \code{\link{ComparePlot}}.
#' @return Invisibly \code{NULL}; called for the plot. Needs \pkg{circlize} and
#'   \pkg{plotrix}.
#' @seealso \code{\link{clustering_concordance}}, \code{\link{ComparePlot}}
#' @examples
#' data(mosaic_toy)
#' sols <- list(
#'   var    = M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3, weighting = c("var", "var")),
#'   nugget = M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3, weighting = c("nugget", "nugget")),
#'   NEMO   = NEMO(mosaic_toy$List, k = 3)$cluster)
#' \dontrun{compare_methods_plot(sols, nrclusters = 3)}
#' @export
compare_methods_plot <- function(solutions, nrclusters, cols = NULL,
                                 cex.labels = 0.5, margins = c(5, 11, 3, 2), ...) {
  if (!requireNamespace("circlize", quietly = TRUE) ||
      !requireNamespace("plotrix", quietly = TRUE))
    stop("compare_methods_plot() needs the 'circlize' and 'plotrix' packages.")
  # object names: from the first fit's DistM, else first labelled vector, else index
  first <- solutions[[1]]
  samp  <- if (is.list(first) && !is.null(first$DistM)) rownames(as.matrix(first$DistM))
           else names(first)
  n <- if (is.list(first) && !is.null(first$DistM)) nrow(as.matrix(first$DistM)) else length(first)
  if (is.null(samp)) samp <- sprintf("obj%03d", seq_len(n))

  # build a hierarchy + co-membership distance from *any* solution shape:
  # a fitted result with a usable $Clust, or a plain label vector, and fall back
  # to the object's labels (via mosaic_labels) if $Clust cannot be coerced.
  from_labels <- function(cl) {
    cl <- as.integer(factor(cl)); if (length(cl) != length(samp)) cl <- rep(1L, length(samp))
    D  <- outer(cl, cl, function(a, b) as.numeric(a != b)); dimnames(D) <- list(samp, samp)
    hc <- stats::as.hclust(cluster::agnes(stats::as.dist(D), method = "average"))
    hc$labels <- samp; list(hc = hc, D = D)
  }
  wrap <- function(x) {
    hc <- NULL; D <- NULL
    if (is.list(x) && !is.null(x$Clust)) {
      hc <- tryCatch(stats::as.hclust(x$Clust), error = function(e) NULL)
      if (!is.null(hc)) {
        D <- tryCatch(as.matrix(x$DistM), error = function(e) NULL)
        if (is.null(D) || length(D) == 0L || nrow(D) != length(samp)) {
          cl <- stats::cutree(hc, min(nrclusters, length(hc$order)))
          D  <- outer(cl, cl, function(a, b) as.numeric(a != b))
        }
        if (is.null(rownames(D))) dimnames(D) <- list(samp, samp)
        if (is.null(hc$labels)) hc$labels <- samp
      }
    }
    if (is.null(hc)) {                       # label vector, or a $Clust we could not coerce
      cl <- if (is.list(x)) tryCatch(mosaic_labels(x, nrclusters), error = function(e) NULL) else x
      fl <- from_labels(if (is.null(cl)) rep(1L, length(samp)) else cl)
      hc <- fl$hc; D <- fl$D
    }
    hc$order.lab <- hc$labels[hc$order]
    el <- list(Clust = hc, DistM = D); attr(el, "method") <- "Single Clustering"; el
  }
  res <- lapply(solutions, wrap)
  if (is.null(cols)) cols <- .mc_pal(nrclusters)
  ComparePlot(res, nrclusters = nrclusters, cols = cols, names = names(solutions),
              circle = FALSE, plottype = "sweave", cex.labels = cex.labels,
              margins = margins, ...)
  invisible(NULL)
}

#' Compare clustering solutions to each other (no ground truth needed)
#'
#' Given several labellings of the same objects (e.g. different weightings or
#' methods), compute their pairwise agreement (ARI) and each object's stability
#' -- how consistently it is co-clustered across the solutions. Useful for real
#' data where no ground truth is available.
#'
#' @param labels_list A named list of label vectors, all of the same length.
#' @return A list with \code{agreement} (pairwise ARI matrix), \code{stability}
#'   (mean pairwise co-membership per object) and \code{changed} (objects whose
#'   label is not constant across solutions).
#' @seealso \code{\link{cluster_agreement}}, \code{\link{compare_clusterings}}
#' @examples
#' data(mosaic_toy)
#' v <- mosaic_labels(M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3,
#'                            weighting = c("var", "var")), 3)
#' n <- mosaic_labels(M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3,
#'                            weighting = c("nugget", "nugget")), 3)
#' clustering_concordance(list(var = v, nugget = n))$agreement
#' @export
clustering_concordance <- function(labels_list) {
  stopifnot(length(labels_list) >= 2)
  nm <- names(labels_list); if (is.null(nm)) nm <- paste0("sol", seq_along(labels_list))
  m  <- length(labels_list); n <- length(labels_list[[1]])
  A  <- matrix(1, m, m, dimnames = list(nm, nm))
  for (i in seq_len(m)) for (j in seq_len(m)) if (i < j) {
    a <- cluster_agreement(labels_list[[i]], labels_list[[j]])[["ARI"]]
    A[i, j] <- A[j, i] <- a
  }
  # per-object stability: average pairwise co-membership agreement across solutions
  co <- matrix(0, n, n)
  for (l in labels_list) co <- co + outer(l, l, function(a, b) as.numeric(a == b))
  co <- co / m
  stability <- (rowSums(co) - 1) / (n - 1)
  changed <- which(apply(vapply(labels_list, function(l)
    match(l, unique(l)), integer(n)), 1, function(r) length(unique(r)) > 1))
  list(agreement = A, stability = stability, changed = changed)
}

#' @title Find differentially expressed genes
#'
#' @description The \code{DiffGenes} function looks for differentially expressed
#' genes for each cluster of a clustering result by comparing the objects of a
#' cluster against all other objects. The limma method is applied to find the
#' differentially expressed genes.
#' @export
#' @param List A list of the clustering outputs to be compared. The first
#' element of the list will be used as the reference in
#' \code{ReorderToReference}.
#' @param Selection If differential gene expression should be investigated for
#' a specific selection of objects, this selection can be provided here.
#' Selection can be of the type "character" (names of the objects) or
#' "numeric" (the number of specific cluster). Default is NULL.
#' @param geneExpr The gene expression matrix or ExpressionSet of the objects.
#' The rows should correspond with the genes.
#' @param nrclusters Optional. The number of clusters to cut the dendrogram in.
#' Default is NULL.
#' @param method The method to applied to look for DE genes. For now, only the
#' limma method is available. Default is "limma".
#' @param sign The significance level to be handled. Default is 0.05.
#' @param topG Overrules sign. The number of top genes to be shown. Default is NULL.
#' @param fusionsLog Logical. To be handed to \code{ReorderToReference}: indicator for the fusion of clusters. Default is TRUE
#' @param weightclust Logical. To be handed to \code{ReorderToReference}: to be used for the outputs of CEC,
#' WeightedClust or WeightedSimClust. If TRUE, only the result of the Clust element is considered. Default is TRUE.
#' @param names Optional. Names of the methods. Default is NULL.
#' @return The result is a list with an element per method. Each element is
#' again a list with for each cluster the found differentially expressed genes.
#' @details This function relies on the suggested Bioconductor package 'limma'.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#' data(geneMat)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(MCF7_F,MCF7_T)
#' names=c('FP','TP')
#'
#' MCF7_FT_DE = DiffGenes(List=L,geneExpr=geneMat,nrclusters=7,method="limma",sign=0.05,
#' topG=10,fusionsLog=TRUE,weightclust=TRUE,names=names)
#' }
DiffGenes=function(List,Selection=NULL,geneExpr=NULL,nrclusters=NULL,method="limma",sign=0.05,topG=NULL,fusionsLog=TRUE,weightclust=TRUE,names=NULL){
	if (!requireNamespace("limma", quietly = TRUE)) stop("DiffGenes() requires the suggested package 'limma' (Bioconductor).")
	if(method != "limma"){
		stop("Only the limma method is implemented to find differentially expressed genes")
	}
	if(!is.null(Selection)){
		ResultLimma=DiffGenesSelection(List,Selection,geneExpr,nrclusters,method,sign,topG,fusionsLog,weightclust,names)
	}
	else{

		ListNew=list()
		element=0
		for(i in 1:length(List)){
			if(attributes(List[[i]])$method != "CEC" & attributes(List[[i]])$method != "Weighted" & attributes(List[[i]])$method!= "WeightedSim"){
				ResultsClust=list()
				ResultsClust[[1]]=list()
				ResultsClust[[1]][[1]]=List[[i]]
				names(ResultsClust[[1]])[1]="Clust"
				element=element+1
				ListNew[[element]]=ResultsClust[[1]]
				#attr(ListNew[element],"method")="Weights"
			}
			else if(attributes(List[[i]])$method=="CEC" | attributes(List[[i]])$method=="Weighted" | attributes(List[[i]])$method == "WeightedSim"){
				ResultsClust=list()
				if(weightclust==TRUE){
					ResultsClust[[1]]=list()
					if(attributes(List[[i]])$method != "WeightedSim"){
						ResultsClust[[1]][[1]]=List[[i]]$Clust
						names(ResultsClust[[1]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[1]]
						attr(ListNew[element],"method")="Weights"
					}
					else{
						ResultsClust[[1]]=list()
						ResultsClust[[1]][[1]]=List[[i]]
						names(ResultsClust[[1]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[1]]
					}
				}
				else{
					for (j in 1:length(List[[i]]$Results)){
						ResultsClust[[j]]=list()
						ResultsClust[[j]][[1]]=List[[i]]$Results[[j]]
						names(ResultsClust[[j]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[j]]
						attr(ListNew[element],"method")="Weights"
					}
				}
			}
		}

		if(is.null(names)){
			names=seq(1,length(ListNew),1)
			for(i in 1:length(ListNew)){
				names[i]=paste("Method",i,sep=" ")
			}
		}

		if(is.null(names)){
			for(j in 1:length(List)){
				names[j]=paste("Method",j,sep=" ")
			}
		}

		names(ListNew)=names

		if(is.null(topG)){
			top1=FALSE
		}
		else{
			top1=TRUE
		}


		MatrixClusters=ReorderToReference(List,nrclusters,fusionsLog,weightclust,names)
		List=ListNew
		ResultLimma=list()
		maxclus=0
		for (k in 1:dim(MatrixClusters)[1]){
			clusters=MatrixClusters[k,]
			if(max(clusters)>maxclus){
				maxclus=max(clusters)
			}
			Genes=list()
			clust=sort(unique(clusters)) #does not matter: Genes[i] puts right elements on right places
			hc<-stats::as.hclust(List[[k]]$Clust$Clust)
			OrderedCpds <- hc$labels[hc$order]
			for (i in clust){

				temp=list()
				LeadCpds=names(clusters)[which(clusters==i)]
				temp[[1]]=list(LeadCpds,OrderedCpds)
				names(temp[[1]])=c("LeadCpds","OrderedCpds") #names of the objects

				label = rep(0,length(names(clusters)))
				label[which(clusters==i)] = 1

				label.factor = factor(label)
				GeneExpr.2=geneExpr[,names(clusters)]

				if(class(GeneExpr.2)[1]=="ExpressionSet"){

					if (!requireNamespace("a4Base", quietly = TRUE)) {
						stop("a4Base needed for this function to work. Please install it.",
								call. = FALSE)
					}


					GeneExpr.2$LeadCmpds<-label.factor
					DElead <- a4Base::limmaTwoLevels(GeneExpr.2,"LeadCpds")

					allDE <-a4Core::topTable(DElead, n = length(DElead@MArrayLM$genes$SYMBOL),sort.by="p")

					if(is.null(allDE$ID)){
						allDE$ID<- rownames(allDE)
					}
					else
					{
						allDE$ID=allDE$ID
					}

					if(top1==TRUE){
						result = list(allDE[1:topG,],allDE)
						names(result)=c("TopDE","AllDE")

					}
					else if(top1==FALSE){
						topG=length(which(allDE$adj.P.Val<=sign))
						result = list(allDE[1:topG,],allDE)
						names(result)=c("TopDE","AllDE")

					}

				}
				else{

					design = stats::model.matrix(~label.factor)
					fit = limma::lmFit(GeneExpr.2,design=design)
					fit = limma::eBayes(fit)
					allDE=limma::topTable(fit,n=dim(geneExpr)[1],coef=2,adjust="fdr",sort.by="P")

					if(is.null(allDE$ID)){
						allDE$ID <- rownames(allDE)
					}
					else
					{
						allDE$ID=allDE$ID
					}

					if(top1==TRUE){
						result = list(allDE[1:topG,],allDE)
						names(result)=c("TopDE","AllDE")
					}
					else if(top1==FALSE){
						topG=length(which(allDE$adj.P.Val<=sign))
						result = list(allDE[1:topG,],allDE)
						names(result)=c("TopDE","AllDE")
					}
				}

				temp[[2]]=result

				names(temp)=c("objects","Genes")

				Genes[[i]]=temp

				names(Genes)[i]=paste("Cluster",i,sep=" ")
			}
			ResultLimma[[k]]=Genes

		}
		names(ResultLimma)=names
		for(i in 1:length(ResultLimma)){
			for(k in 1:length(ResultLimma[[i]])){
				if(is.null(ResultLimma[[i]][[k]])[1]){
					ResultLimma[[i]][[k]]=NA
					names(ResultLimma[[i]])[k]=paste("Cluster",k,sep=" ")
				}
			}
			if(length(ResultLimma[[i]]) != maxclus){
				extra=maxclus-length(ResultLimma[[i]])
				#temp=length(ResultLimma[[i]])
				for(j in 1:extra){
					ResultLimma[[i]][[length(ResultLimma[[i]])+1]]=NA
					names(ResultLimma[[i]])[length(ResultLimma[[i]])]=paste("Cluster",length(ResultLimma[[i]]),sep=" ")
				}
			}
		}

	}
	return(ResultLimma)
}

#' @title Differential expression for a selection of objects
#' @param List A list of the clustering outputs to be compared. The first
#' element of the list will be used as the reference in
#' \code{ReorderToReference}.
#' @param Selection If differential gene expression should be investigated for
#' a specific selection of objects, this selection can be provided here.
#' Selection can be of the type "character" (names of the objects) or
#' "numeric" (the number of specific cluster). Default is NULL.
#' @param geneExpr The gene expression matrix or ExpressionSet of the objects.
#' The rows should correspond with the genes.
#' @param nrclusters Optional. The number of clusters to cut the dendrogram in.
#' The number of clusters should not be specified if the interest lies only in
#' a specific selection of objects which is known by name.  Otherwise, it is
#' required. Default is NULL.
#' @param method The method to applied to look for DE genes. For now, only the
#' limma method is available. Default is "limma".
#' @param sign The significance level to be handled. Default is 0.05.
#' @param topG Overrules sign. The number of top genes to be shown. Default is NULL.
#' @param fusionsLog Logical. To be handed to \code{ReorderToReference}: indicator for the fusion of clusters. Default is TRUE
#' @param weightclust Logical. To be handed to \code{ReorderToReference}: to be used for the outputs of CEC,
#' WeightedClust or WeightedSimClust. If TRUE, only the result of the Clust element is considered. Default is TRUE.
#' @param names Optional. Names of the methods. Default is NULL.
#' @description Internal function of \code{DiffGenes}.
#' @export
#' @examples
#' \dontrun{
#' MCF7_FT_DE = DiffGenes(List=L,Selection=c("Cpd1","Cpd2"),geneExpr=geneMat,method="limma")
#' }
DiffGenesSelection=function(List,Selection,geneExpr=NULL,nrclusters=NULL,method="limma",sign=0.05,topG=NULL,fusionsLog=TRUE,weightclust=TRUE,names=NULL){
	if (!requireNamespace("limma", quietly = TRUE)) stop("DiffGenesSelection() requires the suggested package 'limma' (Bioconductor).")
	if(method != "limma"){
		stop("Only the limma method is implemented to find differentially expressed genes")
	}

	if(is.null(topG)){
		top1=FALSE
	}
	else{
		top1=TRUE
	}

	if(inherits(Selection,"character")){
		ResultLimma=list()
		Genes=list()
		temp=list()

		LeadCpds=Selection #names of the objects
		OrderedCpds=colnames(geneExpr)
		temp[[1]]=list(LeadCpds,OrderedCpds)
		names(temp[[1]])=c("LeadCpds","OrderedCpds")

		label = rep(0,dim(geneExpr)[2])
		label[which(colnames(geneExpr)%in%Selection)] = 1
		label.factor = factor(label)


		if(class(geneExpr)[1]=="ExpressionSet"){

			if (!requireNamespace("a4Base", quietly = TRUE)) {
				stop("a4Base needed for this function to work. Please install it.",
						call. = FALSE)
			}

			geneExpr$LeadCmpds<-label.factor
			DElead <- a4Base::limmaTwoLevels(geneExpr,"LeadCpds")

			allDE <- a4Core::topTable(DElead, n = length(DElead@MArrayLM$genes$SYMBOL),sort.by="p")
			if(is.null(allDE$ID)){
				allDE$ID <- rownames(allDE)
			}
			else
			{
				allDE$ID=allDE$ID
			}
			if(top1==TRUE){
				result = list(allDE[1:topG,],allDE)
				names(result)=c("TopDE","AllDE")

			}
			else if(top1==FALSE){
				topG=length(which(allDE$adj.P.Val<=sign))
				result = list(allDE[0:topG,],allDE)
				names(result)=c("TopDE","AllDE")

			}

		}
		else{

			design = stats::model.matrix(~label.factor)
			fit = limma::lmFit(geneExpr,design=design)
			fit = limma::eBayes(fit)

			allDE=limma::topTable(fit,coef=2,n=dim(geneExpr)[1],adjust="fdr",sort.by="P")
			if(is.null(allDE$ID)){
				allDE$ID <- rownames(allDE)
			}
			else
			{
				allDE$ID=allDE$ID
			}
			if(top1==TRUE){
				result = list(allDE[0:topG,],allDE)
				names(result)=c("TopDE","AllDE")

			}
			else if(top1==FALSE){
				topG=length(which(allDE$adj.P.Val<=sign))
				result = list(allDE[0:topG,],allDE)
				names(result)=c("TopDE","AllDE")

			}

		}
		temp[[2]]=result

		names(temp)=c("objects","Genes")
		ResultLimma[[1]]=temp
		names(ResultLimma)="Selection"

	}
	else if(inherits(Selection,"numeric") & !(is.null(List))){

		ListNew=list()
		element=0

		ListNew=list()
		element=0
		for(i in 1:length(List)){
			if(attributes(List[[i]])$method != "CEC" & attributes(List[[i]])$method != "Weighted" & attributes(List[[i]])$method!= "WeightedSim"){
				ResultsClust=list()
				ResultsClust[[1]]=list()
				ResultsClust[[1]][[1]]=List[[i]]
				names(ResultsClust[[1]])[1]="Clust"
				element=element+1
				ListNew[[element]]=ResultsClust[[1]]
				#attr(ListNew[element],"method")="Weights"
			}
			else if(attributes(List[[i]])$method=="CEC" | attributes(List[[i]])$method=="Weighted" | attributes(List[[i]])$method == "WeightedSim"){
				ResultsClust=list()
				if(weightclust==TRUE){
					ResultsClust[[1]]=list()
					if(attributes(List[[i]])$method != "WeightedSim"){
						ResultsClust[[1]][[1]]=List[[i]]$Clust
						names(ResultsClust[[1]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[1]]
						attr(ListNew[element],"method")="Weights"
					}
					else{
						ResultsClust[[1]]=list()
						ResultsClust[[1]][[1]]=List[[i]]
						names(ResultsClust[[1]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[1]]
					}
				}
				else{
					for (j in 1:length(List[[i]]$Results)){
						ResultsClust[[j]]=list()
						ResultsClust[[j]][[1]]=List[[i]]$Results[[j]]
						names(ResultsClust[[j]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[j]]
						attr(ListNew[element],"method")="Weights"
					}
				}
			}
		}

		if(is.null(names)){
			names=seq(1,length(ListNew),1)
			for(i in 1:length(ListNew)){
				names[i]=paste("Method",i,sep=" ")
			}
		}
		names(ListNew)=names

		Matrix=ReorderToReference(List,nrclusters,fusionsLog,weightclust,names)

		List=ListNew
		ResultLimma=list()
		for(k in 1:dim(Matrix)[1]){
			cluster=Selection

			hc<-stats::as.hclust(List[[k]]$Clust$Clust)
			OrderedCpds <- hc$labels[hc$order]

			Genes=list()
			temp=list()
			LeadCpds=colnames(Matrix)[which(Matrix[k,]==cluster)] #names of the objects
			temp[[1]]=list(LeadCpds,OrderedCpds)
			names(temp[[1]])=c("LeadCpds","OrderedCpds")

			label = rep(0,dim(Matrix)[2])
			label[which(Matrix[k,]==cluster)] = 1
			label.factor = factor(label)

			GeneExpr.2=geneExpr[,colnames(Matrix)]

			if(class(GeneExpr.2)[1]=="ExpressionSet"){

				if (!requireNamespace("a4Base", quietly = TRUE)) {
					stop("a4Base needed for this function to work. Please install it.",
							call. = FALSE)
				}

				GeneExpr.2$LeadCmpds<-label.factor
				DElead <- a4Base::limmaTwoLevels(GeneExpr.2,"LeadCpds")

				allDE <- a4Core::topTable(DElead, n = length(DElead@MArrayLM$genes$SYMBOL),sort.by="p")
				if(is.null(allDE$ID)){
					allDE$ID <- rownames(allDE)
				}
				else
				{
					allDE$ID=allDE$ID
				}
				if(top1==TRUE){
					result = list(allDE[0:topG,],allDE)
					names(result)=c("TopDE","AllDE")

				}
				else if(top1==FALSE){
					topG=length(which(allDE$adj.P.Val<=sign))
					result = list(allDE[0:topG,],allDE)
					names(result)=c("TopDE","AllDE")

				}

			}
			else{


				design = stats::model.matrix(~label.factor)
				fit = limma::lmFit(GeneExpr.2,design=design)
				fit = limma::eBayes(fit)

				allDE=limma::topTable(fit,coef=2,n=dim(geneExpr)[1],adjust="fdr",sort.by="P")
				if(is.null(allDE$ID)){
					allDE$ID <- rownames(allDE)
				}
				else
				{
					allDE$ID=allDE$ID
				}
				if(top1==TRUE){
					result = list(allDE[1:topG,],allDE)
					names(result)=c("TopDE","AllDE")

				}
				else if(top1==FALSE){
					topG=length(which(allDE$adj.P.Val<=sign))

					result = list(allDE[0:topG,],allDE)
					names(result)=c("TopDE","AllDE")

				}

			}
			temp[[2]]=result

			names(temp)=c("objects","Genes")
			ResultLimma[[k]]=temp
			names(ResultLimma)[k]=paste(names[k],": Cluster", cluster, sep="")
		}
	}

	else{
		message("If a specific cluster is specified, clustering results must be provided in List")
	}
	return(ResultLimma)

}

#' @title Find the genes shared across methods
#'
#' @description The \code{FindGenes} function collects the differentially
#' expressed genes per cluster per method (the output of \code{DiffGenes}) and
#' determines which genes are found across the methods.
#' @export
#' @param dataLimma The output of the \code{DiffGenes} function.
#' @param names Optional. Names of the methods. Default is NULL.
#' @return The result is a list with for each cluster a list of the found genes
#' and the methods that found them.
#' @examples
#' \dontrun{
#' MCF7_SharedGenes=FindGenes(dataLimma=MCF7_FT_DE,names=c("FP","TP"))
#' }
FindGenes<-function(dataLimma,names=NULL){

	FoundGenes=list()

	if(is.null(names)){
		for(j in 1:length(dataLimma)){
			names[j]=paste("Method",j,sep=" ")
		}
	}

	nrclusters=length(dataLimma[[1]])


	for(j in 1:nrclusters){
		FoundGenes[[j]]=list()
	}

	for(i in 1:length(dataLimma)){ #i == method

		for(j in 1:nrclusters){ #j == cluster
			if(!(is.na(dataLimma[[i]][[j]]))[1]){

				tempgenes=dataLimma[[i]][[j]]$Genes$TopDE$ID


				if(!(is.null(tempgenes))){
					tempgenes=tempgenes[!(is.na(tempgenes))]
					for(k in 1:length(tempgenes)){
						if(!(tempgenes[k] %in% names(FoundGenes[[j]]))){
							#FoundGenes[[j]][[length(FoundGenes[[j]])+1]]=c()
							FoundGenes[[j]][[length(FoundGenes[[j]])+1]]=names[i]
							names(FoundGenes[[j]])[length(FoundGenes[[j]])]=tempgenes[k]

						}
						else if (tempgenes[k] %in% names(FoundGenes[[j]])){
							found=which(names(FoundGenes[[j]])==tempgenes[k])
							FoundGenes[[j]][[found]]=c(FoundGenes[[j]][[found]],names[i])
						}
					}
				}
			}
		}
	}

	for(l in 1:nrclusters){
		namesl=names(FoundGenes[[l]])
		if(!(is.null(namesl))){

			for(m in l:nrclusters){
				namesm=names(FoundGenes[[m]])
				Templist=FoundGenes[[m]]
				if(length(namesm)!=0){
					if(l != m){

						for(k in 1:length(namesm)){

							if (namesm[k] %in% namesl){
								found=which(namesl==namesm[k])
								methods=Templist[[k]]
								del=which(names(FoundGenes[[m]])==namesm[k])
								FoundGenes[[m]][[del]]=c()
								for(a in 1:length(methods)){
									methods[a]=paste(methods[a],"_",m,sep="")
								}
								FoundGenes[[l]][[found]]=c(FoundGenes[[l]][[found]],methods)
							}

						}
					}
				}
			}
		}
	}


	for(i in 1:length(FoundGenes)){
		names(FoundGenes)[i]=paste("Cluster",i,sep=" ")
	}

	return(FoundGenes)
}

#' @title Intersection over resulting gene sets of \code{PathwaysIter} function
#'
#' @description The function \code{Geneset.intersect} collects the results of the
#' \code{PathwaysIter} function per method for each cluster and takes the
#' intersection over the iterations per cluster per method. This is to see if
#' over the different resamplings of the data, similar pathways were
#' discovered.
#' @export Geneset.intersect
#'
#' @param PathwaysOutput The output of the \code{PathwaysIter} function.
#' @param Selection Logical. Indicates whether or not the output of the
#' pathways function were concentrated on a specific selection of objects. If
#' this was the case, Selection should be put to TRUE. Otherwise, it should be
#' put to FALSE. Default is TRUE.
#' @param sign The significance level to be handled for cutting of the
#' pathways. Default is 0.05.
#' @param names Optional. Names of the methods. Default is NULL.
#' @param seperatetables Logical. If TRUE, a separate element is created per
#' cluster containing the pathways for each iteration. Default is FALSE.
#' @param separatepvals Logical. If TRUE, the p-values of the each iteration of
#' each pathway in the intersection is given. If FALSE, only the mean p-value
#' is provided. Default is FALSE.
#' @return The output is a list with an element per method. For each method, it
#' is portrayed per cluster which pathways belong to the intersection over all
#' iterations and their corresponding mean p-values.
#' @examples
#'
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#' data(geneMat)
#' data(GeneInfo)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(MCF7_F,MCF7_T)
#'
#' MCF7_Paths_FandT=PathwaysIter(List=L, geneExpr = geneMat, nrclusters = 7, method =
#' c("limma", "MLP"), geneInfo = GeneInfo, geneSetSource = "GOBP", topP = NULL,
#' topG = NULL, GENESET = NULL, sign = 0.05,niter=2,fusionsLog = TRUE,
#' weightclust = TRUE, names =names)
#'
#' MCF7_Paths_intersection=Geneset.intersect(PathwaysOutput=MCF7_Paths_FandT,
#' sign=0.05,names=c("FP","TP"),seperatetables=FALSE,separatepvals=FALSE)
#'
#' str(MCF7_Paths_intersection)
#' }
Geneset.intersect<-function(PathwaysOutput,Selection=FALSE,sign=0.05,names=NULL,seperatetables=FALSE,separatepvals=FALSE){

	if (!requireNamespace("plyr", quietly = TRUE)) stop("Geneset.intersect() requires the suggested package 'plyr'.")

	if(Selection==TRUE){
		if(length(PathwaysOutput$'Iteration 1')==1){
			names="Selection"
		}
		Intersect=Geneset.intersectSelection(PathwaysOutput,sign,names,seperatetables,separatepvals)
	}

	else{

		if(is.null(names)){
			for(j in 1:length(PathwaysOutput$"Iteration 1")){
				names[j]=paste("Method",j,sep=" ")
			}
		}


		#put all of same method together:preparation of lists
		subsets=list()
		nmethods=length(PathwaysOutput$"Iteration 1")
		for(i in 1:nmethods){
			subsets[[i]]=list()

		}
		names(subsets)=names

		#put all of same method together: go through PathwaysOutput
		for(j in 1:length(PathwaysOutput)){
			name1=names(PathwaysOutput)[j]
			for(k in 1:nmethods){
				name2=names[k]
				subsets[[name2]][[name1]]=PathwaysOutput[[name1]][[name2]]

			}

		}

		#for every subset (= every method) take intersection over the interations per cluster
		Intersect=list()

		for(i in 1:length(subsets)){
			Method=subsets[[i]]
			Clusters=list()
			nclus=length(Method[[1]])
			for(j in 1:length(Method)){
				name3=paste("Iteration",j,sep=" ")
				for(k in 1:nclus){
					name4=paste("Cluster",k,sep=" ")
					if(!(is.na(Method[[name3]][[name4]])[1])){
						Clusters[[name4]][[name3]]=Method[[name3]][[name4]]
					}
					else{
						Clusters[[name4]][[name3]]=NA
					}
				}

			}

			IntersectM=list()

			for(a in 1:length(Clusters)){ #per cluster
				if(!(is.na(Clusters[[a]])[1])){
					result.out=list()
					result.name = c()
					for(b in 1:length(Clusters[[a]])){#per iteration
						if(b==1){
							objects=Clusters[[a]][[1]]$objects
							Genes=Clusters[[a]][[1]]$Genes
							Names=data.frame("description"=Clusters[[a]][[1]]$Pathways$AllPaths$geneSetDescription,"genesetcode"=rownames(Clusters[[a]][[1]]$Pathways$AllPaths))
							Names$description=as.character(Names$description)
							Names$genesetcode=as.character(Names$genesetcode)
						}
						cut = Clusters[[a]][[b]]$Pathways$AllPaths[Clusters[[a]][[b]]$Pathways$AllPaths$geneSetPValue<=sign,]
						colnames(cut)[4] = paste("pvalues.",b,sep="")
						colnames(cut)[2] = paste("testedgenesetsize.",b,sep="")
						colnames(cut)[3] = paste("genesetstatistic.",b,sep="")
						cut=cut[,c(1,5,2,3,4)]
						result.out[[b]] = cut
						result.name = c(result.name,paste("genesettable",b,sep=""))

					}



					names(result.out) = result.name

					genesets.table.intersect = plyr::join_all(result.out,by=c("totalGeneSetSize","geneSetDescription"),type="inner")
					genesets.table.intersect$mean_testedGeneSetSize=round(apply(genesets.table.intersect[,which(substring(colnames(genesets.table.intersect),1,nchar(colnames(genesets.table.intersect))-nchar(".1"))=='testedgenesetsize')],1,mean),1)
					genesets.table.intersect$mean_geneSetStatistic=apply(genesets.table.intersect[,which(substring(colnames(genesets.table.intersect),1,nchar(colnames(genesets.table.intersect))-nchar(".1"))=='genesetstatistic')],1,mean)
					genesets.table.intersect$mean_geneSetPValue=apply(genesets.table.intersect[,which(substring(colnames(genesets.table.intersect),1,nchar(colnames(genesets.table.intersect))-nchar(".1"))=='pvalues')],1,mean)

					rownames(genesets.table.intersect)=as.character(Names[which(genesets.table.intersect$geneSetDescription%in%Names[,1]),2])

					class(genesets.table.intersect)=c("MLP","data.frame")
					attr(genesets.table.intersect,'geneSetSource')=attributes(Clusters[[1]][[1]]$Pathways$AllPaths)$geneSetSource


					result.out$genesets.table.intersect = genesets.table.intersect



					if(separatepvals==FALSE){
						result.out$genesets.table.intersect=genesets.table.intersect[,c(1,2,(ncol(genesets.table.intersect)-2):ncol(genesets.table.intersect))]
						class(result.out$genesets.table.intersect)=c("MLP","data.frame")
						attr(result.out$genesets.table.intersect,'geneSetSource')=attributes(Clusters[[1]][[1]]$Pathways$AllPaths)$geneSetSource
					}


					if(seperatetables==FALSE){
						result.out=result.out$genesets.table.intersect
						class(result.out)=c("MLP","data.frame")
						attr(result.out,'geneSetSource')=attributes(Clusters[[1]][[1]]$Pathways$AllPaths)$geneSetSource
					}

					newresult=list(objects=objects,Genes=Genes,Pathways=result.out)


					IntersectM[[a]]=newresult
					names(IntersectM)[a]=names(Clusters)[[a]]
				}
				else{
					IntersectM[[a]]=NA
					names(IntersectM)[a]=names(Clusters)[[a]]
				}
			}

			Intersect[[i]]=IntersectM

		}
	}
	names(Intersect)=names
	return(Intersect)
}

#' @title Intersection over resulting gene sets of \code{PathwaysIter} function for a selection of objects
#' @param list.output The output of the \code{PathwaysIter} function.
#' @param sign The significance level to be handled for cutting of the
#' pathways. Default is 0.05.
#' @param names Optional. Names of the methods. Default is NULL.
#' @param seperatetables Logical. If TRUE, a separate element is created per
#' cluster containing the pathways for each iteration. Default is FALSE.
#' @param separatepvals Logical. If TRUE, the p-values of the each iteration of
#' each pathway in the intersection is given. If FALSE, only the mean p-value
#' is provided. Default is FALSE.
#' @description Internal function of \code{Geneset.intersect}.
#' @export
#' @examples
#' \dontrun{
#' Geneset.intersectSelection(MCF7_Paths_FandT,sign=0.05)
#' }
Geneset.intersectSelection<-function(list.output,sign=0.05,names=NULL,seperatetables=FALSE,separatepvals=FALSE){
	if (!requireNamespace("plyr", quietly = TRUE)) stop("Geneset.intersectSelection() requires the suggested package 'plyr'.")
	if(is.null(names)){
		for(j in 1:length(list.output$"Iteration 1")){
			names[j]=paste("Method",j,sep=" ")
		}
	}

	#put all of same method together:preparation of lists
	subsets=list()
	nmethods=length(list.output$"Iteration 1")
	for(i in 1:nmethods){
		subsets[[i]]=list()
	}
	names(subsets)=names

	#put all of same method together: go through list.output
	for(j in 1:length(list.output)){
		name1=names(list.output)[j]
		for(k in 1:nmethods){
			name2=k
			subsets[[name2]][[name1]]=list.output[[name1]][[name2]]

		}

	}

	#for every subset (= every method) take intersection over the interations per cluster
	Intersect=list()

	for(i in 1:length(subsets)){
		Method=subsets[[i]]
		Clusters=list()
		nclus=1
		for(j in 1:length(Method)){
			name3=paste("Iteration",j,sep=" ")
			Clusters[[name3]]=Method[[name3]]
		}

		IntersectM=list()

		result.out=list()
		result.name = c()
		for(a in 1:length(Clusters)){ #per cluster
			if(a==1){
				objects=Clusters[[a]]$objects
				Genes=Clusters[[a]]$Genes
				Names=data.frame("description"=Clusters[[a]]$Pathways$AllPaths$geneSetDescription,"genesetcode"=rownames(Clusters[[a]]$Pathways$AllPaths))
				Names$description=as.character(Names$description)
				Names$genesetcode=as.character(Names$genesetcode)
			}
			cut = Clusters[[a]]$Pathways$AllPaths[Clusters[[a]]$Pathways$AllPaths$geneSetPValue<=sign,]
			colnames(cut)[4] = paste("pvalues.",a,sep="")
			colnames(cut)[2] = paste("testedgenesetsize.",a,sep="")
			colnames(cut)[3] = paste("genesetstatistic.",a,sep="")
			cut=cut[,c(1,5,2,3,4)]
			result.out[[a]] = cut
			result.name = c(result.name,paste("genesettable",a,sep=""))
		}

		names(result.out) = result.name

		genesets.table.intersect = plyr::join_all(result.out,by=c("totalGeneSetSize","geneSetDescription"),type="inner")
		genesets.table.intersect$mean_testedGeneSetSize=round(apply(genesets.table.intersect[,which(substring(colnames(genesets.table.intersect),1,nchar(colnames(genesets.table.intersect))-nchar(".1"))=='testedgenesetsize')],1,mean),1)
		genesets.table.intersect$mean_geneSetStatistic=apply(genesets.table.intersect[,which(substring(colnames(genesets.table.intersect),1,nchar(colnames(genesets.table.intersect))-nchar(".1"))=='genesetstatistic')],1,mean)
		genesets.table.intersect$mean_geneSetPValue=apply(genesets.table.intersect[,which(substring(colnames(genesets.table.intersect),1,nchar(colnames(genesets.table.intersect))-nchar(".1"))=='pvalues')],1,mean)

		rownames(genesets.table.intersect)=as.character(Names[which(genesets.table.intersect$geneSetDescription%in%Names[,1]),2])

		class(genesets.table.intersect)=c("MLP","data.frame")
		attr(genesets.table.intersect,'geneSetSource')=attributes(Clusters[[1]]$Pathways$AllPaths)$geneSetSource


		result.out$genesets.table.intersect = genesets.table.intersect

		if(separatepvals==FALSE){
			result.out$genesets.table.intersect=genesets.table.intersect[,c(1,2,(ncol(genesets.table.intersect)-2):ncol(genesets.table.intersect))]
			class(result.out$genesets.table.intersect)=c("MLP","data.frame")
			attr(result.out$genesets.table.intersect,'geneSetSource')=attributes(Clusters[[1]]$Pathways$AllPaths)$geneSetSource
		}


		if(seperatetables==FALSE){
			result.out=result.out$genesets.table.intersect
			class(result.out)=c("MLP","data.frame")
			attr(result.out,'geneSetSource')=attributes(Clusters[[1]]$Pathways$AllPaths)$geneSetSource
		}


		newresult=list(objects=objects,Genes=Genes,Pathways=result.out)


		#IntersectM[[a]]=
		#names(IntersectM)[a]=names(Clusters)[[a]]
		Intersect[[i]]=newresult

	}
	names(Intersect)=names
	return(Intersect)
}

#' @title Pathway analysis with intersection over iterations
#'
#' @description The \code{PathwayAnalysis} function performs pathway analysis
#' multiple times via \code{PathwaysIter} and takes the intersection over the
#' iterations via \code{Geneset.intersect}.
#' @export
#' @param List A list of clustering outputs or output of the\code{DiffGenes}
#' function.
#' @param Selection If pathway analysis should be conducted for a specific
#' selection of objects, this selection can be provided here. Default is NULL.
#' @param geneExpr The gene expression matrix or ExpressionSet of the objects.
#' @param nrclusters Optional. The number of clusters to cut the dendrogram in. Default is NULL.
#' @param method The methods for gene and pathway analysis. Default is c("limma","MLP").
#' @param geneInfo A data frame with at least the columns ENTREZID and SYMBOL. Default is NULL.
#' @param geneSetSource The source for the getGeneSets function, defaults to "GOBP".
#' @param topP Overrules sign. The number of pathways to display for each cluster. Default is NULL.
#' @param topG Overrules sign. The number of top genes to be returned. Default is NULL.
#' @param GENESET Optional. Can provide own candidate gene sets. Default is NULL.
#' @param sign The significance level to be handled. Default is 0.05.
#' @param niter The number of times to perform pathway analysis. Default is 10.
#' @param fusionsLog Logical. To be handed to \code{ReorderToReference}. Default is TRUE
#' @param weightclust Logical. To be handed to \code{ReorderToReference}. Default is TRUE.
#' @param names Optional. Names of the methods. Default is NULL.
#' @param seperatetables Logical. Default is FALSE.
#' @param separatepvals Logical. Default is FALSE.
#' @return The intersection over the iterations of the pathway analysis.
#' @details This function relies on the suggested Bioconductor packages 'MLP', 'biomaRt' and 'org.Hs.eg.db'.
#' @examples
#' \dontrun{
#' MCF7_PathsFandT=PathwayAnalysis(List=L, geneExpr = geneMat, nrclusters = 7,
#' method = c("limma","MLP"), geneInfo = GeneInfo, geneSetSource = "GOBP",niter=2)
#' }
PathwayAnalysis<-function(List,Selection=NULL,geneExpr=NULL,nrclusters=NULL,method=c("limma", "MLP"),geneInfo=NULL,geneSetSource = "GOBP",topP=NULL,topG=NULL,GENESET=NULL,sign=0.05,niter=10,fusionsLog=TRUE,weightclust=TRUE,names=NULL,seperatetables=FALSE,separatepvals=FALSE){
	if (!requireNamespace("MLP", quietly = TRUE)) {
		stop("MLP needed for this function to work. Please install it.",
				call. = FALSE)
	}

	if (!requireNamespace("biomaRt", quietly = TRUE)) {
		stop("biomaRt needed for this function to work. Please install it.",
				call. = FALSE)
	}

	if (!requireNamespace("org.Hs.eg.db", quietly = TRUE)) {
		stop("org.Hs.eg.db needed for this function to work. Please install it.",
				call. = FALSE)
	}

	Pathways=PathwaysIter(List,Selection,geneExpr,nrclusters,method,geneInfo,geneSetSource,topP,topG,GENESET,sign,niter,fusionsLog,weightclust,names)

	if(is.null(Selection)){
		Selection=FALSE
	}
	else{
		Selection=TRUE
	}

	Intersection=Geneset.intersect(PathwaysOutput=Pathways,Selection,sign,names,seperatetables,separatepvals)

	return(Intersection)

}

#' @title Pathway analysis for multiple clustering results
#'
#' @description A pathway analysis per cluster per method is conducted.
#' @export Pathways
#' @details After finding differently expressed genes, it can be investigated whether
#' pathways are related to those genes. This can be done with the help of the
#' function \code{Pathways} which makes use of the \code{MLP} function of the
#' MLP package. Given the output of a method, the cutree function is performed
#' which results into a specific number of clusters. For each cluster, the
#' limma method is performed comparing this cluster to the other clusters. This
#' to obtain the necessary p-values of the genes. These are used as the input
#' for the \code{MLP} function to find interesting pathways. By default the
#' candidate gene sets are determined by the \code{AnnotateEntrezIDtoGO}
#' function. The default source will be GOBP, but this can be altered.
#' Further, it is also possible to provide own candidate gene sets in the form
#' of a list of pathway categories in which each component contains a vector of
#' Entrez Gene identifiers related to that particular pathway. The default
#' values for the minimum and maximum number of genes in a gene set for it to
#' be considered were used. For MLP this is respectively 5 and 100. If a list
#' of outputs of several methods is provided as data input, the cluster numbers
#' are rearranged according to a reference method. The first method is taken as
#' the reference and ReorderToReference is applied to get the correct ordering.
#' When the clusters haven been re-appointed, the pathway analysis as described
#' above is performed for each cluster of each method.
#' @param List A list of clustering outputs or output of the\code{DiffGenes}
#' function. The first element of the list will be used as the reference in
#' \code{ReorderToReference}. The output of \code{ChooseFeatures} is also
#' accepted.
#' @param Selection If pathway analysis should be conducted for a specific
#' selection of objects, this selection can be provided here. Selection can
#' be of the type "character" (names of the objects) or "numeric" (the number
#' of specific cluster). Default is NULL.
#' @param geneExpr The gene expression matrix or ExpressionSet of the objects.
#' The rows should correspond with the genes.
#' @param nrclusters Optional. The number of clusters to cut the dendrogram in.
#' The number of clusters should not be specified if the interest lies only in
#' a specific selection of objects which is known by name.  Otherwise, it is
#' required. Default is NULL.
#' @param method The method to applied to look for differentially expressed genes and related pathways. For now, only the
#' limma method is available for gene analysis and the MLP method for pathway analysis. Default is c("limma","MLP").
#' @param geneInfo A data frame with at least the columns ENTREZID and SYMBOL.
#' This is necessary to connect the symbolic names of the genes with their
#' EntrezID in the correct order. The order of the gene is here not in the
#' order of the rownames of the gene expression matrix but in the order of
#' their significance. Default is NULL.
#' @param geneSetSource The source for the getGeneSets function, defaults to
#' "GOBP".
#' @param topP Overrules sign. The number of pathways to display for each
#' cluster. If not specified, only the significant genes are shown. Default is NULL.
#' @param topG Overrules sign. The number of top genes to be returned in the
#' result. If not specified, only the significant genes are shown. Defaults is NULL.
#' @param GENESET Optional. Can provide own candidate gene sets. Default is NULL.
#' @param sign The significance level to be handled. Default is 0.05.
#' @param fusionsLog Logical. To be handed to \code{ReorderToReference}: indicator for the fusion of clusters. Default is TRUE
#' @param weightclust Logical. To be handed to \code{ReorderToReference}: to be used for the outputs of CEC,
#' WeightedClust or WeightedSimClust. If TRUE, only the result of the Clust element is considered. Default is TRUE.
#' @param names Optional. Names of the methods. Default is NULL.
#' @return The returned value is a list with an element per cluster per method.
#' @details This function relies on the suggested Bioconductor packages 'MLP', 'biomaRt' and 'org.Hs.eg.db'.
#' @examples
#'
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#' data(geneMat)
#' data(GeneInfo)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(MCF7_F,MCF7_T)
#' names=c('FP','TP')
#'
#' MCF7_PathsFandT=Pathways(List=L, geneExpr = geneMat, nrclusters = 7, method = c("limma",
#' "MLP"), geneInfo = GeneInfo, geneSetSource = "GOBP", topP = NULL,
#' topG = NULL, GENESET = NULL, sign = 0.05,fusionsLog = TRUE, weightclust = TRUE,
#'  names =names)
#'  }
Pathways<-function(List,Selection=NULL,geneExpr=NULL,nrclusters=NULL,method=c("limma", "MLP"),geneInfo=NULL,geneSetSource = "GOBP",topP=NULL,topG=NULL,GENESET=NULL,sign=0.05,fusionsLog=TRUE,weightclust=TRUE,names=NULL){

	if (!requireNamespace("MLP", quietly = TRUE)) {
		stop("MLP needed for this function to work. Please install it.",
				call. = FALSE)
	}

	if (!requireNamespace("biomaRt", quietly = TRUE)) {
		stop("biomaRt needed for this function to work. Please install it.",
				call. = FALSE)
	}

	if (!requireNamespace("org.Hs.eg.db", quietly = TRUE)) {
		stop("org.Hs.eg.db needed for this function to work. Please install it.",
				call. = FALSE)
	}



	if(!(is.null(Selection))){
		ResultMLP=PathwaysSelection(List,Selection,geneExpr,nrclusters,method,geneInfo,geneSetSource,topP,topG,GENESET,sign,fusionsLog,weightclust,names)

	}
	else if(inherits(List,"ChosenClusters")){
		ResultMLP=list()
		for(i in 1:length(List)){
			Selection=List[[i]]$objects$LeadCpds
			L=List[i]
			ResultMLP[[i]]=PathwaysSelection(List=L,Selection,geneExpr,nrclusters,method,geneInfo,geneSetSource,topP,topG,GENESET,sign,fusionsLog,weightclust,names)
			names(ResultMLP)=paste("Choice",i,sep=' ')
		}
	}
	else{

		#Check for gene expression data: if not, reordering to ListNew not necessary
		DataPrepared<-plyr::try_default(PreparePathway(List[[1]],geneExpr,topG,sign),NULL,quiet=TRUE)
		if(is.null(DataPrepared)){

			ListNew=list()
			element=0
			for(i in 1:length(List)){
				if(attributes(List[[i]])$method != "CEC" & attributes(List[[i]])$method != "Weighted" & attributes(List[[i]])$method!= "WeightedSim"){
					ResultsClust=list()
					ResultsClust[[1]]=list()
					ResultsClust[[1]][[1]]=List[[i]]
					names(ResultsClust[[1]])[1]="Clust"
					element=element+1
					ListNew[[element]]=ResultsClust[[1]]
					#attr(ListNew[element],"method")="Weights"
				}
				else if(attributes(List[[i]])$method=="CEC" | attributes(List[[i]])$method=="Weighted" | attributes(List[[i]])$method == "WeightedSim"){
					ResultsClust=list()
					if(weightclust==TRUE){
						ResultsClust[[1]]=list()
						if(attributes(List[[i]])$method != "WeightedSim"){
							ResultsClust[[1]][[1]]=List[[i]]$Clust
							names(ResultsClust[[1]])[1]="Clust"
							element=element+1
							ListNew[[element]]=ResultsClust[[1]]
							attr(ListNew[element],"method")="Weights"
						}
						else{
							ResultsClust[[1]]=list()
							ResultsClust[[1]][[1]]=List[[i]]
							names(ResultsClust[[1]])[1]="Clust"
							element=element+1
							ListNew[[element]]=ResultsClust[[1]]
						}
					}
					else{
						for (j in 1:length(List[[i]]$Results)){
							ResultsClust[[j]]=list()
							ResultsClust[[j]][[1]]=List[[i]]$Results[[j]]
							names(ResultsClust[[j]])[1]="Clust"
							element=element+1
							ListNew[[element]]=ResultsClust[[j]]
							attr(ListNew[element],"method")="Weights"
						}
					}
				}
			}

			if(is.null(names)){
				names=seq(1,length(ListNew),1)
				for(i in 1:length(ListNew)){
					names[i]=paste("Method",i,sep=" ")
				}
			}
			names(ListNew)=names
			MatrixClusters=ReorderToReference(List,nrclusters,fusionsLog,weightclust,names)
			List=ListNew

			maxclus=0
			DataPrepared=list()
			for (k in 1:dim(MatrixClusters)[1]){
				message(k)
				clusters=MatrixClusters[k,]

				if(max(clusters)>maxclus){
					maxclus=max(clusters)
				}

				check<-plyr::try_default(PreparePathway(List[[k]],geneExpr,topG,sign),NULL,quiet=TRUE)
				if(is.null(check)){
					Temp=List[[k]]

					for(i in unique(clusters)){
						objects=list()
						objects$LeadCpds=names(clusters)[which(clusters==i)]
						objects$OrderedCpds=stats::as.hclust(List[[k]]$Clust$Clust)$labels[stats::as.hclust(List[[k]]$Clust$Clust)$order]

						Temp[[i+1]]=list(objects=objects)
						names(Temp)[i+1]=paste("Cluster",i,sep=" ")

					}
					DataPrepared[[k]]<-PreparePathway(Temp,geneExpr,topG,sign)

				}
			}
		}

		else{
			for(k in 1:length(List)){
				DataPrepared[[k]]<-plyr::try_default(PreparePathway(List[[k]],geneExpr,topG,sign),NULL,quiet=TRUE)
				if(is.null(DataPrepared[[k]])){
					Temp=List[[k]]

					for(i in unique(clusters)){
						objects=list()
						objects$LeadCpds=List[[k]]$objects$LeadCpds
						objects$OrderedCpds=List[[k]]$objects$OrderedCpds

						Temp[[i+1]]=list(objects=objects)
						names(Temp)[i+1]=paste("Cluster",i,sep=" ")

					}
					DataPrepared[[k]]<-PreparePathway(Temp,geneExpr,topG,sign)

				}
			}


		}



		method.test = function(sign.method,path.method){
			method.choice = FALSE

			if( sign.method=="limma"  & path.method=="MLP"  ){
				method.choice = TRUE
			}
			if(method.choice==TRUE){
				return(list(sign.method=sign.method,path.method=path.method))
			}
			else{
				stop("Incorrect choice of method.")
			}

		}

		method.out = method.test(method[1],method[2])

		sign.method = method.out$sign.method
		path.method = method.out$path.method

		if(length(geneInfo$ENTREZID)==1){
			geneInfo$ENTREZID = colnames(geneExpr)
		}

		# Determining the genesets if they were not given with the function input
		if((inherits(GENESET,"geneSetMLP"))[1] ){
			geneSet <- GENESET
		}
		else{
			geneSet <- MLP::getGeneSets(species = "Human",geneSetSource = geneSetSource,entrezIdentifiers = geneInfo$ENTREZID)
		}

		if(is.null(topP)){
			top1=FALSE
		}
		else{
			top1=TRUE
		}


		ResultMLP=list()

		for (k in 1:length(DataPrepared)){
			message(k)

			PathwaysResults=list()

			for (i in 1:length(DataPrepared[[k]]$pvalsgenes)){
				message(paste(k,i,sep='.'))
				temp=list()
				temp[[1]]=DataPrepared[[k]]$objects[[i]] # the objects
				temp[[2]]=DataPrepared[[k]]$Genes[[i]]		# the genes
				pvalscluster=DataPrepared[[k]]$pvalsgenes[[i]]

				Entrezs=sapply(names(pvalscluster),function(x) return(geneInfo$ENTREZID[which(geneInfo$SYMBOL==x)]))

				if(path.method=="MLP"){
					## WE WILL USE THE RAW P-VALUES TO PUT IN MLP -> LESS GRANULAR

					names(pvalscluster) = Entrezs

					out.mlp <- MLP::MLP(
							geneSet = geneSet,
							geneStatistic = pvalscluster,
							minGenes = 5,
							maxGenes = 100,
							rowPermutations = TRUE,
							nPermutations = 100,
							smoothPValues = TRUE,
							probabilityVector = c(0.5, 0.9, 0.95, 0.99, 0.999, 0.9999, 0.99999),df = 9,addGeneSetDescription=TRUE)

					output = list()
					#output$gene.p.values = p.adjust(p.values,method="fdr")

					#ranked.genesets.table = data.frame(genesets = (rownames(out.mlp)),p.values = as.numeric(out.mlp$geneSetPValue),descriptions = out.mlp$geneSetDescription)
					#ranked.genesets.table$genesets = as.character(ranked.genesets.table$genesets)
					#ranked.genesets.table$descriptions = as.character(ranked.genesets.table$descriptions)

					#if(is.null(topP)){
					#		topP=length(ranked.genesets.table$p.values<=sign)
					#}

					if(is.null(topP)){
						topP=length(which(out.mlp$geneSetPValue<=sign))
					}

					#TopPaths=ranked.genesets.table[1:topP,]
					#AllPaths=ranked.genesets.table

					TopPaths=out.mlp[1:topP,]
					AllPaths=out.mlp

					output$TopPaths=TopPaths
					attr(output$TopPaths,'geneSetSource')=geneSetSource
					output$AllPaths=AllPaths
					attr(output$AllPaths,'geneSetSource')=geneSetSource
					#output$ranked.genesets.table = ranked.genesets.table[ranked.genesets.table$p.values<=sign,]


					#nr.genesets = c( dim(ranked.genesets.table)[1]  ,  length(geneSet) 	)
					#names(nr.genesets) = c("used.nr.genesets","total.nr.genesets")
					#output$nr.genesets = nr.genesets

					#output$object = out.mlp
					#output$method = "MLP"

					temp[[3]]=output
				}
				names(temp)=c("objects","Genes","Pathways")
				PathwaysResults[[i]]=temp
				names(PathwaysResults)[i]=paste("Cluster",i,sep=" ")

			}

			ResultMLP[[k]]=PathwaysResults
		}
		names(ResultMLP)=names
		for(i in 1:length(ResultMLP)){
			for(k in 1:length(ResultMLP[[i]])){
				if(is.null(ResultMLP[[i]][[k]])[1]){
					ResultMLP[[i]][[k]]=NA
					names(ResultMLP[[i]])[k]=paste("Cluster",k,sep=" ")
				}
			}
			if(length(ResultMLP[[i]]) != maxclus){
				extra=maxclus-length(ResultMLP[[i]])
				for(j in 1:extra){
					ResultMLP[[i]][[length(ResultMLP[[i]])+j]]=NA
					names(ResultMLP[[i]])[length(ResultMLP[[i]])]=paste("Cluster",length(ResultMLP[[i]]),sep=" ")
				}
			}
		}
	}
	return(ResultMLP)
}

#' @title Iterations of the pathway analysis
#'
#' @description The MLP method to perform pathway analysis is based on resampling of the
#' data. Therefore it is recommended to perform the pathway analysis multiple
#' times to observe how much the results are influenced by a different
#' resample. The function \code{PathwaysIter} performs the pathway analysis as
#' described in \code{Pathways} a specified number of times. The input can be
#' one data set or a list as in \code{Pathway.2} and \code{Pathways}.
#' @export PathwaysIter
#'
#' @param List A list of clustering outputs or output of the\code{DiffGenes}
#' function. The first element of the list will be used as the reference in
#' \code{ReorderToReference}. The output of \code{ChooseFeatures} is also
#' accepted.
#' @param Selection If pathway analysis should be conducted for a specific
#' selection of objects, this selection can be provided here. Selection can
#' be of the type "character" (names of the objects) or "numeric" (the number
#' of specific cluster). Default is NULL.
#' @param geneExpr The gene expression matrix of the objects. The rows should
#' correspond with the genes.
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param method The method to applied to look for differentially expressed genes and related pathways. For now, only the
#' limma method is available for gene analysis and the MLP method for pathway analysis. Default is c("limma","MLP").
#' @param geneInfo A data frame with at least the columns ENTREZID and SYMBOL.
#' This is necessary to connect the symbolic names of the genes with their
#' EntrezID in the correct order. The order of the gene is here not in the
#' order of the rownames of the gene expression matrix but in the order of
#' their significance. Default is NULL.
#' @param geneSetSource The source for the getGeneSets function ("GOBP",
#' "GOMF","GOCC", "KEGG" or "REACTOME"). Default is "GOBP".
#' @param topP Overrules sign. The number of pathways to display for each
#' cluster. If not specified, only the significant genes are shown. Default is NULL.
#' @param topG Overrules sign. The number of top genes to be returned in the
#' result. If not specified, only the significant genes are shown. Default is NULL.
#' @param GENESET Optional. Can provide own candidate gene sets. Default is NULL.
#' @param sign The significance level to be handled. Default is 0.05.
#' @param niter The number of times to perform pathway analysis. Default is 10.
#' @param fusionsLog Logical. To be handed to \code{ReorderToReference}: indicator for the fusion of clusters. Default is TRUE
#' @param weightclust Logical. To be handed to \code{ReorderToReference}: to be used for the outputs of CEC,
#' WeightedClust or WeightedSimClust. If TRUE, only the result of the Clust element is considered. Default is TRUE.
#' @param names Optional. Names of the methods. Default is NULL.
#' @return A list with an element per iteration, each being the output of \code{Pathways}.
#' @details This function relies on the suggested Bioconductor packages 'MLP', 'biomaRt' and 'org.Hs.eg.db'.
#' @examples
#'
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#' data(geneMat)
#' data(GeneInfo)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(MCF7_F,MCF7_T)
#' names=c('FP','TP')
#'
#' MCF7_Paths_FandT=PathwaysIter(List=L, geneExpr = geneMat, nrclusters = 7, method =
#' c("limma", "MLP"), geneInfo = GeneInfo, geneSetSource = "GOBP", topP = NULL,
#' topG = NULL, GENESET = NULL, sign = 0.05,niter=2,fusionsLog = TRUE,
#' weightclust = TRUE, names =names)
#' }
PathwaysIter<-function(List,Selection=NULL,geneExpr=NULL,nrclusters=NULL,method=c("limma", "MLP"),geneInfo=NULL,geneSetSource = "GOBP",topP=NULL,topG=NULL,GENESET=NULL,sign=0.05,niter=10,fusionsLog=TRUE,weightclust=TRUE,names=NULL){
	if (!requireNamespace("MLP", quietly = TRUE)) {
		stop("MLP needed for this function to work. Please install it.",
				call. = FALSE)
	}

	if (!requireNamespace("biomaRt", quietly = TRUE)) {
		stop("biomaRt needed for this function to work. Please install it.",
				call. = FALSE)
	}

	if (!requireNamespace("org.Hs.eg.db", quietly = TRUE)) {
		stop("org.Hs.eg.db needed for this function to work. Please install it.",
				call. = FALSE)
	}

	PathwaysOutput = list()
	for (i in 1:niter){
		message(paste("Iteration",i,sep=" "))
		mlp = Pathways(List,Selection,geneExpr,nrclusters,method,geneInfo,geneSetSource,topP,topG,GENESET,sign=sign,fusionsLog,weightclust,names)
		PathwaysOutput [[length(PathwaysOutput )+1]] = mlp
		names(PathwaysOutput )[i]=paste("Iteration",i,sep=" ")
	}


	return(PathwaysOutput)
}

#' @title Pathway analysis for a selection of objects
#' @param List A list of clustering outputs or output of the\code{DiffGenes}
#' function. The first element of the list will be used as the reference in
#' \code{ReorderToReference}. The output of \code{ChooseFeatures} is also
#' accepted.
#' @param Selection If pathway analysis should be conducted for a specific
#' selection of objects, this selection can be provided here. Selection can
#' be of the type "character" (names of the objects) or "numeric" (the number
#' of specific cluster). Default is NULL.
#' @param geneExpr The gene expression matrix or ExpressionSet of the objects.
#' The rows should correspond with the genes.
#' @param nrclusters Optional. The number of clusters to cut the dendrogram in.
#' The number of clusters should not be specified if the interest lies only in
#' a specific selection of objects which is known by name.  Otherwise, it is
#' required. Default is NULL.
#' @param method The method to applied to look for differentially expressed genes and related pathways. For now, only the
#' limma method is available for gene analysis and the MLP method for pathway analysis. Default is c("limma","MLP").
#' @param geneInfo A data frame with at least the columns ENTREZID and SYMBOL.
#' This is necessary to connect the symbolic names of the genes with their
#' EntrezID in the correct order. The order of the gene is here not in the
#' order of the rownames of the gene expression matrix but in the order of
#' their significance. Default is NULL.
#' @param geneSetSource The source for the getGeneSets function, defaults to
#' "GOBP".
#' @param topP Overrules sign. The number of pathways to display for each
#' cluster. If not specified, only the significant genes are shown. Default is NULL.
#' @param topG Overrules sign. The number of top genes to be returned in the
#' result. If not specified, only the significant genes are shown. Defaults is NULL.
#' @param GENESET Optional. Can provide own candidate gene sets. Default is NULL.
#' @param sign The significance level to be handled. Default is 0.05.
#' @param fusionsLog Logical. To be handed to \code{ReorderToReference}: indicator for the fusion of clusters. Default is TRUE
#' @param weightclust Logical. To be handed to \code{ReorderToReference}: to be used for the outputs of CEC,
#' WeightedClust or WeightedSimClust. If TRUE, only the result of the Clust element is considered. Default is TRUE.
#' @param names Optional. Names of the methods. Default is NULL.
#' @description Internal function of \code{Pathways}.
#' @export
#' @details This function relies on the suggested Bioconductor packages 'MLP', 'biomaRt' and 'org.Hs.eg.db'.
#' @examples
#' \dontrun{
#' PathwaysSelection(List=L,Selection=c("Cpd1","Cpd2"),geneExpr=geneMat,geneInfo=GeneInfo)
#' }
PathwaysSelection<-function(List=NULL,Selection,geneExpr=NULL,nrclusters=NULL,method=c("limma", "MLP"),geneInfo=NULL,geneSetSource = "GOBP",topP=NULL,topG=NULL,GENESET=NULL,sign=0.05,fusionsLog=TRUE,weightclust=TRUE,names=NULL){
	if (!requireNamespace("MLP", quietly = TRUE)) {
		stop("MLP needed for this function to work. Please install it.",
				call. = FALSE)
	}

	if (!requireNamespace("biomaRt", quietly = TRUE)) {
		stop("biomaRt needed for this function to work. Please install it.",
				call. = FALSE)
	}

	if (!requireNamespace("org.Hs.eg.db", quietly = TRUE)) {
		stop("org.Hs.eg.db needed for this function to work. Please install it.",
				call. = FALSE)
	}


	method.test = function(sign.method,path.method){
		method.choice = FALSE

		if( sign.method=="limma"  & path.method=="MLP"  ){
			method.choice = TRUE
		}
		if(method.choice==TRUE){
			return(list(sign.method=sign.method,path.method=path.method))
		}
		else{
			stop("Incorrect choice of method.")
		}

	}

	method.out = method.test(method[1],method[2])

	sign.method = method.out$sign.method
	path.method = method.out$path.method

	if(length(geneInfo$ENTREZID)==1){
		geneInfo$ENTREZID = colnames(geneExpr)
	}

	# Determining the genesets if they were not given with the function input
	if((inherits(GENESET,"geneSetMLP"))[1] ){
		geneSet <- GENESET
	}
	else{
		geneSet <- MLP::getGeneSets(species = "Human",geneSetSource = geneSetSource,entrezIdentifiers = geneInfo$ENTREZID)
	}


	if(inherits(Selection,"character")){
		ResultMLP=list()

		DataPrepared<-plyr::try_default(PreparePathway(List[[1]],geneExpr,topG,sign),NULL,quiet=TRUE)
		if(is.null(DataPrepared)){
			Temp=List[[1]]
			objects=list()
			objects$LeadCpds=Selection
			objects$OrderedCpds=colnames(geneExpr)
			Temp[[length(Temp)+1]]=list(objects=objects)
			names(Temp)[length(Temp)]=paste("Cluster")

			DataPrepared<-PreparePathway(Temp,geneExpr,topG,sign)
		}


		temp=list()
		temp[[1]]=DataPrepared$objects[[1]] #names of the objects
		temp[[2]]=DataPrepared$Genes[[1]]




		if(path.method=="MLP"){
			## WE WILL USE THE RAW P-VALUES TO PUT IN MLP -> LESS GRANULAR
			p.values=DataPrepared$pvalsgenes[[1]]

			Entrezs=sapply(names(p.values),function(x) return(geneInfo$ENTREZID[which(geneInfo$SYMBOL==x)]))

			names(p.values) = Entrezs
			out.mlp <- MLP::MLP(
					geneSet = geneSet,
					geneStatistic = p.values,
					minGenes = 5,
					maxGenes = 100,
					rowPermutations = TRUE,
					nPermutations = 100,
					smoothPValues = TRUE,
					probabilityVector = c(0.5, 0.9, 0.95, 0.99, 0.999, 0.9999, 0.99999),df = 9)

			output = list()
			#output$gene.p.values = p.adjust(p.values,method="fdr")

			#ranked.genesets.table = data.frame(genesets = (rownames(out.mlp)),p.values = as.numeric(out.mlp$geneSetPValue),descriptions = out.mlp$geneSetDescription)
			#ranked.genesets.table$genesets = as.character(ranked.genesets.table$genesets)
			#ranked.genesets.table$descriptions = as.character(ranked.genesets.table$descriptions)

			#if(is.null(topP)){
			#		topP=length(ranked.genesets.table$p.values<=sign)
			#}

			if(is.null(topP)){
				topP=length(which(out.mlp$geneSetPValue<=sign))
			}

			#TopPaths=ranked.genesets.table[1:topP,]
			#AllPaths=ranked.genesets.table

			TopPaths=out.mlp[1:topP,]
			AllPaths=out.mlp

			output$TopPaths=TopPaths
			attr(output$TopPaths,'geneSetSource')=geneSetSource
			output$AllPaths=AllPaths
			attr(output$AllPaths,'geneSetSource')=geneSetSource
			#output$ranked.genesets.table = ranked.genesets.table[ranked.genesets.table$p.values<=sign,]

			#nr.genesets = c( dim(ranked.genesets.table)[1]  ,  length(geneSet) )
			#names(nr.genesets) = c("used.nr.genesets","total.nr.genesets")
			#output$nr.genesets = nr.genesets

			#output$object = out.mlp
			#output$method = "MLP"

			temp[[3]]=output


		}
		names(temp)=c("objects","Genes","Pathways")
		ResultMLP[[1]]=temp
		names(ResultMLP)="Selection"
	}



	else if(inherits(Selection,"numeric") & !(is.null(List))){
		check<-plyr::try_default(PreparePathway(List[[1]],geneExpr,topG,sign),NULL,quiet=TRUE)
		if(is.null(check)){

			ListNew=list()
			element=0
			for(i in 1:length(List)){
				if(attributes(List[[i]])$method != "CEC" & attributes(List[[i]])$method != "Weighted" & attributes(List[[i]])$method!= "WeightedSim"){
					ResultsClust=list()
					ResultsClust[[1]]=list()
					ResultsClust[[1]][[1]]=List[[i]]
					names(ResultsClust[[1]])[1]="Clust"
					element=element+1
					ListNew[[element]]=ResultsClust[[1]]
					#attr(ListNew[element],"method")="Weights"
				}
				else if(attributes(List[[i]])$method=="CEC" | attributes(List[[i]])$method=="Weighted" | attributes(List[[i]])$method == "WeightedSim"){
					ResultsClust=list()
					if(weightclust==TRUE){
						ResultsClust[[1]]=list()
						if(attributes(List[[i]])$method != "WeightedSim"){
							ResultsClust[[1]][[1]]=List[[i]]$Clust
							names(ResultsClust[[1]])[1]="Clust"
							element=element+1
							ListNew[[element]]=ResultsClust[[1]]
							attr(ListNew[element],"method")="Weights"
						}
						else{
							ResultsClust[[1]]=list()
							ResultsClust[[1]][[1]]=List[[i]]
							names(ResultsClust[[1]])[1]="Clust"
							element=element+1
							ListNew[[element]]=ResultsClust[[1]]
						}
					}
					else{
						for (j in 1:length(List[[i]]$Results)){
							ResultsClust[[j]]=list()
							ResultsClust[[j]][[1]]=List[[i]]$Results[[j]]
							names(ResultsClust[[j]])[1]="Clust"
							element=element+1
							ListNew[[element]]=ResultsClust[[j]]
							attr(ListNew[element],"method")="Weights"
						}
					}
				}
			}

			if(is.null(names)){
				names=seq(1,length(ListNew),1)
				for(i in 1:length(ListNew)){
					names[i]=paste("Method",i,sep=" ")
				}
			}
			names(ListNew)=names
			Matrix=ReorderToReference(List,nrclusters,fusionsLog,weightclust,names)
			List=ListNew

			DataPrepared=list()
			for (k in 1:dim(Matrix)[1]){

				cluster=Selection

				check<-plyr::try_default(PreparePathway(List[[k]],geneExpr,topG,sign),NULL,quiet=TRUE)
				if(is.null(check)){
					Temp=List[[k]]
					objects=list()
					objects$LeadCpds=colnames(Matrix)[which(Matrix[k,]==cluster)]
					objects$OrderedCpds=stats::as.hclust(List[[k]]$Clust$Clust)$labels[stats::as.hclust(List[[k]]$Clust$Clust)$order]
					Temp[[length(Temp)+1]]=list(objects=objects)
					names(Temp)[length(Temp)]=paste("Cluster")

					DataPrepared[[k]]<-PreparePathway(Temp,geneExpr,topG,sign)
				}
			}
			names(DataPrepared)=names
		}

		else{
			for(k in 1:length(List)){
				DataPrepared[[k]]<-plyr::try_default(PreparePathway(List[[k]],geneExpr,topG,sign),NULL,quiet=TRUE)
				if(is.null(DataPrepared[[k]])){
					Temp=List[[k]]


					objects=list()
					objects$LeadCpds=List[[k]]$objects$LeadCpds
					objects$OrderedCpds=List[[k]]$objects$OrderedCpds

					Temp[[length(Temp)+1]]=list(objects=objects)
					names(Temp)[length(Temp)]=paste("Cluster")

					DataPrepared[[k]]<-PreparePathway(Temp,geneExpr,topG,sign)

				}
			}


		}

		ResultMLP=list()
		for (k in 1:length(DataPrepared)){
			message(k)
			cluster=Selection


			temp=list()
			temp[[1]]=DataPrepared[[k]]$objects[[1]] #names of the objects
			temp[[2]]=DataPrepared[[k]]$Genes[[1]]


			if(path.method=="MLP"){
				## WE WILL USE THE RAW P-VALUES TO PUT IN MLP -> LESS GRANULAR
				p.values=DataPrepared[[k]]$pvalsgenes[[1]]
				Entrezs=sapply(names(p.values),function(x) return(geneInfo$ENTREZID[which(geneInfo$SYMBOL==x)]))

				names(p.values) = Entrezs

				out.mlp <- MLP::MLP(
						geneSet = geneSet,
						geneStatistic = p.values,
						minGenes = 5,
						maxGenes = 100,
						rowPermutations = TRUE,
						nPermutations = 100,
						smoothPValues = TRUE,
						probabilityVector = c(0.5, 0.9, 0.95, 0.99, 0.999, 0.9999, 0.99999),df = 9)

				output = list()
				#output$gene.p.values = p.adjust(p.values,method="fdr")

				#ranked.genesets.table = data.frame(genesets = (rownames(out.mlp)),p.values = as.numeric(out.mlp$geneSetPValue),descriptions = out.mlp$geneSetDescription)
				#ranked.genesets.table$genesets = as.character(ranked.genesets.table$genesets)
				#ranked.genesets.table$descriptions = as.character(ranked.genesets.table$descriptions)

				#if(is.null(topP)){
				#		topP=length(ranked.genesets.table$p.values<=sign)
				#}

				if(is.null(topP)){
					topP=length(which(out.mlp$geneSetPValue<=sign))
				}

				#TopPaths=ranked.genesets.table[1:topP,]
				#AllPaths=ranked.genesets.table

				TopPaths=out.mlp[1:topP,]
				AllPaths=out.mlp

				output$TopPaths=TopPaths
				attr(output$TopPaths,'geneSetSource')=geneSetSource
				output$AllPaths=AllPaths
				attr(output$AllPaths,'geneSetSource')=geneSetSource
				#output$ranked.genesets.table = ranked.genesets.table[ranked.genesets.table$p.values<=sign,]

				#nr.genesets = c( dim(ranked.genesets.table)[1]  ,  length(geneSet) )
				#names(nr.genesets) = c("used.nr.genesets","total.nr.genesets")
				#output$nr.genesets = nr.genesets

				#output$object = out.mlp
				#output$method = "MLP"

				temp[[3]]=output
			}
			names(temp)=c("objects","Genes","Pathways")
			ResultMLP[[k]]=temp

		}
		names(ResultMLP)=names

	}
	else{
		message("If a specific cluster is specified, clustering results must be provided in List")
	}

	return(ResultMLP)
}

#' @title Prepare data for pathway analysis
#'
#' @description The \code{PreparePathway} function prepares the input for the
#' pathway analysis by computing differentially expressed genes (via limma) for
#' a selection of objects if these were not provided.
#' @export
#' @param Object A clustering output or DiffGenes output element.
#' @param geneExpr The gene expression matrix or ExpressionSet of the objects.
#' @param topG Overrules sign. The number of top genes. Default is NULL.
#' @param sign The significance level to be handled. Default is 0.05.
#' @return A list with the p-values of the genes, the objects and the genes.
#' @details This function may rely on the suggested Bioconductor package 'limma'.
#' @examples
#' \dontrun{
#' PreparePathway(Object=MCF7_F,geneExpr=geneMat,topG=NULL,sign=0.05)
#' }
PreparePathway<-function(Object,geneExpr,topG,sign){
	FoundGenes=NULL
	FoundComps=NULL

	FoundGenes=FindElement("Genes",Object)

	if(is.null(FoundGenes)|(is.list(FoundGenes) & length(FoundGenes) == 0)){
		FoundComps=FindElement("objects",Object)
		if(is.null(FoundComps)|(is.list(FoundComps) & length(FoundComps) == 0)){
			stop("Specify either the p-values of the genes or a selection of objects to test for DE genes.")
		}

		pvalsgenes=list()
		FoundGenes=list()
		CompsP=list()
		TopDEP=list()
		for(i in 1:length(FoundComps)){
			LeadCpds=FoundComps[[i]]$LeadCpds
			CompsP[[i]]=FoundComps[[i]]
			names(CompsP)[[i]]=paste("objects_",i,sep="")
			if(is.null(LeadCpds)){
				stop("In the objects element, specify an element LeadCpds")
			}

			group <- factor(ifelse(colnames(geneExpr) %in% LeadCpds, 1, 0))

			if(class(geneExpr)[1]=="ExpressionSet"){
				geneExpr$LeadCmpds<-group
				if (!requireNamespace("a4Base", quietly = TRUE)) {
					stop("a4Base needed for this function to work. Please install it.",
							call. = FALSE)
				}

				DElead <- a4Base::limmaTwoLevels(geneExpr,"LeadCmpds")

				allDE <- a4Core::topTable(DElead, n = length(DElead@MArrayLM$genes$SYMBOL),sort.by="p")

				if(is.null(allDE$ID)){
					allDE$Genes <- rownames(allDE)
				}
				else
				{
					allDE$Genes=allDE$ID
				}

				if(is.null(topG)){
					topG=length(which(allDE$adj.P.Val<=sign))
				}
				TopDE <- allDE[1:topG, ]

				Genes <- list(TopDE,allDE)
				names(Genes)<-c("TopDE","AllDE")
			}
			else{
				label.factor = factor(group)
				design = stats::model.matrix(~label.factor)
				fit = limma::lmFit(geneExpr,design=design)
				fit = limma::eBayes(fit)
				allDE = limma::topTable(fit,coef=2,adjust="fdr",n=dim(geneExpr)[1], sort.by="p")

				if(is.null(allDE$ID)){
					allDE$ID <- rownames(allDE)
				}

				if(is.null(topG)){
					topG=length(which(allDE$adj.P.Val<=sign))
				}


				TopDE<-allDE[1:topG,]

				Genes <- list(TopDE,allDE)
				names(Genes)<-c("TopDE","AllDE")

			}
			FoundGenes[[i]]=Genes
			TopDEP[[i]]=FoundGenes[[i]]
			names(TopDEP)[i]=paste("genes_",i,sep="")
			names(FoundGenes)=paste("Genes_",i,sep="")
			pvalsgenes[[i]]=Genes$AllDE$P.Value
			names(pvalsgenes[[i]])=Genes$AllDE$ID
			names(pvalsgenes)[i]=paste("pvals_",i,sep="")
		}
	}

	else{
		pvalsgenes=list()
		TopDEP=list()
		for(i in 1:length(FoundGenes)){
			#names(FoundGenes)[i]=paste("Genes_",i,sep="")
			TopDEP[[i]]=FoundGenes[[i]]
			names(TopDEP)[i]=paste("genes_",i,sep="")
			pvalsgenes[[i]]=FoundGenes[[i]]$AllDE$P.Value
			names(pvalsgenes[[i]])=FoundGenes[[i]]$AllDE$ID
			names(pvalsgenes)[i]=paste("pvals_",i,sep="")
		}

		FoundComps=FindElement("objects",Object)
		CompsP=list()
		for(i in 1:length(FoundComps)){
			if(is.null(FoundComps[[i]]$LeadCpds)){
				CompsP[[i]]="No LeadCpds specified"
			}
			else{
				CompsP[[i]]=FoundComps[[i]]
				names(CompsP)[[i]]=paste("objects_",i,sep="")
			}
		}
	}


	return(list(pvalsgenes=pvalsgenes,objects=CompsP,Genes=TopDEP))
}

#' @title Shared genes, pathways and features across methods
#'
#' @description The \code{SharedGenesPathsFeat} function takes the intersection
#' of the differentially expressed genes, the pathways and the characteristic
#' features over the methods per cluster.
#' @export
#' @param DataLimma Optional. The output of a \code{DiffGenes} function. Default is NULL.
#' @param DataMLP Optional. The output of \code{Geneset.intersect} function. Default is NULL.
#' @param DataFeat Optional. The output of \code{CharacteristicFeatures} function. Default is NULL.
#' @param names Optional. Names of the methods. Default is NULL.
#' @param Selection Logical. Whether a selection of objects was considered. Default is FALSE.
#' @return A list with a Table summarizing the shared elements and a Which
#' element detailing the shared genes, pathways and features per cluster.
#' @examples
#' \dontrun{
#' SharedGenesPathsFeat(DataLimma=MCF7_FT_DE,DataMLP=MCF7_Paths_intersection,names=c("FP","TP"))
#' }
SharedGenesPathsFeat<-function(DataLimma=NULL,DataMLP=NULL,DataFeat=NULL,names=NULL,Selection=FALSE){  #Input=result of DiffGenes and Geneset.intersect
	#Include sharedLimma and SharedMLP inside the function
	if(Selection==TRUE){
		ResultShared=SharedSelection(DataLimma,DataMLP,DataFeat,names)
	}

	else if(is.null(DataLimma) & is.null(DataMLP) & is.null(DataFeat)){
		stop("At least one Data set should be specified")
	}

	else{

		List=list(DataLimma,DataMLP,DataFeat)
		AvailableData=sapply(seq(length(List)),function(i) if(!(is.null(List[[i]]))) return(i))
		AvailableData=unlist(AvailableData)
		len=c()
		for(i in AvailableData){
			len=c(len,length(List[[i]]))
		}
		if(length(unique(len))!=1){
			stop("Unequal number of methods for limma and MLP")
		}
		else{
			DataSets=lapply(AvailableData,function(i)  return(List[[i]]))
			nmethods=length(DataSets[[1]])
			nclusters=length(DataSets[[1]][[1]])
		}

		if(is.null(names)){
			for(j in 1:length(DataSets[[1]])){
				names[j]=paste("Method",j,sep=" ")
			}
		}

		which=list()
		table=c()


		for (i in 1:nclusters){

			name=paste("Cluster",i,sep=" ")

			comps=c()

			temp1g=c()
			temp1p=c()
			temp1f=list()


			for(j in 1:nmethods){

				if(!(is.na(DataSets[[1]][[j]][[i]])[1])){
					comps=c(comps,length(DataSets[[1]][[j]][[i]]$objects$LeadCpds))
					names(comps)[j]=paste("Ncomps", names[j],sep=" ")
				}
				else{
					comps=c(comps,"-")
				}

				if(!(is.null(DataLimma))){
					if(!(is.na(DataLimma[[j]][[i]])[1])){
						temp1g=c(temp1g,length(DataLimma[[j]][[i]]$Genes$TopDE$ID))
					}
					else{
						temp1g=c(temp1g,"-")
					}
					names(temp1g)[j]=names[j]
				}
				else{
					temp1g=NULL
				}

				if(!(is.null(DataMLP))){
					if(!(is.na(DataMLP[[j]][[i]])[1])){
						temp1p=c(temp1p,length(DataMLP[[j]][[i]][[3]]$geneSetDescription))
					}
					else{
						temp1p=c(temp1g,"-")
					}
					names(temp1p)[j]=names[j]
				}
				else{
					temp1p=NULL
				}

				if(!(is.null(DataFeat))){
					temp=c()
					for(f in 1:length(DataFeat[[j]][[i]]$Characteristics)){
						if(!(is.na(DataFeat[[j]][[i]])[1])){
							temp=c(temp,length(DataFeat[[j]][[i]]$Characteristics[[f]]$TopFeat$Names))
						}
						else{
							temp=c(temp,"-")
						}

						names(temp)[f]=names(DataFeat[[j]][[i]]$Characteristics)[f]
					}
					temp1f[[j]]=temp
					names(temp1f)[j]=names[j]
				}
				else{
					temp1f=NULL
				}
			}

			j=1
			Continue=TRUE
			while (Continue==TRUE){
				cont=c()
				for(d in 1:length(DataSets)){
					cont=c(cont,!(is.na(DataSets[[d]][[j]][[i]])[1]))
				}

				if(any(cont)){

					sharedcomps=DataSets[[1]][[j]][[i]]$objects$LeadCpds
					nsharedcomps=length(sharedcomps)
					names(nsharedcomps)="nsharedcomps"


					if(!(is.null(DataLimma))){
						sharedgenes=DataLimma[[j]][[i]]$Genes$TopDE$ID
						nsharedgenes=length(sharedgenes)
						names(nsharedgenes)="Nshared"
						#pvalsg=DataLimma[[j]][[i]]$Genes$TopDE$adj.P.Val
					}
					else{
						sharedgenes=NULL
						nsharedgenes=NULL
					}


					if(!(is.null(DataMLP))){
						sharedpaths=DataMLP[[j]][[i]][[3]]$geneSetDescription
						nsharedpaths=length(sharedpaths)
						names(nsharedpaths)="Nshared"
						#pvalsp=DataMLP[[j]][[i]][[3]]$mean_geneSetPValue

					}
					else{
						sharedpaths=NULL
						nsharedpaths=NULL
					}
					if(!(is.null(DataFeat))){
						sharedfeat=list()
						nsharedfeat=list()
						#pvalsf=list()
						for(f in 1:length(DataFeat[[j]][[i]]$Characteristics)){
							sharedfeat[[f]]=DataFeat[[j]][[i]]$Characteristics[[f]]$TopFeat$Names
							names(sharedfeat)[f]=paste("shared: ",names(DataFeat[[j]][[i]]$Characteristics)[f],sep="")

							nsharedfeat[[f]]=length(sharedfeat[[f]])
							names(nsharedfeat)[f]=paste("Nshared: ",names(DataFeat[[j]][[i]]$Characteristics)[f],sep="")

							#pvalsf[[f]]=DataFeat[[j]][[i]]$Characteristics[[f]]$TopFeat$adj.P.Val
							#names(pvalsf)[f]=paste("shared: ",names(DataFeat[[j]][[i]]$Characteristics)[f],sep="")

						}

					}
					else{
						sharedfeat=NULL
						nsharedfeat=NULL
					}

					Continue=FALSE
				}
				j=j+1
			}

			if(nmethods>=2){
				for (j in 2:nmethods){
					cont=c()
					for(d in 1:length(DataSets)){
						cont=c(cont,!(is.na(DataSets[[d]][[j]][[i]])[1]))
					}
					if(any(cont)){

						sharedcomps=intersect(sharedcomps,DataSets[[1]][[j]][[i]]$objects$LeadCpds)
						nsharedcomps=length(sharedcomps)
						names(nsharedcomps)="Nsharedcomps"


						if(!(is.null(DataLimma))){
							sharedgenes=intersect(sharedgenes,DataLimma[[j]][[i]]$Genes$TopDE$ID)
							nsharedgenes=length(sharedgenes)
							names(nsharedgenes)="Nshared"

						}
						if(!(is.null(DataMLP))){
							sharedpaths=intersect(sharedpaths,DataMLP[[j]][[i]][[3]]$geneSetDescription)
							nsharedpaths=length(sharedpaths)
							names(nsharedpaths)="Nshared"

						}
						if(!(is.null(DataFeat))){

							for(f in 1:length(DataFeat[[j]][[i]]$Characteristics)){
								sharedfeat[[f]]=intersect(sharedfeat[[f]],DataFeat[[j]][[i]]$Characteristics[[f]]$TopFeat$Names)
								names(sharedfeat)[f]=paste("shared: ",names(DataFeat[[j]][[i]]$Characteristics)[f],sep="")

								nsharedfeat[[f]]=length(sharedfeat[[f]])
								names(nsharedfeat)[f]=paste("Nshared: ",names(DataFeat[[j]][[i]]$Characteristics)[f],sep="")

							}

						}

					}
				}
			}

			pvalsgenes=list()
			meanpvalsgenes=c()

			pvalspaths=list()
			meanpvalspaths=c()

			pvalsfeat=list()
			meanpvalsfeat=c()


			if(!(is.null(sharedgenes))&length(sharedgenes)!=0){
				for(c in 1:nmethods){
					pvalsg=c()
					for(g in sharedgenes){
						if(!(is.na(DataLimma[[c]][[i]])[1])){
							pvalsg=c(pvalsg,DataLimma[[c]][[i]]$Genes$TopDE$adj.P.Val[DataLimma[[c]][[i]]$Genes$TopDE$ID==g])
						}
					}

					pvalsgenes[[c]]=pvalsg
					names(pvalsgenes)[c]=paste("P.Val.",names[c],sep="")
				}

				for(g1 in 1:length(sharedgenes)){
					pvalstemp=c()
					for(c in 1:nmethods){
						if(!(is.na(DataLimma[[c]][[i]])[1])){
							pvalstemp=c(pvalstemp,pvalsgenes[[c]][[g1]])
						}
					}
					meanpvalsgenes=c(meanpvalsgenes,mean(pvalstemp))
				}
				pvalsgenes[[nmethods+1]]=meanpvalsgenes
				names(pvalsgenes)[nmethods+1]="Mean pvals genes"
			}
			else{pvalsgenes=NULL}

			if(!(is.null(sharedpaths))&length(sharedpaths)!=0){
				for(c in 1:nmethods){
					pvalsp=c()
					if(!(is.na(DataMLP[[c]][[i]])[1])){
						for(p in sharedpaths){
							pvalsp=c(pvalsp,DataMLP[[c]][[i]][[3]][DataMLP[[c]][[i]][[3]]$geneSetDescriptions==p,5][1])
						}
					}

					pvalspaths[[c]]=pvalsp
					names(pvalspaths)[c]=paste("P.Val.",names[c],sep="")
				}


				for(p1 in 1:length(sharedpaths)){
					pvalstemp1=c()
					for(c in 1:nmethods){
						if(!(is.na(DataMLP[[c]][[i]])[1])){
							pvalstemp1=c(pvalstemp1,pvalspaths[[c]][[p1]])

						}

					}

					meanpvalspaths=c(meanpvalspaths,mean(pvalstemp1))
				}
				pvalspaths[[nmethods+1]]=meanpvalspaths
				names(pvalspaths)[nmethods+1]="Mean pvals paths"
			}
			else{pvalpaths=NULL}


			if(!(is.null(sharedfeat))){
				for(f in 1:length(DataFeat[[j]][[i]]$Characteristics)){

					if(length(sharedfeat[[f]])!=0){
						pvalschar=list()
						for(c in 1:nmethods){
							pvalsf=c()
							if(!(is.na(DataFeat[[c]][[i]])[1])){
								for(s in sharedfeat[[f]]){
									pvalsf=c(pvalsf,DataFeat[[c]][[i]]$Characteristics[[f]]$TopFeat$adj.P.Val[DataFeat[[c]][[i]]$Characteristics[[f]]$TopFeat$Names==s])
								}
							}
							pvalschar[[c]]=pvalsf
							names(pvalschar)[c]=paste("P.Val.",names[c],sep="")
						}
					}
					else{
						pvalschar=list()
						pvalschar[1:nmethods]=0
						for(c in 1:nmethods){
							names(pvalschar)[c]=paste("P.Val.",names[c],sep="")
						}
					}
					pvalsfeat[[f]]=pvalschar
					names(pvalsfeat)[f]=names(DataFeat[[j]][[i]]$Characteristics)[f]

				}

				for(f in 1:length(DataFeat[[j]][[i]]$Characteristics)){
					meanpvalsfeat=c()

					if((length(sharedfeat[[f]])!=0)){
						for(f1 in 1:length(sharedfeat[[f]])) {

							pvalstemp=c()
							for(c in 1:nmethods){
								if(!(is.na(DataFeat[[c]][[i]])[1])){
									pvalstemp=c(pvalstemp,pvalsfeat[[f]][[c]][[f1]])
								}
							}
							meanpvalsfeat=c(meanpvalsfeat,mean(pvalstemp))
						}
					}
					else{
						meanpvalsfeat=0
					}

					pvalsfeat[[f]][[nmethods+1]]=meanpvalsfeat
					names(pvalsfeat[[f]])[nmethods+1]="Mean pvals feat"
				}
				lenchar=length(DataFeat[[j]][[i]]$Characteristics)

			}
			else{
				pvalsfeat=NULL
				lenchar=0
			}


			if(!(is.null(temp1f))){
				temp1f=do.call(rbind.data.frame, temp1f)
				colnames(temp1f)=names(DataFeat[[j]][[i]]$Characteristics)
				temp1f=as.matrix(temp1f)
				nsharedfeat=do.call(cbind.data.frame, nsharedfeat)
				nsharedfeat=as.matrix(nsharedfeat)
			}
			part1=cbind(cbind(temp1g,temp1p),temp1f)
			part1=as.matrix(part1)
			if(is.null(nsharedgenes) & is.null(nsharedpaths) &is.null(nsharedfeat)){
				if(!(is.null(temp1g))){
					nsharedgenes=0
				}
				if(!(is.null(temp1p))){
					nsharedpaths=0
				}
				if(!(is.null(temp1f))){
					nsharedfeat=rep(0,length(temp1f))
				}
			}
			part2=cbind(cbind(nsharedgenes,nsharedpaths),nsharedfeat)
			part2=as.matrix(part2)
			colnames(part1)=NULL
			colnames(part2)=NULL
			rownames(part2)="NShared"
			part3=c()
			for(r in 1:length(comps)){
				part3=rbind(part3,rep(comps[r],dim(part1)[2]))
			}
			colnames(part3)=NULL
			rownames(part3)=names(comps)
			part4=rep(nsharedcomps,dim(part1)[2])
			names(part4)=NULL
			temp=rbind(part1,part2,part3,part4)
			rownames(temp)[nrow(temp)]="Nsharedcomps"


			table=cbind(table,temp)
			colnames(table)=seq(1,ncol(table))

			if(!(is.null(pvalsgenes))){
				SharedGenes=cbind(sharedgenes,do.call(cbind.data.frame, pvalsgenes))
			}
			else{
				SharedGenes=NULL
			}
			if(!(is.null(pvalspaths))){
				SharedPaths=cbind(sharedpaths,do.call(cbind.data.frame, pvalspaths))
			}
			else{
				SharedPaths=NULL
			}
			if(!(is.null(pvalsfeat))){
				SharedFeat=list()
				for(f in 1:lenchar){
					if(length(sharedfeat[[f]])==0){
						SharedFeat[[f]]=NULL
					}
					else{
						SharedFeat[[f]]=cbind(sharedfeat[[f]],do.call(cbind.data.frame, pvalsfeat[[f]]))
						names(SharedFeat)[f]=names(pvalsfeat[[1]])[f]
					}

				}
			}
			else{
				SharedFeat=NULL
			}
			which[[i]]=list(SharedComps=sharedcomps,SharedGenes=SharedGenes,SharedPaths=SharedPaths,SharedFeat=SharedFeat)
			names(which)[i]=paste("Cluster",i,sep=" ")

		}
		#Sep for all situations?
		if(all(!(is.null(DataLimma)),!(is.null(DataMLP)),!(is.null(DataFeat)))){
			for (i in 1:length(seq(1,dim(table)[2],lenchar+2))){
				number=seq(1,dim(table)[2],2+lenchar)[i]
				colnames(table)[number]=paste("G.Cluster",i,sep=" ")
				colnames(table)[number+1]=paste("P.Cluster",i,sep=" ")
				for(u in seq(1:lenchar)){
					colnames(table)[number+1+u]=paste(paste('Feat.',names(DataFeat[[1]][[1]]$Characteristics)[u],sep=""),paste(".Cluster",i,sep=" "),sep="")
				}

			}
		}

		else if(all(!(is.null(DataLimma)),!(is.null(DataMLP)),is.null(DataFeat))){
			for (i in 1:length(seq(1,dim(table)[2],2))){
				number=seq(1,dim(table)[2],2)[i]
				colnames(table)[number]=paste("G.Cluster",i,sep=" ")
				colnames(table)[number+1]=paste("P.Cluster",i,sep=" ")

			}
		}

		else if(all(!(is.null(DataLimma)),is.null(DataMLP),!(is.null(DataFeat)))){
			for (i in 1:length(seq(1,dim(table)[2],lenchar+1))){
				number=seq(1,dim(table)[2],1+lenchar)[i]
				colnames(table)[number]=paste("G.Cluster",i,sep=" ")
				for(u in seq(1:lenchar)){
					colnames(table)[number+u]=paste(paste('Feat.',names(DataFeat[[1]][[1]]$Characteristics)[u],sep=""),paste(".Cluster",i,sep=" "),sep="")
				}

			}
		}

		else if(all((is.null(DataLimma)),!(is.null(DataMLP)),!(is.null(DataFeat)))){
			for (i in 1:length(seq(1,dim(table)[2],lenchar+1))){
				number=seq(1,dim(table)[2],lenchar+1)[i]
				colnames(table)[number]=paste("P.Cluster",i,sep=" ")
				for(u in seq(1:lenchar)){
					colnames(table)[number+u]=paste(paste('Feat.',names(DataFeat[[1]][[1]]$Characteristics)[u],sep=""),paste(".Cluster",i,sep=" "),sep="")
				}

			}
		}

		else if(all(!(is.null(DataLimma)),(is.null(DataMLP)),(is.null(DataFeat)))){
			for (i in 1:length(seq(1,dim(table)[2],1))){
				colnames(table)[i]=paste("G.Cluster",i,sep=" ")
			}
		}

		else if(all((is.null(DataLimma)),!(is.null(DataMLP)),(is.null(DataFeat)))){
			for (i in 1:length(seq(1,dim(table)[2],1))){
				colnames(table)[i]=paste("P.Cluster",i,sep=" ")
			}
		}

		else if(all((is.null(DataLimma)),(is.null(DataMLP)),!(is.null(DataFeat)))){
			for (i in 1:length(seq(1,dim(table)[2],lenchar))){
				number=seq(1,dim(table)[2],2)[i]
				for(u in c(0,1)){
					colnames(table)[number+u]=paste(paste('Feat.',names(DataFeat[[1]][[1]]$Characteristics)[u+1],sep=""),paste(".Cluster",i,sep=" "),sep="")
				}

			}
		}

		ResultShared=list(Table=table,Which=which)

	}
	return(ResultShared)

}

#' @title Intersection of genes and pathways over multiple methods for a selection of objects.
#' @param DataLimma Optional. The output of a \code{DiffGenes} function. Default is NULL.
#' @param DataMLP Optional. The output of \code{Geneset.intersect} function. Default is NULL.
#' @param DataFeat Optional. The output of \code{CharacteristicFeatures}
#' function. Default is NULL.
#' @param names Optional. Names of the methods or "Selection" if it only
#' considers a selection of objects. Default is NULL.
#' @description Internal function of \code{SharedGenesPathsFeat}.
#' @export
#' @return A list with a Table and a Which element for the selection.
#' @examples
#' \dontrun{
#' SharedSelection(DataLimma=MCF7_FT_DE,names="Selection")
#' }
SharedSelection<-function(DataLimma=NULL,DataMLP=NULL,DataFeat=NULL,names=NULL){  #Input=result of DiffGenes.2 and Geneset.intersect
	if(is.null(DataLimma) & is.null(DataMLP) & is.null(DataFeat)){
		stop("At least one Data set should be specified")
	}

	List=list(DataLimma,DataMLP,DataFeat)
	AvailableData=sapply(seq(length(List)),function(i) if(!(is.null(List[[i]]))) return(i))
	AvailableData=unlist(AvailableData)
	len=c()
	for(i in AvailableData){
		len=c(len,length(List[[i]]))
	}
	if(length(unique(len))!=1){
		stop("Unequal number of methods for limma and MLP")
	}
	else{
		DataSets=lapply(AvailableData,function(i)  return(List[[i]]))
		nmethods=length(DataSets[[1]])
		#nclusters=length(DataSets[[1]][[1]])
	}

	if(is.null(names)){
		for(j in 1:nmethods){
			names[j]=paste("Method",j,sep=" ")
		}
	}

	which=list()
	table=c()

	comps=c()

	temp1g=c()
	temp1p=c()
	temp1f=list()

	for (i in 1:nmethods){

		if(!(is.na(DataSets[[1]][[i]])[1])){
			comps=c(comps,length(DataSets[[1]][[i]]$objects$LeadCpds))
			names(comps)[i]=paste("Ncomps", names[i],sep=" ")
		}
		else{
			comps=c(comps,"-")
		}

		if(!(is.null(DataLimma))){
			if(!(is.na(DataLimma[[i]])[1])){
				temp1g=c(temp1g,length(DataLimma[[i]]$Genes$TopDE$ID))
			}
			else{
				temp1g=c(temp1g,"-")
			}
			names(temp1g)[i]=names[i]
		}
		else{
			temp1g=NULL
		}

		if(!(is.null(DataMLP))){
			if(!(is.na(DataMLP[[i]])[1])){
				temp1p=c(temp1p,length(DataMLP[[i]][[3]]$geneSetDescription))
			}
			else{
				temp1p=c(temp1g,"-")
			}
			names(temp1p)[i]=names[i]
		}
		else{
			temp1p=NULL
		}

		if(!(is.null(DataFeat))){
			temp=c()
			for(f in 1:length(DataFeat[[i]]$Characteristics)){
				if(!(is.na(DataFeat[[i]])[1])){
					temp=c(temp,length(DataFeat[[i]]$Characteristics[[f]]$TopFeat$Names))
				}
				else{
					temp=c(temp,"-")
				}

				names(temp)[f]=names(DataFeat[[i]]$Characteristics)[f]
			}
			temp1f[[i]]=temp
			names(temp1f)[i]=names[i]
		}
		else{
			temp1f=NULL
		}

		if (i==1){
			cont=c()
			for(d in 1:length(DataSets)){
				cont=c(cont,!(is.na(DataSets[[d]][[i]])[1]))
			}

			if(any(cont)){

				sharedcomps=DataSets[[1]][[i]]$objects$LeadCpds
				nsharedcomps=length(sharedcomps)
				names(nsharedcomps)="Nsharedcomps"


				if(!(is.null(DataLimma))){
					sharedgenes=DataLimma[[i]]$Genes$TopDE$ID
					nsharedgenes=length(sharedgenes)
					names(nsharedgenes)="Nshared"
				}
				else{
					sharedgenes=NULL
					nsharedgenes=0
				}


				if(!(is.null(DataMLP))){
					sharedpaths=DataMLP[[i]][[3]]$geneSetDescription
					nsharedpaths=length(sharedpaths)
					names(nsharedpaths)="Nshared"

				}
				else{
					sharedpaths=NULL
					nsharedpaths=0
				}
				if(!(is.null(DataFeat))){
					sharedfeat=list()
					nsharedfeat=list()
					for(f in 1:length(DataFeat[[i]]$Characteristics)){
						sharedfeat[[f]]=DataFeat[[i]]$Characteristics[[f]]$TopFeat$Names
						names(sharedfeat)[f]=paste("Shared: ",names(DataFeat[[1]]$Characteristics)[f],sep="")

						nsharedfeat[[f]]=length(sharedfeat[[f]])
						names(nsharedfeat)[f]=paste("Nshared: ",names(DataFeat[[1]]$Characteristics)[f],sep="")

					}

				}
				else{
					sharedfeat=NULL
					nsharedfeat=0
				}
			}
		}
		else{

			sharedcomps=intersect(sharedcomps,DataSets[[1]][[i]]$objects$LeadCpds)
			nsharedcomps=length(sharedcomps)
			names(nsharedcomps)="Nsharedcomps"


			if(!(is.null(DataLimma))){
				sharedgenes=intersect(sharedgenes,DataLimma[[i]]$Genes$TopDE$ID)
				nsharedgenes=length(sharedgenes)
				names(nsharedgenes)="Nshared"

			}
			if(!(is.null(DataMLP))){
				sharedpaths=intersect(sharedpaths,DataMLP[[i]][[3]]$geneSetDescription)
				nsharedpaths=length(sharedpaths)
				names(nsharedpaths)="Nshared"

			}
			if(!(is.null(DataFeat))){

				for(f in 1:length(DataFeat[[i]]$Characteristics)){
					sharedfeat[[f]]=intersect(sharedfeat[[f]],DataFeat[[i]]$Characteristics[[f]]$TopFeat$Names)
					names(sharedfeat)[f]=paste("Shared: ",names(DataFeat[[1]]$Characteristics)[f],sep="")

					nsharedfeat[[f]]=length(sharedfeat[[f]])
					names(nsharedfeat)[f]=paste("Nshared: ",names(DataFeat[[i]]$Characteristics)[f],sep="")


				}

			}
		}
	}

	pvalsgenes=list()
	meanpvalsgenes=c()

	pvalspaths=list()
	meanpvalspaths=c()

	pvalsfeat=list()
	meanpvalsfeat=c()


	if(!(is.null(sharedgenes)) & nsharedgenes != 0 ){
		for(c in 1:nmethods){
			pvalsg=c()
			for(g in sharedgenes){
				if(!(is.na(DataLimma[[c]])[1])){
					pvalsg=c(pvalsg,DataLimma[[c]]$Genes$TopDE$adj.P.Val[DataLimma[[c]]$Genes$TopDE$ID==g])
				}
			}

			pvalsgenes[[c]]=pvalsg
			names(pvalsgenes)[c]=paste("P.Val.",names[c],sep="")
		}

		for(g1 in 1:length(sharedgenes)){
			pvalstemp=c()
			for(c in 1:nmethods){
				if(!(is.na(DataLimma[[c]])[1])){
					pvalstemp=c(pvalstemp,pvalsgenes[[c]][[g1]])
				}
			}
			meanpvalsgenes=c(meanpvalsgenes,mean(pvalstemp))
		}
		pvalsgenes[[nmethods+1]]=meanpvalsgenes
		names(pvalsgenes)[nmethods+1]="Mean pvals genes"
	}
	else{
		pvalsgenes=NULL
		nsharedgenes=NULL
	}

	if(!(is.null(sharedpaths)) & nsharedpaths != 0){
		for(c in 1:nmethods){
			pvalsp=c()
			if(!(is.na(DataMLP[[c]])[1])){
				for(p in sharedpaths){
					pvalsp=c(pvalsp,DataMLP[[c]][[3]][DataMLP[[c]][[3]]$geneSetDescription==p,5][1])
				}
			}

			pvalspaths[[c]]=pvalsp
			names(pvalspaths)[c]=paste("P.Val.",names[c],sep="")
		}


		for(p1 in 1:length(sharedpaths)){
			pvalstemp1=c()
			for(c in 1:nmethods){
				if(!(is.na(DataMLP[[c]])[1])){
					pvalstemp1=c(pvalstemp1,pvalspaths[[c]][[p1]])

				}

			}

			meanpvalspaths=c(meanpvalspaths,mean(pvalstemp1))
		}
		pvalspaths[[nmethods+1]]=meanpvalspaths
		names(pvalspaths)[nmethods+1]="Mean pvals paths"
	}
	else{
		pvalpaths=NULL
		nsharedpaths=NULL
	}

	if(!(is.null(sharedfeat)) & !(is.null(Reduce("+",nsharedfeat)))){
		for(f in 1:length(DataFeat[[1]]$Characteristics)){
			pvalschar=list()
			for(c in 1:nmethods){
				pvalsf=c()
				if(!(is.na(DataFeat[[c]])[1])){
					for(s in sharedfeat[[f]]){
						pvalsf=c(pvalsf,DataFeat[[c]]$Characteristics[[f]]$TopFeat$adj.P.Val[DataFeat[[c]]$Characteristics[[f]]$TopFeat$Names==s])
					}
				}

				pvalschar[[c]]=pvalsf
				names(pvalschar)[c]=paste("P.Val.",names[c],sep="")
			}
			pvalsfeat[[f]]=pvalschar
			names(pvalsfeat)[f]=names(DataFeat[[1]]$Characteristics)[f]

		}

		for(f in 1:length(DataFeat[[1]]$Characteristics)){
			meanpvalsfeat=c()
			for(f1 in 1:length(sharedfeat[[f]])){
				pvalstemp=c()
				for(c in 1:nmethods){
					if(!(is.na(DataFeat[[c]])[1]) & pvalsfeat[[f]]){
						pvalstemp=c(pvalstemp,pvalsfeat[[f]][[c]][[f1]])
					}
				}
				meanpvalsfeat=c(meanpvalsfeat,mean(pvalstemp))
			}

			pvalsfeat[[f]][[nmethods+1]]=meanpvalsfeat
			names(pvalsfeat[[f]])[nmethods+1]="Mean pvals feat"
		}
		lenchar=length(DataFeat[[1]]$Characteristics)

	}
	else{
		if(is.null(sharedfeat)){
			pvalsfeat=NULL
			nsharedfeat=NULL
			lenchar=0
		}
		else if(!(is.null(sharedfeat))){
			pvalsfeat=NULL
			lenchar=lenchar=length(DataFeat[[1]]$Characteristics)
		}
	}

	if(!(is.null(temp1f))){
		temp1f=do.call(rbind.data.frame, temp1f)
		colnames(temp1f)=names(DataFeat[[1]]$Characteristics)
		temp1f=as.matrix(temp1f)
		nsharedfeat=do.call(cbind.data.frame, nsharedfeat)
		nsharedfeat=as.matrix(nsharedfeat)
	}
	part1=cbind(cbind(temp1g,temp1p),temp1f)
	part1=as.matrix(part1)
	if(is.null(nsharedgenes) | is.null(nsharedpaths)  | is.null(nsharedfeat)){
		if(!(is.null(temp1g))){
			nsharedgenes=0
		}
		if(!(is.null(temp1p))){
			nsharedpaths=0
		}
		if(!(is.null(temp1f))){
			nsharedfeat=rep(0,length(temp1f))
		}
	}
	part2=cbind(cbind(nsharedgenes,nsharedpaths),nsharedfeat)
	part2=as.matrix(part2)
	rownames(part2)="NShared"
	#print(str(part2))
	colnames(part1)=NULL
	colnames(part2)=NULL

	part3=c()
	for(r in 1:length(comps)){
		part3=rbind(part3,rep(comps[r],dim(part1)[2]))
	}
	colnames(part3)=NULL
	rownames(part3)=names(comps)
	part4=rep(nsharedcomps,dim(part1)[2])
	names(part4)=NULL
	temp=rbind(part1,part2,part3,part4)
	rownames(temp)[nrow(temp)]="Nsharedcomps"


	table=cbind(table,temp)
	colnames(table)=seq(1,ncol(table))

	if(!(is.null(pvalsgenes))){
		SharedGenes=cbind(SharedGenes=sharedgenes,do.call(cbind.data.frame, pvalsgenes))
	}
	else{
		SharedGenes=NULL
	}
	if(!(is.null(pvalspaths))){
		SharedPaths=cbind(SharedPaths=sharedpaths,do.call(cbind.data.frame, pvalspaths))
	}
	else{
		SharedPaths=NULL
	}
	if(!(is.null(pvalsfeat))){
		SharedFeat=list()
		for(f in 1:lenchar){
			SharedFeat[[f]]=cbind(SharedFeat=sharedfeat[[f]],do.call(cbind.data.frame, pvalsfeat[[f]]))
			names(SharedFeat)[f]=names(pvalsfeat)[f]
		}
	}
	else{
		SharedFeat=NULL
	}

	which[[1]]=list(SharedComps=sharedcomps,SharedGenes=SharedGenes,SharedPaths=SharedPaths,SharedFeat=SharedFeat)
	names(which)[1]="Selection"

	#Sep for all situations?
	if(all(!(is.null(DataLimma)),!(is.null(DataMLP)),!(is.null(DataFeat)))){
		for (i in 1:length(seq(1,dim(table)[2],lenchar+2))){
			number=seq(1,dim(table)[2],2+lenchar)[i]
			colnames(table)[number]=paste("G.Cluster",sep=" ")
			colnames(table)[number+1]=paste("P.Cluster",sep=" ")
			for(u in seq(1:lenchar)){
				colnames(table)[number+1+u]=paste(paste('Feat.',names(DataFeat[[1]]$Characteristics)[u],sep=""),paste(".Cluster",sep=" "),sep="")
			}

		}
	}

	else if(all(!(is.null(DataLimma)),!(is.null(DataMLP)),is.null(DataFeat))){
		for (i in 1:length(seq(1,dim(table)[2],2))){
			number=seq(1,dim(table)[2],2)[i]
			colnames(table)[number]=paste("G.Cluster",i,sep=" ")
			colnames(table)[number+1]=paste("P.Cluster",i,sep=" ")

		}
	}

	else if(all(!(is.null(DataLimma)),is.null(DataMLP),!(is.null(DataFeat)))){
		for (i in 1:length(seq(1,dim(table)[2],lenchar+1))){
			number=seq(1,dim(table)[2],1+lenchar)[i]
			colnames(table)[number]=paste("G.Cluster",i,sep=" ")
			for(u in seq(1:lenchar)){
				colnames(table)[number+u]=paste(paste('Feat.',names(DataFeat[[1]]$Characteristics)[u],sep=""),paste(".Cluster",i,sep=" "),sep="")
			}

		}
	}

	else if(all((is.null(DataLimma)),!(is.null(DataMLP)),!(is.null(DataFeat)))){
		for (i in 1:length(seq(1,dim(table)[2],lenchar+1))){
			number=seq(1,dim(table)[2],lenchar+1)[i]
			colnames(table)[number]=paste("P.Cluster",i,sep=" ")
			for(u in seq(1:lenchar)){
				colnames(table)[number+u]=paste(paste('Feat.',names(DataFeat[[1]]$Characteristics)[u],sep=""),paste(".Cluster",i,sep=" "),sep="")
			}

		}
	}

	else if(all(!(is.null(DataLimma)),(is.null(DataMLP)),(is.null(DataFeat)))){
		for (i in 1:length(seq(1,dim(table)[2],1))){
			colnames(table)[i]=paste("G.Cluster",sep=" ")
		}
	}

	else if(all((is.null(DataLimma)),!(is.null(DataMLP)),(is.null(DataFeat)))){
		for (i in 1:length(seq(1,dim(table)[2],1))){
			colnames(table)[i]=paste("P.Cluster",sep=" ")
		}
	}

	else if(all((is.null(DataLimma)),(is.null(DataMLP)),!(is.null(DataFeat)))){
		for (i in 1:length(seq(1,dim(table)[2],lenchar))){
			number=seq(1,dim(table)[2],2)[i]
			for(u in c(0,1)){
				colnames(table)[number+u]=paste(paste('Feat.',names(DataFeat[[1]]$Characteristics)[u+1],sep=""),paste(".Cluster",sep=" "),sep="")
			}

		}
	}



	ResultShared=list(Table=table,Which=which)
	return(ResultShared)


}

#' @title Intersection of genes over multiple methods for a selection of objects.
#' @param DataLimma Optional. The output of a \code{DiffGenes} function. Default is NULL.
#' @param names Optional. Names of the methods or "Selection" if it only
#' considers a selection of objects. Default is NULL.
#' @description Internal function of \code{SharedGenesPathsFeat}.
#' @export
#' @return A list with a Table and a Which element for the shared genes.
#' @examples
#' \dontrun{
#' SharedSelectionLimma(DataLimma=MCF7_FT_DE)
#' }
SharedSelectionLimma<-function(DataLimma=NULL,names=NULL){  #Input=result of DiffGenes.2 and Geneset.intersect

	which=list()
	table=c()

	if(is.null(names)){
		for(j in 1:length(DataLimma)){
			names[j]=paste("Method",j,sep=" ")
		}
	}

	nmethods=length(DataLimma)

	temp1g=c()
	comps=c()

	pvalsg=c()
	for (i in 1:nmethods){

		temp1g=c(temp1g,length(DataLimma[[i]]$Genes$TopDE$Genes))
		comps=c(comps,length(DataLimma[[i]]$objects$LeadCpds))


		names(temp1g)[i]=names[i]
		names(comps)[i]=paste("Ncomps",names[i],i,sep=" ")




		if (i==1){
			if(!(is.na(DataLimma[[i]])[1])){
				sharedcomps=DataLimma[[i]]$objects$LeadCpds
				sharedgenes=DataLimma[[i]]$Genes$TopDE$Genes


				pvalsg=c(pvalsg,DataLimma[[i]]$Genes$TopDE$adj.P.Val)


				nsharedcomps=length(DataLimma[[i]]$objects$LeadCpds)
				nsharedgenes=length(DataLimma[[i]]$Genes$TopDE$Genes)

				names(nsharedgenes)="nshared"

				names(nsharedcomps)="nsharedcomps"
			}

		}
		else{
			sharedcomps=intersect(sharedcomps,DataLimma[[i]]$objects$LeadCpds)
			sharedgenes=intersect(sharedgenes,DataLimma[[i]]$Genes$TopDE$Genes)


			nsharedcomps=length(intersect(sharedcomps,DataLimma[[i]]$objects$LeadCpds))
			nsharedgenes=length(intersect(sharedgenes,DataLimma[[i]]$Genes$TopDE$Genes))

			names(nsharedgenes)="nshared"

			names(nsharedcomps)="nsharedcomps"
		}

	}
	pvalsgenes=list()
	meanpvalsgenes=c()

	if(nsharedgenes != 0){
		for(c in 1:nmethods){
			pvalsg=c()
			for(g in sharedgenes){
				if(!(is.na(DataLimma[[c]])[1])){
					pvalsg=c(pvalsg,DataLimma[[c]]$Genes$TopDE$adj.P.Val[DataLimma[[c]]$Genes$TopDE$Genes==g])
				}
			}

			pvalsgenes[[c]]=pvalsg
			names(pvalsgenes)[c]=paste("Method",c,sep=" ")
		}

		for(g1 in 1:length(sharedgenes)){
			pvalstemp=c()
			for(c in 1:nmethods){
				if(!(is.na(DataLimma[[c]])[1])){
					pvalstemp=c(pvalstemp,pvalsgenes[[c]][[g1]])
				}
			}
			meanpvalsgenes=c(meanpvalsgenes,mean(pvalstemp))
		}
		pvalsgenes[[nmethods+1]]=meanpvalsgenes
		names(pvalsgenes)[nmethods+1]="Mean pvals genes"
	}
	else{pvalsgenes=0}


	temp=rbind(temp1g,nsharedgenes,comps,nsharedcomps)

	table=cbind(table,temp)

	which[[1]]=list(sharedcomps=sharedcomps,sharedgenes=sharedgenes,pvalsgenes=pvalsgenes)
	#names(which)[1]=paste("Cluster",i,sep=" ")


	ResultShared=list(Table=table,Which=which)
	return(ResultShared)
}

#' @title Intersection of pathways over multiple methods for a selection of objects.
#' @param DataMLP Optional. The output of \code{Geneset.intersect} function. Default is NULL.
#' @param names Optional. Names of the methods or "Selection" if it only
#' considers a selection of objects. Default is NULL.
#' @description Internal function of \code{SharedGenesPathsFeat}.
#' @export
#' @return A list with a Table and a Which element for the shared pathways.
#' @examples
#' \dontrun{
#' SharedSelectionMLP(DataMLP=MCF7_Paths_intersection)
#' }
SharedSelectionMLP<-function(DataMLP=NULL,names=NULL){  #Input=result of DiffGenes.2 and Geneset.intersect

	which=list()
	table=c()


	nmethods=length(DataMLP)

	if(is.null(names)){
		for(j in 1:length(DataMLP)){
			names[j]=paste("Method",j,sep=" ")
		}
	}

	temp1g=c()
	temp1p=c()
	comps=c()

	pvalsg=c()
	pvalsp=c()
	for (i in 1:nmethods){

		temp1g=c(temp1g,length(DataMLP[[i]]$Genes$TopDE$Genes))
		temp1p=c(temp1p,length(DataMLP[[i]][[3]]$geneSetDescription))
		comps=c(comps,length(DataMLP[[i]]$objects$LeadCpds))


		names(temp1g)[i]=names[i]
		names(temp1p)[i]=names[i]
		names(comps)[i]=paste("Ncomps",names[i],i,sep=" ")


		if (i==1){
			if(!(is.na(DataMLP[[i]])[1]) | !(is.na(DataMLP[[i]])[1])){
				sharedcomps=DataMLP[[i]]$objects$LeadCpds
				sharedgenes=DataMLP[[i]]$Genes$TopDE$Genes
				sharedpaths=DataMLP[[i]][[3]]$geneSetDescription

				pvalsg=c(pvalsg,DataMLP[[i]]$Genes$TopDE$adj.P.Val)
				pvalsp=c(pvalsp,DataMLP[[i]]$mean_geneSetPValue)

				nsharedcomps=length(DataMLP[[i]]$objects$LeadCpds)
				nsharedgenes=length(DataMLP[[i]]$Genes$TopDE$Genes)
				nsharedpaths=length(DataMLP[[i]][[3]]$geneSetDescription)
				names(nsharedgenes)="nshared"
				names(nsharedpaths)="nshared"
				names(nsharedcomps)="nsharedcomps"
			}

		}
		else{
			sharedcomps=intersect(sharedcomps,DataMLP[[i]]$objects$LeadCpds)
			sharedgenes=intersect(sharedgenes,DataMLP[[i]]$Genes$TopDE$Genes)
			sharedpaths=intersect(sharedpaths,DataMLP[[i]][[3]]$geneSetDescription)

			nsharedcomps=length(intersect(sharedcomps,DataMLP[[i]]$objects$LeadCpds))
			nsharedgenes=length(intersect(sharedgenes,DataMLP[[i]]$Genes$TopDE$Genes))
			nsharedpaths=length(intersect(sharedpaths,DataMLP[[i]][[3]]$geneSetDescription))
			names(nsharedgenes)="nshared"
			names(nsharedpaths)="nshared"
			names(nsharedcomps)="nsharedcomps"
		}

	}
	pvalsgenes=list()
	meanpvalsgenes=c()
	meanpvalspaths=c()
	pvalspaths=list()

	if(nsharedgenes != 0){
		for(c in 1:nmethods){
			pvalsg=c()
			for(g in sharedgenes){
				if(!(is.na(DataMLP[[c]])[1])){
					pvalsg=c(pvalsg,DataMLP[[c]]$Genes$TopDE$adj.P.Val[DataMLP[[c]]$Genes$TopDE$Genes==g])
				}
			}

			pvalsgenes[[c]]=pvalsg
			names(pvalsgenes)[c]=paste("Method",c,sep=" ")
		}

		for(g1 in 1:length(sharedgenes)){
			pvalstemp=c()
			for(c in 1:nmethods){
				if(!(is.na(DataMLP[[c]])[1])){
					pvalstemp=c(pvalstemp,pvalsgenes[[c]][[g1]])
				}
			}
			meanpvalsgenes=c(meanpvalsgenes,mean(pvalstemp))
		}
		pvalsgenes[[nmethods+1]]=meanpvalsgenes
		names(pvalsgenes)[nmethods+1]="Mean pvals genes"
	}
	else{pvalsgenes=0}

	if(nsharedpaths!=0){
		for(c in 1:nmethods){
			pvalsp=c()
			if(!(is.na(DataMLP[[c]])[1])){
				for(p in sharedpaths){
					pvalsp=c(pvalsp,DataMLP[[c]][[3]][DataMLP[[c]][[3]]$geneSetDescription==p,5])
				}
			}

			pvalspaths[[c]]=pvalsp


			names(pvalspaths)[c]=paste("Method",c,sep=" ")
		}


		for(p1 in 1:length(sharedpaths)){
			pvalstemp1=c()
			for(c in 1:nmethods){
				if(!(is.na(DataMLP[[c]])[1])){
					pvalstemp1=c(pvalstemp1,pvalspaths[[c]][[p1]])

				}

			}

			meanpvalspaths=c(meanpvalspaths,mean(pvalstemp1))

		}
		pvalspaths[[nmethods+1]]=meanpvalspaths
		names(pvalspaths)[nmethods+1]="Mean pvals paths"
	}
	else{pvalpaths=0}

	temp=rbind(cbind(temp1g,temp1p),cbind(nsharedgenes,nsharedpaths),cbind(comps,comps),cbind(nsharedcomps,nsharedcomps))

	table=cbind(table,temp)

	which[[1]]=list(sharedcomps=sharedcomps,sharedgenes=sharedgenes,pvalsgenes=pvalsgenes,sharedpaths=sharedpaths,pvalspaths=pvalspaths)
	#names(which)[1]=paste("Cluster",i,sep=" ")


	for (i in 1:length(seq(1,dim(table)[2],2))){
		number=seq(1,dim(table)[2],2)[i]
		colnames(table)[number]=c("G.Cluster")
		colnames(table)[number+1]=c("P.Cluster")

	}
	ResultShared=list(Table=table,Which=which)
	return(ResultShared)


}

test_that("pathway/gene-set functions are exported functions", {
	expect_true(is.function(DiffGenes))
	expect_true(is.function(DiffGenesSelection))
	expect_true(is.function(FindGenes))
	expect_true(is.function(Geneset.intersect))
	expect_true(is.function(Geneset.intersectSelection))
	expect_true(is.function(PathwayAnalysis))
	expect_true(is.function(Pathways))
	expect_true(is.function(PathwaysIter))
	expect_true(is.function(PathwaysSelection))
	expect_true(is.function(PreparePathway))
	expect_true(is.function(SharedGenesPathsFeat))
	expect_true(is.function(SharedSelection))
	expect_true(is.function(SharedSelectionLimma))
	expect_true(is.function(SharedSelectionMLP))
})

test_that("DiffGenes errors on missing limma", {
	skip_if(requireNamespace("limma", quietly = TRUE))
	expect_error(DiffGenes(list(), geneExpr = NULL), "limma")
})

test_that("DiffGenesSelection errors on missing limma", {
	skip_if(requireNamespace("limma", quietly = TRUE))
	expect_error(DiffGenesSelection(list(), Selection = "a"), "limma")
})

test_that("Geneset.intersect errors on missing plyr", {
	skip_if(requireNamespace("plyr", quietly = TRUE))
	expect_error(Geneset.intersect(list()), "plyr")
})

test_that("Geneset.intersectSelection errors on missing plyr", {
	skip_if(requireNamespace("plyr", quietly = TRUE))
	expect_error(Geneset.intersectSelection(list()), "plyr")
})

test_that("PathwayAnalysis errors on missing MLP", {
	skip_if(requireNamespace("MLP", quietly = TRUE))
	expect_error(PathwayAnalysis(list()), "MLP")
})

test_that("Pathways errors on missing MLP", {
	skip_if(requireNamespace("MLP", quietly = TRUE))
	expect_error(Pathways(list()), "MLP")
})

test_that("PathwaysIter errors on missing MLP", {
	skip_if(requireNamespace("MLP", quietly = TRUE))
	expect_error(PathwaysIter(list()), "MLP")
})

test_that("PathwaysSelection errors on missing MLP", {
	skip_if(requireNamespace("MLP", quietly = TRUE))
	expect_error(PathwaysSelection(Selection = "a"), "MLP")
})

# =============================================================================
# M_ABCdist_WC.R
# Multi-source M-ABC + WeightedClust fusion. Per source ABCdist.SingleInMultiple
# yields a [0,1] dissimilarity; WeightedClust() then explores convex
# combinations of the source dissimilarities over a weight grid.
# Function: M_ABCdist.WC
# =============================================================================

#' Multi-source M-ABC fused with WeightedClust
#'
#' Runs \code{\link{ABCdist.SingleInMultiple}} on each source, then feeds the
#' per-source dissimilarities to \code{\link{WeightedClust}} (\code{type =
#' "dist"}) to explore convex combinations over a weight grid and return the
#' focus-weight clustering.
#'
#' @inheritParams M_ABCdist
#' @param StopRange Passed to \code{\link{WeightedClust}}.
#' @param wc_weight Weight grid for \code{\link{WeightedClust}}.
#' @param wc_weightclust Focus weight whose result is returned in \code{Clust}.
#' @param wc_linkage,wc_alpha Final clustering controls for \code{\link{WeightedClust}}.
#' @return A list of class \code{"M_ABCdist_WC"} with \code{source_D}, the full
#'   \code{WC} object, the focus \code{DistM} and \code{Clust}, and the
#'   \code{weight_grid}.
#' @references \insertCite{Amaratunga2008}{MosaiClusteR}
#' @seealso \code{\link{M_ABCdist}}, \code{\link{WeightedClust}}
#' @examples
#' data(mosaic_toy)
#' fit <- M_ABCdist.WC(mosaic_toy$List, numsim = 40,
#'                     wc_weight = seq(1, 0, -0.25), wc_weightclust = 0.5)
#' dim(fit$DistM)
#' @export
M_ABCdist.WC <- function(List,
                         distmeasure    = rep("euclidean", length(List)),
                         weighting      = rep("var",       length(List)),
                         normalize      = rep(list(NULL),  length(List)),
                         gr             = c(),
                         bag            = rep(TRUE,        length(List)),
                         numsim         = 1000L,
                         numvar         = rep(100L,        length(List)),
                         linkage        = rep("ward.D",    length(List)),
                         NC             = NULL,
                         NC2            = NULL,
                         StopRange      = FALSE,
                         wc_weight      = seq(1, 0, -0.1),
                         wc_weightclust = 0.5,
                         wc_linkage     = "ward",
                         wc_alpha       = 0.625,
                         mds            = FALSE,
                         nugget_type    = "between",
                         nugget_args    = list()) {
  if (!exists("WeightedClust", mode = "function", inherits = TRUE))
    stop("M_ABCdist.WC() needs WeightedClust(); it ships with MosaiClusteR.")
  K <- length(List)
  if (K < 1L) stop("List must contain at least one data source.")
  distmeasure <- rep_len(distmeasure, K); weighting <- rep_len(weighting, K)
  bag <- as.logical(rep_len(bag, K)); numvar <- as.integer(rep_len(numvar, K))
  linkage <- rep_len(linkage, K)
  if (!is.list(normalize)) normalize <- as.list(normalize)
  normalize <- rep_len(normalize, K)
  if (length(nugget_args) && all(vapply(nugget_args, is.list, logical(1))))
    nugget_args <- rep_len(nugget_args, K) else nugget_args <- rep(list(nugget_args), K)

  List <- lapply(List, as.matrix)
  samp_names <- rownames(List[[1]]); N <- nrow(List[[1]])
  for (k in seq_len(K))
    if (nrow(List[[k]]) != N)
      stop("Source ", k, " has ", nrow(List[[k]]), " rows; expected ", N, ".")

  source_D <- vector("list", K)
  for (k in seq_len(K)) {
    message("M_ABCdist.WC: source ", k, " of ", K, " (weighting=", weighting[k], ") ...")
    source_D[[k]] <- ABCdist.SingleInMultiple(
      data = List[[k]], distmeasure = distmeasure[k], weighting = weighting[k],
      normalize = normalize[[k]], gr = gr, bag = bag[k], numsim = as.integer(numsim),
      numvar = numvar[k], linkage = linkage[k], NC = NC,
      nugget_type = nugget_type, nugget_args = nugget_args[[k]])$D
  }

  WC <- WeightedClust(
    List = source_D, type = "dist", distmeasure = rep(NULL, K),
    normalize = rep(FALSE, K), method = rep(NULL, K), StopRange = StopRange,
    weight = wc_weight, weightclust = wc_weightclust, clust = "agnes",
    linkage = wc_linkage, alpha = wc_alpha)

  DistM <- matrix(as.double(WC$Clust$DistM), N, N,
                  dimnames = list(samp_names, samp_names))
  diag(DistM) <- 0
  Clust <- WC$Clust$Clust

  if (isTRUE(mds)) {
    co <- stats::cmdscale(stats::as.dist(DistM), k = 2L)
    plot(co, pch = 19, col = "darkgreen", xlab = "Dim 1", ylab = "Dim 2",
         main = "MDS - M-ABCdist-WC")
  }

  out <- list(source_D = source_D, WC = WC, DistM = DistM, Clust = Clust,
              weight_grid = names(WC$Results))
  if (!is.null(NC2)) {
    nc <- as.integer(NC2)
    if (nc >= 2L && nc < N) out$cut <- stats::cutree(stats::as.hclust(Clust), k = nc)
  }
  attr(out, "method") <- "M-ABCdist-WC"
  class(out) <- c("M_ABCdist_WC", "list")
  out
}

#' @title Voting-based consensus clustering
#'
#' @description Consensus Clustering performs voting-based consensus on a set of
#' single-source partitions. Each data matrix is clustered separately and the
#' resulting label matrix is reconciled into a single consensus partition by an
#' iterative voting scheme. Three voting methods are available: Iterative Voting
#' Consensus (IVC), Iterative Probabilistic Voting Consensus (IPVC) and Iterative
#' Pairwise Consensus (IPC).
#'
#' @param List A list of data matrices. It is assumed the rows are corresponding with the objects.
#' @param type Indicates whether the provided matrices in "List" are either data matrices ("data"),
#' distance matrices ("dist") or clustering results obtained from the data ("clust").
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").
#' @param normalize Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. More details on normalization in \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".
#' @param nrclusters The number of clusters to divide each individual dendrogram in. Default is c(7,7) for two data sets.
#' @param gap Logical. Whether the optimal number of clusters should be determined with the gap statistic. Defaults to FALSE.
#' @param maxK The maximal number of clusters to investigate in the gap statistic. Default is 15.
#' @param votingMethod The voting method to be performed: one of "IVC", "IPVC" or "IPC".
#' @param optimalk The number of clusters for the final consensus partition. Defaults to 7.
#' @return The returned value is a list of two elements:
#' \item{DistM}{A NULL object}
#' \item{Clust}{The resulting clustering}
#' The value has class 'Ensemble'.
#' @references Nguyen N. and Caruana R. (2007). Consensus Clusterings. Seventh IEEE International Conference on Data Mining (ICDM 2007), 607-612.
#' @export
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' fit <- ConsensusClustering(List = mosaic_toy$List, type = "data",
#'   distmeasure = c("euclidean", "euclidean"), normalize = c(FALSE, FALSE),
#'   method = c(NULL, NULL), clust = "agnes", linkage = c("ward", "ward"),
#'   nrclusters = c(3, 3), gap = FALSE, votingMethod = "IVC", optimalk = 3)
#' }
ConsensusClustering<-function(List,type=c("data","dist","clust"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),clust = "agnes", linkage = c("flexible","flexible"),alpha=0.625,nrclusters=c(7,7),gap = FALSE, maxK = 15,votingMethod=c("IVC","IPVC","IPC"),optimalk=7){

	if(type=="data"){

		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,,drop=FALSE]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type="data",distmeasure[i],normalize[i],method[i],clust,linkage[i],alpha,StopRange=TRUE))

		Dist=lapply(seq(length(List)),function(i) Clusterings[[i]]$DistM)

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=clusters
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))

	}

	else if(type=="dist"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,OrderNames]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type,distmeasure[i],normalize=FALSE,method=NULL,clust,linkage[i],alpha,StopRange=TRUE))

		Dist=List

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=ceiling(mean(clusters))
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))


	}
	else if(type=="clust"){

		Clusterings=List

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		if(is.null(nrclusters)){
			stop("Please specify a number of clusters")
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))

	}


	Y=Reduce(cbind,Clusters)
	rownames(Y)=rownames(List[[1]])
	colnames(Y)=seq(1,ncol(Y))


	if(votingMethod=="IVC"){
		target=sample(optimalk,nrow(Y),replace=TRUE)
		update=rep(0,nrow(Y))
		Continue=TRUE
		while(Continue){

			#representation of each cluster
			P=list()
			y_P=list()
			for(i in unique(target)){
				P_i=which(target==i)
				y_Pi=apply(Y,2,function(k) as.numeric(names(which.max(table(k[P_i])))))
				P[[i]]=P_i
				y_P[[i]]=y_Pi

			}

			#update y
			for(y in 1:length(target)){
				distances=c()
				for(a in 1:length(y_P)){
					distances=c(distances,sum(Y[y,] != y_P[[a]])	)
				}

				update[y]=which.min(distances)

			}
			if(all(update==target)){
				Continue=FALSE
				#print("here")
			}
			else{
				target=update
			}

		}

	}
	else if(votingMethod=="IPVC"){
		target=sample(optimalk,nrow(Y),replace=TRUE)
		update=rep(0,nrow(Y))
		Continue=TRUE

		while(Continue){

			#representation of each cluster
			P=list()
			n_P=list()
			for(i in unique(target)){
				P_i=which(target==i)
				P[[i]]=P_i
				n_P[[i]]=length(P_i)

			}

			#update y
			for(y in 1:length(target)){
				distances=c()

				for(a in 1:length(P)){
					dist=0
					for(b in 1:ncol(Y)){
						dist=dist+sum(Y[y,b] != Y[P[[a]],b])/n_P[[a]]
					}
					distances=c(distances,dist)
				}

				update[y]=which.min(distances)

			}
			if(all(update==target)){
				Continue=FALSE
				#print("here")
			}
			else{
				target=update
			}

		}



	}
	else if(votingMethod=="IPC"){


		#similarity matrix
		H=list()
		for(i in 1:length(List)){
			Bin=matrix(0,nrow=max(Clusters[[i]]),ncol=length(Clusters[[i]]))
			for(j in 1:nrow(Bin)){
				Bin[j,which(Clusters[[i]]==j)]=1
			}
			H[[i]]=as.data.frame(Bin)
		}
		T=data.table::rbindlist(H)
		S=t(as.matrix(T))%*%(as.matrix(T))
		S=S/length(List)

		if(type!="clust"){
			rownames(S)=rownames(List[[1]])
			colnames(S)=rownames(List[[1]])
		}
		else{
			rownames(S)=rownames(List[[1]]$DistM)
			colnames(S)=rownames(List[[1]]$DistM)
		}

		target=sample(optimalk,nrow(Y),replace=TRUE)
		update=rep(0,nrow(Y))
		Continue=TRUE

		while(Continue){

			#representation of each cluster
			P=list()
			n_P=list()
			for(i in unique(target)){
				P_i=which(target==i)
				P[[i]]=P_i
				n_P[[i]]=length(P_i)

			}

			#update x
			for(x in 1:length(target)){
				similarities=c()

				for(a in 1:length(P)){
					sim=sum(S[x,P[[a]]])/n_P[[a]]
					similarities=c(similarities,sim)
				}

				update[x]=which.max(similarities)
			}


			if(all(update==target)){
				Continue=FALSE
				#print("here")
			}
			else{
				target=update
			}

		}

	}



	Clusters=target

	if(type!="clust"){
		names(Clusters)=rownames(List[[1]])
	}
	else{
		names(Clusters)=rownames(List[[1]]$DistM)
	}

	order=sort(Clusters)
	order.lab=names(order)
	Out=list(DistM=NULL,Clust=list(order=order,order.lab=order.lab,Clusters=Clusters))
	attr(Out,"method")="Ensemble"
	return(Out)

}


#' @title Weighting on Membership clustering
#'
#' @description Weighting on Membership (WonM) is a hierarchy-based consensus
#' technique. Each data matrix is hierarchically clustered and the resulting
#' dendrogram is cut into a range of K values. For every cut a co-membership
#' matrix is built, recording which objects fall in the same cluster. These
#' matrices are averaged within and across data sets to produce an overall
#' consensus matrix, which is transformed into a dissimilarity and clustered a
#' final time.
#'
#' @param List A list of data matrices. It is assumed the rows are corresponding with the objects.
#' @param type Indicates whether the provided matrices in "List" are either data matrices ("data"),
#' distance matrices ("dist") or clustering results obtained from the data ("clusters").
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").
#' @param normalize Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. More details on normalization in \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.
#' @param nrclusters A list with, for each data matrix, the sequence of numbers of clusters to cut the dendrogram in. Defaults to list(seq(5,25), seq(5,25)).
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".
#' @return The returned value is a list of two elements:
#' \item{DistM}{The resulting consensus dissimilarity matrix}
#' \item{Clust}{The resulting clustering}
#' The value has class 'WonM'.
#' @references Saeed F., Salim N. and Abdo A. (2012). Voting-based consensus clustering for combining multiple clusterings of chemical structures. Journal of Cheminformatics, 4, 37.
#' @export
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' fit <- WonM(List = mosaic_toy$List, type = "data",
#'   distmeasure = c("euclidean", "euclidean"), normalize = c(FALSE, FALSE),
#'   method = c(NULL, NULL), nrclusters = list(seq(3, 6), seq(3, 6)),
#'   clust = "agnes", linkage = c("ward", "ward"))
#' }
WonM=function(List,type=c("data","dist","clusters"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),nrclusters=list(seq(5,25),seq(5,25)),clust="agnes",linkage=c("flexible","flexible"),alpha=0.625){

	type<-match.arg(type)


	#Step 1: Distance Matrices
	if(type=="data"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,,drop=FALSE]
		}
		Dist=lapply(seq(length(List)),function(i) Distance(List[[i]],distmeasure[i],normalize[i],method[i]))
	}
	else if(type=="dist"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,OrderNames]
		}
		Dist=List
	}
	else{
		Dist=lapply(seq(length(List)),function(i) return(List[[i]]$DistM))

		OrderNames=rownames(Dist[[1]])
		for(i in 1:length(Dist)){
			Dist[[i]]=Dist[[i]][OrderNames,OrderNames]
		}
	}


	#Step 2: perform hierarchical clustering on both distance matrices

	HClustering=lapply(seq(length(List)),function(i) cluster::agnes(Dist[[i]],diss=TRUE,method=linkage[i],par.method=alpha))


	#Step 3: cut the dendrograms into a range of K values

	#Give 0 to pair belonging together, give 1 to a pair not belonging together : ==> Distances created otherwise similarities.
	ClusterMembers<-function(HClust,nrclusters){
		Temp=lapply(seq(length(nrclusters)),function(i) stats::cutree(HClust,nrclusters[i]))
		CM=lapply(seq(length(nrclusters)),function(i) matrix(0,dim(List[[1]])[1],dim(List[[1]])[1]))

		clusters<-function(temp,cm){
			for(l in 1:length(temp)){
				label=temp[l]
				sameclust=which(temp==label)
				cm[l,sameclust]=1
			}
			return(cm)
		}

		CM2=lapply(seq(length(nrclusters)),function(i) clusters(temp=Temp[[i]],cm=CM[[i]]))
		Consensus2=Reduce("+",CM2)
		Consensus2=Consensus2/length(nrclusters)
		return(Consensus2)


	}

	Consensus=lapply(seq(length(List)), function(i) ClusterMembers(HClustering[[i]],nrclusters[[i]]))

	OverallConsensus=Reduce("+",Consensus)
	OverallConsensus=OverallConsensus/length(List)
	OverallConsensus=as.matrix(OverallConsensus)
	rownames(OverallConsensus)=rownames(Dist[[1]])
	colnames(OverallConsensus)=rownames(Dist[[1]])
	OverallConsensusD=1-OverallConsensus
	OverallClusteringR=cluster::agnes(OverallConsensusD,diss=TRUE,method="ward")

	out=list(DistM=OverallConsensusD,Clust=OverallClusteringR)
	attr(out,'method')<-'WonM'
	return(out)
}

test_that("ConsensusClustering (IVC) returns a consensus partition", {
  data(mosaic_toy)
  set.seed(1)
  fit <- ConsensusClustering(
    List = mosaic_toy$List, type = "data",
    distmeasure = c("euclidean", "euclidean"),
    normalize = c(FALSE, FALSE), method = c(NULL, NULL),
    clust = "agnes", linkage = c("ward", "ward"),
    nrclusters = c(3, 3), gap = FALSE,
    votingMethod = "IVC", optimalk = 3
  )
  expect_equal(attr(fit, "method"), "Ensemble")
  cl <- fit$Clust$Clusters
  expect_length(cl, nrow(mosaic_toy$List[[1]]))
  expect_true(all(cl %in% seq_len(3)))
})

test_that("ConsensusClustering (IPVC) returns a consensus partition", {
  data(mosaic_toy)
  set.seed(2)
  fit <- ConsensusClustering(
    List = mosaic_toy$List, type = "data",
    distmeasure = c("euclidean", "euclidean"),
    normalize = c(FALSE, FALSE), method = c(NULL, NULL),
    clust = "agnes", linkage = c("ward", "ward"),
    nrclusters = c(3, 3), gap = FALSE,
    votingMethod = "IPVC", optimalk = 3
  )
  cl <- fit$Clust$Clusters
  expect_length(cl, nrow(mosaic_toy$List[[1]]))
  expect_named(cl)
})

test_that("WonM returns a consensus dissimilarity matrix and clustering", {
  data(mosaic_toy)
  set.seed(3)
  n <- nrow(mosaic_toy$List[[1]])
  fit <- WonM(
    List = mosaic_toy$List, type = "data",
    distmeasure = c("euclidean", "euclidean"),
    normalize = c(FALSE, FALSE), method = c(NULL, NULL),
    nrclusters = list(seq(3, 6), seq(3, 6)),
    clust = "agnes", linkage = c("ward", "ward")
  )
  expect_equal(attr(fit, "method"), "WonM")
  expect_equal(dim(fit$DistM), c(n, n))
  expect_true(all(fit$DistM >= 0 & fit$DistM <= 1))
  expect_s3_class(fit$Clust, "agnes")
  lab <- stats::cutree(fit$Clust, k = 3)
  expect_length(lab, n)
})

.onAttach <- function(libname, pkgname) {
  packageStartupMessage(
    "MosaiClusteR ", utils::packageVersion("MosaiClusteR"),
    " - an umbrella framework for multi-source clustering. ",
    "See vignette('MosaiClusteR') and ?M_ABCpp.")
}

#' @title Ensemble clustering via graph partitioning (CSPA / HGPA / MCLA)
#'
#' @description Cluster-based ensemble clustering that combines several
#' single-source partitions into a single consensus partition through the
#' graph-partitioning algorithms of Strehl and Ghosh: the Cluster-based
#' Similarity Partitioning Algorithm (CSPA), the HyperGraph Partitioning
#' Algorithm (HGPA) and the Meta-CLustering Algorithm (MCLA), as well as a
#' "Best" selection strategy.
#'
#' \strong{External executable required.} This method does not run in pure R.
#' It writes the stacked label matrix to disk and delegates the actual graph
#' partitioning to an \strong{external graph-partitioning executable}
#' (e.g. a compiled CSPA/HGPA/MCLA binary backed by METIS / \code{gpmetis} /
#' \code{shmetis}) or to a \strong{MATLAB} session. These binaries are
#' \strong{not bundled} with the package; the user must supply and configure
#' them. The path to the executable is selected through the \code{executable}
#' argument. When \code{executable=FALSE} (the default), or when the configured
#' executable cannot be located, the function stops with an informative error
#' rather than attempting to launch an unavailable program.
#'
#' @export
#' @param List A list of data matrices. It is assumed the rows are corresponding with the objects.
#' @param type indicates whether the provided matrices in "List" are either data matrices, distance
#' matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
#' matrices is skipped and if type="clust" the single source clustering is skipped.
#' Type should be one of "data", "dist" or "clust".
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").
#' @param normalize Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".
#' @param nrclusters The number of clusters to divide each individual dendrogram in. Default is c(7,7) for two data sets.
#' @param gap Logical. Whether the optimal number of clusters should be determined with the gap statistic. Default is FALSE.
#' @param maxK The maximal number of clusters to investigate in the gap statistic. Default is 15.
#' @param ensembleMethod A character string indicating the consensus function to use. One of "CSPA", "HGPA", "MCLA" or "Best". Defaults to "CSPA".
#' @param waitingtime The maximum number of seconds to wait for the external program to write its result before stopping. Defaults to 300.
#' @param file_number An identifier appended to the temporary files exchanged with the external program. Defaults to 00.
#' @param executable Either FALSE (the default) or the path/flag identifying the external graph-partitioning executable (or MATLAB) to use. When FALSE the function stops, since no consensus can be computed in pure R. Precompiled binaries are not bundled with the package.
#' @return The returned value is a list of two elements:
#' \item{DistM}{A NULL object}
#' \item{Clust}{The resulting consensus clustering}
#' The value has class 'Ensemble'.
#' @references Strehl A. and Ghosh J. (2002). Cluster ensembles - a knowledge reuse framework for combining multiple partitions. Journal of Machine Learning Research, 3, 583-617.
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' L <- mosaic_toy$List
#'
#' # Requires an external graph-partitioning executable / MATLAB to be
#' # configured through the 'executable' argument.
#' res <- EnsembleClustering(List = L, type = "data",
#'   distmeasure = c("euclidean", "euclidean"), normalize = c(FALSE, FALSE),
#'   method = c(NULL, NULL), clust = "agnes",
#'   linkage = c("flexible", "flexible"), nrclusters = c(7, 7),
#'   gap = FALSE, maxK = 15, ensembleMethod = "CSPA",
#'   executable = "./EnsembleClusteringC")
#' }
EnsembleClustering<-function(List,type=c("data","dist","clust"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),clust = "agnes", linkage = c("flexible","flexible"),alpha=0.625,nrclusters=c(7,7),gap = FALSE, maxK = 15,ensembleMethod=c("CSPA","HGPA","MCLA","Best"),waitingtime=300,file_number=00,executable=FALSE){

	# This method relies on an external graph-partitioning executable (METIS /
	# gpmetis / shmetis based) or a MATLAB session to compute the consensus
	# partition. Such binaries are not bundled with the package. Refuse to run
	# (with a clear message) when no usable executable has been configured.
	if(isFALSE(executable) || is.null(executable)){
		stop(paste0("EnsembleClustering requires an external graph-partitioning executable ",
			"(e.g. a compiled CSPA/HGPA/MCLA binary backed by METIS / gpmetis / shmetis) ",
			"or MATLAB to compute the consensus partition. These binaries are not bundled ",
			"with the package. Configure one through the 'executable' argument; ",
			"it is currently FALSE."))
	}
	if(is.character(executable) && !nzchar(Sys.which(executable)) && !file.exists(executable)){
		stop(paste0("EnsembleClustering could not locate the configured graph-partitioning ",
			"executable '", executable, "'. Provide a valid path to a compiled CSPA/HGPA/MCLA ",
			"binary (METIS / gpmetis / shmetis) or to MATLAB. Precompiled binaries are not ",
			"bundled with the package."))
	}

	ensembleMethod <- match.arg(ensembleMethod)

	# Step 1: Generate several clustering results on the objects
	if(type=="data"){

		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,,drop=FALSE]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type="data",distmeasure[i],normalize[i],method[i],clust,linkage[i],alpha,gap,maxK,StopRange=TRUE))

		Dist=lapply(seq(length(List)),function(i) Clusterings[[i]]$DistM)

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=clusters
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))
	}

	else if(type=="dist"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,OrderNames]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type,distmeasure[i],normalize=FALSE,method=NULL,clust,linkage[i],alpha,gap,maxK,StopRange=TRUE))

		Dist=List

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=ceiling(mean(clusters))
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))


	}
	else if(type=="clust"){

		Clusterings=List

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		if(is.null(nrclusters)){
			stop("Please specify a number of clusters")
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))

	}
	#### Step 1 Complete

	# Step 2: Get data in format to be given to matlab =>  matrix with a row per clustering result
	if(type=="data" | type=="dist"){
		nc=nrow(List[[1]])
	}
	else if(type=="clust"){
		nc=nrow(List[[1]]$DistM)
	}

	matlabdata=matrix(unlist(Clusters),ncol=nc,byrow=TRUE)
	utils::write.table(matlabdata,file=paste("matlabdata_",file_number,".csv",sep=""),sep=",",col.names=FALSE,row.names=FALSE)

	if(ensembleMethod=="CSPA"){
		system(paste(executable," ",file_number,sep=""),intern=TRUE)
	}
	if(ensembleMethod=="HGPA"){
		system(paste(executable," ",file_number,sep=""),intern=TRUE)
	}
	if(ensembleMethod=="MCLA"){
		system(paste(executable," ",file_number,sep=""),intern=TRUE)
	}
	if(ensembleMethod=="Best"){
		system(paste(executable," -nodisplay -r \"run('EnsembleClusteringB ",file_number,"'); exit\"",sep=""),intern=TRUE)
	}

	Continue=FALSE
	time=0
	while(Continue==FALSE){
		Sys.sleep(15)
		time=time+15
		Continue=file.exists(paste("ClusterEnsemble_",file_number,".csv",sep=""))
		if(time>waitingtime & Continue==FALSE){
			stop(paste("Waited",waitingtime, "seconds for completion of the ensemble clustering procedure. Increase waiting time to continue.",sep=" "))
		}
	}

	ClusterEnsemble <- utils::read.table(paste("ClusterEnsemble_",file_number,".csv",sep=""),sep=",")
	ClusterEnsemble=as.vector(as.matrix(ClusterEnsemble))
	names(ClusterEnsemble)=rownames(List[[1]])


	DistM=NULL


	Clust=list()
	Clust$order=sort(ClusterEnsemble,index=TRUE)$ix
	Clust$order.lab=names(ClusterEnsemble[sort(ClusterEnsemble,index=TRUE)$ix])
	Clust$Clusters=ClusterEnsemble

	Out=list("DistM"=DistM,"Clust"=Clust)
	attr(Out,"method")="Ensemble"

	return(Out)
}


#' @title Ensemble hierarchical clustering via graph partitioning (METIS / MST)
#'
#' @description Ensemble Hierarchical Clustering (EHC) builds a cluster
#' association graph from the agglomerative clustering results of each data
#' source. Strengths of association between pairs of objects are accumulated
#' across the dendrograms (weighted by cluster depth and intra-cluster
#' proximity), yielding a weighted adjacency matrix that is then partitioned.
#'
#' Two partitioning strategies are available through \code{graphPartitioning}:
#' \describe{
#'   \item{"METIS"}{Delegates the partitioning of the association graph to an
#'     \strong{external graph-partitioning executable} (METIS / \code{gpmetis} /
#'     \code{shmetis}, or the corresponding MATLAB \code{hmetis} routine). These
#'     binaries are \strong{not bundled} with the package and must be configured
#'     through the \code{executable} argument. When \code{executable=FALSE}
#'     (the default), or when the configured executable cannot be located, the
#'     function stops with an informative error.}
#'   \item{"MST"}{A pure-R minimum spanning tree partitioning that needs no
#'     external executable. It does require the \pkg{igraph} package; if
#'     \pkg{igraph} is not installed the function stops with an informative
#'     error.}
#' }
#'
#' @export
#' @param List A list of data matrices. It is assumed the rows are corresponding with the objects.
#' @param type indicates whether the provided matrices in "List" are either data matrices, distance
#' matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
#' matrices is skipped and if type="clust" the single source clustering is skipped.
#' Type should be one of "data", "dist" or "clust".
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").
#' @param normalize Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".
#' @param gap Logical. Whether the optimal number of clusters should be determined with the gap statistic. Default is FALSE.
#' @param maxK The maximal number of clusters to investigate in the gap statistic. Default is 15.
#' @param graphPartitioning A character string indicating the graph partitioning strategy. One of "METIS" (external executable) or "MST" (pure R, requires igraph). Defaults to "METIS".
#' @param optimalk An estimate of the final optimal number of clusters. Default is 7.
#' @param waitingtime The maximum number of seconds to wait for the external program to write its result before stopping. Defaults to 300.
#' @param file_number An identifier appended to the temporary files exchanged with the external program. Defaults to 00.
#' @param executable Either FALSE (the default) or the path/flag identifying the external graph-partitioning executable (METIS / gpmetis / shmetis) or MATLAB to use for the "METIS" strategy. When FALSE the "METIS" path stops, since no partition can be computed in pure R. Precompiled binaries are not bundled with the package.
#' @return The returned value is a list of two elements:
#' \item{DistM}{The weighted cluster association matrix}
#' \item{Clust}{The resulting clustering}
#' The value has class 'Ensemble'.
#' @references Mirzaei A. and Rahmati M. (2010). A novel hierarchical-clustering-combination scheme based on fuzzy-similarity relations. IEEE Transactions on Fuzzy Systems, 18(1), 27-39.
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' L <- mosaic_toy$List
#'
#' # METIS path requires an external graph-partitioning executable / MATLAB
#' # configured through the 'executable' argument.
#' res <- EHC(List = L, type = "data",
#'   distmeasure = c("euclidean", "euclidean"), normalize = c(FALSE, FALSE),
#'   method = c(NULL, NULL), clust = "agnes",
#'   linkage = c("flexible", "flexible"), alpha = 0.625, gap = FALSE,
#'   maxK = 15, graphPartitioning = "METIS", optimalk = 7,
#'   executable = "./MetisAlgorithm")
#'
#' # MST path is pure R but needs the igraph package.
#' res2 <- EHC(List = L, type = "data", graphPartitioning = "MST")
#' }
EHC<-function(List,type=c("data","dist","clust"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),clust = "agnes", linkage = c("flexible","flexible"),alpha=0.625,gap = FALSE, maxK = 15,graphPartitioning=c("METIS","MST"),optimalk=7,waitingtime=300,file_number=00,executable=FALSE){

	graphPartitioning <- match.arg(graphPartitioning)

	# Guard the executable-dependent METIS path: refuse to run with a clear
	# message when no usable external graph-partitioning executable / MATLAB is
	# configured. Such binaries are not bundled with the package.
	if(graphPartitioning=="METIS"){
		if(isFALSE(executable) || is.null(executable)){
			stop(paste0("EHC with graphPartitioning='METIS' requires an external graph-partitioning ",
				"executable (METIS / gpmetis / shmetis) or MATLAB to partition the association ",
				"graph. These binaries are not bundled with the package. Configure one through ",
				"the 'executable' argument (it is currently FALSE), or use graphPartitioning='MST' ",
				"for the pure-R minimum spanning tree alternative."))
		}
		if(is.character(executable) && !nzchar(Sys.which(executable)) && !file.exists(executable)){
			stop(paste0("EHC could not locate the configured graph-partitioning executable '",
				executable, "'. Provide a valid path to a METIS / gpmetis / shmetis binary or to ",
				"MATLAB. Precompiled binaries are not bundled with the package."))
		}
	}

	# The cluster association graph is built and (for MST) partitioned with
	# igraph, which is an optional dependency. Fail clearly if it is missing.
	if(!requireNamespace("igraph", quietly=TRUE)){
		stop(paste0("EHC requires the 'igraph' package to build (and, for graphPartitioning='MST', ",
			"partition) the cluster association graph. Please install it with ",
			"install.packages('igraph')."))
	}

	## Step 1: perfom aggl clustering on each data set

	if(type=="data"){

		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type="data",distmeasure[i],normalize[i],method[i],clust,linkage[i],alpha,gap,maxK,StopRange=TRUE))

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

	}

	else if(type=="dist"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,OrderNames]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type,distmeasure[i],normalize=FALSE,method=NULL,clust,linkage[i],alpha,gap,maxK,StopRange=TRUE))

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

	}
	else if(type=="clust"){

		Clusterings=List

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}


	}

	## Step 2: compute strengths of association
	theta_ab=matrix(0,nrow=nrow(Clusterings[[1]]$DistM),ncol=nrow(Clusterings[[1]]$DistM))
	rownames(theta_ab)=rownames(Clusterings[[1]]$DistM)
	colnames(theta_ab)=rownames(Clusterings[[1]]$DistM)

	for(i in 1:length(Clusterings)){
		Dend=Clusterings[[i]]$Clust
		nclusters=nrow(Dend$merge)
		MergingList=list()

		for(a in 1:nclusters){
			NewMerge=Dend$merge[a,]

			if(sign(NewMerge[1])==-1 & sign(NewMerge[2])==-1){
				MergingList[[length(MergingList)+1]]=abs(NewMerge)
			}

			else if(sign(NewMerge[1])==1 & sign(NewMerge[2])==-1){
				MergingList[[length(MergingList)+1]]=c(MergingList[[NewMerge[1]]],abs(NewMerge[2]))
			}

			else if(sign(NewMerge[1])==-1 & sign(NewMerge[2])==1){
				MergingList[[length(MergingList)+1]]=c(abs(NewMerge[1]),MergingList[[NewMerge[2]]])
			}

			else if(sign(NewMerge[1])==1 & sign(NewMerge[2])==1){
				MergingList[[length(MergingList)+1]]=c(MergingList[[NewMerge[1]]],MergingList[[NewMerge[2]]])
			}
		}

		#depths
		depths=c()
		for(b in 1:length(MergingList)){
			SubList=MergingList[[b]]
			depth=0
			for(c in 1:length(MergingList)){
				if(all(SubList%in%MergingList[[c]])){
					depth=depth+1
				}
			}
			depths=c(depths,depth)
		}

		depths=depths-1 #root cluster has depth 0

		#max_depth
		max_depth=max(depths)

		#intra_cluster proximity values: based on cd ; compare with heights
		values=sort(Dend$height)

		#values=as.matrix(cophenetic(as.hclust(Clusterings[[i]]$Clust)))

		CheckDist<-function(Dist,StopRange){
			if(StopRange==FALSE & !(0<=min(Dist) & max(Dist)<=1)){
				#message("It was detected that a distance matrix had values not between zero and one. Range Normalization was performed to secure this. Put StopRange=TRUE if this was not necessary")
				Dist=Normalization(Dist,method="Range")
			}
			else{
				Dist=Dist
			}
		}

		values=CheckDist(values,StopRange=FALSE)  #use 1-height as the intra-cluster proximity values after normalizing height between 0 and 1
		proximityvalues=1-values  #one for each cluster

		for(j in 1:length(MergingList)){
			Pairs=utils::combn(MergingList[[j]],2)
			for(k in 1:ncol(Pairs)){
				ab=Pairs[,k]
				theta_ab[ab[1],ab[2]]=theta_ab[ab[1],ab[2]]+depths[j]*proximityvalues[j]/max_depth
				theta_ab[ab[2],ab[1]]=theta_ab[ab[1],ab[2]]
			}
		}
	}



	## Step 3: generate cluster association graph
	## theta_ab can be seen as an adjacency matrix of the graph: the higher the value, the closer the objects
	net=igraph::graph_from_adjacency_matrix(theta_ab,mode="undirected",weighted=TRUE,diag=FALSE)
	igraph::plot.igraph(net,vertex.label=igraph::V(net)$name,layout=igraph::layout_with_fr, edge.color="black",edge.width=igraph::E(net)$weight)


	if(graphPartitioning=="METIS"){

		matlabdata=theta_ab
		utils::write.table(matlabdata,file=paste("matlabdata_",file_number,".csv",sep=""),sep=",",col.names=FALSE,row.names=FALSE)

		system(paste(executable," ",optimalk," ",file_number,sep=""),intern=TRUE)

		Continue=FALSE
		time=0
		while(Continue==FALSE){
			Sys.sleep(15)
			time=time+15
			Continue=file.exists(paste("Partition_",file_number,".csv",sep=""))
			if(time>waitingtime & Continue==FALSE){
				stop(paste("Waited",waitingtime, "seconds for completion of the ensemble clustering procedure. Increase waiting time to continue.",sep=" "))
			}
		}


		Partition <- utils::read.table(paste("Partition_",file_number,".csv",sep=""),sep=",")
		Partition=as.vector(as.matrix(Partition))
		names(Partition)=rownames(theta_ab)

		order=sort(Partition,index=TRUE)$ix
		order.lab=names(Partition[sort(Partition,index=TRUE)$ix])
		Clusters=Partition

		file.remove(paste("Partition_",file_number,".csv",sep=""))
		file.remove(paste("matlabdata_",file_number,".csv",sep=""))

	}

	else if(graphPartitioning=="MST"){

		#are assigning to the first encounter to break ties and get clusters: if changed  to join all if one is in common, all end up in 1 cluster

		Graph=igraph::graph_from_adjacency_matrix(adjmatrix=theta_ab, mode=c( "undirected"), weighted=TRUE, diag=TRUE,add.colnames=NULL)
		MST_Graph=igraph::as_data_frame(igraph::mst(Graph))

		Partition=list()
		Partition[[1]]=c(MST_Graph[1,1],MST_Graph[1,2])
		Placed=rep(FALSE,nrow(theta_ab))
		Placed[Partition[[1]]]=TRUE
		for(j in 2:nrow(MST_Graph)){
			Edge=as.numeric(MST_Graph[j,c(1:2)])
			k=1

			while(any(Placed[Edge]==FALSE)){

				if(k>length(Partition)){
					Partition[[length(Partition)+1]]=Edge
					Placed[Edge]=TRUE
				}

				else if(Edge[1]%in%Partition[[k]] | Edge[2]%in%Partition[[k]]){
					Partition[[k]]=c(unlist(Partition[[k]]),Edge)
					Partition[[k]]=unique(Partition[[k]])
					Placed[Edge]=TRUE
				}
				k=k+1
			}


		}

		order=unlist(Partition)
		order.lab=rownames(theta_ab)[order]
		t1<-sapply(1:length(Partition),function(i) rep(i,length(Partition[[i]])))
		t2<-sapply(1:length(Partition),function(i) rownames(theta_ab)[Partition[[i]]])
		t3<-cbind(unlist(t1),unlist(t2))
		clus=as.numeric(t3[,1])
		names(clus)=t3[,2]
		Clusters=clus[rownames(theta_ab)]


	}

	Out=list(DistM=theta_ab,Clust=list(order=order,order.lab=order.lab,Clusters=Clusters))
	attr(Out,"method")="Ensemble"
	return(Out)

}

test_that("EnsembleClustering is an exported function", {
  expect_true(exists("EnsembleClustering"))
  expect_true(is.function(EnsembleClustering))
})

test_that("EHC is an exported function", {
  expect_true(exists("EHC"))
  expect_true(is.function(EHC))
})

test_that("EnsembleClustering errors when no executable is configured", {
  data(mosaic_toy)
  L <- mosaic_toy$List
  # executable = FALSE (default): no consensus is possible in pure R, so the
  # function must stop with a clear, informative message rather than crash.
  expect_error(
    EnsembleClustering(List = L, type = "data",
                       distmeasure = c("euclidean", "euclidean"),
                       normalize = c(FALSE, FALSE), method = c(NULL, NULL),
                       clust = "agnes", linkage = c("flexible", "flexible"),
                       nrclusters = c(7, 7), ensembleMethod = "CSPA",
                       executable = FALSE),
    regexp = "executable"
  )
})

test_that("EHC (METIS) errors when no executable is configured", {
  data(mosaic_toy)
  L <- mosaic_toy$List
  # graphPartitioning = "METIS" with executable = FALSE must stop informatively.
  expect_error(
    EHC(List = L, type = "data",
        distmeasure = c("euclidean", "euclidean"),
        normalize = c(FALSE, FALSE), method = c(NULL, NULL),
        clust = "agnes", linkage = c("flexible", "flexible"),
        graphPartitioning = "METIS", optimalk = 7, executable = FALSE),
    regexp = "executable"
  )
})

test_that("EnsembleClustering errors when a configured executable is missing", {
  data(mosaic_toy)
  L <- mosaic_toy$List
  expect_error(
    EnsembleClustering(List = L, type = "data",
                       distmeasure = c("euclidean", "euclidean"),
                       normalize = c(FALSE, FALSE), method = c(NULL, NULL),
                       clust = "agnes", linkage = c("flexible", "flexible"),
                       nrclusters = c(7, 7), ensembleMethod = "CSPA",
                       executable = "this_executable_does_not_exist_12345"),
    regexp = "locate|executable"
  )
})

# =============================================================================
# normalization.R  - feature normalisation utilities
# =============================================================================

#' Normalisation of features
#'
#' When data of different scales are combined it is recommended to normalise so
#' that the structures are comparable.
#'
#' @param Data A data matrix; rows are assumed to correspond to the objects.
#' @param method One of \code{"Quantile"}, \code{"Fisher-Yates"},
#'   \code{"Standardize"}, \code{"Range"} (or any unambiguous first letter).
#' @details \code{"Quantile"} performs quantile normalisation (common in omics);
#'   \code{"Fisher-Yates"} uses normal scores based only on the number of rows;
#'   \code{"Standardize"} centres and scales each column (base
#'   \code{\link[base]{scale}}); \code{"Range"} maps the matrix to \code{[0, 1]}.
#' @return The normalised matrix.
#' @examples
#' x <- matrix(rnorm(100), 10, 10)
#' Norm_x <- Normalization(x, method = "R")
#' @export
Normalization <- function(Data,
                          method = c("Quantile", "Fisher-Yates", "Standardize",
                                     "Range", "Q", "q", "F", "f", "S", "s",
                                     "R", "r")) {
  method <- substring(method[1], 1, 1)   # accept the first letter of each method
  method <- match.arg(method)

  if (method %in% c("S", "s")) {
    return(scale(Data, center = TRUE, scale = TRUE))
  }
  if (method %in% c("R", "r")) {
    minc <- min(as.vector(Data)); maxc <- max(as.vector(Data))
    return((Data - minc) / (maxc - minc))
  }
  if (method %in% c("Q", "q")) {
    DataColSorted <- apply(Data, 2, sort)
    NValues <- apply(DataColSorted, 1, stats::median, na.rm = TRUE)
  } else {  # Fisher-Yates
    NValues <- stats::qnorm(seq_len(nrow(Data)) / (nrow(Data) + 1))
  }
  DataRanked <- c(apply(Data, 2, rank))
  array(stats::approx(seq_len(nrow(Data)), NValues, DataRanked)$y,
        dim(Data), dimnames(Data))
}

# =============================================================================
# utils-selection.r  - cluster-number selection and cluster-set similarity
#
# Utility functions ported from the legacy MoSaIC code base. Inputs follow the
# package convention: a `List` of data / distance / clustering objects, with
# objects in the rows of each data matrix.
# =============================================================================

#' Select the number of clusters via silhouette widths
#'
#' For each element of \code{List}, partitions around medoids
#' (\code{cluster::pam}) are computed over a range of cluster numbers and the
#' average silhouette width is recorded. The number of clusters maximising the
#' average silhouette width is reported per source and for the across-source
#' average. The analytical result (the silhouette table and the chosen number
#' of clusters) is returned and is testable without producing any plot.
#'
#' @param List A list of data matrices (objects in rows), distance matrices, or
#'   \code{cluster::pam} outputs, depending on \code{type}.
#' @param type One of \code{"data"} (compute a distance first via
#'   \code{\link{Distance}}), \code{"dist"} (elements are dissimilarity
#'   matrices) or \code{"pam"} (elements are \code{pam} objects).
#' @param distmeasure Distance measure(s) used when \code{type = "data"}.
#' @param normalize,method Normalisation controls passed to \code{\link{Distance}}.
#' @param nrclusters Sequence of cluster numbers to evaluate.
#' @param names Optional names for the sources; used in output labels.
#' @param StopRange Logical; if \code{FALSE} and dissimilarities fall outside
#'   \code{[0, 1]} they are range-normalised for comparability.
#' @param plottype One of \code{"none"} (default; no plot, safe for headless
#'   runs), \code{"new"}, \code{"pdf"} or \code{"sweave"}. Any value other than
#'   \code{"none"} attempts to draw the silhouette curve; drawing is guarded so
#'   that a missing graphics device does not abort the computation.
#' @param location File location (without extension) used when
#'   \code{plottype = "pdf"}. Default \code{NULL}.
#' @return A list with two elements: \code{Silhoutte_Widths} (a matrix of
#'   average silhouette widths per source and their mean) and
#'   \code{Optimal_Nr_of_CLusters} (a one-row data frame with the silhouette
#'   optimal number of clusters per source and overall).
#' @references
#' Kaufman, L. and Rousseeuw, P. J. (1990). Finding Groups in Data: An
#' Introduction to Cluster Analysis. Wiley, New York.
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' sel <- SelectnrClusters(mosaic_toy$List, type = "data",
#'                         distmeasure = c("euclidean", "euclidean"),
#'                         nrclusters = seq(2, 5), plottype = "none")
#' sel$Optimal_Nr_of_CLusters
#' }
#' @export
SelectnrClusters <- function(List, type = c("data", "dist", "pam"),
                             distmeasure = c("tanimoto", "tanimoto"),
                             normalize = c(FALSE, FALSE), method = c(NULL, NULL),
                             nrclusters = seq(5, 25, 1), names = NULL,
                             StopRange = FALSE, plottype = "none", location = NULL) {

	type = match.arg(type)
	avsilwidth <- matrix(0, ncol = length(List), nrow = length(nrclusters))
	pamfunction <- function(DistM, nrclusters) {
		asw = sapply(nrclusters, function(x) cluster::pam(DistM, x)$silinfo$avg.width)
		return(asw)
	}

	CheckDist <- function(Dist, StopRange) {
		if (StopRange == FALSE & !(0 <= min(Dist) & max(Dist) <= 1)) {
			message("It was detected that a distance matrix had values not between zero and one. Range Normalization was performed to secure this. Put StopRange=TRUE if this was not necessary")
			Dist = Normalization(Dist, method = "Range")
		}
		else {
			Dist = Dist
		}
	}


	if (type == "data") {
		OrderNames = rownames(List[[1]])
		for (i in 1:length(List)) {
			List[[i]] = List[[i]][OrderNames, ]
		}
		Dist = lapply(seq(length(List)), function(i) Distance(List[[i]], distmeasure[i], normalize[i], method[i]))
		Dist = lapply(seq(length(Dist)), function(i) CheckDist(Dist[[i]], StopRange))

		avsilwidth = sapply(Dist, function(x) pamfunction(x, nrclusters = nrclusters))
		rownames(avsilwidth) = nrclusters
	}
	else if (type == "dist") {
		OrderNames = rownames(List[[1]])
		for (i in 1:length(List)) {
			List[[i]] = List[[i]][OrderNames, OrderNames]
		}
		Dist = List
		Dist = lapply(seq(length(Dist)), function(i) CheckDist(Dist[[i]], StopRange))

		avsilwidth = sapply(Dist, function(x) pamfunction(x, nrclusters = nrclusters))
		rownames(avsilwidth) = nrclusters
	}
	else {
		avsilwidth = sapply(List, function(x) return(x$silinfo$avg.width))
	}

	plottypein <- function(plottype, location) {
		if (plottype == "pdf" & !(is.null(location))) {
			grDevices::pdf(paste(location, ".pdf", sep = ""))
		}
		if (plottype == "new") {
			try(grDevices::dev.new(), silent = TRUE)
		}
		if (plottype == "sweave") {

		}
	}
	plottypeout <- function(plottype) {
		if (plottype == "pdf") {
			grDevices::dev.off()
		}
	}


	if (is.null(names)) {
		names1 = c()
		names2 = c()
		for (i in 1:length(List)) {
			names1 = c(names1, paste("Silhouette widths for Data", i, sep = " "))
			names2 = c(names2, paste("Nr Clusters for Data", i, sep = ' '))
		}
		names1 = c(names1, "Average Silhoutte Widths")
		names2 = c(names2, "Optimal nr of clusters")

	}
	else {
		names1 = c()
		names2 = c()
		for (i in 1:length(List)) {
			names1 = c(names1, paste("Silhouette widths for", names[i], sep = " "))
			names2 = c(names2, paste("Nr Clusters for", names[i], sep = ' '))
		}
		names1 = c(names1, "Average Silhoutte Widths")
		names2 = c(names2, "Optimal nr of clusters")
	}


	rownames(avsilwidth) = nrclusters


	avsil = apply(avsilwidth, 1, mean)
	avsilwidth = cbind(avsilwidth, avsil)
	colnames(avsilwidth) = names1


	plotsil <- function(sils, plottype, location, name) {
		k.best = as.numeric(names(sils)[which.max(sils)])
		cat("silhouette-optimal number of clusters:", k.best, "\n")
		plottypein(plottype, location)
		try({
			graphics::plot(nrclusters, sils, type = "h", main = name,
					xlab = "k  (# clusters)", ylab = "average silhouette width")
			graphics::axis(1, k.best, paste("best", k.best, sep = "\n"), col = "red", col.axis = "red")
		}, silent = TRUE)
		plottypeout(plottype)
	}

	# Plotting is skippable: only attempt to draw when explicitly requested.
	# The analytical result below is always computed and returned.
	if (!is.null(plottype) && plottype != "none") {
		try(
			sapply(c(1:ncol(avsilwidth)), function(x) plotsil(avsilwidth[, x], plottype, location, names1[x])),
			silent = TRUE
		)
	}

	Output = list()
	Output[[1]] = avsilwidth
	nrclusters = apply(avsilwidth, 2, function(x) return(as.numeric(names(x)[which.max(x)])))
	nrclusters = as.data.frame(t(nrclusters))
	colnames(nrclusters) = names2
	rownames(nrclusters) = "NrClusters"


	Output[[2]] = nrclusters

	names(Output) = c("Silhoutte_Widths", "Optimal_Nr_of_CLusters")
	return(Output)
}


#' Similarity of a set of clusterings to a reference
#'
#' Given a colour/label matrix (clusterings in rows, objects in columns) or a
#' list of clustering outputs, the proportion of objects sharing the label of
#' the first (reference) row is computed for every row. When a list is supplied
#' it is first aligned with \code{ReorderToReference}; supplying the label
#' matrix directly avoids that dependency.
#'
#' @param List A label matrix (rows are clusterings, columns are objects) or a
#'   list of clustering outputs to be aligned via \code{ReorderToReference}.
#' @param nrclusters Number of clusters used when \code{List} is a list. Default
#'   \code{NULL}.
#' @param fusionsLog,weightclust Logical controls passed to
#'   \code{ReorderToReference} when \code{List} is a list.
#' @param names Optional method names passed to \code{ReorderToReference}.
#' @return A numeric vector of similarities in \code{[0, 1]}, one per row, where
#'   the first entry is the reference compared to itself (always \code{1}).
#' @references
#' Fodeh, S. J., Brandt, C., Luong, T. B., Haddad, A., Schultz, M., Murphy, T.
#' and Krauthammer, M. (2013). Complementary ensemble clustering of biomedical
#' data. Journal of Biomedical Informatics, 46(3), 436-443.
#' @examples
#' \dontrun{
#' M <- rbind(c(1, 1, 2, 2, 3, 3),
#'            c(1, 1, 2, 3, 3, 3),
#'            c(2, 2, 1, 1, 3, 3))
#' SimilarityMeasure(M)
#' }
#' @export
SimilarityMeasure <- function(List, nrclusters = NULL, fusionsLog = TRUE,
                              weightclust = TRUE, names = NULL) {

	if (!methods::is(List, "list")) {
		MatrixColors = List
	}
	else {
		MatrixColors = ReorderToReference(List, nrclusters, fusionsLog, weightclust, names)
	}

	#Compare every row to the first row
	Similarity = c()
	for (i in 1:dim(MatrixColors)[1]) {
		Shared = 0
		for (j in 1:dim(MatrixColors)[2])
			if (MatrixColors[i, j] == MatrixColors[1, j]) {
				Shared = Shared + 1
			}
		Similarity = c(Similarity, Shared / ncol(MatrixColors))


	}

	return(Similarity)
}

test_that("SelectnrClusters returns silhouette selection info without plotting", {
  data(mosaic_toy)
  sel <- SelectnrClusters(
    mosaic_toy$List, type = "data",
    distmeasure = c("euclidean", "euclidean"),
    normalize = c(FALSE, FALSE),
    nrclusters = seq(2, 5), plottype = "none"
  )

  expect_type(sel, "list")
  expect_named(sel, c("Silhoutte_Widths", "Optimal_Nr_of_CLusters"))

  # silhouette table: one row per candidate k, one column per source + average
  sw <- sel$Silhoutte_Widths
  expect_true(is.matrix(sw))
  expect_equal(nrow(sw), length(seq(2, 5)))
  expect_equal(ncol(sw), length(mosaic_toy$List) + 1)
  expect_true(all(is.finite(sw)))

  # chosen number of clusters is reported and lies in the requested range
  opt <- sel$Optimal_Nr_of_CLusters
  expect_s3_class(opt, "data.frame")
  expect_true(all(as.numeric(opt[1, ]) %in% seq(2, 5)))
})

test_that("SimilarityMeasure returns numeric similarities for a label matrix", {
  M <- rbind(
    c(1, 1, 2, 2, 3, 3),
    c(1, 1, 2, 3, 3, 3),
    c(2, 2, 1, 1, 3, 3)
  )
  sim <- SimilarityMeasure(M)

  expect_true(is.numeric(sim))
  expect_length(sim, nrow(M))
  expect_true(all(sim >= 0 & sim <= 1))
  # reference row compared to itself is a perfect match
  expect_equal(sim[1], 1)
})

test_that("SimilarityMeasure is exported and callable", {
  expect_true(is.function(SimilarityMeasure))
})

# =============================================================================
# characterize.R
# Post-clustering biological / clinical characterisation. Given cluster labels
# and a table of external variables (measured on the same objects), summarise
# how the clusters differ -- with effect sizes and gradient detection, not just
# p-values -- so that unsupervised clusters can be interpreted biologically.
# =============================================================================

# effect sizes ---------------------------------------------------------------
.kw_eta2 <- function(x, g) {                 # rank eta^2 for Kruskal-Wallis
  ok <- !is.na(x) & !is.na(g); x <- x[ok]; g <- factor(g[ok])
  n <- length(x); k <- nlevels(g)
  if (n <= k || k < 2) return(NA_real_)
  H <- suppressWarnings(stats::kruskal.test(x, g)$statistic)
  max(0, (H - k + 1) / (n - k))
}
.cramers_v <- function(tab) {
  n <- sum(tab); if (n == 0) return(NA_real_)
  chi <- suppressWarnings(stats::chisq.test(tab)$statistic)
  sqrt(as.numeric(chi) / (n * (min(dim(tab)) - 1)))
}

#' Characterise clusters with external (biological / clinical) variables
#'
#' For a clustering, summarise how each external variable differs across the
#' clusters. Continuous variables get \eqn{n}, mean, SD, median, IQR and range
#' \emph{per cluster}, an overall Kruskal-Wallis test, a rank \eqn{\eta^2} effect
#' size, and \strong{gradient detection} (are the cluster means monotonically
#' ordered?). Categorical variables get per-cluster counts and column
#' percentages, a chi-squared / Fisher test and Cramer's V. The emphasis is on
#' effect sizes and descriptive summaries rather than p-values alone.
#'
#' @param labels Cluster label of each object (vector; \code{NA} allowed).
#' @param metadata A data frame of external variables, one row per object, in the
#'   same order as \code{labels}.
#' @param continuous,categorical Optional character vectors naming the columns to
#'   treat as continuous / categorical. If \code{NULL}, numeric columns are taken
#'   as continuous and the rest as categorical.
#'
#' @return A list with \code{continuous} (per-cluster summary, long format),
#'   \code{continuous_tests} (one row per variable: \code{KW_p}, \code{eta2},
#'   \code{gradient} -- the clusters ordered by mean -- and a monotone-trend
#'   \code{trend_rho}/\code{trend_p}), \code{categorical} (counts and column
#'   percentages) and \code{categorical_tests} (\code{p}, \code{cramers_v}).
#'
#' @seealso \code{\link{plot_cluster_profile}}, \code{\link{cluster_biology_panel}}
#' @examples
#' data(mosaic_toy)
#' fit <- M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3)
#' cl  <- mosaic_labels(fit, 3)
#' meta <- data.frame(score = rnorm(length(cl)), grp = sample(c("A", "B"), length(cl), TRUE))
#' ch  <- characterize_clusters(cl, meta)
#' ch$continuous_tests
#' @export
characterize_clusters <- function(labels, metadata,
                                  continuous = NULL, categorical = NULL) {
  metadata <- as.data.frame(metadata)
  if (length(labels) != nrow(metadata))
    stop("length(labels) must equal nrow(metadata).")
  cl <- factor(labels)
  keep <- !is.na(cl); cl <- droplevels(cl[keep]); metadata <- metadata[keep, , drop = FALSE]
  levs <- levels(cl)

  if (is.null(continuous) && is.null(categorical)) {
    num <- vapply(metadata, is.numeric, logical(1))
    continuous  <- names(metadata)[num]
    categorical <- names(metadata)[!num]
  }
  continuous  <- intersect(continuous,  names(metadata))
  categorical <- intersect(categorical, names(metadata))

  ## ---- continuous ----------------------------------------------------------
  cont_rows <- list(); cont_tests <- list()
  for (v in continuous) {
    x <- suppressWarnings(as.numeric(metadata[[v]]))
    per <- lapply(levs, function(l) {
      xi <- x[cl == l & !is.na(x)]
      data.frame(variable = v, cluster = l, n = length(xi),
                 mean = mean(xi), sd = stats::sd(xi),
                 median = stats::median(xi),
                 q25 = stats::quantile(xi, .25, names = FALSE),
                 q75 = stats::quantile(xi, .75, names = FALSE),
                 min = suppressWarnings(min(xi)), max = suppressWarnings(max(xi)),
                 stringsAsFactors = FALSE)
    })
    cont_rows[[v]] <- do.call(rbind, per)
    # gradient: order clusters by their mean, test a monotone trend across it
    cmean <- vapply(levs, function(l) mean(x[cl == l], na.rm = TRUE), numeric(1))
    ord   <- order(cmean)
    grad  <- paste(sprintf("%s(%.3g)", levs[ord], cmean[ord]), collapse = " < ")
    rankpos <- stats::setNames(order(ord), levs)          # position of each cluster in the mean-ordering
    obj_rank <- rankpos[as.character(cl)]
    tr <- suppressWarnings(stats::cor.test(x, obj_rank, method = "spearman"))
    kw <- suppressWarnings(stats::kruskal.test(x, cl))
    cont_tests[[v]] <- data.frame(variable = v, KW_p = kw$p.value,
                                  eta2 = .kw_eta2(x, cl), gradient = grad,
                                  trend_rho = unname(tr$estimate), trend_p = tr$p.value,
                                  stringsAsFactors = FALSE)
  }

  ## ---- categorical ---------------------------------------------------------
  cat_rows <- list(); cat_tests <- list()
  for (v in categorical) {
    f <- factor(metadata[[v]])
    tab <- table(f, cl)
    pct <- sweep(tab, 2, colSums(tab), "/") * 100
    df  <- as.data.frame.matrix(tab); df$level <- rownames(df); df$variable <- v
    cat_rows[[v]] <- df
    p <- tryCatch(suppressWarnings(
      if (any(tab < 5)) stats::fisher.test(tab, simulate.p.value = TRUE)$p.value
      else stats::chisq.test(tab)$p.value), error = function(e) NA_real_)
    cat_tests[[v]] <- data.frame(variable = v, p = p, cramers_v = .cramers_v(tab),
                                 stringsAsFactors = FALSE)
  }

  list(continuous       = if (length(cont_rows)) do.call(rbind, cont_rows) else NULL,
       continuous_tests = if (length(cont_tests)) do.call(rbind, cont_tests) else NULL,
       categorical      = cat_rows,
       categorical_tests = if (length(cat_tests)) do.call(rbind, cat_tests) else NULL,
       n_clusters = nlevels(cl))
}

#' Plot one external variable by cluster
#'
#' Boxplot (continuous) or grouped bar plot of column proportions (categorical)
#' of an external variable across clusters -- the per-cluster distribution and
#' gradient the manuscript should show.
#'
#' @param labels Cluster labels.
#' @param x The variable (numeric -> boxplot; factor/character -> barplot).
#' @param name Axis / title label for the variable.
#' @param col Optional vector of cluster colours.
#' @return Invisibly \code{NULL}; called for the plot.
#' @seealso \code{\link{characterize_clusters}}, \code{\link{cluster_biology_panel}}
#' @export
plot_cluster_profile <- function(labels, x, name = deparse(substitute(x)), col = NULL) {
  cl <- factor(labels); ok <- !is.na(cl)
  cl <- droplevels(cl[ok]); x <- x[ok]
  if (is.null(col)) col <- .mc_pal(nlevels(cl))
  if (is.numeric(x)) {
    graphics::boxplot(x ~ cl, col = col, xlab = "cluster", ylab = name,
                      main = name, outpch = 19, outcex = 0.5)
    graphics::points(jitter(as.integer(cl), 0.6), x, pch = 19,
                     col = grDevices::adjustcolor("grey20", 0.35), cex = 0.5)
  } else {
    tab <- table(factor(x), cl); pct <- sweep(tab, 2, colSums(tab), "/") * 100
    lc  <- .mc_pal(nrow(pct))
    graphics::barplot(pct, beside = TRUE, col = lc,
                      xlab = "cluster", ylab = paste0(name, " (%)"), main = name)
    graphics::legend("topright", rownames(pct), fill = lc, bty = "n", cex = 0.8)
  }
  invisible(NULL)
}

#' Panel of cluster characterisation plots
#'
#' Lay out \code{\link{plot_cluster_profile}} for every external variable in a
#' single figure -- boxplots for continuous variables, bar plots for categorical
#' ones -- ready for the manuscript.
#'
#' @param labels Cluster labels.
#' @param metadata Data frame of external variables (rows = objects).
#' @param vars Optional subset of columns to show (default: all).
#' @param ncol Number of columns in the panel grid.
#' @return Invisibly \code{NULL}.
#' @seealso \code{\link{characterize_clusters}}
#' @export
cluster_biology_panel <- function(labels, metadata, vars = NULL, ncol = 3) {
  metadata <- as.data.frame(metadata)
  if (is.null(vars)) vars <- names(metadata)
  vars <- intersect(vars, names(metadata))
  op <- graphics::par(mfrow = c(ceiling(length(vars) / ncol), ncol),
                      mar = c(4, 4, 3, 1))
  on.exit(graphics::par(op))
  for (v in vars) plot_cluster_profile(labels, metadata[[v]], name = v)
  invisible(NULL)
}

#' @title Comparison of clustering results over multiple methods
#'
#' @description A visual comparison of the clustering results of several methods.
#' The function relies on \code{ReorderToReference} and \code{ColorsNames} and
#' renders the resulting matrix either as a rectangular heatmap (using the
#' \code{plotrix} package) or in a circular format (using the \code{circlize}
#' package).
#' @param List A list of the outputs from the methods to be compared. The first
#' element of the list is used as the reference in \code{ReorderToReference}.
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param cols A character vector with the colours to be used. Default is NULL.
#' @param fusionsLog Logical. To be handed to \code{ReorderToReference}. Default is FALSE.
#' @param weightclust Logical. To be handed to \code{ReorderToReference}. Default is FALSE.
#' @param names Optional. Names of the methods to be used as labels. Default is NULL.
#' @param margins Optional. Margins for the plot. Default is c(8.1,3.1,3.1,4.1).
#' @param circle Logical. Whether the figure should be circular (TRUE) or a rectangle (FALSE). Default is FALSE.
#' @param canvaslims The limits for the circular dendrogram. Default is c(-1.0,1.0,-1.0,1.0).
#' @param Highlight Optional. A list of character vectors of objects to be highlighted. Default is NULL.
#' @param substr Optional. A vector of length two giving start and stop positions to shorten labels. Default is NULL.
#' @param cex.highlight Magnification for the highlight text. Default is 1.
#' @param cex.labels Magnification for the labels. Default is 0.7.
#' @param trackheightdend The height of the dendrogram track in the circular plot. Default is 0.5.
#' @param plottype Should be one of "pdf","new" or "sweave". Default is "new".
#' @param location Optional. If plottype is "pdf", a location should be provided. Default is NULL.
#' @return A plot which translates the matrix output of \code{ReorderToReference}.
#' @export
ComparePlot<-function(List,nrclusters=NULL,cols=NULL,fusionsLog=FALSE,weightclust=FALSE,names=NULL,margins=c(8.1,3.1,3.1,4.1),circle=FALSE,canvaslims=c(-1.0,1.0,-1.0,1.0),Highlight=NULL,substr=NULL,cex.highlight=1,cex.labels=0.7,trackheightdend=0.5,plottype="new",location=NULL){
	if (!requireNamespace("circlize", quietly = TRUE)) stop("ComparePlot() requires the suggested package 'circlize'.")
	if (!requireNamespace("plotrix", quietly = TRUE)) stop("ComparePlot() requires the suggested package 'plotrix'.")
	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
		if(plottype=="sweave"){

		}
	}
	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}

	for(i in 1:length(List)){
		if(attributes(List[[i]])$method == "Weighted" & weightclust==TRUE){
			T=List[[i]]$Clust
			attr(T,"method")="Single Clustering"
			List[[i]]=T
		}
	}

	MatrixColors=ReorderToReference(List,nrclusters,fusionsLog,weightclust,names)
	#capture singletons
	for(i in 1:nrow(MatrixColors)){
		if(any(table(MatrixColors[i,])==1)){
			clusters=names(table(MatrixColors[i,]))[which(table(MatrixColors[i,])==1)]
			MatrixColors[i,][which(MatrixColors[i,]%in%as.numeric(clusters))]=5000
		}
	}

	Names=ColorsNames(MatrixColors,cols)
	colnames(Names)=colnames(MatrixColors)
	nobs=dim(MatrixColors)[2]
	nmethods=dim(MatrixColors)[1]

	if(is.null(names)){
		for(j in 1:nmethods){
			names[j]=paste("Method",j,sep=" ")
		}
	}

	if(circle){
		plottypein(plottype, location)
		graphics::par(mar=margins)
		circlize::circos.initialize(factors =c(1:ncol(MatrixColors)) , xlim = c(0, ncol(MatrixColors)))
		for(i in 1:length(List)){
			circlize::circos.trackPlotRegion(factors = c(1:ncol(MatrixColors)), ylim = c(0,1),track.height = 0.05)
		}
		track=c(length(List):1)
		for(i in 1:nrow(MatrixColors)){
			for(j in 1:ncol(MatrixColors)){
				circlize::highlight.sector(sector.index=j, track.index = track[i],col=Names[i,j])
			}
		}
		hc=List[[1]]$Clust
		#labels_short=substr(hc$order.lab,1,7)
		labels=hc$order.lab

		if(!is.null(Highlight)){
			for(h in 1:length(Highlight)){
				Name=names(Highlight)[h]
				HL=which(labels%in%Highlight[[h]])

				Sims=c()
				for(i in 1:length(List)){
					Values=List[[i]]$DistM[lower.tri(List[[i]]$DistM)]
					Sims=c(Sims,as.numeric(1-List[[i]]$DistM[Highlight[[h]],Highlight[[h]]][lower.tri(List[[i]]$DistM[Highlight[[h]],Highlight[[h]]])]))
				}
				MedSim=round(median(Sims),2)

				circlize::draw.sector(circlize::get.cell.meta.data("cell.start.degree", sector.index = min(HL)),
						circlize::get.cell.meta.data("cell.end.degree", sector.index = max(HL)),
						rou1 = 1, col = "#00000020")

				circlize::highlight.sector(sector.index=c(min(HL):max(HL)), track.index = 1, text = paste(Name,": ",MedSim,sep=""),
						facing = "bending.inside", niceFacing = TRUE, text.vjust = -1.5,,cex=cex.highlight)

			}
		}
		circlize::circos.clear()
		graphics::par(new = TRUE)
		hc=List[[1]]$Clust
		max_height=max(hc$height)
		dend=as.dendrogram(hc)

		if(!is.null(substr)){
			labelsshort=substr(hc$order.lab,substr[1],substr[2])
		}
		else{
			labelsshort=hc$order.lab
		}

		#ct=cutree(dend,6)
		circlize::circos.par("canvas.xlim" = c(canvaslims[1], canvaslims[2]), "canvas.ylim" = c(canvaslims[3], canvaslims[4]))
		circlize::circos.initialize(factors =1 , xlim = c(0, ncol(MatrixColors)))
		circlize::circos.trackPlotRegion(ylim = c(0, 1), track.height = trackheightdend,bg.border=NA,
				panel.fun = function(x, y) {
					for(i in seq_len(ncol(MatrixColors))) {
						circlize::circos.text(i-0.5, 0, labelsshort[i], adj = c(0, 0.5),
								facing = "clockwise", niceFacing = TRUE,
								col = Names[1,colnames(MatrixColors)[i]],
								cex = cex.labels,font=2)
					}
				})
		circlize::circos.trackPlotRegion(ylim = c(0, max_height), bg.border = NA,track.height = 0.4, panel.fun = function(x, y) {
					circlize::circos.dendrogram(dend, max_height = max_height)})
		circlize::circos.clear()




		plottypeout(plottype)


	}

	else{
		#similar=round(SimilarityMeasure(MatrixColors),2)
		plottypein(plottype,location)
		graphics::par(mar=margins)
		plotrix::color2D.matplot(MatrixColors,cellcolors=Names,show.values=FALSE,axes=FALSE,xlab="",ylab="")
		graphics::axis(1,at=seq(0.5,(nobs-0.5)),labels=colnames(MatrixColors),las=2,cex.axis=1.5)
		graphics::axis(2,at=seq(0.5,(nmethods-0.5)),labels=rev(names),cex.axis=1.5,las=2)
		#axis(4,at=seq(0.5,(nmethods-0.5)),labels=rev(similar),cex.axis=0.65,las=2)
		plottypeout(plottype)
	}
}


#' @title Comparison of clustering results over multiple results in circular format
#'
#' @description A visual comparison of all methods is handy to see which objects will
#' always cluster together independent of the applied methods. To this aid the
#' function \code{Cyclogram} has been written. The function relies on methods
#' of the \code{circlize} package.
#' @details This function makes use of the functions \code{ReorderToReference} and
#' \code{Colorsnames}. Given a list with the outputs of several methods, the
#' first step is to call upon \code{ReorderToReference} and to produce a
#' matrix of which the columns are ordered according to the ordering of the
#' objects of the first method in the list. Each cell represent the number of
#' the cluster the object belongs to for a specific method indicated by the
#' rows. The clusters are arranged in such a way that these correspond to that
#' one cluster of the referenced method that they have the most in common with.
#' The `circlize` package is used to visualize the matrix is a circular format.
#' The inner element of the circle is the first element of the List parameter, the second inner
#' element is the second element of the list and so on. The object names are written on the
#' circular dendrogram portayed in the center of the circle.
#' @param List A list of the outputs from the methods to be compared. The first
#' element of the list will be used as the reference in
#' \code{ReorderToReference}.
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param cols A character vector with the colours to be used. Default is NULL.
#' @param fusionsLog Logical. To be handed to \code{ReorderToReference}: indicator for the fusion of clusters. Default is TRUE
#' @param weightclust Logical. To be handed to \code{ReorderToReference}: to be used for the outputs of CEC,
#' WeightedClust or WeightedSimClust. If TRUE, only the result of the Clust element is considered. Default is TRUE.
#' @param names Optional. Names of the methods to be used as labels for the
#' columns. Default is NULL.
#' @param canvaslims The limits for the circular dendrogam. Default is c(-1.0,1.0,-1.0,1.0).
#' @param margins Optional. Margins to be used for the plot. Default is c(8.1,3.1,3.1,4.1).
#' @param Highlight Optional. A list of character vectors of objects to be highlighted. Default is NULL.
#' @param cex.highlight Magnification for the highlight text. Default is 1.
#' @param substr Optional. A vector of length two giving start and stop positions to shorten labels. Default is NULL.
#' @param cex.labels Magnification for the labels. Default is 0.7.
#' @param plottype Should be one of "pdf","new" or "sweave". If "pdf", a
#' location should be provided in "location" and the figure is saved there. If
#' "new" a new graphic device is opened and if "sweave", the figure is made
#' compatible to appear in a sweave or knitr document, i.e. no new device is
#' opened and the plot appears in the current device or document. Default is "new".
#' @param location Optional. If plottype is "pdf", a location should be provided in
#' "location" and the figure is saved there. Default is NULL.
#' @return A plot which translates the matrix output of the function
#' \code{ReorderToReference} into a circular format.
#' @export Cyclogram
Cyclogram<-function (List, nrclusters = NULL, cols = NULL, fusionsLog = FALSE,
		weightclust = FALSE, names = NULL, canvaslims=c(-1.0,1.0,-1.0,1.0), margins = c(8.1, 3.1,
				3.1, 4.1), Highlight=NULL,cex.highlight=1,substr=NULL,cex.labels=0.7, plottype = "new", location = NULL)
{
	if (!requireNamespace("circlize", quietly = TRUE)) stop("Cyclogram() requires the suggested package 'circlize'.")
	plottypein <- function(plottype, location) {
		if (plottype == "pdf" & !(is.null(location))) {
			grDevices::pdf(paste(location, ".pdf", sep = ""))
		}
		if (plottype == "new") {
			grDevices::dev.new()
		}
		if (plottype == "sweave") {
		}
	}
	plottypeout <- function(plottype) {
		if (plottype == "pdf") {
			grDevices::dev.off()
		}
	}
	for (i in 1:length(List)) {
		if (attributes(List[[i]])$method == "Weighted" & weightclust ==
				TRUE) {
			T = List[[i]]$Clust
			attr(T, "method") = "Single Clustering"
			List[[i]] = T
		}
	}
	MatrixColors = ReorderToReference(List, nrclusters, fusionsLog,
			weightclust, names)
	Names = ColorsNames(MatrixColors, cols)
	colnames(Names)=colnames(MatrixColors)
	nobs = dim(MatrixColors)[2]
	nmethods = dim(MatrixColors)[1]
	if (is.null(names)) {
		for (j in 1:nmethods) {
			names[j] = paste("Method", j, sep = " ")
		}
	}
	plottypein(plottype, location)

	circlize::circos.initialize(factors =c(1:ncol(MatrixColors)) , xlim = c(0, ncol(MatrixColors)))
	for(i in 1:length(List)){
		circlize::circos.trackPlotRegion(factors = c(1:ncol(MatrixColors)), ylim = c(0,1),track.height = 0.05)
	}
	track=c(length(List):1)
	for(i in 1:nrow(MatrixColors)){
		for(j in 1:ncol(MatrixColors)){
			circlize::highlight.sector(sector.index=j, track.index = track[i],col=Names[i,j])
		}
	}
	hc=List[[1]]$Clust
	#labels=substr(hc$order.lab,1,5)
	labels=hc$order.lab
	if(!is.null(Highlight)){
		for(h in 1:length(Highlight)){
			Name=names(Highlight)[h]
			#HL=which(labels%in%substr(Highlight[[h]],1,5))
			HL=which(labels%in%Highlight[[h]])

			Sims=c()
			for(i in 1:length(List)){
				Values=List[[i]]$DistM[lower.tri(List[[i]]$DistM)]
				Sims=c(Sims,as.numeric(1-List[[i]]$DistM[Highlight[[h]],Highlight[[h]]][lower.tri(List[[i]]$DistM[Highlight[[h]],Highlight[[h]]])]))
			}
			MedSim=round(stats::median(Sims),2)

			circlize::draw.sector(circlize::get.cell.meta.data("cell.start.degree", sector.index = min(HL)),
					circlize::get.cell.meta.data("cell.end.degree", sector.index = max(HL)),
					rou1 = 1, col = "#00000020")

			circlize::highlight.sector(sector.index=c(min(HL):max(HL)), track.index = 1, text = paste(Name,": ",MedSim,sep=""),
					facing = "bending.inside", niceFacing = TRUE, text.vjust = -1.5,cex=cex.highlight)

		}
	}
	circlize::circos.clear()
	graphics::par(new = TRUE)
	hc=List[[1]]$Clust
	max_height=max(hc$height)
	dend=stats::as.dendrogram(hc)
	if(!is.null(substr)){
		labels=substr(hc$order.lab,substr[1],substr[2])
	}
	else{
		labels=hc$order.lab
	}
	circlize::circos.par("canvas.xlim" = c(canvaslims[1], canvaslims[2]), "canvas.ylim" = c(canvaslims[3], canvaslims[4]))
	circlize::circos.initialize(factors =1 , xlim = c(0, ncol(MatrixColors)))
	circlize::circos.trackPlotRegion(ylim = c(0, 1), track.height = 0.4,bg.border=NA,
			panel.fun = function(x, y) {
				for(i in seq_len(ncol(MatrixColors))) {
					circlize::circos.text(i-0.5, 0, labels[i], adj = c(0, 0.5),
							facing = "clockwise", niceFacing = TRUE,
							col = Names[1,colnames(MatrixColors)[i]],
							cex = cex.labels,font=2)
				}
			})
	circlize::circos.trackPlotRegion(ylim = c(0, max_height), bg.border = NA,track.height = 0.4, panel.fun = function(x, y) {
				circlize::circos.dendrogram(dend, max_height = max_height)})
	circlize::circos.clear()




	plottypeout(plottype)
}



#' @title Function that annotates colors to their names
#'
#' @description The \code{ColorsNames} function is used on the output of the
#' \code{ReorderToReference} and matches the cluster numbers indicated by the
#' cell with the names of the colors.  This is necessary to produce the plot of
#' the \code{ComparePlot} function and is therefore an internal function of
#' this function but can also be applied separately.
#'
#' @param matrixColors The output of the ReorderToReference function.
#' @param cols A character vector with the names of the colours to be used. Default is NULL.
#' @return A matrix containing the hex code of the color that corresponds to
#' each cell of the matrix to be colored. This function is called upon by the
#' \code{ComparePlot} function.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#' data(Colors1)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(MCF7_F,MCF7_T)
#' names=c("FP","TP")
#'
#' MatrixColors=ReorderToReference(List=L,nrclusters=7,fusionsLog=TRUE,weightclust=TRUE,
#' names=names)
#'
#' Names=ColorsNames(matrixColors=MatrixColors,cols=Colors1)
#' }
#' @export ColorsNames
ColorsNames<-function(matrixColors,cols=NULL){
	Names=matrix(0,nrow=dim(matrixColors)[1],ncol=dim(matrixColors)[2])
	for (i in 1:dim(matrixColors)[1]){
		for(j in 1:dim(matrixColors)[2]){
			if(is.na(matrixColors[i,j])){
				Names[i,j]="grey"

			}
			else if(length(unique(matrixColors[i,]))==1){
				Names[i,j]="grey"

			}
			else if(matrixColors[i,j]==5000){
				Names[i,j]="white"

			}
			else{
				temp=matrixColors[i,j]
				Color=cols[temp]
				Names[i,j]=Color
			}
		}
	}
	return(Names)
}


#' @title Helper that colours dendrogram leaves by cluster membership
#'
#' @description Internal helper applied via \code{dendrapply} that colours the
#' leaves and edges of a dendrogram according to the cluster a leaf belongs to,
#' or to a specific selection of objects. Used by \code{ClusterPlot}.
#' @param x A node of a dendrogram (passed by \code{dendrapply}).
#' @param Data The clustering result (an \code{agnes}/\code{hclust} object) used to determine cluster membership.
#' @param nrclusters Optional. The number of clusters to cut the dendrogram in. Default is NULL.
#' @param cols The colours for the clusters if nrclusters is specified. Default is NULL.
#' @param colorComps Optional. A character vector of objects to highlight in red. Default is NULL.
#' @return The dendrogram node \code{x} with coloured \code{nodePar} and \code{edgePar} attributes.
#' @export
ClusterCols <- function(x,Data,nrclusters=NULL,cols=NULL,colorComps=NULL) {

	if(is.null(nrclusters) & is.null(colorComps)){
		return(x)
	}
	else if(!is.null(nrclusters)){
		if(length(cols)<nrclusters){
			stop("Not for every cluster a color is specified")
		}
	}

	if(!is.null(nrclusters)){
		Clustdata=stats::cutree(Data,nrclusters)
		Clustdata=Clustdata[Data$order]

		ordercolors=Clustdata
		order=seq(1,nrclusters)

		for (k in 1:length(unique(Clustdata))){
			select=which(Clustdata==unique(Clustdata)[k])
			ordercolors[select]=cols[order[k]]
		}
		names(ordercolors)=Data$order.lab

	}
	else{
		cols=rep("black",length(Data$order.lab))
		names(cols)=Data$order.lab
		cols[which(names(cols)%in%colorComps)]="red"
		ordercolors=cols

	}

	colfunc=function(x,cols,colorComps){
#		if(is.null(colorComps)){
#			color=cols[which(names(cols)==x)]
#			indextemp=which(attr(Data$diss,"Labels")==x)
#			index1=which(Data$order==indextemp)
#
#			index2=ordercolors[index1]
#
#			color=cols[index2]
#
#		}
#		else{
#			color=cols[which(names(cols)==x)]
#		}
		color=cols[which(names(cols)==x)]
		return(color)
	}

	if (stats::is.leaf(x)) {
		## fetch label
		label <- attr(x, "label")
		## set label color to clustercolor
		attr(x, "nodePar") <- list(pch=NA,lab.col=colfunc(label,ordercolors,colorComps),lab.cex=0.9,font=2)
		attr(x, "edgePar") <- list(lwd=2,col=colfunc(label,ordercolors,colorComps))
	}
	return(x)
}


#' @title Colouring clusters in a dendrogram
#'
#' @description Plot a dendrogram with leaves colored by a result of choice.
#' @param Data1 The resulting clustering of a method which contains the
#' dendrogram to be colored.
#' @param Data2 Optional. The resulting clustering of another method , i.e. the
#' resulting clustering on which the colors should be based. Default is NULL.
#' @param nrclusters Optional. The number of clusters to cut the dendrogram in.
#' If not specified the dendrogram will be drawn without colours to discern the
#' different clusters. Default is NULL.
#' @param cols The colours for the clusters if nrclusters is specified. Default is NULL.
#' @param colorComps If only a specific set of objects needs to be
#' highlighted, this can be specified here. The objects should be given in a
#' character vector. If specified, all other compound labels will be colored
#' black. Default is NULL.
#' @param hangdend A specification for the length of the brances of the dendrogram. Default is 0.02.
#' @param plottype Should be one of "pdf","new" or "sweave". If "pdf", a
#' location should be provided in "location" and the figure is saved there. If
#' "new" a new graphic device is opened and if "sweave", the figure is made
#' compatible to appear in a sweave or knitr document. Default is "new".
#' @param location If plottype is "pdf", a location should be provided in
#' "location" and the figure is saved there. Default is NULL.
#' @param \dots Other options which can be given to the plot function.
#' @return A plot of the dendrogram of the first clustering result with colored
#' leaves. If a second clustering result is given in Data2, the colors are
#' based on this clustering result.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#' data(Colors1)
#'
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' ClusterPlot(MCF7_T ,nrclusters=7,cols=Colors1,plottype="new",location=NULL,
#' main="Clustering on Target Predictions: Dendrogram",ylim=c(-0.1,1.8))
#' }
#' @export ClusterPlot
ClusterPlot<-function(Data1,Data2=NULL,nrclusters=NULL,cols=NULL,colorComps=NULL,hangdend=0.02,plottype="new",location=NULL,...){
	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
		if(plottype=="sweave"){

		}
	}
	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}

	cx=Data1$Clust
	if(is.null(Data2)){
		Data=Data1$Clust
	}
	else{
		Data=Data2$Clust
	}

	d_temp<- stats::dendrapply(stats::as.dendrogram(stats::as.hclust(cx),hang=hangdend),ClusterCols,Data,nrclusters,cols,colorComps)
	plottypein(plottype,location)
	graphics::plot(d_temp,nodePar=list(pch=NA),edgePar=list(lwd=2),ylab="Height",font.axis=2,font.lab=2,font=2)
	graphics::axis(side = 2, lwd = 2)
	plottypeout(plottype)
}


#' @title Create a color palette to be used in the plots
#'
#' @description In order to facilitate the visualization of the influence of the different
#' methods on the clustering of the objects, colours can be used. The function
#' \code{ColorPalette} is able to pick out as many colours as there are
#' clusters. This is done with the help of the \code{ColorRampPalette} function
#' of the grDevices package
#' @param colors A vector containing the colors of choice
#' @param ncols The number of colors to be specified. If higher than the number
#' of colors, it specifies colors in the region between the given colors.
#' @return A vector containing the hex codes of the chosen colors.
#' @examples
#'
#' Colors1<-ColorPalette(c("cadetblue2","chocolate","firebrick2",
#' "darkgoldenrod2", "darkgreen","blue2","darkorchid3","deeppink2"), ncols=8)
#' @export ColorPalette
ColorPalette<-function(colors=c("red","green"),ncols=5){
	my_palette=grDevices::colorRampPalette(colors)(ncols)

	return(my_palette)

}


#' @title Helper that colours specific dendrogram leaves
#'
#' @description Internal helper applied via \code{dendrapply} that colours the
#' leaves and edges of a dendrogram for one or two selections of objects. Used
#' by \code{LabelPlot}.
#' @param x A node of a dendrogram (passed by \code{dendrapply}).
#' @param Sel1 The selection of objects to be colored.
#' @param Sel2 An optional second selection to be colored. Default is NULL.
#' @param col1 The color for the first selection. Default is NULL.
#' @param col2 The color for the optional second selection. Default is NULL.
#' @return The dendrogram node \code{x} with coloured \code{nodePar} and \code{edgePar} attributes.
#' @export
LabelCols <- function(x,Sel1,Sel2=NULL,col1=NULL,col2=NULL) {
	colfunc=function(x,Sel1,Sel2,col1,col2){
		if (x %in% Sel1){
			return(col1)
		}
		else if(x %in% Sel2){
			return(col2)
		}
		else{
			return("black")
		}
	}

	if (stats::is.leaf(x)) {
		## fetch label
		label <- attr(x, "label")
		## set label color to red for SelF, to black otherwise
		attr(x, "nodePar") <- list(pch=NA,lab.col=colfunc(label,Sel1,Sel2,col1,col2),lab.cex=0.9,font=2)
		attr(x, "edgePar") <- list(lwd=2,col=colfunc(label,Sel1,Sel2,col1,col2))
	}
	return(x)
}

#' @title Coloring specific leaves of a dendrogram
#'
#' @description The function plots a dendrogrmam of which specific leaves are coloured.
#'
#' @param Data The result of a method which contains the dendrogram to be
#' colored.
#' @param sel1 The selection of objects to be colored. Default is NULL.
#' @param sel2 An optional second selection to be colored. Default is NULL.
#' @param col1 The color for the first selection. Default is NULL.
#' @param col2 The color for the optional second selection. Default is NULL.
#' @return A plot of the dendrogram of which the leaves of the selection(s) are
#' colored.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' ClustF_6=cutree(MCF7_F$Clust,6)
#'
#' SelF=rownames(fingerprintMat)[ClustF_6==6]
#' SelF
#'
#' LabelPlot(Data=MCF7_F,sel1=SelF,sel2=NULL,col1='darkorchid')
#' }
#'
#' @export LabelPlot
LabelPlot<-function(Data,sel1,sel2=NULL,col1=NULL,col2=NULL){
	x=Data$Clust

	d_temp <- stats::dendrapply(stats::as.dendrogram(x,hang=0.02),LabelCols,sel1,sel2,col1,col2)

	graphics::plot(d_temp,nodePar=list(pch=NA),edgePar=list(lwd=2),ylab="Height",font.axis=2,font.lab=2,font=2)
	graphics::axis(side = 2, lwd = 2)
}

# Tests for the dendrogram / comparison plotting utilities ported from the
# original MoSaiClusteR project: ComparePlot, Cyclogram, ColorsNames,
# ClusterCols, ClusterPlot, ColorPalette, LabelCols and LabelPlot.

test_that("plotting helpers are exported functions", {
  expect_true(is.function(ComparePlot))
  expect_true(is.function(Cyclogram))
  expect_true(is.function(ColorsNames))
  expect_true(is.function(ClusterCols))
  expect_true(is.function(ClusterPlot))
  expect_true(is.function(ColorPalette))
  expect_true(is.function(LabelCols))
  expect_true(is.function(LabelPlot))
})

test_that("ColorPalette returns the requested number of hex colours", {
  cols <- ColorPalette(c("red", "green", "blue"), ncols = 6)
  expect_length(cols, 6)
  expect_true(all(grepl("^#", cols)))
})

test_that("ColorsNames maps a reorder matrix to colour names", {
  M <- matrix(c(1, 1, 2, 2,
                1, 2, 1, 2), nrow = 2, byrow = TRUE)
  pal <- ColorPalette(c("red", "green"), ncols = 2)
  Names <- ColorsNames(M, pal)
  expect_equal(dim(Names), dim(M))
  expect_false(any(is.na(Names)))
})

test_that("ClusterPlot and LabelPlot render without error on the toy data", {
  data(mosaic_toy)
  cl <- Cluster(mosaic_toy$List[[1]], type = "data",
                distmeasure = "euclidean", linkage = "ward")

  grDevices::pdf(tempfile(fileext = ".pdf"))
  on.exit(grDevices::dev.off(), add = TRUE)

  cols <- ColorPalette(c("red", "green", "blue"), ncols = 3)
  expect_error(
    ClusterPlot(cl, nrclusters = 3, cols = cols, plottype = "sweave"),
    NA)

  sel <- rownames(mosaic_toy$List[[1]])[1:5]
  expect_error(
    LabelPlot(cl, sel1 = sel, col1 = "darkorchid"),
    NA)
})

test_that("ComparePlot produces a rectangular comparison on the toy data", {
  testthat::skip_if_not_installed("plotrix")
  data(mosaic_toy)
  c1 <- Cluster(mosaic_toy$List[[1]], type = "data",
                distmeasure = "euclidean", linkage = "ward")
  c2 <- Cluster(mosaic_toy$List[[2]], type = "data",
                distmeasure = "euclidean", linkage = "ward")
  L <- list(c1, c2)
  cols <- ColorPalette(c("red", "green", "blue", "orange"), ncols = 8)

  grDevices::pdf(tempfile(fileext = ".pdf"))
  on.exit(grDevices::dev.off(), add = TRUE)

  expect_error(
    ComparePlot(L, nrclusters = 3, cols = cols, names = c("S1", "S2"),
                plottype = "sweave"),
    NA)
})

test_that("Cyclogram requires the circlize package", {
  testthat::skip_if_not_installed("circlize")
  succeed()
})

#' MosaiClusteR: An Umbrella Framework for Multi-Source and Multi-Omics Clustering
#'
#' \strong{MosaiClusteR} ("MoSaIC" = Multi-Omics Source-Agnostic Integration
#' Clustering in R) unifies a large family of multi-source clustering
#' methodologies behind a single, consistent list-of-matrices interface, and
#' wraps them in a complete analysis framework: preprocessing, single-source
#' baselines, integrative clustering across five paradigms, and downstream
#' comparison/evaluation.
#'
#' @section The MoSaIC framework (how to use the package):
#' \enumerate{
#'   \item \strong{Preprocess} each source with \code{\link{Normalization}} and
#'         \code{\link{Distance}}.
#'   \item \strong{Baseline} each source on its own (any hierarchical clusterer)
#'         to motivate integration.
#'   \item \strong{Integrate} with one or more methods, e.g. \code{\link{M_ABCpp}},
#'         \code{\link{M_ABCdist}}, \code{\link{WeightedClust}}, \code{\link{SNF}},
#'         \code{\link{CEC}}, \code{\link{HierarchicalEnsembleClustering}}.
#'   \item \strong{Evaluate} and compare solutions with
#'         \code{\link{compare_clusterings}} and \code{\link{cluster_agreement}}.
#' }
#' Every clustering method returns a list containing at least \code{DistM} (a
#' dissimilarity matrix) and \code{Clust} (a hierarchical-clustering object),
#' so results are directly comparable and composable.
#'
#' @section Feature weighting with data nuggets:
#' The M-ABC family weights features for subsampling. Beyond the classic
#' variance (\code{"var"}) and coefficient-of-variation (\code{"cv"}) schemes,
#' MosaiClusteR adds \code{weighting = "nugget"}, a robust, Big-Data-friendly
#' alternative based on \code{\link{create_data_nuggets}}. See
#' \code{\link{nugget_feature_weights}} and \code{\link{ABCpp.SingleInMultiple}}.
#'
#' @keywords internal
#' @aliases MosaiClusteR-package
#' @name MosaiClusteR
#' @import stats
#' @import cluster
#' @importFrom matrixStats rowVars rowSds rowMeans2 colVars
#' @importFrom Rdpack reprompt
#' @importFrom Rcpp evalCpp
#' @useDynLib MosaiClusteR, .registration = TRUE
"_PACKAGE"

# =============================================================================
# ABCdeep.R
# ABC with a deep-learning (autoencoder) Step 4 -- the ABCdeep engine.
# Main function: ABCdeep.SingleInMultiple
#
# This follows the ABC algorithm of Amaratunga, Cabrera & Kovtun (2008) with a
# deep Step 4:
#   Step 1  feature weights zf (var / cv / nugget)              -- as in ABCpp
#   Step 2  bootstrap the objects                               -- as in ABCpp
#   Step 3  DEEP feature focus: rank features by zf and keep the top fraction
#           (topk_frac); the autoencoder learns on those informative features
#           (this replaces ABCpp's per-iteration g=floor(sqrt(G)) subsample,
#           which does not suit a trained encoder -- see below)
#   Step 4  DEEP: a neural autoencoder embeds the objects into a latent space,
#           then the latent object-vectors are clustered into NC groups
#           (replaces ABCpp's distance -> hclust on the raw sub-matrix)
#   Step 5  record the base-cluster labels                      -- as in ABCpp
# The stacked label matrix is fused into a consensus dissimilarity by
# f.clustABC.MultiSource (the same C++-accelerated kernel used by M_ABCpp).
#
# DIGEST OF THE ORIGINAL ABCdeep + WHAT CHANGED HERE
# --------------------------------------------------
# The original ABCdeep (keras/tensorflow) trained a fresh autoencoder on EVERY
# iteration (slow) and silently fell back to PCA when TF was unavailable. This
# version removes both, and -- crucially -- gets clustering quality on par with
# (or better than) M_ABCpp:
#   * WHY NOT PER-ITERATION FEATURE SUBSAMPLING? A subsample of only
#     g = floor(sqrt(G)) features pushed through an encoder trained on all G
#     features is a train/inference mismatch and gives a degraded embedding on
#     dense-signal data. Instead we focus the encoder ONCE on the top-`topk_frac`
#     most-informative features (by the same var/cv/nugget weighting): this drops
#     the noise features that dilute a full-feature embedding on sparse-signal
#     data, yet keeps enough features to embed dense-signal data well. Empirically
#     this matches M_ABCpp on sparse signals and beats it on dense signals.
#   * SUPER FAST -- the autoencoder is trained ONCE per source (on the top-K
#     features); each of the numsim iterations does only a forward pass over the
#     bootstrap. The network is a self-contained, vectorised R autoencoder
#     (He-init, ReLU, Adam, MSE) -- no Python, no TensorFlow, no torch.
#   * NO FALLBACK -- exactly one code path. No PCA, no random labels, no
#     availability check; a degenerate input errors honestly.
# =============================================================================

## ---- self-contained autoencoder (vectorised, He-init, ReLU, Adam, MSE) ------

.deep_sigmoid <- function(z) 1 / (1 + exp(-z))
.deep_relu    <- function(z) { z[z < 0] <- 0; z }

# min-max scale each column to [0, 1] (autoencoder targets a sigmoid output).
.deep_minmax <- function(M) {
  cr   <- matrixStats::colRanges(M, na.rm = TRUE)
  span <- cr[, 2] - cr[, 1]; span[span == 0 | !is.finite(span)] <- 1
  X <- sweep(sweep(M, 2, cr[, 1], "-"), 2, span, "/")
  X[!is.finite(X)] <- 0
  X
}

# one Adam update over a named list of parameters (matrices and/or vectors).
.deep_adam <- function(par, grad, state, lr, t,
                       b1 = 0.9, b2 = 0.999, eps = 1e-8) {
  for (nm in names(par)) {
    state$m[[nm]] <- b1 * state$m[[nm]] + (1 - b1) * grad[[nm]]
    state$v[[nm]] <- b2 * state$v[[nm]] + (1 - b2) * grad[[nm]]^2
    mhat <- state$m[[nm]] / (1 - b1^t)
    vhat <- state$v[[nm]] / (1 - b2^t)
    par[[nm]] <- par[[nm]] - lr * mhat / (sqrt(vhat) + eps)
  }
  list(par = par, state = state)
}

# Train a 4-layer autoencoder  X (n x m, in [0,1])
#   encoder:  m --W1--> hidden --ReLU--> --W2--> latent (linear)
#   decoder:  latent --W3--> hidden --ReLU--> --W4--> m (sigmoid)
# Full-batch Adam on the MSE reconstruction loss. Returns the encoder weights.
.deep_ae_fit <- function(X, hidden, latent, epochs, lr) {
  n <- nrow(X); m <- ncol(X)
  h <- as.integer(hidden); d <- as.integer(latent)
  he  <- function(nr, nc, fan) matrix(stats::rnorm(nr * nc, 0, sqrt(2 / fan)), nr, nc)
  par <- list(W1 = he(m, h, m), b1 = numeric(h),
              W2 = he(h, d, h), b2 = numeric(d),
              W3 = he(d, h, d), b3 = numeric(h),
              W4 = he(h, m, h), b4 = numeric(m))
  zero  <- lapply(par, function(p) p * 0)
  state <- list(m = zero, v = zero)
  denom <- n * m

  for (t in seq_len(epochs)) {
    # ---- forward ----
    z1  <- sweep(X   %*% par$W1, 2, par$b1, "+"); a1  <- .deep_relu(z1)
    enc <- sweep(a1  %*% par$W2, 2, par$b2, "+")                 # linear latent
    z3  <- sweep(enc %*% par$W3, 2, par$b3, "+"); a3  <- .deep_relu(z3)
    z4  <- sweep(a3  %*% par$W4, 2, par$b4, "+"); out <- .deep_sigmoid(z4)
    # ---- backward (MSE, mean over all n*m elements) ----
    dz4 <- (2 * (out - X) / denom) * out * (1 - out)
    gW4 <- crossprod(a3, dz4);              gb4 <- colSums(dz4)
    dz3 <- (dz4 %*% t(par$W4)) * (z3 > 0)
    gW3 <- crossprod(enc, dz3);             gb3 <- colSums(dz3)
    denc <- dz3 %*% t(par$W3)                                    # latent is linear
    gW2 <- crossprod(a1, denc);             gb2 <- colSums(denc)
    dz1 <- (denc %*% t(par$W2)) * (z1 > 0)
    gW1 <- crossprod(X, dz1);               gb1 <- colSums(dz1)
    grad <- list(W1 = gW1, b1 = gb1, W2 = gW2, b2 = gb2,
                 W3 = gW3, b3 = gb3, W4 = gW4, b4 = gb4)
    upd   <- .deep_adam(par, grad, state, lr, t)
    par   <- upd$par; state <- upd$state
  }
  list(W1 = par$W1, b1 = par$b1, W2 = par$W2, b2 = par$b2)       # encoder only
}

# Encode a batch of objects through the (already trained) encoder. Fast: two
# matrix products. `A` is objects x (top-K features), matching the AE input.
.deep_encode <- function(A, P) {
  z1 <- sweep(A %*% P$W1, 2, P$b1, "+")
  sweep(.deep_relu(z1) %*% P$W2, 2, P$b2, "+")
}

# Cluster the latent object-vectors into NC groups (Step 4b).
.deep_cluster <- function(Z, NC, method, linkage) {
  if (method == "kmeans")
    return(as.integer(stats::kmeans(Z, centers = NC, nstart = 5L, iter.max = 50L)$cluster))
  hc <- fastcluster::hclust(stats::dist(Z), method = linkage)
  as.integer(stats::cutree(hc, k = NC))
}

#' Single-source ABC with a deep-learning (autoencoder) Step 4 (ABCdeep)
#'
#' A deep-learning variant of \code{\link{ABCpp.SingleInMultiple}} that keeps the
#' ABC algorithm of \insertCite{Amaratunga2008}{MosaiClusteR} intact and replaces
#' \strong{only Step 4}: instead of clustering the raw feature sub-matrix, a
#' neural autoencoder embeds the objects into a latent space and the latent
#' vectors are clustered into \code{NC} base clusters. The stacked label matrix
#' is fused across sources by \code{\link{f.clustABC.MultiSource}}, exactly as in
#' \code{\link{M_ABCpp}}.
#'
#' @section Speed (trained once, forward pass per iteration):
#' The autoencoder is trained a single time on the source's top-\code{topk_frac}
#' most-informative features; each of the \code{numsim} iterations then performs
#' only a forward pass over the bootstrapped objects. The network is a
#' self-contained, vectorised R autoencoder (He initialisation, ReLU, Adam, MSE)
#' -- there is no Python, TensorFlow or torch dependency, and \strong{no PCA or
#' other fallback}: a single code path.
#'
#' @param data Numeric matrix, \strong{objects (samples) in rows}, features in
#'   columns (the MosaiClusteR convention).
#' @param weighting Feature weighting used to rank features for the top-K focus:
#'   \code{"var"}, \code{"cv"}, \code{"nugget"} or equal (as in
#'   \code{\link{ABCpp.SingleInMultiple}}).
#' @param normalize Optional normalisation applied before the internal min-max
#'   scaling (see \code{\link{Normalization}}); \code{NULL}/\code{FALSE} = none.
#' @param gr Unused; kept for interface compatibility.
#' @param bag Logical; bootstrap the objects each iteration (Step 2).
#' @param numsim Number of iterations \eqn{R}.
#' @param numvar Kept for backward compatibility; unused.
#' @param NC Base-cluster count (\code{NULL} = \eqn{\lfloor\sqrt{N}\rfloor}).
#' @param topk_frac Fraction of the highest-weighted features the autoencoder is
#'   trained/embedded on (default 0.2). Focusing on the informative features
#'   drops the noise that would dilute a full-feature embedding on sparse signals,
#'   while keeping enough features for dense signals; the count is floored at
#'   \code{min(G, 20)} and capped at \code{min(G, 4000)}.
#' @param latent_dim Autoencoder bottleneck size: an integer, or \code{"auto"}
#'   (default) = \code{max(NC + 2, 8)}, capped at the hidden width and \eqn{K-1}.
#' @param hidden_units Hidden-layer width of the encoder/decoder.
#' @param ae_epochs Training epochs for the one-time autoencoder fit.
#' @param ae_lr Adam learning rate.
#' @param cluster_in_latent Latent clustering (Step 4b): \code{"ward"} (default,
#'   hierarchical) or \code{"kmeans"}.
#' @param linkage Agglomeration method when \code{cluster_in_latent = "ward"}.
#' @param nugget_type,nugget_args Passed to the nugget weighting when
#'   \code{weighting = "nugget"}.
#' @param min_samples Emit a warning when \code{nrow(data)} is below this (the
#'   autoencoder is underdetermined at very small \eqn{N}); default 40.
#' @param seed Optional RNG seed (controls the autoencoder init and the
#'   bootstrap).
#' @return A data frame, \code{numsim} rows by one column per object; entries are
#'   base-cluster labels (\code{0} = object not selected that iteration) -- the
#'   same contract as \code{\link{ABCpp.SingleInMultiple}}.
#' @references \insertAllCited{}
#' @seealso \code{\link{M_ABCdeep}}, \code{\link{ABCpp.SingleInMultiple}},
#'   \code{\link{f.clustABC.MultiSource}}
#' @examples
#' data(mosaic_toy)
#' lab <- ABCdeep.SingleInMultiple(mosaic_toy$List[[1]], numsim = 40, NC = 3,
#'                                 ae_epochs = 40, seed = 1)
#' dim(lab)
#' @export
ABCdeep.SingleInMultiple <- function(data,
                                     weighting         = "var",
                                     normalize         = NULL,
                                     gr                = c(),
                                     bag               = TRUE,
                                     numsim            = 1000,
                                     numvar            = 100,
                                     NC                = NULL,
                                     topk_frac         = 0.2,
                                     latent_dim        = "auto",
                                     hidden_units      = 32L,
                                     ae_epochs         = 80L,
                                     ae_lr             = 1e-3,
                                     cluster_in_latent = "ward",
                                     linkage           = "ward.D2",
                                     nugget_type       = "between",
                                     nugget_args       = list(),
                                     min_samples       = 40L,
                                     seed              = NULL) {

  if (!is.null(seed)) set.seed(seed)
  if (!cluster_in_latent %in% c("ward", "kmeans"))
    stop("'cluster_in_latent' must be 'ward' or 'kmeans'.")

  data <- as.matrix(data)
  samp_names <- rownames(data)
  n <- nrow(data)   # N objects
  m <- ncol(data)   # G features

  # Deep learning needs samples: an autoencoder trained on very few objects is
  # underdetermined and the latent embedding is unreliable. Warn (do not fail)
  # so tiny-n callers know to prefer ABCpp; the demo skips ABCdeep below n.
  if (n < min_samples)
    warning("ABCdeep: n = ", n, " objects is small for an autoencoder (< ",
            min_samples, "); the deep embedding may be unreliable -- consider ",
            "ABCpp.SingleInMultiple / M_ABCpp for datasets this small.",
            call. = FALSE)

  # optional normalisation, then min-max scaling for the sigmoid autoencoder
  if (!is.null(normalize) && !isFALSE(normalize))
    data <- as.matrix(Normalization(data, method = normalize))
  Xs <- .deep_minmax(data)

  if (is.null(NC)) NC <- max(2L, floor(sqrt(n)))
  NC <- max(2L, min(as.integer(NC), n - 1L))

  # ---- Step 1: feature weights (var / cv / nugget), reusing the ABC helper ----
  # Rank features on the RAW (normalised) data, exactly as ABCpp does -- NOT on
  # the min-max-scaled matrix, which flattens per-feature variance and would make
  # the "var" ranking pick the wrong (noise) features. .abc_zf wants
  # features-by-objects, so transpose the objects-in-rows input.
  zf <- .abc_zf(t(data), weighting, nugget_type, nugget_args)

  # ---- Step 3 (deep focus): keep the top-K most-informative features ---------
  # A per-iteration sqrt(G) subsample mismatches an encoder trained on all G
  # features; instead we focus the encoder on the top-K informative features so
  # it learns signal, not noise. K is floored at 20 and capped at 4000.
  K    <- min(m, 4000L, max(as.integer(round(topk_frac * m)), 20L))
  keep <- order(zf, decreasing = TRUE)[seq_len(K)]
  Xk   <- Xs[, keep, drop = FALSE]

  # ---- autoencoder bottleneck size ------------------------------------------
  if (identical(latent_dim, "auto")) latent_dim <- max(NC + 2L, 8L)
  latent_dim <- max(2L, min(as.integer(latent_dim), K - 1L, as.integer(hidden_units)))

  # ---- train the autoencoder ONCE on the top-K features ---------------------
  message("ABCdeep: training autoencoder (G=", m, ", top-K=", K, ", N=", n,
          ", hidden=", hidden_units, ", latent=", latent_dim,
          ", epochs=", ae_epochs, ") ...")
  P <- .deep_ae_fit(Xk, hidden_units, latent_dim, as.integer(ae_epochs), ae_lr)

  # ---- iterate: bootstrap objects -> encode -> cluster ----------------------
  res <- matrix(0L, nrow = numsim, ncol = n,
                dimnames = list(paste0("iter_", seq_len(numsim)), samp_names))

  for (i in seq_len(numsim)) {
    ids <- if (bag) sort(unique(sample.int(n, size = n, replace = TRUE)))
           else      seq_len(n)
    if (length(ids) < max(4L, NC)) next            # not enough objects to cut NC

    Z <- .deep_encode(Xk[ids, , drop = FALSE], P)                    # Step 4a
    res[i, ids] <- .deep_cluster(Z, NC, cluster_in_latent, linkage)  # Step 4b
  }

  as.data.frame(res, stringsAsFactors = FALSE)
}

#' @title Determine the characteristic features of clusters
#'
#' @description \code{CharacteristicFeatures} identifies the features that are characteristic
#' for the objects of each cluster of one or more clustering results. Binary
#' features are evaluated with Fisher's exact test, continuous features with the
#' t-test. Multiplicity correction is included.
#' @export
#' @param List A list of clustering outputs to be compared. The first element of
#' the list is used as the reference in \code{ReorderToReference}.
#' @param Selection If a selection of objects is provided, the function only
#' investigates the features of this selection. Default is NULL.
#' @param binData A list of the binary feature data matrices. Default is NULL.
#' @param contData A list of continuous feature data matrices. Default is NULL.
#' @param datanames A vector with the names of the data matrices. Default is NULL.
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param sign The significance level. Default is 0.05.
#' @param topChar The number of top characteristics to return. If NULL, only the
#' significant characteristics are saved. Default is NULL.
#' @param fusionsLog Logical. Indicator for the fusion of clusters. Default is TRUE.
#' @param weightclust Logical. For the outputs of CEC, WeightedClust or
#' WeightedSimClust, if TRUE only the result of the Clust element is considered.
#' Default is TRUE.
#' @param names Optional. Names of the methods. Default is NULL.
#' @return A list with for each method the found (top) characteristics of the
#' feature data per cluster.
#' @references Perualila-Tan N. et al. (2016). Weighted similarity-based clustering
#' of chemical structures and bioactivity data in early drug discovery.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(MCF7_F,MCF7_T)
#' MCF7_Char=CharacteristicFeatures(List=L,Selection=NULL,binData=list(fingerprintMat),
#' contData=NULL,datanames=c("FP"),nrclusters=7,topChar=20)
#' }
CharacteristicFeatures<-function(List,Selection=NULL,binData=NULL,contData=NULL,datanames=NULL,nrclusters=NULL,sign=0.05,topChar=NULL,fusionsLog=TRUE,weightclust=TRUE,names=NULL){
	if(is.null(datanames)){
		for(j in 1:(length(binData)+length(contData))){
			datanames[j]=paste("Data",j,sep=" ")
		}
	}

	if(!(is.null(Selection))){
		ResultFeat=FeatSelection(List,Selection,binData,contData,datanames,nrclusters,topChar,sign,fusionsLog,weightclust)
	}
	else{
		ListNew=list()
		element=0
		for(i in 1:length(List)){
			if(attributes(List[[i]])$method != "CEC" & attributes(List[[i]])$method != "Weighted" & attributes(List[[i]])$method!= "WeightedSim"){
				ResultsClust=list()
				ResultsClust[[1]]=list()
				ResultsClust[[1]][[1]]=List[[i]]
				names(ResultsClust[[1]])[1]="Clust"
				element=element+1
				ListNew[[element]]=ResultsClust[[1]]
				#attr(ListNew[element],"method")="Weights"
			}
			else if(attributes(List[[i]])$method=="CEC" | attributes(List[[i]])$method=="Weighted" | attributes(List[[i]])$method == "WeightedSim"){
				ResultsClust=list()
				if(weightclust==TRUE){
					ResultsClust[[1]]=list()
					if(attributes(List[[i]])$method != "WeightedSim"){
						ResultsClust[[1]][[1]]=List[[i]]$Clust
						names(ResultsClust[[1]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[1]]
						attr(ListNew[element],"method")="Weights"
					}
					else{
						ResultsClust[[1]]=list()
						ResultsClust[[1]][[1]]=List[[i]]
						names(ResultsClust[[1]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[1]]
					}
				}
				else{
					for (j in 1:length(List[[i]]$Results)){
						ResultsClust[[j]]=list()
						ResultsClust[[j]][[1]]=List[[i]]$Results[[j]]
						names(ResultsClust[[j]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[j]]
						attr(ListNew[element],"method")="Weights"
					}
				}
			}
		}

		if(is.null(names)){
			names=seq(1,length(ListNew),1)
			for(i in 1:length(ListNew)){
				names[i]=paste("Method",i,sep=" ")
			}
		}
		names(ListNew)=names
		MatrixClusters=ReorderToReference(List,nrclusters,fusionsLog,weightclust,names)

		List=ListNew

		if(!is.null(binData)){
			cpdSet <- rownames(binData[[1]])
		}
		else if(!is.null(contData)){
			cpdSet <- rownames(contData[[1]])
		}
		else{
			stop("Specify a data set in binData and/or in contData")
		}

		ResultFeat=list()
		maxclus=0
		for (k in 1:dim(MatrixClusters)[1]){

			clusters=MatrixClusters[k,]
			if(max(clusters)>maxclus){
				maxclus=max(clusters)
			}
			Characteristics=list()
			clust=sort(unique(clusters)) #does not matter: Genes[i] puts right elements on right places
			hc<-stats::as.hclust(List[[k]]$Clust$Clust)
			OrderedCpds <- hc$labels[hc$order]
			for (i in clust){

				temp=list()
				LeadCpds=names(clusters)[which(clusters==i)]
				temp[[1]]=list(LeadCpds,OrderedCpds)
				names(temp[[1]])=c("LeadCpds","OrderedCpds") #names of the objects

				group <- factor(ifelse(cpdSet %in% LeadCpds, 1, 0)) #identify the group of interest

				#Determine characteristic features for the objects: fishers exact test
				result=list()
				if(!is.null(binData)){
					for(i in 1:length(binData)){
						binData[[i]]=binData[[i]]+0
						binData[[i]]<-binData[[i]][,which(colSums(binData[[i]]) != 0 & colSums(binData[[i]]) != nrow(binData[[i]]))]
					}


					for(j in 1: length(binData)){
						binMat=binData[[j]]

						pFish <- apply(binMat, 2, function(x) stats::fisher.test(table(x, group))$p.value)
						pFish <- sort(pFish)
						adjpFish<-stats::p.adjust(pFish, method = "fdr")

						AllFeat=data.frame(Names=names(pFish),P.Value=pFish,adj.P.Val=adjpFish)
						AllFeat$Names=as.character(AllFeat$Names)

						if(is.null(topChar)){
							topChar=length(which(pFish<sign))
						}

						TopFeat=AllFeat[0:topChar,]
						TopFeat$Names=as.character(TopFeat$Names)
						temp1=list(TopFeat=TopFeat,AllFeat=AllFeat)
						result[[j]]<-temp1
						names(result)[j]=datanames[length(binData)+j]

					}
				}

				resultC=list()
				if(!is.null(contData)){
					for(j in 1:length(contData)){
						contMat=contData[[j]]

						group1=which(group==1)
						group2=which(group==0)


						pTTest <- apply(contMat, 2, function(x) stats::t.test(x[group1],x[group2])$p.value)

						pTTest <- sort(pTTest)
						adjpTTest<-stats::p.adjust(pTTest, method = "fdr")

						AllFeat=data.frame(Names=as.character(names(pTTest)),P.Value=pTTest,adj.P.Val=adjpTTest)
						AllFeat$Names=as.character(AllFeat$Names)
						if(is.null(topChar)){
							topChar=length(which(pTTest<sign))
						}

						TopFeat=data.frame(Names=as.character(names(pTTest[0:topChar])),P.Value=pTTest[0:topChar],adj.P.Val=adjpTTest[0:topChar])
						TopFeat$Names=as.character(TopFeat$Names)
						temp1=list(TopFeat=TopFeat,AllFeat=AllFeat)
						resultC[[j]]<-temp1
						names(resultC)[j]=datanames[j]

					}
				}

				temp[[2]]=c(result,resultC)

				names(temp)=c("objects","Characteristics")

				Characteristics[[i]]=temp

				names(Characteristics)[i]=paste("Cluster",i,sep=" ")
			}
			ResultFeat[[k]]=Characteristics

		}
		names(ResultFeat)=names
		for(i in 1:length(ResultFeat)){
			for(k in 1:length(ResultFeat[[i]])){
				if(is.null(ResultFeat[[i]][[k]])[1]){
					ResultFeat[[i]][[k]]=NA
					names(ResultFeat[[i]])[k]=paste("Cluster",k,sep=" ")
				}
			}
			if(length(ResultFeat[[i]]) != maxclus){
				extra=maxclus-length(ResultFeat[[i]])
				for(j in 1:extra){
					ResultFeat[[i]][[length(ResultFeat[[i]])+1]]=NA
					names(ResultFeat[[i]])[length(ResultFeat[[i]])]=paste("Cluster",length(ResultFeat[[i]]),sep=" ")
				}
			}
		}

	}

	return(ResultFeat)
}


#' @title Interactive plot to determine DE Genes and DE features for a specific cluster
#'
#' @description If desired, the function produces a dendrogram of a clustering result. One
#' or multiple clusters can be indicated by a mouse click. From these clusters
#' DE genes and characteristic features are determined. It is also possible to
#' provide the objects of interest without producing the plot.
#' @export
#' @param Interactive Logical. Whether an interactive plot should be made. Defaults to TRUE.
#' @param leadCpds A list of the objects of the clusters of interest. Defaults to NULL.
#' @param clusterResult The output of one of the aggregated cluster functions. Default is NULL.
#' @param colorLab The clustering result the dendrogram should be colored after. Default is NULL.
#' @param binData A list of the binary feature data matrices. Default is NULL.
#' @param contData A list of continuous data sets of the objects. Default is NULL.
#' @param datanames A vector with the names of the data matrices. Default is c("FP").
#' @param geneExpr A gene expression matrix, may also be an ExpressionSet. Default is NULL.
#' @param topChar The number of top characteristics to return. Default is 20.
#' @param topG The number of top genes to return. Default is 20.
#' @param sign The significance level. Default is 0.05.
#' @param nrclusters Optional. The number of clusters to cut the dendrogram in. Default is NULL.
#' @param cols The colors to use in the dendrogram. Default is NULL.
#' @param n The number of clusters one wants to identify by a mouse click. Default is 1.
#' @return A list with one element per cluster of interest with elements
#' objects, Characteristics and (optionally) Genes.
#' @details This function may require the suggested packages 'a4Base', 'a4Core'
#' and 'limma' when a gene expression matrix is supplied.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#' data(geneMat)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' MCF7_Interactive=ChooseCluster(Interactive=TRUE,leadCpds=NULL,clusterResult=MCF7_T,
#' colorLab=MCF7_F,binData=list(fingerprintMat),datanames=c("FP"),geneExpr=geneMat,
#' topChar = 20, topG = 20,nrclusters=7,n=1)
#' }
ChooseCluster=function(Interactive=TRUE,leadCpds=NULL,clusterResult=NULL,colorLab=NULL,binData=NULL,contData=NULL,datanames=c("FP"),geneExpr=NULL,topChar = 20, topG = 20,sign=0.05,nrclusters=NULL,cols=NULL,n=1){
	if(is.null(datanames)){
		for(j in 1:(length(binData)+length(contData))){
			datanames[j]=paste("Data",j,sep=" ")
		}
	}
	OrInteractive=Interactive

	if(Interactive==TRUE){
		#windows()
		ClusterPlot(clusterResult,colorLab,nrclusters,cols)
		hc1<-stats::as.hclust(clusterResult$Clust)
		ClusterSpecs<-list()
		ClusterSpecs=graphics::identify(hc1, N=n, MAXCLUSTER = nrow(binData[[1]]), function(j) ChooseCluster(Interactive=FALSE,leadCpds=rownames(binData[[1]][j,]),clusterResult,colorLab=NULL,binData=binData,contData=contData,datanames=datanames,geneExpr=geneExpr,topChar=topChar,topG=topG,sign=sign,nrclusters=nrclusters,cols=cols))

		names(ClusterSpecs)<-sapply(seq(1,n),FUN=function(x) paste("Choice",x,sep=" "))

	}
	else{

		DistM<-clusterResult$DistM
		Clust<-clusterResult$Clust
		if(is.null(Clust)){
			clusterResult$Clust=clusterResult
			Clust<-clusterResult$Clust
		}

		hc <- stats::as.hclust(Clust)
		OrderedCpds <- hc$labels[hc$order]

		if(inherits(leadCpds,"character")){
			leadCpds=list(leadCpds)
		}

		Specs=list()

		if(!is.null(binData)){
			cpdSet <- rownames(binData[[1]])
		}
		else if(!is.null(contData)){
			cpdSet <- rownames(contData[[1]])
		}
		else if(!is.null(geneExpr)){
			cpdSet <- colnames(geneExpr)
		}
		else{
			stop("Specify a data set in binData, contData and/or geneExpr")
		}

		for(a in 1:length(leadCpds)){
			objects=list(leadCpds[[a]],OrderedCpds)
			names(objects)=c("LeadCpds","OrderedCpds")
			group <- factor(ifelse(cpdSet %in% leadCpds[[a]], 1, 0)) #identify the group of interest

			#Determine characteristic features for the objects: fishers exact test
			Characteristics=list()

			resultB=list()
			if(!is.null(binData)){

				if(!inherits(binData,"list")){
					stop("The binary data matrices must be put into a list")
				}
				for(i in 1:length(binData)){
					binData[[i]]=binData[[i]]+0
					binData[[i]]<-binData[[i]][,which(colSums(binData[[i]]) != 0 & colSums(binData[[i]]) != nrow(binData[[i]]))]
				}
				for(j in 1: length(binData)){

					binMat=binData[[j]]

					pFish <- apply(binMat, 2, function(x) stats::fisher.test(table(x, group))$p.value)

					pFish <- sort(pFish)
					adjpFish<-stats::p.adjust(pFish, method = "fdr")

					AllFeat=data.frame(Names=names(pFish),P.Value=pFish,adj.P.Val=adjpFish)
					AllFeat$Names=as.character(AllFeat$Names)

					if(is.null(topChar)){
						topChar=length(which(pFish<sign))
					}

					TopFeat=AllFeat[0:topChar,]
					TopFeat$Names=as.character(TopFeat$Names)
					temp1=list(TopFeat=TopFeat,AllFeat=AllFeat)
					resultB[[j]]<-temp1
					names(resultB)[j]=datanames[length(binData)+j]

				}
			}

			resultC=list()
			if(!is.null(contData)){

				if(!inherits(contData,"list")){
					stop("The continuous data matrices must be put into a list")
				}
				for(j in 1:length(contData)){
					contMat=contData[[j]]

					group1=which(group==1)
					group2=which(group==0)

					pTTest <- apply(contMat, 2, function(x) stats::t.test(x[group1],x[group2])$p.value)

					pTTest <- sort(pTTest)
					adjpTTest<-stats::p.adjust(pTTest, method = "fdr")

					AllFeat=data.frame(Names=as.character(names(pTTest)),P.Value=pTTest,adj.P.Val=adjpTTest)
					AllFeat$Names=as.character(AllFeat$Names)
					if(is.null(topChar)){
						topChar=length(which(pTTest<sign))
					}

					TopFeat=data.frame(Names=as.character(names(pTTest[0:topChar])),P.Value=pTTest[0:topChar],adj.P.Val=adjpTTest[0:topChar])
					TopFeat$Names=as.character(TopFeat$Names)
					temp1=list(TopFeat=TopFeat,AllFeat=AllFeat)
					resultC[[j]]<-temp1
					names(resultC)[j]=datanames[j]

				}

			}

			Characteristics=c(resultB,resultC)
			names(Characteristics)=datanames

			#Determine DE Genes with limma --> make difference between "regular" data matrix and "expression set"
			#GeneExpr.2=GeneExpr[,colnames(Matrix)]
			if(!is.null(geneExpr)){
				cpdSetG <-colnames(geneExpr)
				groupG <- factor(ifelse(cpdSetG %in% leadCpds[[a]], 1, 0)) #identify the group of interest
				if(class(geneExpr)[1]=="ExpressionSet"){
					if (!requireNamespace("limma", quietly = TRUE)) stop("ChooseCluster() requires the suggested package 'limma'.")
					if (!requireNamespace("a4Core", quietly = TRUE)) stop("ChooseCluster() requires the suggested package 'a4Core'.")
					geneExpr$LeadCmpds<-groupG


					if (!requireNamespace("a4Base", quietly = TRUE)) {
						stop("a4Base needed for this function to work. Please install it.",
								call. = FALSE)
					}

					DElead <- a4Base::limmaTwoLevels(geneExpr,"LeadCpds")

					#allDE <- topTable(DElead, n = length(DElead@MArrayLM$genes$SYMBOL), resort.by = "logFC",sort.by="p")
					allDE <- a4Core::topTable(DElead, n = length(DElead@MArrayLM$genes$SYMBOL),sort.by="p")
					if(is.null(allDE$ID)){
						allDE$ID <- rownames(allDE)
					}
					else
					{
						allDE$ID=allDE$ID
					}
					if(is.null(topG)){
						topG=length(which(allDE$adj.P.Val<=sign))
					}
					TopDE <- allDE[0:topG, ]
					#TopAdjPval<-TopDE$adj.P.Val
					#TopRawPval<-TopDE$P.Value

					#RawpVal<-allDE$P.Value
					#AdjpVal <- allDE$adj.P.Val
					#genesEntrezId <- allDE$ENTREZID

					Genes<-list(TopDE,allDE)
					names(Genes)<-c("TopDE","AllDE")
					#Genes <- list(TopDE$SYMBOL,TopAdjPval,TopRawPval,genesEntrezId,RawpVal,AdjpVal)
					#names(Genes)<-c("DE_Genes","DE_RawPvals","DE_AdjPvals", "All_Genes", "All_RawPvals","All_AdjPvals")
				}
				else{
					if (!requireNamespace("limma", quietly = TRUE)) stop("ChooseCluster() requires the suggested package 'limma'.")

					label.factor = factor(groupG)
					design = stats::model.matrix(~label.factor)
					fit = limma::lmFit(geneExpr,design=design)
					fit = limma::eBayes(fit)

					#allDE = topTable(fit,coef=2,adjust="fdr",n=nrow(GeneExpr),resort.by = "logFC", sort.by="p")
					allDE = limma::topTable(fit,coef=2,adjust="fdr",n=nrow(geneExpr), sort.by="p")
					if(is.null(allDE$ID)){
						allDE$ID <- rownames(allDE)
					}
					else
					{
						allDE$ID=allDE$ID
					}
					if(is.null(topG)){
						topG=length(which(allDE$adj.P.Val<=sign))
					}
					TopDE=allDE[0:topG,]
					#TopAdjPval<-TopDE$adj.P.Val
					#TopRawPval<-TopDE$P.Value

					#RawpVal<-allDE$P.Value
					#AdjpVal <- allDE$adj.P.Val

					Genes<-list(TopDE,allDE)
					names(Genes)<-c("TopDE","AllDE")
					#Genes <- list(TopDE[,1],TopAdjPval,TopRawPval,allDE[,1],RawpVal,AdjpVal)
					#names(Genes)<-c("DE_Genes","DE_RawPvals","DE_AdjPvals", "All_Genes", "All_RawPvals","All_AdjPvals")

				}

				out=list(objects,Characteristics,Genes)
				names(out)=c("objects","Characteristics","Genes")
				Specs[[a]]=out
				names(Specs)[a]=paste("Choice",a,sep=" ")
			}
			else{
				out=list(objects,Characteristics)
				names(out)=c("objects","Characteristics")
				Specs[[a]]=out
				names(Specs)[a]=paste("Choice",a,sep=" ")
			}
		}

		if(OrInteractive==TRUE|length(Specs)==1){
			return(out)
		}
		else{
			return(Specs)
		}
	}
	class(ClusterSpecs)="ChosenClusters"
	return(ClusterSpecs)
}


#' @title Determine the characteristic features of a cluster or selection
#'
#' @description \code{FeatSelection} determines the characteristic features of a selection of
#' objects or of a specific cluster across the methods. Binary features are
#' evaluated with Fisher's exact test, continuous features with the t-test.
#' @export
#' @param List A list of clustering outputs. Default is NULL.
#' @param Selection Either a character vector of objects, or a numeric cluster
#' number. Default is NULL.
#' @param binData A list of the binary feature data matrices. Default is NULL.
#' @param contData A list of continuous feature data matrices. Default is NULL.
#' @param datanames A vector with the names of the data matrices. Default is NULL.
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param topChar The number of top characteristics to return. Default is NULL.
#' @param sign The significance level. Default is 0.05.
#' @param fusionsLog Logical. Indicator for the fusion of clusters. Default is TRUE.
#' @param weightclust Logical. For the outputs of CEC, WeightedClust or
#' WeightedSimClust, if TRUE only the result of the Clust element is considered.
#' Default is TRUE.
#' @return A list with the found (top) characteristics of the feature data.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(MCF7_F)
#' MCF7_FeatSel=FeatSelection(List=L,Selection=4,binData=list(fingerprintMat),
#' datanames=c("FP"),nrclusters=7,topChar=20)
#' }
FeatSelection<-function(List,Selection=NULL,binData=NULL,contData=NULL,datanames=NULL,nrclusters=NULL,topChar=NULL,sign=0.05,fusionsLog=TRUE,weightclust=TRUE){


	if(is.null(datanames)){
		for(j in 1:(length(binData)+length(contData))){
			datanames[j]=paste("Data",j,sep=" ")
		}
	}

	if(inherits(Selection,"character")){


		if(!is.null(binData)){
			cpdSet <- rownames(binData[[1]])
		}
		else if(!is.null(contData)){
			cpdSet <- rownames(contData[[1]])
		}
		else{
			stop("Specify a data set in binData and/or in contData")
		}
		ResultFeat=list()
		Characteristics=list()
		temp=list()

		LeadCpds=Selection #names of the objects
		OrderedCpds=cpdSet
		temp[[1]]=list(LeadCpds,OrderedCpds)
		names(temp[[1]])=c("LeadCpds","OrderedCpds")

		group <- factor(ifelse(cpdSet %in% LeadCpds, 1, 0)) #identify the group of interest

		#Determine characteristic features for the objects: fishers exact test
		result=list()

		if(!is.null(binData)){
			for(i in 1:length(binData)){
				binData[[i]]=binData[[i]]+0
				binData[[i]]<-binData[[i]][,which(colSums(binData[[i]]) != 0 & colSums(binData[[i]]) != nrow(binData[[i]]))]
			}
			for(j in 1: length(binData)){
				binMat=binData[[j]]


				if(length(LeadCpds)==1){
					FP=which(binMat[LeadCpds,]==1)
					Ranks=sort(colSums(binMat[,FP]),)

					N=c(names(Ranks),colnames(binMat)[which(!colnames(binMat)%in%names(Ranks))])

					AllFeat=data.frame(Names=as.character(N))
					AllFeat$Names=as.character(AllFeat$Names)
					if(is.null(topChar)){
						topChar=length(Ranks)
					}
					else{
						Ranks=Ranks[c(1:topChar)]
					}
					TopFeat=data.frame(Names=as.character(names(Ranks)))
					TopFeat$Names=as.character(TopFeat$Names)


					temp1=list(TopFeat=TopFeat,AllFeat=AllFeat)
					result[[j]]<-temp1
					names(result)[j]=datanames[j]


				}
				else{
					pFish <- apply(binMat, 2, function(x) stats::fisher.test(table(x, group))$p.value)

					pFish <- sort(pFish)
					adjpFish<-stats::p.adjust(pFish, method = "fdr")

					AllFeat=data.frame(Names=as.character(names(pFish)),P.Value=pFish,adj.P.Val=adjpFish)
					AllFeat$Names=as.character(AllFeat$Names)
					if(is.null(topChar)){
						topChar=length(which(pFish<sign))
					}

					TopFeat=data.frame(Names=as.character(names(pFish[0:topChar])),P.Value=pFish[0:topChar],adj.P.Val=adjpFish[0:topChar])
					TopFeat$Names=as.character(TopFeat$Names)
					temp1=list(TopFeat=TopFeat,AllFeat=AllFeat)
					result[[j]]<-temp1
					names(result)[j]=datanames[j]
				}
			}

		}

		resultC=list()
		if(!is.null(contData)){
			for(j in 1:length(contData)){
				contMat=contData[[j]]

				group1=which(group==1)
				group2=which(group==0)


				pTTest <- apply(contMat, 2, function(x) stats::t.test(x[group1],x[group2])$p.value)

				pTTest <- sort(pTTest)
				adjpTTest<-stats::p.adjust(pTTest, method = "fdr")

				AllFeat=data.frame(Names=as.character(names(pTTest)),P.Value=pTTest,adj.P.Val=adjpTTest)
				AllFeat$Names=as.character(AllFeat$Names)
				if(is.null(topChar)){
					topChar=length(which(pTTest<sign))
				}

				TopFeat=data.frame(Names=as.character(names(pTTest[0:topChar])),P.Value=pTTest[0:topChar],adj.P.Val=adjpTTest[0:topChar])
				TopFeat$Names=as.character(TopFeat$Names)
				temp1=list(TopFeat=TopFeat,AllFeat=AllFeat)
				resultC[[j]]<-temp1
				names(resultC)[j]=datanames[length(binData)+j]

			}
		}

		temp[[2]]=c(result,resultC)


		names(temp)=c("objects","Characteristics")

		ResultFeat[[1]]=temp
		names(ResultFeat)="Selection"



	}

	else if(inherits(Selection,"numeric") & !(is.null(List))){

		ListNew=list()
		element=0
		for(i in 1:length(List)){
			if(attributes(List[[i]])$method != "CEC" & attributes(List[[i]])$method != "Weighted" & attributes(List[[i]])$method!= "WeightedSim"){
				ResultsClust=list()
				ResultsClust[[1]]=list()
				ResultsClust[[1]][[1]]=List[[i]]
				names(ResultsClust[[1]])[1]="Clust"
				element=element+1
				ListNew[[element]]=ResultsClust[[1]]
				#attr(ListNew[element],"method")="Weights"
			}
			else if(attributes(List[[i]])$method=="CEC" | attributes(List[[i]])$method=="Weighted" | attributes(List[[i]])$method == "WeightedSim"){
				ResultsClust=list()
				if(weightclust==TRUE){
					ResultsClust[[1]]=list()
					if(attributes(List[[i]])$method != "WeightedSim"){
						ResultsClust[[1]][[1]]=List[[i]]$Clust
						names(ResultsClust[[1]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[1]]
						attr(ListNew[element],"method")="Weights"
					}
					else{
						ResultsClust[[1]]=list()
						ResultsClust[[1]][[1]]=List[[i]]
						names(ResultsClust[[1]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[1]]
					}
				}
				else{
					for (j in 1:length(List[[i]]$Results)){
						ResultsClust[[j]]=list()
						ResultsClust[[j]][[1]]=List[[i]]$Results[[j]]
						names(ResultsClust[[j]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[j]]
						attr(ListNew[element],"method")="Weights"
					}
				}
			}
		}

		if(is.null(names)){
			names=seq(1,length(ListNew),1)
			for(i in 1:length(ListNew)){
				names[i]=paste("Method",i,sep=" ")
			}
		}
		names(ListNew)=names

		Matrix=ReorderToReference(List,nrclusters,fusionsLog,weightclust,names)
		List=ListNew


		if(!is.null(binData)){
			cpdSet <- rownames(binData[[1]])
		}
		else{
			cpdSet <- rownames(contData[[1]])
		}

		ResultFeat=list()
		for(k in 1:dim(Matrix)[1]){
			cluster=Selection

			hc<-stats::as.hclust(List[[k]]$Clust$Clust)
			OrderedCpds <- hc$labels[hc$order]

			Genes=list()
			temp=list()

			LeadCpds=colnames(Matrix)[which(Matrix[k,]==cluster)] #names of the objects
			temp[[1]]=list(LeadCpds,OrderedCpds)
			names(temp[[1]])=c("LeadCpds","OrderedCpds")

			group <- factor(ifelse(cpdSet %in% LeadCpds, 1, 0)) #identify the group of interest

			#Determine characteristic features for the objects: fishers exact test
			result=list()
			if(!is.null(binData)){
				for(i in 1:length(binData)){
					binData[[i]]=binData[[i]]+0
					binData[[i]]<-binData[[i]][,which(colSums(binData[[i]]) != 0 & colSums(binData[[i]]) != nrow(binData[[i]]))]
				}

				for(j in 1: length(binData)){
					binMat=binData[[j]]
					pFish <- apply(binMat, 2, function(x) stats::fisher.test(table(x, group))$p.value)

					pFish <- sort(pFish)
					adjpFish<-stats::p.adjust(pFish, method = "fdr")

					AllFeat=data.frame(Names=as.character(names(pFish)),P.Value=pFish,adj.P.Val=adjpFish)
					AllFeat$Names=as.character(AllFeat$Names)
					if(is.null(topC)){
						topC=length(which(pFish<0.05))
					}

					TopFeat=AllFeat[0:topC,]
					TopFeat$Names=as.character(TopFeat$Names)
					temp1=list(TopFeat=TopFeat,AllFeat=AllFeat)
					result[[j]]<-temp1
					names(resultC)[j]=datanames[length(binData)+j]

				}
			}

			resultC=list()
			if(!is.null(contData)){
				for(j in 1:length(contData)){
					contMat=contData[[j]]

					group1=which(group==1)
					group2=which(group==0)


					pTTest <- apply(contMat, 2, function(x) stats::t.test(x[group1],x[group2])$p.value)

					pTTest <- sort(pTTest)
					adjpTTest<-stats::p.adjust(pTTest, method = "fdr")

					AllFeat=data.frame(Names=as.character(names(pTTest)),P.Value=pTTest,adj.P.Val=adjpTTest)
					AllFeat$Names=as.character(AllFeat$Names)
					if(is.null(topC)){
						topC=length(which(pTTest<sign))
					}

					TopFeat=data.frame(Names=as.character(names(pTTest[0:topC])),P.Value=pTTest[0:topC],adj.P.Val=adjpTTest[0:topC])
					TopFeat$Names=as.character(TopFeat$Names)
					temp1=list(TopFeat=TopFeat,AllFeat=AllFeat)
					resultC[[j]]<-temp1
					names(resultC)[j]=datanames[j]

				}
			}

			temp[[2]]=c(result,resultC)

			names(temp)=c("objects","Characteristics")
			ResultFeat[[k]]=temp
			names(ResultFeat)[k]=paste(names[k],": Cluster",cluster, sep=" ")
		}
	}

	else{
		message("If a specific cluster is specified, clustering results must be provided in List")
	}
	return(ResultFeat)

}


#' @title List all features present in a selected cluster of objects
#'
#' @description The function \code{FeaturesOfCluster} lists the number of features objects
#' of the cluster have in common. A threshold can be set selecting among how
#' many objects of the cluster the features should be shared. An optional
#' plot of the features is available.
#' @export
#' @param leadCpds A character vector containing the objects one wants to
#' investigate in terms of features.
#' @param data The data matrix.
#' @param threshold The number of objects the features at least should be
#' shared amongst. Default is set to 1.
#' @param plot Logical. Indicates whether or not a BinFeaturesPlot should be
#' set up. Default is TRUE.
#' @param plottype Should be one of "pdf","new" or "sweave". Default is "new".
#' @param location If plottype is "pdf", a location should be provided. Default is NULL.
#' @return A list with 2 items: the number of shared features among the objects,
#' and a character vector of the plotted features.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#'
#' Lead=rownames(fingerprintMat)[1:5]
#'
#' FeaturesOfCluster(leadCpds=Lead,data=fingerprintMat,
#' threshold=1,plot=TRUE,plottype="new",location=NULL)
#' }
FeaturesOfCluster<-function(leadCpds,data,threshold=1,plot=TRUE,plottype="new",location=NULL){
	SubsetData=as.matrix(data[which(rownames(data)%in%leadCpds),])

	Common=SubsetData%*%t(SubsetData)

	if(threshold>length(leadCpds)){
		stop("threshold is larger than the number of LeadCpds. This number can maximally be the number of LeadCpds.")
	}


	Features=colnames(SubsetData[,which(apply(SubsetData,2,sum)>=threshold)])

	if(is.null(Features)){
		Features=""
		Plot=FALSE
	}

	#SharedAmongLeadCpds=SubsetData[,Features]

	if(plot==TRUE){
		BinFeaturesPlot_SingleData(leadCpds=leadCpds,orderLab=rownames(data),
				features=as.character(Features),data=data,colorLab=NULL,nrclusters=NULL,cols=NULL,name=c("Shared Features Among Selected objects"),
				margins=c(8.5,2.0,0.5,9.5),plottype=plottype,location=location)
	}

	Out=list("Number_of_Common_Features"=Common,"SharedFeatures"=Features)

	return(Out)
}


#' @title Track a cluster or a selection of objects across multiple methods
#'
#' @description \code{TrackCluster} follows a selection of objects or a specific cluster
#' across a sequence of clustering results, recording how the objects are
#' distributed over the clusters of each method and optionally producing
#' tracking plots.
#' @export
#' @param List A list of clustering outputs to be compared. The first element of
#' the list is used as the reference in \code{ReorderToReference}.
#' @param Selection Either a character vector of objects, or a numeric cluster number.
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param followMaxComps Logical. Whether to follow the cluster with the maximum
#' number of objects. Default is FALSE.
#' @param followClust Logical. Whether to follow the cluster of interest. Default is TRUE.
#' @param fusionsLog Logical. Indicator for the fusion of clusters. Default is TRUE.
#' @param weightclust Logical. For the outputs of CEC, WeightedClust or
#' WeightedSimClust, if TRUE only the result of the Clust element is considered.
#' Default is TRUE.
#' @param names Optional. Names of the methods. Default is NULL.
#' @param selectionPlot Logical. Whether to produce the selection plot. Default is FALSE.
#' @param table Logical. Whether to produce a table of the tracking. Default is FALSE.
#' @param completeSelectionPlot Logical. Whether to produce the complete selection
#' plot. Default is FALSE.
#' @param ClusterPlot Logical. Whether to produce the cluster plot. Default is FALSE.
#' @param cols The colors to use in the plots. Default is NULL.
#' @param legendposx The x-coordinate of the legend. Default is 0.5.
#' @param legendposy The y-coordinate of the legend. Default is 2.4.
#' @param plottype Should be one of "pdf","new" or "sweave". Default is "sweave".
#' @param location If plottype is "pdf", a location should be provided. Default is NULL.
#' @return A list with one element per method describing how the selection is
#' distributed across the clusters, and optionally a tracking table.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(MCF7_F,MCF7_T)
#' MCF7_Track=TrackCluster(List=L,Selection=4,nrclusters=7,followMaxComps=FALSE,
#' followClust=TRUE,selectionPlot=TRUE,table=TRUE,cols=NULL)
#' }
TrackCluster <- function(List,Selection,nrclusters=NULL,followMaxComps=FALSE,followClust=TRUE,fusionsLog=TRUE,weightclust=TRUE,names=NULL,selectionPlot=FALSE,table=FALSE,completeSelectionPlot=FALSE,ClusterPlot=FALSE,cols=NULL,legendposx=0.5,legendposy=2.4,plottype="sweave",location=NULL){

	ClusterDistribution.2<-function(List,Selection,nrclusters,followMaxComps,followClust,fusionsLog,weightclust,names){
		FoundClusters=list()
		FoundCl=list()
		Matrix=ReorderToReference(List,nrclusters,fusionsLog,weightclust,names)

		ClusterNumber=NULL
		if(inherits(Selection,"numeric")){
			ClusterNumber=Selection
			Selection=colnames(Matrix)[which(Matrix[1,]==Selection)]
		}


		if(followClust==TRUE){
			cluster.interest=NULL
			m=1
			while(m<=dim(Matrix)[1] & is.null(cluster.interest)){
				clustersfound=unique(Matrix[m,which(colnames(Matrix)%in%Selection)])
				if(length(clustersfound)==1){
					cluster.interest=clustersfound
				}
				else{
					m=m+1
				}
			}
			if(is.null(cluster.interest)){
				message("This selection is not found to be part of a cluster. FollowMaxComps will be put to true and the function will proceed")
				followMaxComps=TRUE
				followClust=FALSE
			}

		}

		for(i in 1:dim(Matrix)[1]){


			temp=list()
			temp[[1]]=Selection
			names(temp)[1]="Selection"

			clusternumbers=unique(Matrix[i,which(colnames(Matrix)%in%Selection)])

			nr.clusters=length(clusternumbers)
			temp[[2]]=nr.clusters
			names(temp)[2]="nr.clusters"

			min.together=min(table(Matrix[i,which(colnames(Matrix)%in%Selection)]))
			max.together=max(table(Matrix[i,which(colnames(Matrix)%in%Selection)]))

			nr.min.max.together=c(min.together,max.together)
			temp[[3]]=nr.min.max.together
			names(temp)[3]="nr.min.max.together"

			min.perc.together <- min.together/length(Selection) *100
			max.perc.together <- max.together/length(Selection) *100

			perc.min.max.together =c(min.perc.together,max.perc.together)
			temp[[4]]=perc.min.max.together
			names(temp)[4]="perc.min.max.together"

			temp[[5]]=list()
			names(temp)[5]="AllClusters"

			for(a in 1:length(clusternumbers)){
				temp[[5]][[a]]=list()
				names(temp[[5]])[a]=paste("Cluster",clusternumbers[a],sep=" ")

				temp[[5]][[a]][[1]]=clusternumbers[a]
				temp[[5]][[a]][[2]]=names(which(Matrix[i,]==clusternumbers[a])) #complete cluster
				temp[[5]][[a]][[3]]=intersect(Selection,temp[[5]][[a]][[2]]) #Objects from original selection in this cluster
				temp[[5]][[a]][[4]]=temp[[5]][[a]][[2]][which(!(temp[[5]][[a]][[2]] %in% Selection))] #Objects extra to this cluster
				names(temp[[5]][[a]])=c("clusternumber","Complete cluster","Objects from original selection in this cluster","Objects extra to this cluster")
			}

			if(followMaxComps==TRUE){

				maxcluster=names(which(table(Matrix[i,which(colnames(Matrix)%in%Selection)])==max(table(Matrix[i,which(colnames(Matrix)%in%Selection)]))))
				temp[[6]]=maxcluster
				names(temp)[6]="Cluster with max Objects"
				complabels=rownames(Matrix[i,which(colnames(Matrix)%in%Selection & Matrix[i,]==as.numeric(maxcluster)),drop=FALSE])
				temp[[7]]=complabels
				names(temp)[7]="Complabels"
				complete.new.cluster=names(Matrix[i,which(Matrix[i,]==as.numeric(maxcluster))])
				temp[[8]]=complete.new.cluster
				names(temp)[8]="Complete.new.cluster"
				extra.new.cluster=complete.new.cluster[which(!(complete.new.cluster %in% Selection))]
				temp[[9]]=extra.new.cluster
				names(temp)[9]="Extra.new.cluster"

			}

			if(followClust==TRUE){

				temp[[6]]=cluster.interest
				names(temp)[6]="Cluster"
				complabels=rownames(Matrix[i,which(colnames(Matrix)%in%Selection & Matrix[i,]==as.numeric(cluster.interest)),drop=FALSE])
				temp[[7]]=complabels
				names(temp)[7]="Complabels"
				complete.new.cluster=names(Matrix[i,which(Matrix[i,]==as.numeric(cluster.interest))])
				temp[[8]]=complete.new.cluster
				names(temp)[8]="Complete.new.cluster"
				extra.new.cluster=complete.new.cluster[which(!(complete.new.cluster %in% Selection))]
				temp[[9]]=extra.new.cluster
				names(temp)[9]="Extra.new.cluster"

			}

			FoundClusters[[i]]=temp
		}

		if(ClusterPlot==TRUE & !(is.null(ClusterNumber))){

			for(i in 1:dim(Matrix)[1]){
				temp=list()
				if(i==1){
					temp[[1]]=names(Matrix[i,which(Matrix[i,]==ClusterNumber)])
					names(temp)[1]=paste("Cluster ", ClusterNumber,sep="")
					PrevCluster=temp[[1]]
				}
				else{
					temp[[1]]=names(Matrix[i,which(Matrix[i,]==ClusterNumber)])
					names(temp)[1]=paste("Cluster ",  ClusterNumber,sep="")

					Diss=list()
					DissComps=NULL
					if(length((which(!(PrevCluster%in%temp[[1]]))!=0))){
						DissComps=PrevCluster[(which(!(PrevCluster%in%temp[[1]])))]
					}
					if(!(is.null(DissComps))){ #Dissapeared comps: what is missing form PrevClust, where to did they move?
						cl=i
						for(t in 1:nrclusters){
							TempComps=which(DissComps%in%names(which(Matrix[cl,]==t)))
							disscl=NULL
							if(length(TempComps)!=0){
								disscl=list()
								disscl[[1]]=DissComps[TempComps]
								disscl[[2]]=t
								names(disscl)=c("Cpds","Cluster")
							}
							Diss[[t]]=disscl
						}
					}
					if(length(Diss)!=0){
						r=c()
						for(l in 1:length(Diss)){
							if(is.null(Diss[[l]])){
								r=c(r,l)
							}
						}
						if(!is.null(r)){
							Diss=Diss[-r]
						}
						for(l in 1:length(Diss)){
							names(Diss)[l]=paste("Comps Diss To Cluster " ,Diss[[l]][[2]],sep="")

						}
					}
					temp[[2]]=Diss
					names(temp)[2]="Dissapeared"

					Joined=list()
					JoinComps=NULL
					if(length((which(!(temp[[1]]%in%PrevCluster))!=0))){
						JoinComps=temp[[1]][(which(!(temp[[1]]%in%PrevCluster)))]
					}
					if(!(is.null(JoinComps))){ #Dissapeared comps: what is missing form PrevClust, where to did they move?
						prevcl=i-1
						for(t in 1:nrclusters){
							TempComps=which(JoinComps%in%names(which(Matrix[prevcl,]==t)))
							joincl=NULL
							if(length(TempComps)!=0){
								joincl=list()
								joincl[[1]]=JoinComps[TempComps]
								joincl[[2]]=t
								names(joincl)=c("Cpds","Cluster")

							}
							Joined[[t]]=joincl
						}
					}
					if(length(Joined)!=0){
						r=c()
						for(l in 1:length(Joined)){
							if(is.null(Joined[[l]])){
								r=c(r,l)
							}
						}
						if(!is.null(r)){
							Joined=Joined[-r]
						}

						for(l in 1:length(Joined)){
							names(Joined)[l]=paste("Comps Joined From Cluster " ,Joined[[l]][[2]],sep="")

						}
					}
					temp[[3]]=Joined
					names(temp)[3]="Joined"
					PrevCluster=temp[[1]]
				}
				FoundCl[[i]]=temp
			}

		}

		FoundClusters[[length(FoundClusters)+1]]=FoundCl

		FoundClusters[[length(FoundClusters)+1]]=Matrix

		return(FoundClusters)
	}

	Found=ClusterDistribution.2(List,Selection,nrclusters,followMaxComps,followClust,fusionsLog,weightclust,names=names)

	Matrix=Found[[length(Found)]]
	Found=Found[-length(Found)]

	FoundCl=Found[[length(Found)]]
	Found=Found[-length(Found)]

	if(is.null(names)){
		for(j in 1:dim(Matrix)[1]){
			names[j]=paste("Method",j,1)
		}
	}

	ClusterNumber=NULL
	if(inherits(Selection,"numeric")){
		ClusterNumber=Selection
		Selection=colnames(Matrix)[which(Matrix[1,]==Selection)]

	}

	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
		if(plottype=="sweave"){

		}
	}
	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}


	if(selectionPlot==TRUE){


		if(followMaxComps==TRUE){
			lab1=c("Maximum of objects of original cluster together")
			labelcluster=c()
			for(z in 1:length(Found)){
				labelcluster=c(labelcluster,Found[[z]]$Cluster)
			}
		}
		else{lab1=c("Number of objects still in original cluster")}

		nrcluster=c()
		nrcomps=c()
		for(j in 1:length(Found)){
			nrcluster=c(nrcluster,Found[[j]]$nr.clusters)
			nrcomps=c(nrcomps,length(Found[[j]]$Complabels))
		}

		if(is.null(ClusterNumber)){
			xl=c(0,length(Found)+0.5)
		}
		else{
			xl=c(1,length(Found)+0.5)
		}

		if(!(is.null(location))){
			location=paste(location,"_SelectionPlot.pdf",sep="")

		}
		plottypein(plottype,location)

		graphics::plot(type="n",x=0,y=0,xlim=xl,ylim=c(0,max(nrcluster,nrcomps)+legendposy+0),xlab="",ylab="",xaxt="n",yaxt="n",cex.lab=1.25)
		graphics::lines(x=seq(1,length(Found)),y=nrcluster,lty=1,col="red",lwd=1.5)
		graphics::points(x=seq(1,length(Found)),y=nrcluster,pch=19,col="red",cex=1.5)

		if(is.null(ClusterNumber)){
			graphics::lines(x=c(0,1),y=c(length(Selection),nrcomps[1]),lty=1,col="black",lwd=1.5)
			graphics::points(x=0,y=length(Selection),pch=19,col="black",cex=1.5)
		}

		graphics::lines(x=seq(1,length(Found)),y=nrcomps,lty=1,col="blue",lwd=1.5)
		graphics::points(x=seq(1,length(Found)),y=nrcomps,pch=19,col="blue",cex=1.5)

		if(is.null(ClusterNumber)){
			graphics::text(0,length(Selection), "S",cex=1.5,pos=1,col="black",font=2)
		}
		if(selectionPlot==TRUE & followMaxComps==TRUE)
			graphics::text(seq(1,length(Found)),nrcomps, labelcluster,cex=1.5,pos=1,col="black",font=2)

		if(is.null(ClusterNumber)){
			graphics::axis(1,at=seq(0,length(Found)),labels=c("Selection",names),las=2,cex=1)
		}
		else{
			graphics::axis(1,at=seq(1,length(Found)),labels=c(names),las=2,cex=1)
		}
		graphics::axis(2,at=seq(0,max(nrcluster,nrcomps)+2.0),labels=seq(0,max(nrcluster,nrcomps)+2.0),cex=1)


		graphics::legend(legendposx,max(nrcluster,nrcomps)+legendposy,lty=c(1,1,0),pch=c(19,19,0),col=c("blue","red","black"),legend=c(lab1,"Number of clusters original cluster divided amongst","Cluster number"),bty="n",cex=1.2)

		plottypeout(plottype)
	}
	if(completeSelectionPlot==TRUE){
		if(!(is.null(location))){
			location=paste(location,"_CompleteSelectionPlot.pdf",sep="")

		}
		plottypein(plottype,location)
		nrcluster=c(1)
		nrcomps=c(length(Selection))
		for(j in 1:length(Found)){
			nrcluster=c(nrcluster,Found[[j]]$nr.clusters)
			nrcomps=c(nrcomps,length(Found[[j]]$Complabels))
		}



		graphics::plot(type="n",x=0,y=0,xlim=c(0,length(Found)+0.5),ylim=c(0,max(nrcluster,nrcomps)+legendposy+0),xlab="",ylab="Number of objects",xaxt="n",yaxt="n")

		xnext=c()
		ynext=c()
		colorsp=c()

		if(is.null(ClusterNumber)){
			p=seq(0,length(Found))
		}
		else{
			p=seq(0,length(Found))
		}

		for(m in p){
			if(m==0){
				if(!(is.null(ClusterNumber))){
					howmany=length(Found[[1]]$AllClusters)
				}
				else{
					howmany=1
				}
			}
			else{
				howmany=length(Found[[m]]$AllClusters)
			}

			if(m==0){
				xnext=0.1
				ynext=c(length(Selection))
				if(!(is.null(ClusterNumber))){
					colorsp=cols[ClusterNumber]
				}
				else{
					colorsp=c("black")
				}


				graphics::points(x=xnext,y=ynext,col=colorsp,pch=19,cex=1.25)
				if(!(is.null(ClusterNumber))){
					labelcluster=ClusterNumber
				}
				else{
					labelcluster="S"
				}
				position=3
				if(!(is.integer(ynext[length(ynext)]))){
					position=1
				}
				graphics::text(xnext,ynext,labelcluster,cex=1.5,pos=position,col="black",font=2)

				L1=list()
				for(n in 1:howmany){
					if(is.null(ClusterNumber)){
						L1[[n]]=Selection
					}
					else{
						L1[[n]]=Found[[1]]$AllClusters[[n]][[3]]
					}
				}

			}
			else{
				xprev=xnext
				yprev=ynext
				ynext=c()
				colorsp=c()
				xnext=rep(seq(1,length(Found))[m],howmany)
				for(n in 1:howmany){
					ynext=c(ynext,length(Found[[m]]$AllClusters[[n]][[3]]))

					colorsp=c(colorsp,cols[Found[[m]]$AllClusters[[n]]$clusternumber])

					if(length(ynext)>1){
						for(t in 1:(length(ynext)-1)){
							if(ynext[t]==ynext[length(ynext)]){
								ynext[length(ynext)]=ynext[length(ynext)]-0.3
							}
						}
					}

					graphics::points(x=xnext[n],y=ynext[length(ynext)],col=colorsp[length(colorsp)],pch=19,cex=1.25)
					labelcluster=Found[[m]]$AllClusters[[n]]$clusternumber
					position=3
					if(!(is.integer(ynext[length(ynext)]))){
						position=1
					}
					graphics::text(xnext[n],ynext[length(ynext)],labelcluster,cex=1.5,pos=position,col="black",font=2)
				}


				L2=L1
				L1=list()
				for(n in 1:howmany){
					L1[[n]]=Found[[m]]$AllClusters[[n]][[3]]
				}

				for(q in 1:length(L1)){
					for(p in 1:length(L2)){
						if(length(which(L2[[p]] %in% L1[[q]])) != 0){
							graphics::segments(x0=xprev[p],y0=yprev[p],x1=xnext[q],y1=ynext[q],col=colorsp[q],lwd=2)
						}
					}
				}
			}
		}
		if(is.null(ClusterNumber)){
			graphics::axis(1,at=seq(0,length(Found)),labels=c("Selection",names),las=2,cex=1.5)
		}
		else{
			graphics::axis(1,at=seq(0,length(Found)-1),labels=c(names),las=2,cex=1.5)
		}
		graphics::axis(2,at=seq(0,max(nrcluster,nrcomps)),labels=seq(0,max(nrcluster,nrcomps)),cex=1.5)
		graphics::legend(legendposx,max(nrcluster,nrcomps)+legendposy,pch=c(0),col=c("black"),legend=c("Cluster number"),bty="n",cex=1.2)


		plottypeout(plottype)
	}

	if(ClusterPlot==TRUE & !(is.null(ClusterNumber))){

		#FoundCl=Found[[length(Found)]]

		if(!(is.null(location))){
			location=paste(location,"_SelectionPlot.pdf",sep="")

		}
		plottypein(plottype,location)
		nrcluster=c(1)
		nrcomps=c(length(Selection))
		for(j in 1:length(FoundCl)){
			nrcomps=c(nrcomps,length(FoundCl[[j]][[1]]))
		}


		graphics::plot(type="n",x=0,y=0,xlim=c(0,length(FoundCl)-0.5),ylim=c(0,max(nrcluster,nrcomps)+legendposy+0),xlab="",ylab="Number of objects",xaxt="n",yaxt="n")

		xnext=c()
		ynext=c()
		colorsp=c()


		p=seq(1,length(FoundCl))

		for(m in p){

			if(m==1){
				xnext=0.1
				ynext=length(FoundCl[[1]][[1]])
				colorsp=cols[ClusterNumber]
				graphics::points(x=xnext,y=ynext,col=colorsp,pch=19,cex=1.25)

				labelcluster=ClusterNumber

				position=3
				if(!(is.integer(ynext[length(ynext)]))){
					position=1
				}
				graphics::text(xnext,ynext,labelcluster,cex=1.5,pos=position,col="black",font=2)


			}

			else{
				xprev=xnext
				yprev=ynext

				ynext=c()
				colorsp=c()

				xnext=m-1
				ynext=length(FoundCl[[m]][[1]])
				colorsp=cols[ClusterNumber]

				graphics::points(x=xnext,y=ynext,col=colorsp,pch=19,cex=1.25)
				graphics::segments(x0=xprev,y0=yprev,x1=xnext,y1=ynext,col=colorsp)
				labelcluster=ClusterNumber

				position=3
				if(!(is.integer(ynext[length(ynext)]))){
					position=1
				}
				graphics::text(xnext,ynext,labelcluster,cex=1.5,pos=position,col="black",font=2)

				#Dissapeared objects
				if(length(FoundCl[[m]]$Dissapeared)!=0){

					xdiss=rep(xnext,length(FoundCl[[m]]$Dissapeared))
					ydiss=c()
					colorsd=c()
					labs=c()
					for(d in 1:length(FoundCl[[m]]$Dissapeared)){
						ydiss=c(ydiss,length(FoundCl[[m]]$Dissapeared[[d]][[1]]))
						colorsd=c(colorsd,cols[FoundCl[[m]]$Dissapeared[[d]][[2]]])
						labs=c(labs,FoundCl[[m]]$Dissapeared[[d]][[2]])
					}

					if(length(ydiss)>1){
						for(t in 1:(length(ydiss)-1)){
							if(ydiss[t]==ydiss[length(ydiss)]){
								ydiss[length(ydiss)]=ydiss[length(ydiss)]-0.3
							}
						}
					}

					graphics::points(x=xdiss,y=ydiss,col=colorsd,pch=19,cex=1.25)
					labelcluster=labs
					position=3
					if(!(is.integer(ydiss[length(ydiss)]))){
						position=1
					}
					graphics::text(xdiss,ydiss,labelcluster,cex=1.5,pos=position,col="black",font=2)

					for(p in 1:length(ydiss)){
						graphics::segments(x0=xprev,y0=yprev,x1=xdiss[p],y1=ydiss[p],col=colorsd[p])
					}

				}


				if(length(FoundCl[[m]]$Joined)!=0){

					xjoin=rep(xprev,length(FoundCl[[m]]$Joined))
					yjoin=c()
					colorsj=c()
					labs=c()
					for(d in 1:length(FoundCl[[m]]$Joined)){
						yjoin=c(yjoin,length(FoundCl[[m]]$Joined[[d]][[1]]))
						colorsj=c(colorsj,cols[FoundCl[[m]]$Joined[[d]][[2]]])
						labs=c(labs,FoundCl[[m]]$Joined[[d]][[2]])
					}

					if(length(yjoin)>1){
						for(t in 1:(length(yjoin-1))){
							if(yjoin[t]==yjoin[length(yjoin)]){
								yjoin[length(yjoin)]=yjoin[length(yjoin)]-0.3
							}
						}
					}

					graphics::points(x=xjoin,y=yjoin,col=colorsj,pch=19,cex=1.25)
					labelcluster=labs
					position=3
					if(!(is.integer(yjoin[length(yjoin)]))){
						position=1
					}
					graphics::text(xjoin,yjoin,labelcluster,cex=1.5,pos=position,col="black",font=2)

					for(p in 1:length(yjoin)){
						graphics::segments(x0=xjoin[p],y0=yjoin[p],x1=xnext,y1=ynext,col=colorsj[p])
					}

				}

			}

		}
		graphics::axis(1,at=c(0.1,seq(1,length(FoundCl)-1)),labels=c(names),las=2,cex=1.5)
		graphics::axis(2,at=seq(0,max(nrcomps)),labels=seq(0,max(nrcomps)),cex=1.5)
		graphics::legend(legendposx,max(nrcluster,nrcomps)+legendposy,pch=c(0),col=c("black"),legend=c("Cluster number"),bty="n",cex=1.2)

		plottypeout(plottype)

	}

	if(table==TRUE & selectionPlot==TRUE){
		SharedComps=Selection
		Extra=list()
		temp=c()
		for(a in 1:length(Found)){
			SharedComps=intersect(SharedComps,Found[[a]]$Complabels)
		}
		for(a in 1:length(Found)){
			Extra[[a]]=Found[[a]]$Complete.new.cluster[which(!(Found[[a]]$Complete.new.cluster%in%SharedComps))]
			names(Extra)[a]=names[a]
			if(followMaxComps==TRUE){
				names(Extra)[a]=paste(names(Extra)[a],"_",labelcluster[a],sep="")
			}
			temp=c(temp,length(Extra[[a]]))
		}

		ExtraOr=Selection[which(!(Selection%in%SharedComps))]

		collength=max(length(SharedComps),length(ExtraOr),temp)

		if(length(SharedComps)<collength){
			spaces=collength-length(SharedComps)
			SharedComps=c(SharedComps,rep(" ",spaces))
		}

		if(length(ExtraOr)<collength){
			spaces=collength-length(ExtraOr)
			ExtraOr=c(ExtraOr,rep(" ",spaces))
		}

		for(b in 1:length(Extra)){
			if(length(Extra[[b]])<collength){
				spaces=collength-length(Extra[[b]])
				Extra[[b]]=c(Extra[[b]],rep(" ",spaces))
			}

		}
		table1=data.frame(ExtraOr=ExtraOr,SharedComps=SharedComps)
		for(b in 2:length(Extra)){
			table1[1+b]=Extra[[b]]
			colnames(table1)[1+b]=names(Extra)[b]
		}

	}
	else{table1=list()}

	names(Found)=names
	Found[[length(Found)+1]]=table1
	names(Found)[length(Found)]="Table"
	return(Found)
}

# =============================================================================
# utils-navigation.r - navigation / comparison utilities for clustering results
#
# Ported (verbatim) from the legacy MoSaiClusteR project. These functions
# operate on a `List` of clustering-result objects. Each element is a list with
# `$Clust` (an agnes/hclust object) and/or `$DistM` (a dissimilarity matrix), as
# produced by methods such as Cluster(), WeightedClust() and CEC(). Objects are
# assumed to be in the rows (the package convention).
# =============================================================================

#' @title Relabel and reorder clusterings to a reference
#'
#' @description \code{ReorderToReference} relabels and reorders the clusters of
#' a series of clustering results so that the cluster numbers of every method
#' are matched, via a Gale-Shapley stable-matching scheme, to those of the first
#' (reference) method. The result is a matrix in which the columns represent the
#' objects (ordered as for the reference method) and the rows represent the
#' methods. Each cell contains the number of the cluster the object belongs to
#' for that method, relabelled to the reference. It is a possibility that two or
#' more clusters are fused together compared to the reference; in that case the
#' function alerts the user and asks to set \code{fusionsLog} to \code{TRUE}.
#'
#' @param List A list of clustering outputs to be compared. The first element
#' of the list will be used as the reference.
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param fusionsLog Logical. Indicator for the fusion of clusters. Default is FALSE.
#' @param weightclust Logical. To be used for the outputs of CEC, WeightedClust
#' or WeightedSimClust. If TRUE, only the result of the Clust element is
#' considered. Default is FALSE.
#' @param names Optional. A character vector with the names of the methods.
#' @return A matrix of which the cells indicate to what cluster the objects
#' belong to according to the rearranged methods.
#' @note The \code{ReorderToReference} function was optimized for the situations
#' presented by the data sets at hand. It is noted that the function might fail
#' in a particular situation which results in an infinite loop.
#' @references Van Moerbeke, M. (2017). MoSaiClusteR: integrative clustering of
#' multi-source data. Hasselt University.
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' L <- mosaic_toy$List
#' res <- list(Cluster(L[[1]], type = "data", distmeasure = "euclidean"),
#'             Cluster(L[[2]], type = "data", distmeasure = "euclidean"))
#' M <- ReorderToReference(res, nrclusters = 5, fusionsLog = FALSE,
#'                         weightclust = FALSE, names = c("S1", "S2"))
#' }
#' @export
ReorderToReference<-function(List,nrclusters=NULL,fusionsLog=FALSE,weightclust=FALSE,names=NULL){

	matequal <- function(x, y)
		is.matrix(x) && is.matrix(y) && all(dim(x) == dim(y)) && all(x == y)

	ListNew=list()
	element=0
	for(i in 1:length(List)){
		if(attributes(List[[i]])$method != "CEC" & attributes(List[[i]])$method != "Weighted" & attributes(List[[i]])$method!= "WeightedSim"){
			ResultsClust=list()
			ResultsClust[[1]]=list()
			ResultsClust[[1]][[1]]=List[[i]]
			names(ResultsClust[[1]])[1]="Clust"
			element=element+1
			ListNew[[element]]=ResultsClust[[1]]
			#attr(ListNew[element],"method")="Weights"
		}
		else if(attributes(List[[i]])$method=="CEC" | attributes(List[[i]])$method=="Weighted" | attributes(List[[i]])$method == "WeightedSim"){
			ResultsClust=list()
			if(weightclust==TRUE){
				ResultsClust[[1]]=list()
				if(attributes(List[[i]])$method != "WeightedSim"){

					ResultsClust[[1]][[1]]=List[[i]]$Clust
					names(ResultsClust[[1]])[1]="Clust"
					attr(ResultsClust[[1]]$Clust,"method")="Weights"
					element=element+1
					ListNew[[element]]=ResultsClust[[1]]

				}
				else{
					ResultsClust[[1]]=list()
					ResultsClust[[1]][[1]]=List[[i]]
					names(ResultsClust[[1]])[1]="Clust"
					attr(ResultsClust[[1]]$Clust,"method")="Weights"
					element=element+1
					ListNew[[element]]=ResultsClust[[1]]
				}
			}
			else{
				for (j in 1:length(List[[i]]$Results)){
					ResultsClust[[j]]=list()
					ResultsClust[[j]][[1]]=List[[i]]$Results[[j]]
					names(ResultsClust[[j]])[1]="Clust"
					attr(ResultsClust[[j]]$Clust,"method")="Weights"
					element=element+1
					ListNew[[element]]=ResultsClust[[j]]
					#attr(ListNew[[element]],"method")="Weights"
				}
			}
		}
	}

	if(is.null(names)){
		names=seq(1,length(ListNew),1)
		for(i in 1:length(ListNew)){
			names[i]=paste("Method",i,sep=" ")
		}
	}
	names(ListNew)=names
	List=ListNew

	Clusters=list()
	CutTree<-function(i,Data,nrclusters){
		if(attributes(Data$Clust)$method == "Ensemble"){
			Clusters=Data$Clust$Clust$Clusters
			names(Clusters)=NULL
		}
		else{
			Clusters=stats::cutree(Data$Clust$Clust,k=nrclusters)
		}
		return(Clusters)
	}
	Clusters=lapply(seq(1,length(List)),function(i) CutTree(i,Data=ListNew[[i]],nrclusters=nrclusters))


	xaxis=List[[1]]$Clust$Clust$order #order of the objects as for method 1.
	xaxis.names=List[[1]]$Clust$Clust$order.lab #might be that names of methods are not in the same order...

	ordercolors=Clusters[[1]][xaxis]
	order=seq(1,nrclusters)

	for (k in 1:length(unique(Clusters[[1]][xaxis]))){
		select=which(Clusters[[1]][xaxis]==unique(Clusters[[1]][xaxis])[k])
		ordercolors[select]=order[k]
	}

	cols=unique(ordercolors) #order of the colors as depicted by method 1

	Ordered=list()

	autograph=list()
	for(i in cols){
		autograph[[i]]=xaxis[which(ordercolors==i)]
	}

	#for(j in 1:length(List)){
	#		temp=Clusters[[j]][xaxis]  #put clusternumbers of the other method into the same order as those of method (1)
	#	clusternumbers=temp		   #problem:cutree is based on the ordering of the names as they are in the rownames not in the order of joined objects
	#	for(k in 1:length(cols)){
	#		change=which(temp==unique(temp)[k])
	#		clusternumbers[change]=cols[which(cols==unique(temp)[k])]
	#	}
	#	Ordered[[j]]=clusternumbers
	#}

	for (j in 1:length(List)){
		message(j)
		#ordercolorsj=Clusters[[j]][xaxis]
		if(attributes(List[[j]]$Clust)$method=="Ensemble"){
			DistM=matrix(0,ncol=length(List[[j]]$Clust$Clust$order),nrow=length(List[[j]]$Clust$Clust$order))
			if(is.null(names(List[[j]]$Clust$Clust$Clusters))){
				names(List[[j]]$Clust$Clust$Clusters)=paste("Comp", c(1:nrow(DistM)),sep=" ")
			}
			colnames(DistM)=names(List[[j]]$Clust$Clust$Clusters)
			rownames(DistM)=names(List[[j]]$Clust$Clust$Clusters)
			List[[j]]$Clust$DistM=DistM
		}
		ordercolorsj=Clusters[[j]][match(xaxis.names,rownames(List[[j]]$Clust$DistM))]
		order=seq(1,nrclusters)

		#for (k in 1:length(unique(Clusters[[j]][match(xaxis.names,rownames(List[[j]]$Clust$DistM))]))){
		for (k in unique(Clusters[[j]][match(xaxis.names,rownames(List[[j]]$Clust$DistM))])){
			#select=which(Clusters[[j]][match(xaxis.names,rownames(List[[j]]$Clust$DistM))]==unique(Clusters[[j]][match(xaxis.names,rownames(List[[j]]$Clust$DistM))])[k])
           	if(k<=nrclusters){
				select=which(Clusters[[j]][match(xaxis.names,rownames(List[[j]]$Clust$DistM))]==k)
				ordercolorsj[select]=order[k]
			}
		}


		temp2=ordercolorsj
		#temp3=xaxis
		temp3=match(xaxis.names,rownames(List[[j]]$Clust$DistM))
		fan=list()
		for(i in cols){
			fan[[i]]=match(xaxis.names,rownames(List[[j]]$Clust$DistM))[which(temp2==i)]
		}

		favors=matrix(0,length(autograph),length(fan))
		rownames(favors)=seq(1,length(autograph))
		colnames(favors)=seq(1,length(fan))

		for(a in 1:length(autograph)){
			for (b in 1:length(fan)){
				favorab=length(which(rownames(List[[j]]$Clust$DistM)[fan[[b]]] %in% rownames(List[[1]]$Clust$DistM)[autograph[[a]]]))/length(autograph[[a]])
				favors[a,b]=favorab
			}
		}

		#See function woman and men CB (put back what has value replaced)

		tempfavors=favors

		matched=c(rep("Free",nrclusters))
		proposed=c(rep("No",nrclusters))
		Switches=c(rep("Open",nrclusters))

		proposals=matrix(0,length(autograph),length(fan))

		#First match does "fans" that only have 1 element in their column: only one choice
		for(a in 1:dim(tempfavors)[1]){
			for (b in 1:dim(tempfavors)[2]){
				if(favors[a,b]==1){
					matched[a]=b
					proposed[b]="Yes"
					proposals[a,b]=1
					col=a

					change=which(xaxis.names %in% rownames(List[[j]]$Clust$DistM)[fan[[b]]])
					temp3[change]=col

					tempfavors[,b]=0
					tempfavors[a,]=0

					Switches[a]="Closed"
				}

			}
		}


		#OneLeftC=FALSE
		#OneLeftR=FALSE
		for(b in 1:dim(tempfavors)[2]){
			if(length(which(tempfavors[,b]!=0))==1){
				match=which(tempfavors[,b]!=0)
				test=which(tempfavors[match,]==max(tempfavors[match,]))[1]
				if(length(which(tempfavors[,test]!=0))!=1 | b %in% which(tempfavors[match,]==max(tempfavors[match,])) ){
					matched[match]=b
					proposed[b]="Yes"
					proposals[match,b]=1
					col=match

					change=which(xaxis.names %in% rownames(List[[j]]$Clust$DistM)[fan[[b]]])
					temp3[change]=col

					tempfavors[,b]=0
					tempfavors[match,]=0

					Switches[match]="Closed"
				}
			}
			#Unneccesary?
			#if(length(which(tempfavors[,b]==1))>1){
			#	matches=which(tempfavors[,b]==1)
			#	matched[matches]=b
			#	proposed[b]="Yes"
			#	proposals[matches,b]=1
			#	col=matches[1]
			#
			#	change=which(xaxis %in% fan[[b]])
			#	temp3[change]=col
			#
			#	tempfavors[,b]=0
			#	tempfavors[matches,]=0
			#
			#	Switches[matches]="Closed"
			#
			#	OneLeftC=TRUE
			#}
		}

		for(a in 1:dim(tempfavors)[1]){
			if(length(which(tempfavors[a,]!=0))==1){
				propose=which(tempfavors[a,]!=0)
				test=which(tempfavors[,propose]==max(tempfavors[,propose]))[1]
				if(length(which(tempfavors[test,]!=0))!=1 | a %in% which(tempfavors[,propose]==max(tempfavors[,propose]))){

					matched[a]=propose
					proposed[propose]="Yes"
					proposals[a,propose]=1
					col=a

					change=which(xaxis.names %in% rownames(List[[j]]$Clust$DistM)[fan[[propose]]])
					temp3[change]=col

					tempfavors[a,]=0
					tempfavors[,propose]=0

					Switches[a]="Closed"
				}
			}
			#Unnecessary?
			#if(length(which(tempfavors[a,]==1))>1){
			#	proposes=which(tempfavors[a,]==1)
			#	matched[a]="Left"
			#	proposed[proposes]="Yes"
			#	proposals[a,proposes]=1
			#	col=a
			#
			#	change=which(xaxis %in% fan[[proposes]])
			#	temp3[change]=col
			#
			#	tempfavors[a,]=0
			#	tempfavors[,proposes]=0
			#
			#	Switches[a]="Closed"
			#
			#	OneLeftR=TRUE
			#}
		}
		Continue=TRUE
		if(length(which(matched=="Free")) == 0){
			Continue=FALSE
		}

		while(length(which(matched=="Free")) != 0 | !(matequal(proposals[which(matched=="Free"),], matrix(1, length(which(matched=="Free")), nrclusters))) | Continue!=FALSE){
			#for(a in which(matched=="Free")){
			#if(length(which(tempfavors[a,]!=0))==1){
			#	propose=which.max(tempfavors[a,])
			#
			#	matched[a]=propose
			#	proposed[propose]="Yes"
			#	proposals[a,propose]=1
			#	col=a
			#
			#	change=which(xaxis %in% fan[[propose]])
			#	temp3[change]=col
			#
			#	tempfavors[a,propose]=0
			#}

			#else{
			a=which(matched=="Free")[1]
			propose=which.max(tempfavors[a,])
			if(tempfavors[a,propose]==0){
				if(length(which(matched=="Free"))==1){
					Continue=FALSE
				}
				matched[a]="Left"
			}
			else{
				if(proposed[propose]=="No"){
					proposed[propose]="Yes"
					matched[a]=propose
					proposals[a,propose]=1
					col=a

					change=which(xaxis.names %in% rownames(List[[j]]$Clust$DistM)[fan[[propose]]])
					temp3[change]=col

					tempfavors[a,propose]=0

					if(length(which(tempfavors[a,]==0))==dim(tempfavors)[2]){
						Switches[a]="Closed"
						tempfavors[,propose]=0

						c=1
						while(c < a){
							if(Switches[c] != "Closed" & length(which(tempfavors[c,]==0))==dim(tempfavors)[2]){
								Switches[c]="Closed"
								if(matched[c]=="Left"){
									tempfavors[c,]=0
								}
								else{
									tempfavors[,matched[c]]=0
								}
								c=1
							}
							else{
								c=c+1
							}
						}
					}
				}
				else if(proposed[propose]=="Yes"){
					if(favors[a,propose] > max(favors[which(matched==propose),propose]) & Switches[which(matched==propose)]=="Open"){


						#first undo then replace
						#tempfavors[which(matched==propose),propose]=favors[which(matched==propose),propose]

						changeback=which(xaxis.names %in%  rownames(List[[j]]$Clust$DistM)[fan[[propose]]])
						temp3[changeback]=match(xaxis.names,rownames(List[[j]]$Clust$DistM))[changeback]
						matched[which(matched==propose)]="Free"

						matched[a]=propose
						proposals[a,propose]=1
						col=a
						change=which(xaxis.names %in% rownames(List[[j]]$Clust$DistM)[fan[[propose]]])
						temp3[change]=col

						tempfavors[a,propose]=0
					}
					else if(length(which(tempfavors[a,]!=0))==1){
						#if only 1 remains, these MUST BE matched
						changeback=which(xaxis.names %in% rownames(List[[j]]$Clust$DistM)[fan[[propose]]])
						temp3[changeback]=match(xaxis.names,rownames(List[[j]]$Clust$DistM))[changeback]
						matched[which(matched==propose)]="Free"

						matched[a]=propose
						proposals[a,propose]=1
						col=a

						change=which(xaxis.names %in% rownames(List[[j]]$Clust$DistM)[fan[[propose]]])
						temp3[change]=col

						tempfavors[a,propose]=0

					}
					else{
						proposals[a,propose]=1
						tempfavors[a,propose]=0
					}

				}
			}
			if(length(which(matched=="Free"))==0){
				Continue=FALSE
			}
		}
		fusions=0
		for( i in unique(matched)){
			if(length(which(!(seq(1,nrclusters) %in% matched)))>=1){
				fusions=length(which(!(seq(1,nrclusters) %in% matched)))
			}
		}

		if(fusions != 0 & fusionsLog==FALSE){
			message(paste("specify",fusions,"more color(s) and put fusionsLog equal to TRUE",sep=" "))
		}
		premiumcol=c()
		for (i in 1:(fusions)){
			premiumcol=c(premiumcol,length(matched)+i)
		}

		if((length(which(matched=="Left"))!=0) | (length(which(proposed=="No"))!=0)){
			if(length(which(proposed=="No"))!=0){
				for(i in 1:length(which(proposed=="No"))){
					Left=which(proposed=="No")[1]
					maxLeft=which(favors[,Left]==max(favors[,Left]))

					proposed[Left]="Yes"
					proposals[maxLeft,Left]=1
					col=premiumcol[i]

					change=which(xaxis.names %in% rownames(List[[j]]$Clust$DistM)[fan[[Left]]])
					temp3[change]=col

					tempfavors[,Left]=0
					tempfavors[maxLeft,]=0
				}

			}
			if(length(which(matched=="Left"))!=0){
				for (i in 1:length(which(matched=="Left")))
					Left=which(matched=="Left")[1]
				message(paste("Cluster",Left,"of the reference has found no suitable match.",sep=" "))
				#maxLeft=which(favors[Left,]==max(favors[Left,]))

			}
		}

		Ordered[[j]]=temp3

		if(max(ordercolorsj)>nrclusters){
			Ordered[[j]][which(ordercolorsj>nrclusters)]=ordercolorsj[which(ordercolorsj>nrclusters)]
		}
	}

	Matrix=c()
	for(j in 1:length(Ordered)){
		Matrix=rbind(Matrix,Ordered[[j]])
	}
	colnames(Matrix)=List[[1]]$Clust$Clust$order.lab
	rownames(Matrix)=names
	return(Matrix)

}


#' @title Find a selection of objects in the output of \code{ReorderToReference}
#'
#' @description \code{FindCluster} selects the objects belonging to a cluster
#' after the results of the methods have been rearranged by
#' \code{ReorderToReference}.
#'
#' @param List A list of the clustering outputs to be compared. The first
#' element of the list will be used as the reference in
#' \code{ReorderToReference}.
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param select The row (the method) and the number of the cluster to select. Default is c(1,1).
#' @param fusionsLog Logical. To be handed to \code{ReorderToReference}:
#' indicator for the fusion of clusters. Default is TRUE.
#' @param weightclust Logical. To be handed to \code{ReorderToReference}: to be
#' used for the outputs of CEC, WeightedClust or WeightedSimClust. If TRUE, only
#' the result of the Clust element is considered. Default is TRUE.
#' @param names Optional. Names of the methods. Default is NULL.
#' @return A character vector containing the names of the objects in the
#' selected cluster.
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' L <- mosaic_toy$List
#' res <- list(Cluster(L[[1]], type = "data", distmeasure = "euclidean"),
#'             Cluster(L[[2]], type = "data", distmeasure = "euclidean"))
#' Comps <- FindCluster(List = res, nrclusters = 5, select = c(1, 2),
#'                      weightclust = FALSE)
#' }
#' @export FindCluster
FindCluster<-function(List,nrclusters=NULL,select=c(1,1),fusionsLog=TRUE, weightclust=TRUE,names=NULL){
	if(length(List)==1 & attributes(List[[1]])$method == "Weighted" & weightclust==TRUE){
		T=List[[1]]$Clust
		attr(T,"method")="Single Clustering"
		List=list(T)
	}

	if(length(List)==1){
		Matrix=stats::cutree(List[[1]]$Clust,nrclusters)
		names(Matrix)=rownames(List[[1]]$DistM)
		clusternr=select[2]
		Comps=names(which(Matrix==clusternr))
	}

	else{
		Matrix=ReorderToReference(List,nrclusters,fusionsLog,weightclust,names)
		methodnr=select[1]
		clusternr=select[2]
		Comps=names(which(Matrix[methodnr,]==clusternr))
	}
	return(Comps)
}


#' @title Find an element in a data structure
#'
#' @description The function \code{FindElement} is used internally in the
#' \code{PreparePathway} function but might come in handy for other uses as well.
#' Given the name of an object, the function searches for that object in the data
#' structure and extracts it. When multiple objects have the same name, all are
#' extracted.
#'
#' @param what A character string indicating which object to look for. Default is NULL.
#' @param object The data structure to look into. Only the classes data frame
#' and list are supported. Default is NULL.
#' @param element Not to be specified by the user.
#' @return The returned value is a list with an element for each object found.
#' The element contains everything the object contained in the original data
#' structure.
#' @examples
#' \dontrun{
#' obj <- list(a = 1, b = list(TopDE = 42, c = list(TopDE = 7)))
#' FindElement(what = "TopDE", object = obj)
#' }
#' @export FindElement
FindElement<-function(what=NULL,object=NULL,element=list()){
	#str(Object)
	if(is.data.frame(object)){
		#search in columns
		if(what %in% colnames(object)){
			element[[length(element)+1]]<-object[,what]
			names(element)[length(element)]=paste(what,"_",length(element),sep="")
		}
		else if(what %in% rownames(object)){
			element[[length(element)+1]]<-object[what,]
			names(element)[length(element)]=paste(what,"_",length(element),sep="")
		}
	}
	if(is.list(object)){
		#Element=list()

		for(i in 0:length(object)){
			if(i==0){
				Names=names(object)
				if(what%in%Names){
					for(j in which(what==Names)){
						element[length(element)+1]=object[j]
						names(element)[length(element)]=paste(what,"_",length(element),sep="")
						return(element)
					}
				}
			}
			else if(class(object[[i]])[1]=="list"){
				#Names=names(Object[[i]])
				#if(What%in%Names){
				#	for(j in which(What==Names)){
				#			Element[length(Element)+1]=Object[[i]][j]
				#		names(Element)[length(Element)]=paste(What,"_",length(Element),sep="")
				#
				#	}
				#}
				element=FindElement(what,object[[i]],element=element)
				#for(j in 1:length(temp)){
				#	Element[length(Element)+1]=temp[j]
				#	names(Element)[length(Element)]=paste(What,"_",length(Element),sep="")
				#}

			}
			else if(class(object[[i]])[1]=="data.frame"){
				element=FindElement(what,object[[i]],element=element)

			}
		}
	}
	return(element)
}


#' @title Objects shared across clusterings
#'
#' @description \code{SharedComps} determines, per cluster, which objects are
#' shared across all listed clustering results. The clusterings are first
#' rearranged to a common reference with \code{ReorderToReference} and the
#' intersection of the lead objects of every method is taken per cluster.
#'
#' @param List A list of the clustering outputs to be compared. The first
#' element of the list will be used as the reference in
#' \code{ReorderToReference}.
#' @param nrclusters If List is the output of several clustering methods, it has
#' to be provided in how many clusters to cut the dendrograms in. Default is NULL.
#' @param fusionsLog Logical. To be handed to \code{ReorderToReference}:
#' indicator for the fusion of clusters. Default is FALSE.
#' @param weightclust Logical. To be handed to \code{ReorderToReference}: to be
#' used for the outputs of CEC, WeightedClust or WeightedSimClust. If TRUE, only
#' the result of the Clust element is considered. Default is FALSE.
#' @param names Names of the methods or clusters. Default is NULL.
#' @return A list containing the shared objects of all listed elements per cluster.
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' L <- mosaic_toy$List
#' res <- list(Cluster(L[[1]], type = "data", distmeasure = "euclidean"),
#'             Cluster(L[[2]], type = "data", distmeasure = "euclidean"))
#' Comps <- SharedComps(List = res, nrclusters = 5, fusionsLog = FALSE,
#'                      weightclust = FALSE, names = c("S1", "S2"))
#' }
#' @export SharedComps
SharedComps<-function(List,nrclusters=NULL,fusionsLog=FALSE,weightclust=FALSE,names=NULL){
	FoundComps=NULL

	FoundComps=FindElement("objects",List)

	if(is.null(FoundComps)|(is.list(FoundComps) & length(FoundComps) == 0)){
		ListNew=list()
		element=0
		for(i in 1:length(List)){
			if(attributes(List[[i]])$method != "CEC" & attributes(List[[i]])$method != "Weighted" & attributes(List[[i]])$method!= "WeightedSim"){
				ResultsClust=list()
				ResultsClust[[1]]=list()
				ResultsClust[[1]][[1]]=List[[i]]
				names(ResultsClust[[1]])[1]="Clust"
				element=element+1
				ListNew[[element]]=ResultsClust[[1]]
				#attr(ListNew[element],"method")="Weights"
			}
			else if(attributes(List[[i]])$method=="CEC" | attributes(List[[i]])$method=="Weighted" | attributes(List[[i]])$method == "WeightedSim"){
				ResultsClust=list()
				if(weightclust==TRUE){
					ResultsClust[[1]]=list()
					if(attributes(List[[i]])$method != "WeightedSim"){
						ResultsClust[[1]][[1]]=List[[i]]$Clust
						names(ResultsClust[[1]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[1]]
						attr(ListNew[element],"method")="Weights"
					}
					else{
						ResultsClust[[1]]=list()
						ResultsClust[[1]][[1]]=List[[i]]
						names(ResultsClust[[1]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[1]]
					}
				}
				else{
					for (j in 1:length(List[[i]]$Results)){
						ResultsClust[[j]]=list()
						ResultsClust[[j]][[1]]=List[[i]]$Results[[j]]
						names(ResultsClust[[j]])[1]="Clust"
						element=element+1
						ListNew[[element]]=ResultsClust[[j]]
						attr(ListNew[element],"method")="Weights"
					}
				}
			}
		}

		if(is.null(names)){
			names=seq(1,length(ListNew),1)
			for(i in 1:length(ListNew)){
				names[i]=paste("Method",i,sep=" ")
			}
		}
		names(ListNew)=names


		MatrixClusters=ReorderToReference(List,nrclusters,fusionsLog,weightclust,names)
		List=ListNew
		Comps=list()
		for (k in 1:dim(MatrixClusters)[1]){
			clusters=MatrixClusters[k,]

			clust=sort(unique(clusters)) #does not matter: Genes[i] puts right elements on right places
			hc<-stats::as.hclust(List[[k]]$Clust$Clust)
			OrderedCpds <- hc$labels[hc$order]
			clusters=MatrixClusters[k,]
			Method=list()
			for (i in 1:nrclusters){

				temp=list()
				LeadCpds=names(clusters)[which(clusters==i)]
				temp[[1]]=list(LeadCpds,OrderedCpds)
				names(temp[[1]])=c("LeadCpds","OrderedCpds")
				names(temp)="objects"
				Method[[i]]=temp
				names(Method)[i]=paste("Cluster",i,sep=" ")
			}
			Comps[[k]]=Method

		}
		names(Comps)=names
		List=Comps
	}

	for(a in 1:length(List)){
		if(a==1){
			Comps1=FindElement("LeadCpds",List[[a]])
		}
		else if(is.list(Comps1) & length(Comps1) != 0){
			Comps2=FindElement("LeadCpds",List[[a]])
			for(b in 1:length(Comps1)){
				Comps1[[b]]=intersect(Comps1[[b]],Comps2[[b]])
			}
		}

	}

	namesCl=c()
	for(c in 1:length(Comps1)){
		namesCl=c(namesCl,paste("Cluster",c,sep=" "))
	}

	names(Comps1)=namesCl


	return(Comps1)
}

# Tests for the navigation / comparison utilities ported into utils-navigation.r

# Build a realistic input: two single-source clustering results on mosaic_toy,
# each a list with $Clust (agnes) and $DistM and attr "method" == "Cluster".
make_results <- function(nrclusters = 5) {
  data(mosaic_toy, package = "MosaiClusteR")
  L <- mosaic_toy$List
  res <- list(
    Cluster(L[[1]], type = "data", distmeasure = "euclidean"),
    Cluster(L[[2]], type = "data", distmeasure = "euclidean")
  )
  res
}

test_that("the navigation functions are exported and available", {
  expect_true(is.function(ReorderToReference))
  expect_true(is.function(FindCluster))
  expect_true(is.function(FindElement))
  expect_true(is.function(SharedComps))
})

test_that("ReorderToReference relabels clusterings to the reference", {
  res <- make_results()
  M <- suppressMessages(ReorderToReference(
    res, nrclusters = 5, fusionsLog = FALSE,
    weightclust = FALSE, names = c("S1", "S2")
  ))

  expect_true(is.matrix(M))
  # one row per method, one column per object
  expect_equal(nrow(M), 2L)
  expect_equal(ncol(M), nrow(res[[1]]$DistM))
  expect_equal(rownames(M), c("S1", "S2"))
  # columns are labelled by the reference ordering
  expect_setequal(colnames(M), rownames(res[[1]]$DistM))
  # the reference row is itself relabelled to 1..k in dendrogram order
  expect_true(all(M[1, ] %in% seq_len(5)))
})

test_that("FindCluster returns the members of a chosen cluster", {
  res <- make_results()
  comps <- suppressMessages(FindCluster(
    List = res, nrclusters = 5, select = c(1, 2),
    fusionsLog = FALSE, weightclust = FALSE
  ))

  expect_true(is.character(comps))
  # all returned objects are valid object names
  expect_true(all(comps %in% rownames(res[[1]]$DistM)))

  # cross-check against ReorderToReference: method 1, cluster 2
  M <- suppressMessages(ReorderToReference(
    res, nrclusters = 5, fusionsLog = FALSE, weightclust = FALSE, names = NULL
  ))
  expect_setequal(comps, names(which(M[1, ] == 2)))
})

test_that("FindElement recursively extracts a named element", {
  obj <- list(a = 1, b = list(TopDE = 42, c = list(TopDE = 7)))
  found <- FindElement(what = "TopDE", object = obj)

  expect_true(is.list(found))
  expect_true(length(found) >= 1)
  expect_true(all(grepl("^TopDE_", names(found))))
  expect_true(42 %in% unlist(found))

  # absent name yields an empty list
  expect_length(FindElement(what = "DOES_NOT_EXIST", object = obj), 0L)
})

test_that("SharedComps returns shared objects per cluster", {
  res <- make_results()
  sc <- suppressMessages(SharedComps(
    List = res, nrclusters = 5, fusionsLog = FALSE,
    weightclust = FALSE, names = c("S1", "S2")
  ))

  expect_true(is.list(sc))
  expect_length(sc, 5L)
  expect_true(all(grepl("^Cluster ", names(sc))))
  # every shared object is a genuine object name
  allnames <- rownames(res[[1]]$DistM)
  expect_true(all(unlist(sc) %in% allnames))
})

test_that("functions error informatively on bad input", {
  # an empty list cannot be reordered
  expect_error(suppressMessages(ReorderToReference(list(), nrclusters = 3)))
})

#' @title Complementary ensemble clustering
#'
#' @description Complementary Ensemble Clustering (CEC) Complementary Ensemble Clustering (CEC, \cite{Fodeh2013}) shows similarities with ADEC.
#' However, instead of merging the data matrices, ensemble clustering is performedon each data matrix separately. The resulting incidence
#' matrices for each data sets are combined in a weighted linear equation. The weighted incidence matrix is the input for the final clustering
#' algorithm. Similarly as ADEC, there are versions depending of the specification of the number of features to sample and the number of clusters.
#' @export
#' @param List A list of data matrices. It is assumed the rows are corresponding with the objects.
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").
#' @param normalize	Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.
#' @param t The number of iterations. Defaults to 10.
#' @param r A vector with the number of features to take for the random sample for each element in List. If NULL (default), all features are considered.
#' @param nrclusters A list with a sequence of numbers of clusters to cut the dendrogram in for each element in List. If NULL (default), the function stops.
#' @param weight The weights for the weighted linear combination.
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible"
#' @param weightclust A weight for which the result will be put aside of the other results. This was done for comparative reason and easy access.
#' @return The returned value is a list of four elements:
#' \item{DistM}{The resulting incidence matrix}
#' \item{Results}{The hierarchical clustering result for each element in WeightedDist}
#' \item{Clust}{The result for the weight specified in Clustweight}
#' The value has class 'CEC'.
#' @details If r is specified and nrclusters is a fixed number, only a random sampling of the features will be performed for the t iterations (CECa). If r is NULL and the nrclusters is a sequence, the clustering is performedon all features and the dendrogam is divided into clusters for the values of nrclusters (CECb). If both r is specified and nrclusters is a sequence, the combination is performed (CECc).
#' After every iteration, either be random sampling, multiple divisions of the dendrogram or both, an incidence matrix is set up. All incidence matrices are summed and represent the distance matrix on which a final clustering is performed.
#' @references
#' \insertRef{Fodeh2013}{MosaiClusteR}
CEC<-function(List,distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),t=10,r=NULL,nrclusters=NULL,weight=NULL,clust="agnes",linkage=c("flexible","flexible"),alpha=0.625,weightclust=0.5){
  
  if(!is.list(List)){
    stop("Data must be of type list")
  }
  
  if(is.null(nrclusters)){
    stop("Give a number of clusters to cut the dendrogram into for each data modality.")
  }
  
  if(!is.null(r)&length(nrclusters[[1]])==1){
    message("Performing a random sampling of the features with a fixed number of clusters.")
  }else if(is.null(r)&length(nrclusters[[1]])>1){
    message("Dividing the dendrogram in k clusters for a range of values of k.")
    t=1
  }else if(!is.null(r)&length(nrclusters[[1]])>1){
    message("Performing a random sampling of the features and dividing the dendrogram in k clusters for a range of values of k.")
  }
  else{
    stop("Specify r and/or nrclusters in order to perform an ADEC method.")
  }
  
  #Put all data in the same order
  OrderNames=rownames(List[[1]])
  for(i in 1:length(List)){
    List[[i]]=List[[i]][OrderNames,]
  }
  
  #Put up Incidence matrix for each data modality
  nc=c()
  Incidence=list()
  for (i in 1:length(List)){
    Incidence[[i]]=matrix(0,dim(List[[i]])[1],dim(List[[i]])[1])
    rownames(Incidence[[i]])=rownames(List[[i]])
    colnames(Incidence[[i]])=rownames(List[[i]])
    nc=c(nc,ncol(List[[i]]))
  }
  evenn=function(x){if(x%%2!=0)x=x-1 else x}
  nc=lapply(nc,FUN=evenn)
  nc=unlist(nc)
  
  #Repeat for t iterations
  for(g in 1:t){
    #message(g)
    if(is.null(r)){
      r=unlist(sapply(List,ncol))
    }
    
    #take random sample:
    A_prime=list()
    for(i in 1:length(r)){
      A=List[[i]]
      temp=sample(ncol(A),r[i],replace=FALSE)
      A_prime[[i]]=A[,temp]
      
      Ok=FALSE
      while(Ok==FALSE){
        if(is.numeric(A_prime[[i]])){
          if(any(rowSums(A_prime[[i]])==0)){
            temp=sample(ncol(A),r[i],replace=FALSE)
            A_prime[[i]]=A[,temp]
          }
          else{
            Ok=TRUE
          }
        }
        else{
          Ok=TRUE
        }
      }
    }
    
    #protect against zero rows:
    
    
    #Step 2: apply hierarchical clustering on each + cut tree into nrclusters
    
    DistM=lapply(seq(length(A_prime)),function(i) Distance(A_prime[[i]],distmeasure=distmeasure[i],normalize[i],method[i]))
    
    
    HClust_A_prime=lapply(seq(length(DistM)),function(i) cluster::agnes(DistM[[i]],diss=TRUE,method=linkage[i],par.method=alpha))
    
    
    for(o in seq(length(HClust_A_prime))){
      for(k in 1:length(nrclusters[[o]])){
        Temp=stats::cutree(HClust_A_prime[[o]],nrclusters[[o]][k])
        MembersofClust=matrix(1,dim(List[[o]])[1],dim(List[[o]])[1])
        
        for(l in 1:length(Temp)){
          label=Temp[l]
          sameclust=which(Temp==label)
          MembersofClust[l,sameclust]=0
        }
        Incidence[[o]]=Incidence[[o]]+MembersofClust
      }
    }
    
    
    
    
  }
  
  if(is.null(weight)){
    equalweights=1/length(List)
    weight=list(rep(equalweights,length(List)))
  }
  else if(is.list(weight) & length(weight[[1]])!=length(List)){
    stop("Give a weight for each data matrix or specify a sequence of weights")
  }
  else{
    #message('The weights are considered to be a sequence, each situation is investigated')
  }
  
  if(!is.list(weight)){
    condition<-function(l){
      l=as.numeric(l)
      if( sum(l)==1 ){  #working with characters since with the numeric values of comb or permutations something goes not the way is should: 0.999999999<0.7+0.3<1??
        #return(row.match(l,t1))
        return(l)
      }
      else(return(0))
    }
    
    if(all(seq(1,0,-0.1)!=weight)){
      for(i in 1:length(weight)){
        rest=1-weight[i]
        if(!(rest%in%weight)){
          weight=c(weight,rest)
        }
      }
    }
    
    
    t1=gtools::permutations(n=length(weight),r=length(List),v=as.character(weight),repeats.allowed = TRUE)
    t2=lapply(seq_len(nrow(t1)), function(i) if(sum(as.numeric(t1[i,]))==1) return(as.numeric(t1[i,])) else return(0)) #make this faster: lapply on a list or adapt permutations function itself: first perform combinations under restriction then perform permutations
    t3=sapply(seq(length(t2)),function(i) if(!all(t2[[i]]==0)) return (i) else return(0))
    t4=t2[which(t3!=0)]
    weight=lapply(seq(length(t4)),function(i) rev(t4[[i]]))
    
  }
  
  if(is.list(weight) & "x" %in% weight[[1]]){ #x indicates a free weight
    for(i in 1:length(weight)){
      w=weight[[i]]
      weightsfordata=which(w!="x") #position of the provided weight = position of the data to which the weight is given
      givenweights=as.numeric(w[weightsfordata])
      
      stilltodistribute=1-sum(givenweights)
      
      newweights=seq(stilltodistribute,0,-0.1)
      
      t1=gtools::permutations(n=length(newweights),r=length(List)-length(weightsfordata),v=as.character(newweights),repeats.allowed = TRUE)
      Input1=as.list(seq_len(nrow(t1)))
      Input2=lapply(seq(length(Input1)),function(i) {Input1[[i]][length(Input1[[i]])+1]=stilltodistribute
      return(Input1[[i]])})
      t2=lapply(seq(length(Input2)), FUN=function(i){if(sum(as.numeric(t1[Input2[[i]][1],])+0.00000000000000002775)==Input2[[i]][2]) return(as.numeric(t1[i,])) else return(0)}) #make this faster: lapply on a list or adapt permutations function itself: first perform combinations under restriction then perform permutations
      t3=sapply(seq(length(t2)),function(i) if(!all(t2[[i]]==0)) return (i) else return(0))
      weightsforotherdata=t2[which(t3!=0)]
      
      new=list()
      for(i in 1:length(weightsforotherdata)){
        w1=weightsforotherdata[[i]]
        new[[i]]=rep(0,length(List))
        new[[i]][weightsfordata]=givenweights
        new[[i]][which(new[[i]]==0)]=w1
      }
      
      weight=new
    }
  }
  
  weightedcomb<-function(w,Dist){
    temp=lapply(seq_len(length(Dist)),function(i) w[i]*Dist[[i]])
    temp=Reduce("+",temp)
    return(temp)
  }
  
  IncidenceComb=lapply(weight,weightedcomb,Incidence)
  namesweights=c()
  CEC=list()
  Clust=NULL; DistClust=NULL
  for (i in 1:length(IncidenceComb)){
    CEC[[i]]=cluster::agnes(IncidenceComb[[i]],diss=TRUE,method="ward")
    namesweights=c(namesweights,paste("Weight",weight[i],sep=" "))
    if(length(weight[[i]])==length(weightclust) && all(weight[[i]]==weightclust)){
      Clust=CEC[i]
      DistClust=IncidenceComb[i]
    }
  }
  # Fallback when the scalar 'weightclust' matches no generated combination
  # (e.g. the equal-weight focus does not sum to 1 for >2 sources): use the
  # equal-weights combination as the focus result.
  if(is.null(DistClust)){
    wf=rep(1/length(Incidence),length(Incidence))
    Dfocus=weightedcomb(wf,Incidence)
    Clust=list(cluster::agnes(Dfocus,diss=TRUE,method="ward"))
    DistClust=list(Dfocus)
  }

  Results=lapply(seq(1,length(CEC)),function(i) return(c("DistM"=IncidenceComb[i],"Clust"=CEC[i])))
  names(Results)=namesweights
  
  out=list(Incidence=Incidence,Results=Results,Clust=c("DistM"=DistClust,"Clust"=Clust))
  attr(out,'method')<-'CEC'
  return(out)
}

# =============================================================================
# nuggets.R
# Data nuggets: a representative-sample compression of a data set, and the
# nugget-based feature-weighting scheme used as an alternative to variance
# weighting inside the ABC / M-ABC family.
#
# A data nugget summarises a subset of observations by three quantities
# (Cherasia et al. 2023; Beavers/Cabrera, WCluster):
#   * center : the within-nugget centroid (location)
#   * weight : the number of observations assigned to the nugget (importance)
#   * scale  : the within-nugget variability (spread / internal variance)
#
# In MosaiClusteR data nuggets are used in two ways:
#   1. As a Big-Data compression that any downstream clusterer can run on.
#   2. To derive *feature* weights for M-ABC: a feature that varies a lot
#      *between* nugget centres (weighted by nugget size) carries signal,
#      whereas a feature whose variance is mostly *within* nuggets is noise.
#      The nugget between-group weighted variance therefore replaces the plain
#      feature variance as the basis for the ABC selection probabilities.
# =============================================================================

#' Create data nuggets from a data matrix
#'
#' \code{create_data_nuggets} compresses \code{n} observations into
#' \code{M <= n} representative "nuggets". Each nugget stores a centre
#' (location), a weight (the number of member observations) and a scale
#' (within-nugget variability). Data nuggets preserve the covariance structure
#' of the data - including the periphery - far better than a random sub-sample,
#' which makes them a robust, memory-light surrogate for the full data set when
#' clustering or computing feature weights.
#'
#' @param x A numeric matrix or data frame with observations in the \emph{rows}
#'   and variables in the \emph{columns}.
#' @param max_nuggets Target (maximum) number of nuggets \eqn{M}. Defaults to a
#'   data-driven value (see Details). Ignored when \code{engine = "datanugget"}.
#' @param min_size Minimum number of observations per nugget (native engine
#'   only). Defaults to \code{1}.
#' @param engine Either \code{"native"} (a self-contained k-means based
#'   partition, no extra dependencies) or \code{"datanugget"} (delegates to the
#'   \pkg{datanugget} package via \code{datanugget::create.DN} /
#'   \code{datanugget::refine.DN} when it is installed).
#' @param scale_features Logical; if \code{TRUE} (default) the partition is
#'   computed on standardised features so that no single high-variance variable
#'   dominates the nugget assignment. Centres and scales are always reported on
#'   the original scale.
#' @param seed Optional integer seed for reproducible partitioning.
#' @param ... Additional arguments forwarded to
#'   \code{datanugget::create.DN} when \code{engine = "datanugget"}.
#'
#' @details
#' With the native engine the number of nuggets defaults to
#' \code{min(nrow(x), max(2, floor(nrow(x) / 10)))} when \code{n > 50}, and to
#' \code{n} otherwise (in which case every observation is its own nugget and the
#' nugget-weighted variance reduces to the ordinary variance - a safe fallback
#' for small samples). The partition is obtained with \code{stats::kmeans};
#' missing values are mean-imputed for the partition step only.
#'
#' @return An object of class \code{"data_nugget"}: a list with elements
#'   \item{centers}{\eqn{M \times p} matrix of nugget centres.}
#'   \item{weights}{Length-\eqn{M} integer vector of nugget sizes \eqn{n_i}.}
#'   \item{scales}{\eqn{M \times p} matrix of within-nugget variances.}
#'   \item{membership}{Length-\eqn{n} integer vector mapping each observation to
#'         its nugget.}
#'   \item{x_scale}{Length-\eqn{M} vector of total within-nugget variability
#'         \eqn{\mathrm{tr}(\mathrm{Cov})}.}
#'   \item{engine}{The engine used.}
#'
#' @references
#' \insertRef{Cherasia2023}{MosaiClusteR}
#'
#' \insertRef{WCluster2025}{MosaiClusteR}
#'
#' @seealso \code{\link{nugget_feature_weights}}, \code{\link{M_ABCpp}}
#' @examples
#' set.seed(1)
#' x <- rbind(matrix(rnorm(200 * 5,  0), ncol = 5),
#'            matrix(rnorm(200 * 5,  4), ncol = 5))
#' dn <- create_data_nuggets(x, max_nuggets = 40)
#' dn
#' @export
create_data_nuggets <- function(x,
                                 max_nuggets   = NULL,
                                 min_size      = 1L,
                                 engine        = c("native", "datanugget"),
                                 scale_features = TRUE,
                                 seed          = NULL,
                                 ...) {
  engine <- match.arg(engine)
  x <- as.matrix(x)
  storage.mode(x) <- "double"
  n <- nrow(x)
  p <- ncol(x)
  if (n < 2L) stop("create_data_nuggets: need at least 2 observations (rows).")
  if (!is.null(seed)) set.seed(seed)

  # ---- delegate to the datanugget package when requested & available --------
  if (engine == "datanugget") {
    if (!requireNamespace("datanugget", quietly = TRUE))
      stop("engine = 'datanugget' requires the 'datanugget' package; ",
           "install it or use engine = 'native'.")
    return(.dn_from_datanugget(x, ...))
  }

  # ---- native k-means partition --------------------------------------------
  if (is.null(max_nuggets)) {
    max_nuggets <- if (n > 50L) min(n, max(2L, floor(n / 10))) else n
  }
  M <- max(2L, min(as.integer(max_nuggets), n))

  xp <- x
  if (anyNA(xp)) {
    cm <- colMeans(xp, na.rm = TRUE); cm[is.na(cm)] <- 0
    for (j in seq_len(p)) xp[is.na(xp[, j]), j] <- cm[j]
  }
  xs <- if (isTRUE(scale_features)) scale(xp) else xp
  xs[is.na(xs)] <- 0  # zero-variance columns -> NaN after scale()

  if (M >= n) {
    membership <- seq_len(n)
  } else {
    km <- stats::kmeans(xs, centers = M, iter.max = 30L, nstart = 1L)
    membership <- km$cluster
    # enforce min_size by absorbing tiny nuggets into their nearest neighbour
    if (min_size > 1L) membership <- .absorb_small(xs, membership, min_size)
  }

  ids     <- sort(unique(membership))
  M       <- length(ids)
  centers <- matrix(0, M, p, dimnames = list(NULL, colnames(x)))
  scales  <- matrix(0, M, p, dimnames = list(NULL, colnames(x)))
  weights <- integer(M)
  remap   <- match(membership, ids)

  for (i in seq_len(M)) {
    rows <- which(remap == i)
    weights[i]   <- length(rows)
    sub          <- x[rows, , drop = FALSE]
    centers[i, ] <- colMeans(sub, na.rm = TRUE)
    scales[i, ]  <- if (length(rows) > 1L)
      matrixStats::colVars(sub, na.rm = TRUE) else rep(0, p)
  }
  scales[is.na(scales)] <- 0

  out <- list(
    centers    = centers,
    weights    = weights,
    scales     = scales,
    membership = remap,
    x_scale    = rowSums(scales),
    engine     = "native",
    n_obs      = n
  )
  class(out) <- "data_nugget"
  out
}

# delegate path: build a data_nugget object from the datanugget package --------
.dn_from_datanugget <- function(x, ...) {
  dn <- datanugget::create.DN(x, ...)
  dn <- tryCatch(datanugget::refine.DN(x, dn), error = function(e) dn)
  tab <- dn[["Data Nuggets"]]
  cnm <- grep("^Center", colnames(tab), value = TRUE)
  snm <- grep("^Scale|^Shape", colnames(tab), value = TRUE)
  centers <- as.matrix(tab[, cnm, drop = FALSE])
  weights <- as.numeric(tab[["Weight"]])
  scales  <- if (length(snm)) as.matrix(tab[, snm, drop = FALSE]) else
    matrix(0, nrow(centers), ncol(centers))
  out <- list(centers = centers, weights = weights, scales = scales,
              membership = NULL, x_scale = rowSums(scales),
              engine = "datanugget", n_obs = nrow(x))
  class(out) <- "data_nugget"
  out
}

# absorb nuggets smaller than min_size into the nearest surviving centre -------
.absorb_small <- function(xs, membership, min_size) {
  repeat {
    tab <- table(membership)
    small <- as.integer(names(tab)[tab < min_size])
    if (!length(small)) break
    keep <- as.integer(names(tab)[tab >= min_size])
    if (!length(keep)) break
    cent <- t(vapply(keep, function(k)
      colMeans(xs[membership == k, , drop = FALSE]), numeric(ncol(xs))))
    for (s in small) {
      rows <- which(membership == s)
      d <- as.matrix(stats::dist(rbind(colMeans(xs[rows, , drop = FALSE]), cent)))[1, -1]
      membership[rows] <- keep[which.min(d)]
    }
  }
  membership
}

#' @export
print.data_nugget <- function(x, ...) {
  cat("<data_nugget>\n")
  cat("  engine     :", x$engine, "\n")
  cat("  observations:", x$n_obs, "compressed into", length(x$weights), "nuggets\n")
  cat("  reduction  :", sprintf("%.1f%%", 100 * (1 - length(x$weights) / x$n_obs)), "\n")
  cat("  weight range:", paste(range(x$weights), collapse = " - "), "(obs per nugget)\n")
  invisible(x)
}

#' Feature weights derived from data nuggets
#'
#' Computes a per-feature importance score from a data-nugget
#' (\code{\link{create_data_nuggets}}) object built over the \emph{observations}
#' (samples). This score is the
#' data-nugget analogue of the feature variance used by the ABC / M-ABC family:
#' it rewards features that separate the representative groups while discounting
#' within-nugget noise, giving a robust, Big-Data-friendly weighting.
#'
#' @param nuggets A \code{"data_nugget"} object from
#'   \code{\link{create_data_nuggets}} (built with samples in the rows).
#' @param type One of:
#'   \describe{
#'     \item{\code{"between"}}{(default) size-weighted between-nugget variance of
#'       each feature: \eqn{\sum_i w_i (c_{ij} - \bar c_j)^2 / \sum_i w_i}.}
#'     \item{\code{"ratio"}}{a separability / pseudo-\eqn{F} score,
#'       \eqn{\mathrm{between}_j / (\mathrm{between}_j + \overline{\mathrm{within}}_j)}.}
#'     \item{\code{"inv_scale"}}{inverse mean within-nugget variance, which
#'       down-weights noisy features.}
#'   }
#'
#' @return A named numeric vector of length \code{ncol(centers)} of
#'   non-negative feature scores.
#'
#' @references \insertRef{Cherasia2023}{MosaiClusteR}
#' @seealso \code{\link{create_data_nuggets}}, \code{\link{M_ABCpp}}
#' @examples
#' set.seed(1)
#' x <- cbind(signal = c(rnorm(50, 0), rnorm(50, 5)),  # separates groups
#'            noise  = rnorm(100))                      # pure noise
#' dn <- create_data_nuggets(x, max_nuggets = 20)
#' nugget_feature_weights(dn, type = "between")
#' @export
nugget_feature_weights <- function(nuggets, type = c("between", "ratio", "inv_scale")) {
  type <- match.arg(type)
  if (!inherits(nuggets, "data_nugget"))
    stop("nuggets must be a 'data_nugget' object (see create_data_nuggets).")
  C <- nuggets$centers
  w <- nuggets$weights
  W <- sum(w)
  cbar    <- colSums(C * w) / W
  between <- colSums(w * sweep(C, 2, cbar)^2) / W
  within  <- if (!is.null(nuggets$scales))
    colSums(nuggets$scales * w) / W else rep(0, ncol(C))

  score <- switch(type,
    between   = between,
    ratio     = between / (between + within + .Machine$double.eps),
    inv_scale = 1 / (within + .Machine$double.eps)
  )
  score[!is.finite(score)] <- 0
  stats::setNames(as.numeric(score), colnames(C))
}

test_that("create_data_nuggets compresses and preserves total weight", {
  set.seed(1)
  x <- rbind(matrix(rnorm(100 * 4, 0), ncol = 4),
             matrix(rnorm(100 * 4, 5), ncol = 4))
  dn <- create_data_nuggets(x, max_nuggets = 30, seed = 1)
  expect_s3_class(dn, "data_nugget")
  expect_lte(length(dn$weights), 30)
  expect_equal(sum(dn$weights), nrow(x))          # weights partition the data
  expect_equal(ncol(dn$centers), ncol(x))
  expect_true(all(dn$weights >= 1))
})

test_that("small samples fall back to one-nugget-per-observation", {
  x <- matrix(rnorm(20 * 3), ncol = 3)
  dn <- create_data_nuggets(x, seed = 1)        # n = 20 <= 50
  expect_equal(length(dn$weights), 20)
})

test_that("nugget feature weights rank signal above noise", {
  set.seed(2)
  signal <- c(rnorm(60, 0), rnorm(60, 6))       # separates two groups
  noise  <- rnorm(120)
  x <- cbind(signal = signal, noise = noise)
  dn <- create_data_nuggets(x, max_nuggets = 20, seed = 2)
  fw <- nugget_feature_weights(dn, "between")
  expect_gt(fw["signal"], fw["noise"])
  expect_true(all(fw >= 0))
})

#' @title Visualization of characteristic binary features of a single data set
#'
#' @description  A tool to visualize characteristic binary features of a set of objects in comparison with the remaining objects for a single data set. The result is a matrix with coloured cells. Columns represent
#' objects and rows represent the specified features. A feature which is present is give a coloured cell while an absent feature is represented by a grey cell. The labels on the right indicate the names of the features while the labels on the bottom are the names of the objects.
#' @param leadCpds A character vector with the names of the objects in a first group, i.e., the group for which the specified features are characteristic. Default is NULL.
#' @param orderLab A character vector with the order of the objects. Default is NULL.
#' @param features A character vector with the names of the features to be visualized. Default is NULL.
#' @param data The data matrix. Default is NULL.
#' @param colorLab Optional. A clustering object if the objects are to be coloured accoring to their clustering order. Default is NULL.
#' @param nrclusters Optional. The number of clusters to divide the dendrogram of ColorLab. Default is NULL.
#' @param cols Optional. A character vector with the colours of the different clusters. Default is NULL.
#' @param name A character string with the name of the data. Default is "Data".
#' @param colors1 A character vector with the colours to indicate the presence (first element) or the absence of the features for the objects in LeadCpds. Default is c('gray90','blue').
#' @param colors2 A character vector with the colours to indicate the presence (first element) or the absence of the features for the objects in the remaining objects. Default is c('gray90','green').
#' @param highlightFeat Optional. A character vector with names of features to be highlighted. The names of the features are coloured purple. Default is NULL.
#' @param margins A vector with the margings of the plot. Default is c(5.5,3.5,0.5,5.5).
#' @param plottype Should be one of "pdf","new" or "sweave". If "pdf", a
#' location should be provided in "location" and the figure is saved there. If
#' "new" a new graphic device is opened and if "sweave", the figure is made
#' compatible to appear in a sweave or knitr document, i.e. no new device is
#' opened and the plot appears in the current device or document. Default is "new".
#' @param location Optional. If plottype is "pdf", a location should be provided in
#' "location" and the figure is saved there. Default is NULL.
#' @return A plot visualizing the presence and absence of the characteristic binary features.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' Comps=FindCluster(list(MCF7_F),nrclusters=10,select=c(1,8))
#'
#' MCF7_Char=CharacteristicFeatures(List=list(fingerprintMat),Selection=Comps,
#' binData=list(fingerprintMat),datanames=c("FP"),nrclusters=NULL,topC=NULL,
#' sign=0.05,fusionsLog=TRUE,weightclust=TRUE,names=c("FP"))
#' Feat=MCF7_Char$Selection$Characteristics$FP$TopFeat$Names[c(1:10)]
#'
#' BinFeaturesPlot_SingleData(leadCpds=Comps,orderLab=MCF7_Char$Selection$
#' objects$OrderedCpds,features=Feat,data=fingerprintMat,colorLab=NULL,
#' nrclusters=NULL,cols=NULL,name=c("FP"),colors1=c('gray90','blue'),colors2=
#' c('gray90','green'),highlightFeat=NULL,margins=c(5.5,3.5,0.5,5.5),
#' plottype="new",location=NULL)
#' }
#' @export BinFeaturesPlot_SingleData
BinFeaturesPlot_SingleData<-function(leadCpds=c(),orderLab=c(),features=c(),data=NULL,colorLab=NULL,nrclusters=NULL,cols=NULL,name=c("Data"),colors1=c('gray90','blue'),
		colors2=c('gray90','green'),highlightFeat=NULL,margins=c(5.5,3.5,0.5,5.5),plottype="new",location=NULL){

	if(all(leadCpds%in%rownames(data))){
		data=t(data)
	}

	if(!is.null(orderLab)){
		if(inherits(orderLab,"character")){
			orderlabs=orderLab
		}
		else{
			orderlabs=orderLab$Clust$order.lab
			data=data[,match(orderlabs,colnames(data))]

		}
	}
	else{
		orderlabs=colnames(data)
	}

	temp=orderlabs[which(!(orderlabs%in%leadCpds))]
	AllCpds=c(leadCpds,temp)


	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
	}
	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}

	x<-c(1:length(AllCpds)) #x=comps
	y<-c(1:length(features)) #y=feat
	PlotData<-t(data[as.character(features),AllCpds,drop=FALSE])
	plottypein(plottype,location)
	graphics::par(mar=margins)
	graphics::image(x,y,PlotData,col=colors1,xlab="",axes=FALSE,ann=FALSE,xaxt='n')
	if(length(unique(as.vector(PlotData[1:length(leadCpds),])))==1){
		if(unique(as.vector(PlotData[1:length(leadCpds),]))==1){
			PlotData[1:length(leadCpds),]=2
			colors2=c("gray90","blue","green")
			graphics::image(x,y,PlotData,col=colors2,xlab="",axes=FALSE,add=TRUE,ann=FALSE,xaxt='n')
		}
		else{
			colors2=c("gray90","blue")
			graphics::image(x,y,PlotData,col=colors2,xlab="",axes=FALSE,add=TRUE,ann=FALSE,xaxt='n')
		}
	}
	else{
		graphics::image(x[1:length(leadCpds)],y,PlotData[1:length(leadCpds),,drop=FALSE],col=colors2,add=TRUE,xlab="",axes=FALSE,ann=FALSE,xaxt='n')
	}

	if(!(is.null(colorLab)) & !is.null(nrclusters)){
		Data1 <- colorLab$Clust
		ClustData1=stats::cutree(Data1,nrclusters)

		ordercolors=ClustData1[Data1$order]
		names(ordercolors)=Data1$order.lab

		ClustData1=ClustData1[Data1$order]


		order=seq(1,nrclusters)

		for (k in 1:length(unique(ClustData1))){
			select=which(ClustData1==unique(ClustData1)[k])
			ordercolors[select]=order[k]
		}

		colors<- cols[ordercolors]
		names(colors) <-names(ordercolors)
	}
	else{
		colors1<-rep("green",length(leadCpds))
		colors2<-rep("black",length(temp))
		colors=c(colors1,colors2)
		names(colors)=AllCpds
	}
	if(!is.null(highlightFeat)){
		colfeat=rep("black",ncol(PlotData))
		colfeat[which(colnames(PlotData)%in%highlightFeat)]="purple"
		graphics::mtext(colnames(PlotData), side = 4, at= c(1:ncol(PlotData)), line=0.2, las=2,cex=0.8,col=colfeat)
	}
	else{
		graphics::mtext(colnames(PlotData), side = 4, at= c(1:ncol(PlotData)), line=0.2, las=2,cex=0.8)
	}
	graphics::mtext(name, side = 2,  line=1, las=0, cex=1)
	graphics::mtext(rownames(PlotData), side = 1, at= c(1:nrow(PlotData)), line=0.2, las=2, cex=0.8,col=colors[AllCpds])
	plottypeout(plottype)
}

#' @title Visualization of characteristic binary features of multiple data sets
#'
#' @description A tool to visualize characteristic binary features of a set of objects in comparison with the remaining objects for multiple data sets. The result is a matrix with coloured cells. Columns represent
#' objects and rows represent the specified features. A feature which is present is give a coloured cell while an absent feature is represented by a grey cell. The labels on the right indicate the names of the features while the labels on the bottom are the names of the objects.
#' @param leadCpds A character vector with the names of the objects in a first group, i.e., the group for which the specified features are characteristic. Default is NULL.
#' @param orderLab A character vector with the order of the objects. Default is NULL.
#' @param features A list with as elements character vectors with the names of the features to be visualized for each data set. Default is NULL.
#' @param data A list with the different data sets. Default is NULL.
#' @param validate Optional. A list with validation data sets. If a feature has a validation reference, these are added in a red colour. Default is NULL.
#' @param colorLab Optional. A clustering object if the objects are to be coloured accoring to their clustering order. Default is NULL.
#' @param nrclusters Optional. The number of clusters to divide the dendrogram of ColorLab. Default is NULL.
#' @param cols Optional. A character vector with the colours of the different clusters. Default is NULL.
#' @param name A character string with the names of the data sets. Default is c("Data1" ,"Data2") for two data sets.
#' @param colors1 A character vector with the colours to indicate the presence (first element) or the absence of the features for the objects in LeadCpds. Default is c('gray90','blue').
#' @param colors2 A character vector with the colours to indicate the presence (first element) or the absence of the features for the objects in the remaining objects. Default is c('gray90','green').
#' @param margins A vector with the margings of the plot. Default is c(5.5,3.5,0.5,5.5).
#' @param cexB The font size of the labels on the bottom: the object labels. Default is 0.80.
#' @param cexL The font size of the labels on the left: the data labels. Default is 0.80.
#' @param cexR The font size of the labels on the right: the feature labels. Default is 0.80.
#' @param spaceNames A percentage of the height of the figure to be reserved for the names of the objects. Default is 0.20.
#' @param plottype Should be one of "pdf","new" or "sweave". If "pdf", a
#' location should be provided in "location" and the figure is saved there. If
#' "new" a new graphic device is opened and if "sweave", the figure is made
#' compatible to appear in a sweave or knitr document, i.e. no new device is
#' opened and the plot appears in the current device or document. Default is "new".
#' @param location Optional. If plottype is "pdf", a location should be provided in
#' "location" and the figure is saved there. Default is NULL.
#' @return A plot visualizing the presence and absence of the characteristic binary features across multiple data sets.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' Comps=FindCluster(list(MCF7_F),nrclusters=10,select=c(1,8))
#'
#' MCF7_Char=CharacteristicFeatures(List=NULL,Selection=Comps,binData=
#' list(fingerprintMat,targetMat),datanames=c("FP","TP"),nrclusters=NULL,
#' topC=NULL,sign=0.05,fusionsLog=TRUE,weightclust=TRUE,names=c("FP","TP"))
#'
#' FeatFP=MCF7_Char$Selection$Characteristics$FP$TopFeat$Names[c(1:10)]
#' FeatTP=MCF7_Char$Selection$Characteristics$TP$TopFeat$Names[c(1:10)]
#'
#' BinFeaturesPlot_MultipleData(leadCpds=Comps,orderLab=MCF7_Char$Selection$
#' objects$OrderedCpds,features=list(FeatFP,FeatTP),data=list(fingerprintMat,targetMat),
#' validate=NULL,colorLab=NULL,nrclusters=NULL,cols=NULL,name=c("FP","TP"),colors1=
#' c('gray90','blue'),colors2=c('gray90','green'),margins=c(5.5,3.5,0.5,5.5),cexB=0.80,
#' cexL=0.80,cexR=0.80,spaceNames=0.20,plottype="new",location=NULL)
#' }
#' @export BinFeaturesPlot_MultipleData
BinFeaturesPlot_MultipleData<-function(leadCpds,orderLab,features=list(),data=list(),validate=NULL,colorLab=NULL,nrclusters=NULL,cols=NULL,name=c("Data1" ,"Data2"),colors1=c('gray90','blue'),colors2=c('gray90','green'),margins=c(5.5,3.5,0.5,5.5),cexB=0.80,cexL=0.80,cexR=0.80,spaceNames=0.20,plottype="new",location=NULL){

	if (!requireNamespace("plotrix", quietly = TRUE)) {
		stop("BinFeaturesPlot_MultipleData() requires the suggested package 'plotrix'.")
	}

	if(all(leadCpds%in%rownames(data))){
		Data=t(Data)
	}

	if(!is.null(orderLab)){
		if(inherits(orderLab,"character")){
			orderlabs=orderLab
		}
		else{
			orderlabs=orderLab$Clust$order.lab
			data=data[,match(orderlabs,colnames(data))]
		}
	}
	else{
		orderlabs=rownames(data[[1]])
	}



	temp=orderlabs[which(!(orderlabs%in%leadCpds))]
	AllCpds=c(leadCpds,temp)


	if(!is.null(validate)){
		for(v in 1:length(validate)){
			validate[[v]]=validate[[v]][AllCpds,]
		}
	}

	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
		if(plottype=="sweave"){

		}
	}
	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}

	x<-c(1:length(AllCpds)) #x=comps

	Data_new=list()
	for(j in 1:length(data)){
		tempD=data[[j]]
		Data_temp=tempD[AllCpds,as.character(features[[j]]),drop=FALSE]
		Data_new[[j]]=as.matrix(Data_temp)
#		 if(ncol(Data_new[[j]])==1){
#			 colnames(Data_new[[j]])=Features[[j]][which(as.character(Features[[j]])%in%colnames(tempD))]
#			 rownames(Data_new[[j]])=rownames(Data[[1]])
#		 }
	}
	names(Data_new)=name

	Draw=matrix(0,ncol=(length(name)+2),nrow=length(unique(unlist(features))))
	i=0
	for(f in unique(unlist(features))){
		Row=c(f,rep(0,length(name)+1))
		for(j in 1:length(Data_new)){
			if(f%in%colnames(Data_new[[j]])){
				Row[j+1]=name[j]
			}
		}
		Nr=length(which((Row[c(2:(ncol(Draw)-1))])!="0"))
		Row[ncol(Draw)]=Nr
		i=i+1
		Draw[i,]=Row
	}
	Draw=Draw[order(as.numeric(Draw[,ncol(Draw)]),decreasing=TRUE),,drop=FALSE]


	Temp_Draw=Draw[,-c(1,ncol(Draw)),drop=FALSE]
	Fin_Draw=c()
	for(a in 1:nrow(Temp_Draw[!duplicated(Temp_Draw),,drop=FALSE])){
		example=Temp_Draw[!duplicated(Temp_Draw),,drop=FALSE][a,,drop=FALSE]
		join=c()
		for(b in 1:nrow(Temp_Draw)){
			if(all(Temp_Draw[b,]==example)){
				join=c(join,b)
			}
		}

		t=paste(Draw[join,1],collapse=",")
		Fin_Draw=rbind(Fin_Draw,cbind(t,paste(example,collapse=",")))
	}

	plottypein(plottype,location)
	Mat <- matrix(c(1:(nrow(Fin_Draw)+1)),nrow = (nrow(Fin_Draw)+1),ncol = 1,byrow = TRUE)
	H=rep(0,nrow(Fin_Draw))
	Nrtotal=length(unlist(features))
	for(m in 1:nrow(Fin_Draw)){
		Methods=unlist(strsplit(Fin_Draw[m,2],","))
		NrofMethods=length(which(Methods!="0"))
		if(NrofMethods==1){
			Nrtotal=Nrtotal+1
		}
	}

	for(m in 1:nrow(Fin_Draw)){
		NrofFeat=length(unlist(strsplit(Fin_Draw[m,1],",")))
		Methods=unlist(strsplit(Fin_Draw[m,2],","))
		NrofMethods=length(which(Methods!="0"))
		if(!is.null(validate)){
			NrofMethods=NrofMethods+length(validate)
		}
		if(NrofFeat*NrofMethods==1){
			NrofMethods=1.5
		}
		H[m]=((NrofFeat*NrofMethods)/Nrtotal)*(1-spaceNames)

	}
	graphics::layout(mat = Mat,heights=c(H,spaceNames))

	if(length(validate)>0){
		Data_new=c(Data_new,validate)
	}

	for(n in 1:nrow(Fin_Draw)){
		print(n)
		Shared=unlist(strsplit(Fin_Draw[n,1],","))
		Datasets=unlist(strsplit(Fin_Draw[n,2],","))
		V=0
		if(!is.null(validate)){
			V=length(validate)
			Datasets=c(Datasets,names(validate))
		}

		if(any("0"%in%Datasets)){
			Datasets=Datasets[-which(Datasets==0)]
		}

		ImageData=c()
		print(Datasets)
		for(d in Datasets){
			if(all(Shared%in%colnames(Data_new[[d]]))){
				ImageData=cbind(ImageData,Data_new[[d]][,Shared,drop=FALSE])
			}
			else{
				ImageData=cbind(ImageData,Data_new[[d]][,Shared[which(Shared%in%colnames(Data_new[[d]]))],drop=FALSE])
				Fill=Shared[which(!Shared%in%colnames(Data_new[[d]]))]
				for(f in 1:length(Fill)){
					Data_new[[d]]=cbind(Data_new[[d]],rep(0,nrow(Data_new[[d]])))
					colnames(Data_new[[d]])[ncol(Data_new[[d]])]=Fill[f]
				}
				ImageData=cbind(ImageData,Data_new[[d]][,Shared,drop=FALSE])
			}
		}


		if(V>0){
			for(i in 1:(length(Shared)*length(validate))){
				ImageData[,(ncol(ImageData)-(i-1))][which(ImageData[,(ncol(ImageData)-(i-1))]==1)]=3
			}
		}

		ImageData=as.matrix(ImageData)
		ImageData=ImageData[,order(colnames(ImageData)),drop=FALSE]
		colnames(ImageData)=paste(colnames(ImageData),rep(Datasets,length(Shared)),sep="_")
		rownames(ImageData)=rownames(Data_new[[1]])
		#ImageData=ImageData[,c(ncol(ImageData):1),drop=FALSE]
		Colors=matrix(0,nrow(ImageData),ncol(ImageData))
		for(nr in 1:nrow(ImageData)){
			for(nc in 1:ncol(ImageData)){
				if(rownames(ImageData)[nr]%in%leadCpds){
					if(ImageData[nr,nc]==1){
						Colors[nr,nc]="green"
					}
					else if(ImageData[nr,nc]==3){
						Colors[nr,nc]=grDevices::adjustcolor("red", alpha.f = 0.3)
					}
					else{
						Colors[nr,nc]="grey90"
					}
				}
				else{
					if(ImageData[nr,nc]==1){
						Colors[nr,nc]="blue"
					}
					else if(ImageData[nr,nc]==3){
						Colors[nr,nc]=grDevices::adjustcolor("red", alpha.f = 0.3)
					}
					else{
						Colors[nr,nc]="grey90"
					}
				}
			}
		}

		graphics::par(mar=margins)


		plotrix::color2D.matplot(t(ImageData),cellcolors=t(Colors),show.values=FALSE,axes=FALSE,xlab="",ylab="",border=NA)

		ColorsF=rep("black",ncol(ImageData))
		names(ColorsF)=colnames(ImageData)

		if(!(is.null(colorLab)) & !is.null(nrclusters)){
			Data1 <- colorLab$Clust
			ClustData1=stats::cutree(Data1,nrclusters)

			ordercolors=ClustData1[Data1$order]
			names(ordercolors)=Data1$order.lab

			ClustData1=ClustData1[Data1$order]


			order=seq(1,nrclusters)

			for (k in 1:length(unique(ClustData1))){
				select=which(ClustData1==unique(ClustData1)[k])
				ordercolors[select]=order[k]
			}

			colors<- cols[ordercolors]
			names(colors) <-names(ordercolors)
		}
		else{
			Colors1<-rep("green",length(leadCpds))
			Colors2<-rep("black",length(temp))
			Colors=c(Colors1,Colors2)
			names(Colors)=AllCpds

		}

		graphics::mtext(colnames(ImageData), side = 4, at= c(ncol(ImageData):1), line=0.2, las=2,cex=cexR,col=ColorsF)
		if(length(Datasets)==length(Data_new)){
			graphics::mtext("All", side = 2,  line=1, las=1, cex=2)
		}
		else{
			if(length(Datasets)>2){
				N=c()
				for(w in 1:length(Datasets)){
					if(w%%2==0&w!=length(Datasets)){
						N=c(N," & ",Datasets[w],"\n")
					}
					else if(w==1){

						N=c(N,Datasets[w])
					}
					else{
						N=c(N," & ",Datasets[w])
					}
				}
				graphics::mtext(paste(N,collapse=""), side = 2,  line=1, las=1, cex=cexL)
			}
			else{
				graphics::mtext(paste(Datasets,collapse=" & "), side = 2,  line=1, las=1, cex=cexL)
			}
		}

		if(n==nrow(Fin_Draw)){
			graphics::mtext(rownames(ImageData), side = 1, at= c(1:nrow(ImageData)), line=0.2, las=2, cex=cexB,col=Colors[AllCpds])
		}

	}

	plottypeout(plottype)
}

#' @title Plot of continuous features
#'
#' @description The function \code{ContFeaturesPlot} plots the values of continuous features.
#' It is possible to separate between objects of interest and the
#' other objects.
#' @param leadCpds A character vector containing the objects one wants to
#' separate from the others.
#' @param data The data matrix.
#' @param nrclusters Optional. The number of clusters to consider if colorLab
#' is specified. Default is NULL.
#' @param orderLab Optional. If the objects are to set in a specific order of
#' a specific method. Default is NULL.
#' @param colorLab The clustering result that determines the color of the
#' labels of the objects in the plot. If NULL, the labels are black. Default is NULL.
#' @param cols The colors for the labels of the objects. Default is NULL.
#' @param ylab The lable of the y-axis. Default is "features".
#' @param addLegend Logical. Indicates whether a legend should be added to the
#' plot. Default is TRUE.
#' @param margins Optional. Margins to be used for the plot. Default is c(5.5,3.5,0.5,8.7).
#' @param plottype Should be one of "pdf","new" or "sweave". If "pdf", a
#' location should be provided in "location" and the figure is saved there. If
#' "new" a new graphic device is opened and if "sweave", the figure is made
#' compatible to appear in a sweave or knitr document, i.e. no new device is
#' opened and the plot appears in the current device or document. Default is "new".
#' @param location If plottype is "pdf", a location should be provided in
#' "location" and the figure is saved there. Default is NULL.
#' @return A plot in which the values of the features of the leadCpds are
#' separeted from the others.
#' @examples
#' \dontrun{
#' data(Colors1)
#' Comps=c("Cpd1", "Cpd2", "Cpd3", "Cpd4", "Cpd5")
#'
#' Data=matrix(sample(15, size = 50*5, replace = TRUE), nrow = 50, ncol = 5)
#' colnames(Data)=colnames(Data, do.NULL = FALSE, prefix = "col")
#' rownames(Data)=rownames(Data, do.NULL = FALSE, prefix = "row")
#' for(i in 1:50){
#' 	rownames(Data)[i]=paste("Cpd",i,sep="")
#' }
#'
#' ContFeaturesPlot(leadCpds=Comps,orderLab=rownames(Data),colorLab=NULL,data=Data,
#' nrclusters=7,cols=Colors1,ylab="features",addLegend=TRUE,margins=c(5.5,3.5,0.5,8.7),
#' plottype="new",location=NULL)
#' }
#' @export ContFeaturesPlot
ContFeaturesPlot<-function(leadCpds,data,nrclusters=NULL,orderLab=NULL,colorLab=NULL,cols=NULL,ylab="features",addLegend=TRUE,margins=c(5.5,3.5,0.5,8.7),plottype="new",location=NULL){

	if(all(leadCpds%in%rownames(data))){
		data=t(data)
	}

	if(!is.null(orderLab)){
		if(inherits(orderLab,"character")){
			orderlabs=orderLab
		}
		else{
			orderlabs=orderLab$Clust$order.lab
			data=data[,match(orderlabs,colnames(data))]
		}
	}
	else{
		orderlabs=colnames(data)
	}

	temp=orderlabs[which(!(orderlabs%in%leadCpds))]
	AllCpds=c(leadCpds,temp)

	if(is.null(dim(data))){
		data=t(as.matrix(data))
		rownames(data)="Feature"
	}
	data=data[,AllCpds,drop=FALSE]

	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
		if(plottype=="sweave"){

		}
	}
	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}


	plottypein(plottype,location)
	graphics::par(mar=margins)
	graphics::plot(x=0,y=0,xlim=c(0,(ncol(data)+3)),ylim=c(min(data)-0.5,max(data)+0.5),type="n",ylab=ylab,xlab='',xaxt='n')
	for(i in c(1:nrow(data))){
		graphics::lines(x=seq(1,length(leadCpds)),y=data[i,which(colnames(data)%in%leadCpds)],col=i)
	}
	for(i in c(1:nrow(data))){
		graphics::lines(x=seq(length(leadCpds)+4,(ncol(data)+3)),y=data[i,which(!(colnames(data)%in%leadCpds))],col=i)
	}



	if(!(is.null(colorLab)) | is.null(nrclusters)){
		Data1 <- colorLab$Clust
		ClustData1=stats::cutree(Data1,nrclusters)

		ordercolors=ClustData1[Data1$order]
		names(ordercolors)=Data1$order.lab

		ClustData1=ClustData1[Data1$order]


		order=seq(1,nrclusters)

		for (k in 1:length(unique(ClustData1))){
			select=which(ClustData1==unique(ClustData1)[k])
			ordercolors[select]=order[k]
		}

		colors<- cols[ordercolors]
		names(colors) <-names(ordercolors)
	}
	else{
		colors1<-rep("green",length(leadCpds))
		colors2<-rep("black",length(temp))
		colors=c(colors1,colors2)
		names(colors)=AllCpds
	}

	graphics::mtext(leadCpds,side=1,at=seq(1,length(leadCpds)),line=0.6,las=2,cex=0.70,col=colors[leadCpds])
	graphics::mtext(temp, side = 1, at=c(seq(length(leadCpds)+4,(ncol(data)+3))), line=0.5, las=2, cex=0.70,col=colors[temp])
	if(addLegend==TRUE){

		labels=rownames(data)
		colslegend=seq(1,length(rownames(data)))

		graphics::par(xpd=T,mar=margins)
		graphics::legend(ncol(data)+5,mean(c(min(c(min(data)-0.5,max(data)+0.5)),max(c(min(data)-0.5,max(data)+0.5)))),legend=c(labels),col=c(colslegend),lty=1,lwd=3,cex=0.8)

	}
	plottypeout(plottype)
}

#' @title A GO plot of a pathway analysis output.
#'
#' @description The \code{PlotPathways} function takes an output of the
#' \code{PathwayAnalysis} function and plots a GO graph with the help of the
#' \code{plotGOgraph} function of the MLP package.
#' @param Pathways One element of the output list returned by
#' \code{PathwayAnalysis} or \code{Geneset.intersect}.
#' @param nRow Number of GO IDs for which to produce the plot. Default is 5.
#' @param main Title of the plot. Default is NULL.
#' @param plottype Should be one of "pdf","new" or "sweave". If "pdf", a
#' location should be provided in "location" and the figure is saved there. If
#' "new" a new graphic device is opened and if "sweave", the figure is made
#' compatible to appear in a sweave or knitr document. Default is "new".
#' @param location If plottype is "pdf", a location should be provided in
#' "location" and the figure is saved there. Default is NULL.
#' @return The output is a GO graph.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#' data(geneMat)
#' data(GeneInfo)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(MCF7_F,MCF7_T)
#' names=c('FP','TP')
#'
#' MCF7_PathsFandT=PathwayAnalysis(List=L, geneExpr = geneMat, nrclusters = 7, method = c("limma",
#' "MLP"), geneInfo = GeneInfo, geneSetSource = "GOBP", topP = NULL,
#' topG = NULL, GENESET = NULL, sign = 0.05,niter=2,fusionsLog = TRUE, weightclust = TRUE,
#'  names =names,seperatetables=FALSE,separatepvals=FALSE)
#'
#' PlotPathways(MCF7_PathsFandT$FP$"Cluster 1"$Pathways,nRow=5,main=NULL)
#' }
#' @export PlotPathways
PlotPathways<-function(Pathways,nRow=5,main=NULL,plottype="new",location=NULL){

	if (!requireNamespace("MLP", quietly = TRUE)) {
		stop("MLP needed for this function to work. Please install it.",
				call. = FALSE)
	}

	if (!requireNamespace("biomaRt", quietly = TRUE)) {
		stop("biomaRt needed for this function to work. Please install it.",
				call. = FALSE)
	}

	if (!requireNamespace("org.Hs.eg.db", quietly = TRUE)) {
		stop("org.Hs.eg.db needed for this function to work. Please install it.",
				call. = FALSE)
	}


	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
		if(plottype=="sweave"){

		}
	}
	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}
	#preparing data structure for the plotGOGraph
	colnames(Pathways)[3:ncol(Pathways)]=sub("mean_","",colnames(Pathways)[3:ncol(Pathways)])
	#plot GOgraph
	plottypein(plottype,location)
	MLP::plotGOgraph(Pathways,nRow=nRow,main=main)
	plottypeout(plottype)

}

#' @title Plotting gene profiles
#'
#' @description In \code{ProfilePlot}, the gene profiles of the significant genes for a
#' specific cluster are shown on 1 plot. Therefore, each gene is normalized by
#' subtracting its the mean.
#'
#' @param Genes The genes to be plotted.
#' @param Comps The objects to be plotted or to be separated from the other
#' objects.
#' @param geneExpr The gene expression matrix or ExpressionSet of the objects.
#' @param raw Logical. Should raw p-values be plotted? Default is FALSE.
#' @param orderLab Optional. If the objects are to set in a specific order of
#' a specific method. Default is NULL.
#' @param colorLab The clustering result that determines the color of the
#' labels of the objects in the plot. Default is NULL.
#' @param nrclusters Optional. The number of clusters to cut the dendrogram in.
#' @param cols Optional. The color to use for the objects in Clusters for each
#' method.
#' @param addLegend Optional. Whether a legend of the colors should be added to
#' the plot.
#' @param margins Optional. Margins to be used for the plot. Default is margins=c(8.1,4.1,1.1,6.5).
#' @param extra The space between the plot and the legend. Default is 5.
#' @param plottype Should be one of "pdf","new" or "sweave". If "pdf", a
#' location should be provided in "location" and the figure is saved there. If
#' "new" a new graphic device is opened and if "sweave", the figure is made
#' compatible to appear in a sweave or knitr document, i.e. no new device is
#' opened and the plot appears in the current device or document. Default is "new".
#' @param location If plottype is "pdf", a location should be provided in
#' "location" and the figure is saved there. Default is NULL.
#' @return A plot which contains multiple gene profiles. A distinction is made
#' between the values for the objects in Comps and the others.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#' data(geneMat)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#'		method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#'		method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#' L=list(MCF7_F,MCF7_T)
#' names=c('FP','TP')
#'
#' MCF7_FT_DE = DiffGenes(List=L,geneExpr=geneMat,nrclusters=7,method="limma",sign=0.05,topG=10,
#' fusionsLog=TRUE,weightclust=TRUE)
#'
#' Comps=SharedComps(list(MCF7_FT_DE$`Method 1`$"Cluster 1",MCF7_FT_DE$`Method 2`$"Cluster 1"))[[1]]
#'
#' MCF7_SharedGenes=FindGenes(dataLimma=MCF7_FT_DE,names=c("FP","TP"))
#'
#' Genes=names(MCF7_SharedGenes[[1]])[-c(2,4,5)]
#'
#' colscl=ColorPalette(colors=c("red","green","purple","brown","blue","orange"),ncols=9)
#'
#' ProfilePlot(Genes=Genes,Comps=Comps,geneExpr=geneMat,raw=FALSE,orderLab=MCF7_F,
#' colorLab=NULL,nrclusters=7,cols=colscl,addLegend=TRUE,margins=c(16.1,6.1,1.1,13.5),
#' extra=4,plottype="sweave",location=NULL)
#' }
#' @export ProfilePlot
ProfilePlot<-function(Genes,Comps,geneExpr=NULL,raw=FALSE,orderLab=NULL,colorLab=NULL,nrclusters=NULL,cols=NULL,addLegend=TRUE,margins=c(8.1,4.1,1.1,6.5),extra=5,plottype="new",location=NULL){
	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
		if(plottype=="sweave"){

		}
	}
	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}



	if(inherits(geneExpr,"ExpressionSet")){
		if (!requireNamespace("Biobase", quietly = TRUE)) {
			stop("ProfilePlot() requires the suggested package 'Biobase'.")
		}
		geneExpr <- Biobase::exprs(geneExpr)

	}

	if(!is.null(orderLab)){
		if(inherits(orderLab,"character")){
			orderlabs=orderLab
		}
		else{
			orderlabs=orderLab$Clust$order.lab
			geneExpr=geneExpr[,match(orderlabs,colnames(geneExpr))]
		}
	}
	else{
		orderlabs=colnames(geneExpr)
	}

	if(!is.null(colorLab)){
		Data1 <- colorLab$Clust
		ClustData1=stats::cutree(Data1,nrclusters)

		ordercolors=ClustData1[Data1$order]
		names(ordercolors)=Data1$order.lab

		ClustData1=ClustData1[Data1$order]

		order=seq(1,nrclusters)

		for (k in 1:length(unique(ClustData1))){
			select=which(ClustData1==unique(ClustData1)[k])
			ordercolors[select]=order[k]
		}

		if(!is.null(orderLab)){
			if(inherits(orderLab,"character")){
				ordernames=orderLab
			}
			else{
				ordernames=orderLab$Clust$order.lab
			}
			ordercolors=ordercolors[ordernames]
		}

		colors<- cols[ordercolors]
		names(colors) <-names(ordercolors)

	}
	else{
		#colors1<-rep("green",length(Comps))
		#colors2<-rep("black",length(orderlabs[which(!(orderlabs%in%Comps))]))
		colors1<-rep("blue",length(Comps))
		colors2<-rep("red",length(orderlabs[which(!(orderlabs%in%Comps))]))
		colors=c(colors1,colors2)
		AllCpds=c(Comps,orderlabs[which(!(orderlabs%in%Comps))])
		names(colors)=AllCpds
	}

	#yvalues=c()
	#allvalues=c()
	#for(i in 1:length(Genes)){
	#	yvalues=as.vector(GeneExpr[which(rownames(GeneExpr)==Genes[i]),])
	#	allvalues=c(allvalues,yvalues-mean(yvalues))
	#}
	yvalues=geneExpr[Genes,]
	if(raw==FALSE & !inherits(yvalues,"numeric")){
		allvalues=as.vector(apply(yvalues,1,function(c) c-mean(c)))
	}
	else if(raw==FALSE & inherits(yvalues,"numeric")){
		allvalues=as.vector(sapply(yvalues,function(c) c-mean(yvalues)))
	}
	else{
		allvalues=as.vector(yvalues)
	}
	ylims=c(min(allvalues)-0.1,max(allvalues)+0.1)

	plottypein(plottype,location)
	graphics::par(mar=margins,xpd=TRUE)
	graphics::plot(type="n",x=0,y=0,xlim=c(0,ncol(geneExpr)),ylim=ylims,ylab=expression(log[2] ~ paste("fold ", "change")),xlab=" ",xaxt="n",cex.axis=1.5,cex.lab=2)
	#ylims=c()
	Indices=c(colnames(geneExpr)[which(colnames(geneExpr)%in%Comps)],colnames(geneExpr)[which(!colnames(geneExpr)%in%Comps)])

	for(i in 1:length(Genes)){
		GenesComps=as.numeric(geneExpr[which(rownames(geneExpr)==Genes[i]),colnames(geneExpr)%in%Comps])
		Others=as.numeric(geneExpr[which(rownames(geneExpr)==Genes[i]),!(colnames(geneExpr)%in%Comps)])
		if(length(Others)==0){
			Continue=FALSE
		}else{Continue=TRUE}

		#ylims=c(ylims,c(GenesComps,Others)-mean(c(GenesComps,Others)))
		if(raw==FALSE){
			yvalues1=GenesComps-mean(c(GenesComps,Others))
		}
		else{
			yvalues1=GenesComps
		}

		graphics::lines(x=seq(1,length(GenesComps)),y=yvalues1,lty=1,col=i,lwd=1.6)
		#points(x=seq(1,length(GenesComps)),y=yvalues1,pch=19,col=i)
		graphics::segments(x0=1,y0=mean(yvalues1[1:length(GenesComps)]),x1=length(GenesComps),y1=mean(yvalues1[1:length(GenesComps)]),lwd=1.5,col=i)


		if(Continue==TRUE){
			if(raw==FALSE){
				yvalues2=Others-mean(c(GenesComps,Others))
			}
			else{
				yvalues2=Others
			}


			graphics::lines(x=seq(length(GenesComps)+1,ncol(geneExpr)),y=yvalues2,lty=1,col=i,lwd=1.6)
			graphics::segments(x0=length(GenesComps)+1,y0=mean(yvalues2[1:length(Others)]),x1=ncol(geneExpr),y1=mean(yvalues2[1:length(Others)]),lwd=1.5,col=i)

		}


	}
	#Indices=c(colnames(GeneExpr)[which(colnames(GeneExpr)%in%Comps)],colnames(GeneExpr)[which(!colnames(GeneExpr)%in%Comps)])
	if(!is.null(colorLab)){
		graphics::axis(1, labels=FALSE)
		#box("outer")
		graphics::mtext(substr(Indices,1,15), side = 1,  at=seq(0.5,(ncol(geneExpr)-0.5)), line=0.2, las=2, cex=1.5,col=colors[Indices])
		#mtext(substr(Indices,1,15), side = 1,  at=seq(0.5,(ncol(GeneExpr)-0.5)), line=0.2, las=2, cex=0.70,col=c(rep("blue",7),rep("black",(56-7))))
	}
	else{
		#axis(1,at=seq(0.5,(ncol(GeneExpr)-0.5)),labels=Indices,las=2,cex.axis=0.70,xlab=" ",col=colors[Indices])
		#axis(1,at=seq(0.5,(ncol(GeneExpr)-0.5)), labels=FALSE)
		graphics::mtext(Indices, side = 1,  at=seq(0.5,(ncol(geneExpr)-0.5)),line=0.2, las=2, cex=1.5,col=colors[Indices])
	}
	graphics::axis(2,ylab=expression(log[2] ~ paste("fold ", "change")),cex=2,cex.axis=1.5)

	if(addLegend==TRUE){

		labels=Genes
		colslegend=seq(1,length(Genes))

		graphics::par(xpd=T,mar=margins)
		graphics::legend(ncol(geneExpr)+extra,max(ylims),legend=c(labels),col=c(colslegend),lty=1,lwd=3,cex=1.5)

	}
	plottypeout(plottype)
}

# =============================================================================
# spectral.R  - spectral clustering on a similarity/affinity matrix
# Used directly and as the final clustering step of NEMO and similarity fusion.
# =============================================================================

#' Spectral clustering of an affinity matrix
#'
#' Normalised (Ng-Jordan-Weiss) spectral clustering. Given a symmetric,
#' non-negative affinity matrix it forms the normalised Laplacian, embeds the
#' objects in the space of its leading eigenvectors and applies k-means.
#'
#' @param affinity A symmetric \eqn{n \times n} non-negative similarity/affinity
#'   matrix over the objects.
#' @param k Number of clusters. If \code{NULL}, chosen by the largest eigengap
#'   of the normalised Laplacian (searched over \code{2:max_k}).
#' @param max_k Upper bound for the eigengap search when \code{k = NULL}.
#' @param nstart Number of k-means restarts.
#' @return A list with \code{cluster} (named integer labels), \code{k} and the
#'   spectral \code{embedding}.
#' @references Ng, A. Y., Jordan, M. I. & Weiss, Y. (2002). On spectral
#'   clustering: analysis and an algorithm. NIPS 14.
#' @seealso \code{\link{NEMO}}, \code{\link{SNF}}
#' @examples
#' set.seed(1)
#' X <- rbind(matrix(rnorm(20 * 5, 0), 20), matrix(rnorm(20 * 5, 6), 20))
#' A <- exp(-as.matrix(dist(X))^2 / 2)
#' spectral_clustering(A, k = 2)$cluster[1:5]
#' @export
spectral_clustering <- function(affinity, k = NULL, max_k = 10, nstart = 10) {
  W <- as.matrix(affinity)
  W <- (W + t(W)) / 2
  diag(W) <- 0
  n <- nrow(W)
  d <- rowSums(W)
  d[d <= 0] <- .Machine$double.eps
  Dm <- 1 / sqrt(d)
  Lsym <- diag(n) - (Dm * W) * rep(Dm, each = n)   # I - D^-1/2 W D^-1/2
  Lsym <- (Lsym + t(Lsym)) / 2

  ev <- eigen(Lsym, symmetric = TRUE)
  vals <- sort(ev$values)                            # ascending

  if (is.null(k)) {
    upper <- min(max_k, n - 1L)
    gaps  <- diff(vals[seq_len(upper + 1L)])
    k <- which.max(gaps[-1]) + 1L                    # ignore trivial first gap
    k <- max(2L, k)
  }
  k <- max(2L, min(as.integer(k), n - 1L))

  # k smallest eigenvectors of Lsym (largest of the normalised affinity)
  U <- ev$vectors[, order(ev$values)[seq_len(k)], drop = FALSE]
  rn <- sqrt(rowSums(U^2)); rn[rn == 0] <- 1
  U  <- U / rn                                        # row-normalise

  km <- stats::kmeans(U, centers = k, nstart = nstart, iter.max = 50L)
  cl <- stats::setNames(as.integer(km$cluster), rownames(W))
  list(cluster = cl, k = k, embedding = U)
}

# =============================================================================
# lucid.R  - wrapper for LUCID (Latent Unknown Clustering Integrating multi-omics
# Data with phenotypic traits). Delegates to the LUCIDus package (Suggests).
# Zhao, Jia, Goodrich & Conti (2024), The R Journal 16(2).
# =============================================================================

#' LUCID: latent unknown clustering integrating omics with an outcome
#'
#' Thin wrapper around \pkg{LUCIDus}' \code{estimate_lucid}. LUCID is a
#' \emph{model-based, supervised} integrative clustering method: it defines a
#' categorical latent variable (the clusters) jointly from upstream exposures
#' \code{G}, multi-omics data \code{Z} and an outcome \code{Y} via an EM
#' algorithm (a quasi-mediation model). Unlike the unsupervised methods in
#' MosaiClusteR it requires an outcome and returns posterior cluster membership.
#'
#' @param G Exposure/predictor matrix (objects in rows).
#' @param Z Omics matrix, or a list of matrices for the parallel/serial models
#'   (objects in rows).
#' @param Y Outcome vector (length = number of objects).
#' @param K Number of latent clusters.
#' @param lucid_model One of \code{"early"}, \code{"parallel"}, \code{"serial"}.
#' @param family Outcome family, \code{"normal"} or \code{"binary"}.
#' @param ... Further arguments forwarded to \code{LUCIDus::estimate_lucid}.
#' @return A list of class \code{"LUCID"} with the fitted \code{model} and the
#'   posterior \code{cluster} assignment per object.
#' @references Zhao, Y., Jia, K., Goodrich, J. & Conti, D. V. (2024). LUCIDus:
#'   an R package for implementing LUCID with phenotypic traits. The R Journal
#'   16(2).
#' @seealso \code{\link{M_ABCpp}}, \code{\link{intNMF}}
#' @examples
#' \dontrun{
#' # requires the 'LUCIDus' package
#' data(mosaic_toy)
#' Z <- mosaic_toy$List[[1]]
#' G <- mosaic_toy$List[[2]][, 1:3]
#' Y <- as.numeric(mosaic_toy$truth)
#' fit <- LUCID(G = G, Z = Z, Y = Y, K = 3)
#' table(fit$cluster)
#' }
#' @export
LUCID <- function(G, Z, Y, K, lucid_model = c("early", "parallel", "serial"),
                  family = c("normal", "binary"), ...) {
  if (!requireNamespace("LUCIDus", quietly = TRUE))
    stop("LUCID() requires the suggested package 'LUCIDus'. ",
         "Install it with install.packages('LUCIDus').")
  lucid_model <- match.arg(lucid_model)
  family      <- match.arg(family)

  fit <- LUCIDus::estimate_lucid(
    lucid_model = lucid_model, G = as.matrix(G), Z = Z, Y = as.matrix(Y),
    K = K, family = family, ...)

  # posterior cluster membership across LUCIDus versions
  post <- fit$inclusion.p
  if (is.null(post)) post <- fit$post.p
  if (is.null(post)) post <- fit$z
  cluster <- if (!is.null(post)) max.col(as.matrix(post), ties.method = "first")
             else fit$pred

  out <- list(model = fit,
              cluster = stats::setNames(as.integer(cluster), rownames(as.matrix(G))),
              K = K)
  attr(out, "method") <- "LUCID"
  class(out) <- c("LUCID", "list")
  out
}

# =============================================================================
# utils-evaluation.R
# Partition extraction, agreement metrics and method comparison.
# =============================================================================

#' Extract a flat partition from a MosaiClusteR result
#'
#' Cuts the hierarchical clustering carried in a MosaiClusteR result object into
#' \code{k} groups. Works with any method returning a \code{Clust} element (an
#' \code{agnes}/\code{hclust} object) and/or a \code{DistM}.
#'
#' @param object A MosaiClusteR result (e.g. from \code{\link{M_ABCpp}}), an
#'   \code{agnes}/\code{hclust} object, or a distance matrix.
#' @param k Number of clusters.
#' @param linkage Linkage used when \code{object} is a bare distance matrix.
#' @return An integer vector of cluster labels, named by object.
#' @examples
#' set.seed(1)
#' s <- paste0("o", 1:20)
#' L <- list(matrix(rnorm(20 * 40), 20, dimnames = list(s, NULL)),
#'           matrix(rnorm(20 * 50), 20, dimnames = list(s, NULL)))
#' fit <- M_ABCpp(L, numsim = 30)
#' mosaic_labels(fit, k = 3)
#' @export
mosaic_labels <- function(object, k, linkage = "ward") {
  cl <- NULL
  if (is.list(object) && !is.null(object$Clust)) {
    cl <- object$Clust
    if (is.list(cl) && !is.null(cl$Clust)) cl <- cl$Clust  # WeightedClust nesting
  } else if (inherits(object, c("agnes", "hclust", "twins"))) {
    cl <- object
  } else if (is.matrix(object)) {
    cl <- cluster::agnes(as.matrix(object), diss = TRUE, method = linkage)
  } else {
    stop("Cannot extract a clustering from 'object'.")
  }
  stats::cutree(stats::as.hclust(cl), k = k)
}

#' Agreement between two partitions
#'
#' Computes the adjusted Rand index (ARI), normalised mutual information (NMI)
#' and pairwise Jaccard between two label vectors of equal length.
#'
#' @param a,b Integer/character/factor label vectors of equal length.
#' @return A named numeric vector \code{c(ARI, NMI, Jaccard)}.
#' @details ARI rule of thumb: \code{>0.90} excellent; \code{0.75-0.90} good;
#'   \code{0.50-0.75} moderate; \code{<0.50} weak.
#' @examples
#' cluster_agreement(c(1, 1, 2, 2), c(2, 2, 1, 1))   # identical up to relabel
#' @export
cluster_agreement <- function(a, b) {
  a <- as.integer(as.factor(a)); b <- as.integer(as.factor(b))
  if (length(a) != length(b)) stop("'a' and 'b' must have equal length.")
  n  <- length(a)
  ct <- table(a, b)
  rs <- rowSums(ct); cs <- colSums(ct)

  sum_a <- sum(choose(rs, 2)); sum_b <- sum(choose(cs, 2))
  sum_ab <- sum(choose(ct, 2)); sum_n <- choose(n, 2)
  expected <- sum_a * sum_b / sum_n
  max_val  <- 0.5 * (sum_a + sum_b)
  ari <- if (max_val == expected) 0 else (sum_ab - expected) / (max_val - expected)

  p <- ct / n; p1 <- rowSums(p); p2 <- colSums(p)
  H1 <- -sum(p1[p1 > 0] * log(p1[p1 > 0]))
  H2 <- -sum(p2[p2 > 0] * log(p2[p2 > 0]))
  pm <- outer(p1, p2); mask <- p > 0 & pm > 0
  MI <- sum(p[mask] * log(p[mask] / pm[mask]))
  nmi <- if (H1 + H2 == 0) 0 else 2 * MI / (H1 + H2)

  same1 <- outer(a, a, "=="); same2 <- outer(b, b, "==")
  lt <- lower.tri(same1)
  A  <- sum(same1[lt] & same2[lt]); B <- sum(same1[lt] & !same2[lt])
  C  <- sum(!same1[lt] & same2[lt])
  jac <- if (A + B + C == 0) 1 else A / (A + B + C)

  c(ARI = ari, NMI = nmi, Jaccard = jac)
}

#' Compare two clusterings derived from distance matrices
#'
#' Clusters two dissimilarity matrices with the same linkage and number of
#' clusters and reports their agreement (ARI, NMI, pairwise Jaccard) and the
#' contingency table.
#'
#' @param D1,D2 Square dissimilarity matrices over the same objects.
#' @param NC Number of clusters (default \code{floor(sqrt(n))}).
#' @param linkage Linkage method. Default \code{"ward.D2"}.
#' @param labels Length-2 labels for the two solutions.
#' @param verbose Logical; print a short report.
#' @return Invisibly, a list with the two label vectors, \code{ARI},
#'   \code{NMI}, \code{pair_Jaccard} and the \code{contingency} table.
#' @examples
#' set.seed(1)
#' D1 <- as.matrix(dist(matrix(rnorm(60), 20)))
#' D2 <- as.matrix(dist(matrix(rnorm(60), 20)))
#' compare_clusterings(D1, D2, NC = 3, verbose = FALSE)$ARI
#' @export
compare_clusterings <- function(D1, D2, NC = NULL, linkage = "ward.D2",
                                labels = c("D1", "D2"), verbose = TRUE) {
  stopifnot(identical(dim(D1), dim(D2)))
  ns <- nrow(D1)
  if (is.null(NC)) NC <- max(2L, floor(sqrt(ns)))
  h1 <- fastcluster::hclust(stats::as.dist(D1), method = linkage)
  h2 <- fastcluster::hclust(stats::as.dist(D2), method = linkage)
  c1 <- as.integer(stats::cutree(h1, k = NC))
  c2 <- as.integer(stats::cutree(h2, k = NC))
  names(c1) <- names(c2) <- rownames(D1)

  ag <- cluster_agreement(c1, c2)
  ct <- table(c1, c2)
  dimnames(ct) <- list(sprintf("%s_c%d", labels[1], seq_len(nrow(ct))),
                       sprintf("%s_c%d", labels[2], seq_len(ncol(ct))))
  if (verbose) {
    cat(sprintf("NC = %d, linkage = %s, n = %d\n", NC, linkage, ns))
    cat(sprintf("Adjusted Rand Index : %.4f\n", ag["ARI"]))
    cat(sprintf("Normalized Mut. Info: %.4f\n", ag["NMI"]))
    cat(sprintf("Pair-Jaccard (same) : %.4f\n", ag["Jaccard"]))
    print(ct)
  }
  invisible(list(clusters_1 = c1, clusters_2 = c2, ARI = ag["ARI"],
                 NMI = ag["NMI"], pair_Jaccard = ag["Jaccard"], contingency = ct))
}

test_that("M_ABCpp returns a standard MosaiClusteR result", {
  data(mosaic_toy)
  fit <- M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3,
                 distmeasure = c("euclidean", "euclidean"))
  expect_true(all(c("DistM", "Clust") %in% names(fit)))
  expect_true(inherits(fit$Clust, c("agnes", "twins")))
  expect_equal(dim(fit$DistM), c(60, 60))
  expect_true(isSymmetric(unname(fit$DistM)))
  expect_equal(unname(diag(fit$DistM)), rep(0, 60))
})

test_that("M_ABCpp recovers structure with var and data-nugget weighting", {
  data(mosaic_toy)
  truth <- mosaic_toy$truth
  set.seed(11)
  fit_v <- M_ABCpp(mosaic_toy$List, numsim = 60, NC = 3,
                   weighting = c("var", "var"),
                   distmeasure = c("euclidean", "euclidean"))
  fit_n <- M_ABCpp(mosaic_toy$List, numsim = 60, NC = 3,
                   weighting = c("nugget", "nugget"),
                   distmeasure = c("euclidean", "euclidean"),
                   nugget_args = list(max_nuggets = 25, seed = 1))
  expect_gt(cluster_agreement(mosaic_labels(fit_v, 3), truth)[["ARI"]], 0.6)
  expect_gt(cluster_agreement(mosaic_labels(fit_n, 3), truth)[["ARI"]], 0.6)
})

test_that("ABCpp.SingleInMultiple runs every weighting scheme", {
  data(mosaic_toy)
  for (w in c("var", "cv", "nugget", "equal")) {
    lab <- ABCpp.SingleInMultiple(mosaic_toy$List[[1]], numsim = 20, NC = 3,
                                  weighting = w,
                                  nugget_args = list(max_nuggets = 20, seed = 1))
    expect_equal(dim(lab), c(20, 60))
  }
})

test_that("M_ABCdist fuses sources into a valid dissimilarity", {
  data(mosaic_toy)
  fit <- M_ABCdist(mosaic_toy$List, numsim = 40,
                   weighting = c("var", "nugget"))
  expect_equal(dim(fit$DistM), c(60, 60))
  expect_true(all(fit$DistM >= 0))
  expect_length(fit$source_D, 2)
})

test_that("M_ABCdist.WC fuses via WeightedClust", {
  data(mosaic_toy)
  fit <- suppressWarnings(M_ABCdist.WC(mosaic_toy$List, numsim = 30,
                      wc_weight = seq(1, 0, -0.25), wc_weightclust = 0.5))
  expect_equal(dim(fit$DistM), c(60, 60))
  expect_true(inherits(fit$Clust, c("agnes", "twins")))
})

test_that("compiled C++ kernels produce the same co-clustering counts as pure R", {
  cc <- getFromNamespace("count_coclustering_cpp", "MosaiClusteR")
  set.seed(7)
  res <- matrix(sample(0:3, 8 * 6, replace = TRUE), nrow = 8)  # R x N labels
  out <- cc(res)
  # pure-R reference
  N <- ncol(res); co <- matrix(0, N, N); nc <- matrix(0, N, N)
  for (i in 1:N) for (j in i:N) {
    both <- res[, i] != 0 & res[, j] != 0
    co[i, j] <- co[j, i] <- sum(both)
    nc[i, j] <- nc[j, i] <- sum(both & res[, i] != res[, j])
  }
  expect_equal(out$co_sel, co)
  expect_equal(out$not_co_clust, nc)
})

# Data-nugget weighting on the ABC engine (the .abc_zf internal + ABCpp).

test_that("internal .abc_zf returns a probability vector for every scheme", {
  zf <- getFromNamespace(".abc_zf", "MosaiClusteR")
  set.seed(1)
  X <- matrix(rnorm(40 * 15), nrow = 40)        # features x objects (internal)
  for (w in c("var", "cv", "nugget", "equal")) {
    p <- zf(X, w, nugget_args = list(max_nuggets = 8, seed = 1))
    expect_length(p, nrow(X))
    expect_equal(sum(p), 1, tolerance = 1e-8)
    expect_true(all(p >= 0))
  }
})

test_that("nugget weighting concentrates probability on informative features", {
  zf <- getFromNamespace(".abc_zf", "MosaiClusteR")
  set.seed(3)
  X <- matrix(rnorm(30 * 20), nrow = 30)        # 30 features x 20 objects
  X[1:5, 1:10] <- X[1:5, 1:10] + 5              # 5 informative features
  p <- zf(X, "nugget", nugget_args = list(max_nuggets = 6, seed = 3))
  expect_gt(mean(p[1:5]), mean(p[6:30]))
})

test_that("data-nugget weighting recovers structure through M_ABCpp", {
  data(mosaic_toy)
  fit <- M_ABCpp(mosaic_toy$List, numsim = 60, NC = 3,
                 weighting = c("nugget", "nugget"),
                 nugget_args = list(max_nuggets = 25, seed = 1))
  expect_gt(cluster_agreement(mosaic_labels(fit, 3), mosaic_toy$truth)[["ARI"]], 0.6)
})

test_that("Distance is symmetric with zero diagonal", {
  set.seed(1)
  m <- matrix(rnorm(8 * 5), nrow = 8)
  D <- Distance(m, "euclidean")
  expect_equal(dim(D), c(8, 8))
  expect_true(isSymmetric(unname(round(D, 10))))
  expect_equal(unname(diag(D)), rep(0, 8))

  b <- matrix(rbinom(8 * 6, 1, 0.5), nrow = 8)
  Dt <- Distance(b, "tanimoto")
  expect_true(all(Dt >= 0 & Dt <= 1))
})

test_that("Normalization range maps to [0,1]", {
  m <- matrix(rnorm(50), 10, 5)
  r <- Normalization(m, "Range")
  expect_gte(min(r), 0); expect_lte(max(r), 1)
  s <- Normalization(m, "S")
  expect_equal(unname(round(colMeans(s), 8)), rep(0, 5))
})

test_that("cluster_agreement is 1 for identical partitions up to relabelling", {
  a <- c(1, 1, 2, 2, 3, 3)
  b <- c(3, 3, 1, 1, 2, 2)
  ag <- cluster_agreement(a, b)
  expect_equal(unname(ag["ARI"]), 1)
  expect_equal(unname(ag["Jaccard"]), 1)
})

test_that("mosaic_labels extracts a partition of the right size", {
  data(mosaic_toy)
  fit <- M_ABCpp(mosaic_toy$List, numsim = 30, NC = 3)
  lab <- mosaic_labels(fit, k = 3)
  expect_length(lab, 60)
  expect_equal(length(unique(lab)), 3)
})

test_that("ABCdist.SingleInMultiple yields a scaled [0,1] dissimilarity", {
  data(mosaic_toy)
  out <- ABCdist.SingleInMultiple(mosaic_toy$List[[1]], numsim = 30)
  expect_true(all(out$D >= 0 & out$D <= 1))
  expect_equal(unname(diag(out$D)), rep(0, 60))
})

# =============================================================================
# ABCdist.R
# ABC variant using direct distance-matrix accumulation instead of label
# co-clustering. Steps 1-3 follow Amaratunga et al. (2008); steps 4-5 accumulate
# the mean scaled pairwise distance over co-selected objects.
# Main function: ABCdist.SingleInMultiple
# =============================================================================

#' Single-source ABC with distance accumulation
#'
#' A variant of \code{\link{ABCpp.SingleInMultiple}} that accumulates the mean
#' scaled pairwise distance between co-selected objects across iterations,
#' yielding a \code{[0, 1]} dissimilarity matrix that \code{\link{M_ABCdist}}
#' fuses across sources.
#'
#' @inheritParams ABCpp.SingleInMultiple
#' @return A list with \code{D} (\eqn{N \times N} dissimilarity), \code{NC} and
#'   \code{ng} (per-iteration feature count); if \code{mds = TRUE} also
#'   \code{mds_coords}.
#' @references \insertCite{Amaratunga2008}{MosaiClusteR}
#' @seealso \code{\link{M_ABCdist}}, \code{\link{ABCpp.SingleInMultiple}}
#' @examples
#' data(mosaic_toy)
#' out <- ABCdist.SingleInMultiple(mosaic_toy$List[[1]], numsim = 40)
#' dim(out$D)
#' @export
ABCdist.SingleInMultiple <- function(data,
                                     distmeasure = "euclidean",
                                     weighting   = "var",
                                     normalize   = NULL,
                                     gr          = c(),
                                     bag         = TRUE,
                                     numsim      = 1000,
                                     numvar      = 100,
                                     linkage     = "ward.D",
                                     NC          = NULL,
                                     mds         = FALSE,
                                     nfeat       = NULL,
                                     nugget_type = "between",
                                     nugget_args = list()) {
  data <- as.matrix(data)
  samp_names <- rownames(data)
  X  <- t(data)        # internal: features x objects
  ns <- ncol(X)
  nf <- nrow(X)

  if (is.null(NC)) NC <- max(2L, floor(sqrt(ns)))
  NC <- max(2L, min(as.integer(NC), ns - 1L))
  g  <- .abc_gsize(nf, nfeat)

  zf <- .abc_zf(X, weighting, nugget_type, nugget_args)

  D_sum <- matrix(0.0, ns, ns)
  D_cnt <- matrix(0L,  ns, ns)

  for (i in seq_len(numsim)) {
    if (i %% 100 == 0) message("ABCdist iteration ", i, " / ", numsim)
    ids <- if (bag) sort(unique(sample.int(ns, size = ns, replace = TRUE)))
           else      seq_len(ns)
    if (length(ids) < 2L) next
    idf <- sort(sample.int(nf, size = g, replace = FALSE, prob = zf))
    dat <- X[idf, ids, drop = FALSE]

    d_mat <- as.matrix(Distance(t(dat), distmeasure = distmeasure,
                                normalize = normalize))
    # robustness: Gower distance is NaN when a sampled feature is constant across
    # the selected objects -> fall back to Euclidean for this iteration.
    if (anyNA(d_mat) || any(!is.finite(d_mat))) d_mat <- as.matrix(stats::dist(t(dat)))
    lt   <- which(lower.tri(d_mat), arr.ind = TRUE)
    ii   <- ids[lt[, 1L]]; jj <- ids[lt[, 2L]]; vals <- d_mat[lt]
    D_sum[cbind(ii, jj)] <- D_sum[cbind(ii, jj)] + vals
    D_sum[cbind(jj, ii)] <- D_sum[cbind(jj, ii)] + vals
    D_cnt[cbind(ii, jj)] <- D_cnt[cbind(ii, jj)] + 1L
    D_cnt[cbind(jj, ii)] <- D_cnt[cbind(jj, ii)] + 1L
  }

  D_mean <- ifelse(D_cnt > 0L, D_sum / D_cnt, NA_real_)
  max_d  <- max(D_mean, na.rm = TRUE)
  if (!is.finite(max_d) || max_d <= 0) max_d <- 1
  D_mean[is.na(D_mean)] <- max_d
  D_mat <- D_mean / max_d
  diag(D_mat) <- 0.0
  rownames(D_mat) <- colnames(D_mat) <- samp_names

  out <- list(D = D_mat, NC = NC, ng = g)
  if (isTRUE(mds)) {
    fit <- stats::cmdscale(stats::as.dist(D_mat), k = 2)
    colnames(fit) <- c("Dim1", "Dim2"); out$mds_coords <- fit
  }
  out
}

#' @title Link-based cluster ensembles
#'
#' @description Link-based cluster ensembles combine multiple base clusterings
#' into a single, refined similarity matrix by exploiting the link structure
#' between clusters of the ensemble. Three refinement schemes are available:
#' Connected-Triple-Based Similarity ("cts"), SimRank-Based Similarity ("srs")
#' and Approximate SimRank-Based Similarity ("asrs"). The refined object-by-object
#' similarity matrix is turned into a dissimilarity matrix on which a final
#' agglomerative hierarchical clustering is performed.
#' @export
#' @param List A list of data matrices, distance matrices or clustering results.
#'   It is assumed the rows are corresponding with the objects.
#' @param type Indicates whether the provided matrices in "List" are either data
#'   matrices, distance matrices or clustering results obtained from the data.
#'   Should be one of "data", "dist" or "clust".
#' @param distmeasure A vector of the distance measures to be used on each data
#'   matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming".
#'   Defaults to c("tanimoto","tanimoto").
#' @param normalize Logical. Indicates whether to normalize the distance matrices
#'   or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if
#'   different distance types are used. More details on normalization in
#'   \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile",
#'   "Fisher-Yates", "standardize","Range" or any of the first letters of these
#'   names. Default is c(NULL,NULL) for two data sets.
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character) for each data
#'   set. Defaults to c("flexible", "flexible") for two data sets.
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the
#'   agnes function. Defaults to 0.625 and is only used if the linkage is set to
#'   "flexible".
#' @param nrclusters A vector with the number of clusters to cut each base
#'   dendrogram into. If NULL and gap=FALSE, the function stops.
#' @param gap Logical. Whether to use the gap statistic to determine the number
#'   of clusters per data set. Defaults to FALSE.
#' @param maxK The maximal number of clusters to consider when gap=TRUE.
#' @param linkBasedMethod The link-based refinement scheme. One of "cts", "srs"
#'   or "asrs". Defaults to "cts".
#' @param decayfactor The decay (confidence) factor used by the refinement scheme.
#'   Defaults to 0.8.
#' @param niter The number of iterations for the SimRank-based scheme ("srs").
#'   Defaults to 5.
#' @param linkBasedLinkage The linkage used in the final agglomerative clustering
#'   of the refined similarity matrix. Defaults to "ward".
#' @param waitingtime Retained for backward compatibility with the external
#'   executable / MATLAB workflow; ignored by the pure-R schemes.
#' @param file_number Retained for backward compatibility with the external
#'   executable / MATLAB workflow; ignored by the pure-R schemes.
#' @param executable Logical. If TRUE, the refined similarity matrix is computed
#'   by an external executable / MATLAB. The "cts" and "srs" schemes are pure R
#'   and work without an executable. The "asrs" scheme is only available through
#'   the external executable; if executable=FALSE it stops with an informative
#'   message.
#' @return The returned value is a list of two elements:
#' \item{DistM}{The resulting (refined) distance matrix}
#' \item{Clust}{The resulting hierarchical clustering}
#' The value has class 'LinkBased'.
#' @details Each base clustering of the ensemble is represented as a hard
#' partition. The clusters of all partitions form the vertices of a weighted
#' graph; the link weights between clusters are derived from shared objects. The
#' refined object-by-object similarity is obtained by propagating these
#' cluster-to-cluster links ("cts", "srs", "asrs"). See Iam-On et al. (2010).
#' @references
#' Iam-On, N., Boongoen, T., Garrett, S. and Price, C. (2010). Link-based cluster
#' ensembles for the combination of multiple clusterings. IEEE Transactions on
#' Knowledge and Data Engineering, 23(12), 1839-1853.
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' L <- mosaic_toy$List
#' res <- LinkBasedClustering(List = L, type = "data",
#'   distmeasure = c("euclidean", "euclidean"), normalize = c(FALSE, FALSE),
#'   linkBasedMethod = "cts", decayfactor = 0.8, niter = 5,
#'   linkBasedLinkage = "ward", nrclusters = c(3, 3))
#' }
LinkBasedClustering<-function(List,type=c("data","dist","clust"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),clust = "agnes", linkage = c("flexible","flexible"),alpha=0.625,nrclusters=c(7,7),gap = FALSE, maxK = 15,linkBasedMethod=c("cts","srs","asrs"),decayfactor=0.8,niter=5,linkBasedLinkage="ward",waitingtime=300,file_number=00,executable=FALSE){

	linkBasedMethod <- match.arg(linkBasedMethod)

	# Guard: the "asrs" scheme is only available through the external
	# executable / MATLAB. Refuse early if it is requested without one.
	if(linkBasedMethod=="asrs" && !executable){
		stop("The 'asrs' (Approximate SimRank-Based Similarity) scheme requires an external executable or MATLAB. Set executable=TRUE and make the executable available, or use linkBasedMethod='cts' or 'srs' (pure R).")
	}

	# Step 1: Generate several clustering results on the objects
	if(type=="data"){

		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type="data",distmeasure[i],normalize[i],method[i],clust,linkage[i],alpha,gap,maxK,StopRange=TRUE))

		Dist=lapply(seq(length(List)),function(i) Clusterings[[i]]$DistM)

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=clusters
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))
	}

	else if(type=="dist"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,OrderNames]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type,distmeasure[i],normalize=FALSE,method=NULL,clust,linkage[i],alpha,gap,maxK,StopRange=TRUE))

		Dist=List

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=clusters
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))


	}
	else if(type=="clust"){

		Clusterings=List

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		if(is.null(nrclusters)){
			stop("Please specify a number of clusters")
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))

	}
	#### Step 1 Complete

	# Step 2: Get the ensemble in matrix format: one row per clustering result,
	# one column per object.
	if(type=="data" | type=="dist"){
		nc=nrow(List[[1]])
	}
	else if(type=="clust"){
		nc=nrow(List[[1]]$DistM)
	}
	matlabdata=t(matrix(unlist(Clusters),ncol=nc,byrow=TRUE))

	if(executable){
		# Legacy path: hand the ensemble to an external executable / MATLAB,
		# which writes the refined similarity matrix to S_<file_number>.csv.
		utils::write.table(matlabdata,file=paste("matlabdata_",file_number,".csv",sep=""),sep=",",col.names=FALSE,row.names=FALSE)

		if(linkBasedMethod=="cts"){
			system(paste("./LinkBasedClusteringcts ",file_number,sep=""),intern=TRUE)
		}
		if(linkBasedMethod=="srs"){
			system(paste("./LinkBasedClusteringsrs ",file_number,sep=""),intern=TRUE)
		}
		if(linkBasedMethod=="asrs"){
			system(paste("./LinkBasedClusteringasrs ",file_number,sep=""),intern=TRUE)
		}

		Continue=FALSE
		time=0
		while(Continue==FALSE){
			Sys.sleep(15)
			time=time+15
			Continue=file.exists(paste("S_",file_number,".csv",sep=""))
			if(time>waitingtime & Continue==FALSE){
				stop(paste("Waited",waitingtime, "seconds for completion of the ensemble clustering procedure. Increase waiting time to continue.",sep=" "))
			}
		}

		SimM=as.matrix(utils::read.table(paste("S_",file_number,".csv",sep=""),sep=","))

		file.remove(paste("matlabdata_",file_number,".csv",sep=""))
		file.remove(paste("S_",file_number,".csv",sep=""))
	}
	else{
		# Pure-R path: compute the refined object-by-object similarity matrix
		# directly. matlabdata has one column per base clustering (objects in rows).
		SimM=.linkBasedSimilarity(matlabdata,linkBasedMethod=linkBasedMethod,decayfactor=decayfactor,niter=niter)
	}

	SimM=as.matrix(SimM)
	DistM=as.matrix(1-SimM)
	rownames(DistM)=rownames(Clusterings[[1]]$DistM)
	colnames(DistM)=rownames(Clusterings[[1]]$DistM)

	LinkBasedCluster=cluster::agnes(DistM,diss=TRUE,method=linkBasedLinkage,par.method=alpha)

	Out=list("DistM"=DistM,"Clust"=LinkBasedCluster)
	attr(Out,"method")="LinkBased"

	return(Out)
}


# -----------------------------------------------------------------------------
# Internal helpers: pure-R link-based similarity schemes (Iam-On et al., 2010)
# -----------------------------------------------------------------------------

# Compute the refined object-by-object similarity matrix from an ensemble.
# Ensemble: a matrix with one row per object and one column per base clustering;
# entries are (arbitrary) cluster labels.
.linkBasedSimilarity <- function(Ensemble, linkBasedMethod = c("cts", "srs", "asrs"),
                                 decayfactor = 0.8, niter = 5) {
	linkBasedMethod <- match.arg(linkBasedMethod)
	Ensemble <- as.matrix(Ensemble)
	N <- nrow(Ensemble)   # number of objects
	M <- ncol(Ensemble)   # number of base clusterings

	# Build a binary object-by-cluster membership matrix B (N x sum_of_clusters)
	# and record, for each global cluster, which base clustering it belongs to.
	blocks <- vector("list", M)
	partOf <- integer(0)
	for (m in seq_len(M)) {
		labs <- Ensemble[, m]
		ul <- sort(unique(labs))
		Bm <- matrix(0, N, length(ul))
		for (j in seq_along(ul)) Bm[, j] <- as.numeric(labs == ul[j])
		blocks[[m]] <- Bm
		partOf <- c(partOf, rep(m, length(ul)))
	}
	B <- do.call(cbind, blocks)   # N x C
	C <- ncol(B)

	# Cluster-to-cluster shared-object overlap (used by CTS).
	# Compute cluster-to-cluster similarity S_cluster (C x C).
	if (linkBasedMethod == "cts") {
		S_cluster <- .ctsClusterSim(B, partOf, decayfactor)
	} else { # srs (and asrs falls back here only when forced, but is guarded out)
		S_cluster <- .srsClusterSim(B, decayfactor, niter)
	}

	# Refined object similarity: average cluster similarity of the clusters that
	# each pair of objects belongs to, across the M base clusterings.
	# membership index of object i in base clustering m -> column of B.
	# Build, per object, the list of its C-space cluster column for each m.
	colOf <- matrix(0L, N, M)
	offset <- 0L
	for (m in seq_len(M)) {
		Bm <- blocks[[m]]
		# column (within block) that each object belongs to
		within <- max.col(Bm, ties.method = "first")
		colOf[, m] <- offset + within
		offset <- offset + ncol(Bm)
	}

	Sim <- matrix(0, N, N)
	for (m in seq_len(M)) {
		idx <- colOf[, m]
		Sim <- Sim + S_cluster[idx, idx, drop = FALSE]
	}
	Sim <- Sim / M
	diag(Sim) <- 1
	rownames(Sim) <- rownames(Ensemble)
	colnames(Sim) <- rownames(Ensemble)
	Sim
}

# Connected-Triple-Based Similarity between clusters.
# B: N x C binary membership matrix; partOf: base-clustering index per cluster.
.ctsClusterSim <- function(B, partOf, decayfactor) {
	C <- ncol(B)
	# weighted graph of clusters: edge weight = number of shared objects (only
	# meaningful across different base clusterings).
	W <- crossprod(B)           # C x C, W[p,q] = #objects shared by clusters p,q
	diag(W) <- 0
	# zero out edges within the same base clustering (clusters of one partition
	# never share objects anyway, but keep it explicit).
	same <- outer(partOf, partOf, FUN = "==")
	W[same] <- 0

	# Connected-triple count between two clusters via common neighbours.
	wMax <- max(W)
	if (wMax == 0) {
		S <- diag(1, C)
		return(S)
	}
	# Number of connected triples shared: for clusters i,j the sum over common
	# neighbour k of min(W[i,k], W[k,j]) (simple weighted triple count).
	S <- diag(1, C)
	for (i in seq_len(C)) {
		for (j in seq_len(C)) {
			if (i < j) {
				wc <- sum(pmin(W[i, ], W[, j]))
				S[i, j] <- wc
				S[j, i] <- wc
			}
		}
	}
	maxS <- max(S[upper.tri(S)])
	if (maxS > 0) S <- S / maxS * decayfactor
	diag(S) <- 1
	S
}

# SimRank-Based Similarity between clusters (iterative).
# B: N x C binary membership matrix.
.srsClusterSim <- function(B, decayfactor, niter) {
	C <- ncol(B)
	# bipartite weighted adjacency cluster<->object: B is N x C, so the
	# cluster-to-object weights are columns of B.
	# SimRank on the cluster side, where the "neighbours" of a cluster are its
	# member objects, and the neighbours of an object are its clusters.
	# Normalised neighbour weights.
	clusterDeg <- colSums(B)               # number of objects per cluster
	objectDeg  <- rowSums(B)               # number of clusters per object
	clusterDeg[clusterDeg == 0] <- 1
	objectDeg[objectDeg == 0]   <- 1

	# Transition: from cluster to object (C x N), row-normalised
	P_co <- t(B) / clusterDeg              # C x N
	# Transition: from object to cluster (N x C), row-normalised
	P_oc <- B / objectDeg                  # N x C

	Sc <- diag(1, C)                       # cluster-cluster similarity
	So <- diag(1, nrow(B))                 # object-object similarity
	for (it in seq_len(niter)) {
		Sc_new <- decayfactor * (P_co %*% So %*% t(P_co))
		So_new <- decayfactor * (P_oc %*% Sc %*% t(P_oc))
		diag(Sc_new) <- 1
		diag(So_new) <- 1
		Sc <- Sc_new
		So <- So_new
	}
	maxS <- max(Sc[upper.tri(Sc)])
	if (is.finite(maxS) && maxS > 0) {
		offdiag <- Sc
		diag(offdiag) <- 0
		Sc <- offdiag / maxS * decayfactor
	} else {
		Sc <- matrix(0, C, C)
	}
	diag(Sc) <- 1
	Sc
}

test_that("LinkBasedClustering CTS (pure R) runs on mosaic_toy", {
  data(mosaic_toy)
  L <- mosaic_toy$List

  res <- LinkBasedClustering(
    List = L, type = "data",
    distmeasure = c("euclidean", "euclidean"),
    normalize = c(FALSE, FALSE),
    linkBasedMethod = "cts", decayfactor = 0.8, niter = 5,
    linkBasedLinkage = "ward", nrclusters = c(3, 3)
  )

  expect_type(res, "list")
  expect_equal(attr(res, "method"), "LinkBased")
  expect_true(all(c("DistM", "Clust") %in% names(res)))
  expect_equal(dim(res$DistM), c(nrow(L[[1]]), nrow(L[[1]])))
  expect_s3_class(res$Clust, "agnes")

  labs <- stats::cutree(res$Clust, k = 3)
  expect_length(labs, nrow(L[[1]]))
  expect_equal(length(unique(labs)), 3)
})

test_that("LinkBasedClustering SRS (pure R) runs on mosaic_toy", {
  data(mosaic_toy)
  L <- mosaic_toy$List

  res <- LinkBasedClustering(
    List = L, type = "data",
    distmeasure = c("euclidean", "euclidean"),
    normalize = c(FALSE, FALSE),
    linkBasedMethod = "srs", decayfactor = 0.8, niter = 5,
    linkBasedLinkage = "ward", nrclusters = c(3, 3)
  )

  expect_equal(attr(res, "method"), "LinkBased")
  expect_equal(dim(res$DistM), c(nrow(L[[1]]), nrow(L[[1]])))
  expect_s3_class(res$Clust, "agnes")
})

test_that("LinkBasedClustering ASRS requires an executable", {
  data(mosaic_toy)
  L <- mosaic_toy$List

  expect_error(
    LinkBasedClustering(
      List = L, type = "data",
      distmeasure = c("euclidean", "euclidean"),
      linkBasedMethod = "asrs", nrclusters = c(3, 3),
      executable = FALSE
    ),
    "asrs"
  )
})

# =============================================================================
# intnmf.R  - integrative Non-negative Matrix Factorisation (intNMF)
# Concatenated low-rank integration: a shared basis W (objects x r) and per-omic
# coefficient matrices H_k are found so that X_k ~ W H_k for all sources, via
# multiplicative updates. Objects are clustered from W by consensus over
# restarts. (Chalise & Fridley 2017; review: Zhang et al. 2022.)
# =============================================================================

#' Integrative NMF clustering (intNMF)
#'
#' Joint non-negative matrix factorisation across data sources: a single shared
#' basis matrix \eqn{W} (objects \eqn{\times} rank) and source-specific
#' coefficient matrices \eqn{H_k} minimise \eqn{\sum_k \alpha_k \lVert X_k - W
#' H_k \rVert_F^2} with all factors non-negative. Sources are weighted by
#' \eqn{\alpha_k = \max_j \overline{SS}_j / \overline{SS}_k}. Object clusters are
#' obtained by consensus of the dominant-factor assignment across restarts.
#'
#' @param List A list of data matrices over the same objects, \strong{objects in
#'   rows}. Matrices are shifted to be non-negative internally.
#' @param k Number of clusters (also the factorisation rank).
#' @param nstart Number of random restarts forming the consensus.
#' @param max_iter Maximum multiplicative-update iterations per restart.
#' @param seed Optional RNG seed.
#' @return A list of class \code{"intNMF"} with \code{cluster}, the consensus
#'   \code{DistM}, an \code{agnes} \code{Clust}, the best-fit basis \code{W} and
#'   source weights \code{alpha}.
#' @references Chalise, P. & Fridley, B. L. (2017). Integrative clustering of
#'   multi-level omic data based on non-negative matrix factorization. PLoS ONE
#'   12:e0176278. Reviewed in Zhang et al. (2022), WIREs Comput Stat 14:e1553.
#' @seealso \code{\link{M_ABCpp}}, \code{\link{SNF}}
#' @examples
#' data(mosaic_toy)
#' fit <- intNMF(mosaic_toy$List, k = 3, nstart = 5, seed = 1)
#' table(fit$cluster)
#' @export
intNMF <- function(List, k, nstart = 10, max_iter = 200, seed = NULL) {
  if (!is.list(List)) stop("'List' must be a list of data matrices.")
  if (!is.null(seed)) set.seed(seed)
  Xs <- lapply(List, function(X) {
    X <- as.matrix(X); X <- X - min(X, na.rm = TRUE); X[is.na(X)] <- 0; X })
  n  <- nrow(Xs[[1]])
  r  <- max(2L, as.integer(k))
  eps <- .Machine$double.eps

  ssm   <- vapply(Xs, function(X) mean(X^2), numeric(1))
  alpha <- max(ssm) / pmax(ssm, eps)

  fit_one <- function() {
    W  <- matrix(stats::runif(n * r), n, r)
    Hs <- lapply(Xs, function(X) matrix(stats::runif(r * ncol(X)), r, ncol(X)))
    for (it in seq_len(max_iter)) {
      for (kk in seq_along(Xs)) {
        WtW <- crossprod(W)
        Hs[[kk]] <- Hs[[kk]] * (crossprod(W, Xs[[kk]]) /
                                  pmax(WtW %*% Hs[[kk]], eps))
      }
      num <- matrix(0, n, r); den <- matrix(0, n, r)
      for (kk in seq_along(Xs)) {
        num <- num + alpha[kk] * tcrossprod(Xs[[kk]], Hs[[kk]])
        den <- den + alpha[kk] * W %*% tcrossprod(Hs[[kk]])
      }
      W <- W * (num / pmax(den, eps))
    }
    err <- sum(vapply(seq_along(Xs), function(kk)
      alpha[kk] * sum((Xs[[kk]] - W %*% Hs[[kk]])^2), numeric(1)))
    list(W = W, err = err, lab = max.col(W, ties.method = "first"))
  }

  CO <- matrix(0, n, n); best <- NULL; best_err <- Inf
  for (s in seq_len(nstart)) {
    f <- fit_one()
    CO <- CO + outer(f$lab, f$lab, "==")
    if (f$err < best_err) { best_err <- f$err; best <- f }
  }
  CO <- CO / nstart
  D  <- 1 - CO; diag(D) <- 0
  rownames(D) <- colnames(D) <- rownames(Xs[[1]])
  Clust <- cluster::agnes(D, diss = TRUE, method = "ward")
  cl <- stats::setNames(stats::cutree(stats::as.hclust(Clust), k = r),
                        rownames(Xs[[1]]))

  out <- list(cluster = cl, DistM = D, Clust = Clust,
              W = best$W, alpha = alpha, k = r)
  attr(out, "method") <- "intNMF"
  class(out) <- c("intNMF", "list")
  out
}

# =============================================================================
# nugget-clustering.R
# Weighted clustering of (data-nugget) representatives - the clustering side of
# the data-nugget toolkit (WCluster: Dey, Duan, Cabrera & Cheng 2025;
# Cherasia et al. 2023). Weighted k-means minimises the Weighted Within-Cluster
# Sum of Squares (WWCSS); weighted hierarchical clustering uses the weighted
# Ward merge cost SS(A union B) - SS(A) - SS(B).
# =============================================================================

#' Weighted k-means (minimises WWCSS)
#'
#' k-means in which each observation carries a non-negative weight. Cluster
#' centres are weighted means and the objective is the Weighted Within-Cluster
#' Sum of Squares \eqn{\sum_c \sum_{i \in c} w_i \lVert x_i - \mu_c \rVert^2}.
#'
#' @param x Numeric matrix, observations in rows.
#' @param k Number of clusters.
#' @param weights Non-negative weight per observation (default all 1).
#' @param nstart Number of random restarts (best WWCSS kept).
#' @param max_iter Maximum Lloyd iterations per start.
#' @return A list with \code{cluster}, \code{centers} and \code{wwcss}.
#' @references Dey, T., Duan, Y., Cabrera, J. & Cheng, X. (2025). WCluster:
#'   Clustering and PCA with Weights, and Data Nuggets Clustering.
#' @seealso \code{\link{nugget_cluster}}, \code{\link{Whclust}}
#' @examples
#' set.seed(1)
#' x <- rbind(matrix(rnorm(20 * 3, 0), 20), matrix(rnorm(20 * 3, 5), 20))
#' Wkmeans(x, k = 2, weights = runif(40, 1, 5))$cluster[1:5]
#' @export
Wkmeans <- function(x, k, weights = rep(1, nrow(x)), nstart = 10, max_iter = 100) {
  x <- as.matrix(x); n <- nrow(x)
  w <- as.numeric(weights); w[w < 0] <- 0
  k <- max(1L, min(as.integer(k), n))

  wmeans <- function(rows) {
    if (!length(rows)) return(NULL)
    colSums(x[rows, , drop = FALSE] * w[rows]) / sum(w[rows])
  }
  best <- NULL; best_obj <- Inf
  for (s in seq_len(nstart)) {
    centers <- x[sample.int(n, k, prob = w / sum(w)), , drop = FALSE]
    cl <- integer(n)
    for (it in seq_len(max_iter)) {
      D  <- as.matrix(stats::dist(rbind(centers, x)))[seq_len(k), -seq_len(k), drop = FALSE]
      new <- max.col(-t(D), ties.method = "first")
      if (identical(new, cl)) break
      cl <- new
      for (c in seq_len(k)) {
        m <- wmeans(which(cl == c))
        if (!is.null(m)) centers[c, ] <- m
      }
    }
    obj <- sum(vapply(seq_len(k), function(c) {
      rows <- which(cl == c); if (!length(rows)) return(0)
      sum(w[rows] * rowSums(sweep(x[rows, , drop = FALSE], 2, centers[c, ])^2))
    }, numeric(1)))
    if (obj < best_obj) { best_obj <- obj; best <- list(cluster = cl, centers = centers) }
  }
  list(cluster = stats::setNames(best$cluster, rownames(x)),
       centers = best$centers, wwcss = best_obj)
}

# weighted Ward agglomeration -> hclust object (centres in rows, weights w)
.weighted_ward <- function(C, w) {
  C <- as.matrix(C); m <- nrow(C)
  active <- seq_len(m); wt <- w; cen <- C
  id <- -seq_len(m)                       # hclust leaf ids are negative
  merge <- matrix(0L, m - 1L, 2L); height <- numeric(m - 1L)
  ward <- function(a, b)
    (wt[a] * wt[b] / (wt[a] + wt[b])) * sum((cen[a, ] - cen[b, ])^2)
  for (step in seq_len(m - 1L)) {
    best <- Inf; bi <- bj <- NA
    for (ii in seq_along(active)) for (jj in seq_len(ii - 1L)) {
      a <- active[ii]; b <- active[jj]; d <- ward(a, b)
      if (d < best) { best <- d; bi <- a; bj <- b }
    }
    height[step] <- best
    merge[step, ] <- sort(c(id[bi], id[bj]))
    nw  <- wt[bi] + wt[bj]
    cen[bi, ] <- (wt[bi] * cen[bi, ] + wt[bj] * cen[bj, ]) / nw
    wt[bi] <- nw; id[bi] <- step
    active <- setdiff(active, bj)
  }
  ord <- stats::hclust(stats::dist(C), method = "ward.D2")$order  # plausible leaf order
  structure(list(merge = merge, height = cumsum(sort(height)) / seq_along(height),
                 order = ord, labels = rownames(C), method = "weighted.ward",
                 dist.method = "euclidean"), class = "hclust")
}

#' Weighted hierarchical clustering (weighted Ward)
#'
#' Agglomerative clustering of weighted observations using the weighted Ward
#' merge cost. Intended for the small set of data-nugget centres.
#'
#' @param x Numeric matrix, observations (e.g. nugget centres) in rows.
#' @param weights Non-negative weight per observation.
#' @return An \code{\link[stats]{hclust}} object.
#' @references Dey et al. (2025). WCluster.
#' @seealso \code{\link{Wkmeans}}, \code{\link{nugget_cluster}}
#' @examples
#' set.seed(1)
#' x <- rbind(matrix(rnorm(8 * 3, 0), 8), matrix(rnorm(8 * 3, 5), 8))
#' stats::cutree(Whclust(x, weights = rep(2, 16)), k = 2)[1:4]
#' @export
Whclust <- function(x, weights = rep(1, nrow(x))) {
  x <- as.matrix(x)
  if (is.null(rownames(x))) rownames(x) <- as.character(seq_len(nrow(x)))
  .weighted_ward(x, as.numeric(weights))
}

#' Data-nugget clustering
#'
#' Cluster a (potentially large) data set by first compressing its observations
#' into weighted data nuggets (\code{\link{create_data_nuggets}}), clustering the
#' nugget centres with a weighted clusterer, and mapping the nugget labels back
#' to the original observations. This is the clustering analogue of the
#' data-nugget weighting used by \code{\link{M_ABCpp}} and follows
#' \code{DN.Wkmeans} / \code{DN.Whclust} from the WCluster package.
#'
#' @param data A numeric matrix (objects in rows) \emph{or} a precomputed
#'   \code{\link{create_data_nuggets}{data_nugget}} object. For multi-source
#'   data, column-bind the per-source matrices first (the direct/\code{ADC}
#'   strategy) or pass a single fused matrix.
#' @param k Number of clusters.
#' @param method Weighted clusterer for the nugget centres: \code{"kmeans"}
#'   (WWCSS) or \code{"ward"} (weighted Ward).
#' @param max_nuggets,nugget_args Passed to \code{\link{create_data_nuggets}}
#'   when \code{data} is a raw matrix.
#' @return A list of class \code{"nugget_cluster"} with \code{cluster} (labels
#'   for the original objects), \code{nugget_cluster} (labels for the nuggets)
#'   and the \code{nuggets} object.
#' @references Cherasia et al. (2023); Dey et al. (2025), WCluster.
#' @seealso \code{\link{create_data_nuggets}}, \code{\link{Wkmeans}}
#' @examples
#' data(mosaic_toy)
#' X <- do.call(cbind, mosaic_toy$List)        # fuse sources (objects in rows)
#' fit <- nugget_cluster(X, k = 3, max_nuggets = 30)
#' table(fit$cluster)
#' @export
nugget_cluster <- function(data, k, method = c("kmeans", "ward"),
                           max_nuggets = NULL, nugget_args = list()) {
  method <- match.arg(method)
  dn <- if (inherits(data, "data_nugget")) data
        else do.call(create_data_nuggets,
                     c(list(x = as.matrix(data), max_nuggets = max_nuggets), nugget_args))
  if (is.null(dn$membership))
    stop("nugget_cluster needs nugget membership; build nuggets with the ",
         "native engine (create_data_nuggets(..., engine='native')).")

  nlab <- if (method == "kmeans")
    Wkmeans(dn$centers, k = k, weights = dn$weights)$cluster
  else
    stats::cutree(Whclust(dn$centers, weights = dn$weights), k = k)

  obj_lab <- nlab[dn$membership]
  if (inherits(data, "data_nugget")) names(obj_lab) <- NULL
  else names(obj_lab) <- rownames(as.matrix(data))

  out <- list(cluster = obj_lab, nugget_cluster = nlab, nuggets = dn,
              k = k, method = method)
  attr(out, "method") <- paste0("DN.", method)
  class(out) <- c("nugget_cluster", "list")
  out
}

# Tests for the reference-derived methods: spectral, NEMO, data-nugget
# clustering, intNMF and the LUCID wrapper.

test_that("spectral_clustering separates two well-separated blobs", {
  set.seed(1)
  X <- rbind(matrix(rnorm(25 * 4, 0), 25), matrix(rnorm(25 * 4, 8), 25))
  A <- exp(-as.matrix(dist(X))^2 / 4)
  sc <- spectral_clustering(A, k = 2)
  expect_length(sc$cluster, 50)
  expect_equal(cluster_agreement(sc$cluster, rep(1:2, each = 25))[["ARI"]], 1)
})

test_that("NEMO returns a fused affinity and recovers planted structure", {
  data(mosaic_toy)
  fit <- NEMO(mosaic_toy$List, k = 3, NN = 15)
  expect_true(all(c("FusedM", "DistM", "cluster", "Clust") %in% names(fit)))
  expect_equal(dim(fit$FusedM), c(60, 60))
  expect_gt(cluster_agreement(fit$cluster, mosaic_toy$truth)[["ARI"]], 0.6)
})

test_that("NEMO tolerates partial data (a source missing some objects)", {
  data(mosaic_toy)
  L <- mosaic_toy$List
  L[[2]] <- L[[2]][1:50, ]                 # source 2 covers only 50/60 objects
  fit <- NEMO(L, k = 3, NN = 12)
  expect_equal(nrow(fit$FusedM), 60)       # union of objects
})

test_that("Wkmeans minimises WWCSS and recovers blobs", {
  set.seed(2)
  x <- rbind(matrix(rnorm(30 * 3, 0), 30), matrix(rnorm(30 * 3, 6), 30))
  w <- runif(60, 1, 4)
  km <- Wkmeans(x, k = 2, weights = w)
  expect_equal(cluster_agreement(km$cluster, rep(1:2, each = 30))[["ARI"]], 1)
  expect_true(is.finite(km$wwcss))
})

test_that("Whclust returns a valid hclust over weighted points", {
  set.seed(3)
  x <- rbind(matrix(rnorm(8 * 3, 0), 8), matrix(rnorm(8 * 3, 7), 8))
  hc <- Whclust(x, weights = rep(3, 16))
  expect_s3_class(hc, "hclust")
  expect_equal(cluster_agreement(stats::cutree(hc, 2), rep(1:2, each = 8))[["ARI"]], 1)
})

test_that("nugget_cluster compresses then clusters back to all objects", {
  data(mosaic_toy)
  X <- do.call(cbind, mosaic_toy$List)     # fuse sources, objects in rows
  fit <- nugget_cluster(X, k = 3, max_nuggets = 30)
  expect_length(fit$cluster, 60)
  expect_gt(cluster_agreement(fit$cluster, mosaic_toy$truth)[["ARI"]], 0.6)
})

test_that("intNMF integrates sources and recovers structure", {
  data(mosaic_toy)
  fit <- intNMF(mosaic_toy$List, k = 3, nstart = 5, seed = 1)
  expect_equal(ncol(fit$W), 3)
  expect_equal(dim(fit$DistM), c(60, 60))
  expect_gt(cluster_agreement(fit$cluster, mosaic_toy$truth)[["ARI"]], 0.5)
})

test_that("LUCID is exported and guards the missing LUCIDus package", {
  expect_true(is.function(LUCID))
  skip_if(requireNamespace("LUCIDus", quietly = TRUE),
          "LUCIDus installed; guard not exercised")
  data(mosaic_toy)
  expect_error(
    LUCID(G = mosaic_toy$List[[2]][, 1:3], Z = mosaic_toy$List[[1]],
          Y = as.numeric(mosaic_toy$truth), K = 3),
    "LUCIDus")
})

# =============================================================================
# M_ABCdist.R
# Multi-source M-ABC, distance-accumulation variant. Each source yields a
# [0,1]-scaled mean-distance matrix from ABCdist.SingleInMultiple; these are
# combined as a weighted mean D_multi = sum_k w_k D_k and clustered.
# Function: M_ABCdist
# =============================================================================

#' Multi-source M-ABC with distance accumulation
#'
#' Runs \code{\link{ABCdist.SingleInMultiple}} on each source to obtain a
#' \code{[0, 1]} dissimilarity, combines them as a weighted mean
#' \eqn{D = \sum_k w_k D_k} and applies a final agglomerative clustering.
#'
#' @inheritParams M_ABCpp
#' @param source_weights Optional non-negative source weights (normalised to sum
#'   to one); \code{NULL} = equal weights.
#' @param NC2 Optional number of clusters to cut the final tree into.
#' @return A list of class \code{"M_ABCdist"} with \code{source_D}, \code{DistM},
#'   \code{weights} and \code{Clust}.
#' @references \insertCite{Amaratunga2008}{MosaiClusteR}
#' @seealso \code{\link{M_ABCpp}}, \code{\link{ABCdist.SingleInMultiple}},
#'   \code{\link{M_ABCdist.WC}}
#' @examples
#' data(mosaic_toy)
#' fit <- M_ABCdist(mosaic_toy$List, numsim = 40, weighting = c("var", "nugget"))
#' dim(fit$DistM)
#' @export
M_ABCdist <- function(List,
                      source_weights = NULL,
                      distmeasure    = rep("euclidean", length(List)),
                      weighting      = rep("var",       length(List)),
                      normalize      = rep(list(NULL),  length(List)),
                      gr             = c(),
                      bag            = rep(TRUE,        length(List)),
                      numsim         = 1000L,
                      numvar         = rep(100L,        length(List)),
                      linkage        = rep("ward.D",    length(List)),
                      NC             = NULL,
                      NC2            = NULL,
                      final_linkage  = "ward",
                      alpha          = 0.625,
                      mds            = FALSE,
                      nfeat          = NULL,
                      nugget_type    = "between",
                      nugget_args    = list()) {
  if (!is.list(List)) stop("List must contain at least one data source.")
  K <- length(List)
  .rec <- function(x, nm) rep_len(x, K)
  distmeasure <- .rec(distmeasure); weighting <- .rec(weighting)
  bag <- as.logical(.rec(bag)); numvar <- as.integer(.rec(numvar))
  linkage <- .rec(linkage)
  if (!is.list(normalize)) normalize <- as.list(normalize)
  normalize <- rep_len(normalize, K)
  if (length(nugget_args) && all(vapply(nugget_args, is.list, logical(1))))
    nugget_args <- rep_len(nugget_args, K) else nugget_args <- rep(list(nugget_args), K)

  if (is.null(source_weights)) source_weights <- rep(1.0 / K, K)
  else {
    if (length(source_weights) != K) stop("source_weights must have length ", K, ".")
    source_weights <- source_weights / sum(source_weights)
  }

  List <- lapply(List, as.matrix)
  samp_names <- rownames(List[[1]]); N <- nrow(List[[1]])
  for (k in seq_len(K))
    if (nrow(List[[k]]) != N)
      stop("Source ", k, " has ", nrow(List[[k]]), " rows; expected ", N, ".")

  source_D <- vector("list", K)
  for (k in seq_len(K)) {
    message("M_ABCdist: source ", k, " of ", K,
            " (weight=", round(source_weights[k], 4), ", weighting=", weighting[k], ") ...")
    source_D[[k]] <- ABCdist.SingleInMultiple(
      data = List[[k]], distmeasure = distmeasure[k], weighting = weighting[k],
      normalize = normalize[[k]], gr = gr, bag = bag[k], numsim = as.integer(numsim),
      numvar = numvar[k], linkage = linkage[k], NC = NC, nfeat = nfeat,
      nugget_type = nugget_type, nugget_args = nugget_args[[k]])$D
  }

  D_multi <- Reduce("+", Map("*", source_weights, source_D))
  D_multi <- matrix(as.double(D_multi), N, N); diag(D_multi) <- 0
  rownames(D_multi) <- colnames(D_multi) <- samp_names

  if (isTRUE(mds)) {
    co <- stats::cmdscale(stats::as.dist(D_multi), k = 2L)
    plot(co, pch = 19, col = "darkgreen", xlab = "Dim 1", ylab = "Dim 2",
         main = "MDS - M-ABCdist combined dissimilarity")
  }

  Clust <- cluster::agnes(stats::as.dist(D_multi), diss = TRUE,
                          method = final_linkage, par.method = alpha)
  out <- list(source_D = source_D, DistM = D_multi,
              weights = source_weights, Clust = Clust)
  if (!is.null(NC2)) {
    nc <- as.integer(NC2)
    if (nc >= 2L && nc < N) out$cut <- stats::cutree(stats::as.hclust(Clust), k = nc)
  }
  attr(out, "method") <- "M-ABCdist"
  class(out) <- c("M_ABCdist", "list")
  out
}

# =============================================================================
# nemo.R  - NEMO: NEighborhood based Multi-Omics clustering
# Rappoport & Shamir (2019), Bioinformatics. Pure R.
# Per-omic locally-scaled kNN affinities are averaged across the omics in which
# each pair of objects is measured (so partial / missing modalities need no
# imputation), then spectral clustering is applied to the integrated affinity.
# =============================================================================

# locally-scaled, kNN-sparsified affinity for one omic (objects in rows)
.nemo_affinity <- function(X, NN) {
  X  <- as.matrix(X)
  Dm <- as.matrix(stats::dist(X))
  n  <- nrow(Dm)
  NN <- max(1L, min(NN, n - 1L))
  # local scale: mean distance to the NN nearest neighbours
  sigma <- apply(Dm, 1, function(d) mean(sort(d)[2:(NN + 1L)]))
  sigma[sigma <= 0] <- .Machine$double.eps
  W <- exp(-(Dm^2) / outer(sigma, sigma))
  diag(W) <- 0
  # keep only the NN strongest neighbours per row, then symmetrise by max
  for (i in seq_len(n)) {
    thr <- sort(W[i, ], decreasing = TRUE)[NN]
    W[i, W[i, ] < thr] <- 0
  }
  W <- pmax(W, t(W))
  dimnames(W) <- list(rownames(X), rownames(X))
  W
}

#' NEMO: neighborhood-based multi-omics clustering
#'
#' NEMO \insertCite{Wang2014a}{MosaiClusteR} integrates several omics by building
#' a locally-scaled, k-nearest-neighbour affinity network for each one and
#' averaging them, for every pair of objects, over only the omics in which both
#' objects are measured. Partial / missing modalities are therefore handled
#' without imputation. The integrated affinity is clustered spectrally.
#'
#' @param List A list of data matrices over the same objects, \strong{objects in
#'   rows}. Sources may cover different subsets of objects (matched by row name)
#'   for the partial-data setting.
#' @param k Number of clusters (\code{NULL} = eigengap estimate).
#' @param NN Number of neighbours for the affinity graphs and local scaling.
#' @param normalize Logical; row-normalise each affinity to a transition matrix
#'   before integration (NEMO default \code{TRUE}).
#' @return A list of class \code{"NEMO"} with \code{FusedM} (integrated
#'   affinity), \code{DistM} (\code{1 - FusedM}), \code{cluster} (labels) and
#'   \code{Clust} (an \code{agnes} object on \code{DistM} for interoperability).
#' @references \insertCite{Wang2014a}{MosaiClusteR}; Rappoport, N. & Shamir, R.
#'   (2019). NEMO: cancer subtyping by integration of partial multi-omic data.
#'   Bioinformatics 35:3348-3356.
#' @seealso \code{\link{SNF}}, \code{\link{spectral_clustering}}
#' @examples
#' data(mosaic_toy)
#' fit <- NEMO(mosaic_toy$List, k = 3, NN = 15)
#' table(fit$cluster)
#' @export
NEMO <- function(List, k = NULL, NN = 20, normalize = TRUE) {
  if (!is.list(List)) stop("'List' must be a list of data matrices.")
  objs <- Reduce(union, lapply(List, rownames))
  if (is.null(objs))
    stop("NEMO needs row names (objects) on the input matrices.")
  n <- length(objs)

  acc <- matrix(0, n, n, dimnames = list(objs, objs))
  cnt <- matrix(0, n, n, dimnames = list(objs, objs))
  for (X in List) {
    W <- .nemo_affinity(X, NN)
    if (isTRUE(normalize)) {
      rs <- rowSums(W); rs[rs <= 0] <- .Machine$double.eps
      W  <- W / rs
      W  <- (W + t(W)) / 2
    }
    idx <- match(rownames(W), objs)
    acc[idx, idx] <- acc[idx, idx] + W
    cnt[idx, idx] <- cnt[idx, idx] + 1
  }
  cnt[cnt == 0] <- NA
  Fused <- acc / cnt                       # average over omics measuring each pair
  Fused[is.na(Fused)] <- 0
  Fused <- (Fused + t(Fused)) / 2
  diag(Fused) <- 0

  sc <- spectral_clustering(Fused, k = k)

  D <- 1 - Fused / max(Fused[Fused > 0], 1)
  diag(D) <- 0
  Clust <- cluster::agnes(D, diss = TRUE, method = "ward")

  out <- list(FusedM = Fused, DistM = D, cluster = sc$cluster,
              k = sc$k, Clust = Clust)
  attr(out, "method") <- "NEMO"
  class(out) <- c("NEMO", "list")
  out
}

#' @title Weighted clustering
#'
#' @description Weighted Clustering (Weighted) is a similarity-based multi-source clustering technique. Weighted clustering is performed with the function \code{WeightedClust}. Given a
#' list of the data matrices, a dissimilarity matrix is computed of each with
#' the provided distance measures. These matrices are then combined resulting
#' in a weighted dissimilarity matrix. Hierarchical clustering is performed
#' on this weighted combination with the agnes function and the ward link
#'
#' @export
#' @param List A list of data matrices. It is assumed the rows are corresponding with the objects.
#' @param type indicates whether the provided matrices in "List" are either data matrices, distance
#' matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
#' matrices is skipped and if type="clusters" the single source clustering is skipped.
#' Type should be one of "data", "dist" or "clusters".
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").
#' @param normalize	Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.
#' @param StopRange Logical. Indicates whether the distance matrices with values not between zero and one should be standardized to have values between zero and one.
#' If FALSE the range normalization is performed. See \code{Normalization}. If TRUE, the distance matrices are not changed.
#' This is recommended if different types of data are used such that these are comparable. Default is FALSE.
#' @param weight Optional. A list of different weight combinations for the data sets in List.
#' If NULL, the weights are determined to be equal for each data set.
#' It is further possible to fix weights for some data matrices and to
#'  let it vary randomly for the remaining data sets. Defaults to seq(1,0,-0.1).  An example is provided in the details.
#' @param weightclust A weight for which the result will be put aside of the other results. This was done for comparative reason and easy access. Defaults to 0.5 (two data sets)
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character) for the final clustering. Defaults to "ward".
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".
#' @return The returned value is a list of four elements:
#' \item{DistM}{A list with the distance matrix for each data structure}
#' \item{WeightedDist }{A list with the weighted distance matrices}
#' \item{Results}{The hierarchical clustering result for each element in WeightedDist}
#' \item{Clust}{The result for the weight specified in Clustweight}
#' The value has class 'Weighted'.
#' @details The weight combinations should be provided as elements in a list. For three data
#' matrices an example could be: weights=list(c(0.5,0.2,0.3),c(0.1,0.5,0.4)). To provide
#' a fixed weight for some data sets and let it vary randomly for others, the element "x"
#' indicates a free parameter. An example is weights=list(c(0.7,"x","x")). The weight 0.7
#' is now fixed for the first data matrix while the remaining 0.3 weight will be divided over
#' the other two data sets. This implies that every combination of the sequence from 0 to 0.3
#' with steps of 0.1 will be reported and clustering will be performed for each.
#' @references
#' \insertRef{PerualilaTan2016}{MosaiClusteR}
WeightedClust <- function(List,type=c("data","dist","clusters"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),StopRange=FALSE,weight=seq(1,0,-0.1),weightclust=0.5,clust="agnes",linkage="ward",alpha=0.625){ # weight = weight to data1
  
  
  #Step 1: compute distance matrices:
  type<-match.arg(type)
  
  CheckDist<-function(Dist,StopRange){
    if(StopRange==FALSE & !(0<=min(Dist) & max(Dist)<=1)){
      #message("It was detected that a distance matrix had values not between zero and one. Range Normalization was performed to secure this. Put StopRange=TRUE if this was not necessary")
      Dist=Normalization(Dist,method="Range")
    }
    else{
      Dist=Dist
    }
  }
  
  
  if(type=="data"){
    OrderNames=rownames(List[[1]])
    for(i in 1:length(List)){
      List[[i]]=List[[i]][OrderNames,,drop=FALSE]
    }
    Dist=lapply(seq(length(List)),function(i) Distance(List[[i]],distmeasure[i],normalize[i],method[i]))
    Dist=lapply(seq(length(Dist)),function(i) CheckDist(Dist[[i]],StopRange))
  }
  else if(type=="dist"){
    OrderNames=rownames(List[[1]])
    for(i in 1:length(List)){
      List[[i]]=List[[i]][OrderNames,OrderNames]
    }
    Dist=List
    Dist=lapply(seq(length(Dist)),function(i) CheckDist(Dist[[i]],StopRange))
  }
  else{
    Dist=lapply(seq(length(List)),function(i) return(List[[i]]$Dist))
    Dist=lapply(seq(length(Dist)),function(i) CheckDist(Dist[[i]],StopRange))
    OrderNames=rownames(DistM[[1]])
    for(i in 1:length(DistM)){
      DistM[[i]]=DistM[[i]][OrderNames,OrderNames]
    }
  }
  
  #Step 2: Weighted linear combination of the distance matrices:
  if(is.null(weight)){
    equalweights=1/length(List)
    weight=list(rep(equalweights,length(List)))
    
  }
  else if(is.list(weight) & length(weight[[1]])!=length(List)){
    stop("Give a weight for each data matrix or specify a sequence of weights")
  }

  if(!is.list(weight)){
    condition<-function(l){
      l=as.numeric(l)
      if( sum(l)==1 ){  #working with characters since with the numeric values of comb or permutations something goes not the way is should: 0.999999999<0.7+0.3<1??
        #return(row.match(l,t1))
        return(l)
      }
      else(return(0))
    }
    
    if(all(seq(1,0,-0.1)!=weight)){
      for(i in 1:length(weight)){
        rest=1-weight[i]
        if(!(rest%in%weight)){
          weight=c(weight,rest)
        }
      }
    }
    
    
    
    
    t1=gtools::permutations(n=length(weight),r=length(List),v=as.character(weight),repeats.allowed = TRUE)
    t2=lapply(seq_len(nrow(t1)), function(i) if(sum(as.numeric(t1[i,]))==1) return(as.numeric(t1[i,])) else return(0)) #make this faster: lapply on a list or adapt permutations function itself: first perform combinations under restriction then perform permutations
    t3=sapply(seq(length(t2)),function(i) if(!all(t2[[i]]==0)) return (i) else return(0))
    t4=t2[which(t3!=0)]
    weight=lapply(seq(length(t4)),function(i) rev(t4[[i]]))
    
  }
  if(is.list(weight) & "x" %in% weight[[1]]){ #x indicates a free weight
    newweight=list()
    for(k in 1:length(weight)){
      w=weight[[k]]
      weightsfordata=which(w!="x") #position of the provided weight = position of the data to which the weight is given
      givenweights=as.numeric(w[weightsfordata])
      
      stilltodistribute=1-sum(givenweights)
      
      newweights=seq(stilltodistribute,0,-0.1)
      
      t1=gtools::permutations(n=length(newweights),r=length(List)-length(weightsfordata),v=as.character(newweights),repeats.allowed = TRUE)
      Input1=as.list(seq_len(nrow(t1)))
      Input2=lapply(seq(length(Input1)),function(i) {Input1[[i]][length(Input1[[i]])+1]=stilltodistribute
      return(Input1[[i]])})
      t2=lapply(seq(length(Input2)), FUN=function(i){if(sum(as.numeric(t1[Input2[[i]][1],])+0.00000000000000002775)==Input2[[i]][2]) return(as.numeric(t1[i,])) else return(0)}) #make this faster: lapply on a list or adapt permutations function itself: first perform combinations under restriction then perform permutations
      t3=sapply(seq(length(t2)),function(i) if(!all(t2[[i]]==0)) return (i) else return(0))
      weightsforotherdata=t2[which(t3!=0)]
      
      new=list()
      for(i in 1:length(weightsforotherdata)){
        w1=weightsforotherdata[[i]]
        new[[i]]=rep(0,length(List))
        new[[i]][weightsfordata]=givenweights
        new[[i]][which(new[[i]]==0)]=w1
      }
      
      newweight[k]=new
    }
    
    weight=newweight
  }
  weightedcomb<-function(w,Dist){
    temp=lapply(seq_len(length(Dist)),function(i) w[i]*Dist[[i]])
    temp=Reduce("+",temp)
    return(temp)
  }
  
  DistClust=NULL
  Clust=NULL
  
  DistM=lapply(seq(length(weight)),function(i) weightedcomb(weight[[i]],Dist=Dist))
  namesweights=c()
  WeightedClust=lapply(seq(length(weight)),function(i) cluster::agnes(DistM[[i]],diss=TRUE,method=linkage,par.method=alpha))
  for(i in 1:length(WeightedClust)){
    namesweights=c(namesweights,paste("Weight",weight[i],sep=" "))
    if(length(weight[[i]])==length(weightclust) && all(weight[[i]]==weightclust)){
      Clust=WeightedClust[[i]]
      DistClust=DistM[[i]]
    }
  }

  if(is.null(DistClust)){
    # Focus weight for the returned 'Clust'. A scalar 'weightclust' only makes
    # sense for two sources; for a different number (or a non-matching scalar)
    # fall back to equal weights so weightedcomb() gets a per-source vector
    # (a scalar would recycle to NA for sources 2..K and break agnes).
    wf=if(length(weightclust)==length(Dist)) weightclust else rep(1/length(Dist),length(Dist))
    DistClust=weightedcomb(wf,Dist=Dist)
    Temp=cluster::agnes(DistClust,diss=TRUE,method=linkage,par.method=alpha)
    Clust=Temp
  }
  
  Results=lapply(seq(1,length(WeightedClust)),function(i) return(c("DistM"=DistM[i],"Clust"=WeightedClust[i])))
  names(Results)=namesweights
  
  # return list with objects
  out=list(Dist=Dist,Results=Results,Clust=list("DistM"=DistClust,"Clust"=Clust))
  attr(out,'method')<-'Weighted'
  return(out)
  
}

#' @title Cumulative Voting Aggregation (CVAA / W-CVAA)
#'
#' @description \code{CVAA} performs cumulative voting consensus clustering. A
#' reference partition is used to align the individual source partitions, which
#' are then aggregated into a consensus stochastic membership matrix. Two
#' variants are available: cumulative voting (\code{"CVAA"}) and the
#' information-weighted variant (\code{"W-CVAA"}).
#'
#' @export
#' @param Reference A reference partition (a clustering result, optionally an
#' object produced by a weighted/CEC method). Required.
#' @param nrclustersR The number of clusters of the reference partition. Default is 7.
#' @param List A list of data matrices. It is assumed the rows correspond with the objects.
#' @param typeL Indicates whether the provided matrices in "List" are either data
#' matrices ("data"), distance matrices ("dist") or clustering results ("clust").
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").
#' @param normalize Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE).
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL).
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible","flexible").
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625.
#' @param nrclusters The number of clusters to divide each individual dendrogram in. Default is c(7,7).
#' @param gap Logical. Whether the optimal number of clusters should be determined with the gap statistic. Defaults to FALSE.
#' @param maxK The maximal number of clusters to investigate in the gap statistic. Default is 15.
#' @param votingMethod The method to be performed: "CVAA" or "W-CVAA".
#' @param optimalk An estimate of the final optimal number of clusters. Defaults to nrclustersR.
#' @return A list of two elements:
#' \item{DistM}{A NULL object}
#' \item{Clust}{The resulting clustering (order, order.lab and Clusters)}
#' The value has class 'Ensemble'.
#' @details Cumulative voting aggregation aligns each source membership matrix to
#' a running consensus by least-squares voting, then averages cumulatively. The
#' weighted variant orders the partitions by average information content and
#' weights their contribution accordingly. When \code{optimalk} is smaller than
#' \code{nrclustersR} the JS-ALink agglomeration is used to merge reference
#' clusters.
#' @references Ayad H.G. and Kamel M.S. (2008). Cumulative voting consensus
#' method for partitions with variable number of clusters. IEEE Transactions on
#' Pattern Analysis and Machine Intelligence, 30(1), 160-173.
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' L <- mosaic_toy$List
#' Ref <- Cluster(L[[1]], type = "data", distmeasure = "euclidean")
#' attr(Ref, "method") <- "Single"
#' res <- CVAA(Reference = Ref, nrclustersR = 5, List = L, typeL = "data",
#'   distmeasure = c("euclidean","euclidean"), votingMethod = "CVAA", optimalk = 5)
#' }
CVAA<-function(Reference=NULL,nrclustersR=7,List,typeL=c("data","dist","clust"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),clust = "agnes", linkage = c("flexible","flexible"),alpha=0.625,nrclusters=c(7,7),gap = FALSE, maxK = 15,votingMethod=c("CVAA","W-CVAA"),optimalk=nrclustersR){

	#needs a reference partition: which one to choose? Weighted? Or one of the single source clusterings? Let user decide but suggest Weighted as default
	#Reference can be a "method": List needs to be data or dist
	#if Reference is a clust, list can still be anything

	if(typeL=="data"){

		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type="data",distmeasure[i],normalize[i],method[i],clust,linkage[i],alpha,gap,maxK,StopRange=TRUE))

		Dist=lapply(seq(length(List)),function(i) Clusterings[[i]]$DistM)

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=clusters
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))

		U_all=lapply(seq(length(Clusters)),function(i) stats::model.matrix(~as.factor(Clusters[[i]])-1))
		for(i in 1:length(U_all)){
			colnames(U_all[[i]])=seq(1,ncol(U_all[[i]]))
			rownames(U_all[[i]])=rownames(List[[i]])
		}

	}

	else if(typeL=="dist"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,OrderNames]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],typeL,distmeasure[i],normalize=FALSE,method=NULL,clust,linkage[i],alpha,gap,maxK,StopRange=TRUE))

		Dist=List

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=ceiling(mean(clusters))
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))
		U_all=lapply(seq(length(Clusters)),function(i) stats::model.matrix(~as.factor(Clusters[[i]])-1))
		for(i in 1:length(U_all)){
			colnames(U_all[[i]])=seq(1,ncol(U_all[[i]]))
			rownames(U_all[[i]])=rownames(List[[i]])
		}


	}
	else if(typeL=="clust"){

		Clusterings=List

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		if(is.null(nrclusters)){
			stop("Please specify a number of clusters")
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))
		U_all=lapply(seq(length(Clusters)),function(i) stats::model.matrix(~as.factor(Clusters[[i]])-1))
		for(i in 1:length(U_all)){
			colnames(U_all[[i]])=seq(1,ncol(U_all[[i]]))
			rownames(U_all[[i]])=rownames(List[[i]]$DistM)

		}

	}

	if(!(is.null(Reference))){

		ListNew=list()
		element=0

		if(attributes(Reference)$method != "CEC" & attributes(Reference)$method != "Weighted" & attributes(Reference)$method!= "WeightedSim"){
			ResultsClust=list()
			ResultsClust[[1]]=list()
			ResultsClust[[1]][[1]]=Reference
			names(ResultsClust[[1]])[1]="Clust"
			element=element+1
			ListNew[[element]]=ResultsClust[[1]]
			#attr(ListNew[element],"method")="Weights"
		}
		else if(attributes(Reference)$method=="CEC" | attributes(Reference)$method=="Weighted" | attributes(Reference)$method == "WeightedSim"){
			ResultsClust=list()
			ResultsClust[[1]]=list()
			if(attributes(Reference)$method != "WeightedSim"){
				ResultsClust[[1]][[1]]=Reference$Clust
				names(ResultsClust[[1]])[1]="Clust"
				element=element+1
				ListNew[[element]]=ResultsClust[[1]]
				attr(ListNew[[element]]$Clust,"method")="Weights"

			}
			else{
				for (j in 1:length(Reference$Results)){
					ResultsClust[[j]]=list()
					ResultsClust[[j]][[1]]=Reference$Results[[j]]
					names(ResultsClust[[j]])[1]="Clust"
					attr(ResultsClust[[1]],"method")="Weights"
					element=element+1
					ListNew[[element]]=ResultsClust[[j]]
					ListNew=unlist(ListNew,recursive=FALSE)
				}
			}
		}

		Reference=ListNew

		if(is.null(nrclustersR)){
			stop("Please specify a number of clusters for the reference partition")
		}

		CutTree<-function(Data,nrclusters){
			if(attributes(Data$Clust)$method == "Ensemble"){
				Clusters=Data$Clust$Clust$Clusters
				names(Clusters)=NULL
			}
			else{
				Clusters=stats::cutree(Data$Clust$Clust,k=nrclusters)
			}
			return(Clusters)
		}

		Ref_Clusters=CutTree(Reference[[1]],nrclusters=nrclustersR)

		U_0=stats::model.matrix(~as.factor(Ref_Clusters)-1)
		colnames(U_0)=seq(1,ncol(U_0))  #assign reference to U_0
		rownames(U_0)=rownames(Reference[[1]]$Clust$DistM)
		U_Ref=U_0

	}
	else{
		stop("No Reference specified")
	}

	if(votingMethod=="CVAA"){

		if(is.null(Reference)){
			random_part<-sample(1:length(U_all),size=1)
			U_0=U_all[[random_part]]
			U_all=U_all[-random_part]
		}

		for(i in 1:length(U_all)){ #for i in 2 to b to:

			W_i=solve((t(U_all[[i]])%*%U_all[[i]]))%*%t(U_all[[i]])%*%U_0

			V_i=U_all[[i]]%*%W_i

			U_0=((i-1)/i)*U_0+(1/i)*V_i
		}
	}

	else if(votingMethod=="W-CVAA"){

		#Preparation for the Weights for the partitions
		H_c=c()
		for(i in 1:length(U_all)){
			P_c=(colSums(U_all[[i]]))/nrow(U_all[[i]])
			H_c=c(H_c,-sum(P_c*log(P_c)))
		}

		#message("Reordering the clusterings in decreasing order of average amount of information")
		Order=sort(H_c,decreasing=TRUE,index.return=TRUE)$ix
		U_all=U_all[Order]

		if(is.null(Reference)){
			#message("Performing the Ada-cVote algorithm by Ayad and Kamel")
			U_0=U_all[Order[1]]
			U_all=U_all[Order[-c(1)]]
		}

		for(i in 1:length(U_all)){ #for i in 2 to b to:

			# The weight for partition i
			T_i=(H_c[i])/sum(H_c)


			W_i=solve((t(U_all[[i]])%*%U_all[[i]]))%*%t(U_all[[i]])%*%U_0

			V_i=U_all[[i]]%*%W_i

			U_0=((i-1)/i)*U_0+(T_i/i)*V_i
		}


	}

	U_consensus=U_0

	if(!is.null(optimalk) & optimalk==nrclustersR){

		Clusters=apply(U_consensus,1,function(i) which.max(i))
		names(Clusters)=rownames(U_consensus)
		clusters=unique(Clusters)
		order=c()
		for(j in clusters){
			order=c(order,which(Clusters==j))
		}

		order.lab=as.character(order)
	}

	else if(!is.null(optimalk)){

		p_hat_joint_c_x=(1/nrow(U_consensus))*U_consensus
		p_hat_x=1/nrow(U_consensus)

		#The JS-Alink Algorithm (applied to get the best partition out of the consensus partion U_consensus).
		#This does not do anything if the optimal number of clusters is equal to the number of clusters of the reference. It will when cluster of the reference are to be merged (less clusters than the reference)

		Pairs=utils::combn(ncol(U_consensus),2)

		JS=c()
		JS_ALink<-function(Pair,U_Ref,p_hat_joint_c_x){
			p_hat_cl=colSums(U_Ref)[Pair[1]]/nrow(U_Ref)    #the number of objects assigned to cluster l in the reference divided by the total
			p_hat_cm=colSums(U_Ref)[Pair[2]]/nrow(U_Ref)  	#the number of objects assigned to cluster m in the reference divided by the total

			p_hat_x_cl=p_hat_joint_c_x[,Pair[1]]/p_hat_cl
			p_hat_x_cm=p_hat_joint_c_x[,Pair[2]]/p_hat_cm

			alpha_1=p_hat_cl/(p_hat_cl+p_hat_cm)
			alpha_2=p_hat_cm/(p_hat_cl+p_hat_cm)


			temp1=alpha_1*p_hat_x_cl+alpha_2*p_hat_x_cm
			temp2=temp1*log(temp1)
			temp2[which(is.na(temp2))]=0
			part1=-sum(temp2)

			temp3=p_hat_x_cl*log(p_hat_x_cl)
			temp3[which(is.na(temp3))]=0
			part2=alpha_1*(-sum(temp3))

			temp4=p_hat_x_cm*log(p_hat_x_cm)
			temp4[which(is.na(temp4))]=0
			part3=alpha_2*(-sum(temp4))


			JS_cl_cm=part1-part2-part3
			JS=c(JS,JS_cl_cm)

		}
		JS=apply(Pairs,2,function(i) JS_ALink(Pair=i,U_Ref,p_hat_joint_c_x))

		Clust=cluster::agnes(JS,diss=TRUE,method="average")

		if(is.null(optimalk)){
			k_part=stats::cutree(stats::as.hclust(Clust),k=optimalk)

			priors=c()
			jointdistr=c()
			for(a in 1:optimalk){
				S_g=which(k_part==a)
				prior=sum(colSums(U_Ref)[S_g]/nrow(U_Ref))
				priors=c(priors,prior)

				joint_temp=rowSums(p_hat_joint_c_x[,S_g,drop=FALSE]/(colSums(U_Ref)[S_g,drop=FALSE]/nrow(U_Ref)))*colSums(U_Ref)[S_g,drop=FALSE]/nrow(U_Ref)
				jointdistr=cbind(jointdistr,joint_temp)

			}
			colnames(jointdistr)=c(1:ncol(jointdistr))

			U_hat=jointdistr/p_hat_x
			Clusters=apply(U_consensus,1,function(i) which.max(i))
			names(Clusters)=rownames(U_consensus)

			clusters=unique(Clusters)
			order=c()
			for(j in clusters){
				order=c(order,which(Clusters==j))
			}

			order.lab=as.character(order)

		}
	}
	else if(is.null(optimalk)){
		#message("No optimal number of clusters was specified. The consenus matrix will be returned but an optimal partition was not extracted by the JS-ALink algorithm")
		Out=U_consensus
		return(Out)

	}

	Out=list(DistM=NULL,Clust=list(order=order,order.lab=order.lab,Clusters=Clusters))
	attr(Out,"method")="Ensemble"
	return(Out)

}


#' @title Evidence accumulation clustering (co-association)
#'
#' @description \code{EvidenceAccumulation} builds a co-association
#' (similarity) matrix from the individual source partitions and extracts a
#' consensus partition by graph partitioning. Three graph-partitioning options
#' are available: a minimum-spanning-tree procedure (\code{"MST"}, requires the
#' \pkg{igraph} package), a thresholded single-link procedure (\code{"SL"}), and
#' an agnes single-link procedure (\code{"SL_agnes"}).
#'
#' @export
#' @param List A list of data matrices. It is assumed the rows correspond with the objects.
#' @param type Indicates whether the provided matrices in "List" are either data
#' matrices ("data"), distance matrices ("dist") or clustering results ("clust").
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").
#' @param normalize Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE).
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL).
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible","flexible").
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625.
#' @param nrclusters The number of clusters to divide each individual dendrogram in. Default is c(7,7).
#' @param gap Logical. Whether the optimal number of clusters should be determined with the gap statistic. Defaults to FALSE.
#' @param maxK The maximal number of clusters to investigate in the gap statistic. Default is 15.
#' @param graphPartitioning The graph partitioning method to be performed: "MST", "SL" or "SL_agnes".
#' @param t A threshold used by the "MST" and "SL" partitioning procedures. Defaults to NULL.
#' @return A list of two elements:
#' \item{DistM}{A NULL object}
#' \item{Clust}{The resulting clustering}
#' The value has class 'Ensemble' (or 'Single Clustering' for "SL_agnes").
#' @details The co-association matrix \code{S} records the fraction of source
#' partitions in which two objects are co-clustered. \code{"MST"} prunes a
#' minimum spanning tree of the similarity graph (optionally at threshold
#' \code{t}); \code{"SL"} performs a thresholded single-link grouping; and
#' \code{"SL_agnes"} runs agnes single linkage on the dissimilarity \code{1-S}.
#' The \code{"MST"} path requires the suggested package \pkg{igraph}.
#' @references Fred A.L.N. and Jain A.K. (2005). Combining multiple clusterings
#' using evidence accumulation. IEEE Transactions on Pattern Analysis and
#' Machine Intelligence, 27(6), 835-850.
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' L <- mosaic_toy$List
#' res <- EvidenceAccumulation(List = L, type = "data",
#'   distmeasure = c("euclidean","euclidean"), nrclusters = c(5,5),
#'   graphPartitioning = "SL_agnes")
#' }
EvidenceAccumulation<-function(List,type=c("data","dist","clust"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),clust = "agnes", linkage = c("flexible","flexible"),alpha=0.625,nrclusters=c(7,7),gap = FALSE, maxK = 15,graphPartitioning=c("MST","SL","SL_agnes"),t=NULL){

	#needs a reference partition: which one to choose? Weighted? Or one of the single source clusterings? Let user decide but suggest Weighted as default
	#Reference can be a "method": List needs to be data or dist
	#if Reference is a clust, list can still be anything

	if(type=="data"){

		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,,drop=FALSE]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type="data",distmeasure[i],normalize[i],method[i],clust,linkage[i],alpha,gap,maxK,StopRange=TRUE))

		Dist=lapply(seq(length(List)),function(i) Clusterings[[i]]$DistM)

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=clusters
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))

	}

	else if(type=="dist"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,OrderNames]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type,distmeasure[i],normalize=FALSE,method=NULL,clust,linkage[i],alpha,gap,maxK,StopRange=TRUE))

		Dist=List

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=ceiling(mean(clusters))
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))


	}
	else if(type=="clust"){

		Clusterings=List

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		if(is.null(nrclusters)){
			stop("Please specify a number of clusters")
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))

	}

	#similarity matrix
	H=list()
	for(i in 1:length(List)){
		Bin=matrix(0,nrow=max(Clusters[[i]]),ncol=length(Clusters[[i]]))
		for(j in 1:nrow(Bin)){
			Bin[j,which(Clusters[[i]]==j)]=1
		}
		H[[i]]=as.data.frame(Bin)
	}
	T=do.call(rbind,H)
	S=t(as.matrix(T))%*%(as.matrix(T))
	S=S/length(List)

	if(type!="clust"){
		rownames(S)=rownames(List[[1]])
		colnames(S)=rownames(List[[1]])
	}
	else{
		rownames(S)=rownames(List[[1]]$DistM)
		colnames(S)=rownames(List[[1]]$DistM)
	}


	if(graphPartitioning=="MST"){

		#are assigning to the first encounter to break ties and get clusters: if changed  to join all if one is in common, all end up in 1 cluster

		if (!requireNamespace("igraph", quietly = TRUE)) {
			stop("graphPartitioning = \"MST\" requires the suggested package 'igraph'. ",
				 "Install it or use graphPartitioning = \"SL\" / \"SL_agnes\".")
		}

		# Strip dimnames so igraph vertices are integer-indexed (1..n); modern
		# igraph::as_data_frame() otherwise returns edge endpoints as vertex NAMES
		# (character), which breaks the integer-based Placed[]/Partition[] logic.
		Su=S; dimnames(Su)=NULL
		Graph=igraph::graph_from_adjacency_matrix(adjmatrix=Su, mode="undirected", weighted=TRUE, diag=TRUE, add.colnames=NULL)
		MST_Graph=igraph::as_data_frame(igraph::mst(Graph))
		MST_Graph[,1]=as.integer(MST_Graph[,1]); MST_Graph[,2]=as.integer(MST_Graph[,2])

		if(!is.null(t)){
			MST_Graph=MST_Graph[-c(which(as.numeric(MST_Graph[,3])<t)),]
		}

		Partition=list()
		Partition[[1]]=c(MST_Graph[1,1],MST_Graph[1,2])
		Placed=rep(FALSE,nrow(S))
		Placed[Partition[[1]]]=TRUE
		for(j in 2:nrow(MST_Graph)){
			Edge=as.numeric(MST_Graph[j,c(1:2)])
			k=1

			while(any(Placed[Edge]==FALSE)){

				if(k>length(Partition)){
					Partition[[length(Partition)+1]]=Edge
					Placed[Edge]=TRUE
				}

				else if(Edge[1]%in%Partition[[k]] | Edge[2]%in%Partition[[k]]){
					Partition[[k]]=c(unlist(Partition[[k]]),Edge)
					Partition[[k]]=unique(Partition[[k]])
					Placed[Edge]=TRUE
				}
				k=k+1
			}


		}

		order=unlist(Partition)
		order.lab=rownames(S)[order]
		t1<-sapply(1:length(Partition),function(i) rep(i,length(Partition[[i]])))
		t2<-sapply(1:length(Partition),function(i) rownames(S)[Partition[[i]]])
		t3<-cbind(unlist(t1),unlist(t2))
		clus=as.numeric(t3[,1])
		names(clus)=t3[,2]
		Clusters=clus[rownames(S)]
		if(any(is.na(Clusters))){
			Clusters[which(is.na(Clusters))]=c(1:length(which(is.na(Clusters))))
			names(Clusters)=rownames(S)
		}


	}

	else if(graphPartitioning=="SL"){
		if(is.null(t)){
			stop("Specify a treshold t for the Sl algorithm")
		}
		#better to let run for a number of thresholds t (default is 0.5)
		#Not discerning enough if only 2 datasets with 1 clustering each: need more to get more clusters

		Clusters=list()
		for(a in 1:nrow(S)){
			if(a==1){
				Clusters[[length(Clusters)+1]]=a
			}
			else{
				for(b in 1:(a-1)){

					if(S[a,b]>t){

						if(a%in%unlist(Clusters) & b%in%unlist(Clusters)){

							Clus1=which(sapply(1:length(Clusters),function(i) a%in%Clusters[[i]]))
							Clus2=which(sapply(1:length(Clusters),function(i) b%in%Clusters[[i]]))
							if(Clus1 != Clus2){
								Clusters[[Clus1]]=unique(c(unlist(Clusters[[Clus1]]),unlist(Clusters[[Clus2]])))
								Clusters[[Clus2]]=NULL
							}

						}
						else if(a%in%unlist(Clusters) & !(b%in%unlist(Clusters))){
							Clus1=which(sapply(1:length(Clusters),function(i) a%in%Clusters[[i]]))
							Clusters[[Clus1]]=unique(c(unlist(Clusters[[Clus1]]),b))

						}
						else if(!(a%in%unlist(Clusters)) & b%in%unlist(Clusters)){
							Clus1=which(sapply(1:length(Clusters),function(i) b%in%Clusters[[i]]))
							Clusters[[Clus1]]=unique(c(unlist(Clusters[[Clus1]]),a))
						}
						else{
							Clusters[[length(Clusters)+1]]=unique(c(a,b))
						}

					}

				}
			}
		}

		Singletons=which(!seq(1,nrow(S))%in%unlist(Clusters))
		if(length(Singletons)>0){

			for(l in Singletons){
				Clusters[[length(Clusters)+1]]=l
			}

		}


		order=unlist(Clusters)
		order.lab=rownames(S)[order]
		t1<-sapply(1:length(Clusters),function(i) rep(i,length(Clusters[[i]])))
		t2<-sapply(1:length(Clusters),function(i) rownames(S)[Clusters[[i]]])
		t3<-cbind(unlist(t1),unlist(t2))
		clus=as.numeric(t3[,1])
		names(clus)=t3[,2]
		Clusters=clus[rownames(S)]

		#Or perform single linkage on the similarity matrix S
		#Clusters=agnes(1-S,diss=TRUE,method="single")

	}

	else if(graphPartitioning=="SL_agnes"){
		Clust=cluster::agnes(1-S,diss=TRUE,method="single")
		order=Clust$order
		order.lab=rownames(S)[order]
		if(is.null(nrclusters)){
			Clusters=stats::cutree(stats::as.hclust(Clust),k=2)
		}
		else{
			Clusters=stats::cutree(stats::as.hclust(Clust),k=nrclusters[1])
		}
		names(Clusters)=rownames(S)
		Out=list(DistM=NULL,Clust=list(order=order,order.lab=order.lab,Clusters=Clusters))
		attr(Out,"method")="Single Clustering"
		return(Out)

	}

	Out=list(DistM=NULL,Clust=list(order=order,Clusters=Clusters))
	attr(Out,"method")="Ensemble"
	return(Out)

}

test_that("CVAA runs on mosaic_toy (CVAA, optimalk == nrclustersR)", {
  data(mosaic_toy)
  L <- mosaic_toy$List

  # Build a reference partition from the first data source.
  Ref <- Cluster(L[[1]], type = "data", distmeasure = "euclidean",
                 normalize = FALSE, method = NULL, clust = "agnes",
                 linkage = "ward", alpha = 0.625)
  attr(Ref, "method") <- "Single"

  res <- CVAA(Reference = Ref, nrclustersR = 5, List = L, typeL = "data",
              distmeasure = c("euclidean", "euclidean"),
              normalize = c(FALSE, FALSE), method = c(NULL, NULL),
              clust = "agnes", linkage = c("ward", "ward"), alpha = 0.625,
              nrclusters = c(5, 5), gap = FALSE, maxK = 15,
              votingMethod = "CVAA", optimalk = 5)

  expect_type(res, "list")
  expect_equal(attr(res, "method"), "Ensemble")
  expect_true(!is.null(res$Clust$Clusters))
  expect_equal(length(res$Clust$Clusters), nrow(L[[1]]))
})

test_that("CVAA runs on mosaic_toy (W-CVAA)", {
  data(mosaic_toy)
  L <- mosaic_toy$List

  Ref <- Cluster(L[[1]], type = "data", distmeasure = "euclidean",
                 linkage = "ward")
  attr(Ref, "method") <- "Single"

  res <- CVAA(Reference = Ref, nrclustersR = 5, List = L, typeL = "data",
              distmeasure = c("euclidean", "euclidean"),
              linkage = c("ward", "ward"), nrclusters = c(5, 5),
              votingMethod = "W-CVAA", optimalk = 5)

  expect_equal(attr(res, "method"), "Ensemble")
  expect_equal(length(res$Clust$Clusters), nrow(L[[1]]))
})

test_that("EvidenceAccumulation runs on mosaic_toy (SL_agnes, pure R)", {
  data(mosaic_toy)
  L <- mosaic_toy$List

  res <- EvidenceAccumulation(List = L, type = "data",
                              distmeasure = c("euclidean", "euclidean"),
                              normalize = c(FALSE, FALSE), method = c(NULL, NULL),
                              clust = "agnes", linkage = c("ward", "ward"),
                              alpha = 0.625, nrclusters = c(5, 5),
                              gap = FALSE, maxK = 15,
                              graphPartitioning = "SL_agnes")

  expect_type(res, "list")
  expect_equal(attr(res, "method"), "Single Clustering")
  expect_equal(length(res$Clust$Clusters), nrow(L[[1]]))
})

test_that("EvidenceAccumulation runs on mosaic_toy (SL threshold path)", {
  data(mosaic_toy)
  L <- mosaic_toy$List

  res <- EvidenceAccumulation(List = L, type = "data",
                              distmeasure = c("euclidean", "euclidean"),
                              nrclusters = c(5, 5),
                              graphPartitioning = "SL", t = 0.5)

  expect_equal(attr(res, "method"), "Ensemble")
  expect_equal(length(res$Clust$Clusters), nrow(L[[1]]))
})

test_that("EvidenceAccumulation MST path requires igraph", {
  skip_if_not_installed("igraph")
  data(mosaic_toy)
  L <- mosaic_toy$List

  res <- EvidenceAccumulation(List = L, type = "data",
                              distmeasure = c("euclidean", "euclidean"),
                              nrclusters = c(5, 5),
                              graphPartitioning = "MST")
  expect_equal(attr(res, "method"), "Ensemble")
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", warning = FALSE,
                      message = FALSE, fig.width = 7, fig.height = 4.5)
set.seed(1)

## ----setup--------------------------------------------------------------------
library(MosaiClusteR)

## ----data---------------------------------------------------------------------
data(mosaic_toy)
L     <- mosaic_toy$List      # a list of two matrices, objects in rows
truth <- mosaic_toy$truth     # the known group of each object (for illustration)
S     <- length(L)            # number of sources
K     <- 3                    # number of groups
vapply(L, dim, integer(2))    # 60 objects x (120, 150) features

## ----sim, eval = FALSE--------------------------------------------------------
# sim <- mosaic_sim(n = 90, g = 3, seed = 1)   # n objects, g groups
# str(sim$List, max.level = 1)

## ----preprocess---------------------------------------------------------------
Xn <- Normalization(L[[1]], method = "Quantile")   # optional per-source scaling
D1 <- Distance(L[[1]], distmeasure = "euclidean")   # object-by-object distance
round(as.matrix(D1)[1:4, 1:4], 2)

## ----baseline-----------------------------------------------------------------
b1 <- Cluster(L[[1]], distmeasure = "euclidean", normalize = FALSE, nrclusters = 3)
b2 <- Cluster(L[[2]], distmeasure = "euclidean", normalize = FALSE, nrclusters = 3)
c(source1 = cluster_agreement(mosaic_labels(b1, 3), truth)[["ARI"]],
  source2 = cluster_agreement(mosaic_labels(b2, 3), truth)[["ARI"]])

## ----integrate----------------------------------------------------------------
DM <- rep("euclidean", S); NM <- rep(FALSE, S); ME <- rep(list(NULL), S)

methods <- list(
  "ADC (direct)"           = function() mosaic_labels(
      ADC(L, distmeasure = "euclidean", normalize = FALSE), K),
  "WeightedClust (simil.)" = function() mosaic_labels(
      WeightedClust(L, type = "data", distmeasure = DM, normalize = NM,
                    method = ME, weight = seq(1, 0, -0.25)), K),
  "WonM (similarity)"      = function() mosaic_labels(
      WonM(L, type = "data", distmeasure = DM, normalize = NM, method = ME,
           nrclusters = rep(list(2:6), S), linkage = rep("ward", S)), K),
  "SNF (graph fusion)"     = function() mosaic_labels(
      SNF(L, type = "data", distmeasure = DM, normalize = NM, method = ME,
          NN = 20, T = 20), K),
  "NEMO (neighbourhood)"   = function() NEMO(L, k = K, NN = 20)$cluster,
  "CEC (consensus)"        = function() mosaic_labels(
      CEC(L, distmeasure = DM, normalize = NM, method = ME,
          r = vapply(L, ncol, 1L) %/% 2L, nrclusters = as.list(rep(K, S)),
          linkage = rep("ward", S)), K),
  "intNMF (factor)"        = function() intNMF(L, k = K, nstart = 4)$cluster,
  "Data nuggets"           = function() nugget_cluster(
      do.call(cbind, L), k = K, max_nuggets = 40)$cluster,
  "ABC (voting)"           = function() mosaic_labels(
      M_ABCpp(L, numsim = 60, NC = K), K)
)

ari <- vapply(methods, function(f)
  tryCatch(round(cluster_agreement(f(), truth)[["ARI"]], 2),
           error = function(e) NA_real_), numeric(1))
sort(ari, decreasing = TRUE)

## ----direct, eval = FALSE-----------------------------------------------------
# ADC(L, distmeasure = "euclidean", normalize = FALSE)
# ADEC(L, distmeasure = DM, normalize = NM, method = ME, nrclusters = 3, t = 10)

## ----simil, eval = FALSE------------------------------------------------------
# WeightedClust(L, type = "data", distmeasure = DM, normalize = NM, method = ME,
#               weight = seq(1, 0, -0.25))
# WonM(L, type = "data", distmeasure = DM, normalize = NM, method = ME,
#      nrclusters = rep(list(2:6), S), linkage = rep("ward", S))
# SNF(L, type = "data", distmeasure = DM, normalize = NM, method = ME, NN = 20, T = 20)
# NEMO(L, k = K, NN = 20)$cluster

## ----graph, eval = FALSE------------------------------------------------------
# HBGF(L, type = "data", distmeasure = DM, normalize = NM, method = ME, nrclusters = 3)
# ClusteringAggregation(L, type = "data", distmeasure = DM, normalize = NM,
#                       method = ME, nrclusters = 3)
# # EnsembleClustering() / EHC() call an external graph-partitioning executable
# # (e.g. METIS); install it and pass its path via 'executable' to use them.

## ----voting, eval = FALSE-----------------------------------------------------
# CEC(L, distmeasure = DM, normalize = NM, method = ME,
#     r = vapply(L, ncol, 1L) %/% 2L, nrclusters = as.list(rep(K, S)),
#     linkage = rep("ward", S))
# ConsensusClustering(L, type = "data", distmeasure = DM, normalize = NM,
#                     method = ME, nrclusters = 3)
# EvidenceAccumulation(L, type = "data", distmeasure = DM, normalize = NM,
#                      method = ME, nrclusters = 3)
# LinkBasedClustering(L, type = "data", distmeasure = DM, normalize = NM,
#                     method = ME, nrclusters = 3, linkBasedMethod = "SRS")
# M_ABCpp(L, numsim = 200, NC = 3)                 # co-clustering consensus
# M_ABCdist(L, numsim = 200, NC = 3)               # distance-accumulating variant
# M_ABCdeep(L, numsim = 200, NC = 3, ae_epochs = 60)   # autoencoder embedding

## ----hier, eval = FALSE-------------------------------------------------------
# HierarchicalEnsembleClustering(L, type = "data", distmeasure = DM,
#                                normalize = NM, method = ME)

## ----factor, eval = FALSE-----------------------------------------------------
# intNMF(L, k = K, nstart = 8)$cluster
# spectral_clustering(affinity = as.matrix(exp(-as.matrix(D1))), k = K)
# LUCID(G = exposures, Z = L, Y = outcome, K = 3)   # supervised; needs 'LUCIDus'

## ----nuggets------------------------------------------------------------------
dn <- create_data_nuggets(L[[1]], max_nuggets = 25)   # compress objects
dn
w  <- nugget_feature_weights(dn, type = "between")    # feature importance
head(sort(w, decreasing = TRUE))

## ----nuggets2, eval = FALSE---------------------------------------------------
# nugget_cluster(do.call(cbind, L), k = K, max_nuggets = 40)$cluster  # cluster nuggets
# Wkmeans(dn, k = K); Whclust(dn, k = K)                              # weighted k-means / hclust
# M_ABCpp(L, numsim = 200, NC = 3, weighting = c("nugget", "nugget")) # nugget-weighted

## ----selectk------------------------------------------------------------------
sel <- SelectnrClusters(L, type = "data", distmeasure = DM, normalize = NM,
                        method = ME, nrclusters = 2:6)
sel$Optimal_Nr_of_CLusters

## ----evaluate-----------------------------------------------------------------
fit_a <- ADC(L, distmeasure = "euclidean", normalize = FALSE)
fit_b <- SNF(L, type = "data", distmeasure = DM, normalize = NM, method = ME)
compare_clusterings(fit_a$DistM, fit_b$DistM, NC = 3)$ARI
cluster_agreement(mosaic_labels(fit_b, 3), truth)

## ----dendro, fig.height = 4---------------------------------------------------
fit <- SNF(L, type = "data", distmeasure = DM, normalize = NM, method = ME)
hc  <- stats::as.hclust(fit$Clust)
plot(hc, labels = FALSE, main = "SNF consensus", xlab = "", sub = "")
stats::rect.hclust(hc, k = 3, border = 2:4)

## ----viz, eval = FALSE--------------------------------------------------------
# ComparePlot(list("ADC" = fit_a, "SNF" = fit_b), nrclusters = 3)
# SimilarityHeatmap(fit_b$DistM)
# ProfilePlot(L[[1]], mosaic_labels(fit_b, 3))

## ----charac, eval = FALSE-----------------------------------------------------
# CharacteristicFeatures(L, mosaic_labels(fit_b, 3))
# ReorderToReference(reference = truth, tocompare = mosaic_labels(fit_a, 3))

## ----bio, eval = FALSE--------------------------------------------------------
# dg <- DiffGenes(L, mosaic_labels(fit_b, 3), geneexpr = L[[1]])
# pa <- PathwayAnalysis(L, mosaic_labels(fit_b, 3), geneexpr = L[[1]])
# PlotPathways(pa)

## ----together, eval = FALSE---------------------------------------------------
# L    <- my_sources                                   # list of matrices, objects in rows
# k    <- SelectnrClusters(L, type = "data", distmeasure = rep("euclidean", length(L)),
#                          normalize = rep(FALSE, length(L)),
#                          method = rep(list(NULL), length(L)),
#                          nrclusters = 2:8)$Optimal_Nr_of_CLusters
# fit  <- SNF(L, type = "data", distmeasure = rep("euclidean", length(L)),
#             normalize = rep(FALSE, length(L)), method = rep(list(NULL), length(L)))
# lab  <- mosaic_labels(fit, k)
# CharacteristicFeatures(L, lab)                        # what defines the clusters

## ----session, echo = FALSE----------------------------------------------------
sessionInfo()

# =============================================================================
# plots-heatmap.r  - heatmap / box-plot comparison utilities
# =============================================================================

#' @title Box plots of one distance matrix categorized against another distance
#' matrix.
#'
#' @description Given two distance matrices, the function categorizes one distance matrix
#' and produces a box plot from the other distance matrix against the created
#' categories. The option is available to choose one of the plots or to have
#' both plots. The function also works on outputs from ADEC and CEC functions
#' which do not have distance matrices but incidence matrices.
#'
#'
#' @param Data1 The first data matrix, cluster outcome or distance matrix to be
#' plotted.
#' @param Data2 The second data matrix, cluster outcome or distance matrix to
#' be plotted.
#' @param type indicates whether the provided matrices in "List" are either data matrices, distance
#' matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
#' matrices is skipped and if type="clusters" the single source clustering is skipped.
#' Type should be one of "data", "dist" or "clusters".
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").
#' @param normalize	Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.
#' @param lab1 The label to plot for Data1.
#' @param lab2 The label to plot for Data2.
#' @param limits1 The limits for the categories of Data1.
#' @param limits2 The limits for the categories of Data2.
#' @param plot The type of plots: 1 - Plot the values of Data1 versus the
#' categories of Data2. 2 - Plot the values of Data2 versus the categories of
#' Data1. 3 - Plot both types 1 and 2.
#' @param StopRange Logical. Indicates whether the distance matrices with
#' values not between zero and one should be standardized to have so. If FALSE
#' the range normalization is performed. See \code{Normalization}. If TRUE, the
#' distance matrices are not changed. This is recommended if different types of
#' data are used such that these are comparable. Default is FALSE.
#' @param plottype Should be one of "pdf","new" or "sweave". If "pdf", a
#' location should be provided in "location" and the figure is saved there. If
#' "new" a new graphic device is opened and if "sweave", the figure is made
#' compatible to appear in a sweave or knitr document, i.e. no new device is
#' opened and the plot appears in the current device or document. Default is "new".
#' @param location If plottype is "pdf", a location should be provided in
#' "location" and the figure is saved there. Default is NULL.
#' @return One/multiple box plots.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
#'
#'
#' BoxPlotDistance(MCF7_F,MCF7_T,type="cluster",lab1="FP",lab2="TP",limits1=c(0.3,0.7),
#' limits2=c(0.3,0.7),plot=1,StopRange=FALSE,plottype="new", location=NULL)
#' }
#' @export BoxPlotDistance
BoxPlotDistance<-function(Data1,Data2,type=c('data','dist','clusters'),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),lab1,lab2,limits1=NULL,limits2=NULL,plot=1,StopRange=FALSE,plottype="new",location=NULL){
	if (!requireNamespace("ggplot2", quietly=TRUE)) stop("BoxPlotDistance() requires the suggested package 'ggplot2'.")
	if (!requireNamespace("gridExtra", quietly=TRUE)) stop("BoxPlotDistance() requires the suggested package 'gridExtra'.")
	type<-match.arg(type)
	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
		if(plottype=="sweave"){

		}
	}
	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}

	C1=D1=C2=D2=NULL
	if(type=='clusters'){

		Dist1<-Data1$DistM
		Dist2<-Data2$DistM


	}
	else if(type=='data'){
		Dist1<-Distance(Data1,distmeasure[1],normalize[1],method[1])
		Dist2<-Distance(Data2,distmeasure[2],normalize[2],method[2])
		DistL=list(Dist1,Dist2)
		for(i in 1:2){
			if(StopRange==FALSE & !(0<=min(DistL[[i]]) & max(DistL[[i]])<=1)){
				message("It was detected that a distance matrix had values not between zero and one. Range Normalization was performed to secure this. Put StopRange=TRUE if this was not necessary")
				DistL[[i]]=Normalization(DistL[[i]],method="Range")
			}
		}

	}
	else if(type=='dist'){
		Dist1=Data1
		Dist2=Data2
		DistL=list(Dist1,Dist2)
		for(i in 1:2){
			if(StopRange==FALSE &  !(0<=min(DistL[[i]]) & max(DistL[[i]])<=1)){
				message("It was detected that a distance matrix had values not between zero and one. Range Normalization was performed to secure this. Put StopRange=TRUE if this was not necessary")
				DistL[[i]]=Normalization(DistL[[i]],method="Range")
			}
		}
	}

	OrderNames=rownames(Dist1)
	Dist2=Dist2[OrderNames,OrderNames]

	Dist1lower <- Dist1[lower.tri(Dist1)]
	Dist2lower <- Dist2[lower.tri(Dist2)]

	Categorize<-function(Distlower,limits){
		Cat=c(rep(0,length(Distlower)))
		for(j in 1:(length(limits)+1)){
			if(j==1){
				Cat[Distlower<=limits[j]]=j
			}
			else if(j<=length(limits)){
				Cat[Distlower>limits[j-1] & Distlower<=limits[j]]=j
			}
			else{
				Cat[Distlower>limits[j-1]]=j
			}
		}
		Cat<-factor(Cat)
		return(Cat)

	}

	#plot2
	if(!(is.null(limits1))){
		Dist1cat<-Categorize(Dist1lower,limits1)

		dataBox2<-data.frame(D2=Dist2lower,C1=Dist1cat)
		p2<-ggplot2::ggplot(dataBox2,ggplot2::aes(factor(C1),D2)) #x,y
		p2<-p2+ggplot2::geom_boxplot(outlier.shape=NA)+ggplot2::geom_point(color="blue",size=2,shape=19,position="jitter",cex=1.5)+ggplot2::xlab(lab1)+ggplot2::ylab(lab2)
	}
	#plot1
	if(!(is.null(limits2))){
		Dist2cat<-Categorize(Dist2lower,limits2)
		dataBox1<-data.frame(D1=Dist1lower,C2=Dist2cat)
		p1<-ggplot2::ggplot(dataBox1,ggplot2::aes(factor(C2),D1)) #x,y
		p1<-p1+ggplot2::geom_boxplot(outlier.shape=NA)+ggplot2::geom_point(color="blue",size=2,shape=19,position="jitter",cex=1.5)+ggplot2::xlab(lab2)+ggplot2::ylab(lab1)

	}
	if(plot==3){
		if(plottype=="pdf"){
			location=paste(location,'_type3.pdf',sep="")
		}
		plottypein(plottype,location)
		gridExtra::grid.arrange(p1, p2, ncol=2,nrow=1)

	}
	else if(plot==1){
		if(plottype=="pdf"){
			location=paste(location,'_type1.pdf',sep="")
		}
		plottypein(plottype,location)
		print(p2)

	}
	else if(plot==2){
		if(plottype=="pdf"){
			location=paste(location,'_type2.pdf',sep="")
		}
		plottypein(plottype,location)
		print(p2)
	}

}


#' @title Determine the distance in a heatmap
#' @param Data1 The resulting clustering of method 1.
#' @param Data2 The resulting clustering of method 2.
#' @param names The names of the objects in the data sets. Default is NULL.
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @description Internal function of \code{HeatmapPlot}
distanceheatmaps<-function(Data1,Data2,names=NULL,nrclusters=7){
	ClustData1=stats::cutree(Data1,nrclusters) #original clusters (aggregated data clustering)
	ClustData2=stats::cutree(Data2,nrclusters) #clusters of changed method

	ClustData1=ClustData1[Data1$order]
	ClustData2=ClustData2[Data2$order]

	trueorder1=sort(Data1$order,index.return = TRUE)
	trueorder2=sort(Data2$order,index.return = TRUE)

	ordercolors=ClustData1
	order=seq(1,nrclusters)

	for (k in 1:length(unique(ClustData1))){
		select=which(ClustData1==unique(ClustData1)[k])
		ordercolors[select]=order[k]
	}

	ordercolors2=ClustData2


	for (k in 1:length(unique(ClustData2))){
		select=which(ClustData2==unique(ClustData2)[k])
		ordercolors2[select]=order[k]
	}

	ClustData1=ordercolors[trueorder1$ix]
	ClustData2=ordercolors2[trueorder2$ix]

	out=matrix(0,length(ClustData1),length(ClustData2)) #want the rows to be the other method and the columns to be the aggregated data clustering

	#names=names[Data2$order]

	rownames(out)=names
	colnames(out)=names

	for(i in 1:length(names)){
		focus=names[i] #defines the column

		label=ClustData2[i] #color of the cluster is defined by the original cluster that contains focus (1 to 7)

		for(j in 1:length(names)){ #go over the rows
			other=names[j]
			found=FALSE  #find cluster of other
			k=1
			while(found==FALSE & k<=nrclusters){
				label2=k
				if(other %in% names[ClustData1==label2]){
					othercluster=names[ClustData1==label2]
					found=TRUE
				}
				k=k+1
			}

			if(focus %in% othercluster){ #if other and focus still together: give it color of cluster defined by focus
				out[j,i]=label
			}
		}
	}
	return(out)
}


#' @title A heatmap of the comparison of two clustering results.
#'
#' @description A function to plot a heatmap of the agreement between two clustering
#' results.
#'
#' These are the clusters to which a comparison is made. A matrix is set up of
#' which the columns are determined by the ordering of clustering of method 2
#' and the rows by the ordering of method 1. Every column represent one object
#' just as every row and every column represent the color of its cluster. A
#' function visits every cell of the matrix. If the objects represented by the
#' cell are still together in a cluster, the color of the column is passed to
#' the cell. This creates the distance matrix which can be given to the
#' HeatmapCols function to create the heatmap.
#'
#' @param Data1 The resulting clustering of method 1.
#' @param Data2 The resulting clustering of method 2.
#' @param names The names of the objects in the data sets. Default is NULL.
#' @param nrclusters The number of clusters to cut the dendrogram in. Default is NULL.
#' @param cols A character vector with the colours for the clusters. Default is NULL.
#' @param plottype Should be one of "pdf","new" or "sweave". If "pdf", a
#' location should be provided in "location" and the figure is saved there. If
#' "new" a new graphic device is opened and if "sweave", the figure is made
#' compatible to appear in a sweave or knitr document, i.e. no new device is
#' opened and the plot appears in the current device or document. Default is "new".
#' @param location If plottype is "pdf", a location should be provided in
#' "location" and the figure is saved there. Default is NULL.
#' @return A heatmap based on the distance matrix created by the function with
#' the dendrogram of method 2 on top of the plot and the one from method 1 on
#' the left. The names of the objects are depicted on the bottom in the order
#' of clustering of method 2 and on the right by the ordering of method 1.
#' Vertically the cluster of method 2 can be seen while horizontally those of
#' method 1 are portrayed.
#' @examples
#' \dontrun{
#' data(fingerprintMat)
#' data(targetMat)
#' data(Colors1)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' clust="agnes",linkage="flexible",gap=FALSE,maxK=15)
#' MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' clust="agnes",linkage="flexible",gap=FALSE,maxK=15)
#'
#'
#' L=list(MCF7_F,MCF7_T)
#' names=c("FP","TP")
#'
#' HeatmapPlot(Data1=MCF7_T,Data2=MCF7_F,names=rownames(fingerprintMat)
#' ,nrclusters=7,cols=Colors1,plottype="new", location=NULL)
#' }
#'
#' @export HeatmapPlot
HeatmapPlot<-function(Data1,Data2,names=NULL,nrclusters=NULL,cols=NULL,plottype="new",location=NULL){
	if (!requireNamespace("gplots", quietly=TRUE)) stop("HeatmapPlot() requires the suggested package 'gplots'.")
	data1=Data1$Clust
	data2=Data2$Clust
	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
		if(plottype=="sweave"){

		}
	}
	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}
	DistM=distanceheatmaps(data1,data2,names,nrclusters)
	plottypein(plottype,location)
	gplots::heatmap.2(DistM,Rowv =stats::as.dendrogram(data1), Colv=stats::as.dendrogram(data2),trace="none",col=cols,key=FALSE)
	plottypeout(plottype)
}

#' @title A function to select a group of objects via the similarity heatmap.
#'
#' @description The function \code{HeatmapSelection} plots the similarity values between
#' objects. The plot is similar to the one produced by
#' \code{SimilarityHeatmap} but without the dendrograms on the sides. The
#' function is rather explorative and experimental and is to be used with some
#' caution. By clicking in the plot, the user can select a group of objects
#' of interest. See more in \code{Details}.
#'
#' A similarity heatmap is created in the same way as in
#' \code{SimilarityHeatmap}. The user is now free to select two points on the
#' heatmap. It is advised that these two points are in opposite corners of a
#' square that indicates a high similarity among the objects. The points do
#' not have to be the exact corners of the group of interest, a little
#' deviation is allowed as rows and columns of the selected subset of the
#' matrix with sum equal to 1 are filtered out. A sum equal to one, implies
#' that the compound is only similar to itself.
#'
#' The function is meant to be explorative but is experimental. The goal was to
#' make the selection of interesting objects easier as sometimes the labels
#' of the dendrograms are too distorted to be read. If the figure is exported
#' to a pdf file with an appropriate width and height, the labels can be become
#' readable again.
#'
#' @param Data The data of which a heatmap should be drawn.
#' @param type indicates whether the provided matrices in "List" are either data matrices, distance
#' matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
#' matrices is skipped and if type="clusters" the single source clustering is skipped.
#' Type should be one of "data", "dist" or "clusters".
#' @param distmeasure The distance measure. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to "tanimoto".
#' @param normalize	Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is NULL.
#' @param linkage Choice of inter group dissimilarity (character). Defaults to "flexible".
#' @param cutoff Optional. If a cutoff value is specified, all values lower are
#' put to zero while all other values are kept. This helps to highlight the
#' most similar objects. Default is NULL.
#' @param percentile Logical. The cutoff value can be a percentile. If one want
#' the cutoff value to be the 90th percentile of the data, one should specify
#' cutoff = 0.90 and percentile = TRUE. Default is FALSE.
#' @param dendrogram Optional. If the clustering results of the data is already
#' available and should not be recalculated, this results can be provided here.
#' Otherwise, it will be calculated given the data. This is necessary to have
#' the objects in their order of clustering on the plot. Default is NULL.
#' @param width The width of the plot to be made. This can be adjusted since
#' the default size might not show a clear picture. Default is 7.
#' @param height The height of the plot to be made. This can be adjusted since
#' the default size might not show a clear picture. Default is 7.
#' @return A heatmap with the names of the objects on the right and bottom.
#' Once points are selected, it will return the names of the objects that are
#' in the selected square provided that these show similarity among each other.
#' @examples
#'
#' \dontrun{
#' data(fingerprintMat)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55)
#'
#' HeatmapSelection(Data=MCF7_F$DistM,type="dist",cutoff=0.90,percentile=TRUE,
#' dendrogram=MCF7_F,width=7,height=7)
#' }
#'
#' @export HeatmapSelection
HeatmapSelection<-function(Data,type=c("data","dist","clust","sim"),distmeasure="tanimoto",normalize=FALSE,method=NULL,linkage="flexible",cutoff=NULL,percentile=FALSE,dendrogram=NULL,width=7,height=7){

	#create binary similarity heatmap first
	if(type=="data"){
		ClustData<-Cluster(Data=Data,distmeasure=distmeasure,normalize=normalize,method=method,clust="agnes",linkage=linkage,gap=FALSE,maxK=55,StopRange=FALSE)
		Data=ClustData$DistM
		type="dist"
	}


	if(type=="clust"){
		Dist=Data$DistM
		if(0<=min(Dist) & max(Dist)<=1){
			SimData=1-Dist
		}
		else{
			NormData=Normalization(Dist,method="Range")
			SimData=1-NormData
		}
		if(is.null(dendrogram)){
			dendrogram=Data
		}
	}

	else if(type=="dist"){
		if(0<=min(Data) & max(Data)<=1){
			SimData=1-Data
			if(is.null(dendrogram)){
				dendrogram=Cluster(Data=Data,type="dist",distmeasure="tanimoto",normalize=FALSE,method=NULL,clust="agnes",linkage="ward",gap=FALSE,maxK=55,StopRange=FALSE)
			}
		}
		else{
			NormData=Normalization(Data,method="Range")
			SimData=1-NormData
			if(is.null(dendrogram)){
				dendrogram=Cluster(Data=Data,type="dist",distmeasure="tanimoto",normalize=TRUE,method="Q",clust="agnes",linkage="ward",gap=FALSE,maxK=55,StopRange=FALSE)
			}
		}


	}
	else if(type=="sim"){
		SimData=Data
		if(0<=min(SimData) & max(SimData)<=1){
			if(is.null(dendrogram)){
				DistData=1-Data
				ClustData=Cluster(Data=DistData,type="dist",distmeasure="tanimoto",normalize=FALSE,method=NULL,clust="agnes",linkage="ward",gap=FALSE,maxK=55,StopRange=FALSE)
			}
		}
		else{
			if(is.null(dendrogram)){
				NormData=Normalization(Dist,method="Range")
				DistData=1-Data
				ClustData=Cluster(Data=DistData,type="dist",distmeasure="tanimoto",normalize=FALSE,method=NULL,clust="agnes",linkage="ward",gap=FALSE,maxK=55,StopRange=FALSE)
			}
		}
	}


	if(!is.null(cutoff)){
		if(percentile==TRUE){
			cutoff=stats::quantile(SimData[lower.tri(SimData)], cutoff)
		}

		SimData_bin <- ifelse(SimData<=cutoff,0,SimData) # Every value higher than the 90ieth percentile is kept, all other are put to zero
	}

	else{
		SimData_bin=SimData
	}



	dend <- stats::as.dendrogram(dendrogram$Clust)
	Ind <- stats::order.dendrogram(dend)

	SimData_bin=SimData_bin[Ind,Ind]

	#Layout<-rbind(4:3, 2:1)
	#lhei <- c(0.4, 4)
	#lwid <- c(0.4, 4)
	#layout(Layout, widths = lwid, heights = lhei, respect = FALSE)
	grDevices::dev.new(width=width,height=height)
	graphics::par(mar = c(9,7, 7, 9))
	graphics::image(x=1:nrow(SimData_bin),y=1:ncol(SimData_bin),z=t(SimData_bin),col=(grDevices::gray(seq(0.9,0,len=1000))),axes=FALSE,xlab="",ylab="")
	graphics::axis(1, 1:ncol(SimData_bin), labels = colnames(SimData_bin), las = 2, line =0, tick = 0, cex.axis = 0.6)
	graphics::axis(4, 1:nrow(SimData_bin), labels = rownames(SimData_bin), las = 2, line = 0, tick = 0, cex.axis = 0.6)

	points=graphics::locator(n=2,type="l")
	cols=c(floor(points$x[1]),ceiling(points$x[2]))
	rows=c(floor(points$y[1]),ceiling(points$y[2]))

	if(cols[1]>cols[2]){
		colseq=seq(cols[2],cols[1],1)
	}
	else{
		colseq=seq(cols[1],cols[2],1)
	}

	if(rows[1]>rows[2]){
		rowseq=seq(rows[2],rows[1],1)
	}
	else{
		rowseq=seq(rows[1],rows[2],1)
	}

	print(rowseq)
	print(colseq)
	SubsetData=SimData_bin[rowseq,colseq]
#	DelRows=rownames(SubsetData)[which(rowSums(SubsetData)==1)]
#	DelCols=colnames(SubsetData)[which(colSums(SubsetData)==1)]
#
#	if(length(DelRows)!=0 & length(DelCols)!=0){
#		Subset=SubsetData[-which(rownames(SubsetData)%in%c(DelRows,DelCols)),-which(colnames(SubsetData)%in%c(DelRows,DelCols))]
#	}
#	else if(length(DelRows)!=0 & length(DelCols)==0){
#		Subset=SubsetData[-which(rownames(SubsetData)%in%c(DelRows)),]
#
#	}
#	else if(length(DelRows)==0 & length(DelCols)!=0){
#		Subset=SubsetData[,-which(colnames(SubsetData)%in%c(DelCols))]
#
#	}
#	else if(length(DelRows)==0 & length(DelCols)==0){
#		Subset=SubsetData
#
#	}
#	SelComps=colnames(Subset)
	SelComps=colnames(SubsetData)

	return(SelComps)
}

#' @title A heatmap of similarity values between objects
#'
#' @description The function \code{SimilarityHeatmap} plots the similarity values between
#' objects. The darker the shade, the more similar objects are. The option
#' is available to set a cutoff value to highlight the most similar objects.
#'
#' @details If data is of type "clust", the distance matrix is extracted from the result
#' and transformed to a similarity matrix. Possibly a range normalization is
#' performed. If data is of type "dist", it is also transformed to a similarity
#' matrix and cluster is performed on the distances. If data is of type "sim",
#' the data is tranformed to a distance matrix on which clustering is
#' performed. Once the similarity mattrix is obtained, the cutoff value is
#' applied and a heatmap is drawn. If no cutoff value is desired, one can leave
#' the default NULL specification.
#'
#' @param Data The data of which a heatmap should be drawn.
#' @param type indicates whether the provided matrices in "List" are either data matrices, distance
#' matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
#' matrices is skipped and if type="clusters" the single source clustering is skipped.
#' Type should be one of "data", "dist" ,"sim" or "clusters".
#' @param distmeasure The distance measure. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to "tanimoto".
#' @param normalize	Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is NULL.
#' @param linkage Choice of inter group dissimilarity (character). Defaults to "flexible".
#' @param cutoff Optional. If a cutoff value is specified, all values lower are
#' put to zero while all other values are kept. This helps to highlight the
#' most similar objects. Default is NULL.
#' @param percentile Logical. The cutoff value can be a percentile. If one want
#' the cutoff value to be the 90th percentile of the data, one should specify
#' cutoff = 0.90 and percentile = TRUE. Default is FALSE.
#' @param plottype Should be one of "pdf","new" or "sweave". If "pdf", a
#' location should be provided in "location" and the figure is saved there. If
#' "new" a new graphic device is opened and if "sweave", the figure is made
#' compatible to appear in a sweave or knitr document, i.e. no new device is
#' opened and the plot appears in the current device or document. Default is "new".
#' @param location If plottype is "pdf", a location should be provided in
#' "location" and the figure is saved there. Default is NULL.
#' @return A heatmap with the names of the objects on the right and bottom
#' and a dendrogram of the clustering at the left and top.
#' @examples
#'
#' \dontrun{
#' data(fingerprintMat)
#'
#' MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
#' method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55)
#'
#' SimilarityHeatmap(Data=MCF7_F,type="clust",cutoff=0.90,percentile=TRUE)
#' SimilarityHeatmap(Data=MCF7_F,type="clust",cutoff=0.75,percentile=FALSE)
#'
#' }
#'
#' @export SimilarityHeatmap
SimilarityHeatmap<-function(Data,type=c("data","clust","sim","dist"),distmeasure="tanimoto",normalize=FALSE,method=NULL,linkage="flexible",cutoff=NULL,percentile=FALSE,plottype="new",location=NULL){
	if (!requireNamespace("gplots", quietly=TRUE)) stop("SimilarityHeatmap() requires the suggested package 'gplots'.")
	plottypein<-function(plottype,location){
		if(plottype=="pdf" & !(is.null(location))){
			grDevices::pdf(paste(location,".pdf",sep=""))
		}
		if(plottype=="new"){
			grDevices::dev.new()
		}
		if(plottype=="sweave"){

		}
	}
	plottypeout<-function(plottype){
		if(plottype=="pdf"){
			grDevices::dev.off()
		}
	}

	if(type=="data"){
		ClustData<-Cluster(Data=Data,distmeasure=distmeasure,normalize=normalize,method=method,clust="agnes",linkage=linkage,gap=FALSE,maxK=55,StopRange=FALSE)
		Data=ClustData$DistM
		type="dist"
	}


	if(type=="clust"){
		Dist=Data$DistM
		if(0<=min(Dist) & max(Dist)<=1){
			SimData=1-Dist
		}
		else{
			NormData=Normalization(Dist,method="Range")
			SimData=1-NormData
		}

		ClustData=Data
	}

	else if(type=="dist"){
		if(0<=min(Data) & max(Data)<=1){
			SimData=1-Data
			ClustData=Cluster(Data=Data,type="dist",distmeasure="tanimoto",normalize=FALSE,method=NULL,clust="agnes",linkage="ward",gap=FALSE,maxK=55,StopRange=FALSE)

		}
		else{
			NormData=Normalization(Data,method="Range")
			SimData=1-NormData
			ClustData=Cluster(Data=Data,type="dist",distmeasure="tanimoto",normalize=TRUE,method="Q",clust="agnes",linkage="ward",gap=FALSE,maxK=55,StopRange=FALSE)

		}


	}
	else if(type=="sim"){
		SimData=Data
		if(0<=min(SimData) & max(SimData)<=1){
			DistData=1-Data
			ClustData=Cluster(Data=DistData,type="dist",distmeasure="tanimoto",normalize=FALSE,method=NULL,clust="agnes",linkage="ward",gap=FALSE,maxK=55,StopRange=FALSE)

		}
		else{
			NormData=Normalization(Dist,method="Range")
			DistData=1-Data
			ClustData=Cluster(Data=DistData,type="dist",distmeasure="tanimoto",normalize=FALSE,method=NULL,clust="agnes",linkage="ward",gap=FALSE,maxK=55,StopRange=FALSE)
		}
	}


	if(!is.null(cutoff)){
		if(percentile==TRUE){
			cutoff=stats::quantile(SimData[lower.tri(SimData)], cutoff)
		}

		SimData_bin <- ifelse(SimData<=cutoff,0,SimData) # Every value higher than the 90ieth percentile is kept, all other are put to zero
	}

	else{
		SimData_bin=SimData
	}

	plottypein(plottype,location)
	gplots::heatmap.2(SimData_bin,
			Rowv = stats::as.dendrogram(stats::as.hclust(ClustData$Clust)), Colv=stats::as.dendrogram(stats::as.hclust(ClustData$Clust)),trace="none",
			col=(grDevices::gray(seq(0.9,0,len=1000))),
			cexRow=0.6, cexCol=0.6,
			margins=c(9,9),
			key=FALSE,
			keysize=0.4,
			symkey=FALSE,
			sepwidth=c(0.01,0.01),
			sepcolor="black",
			colsep=c(0,ncol(SimData_bin)),
			rowsep=c(0,nrow(SimData_bin))
	)
	plottypeout(plottype)

}

test_that("BoxPlotDistance is exported and callable", {
  expect_true(is.function(BoxPlotDistance))
})

test_that("distanceheatmaps internal helper exists", {
  expect_true(is.function(MosaiClusteR:::distanceheatmaps))
})

test_that("HeatmapPlot is exported and callable", {
  expect_true(is.function(HeatmapPlot))
})

test_that("HeatmapSelection is exported and callable", {
  expect_true(is.function(HeatmapSelection))
})

test_that("SimilarityHeatmap is exported and callable", {
  expect_true(is.function(SimilarityHeatmap))
})

test_that("BoxPlotDistance requires its suggested plotting packages", {
  testthat::skip_if_not_installed("ggplot2")
  testthat::skip_if_not_installed("gridExtra")
  data(mosaic_toy)
  grDevices::pdf(tempfile())
  on.exit(grDevices::dev.off(), add = TRUE)
  # type="dist" path with two distance matrices avoids interactive device use
  D1 <- Distance(mosaic_toy$List[[1]], distmeasure = "euclidean")
  D2 <- Distance(mosaic_toy$List[[2]], distmeasure = "euclidean")
  expect_error(suppressMessages(suppressWarnings(
    BoxPlotDistance(D1, D2, type = "dist", lab1 = "S1", lab2 = "S2",
                    limits1 = c(0.3, 0.7), plot = 1,
                    StopRange = FALSE, plottype = "sweave")
  )), NA)
})

test_that("SimilarityHeatmap draws on a clustering result", {
  testthat::skip_if_not_installed("gplots")
  data(mosaic_toy)
  grDevices::pdf(tempfile())
  on.exit(grDevices::dev.off(), add = TRUE)
  cl <- Cluster(mosaic_toy$List[[1]], type = "data",
                distmeasure = "euclidean", normalize = FALSE,
                clust = "agnes", linkage = "ward")
  expect_error(suppressMessages(suppressWarnings(
    SimilarityHeatmap(cl, type = "clust", plottype = "sweave")
  )), NA)
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

mcpp_rf_sample <- function(populationSize, sampleSize, useWeights = FALSE, withReplacement = TRUE, weights = NULL) {
    .Call(`_MosaiClusteR_mcpp_rf_sample`, populationSize, sampleSize, useWeights, withReplacement, weights)
}

mcpp_sample_weighted_no_replace <- function(z, ng) {
    .Call(`_MosaiClusteR_mcpp_sample_weighted_no_replace`, z, ng)
}

count_coclustering_cpp <- function(res) {
    .Call(`_MosaiClusteR_count_coclustering_cpp`, res)
}

# =============================================================================
# ABCpp.R
# Single-source ABC inner loop -- the canonical, C++-accelerated ABC engine.
# Main function: ABCpp.SingleInMultiple
#
# This retains the ORIGINAL implementation (Osang'ir, based on Amaratunga,
# Cabrera & Kovtun 2008): the per-iteration weighted feature subsample is drawn
# in pure R with sample.int(prob = zf); clustering is done with
# fastcluster::hclust; the C++ acceleration lives in the consensus step
# (count_coclustering_cpp, called by f.clustABC.MultiSource in M_ABCpp.R).
#
# Corrections carried over from the original engine:
#   B1  c solved adaptively via uniroot (.solve_weight_c), not hardcoded nf/183
#   B2  feature subsampling (Step 3) always runs; bag only controls Step 2
#   B3  NC defaults to floor(sqrt(N))
#   B4  g = floor(sqrt(G)) as an explicit integer
#   B5  as.dist() applied before hclust
#
# NEW in 0.1.1:
#   * weighting = "nugget" -- data-nugget feature weighting (see the weighting
#     block below); the classic "var"/"cv" schemes are unchanged.
#   * nfeat -- optional override of the feature-subsample size g.
# =============================================================================

#' Single-source Aggregating Bundles of Clusters (ABC), C++-accelerated engine
#'
#' The canonical single-source ABC engine of
#' \insertCite{Amaratunga2008}{MosaiClusteR}. Over \code{numsim} iterations it
#' (optionally) bootstraps the objects, draws a weighted subsample of features,
#' clusters the sub-matrix with \code{fastcluster::hclust}, and records each
#' selected object's base-cluster label. \code{\link{M_ABCpp}} turns the stacked
#' label matrix into a consensus dissimilarity using the C++ co-clustering
#' kernel.
#'
#' @section Feature weighting (incl. data nuggets):
#' Features are subsampled with probability proportional to a rank-transformed
#' importance score:
#' \itemize{
#'   \item \code{"var"} -- feature variance (the classic ABC choice);
#'   \item \code{"cv"}  -- coefficient of variation;
#'   \item \code{"nugget"} -- \emph{NEW}: the data-nugget between-group weighted
#'     variance (\code{\link{create_data_nuggets}}), a robust, Big-Data-friendly
#'     alternative to the raw variance;
#'   \item anything else -- equal weights.
#' }
#'
#' @param data Numeric matrix with \strong{objects (samples) in rows} and
#'   features in columns (the MosaiClusteR convention; transposed internally to
#'   the features-by-objects layout of the original engine).
#' @param distmeasure Distance measure passed to \code{\link{Distance}}.
#' @param weighting \code{"var"}, \code{"cv"}, \code{"nugget"} or equal.
#' @param normalize Optional normalisation (see \code{\link{Normalization}}).
#' @param gr Unused; kept for interface compatibility.
#' @param bag Logical; bootstrap the objects each iteration (Step 2).
#' @param numsim Number of iterations \eqn{R}.
#' @param numvar Kept for backward compatibility; superseded by \code{nfeat}.
#' @param linkage Agglomeration method for \code{fastcluster::hclust}.
#' @param NC Base-cluster count (\code{NULL} = \eqn{\lfloor\sqrt{N}\rfloor}).
#' @param mds Unused placeholder.
#' @param nfeat Optional feature-subsample size. \code{NULL} (default) uses the
#'   classic \eqn{g=\lfloor\sqrt{G}\rfloor}; a value \eqn{\ge 1} sets an absolute
#'   count (raise it for low-dimensional sources where most features matter).
#' @param nugget_type,nugget_args Passed to the data-nugget weighting when
#'   \code{weighting = "nugget"}.
#' @return A data frame, \code{numsim} rows by one column per object; entries are
#'   base-cluster labels (\code{0} = object not selected that iteration).
#' @references \insertAllCited{}
#' @seealso \code{\link{M_ABCpp}}, \code{\link{create_data_nuggets}}
#' @examples
#' data(mosaic_toy)
#' lab <- ABCpp.SingleInMultiple(mosaic_toy$List[[1]], numsim = 40, NC = 3,
#'                               weighting = "nugget")
#' dim(lab)
#' @export
ABCpp.SingleInMultiple <- function(data,
                                   distmeasure = "euclidean",
                                   weighting   = "var",
                                   normalize   = NULL,
                                   gr          = c(),
                                   bag         = TRUE,
                                   numsim      = 1000,
                                   numvar      = 100,
                                   linkage     = "ward.D2",
                                   NC          = NULL,
                                   mds         = FALSE,
                                   nfeat       = NULL,
                                   nugget_type = "between",
                                   nugget_args = list()) {

  # ---- input orientation -----------------------------------------------------
  # MosaiClusteR feeds objects-in-rows; the original engine works on a
  # features-by-objects matrix, so we transpose once here and keep the original
  # internal logic unchanged below.
  data <- as.matrix(data)
  samp_names <- rownames(data)
  X  <- t(data)        # X: features (rows) x objects (columns)
  ns <- ncol(X)        # N objects
  nf <- nrow(X)        # G features

  # ---- Step 4: base-cluster count  n = floor(sqrt(N))                 [B3] ----
  if (is.null(NC)) NC <- max(2L, floor(sqrt(ns)))
  NC <- max(2L, min(as.integer(NC), ns - 1L))

  # ---- Step 3: feature count  g = floor(sqrt(G))                      [B4] ----
  # (nfeat overrides g when supplied; useful for low-dimensional sources.)
  g <- if (is.null(nfeat)) max(2L, floor(sqrt(nf))) else max(2L, min(nf, as.integer(nfeat)))

  # ---- Step 1: feature weights (computed once, invariant over iterations)[B1] ----
  zf <- rep(1.0, nf)
  if (weighting == "var") {
    # classic ABC: rank features by variance; top features get more mass
    vars <- matrixStats::rowVars(X, na.rm = TRUE)
    cc   <- .solve_weight_c(nf)
    zf   <- 1.0 / (rank(-vars, ties.method = "first") + cc)
  } else if (weighting == "cv") {
    # coefficient of variation
    mn   <- matrixStats::rowMeans2(X, na.rm = TRUE)
    mn[abs(mn) < .Machine$double.eps] <- .Machine$double.eps
    cvs  <- matrixStats::rowSds(X, na.rm = TRUE) / abs(mn)
    cc   <- .solve_weight_c(nf)
    zf   <- 1.0 / (rank(-cvs, ties.method = "first") + cc)
  } else if (weighting == "nugget") {
    # NEW (0.1.1): data-nugget feature weighting. Compress the SAMPLES into
    # weighted data nuggets, then score each feature by how strongly it separates
    # the nugget centres (size-weighted between-nugget variance). Robust to
    # outliers and scales to large N. Drops straight into the same rank scheme.
    dn    <- do.call(create_data_nuggets, c(list(x = t(X)), nugget_args))  # t(X): objects x features
    score <- nugget_feature_weights(dn, type = nugget_type)
    cc    <- .solve_weight_c(nf)
    zf    <- 1.0 / (rank(-score, ties.method = "first") + cc)
  }
  zf <- zf / sum(zf)   # normalise to a probability vector over features

  ok_lnk <- c("single", "complete", "average", "mcquitty",
              "ward", "ward.D", "ward.D2", "centroid", "median")
  if (!(linkage %in% ok_lnk))
    stop("'linkage' must be one of: ", paste(ok_lnk, collapse = ", "))

  # label matrix: rows = iterations, cols = objects; 0 = object not selected
  res <- matrix(0L, nrow = numsim, ncol = ns,
                dimnames = list(paste0("iter_", seq_len(numsim)), samp_names))

  for (i in seq_len(numsim)) {

    # ---- Step 2: bootstrap sample selection                           [B2] ----
    if (bag) {
      ids <- sort(unique(sample.int(ns, size = ns, replace = TRUE)))
    } else {
      ids <- seq_len(ns)
    }
    dat_s <- X[, ids, drop = FALSE]

    # ---- Step 3: weighted feature subsample (pure R, prob = zf) -------------
    idf <- sort(sample.int(nf, size = g, replace = FALSE, prob = zf))
    dat <- dat_s[idf, , drop = FALSE]

    # ---- Step 4: cluster the g x N* sub-matrix                        [B5] ----
    DistM <- as.matrix(Distance(t(dat), distmeasure = distmeasure, normalize = normalize))
    # Robustness: the Gower-based distance is undefined (NaN) when a sampled
    # feature is constant across the selected objects (common with sparse data).
    # Fall back to a plain Euclidean distance for that iteration so the run never
    # aborts; the fallback is only reached when the primary distance fails.
    if (anyNA(DistM) || any(!is.finite(DistM))) DistM <- as.matrix(stats::dist(t(dat)))
    HClust <- fastcluster::hclust(stats::as.dist(DistM), method = linkage)
    labels <- as.integer(stats::cutree(HClust, k = NC))

    # ---- Step 5: record the labels of the selected objects ------------------
    res[i, ids] <- labels
  }

  as.data.frame(res, stringsAsFactors = FALSE)
}

#' @title Similarity network fusion
#'
#' @description Similarity Network Fusion (SNF) is a similarity-based multi-source clustering technique. SNF consists of two steps. In the initial step a similarity network is set up for each data matrix. The network is the visualization of the similarity matrix as a weighted graph with the objects as vertices and the pairwise similarities as weights on the edges. In the network-fusion step, each network is iteratively updated with information of the other network which results in more alike networks every time. This eventually converges to a single network.
#' @export
#' @param List A list of data matrices of the same type. It is assumed the rows are corresponding with the objects.
#' @param type indicates whether the provided matrices in "List" are either data matrices, distance
#' matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
#' matrices is skipped and if type="clusters" the single source clustering is skipped.
#' Type should be one of "data", "dist" or "clusters".
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").
#' @param normalize	Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.
#' @param StopRange Logical. Indicates whether the distance matrices with values not between zero and one should be standardized to have so.
#' If FALSE the range normalization is performed. See \code{Normalization}. If TRUE, the distance matrices are not changed.
#' This is recommended if different types of data are used such that these are comparable. Default is FALSE.
#' @param NN The number of neighbours to be used in the procedure. Defaults to 20.
#' @param mu The parameter epsilon. The value is recommended to be between 0.3 and 0.8. Defaults to 0.5.
#' @param T The number of iterations.
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character) for the final clustering. Defaults to "ward".
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible"
#' @return The returned value is a list with the following three elements.
#' \item{FusedM }{The fused similarity matrix}
#' \item{DistM }{The distance matrix computed by subtracting FusedM from one}
#' \item{Clust}{The resulting clustering}
#' The value has class 'SNF'.
#' @details If r is specified and nrclusters is a fixed number, only a random sampling of the features will be performed for the t iterations (ADECa). If r is NULL and the nrclusters is a sequence, the clustering is performedon all features and the dendrogam is divided into clusters for the values of nrclusters (ADECb). If both r is specified and nrclusters is a sequence, the combination is performed (ADECc).
#' After every iteration, either be random sampling, multiple divisions of the dendrogram or both, an incidence matrix is set up. All incidence matrices are summed and represent the distance matrix on which a final clustering is performed.
#' @references
#' \insertRef{Wang2014a}{MosaiClusteR}
SNF=function(List,type=c("data","dist","clusters"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),StopRange=FALSE,NN=20,mu=0.5,T=20,clust="agnes",linkage="ward",alpha=0.625){
  
  #Checking required data types and methods:
  if (!requireNamespace("SNFtool", quietly = TRUE))
    stop("SNF() requires the suggested package 'SNFtool'. ",
         "Install it with install.packages('SNFtool').")
  if(!is.list(List)){
    stop("Data must be of type list")
  }
  
  if(mu<0.3 | mu >0.8){
    message("Warning: mu is recommended to be between 0.3 and 0.8 for the SNF method. Default is 0.5.")
  }
  
  
  CheckDist<-function(Dist,StopRange){
    if(StopRange==FALSE & !(0<=min(Dist) & max(Dist)<=1)){
      #message("It was detected that a distance matrix had values not between zero and one. Range Normalization was performed to secure this. Put StopRange=TRUE if this was not necessary")
      Dist=Normalization(Dist,method="Range")
    }
    else{
      Dist=Dist
    }
  }
  
  
  #STEP 1: Distance Matrices
  if(type=="data"){
    OrderNames=rownames(List[[1]])
    for(i in 1:length(List)){
      List[[i]]=List[[i]][OrderNames,,drop=FALSE]
    }
    DistM=lapply(seq(length(List)),function(i) Distance(List[[i]],distmeasure[i],normalize[i],method[i]))
    DistM=lapply(seq(length(DistM)),function(i) CheckDist(DistM[[i]],StopRange))
  }
  else if(type=="dist"){
    OrderNames=rownames(List[[1]])
    for(i in 1:length(List)){
      List[[i]]=List[[i]][OrderNames,OrderNames]
    }
    DistM=List
    DistM=lapply(seq(length(DistM)),function(i) CheckDist(DistM[[i]],StopRange))
  }
  else{
    DistM=lapply(seq(length(List)),function(i) return(List[[i]]$DistM))
    DistM=lapply(seq(length(DistM)),function(i) CheckDist(DistM[[i]],StopRange))
    
    OrderNames=rownames(DistM[[1]])
    for(i in 1:length(DistM)){
      DistM[[i]]=DistM[[i]][OrderNames,OrderNames]
    }
  }
  
  
  #STEP 2: Affinity Matrices
  
  AffM=lapply(seq(length(List)), function(i) SNFtool::affinityMatrix(DistM[[i]], NN, mu))
  
  #STEP 3: Fuse Networks Into 1 Single Network
  
  SNF_FusedM=SNFtool::SNF(AffM, NN, T)
  rownames(SNF_FusedM)=rownames(List[[1]])
  colnames(SNF_FusedM)=rownames(List[[1]])
  Dist=1-SNF_FusedM
  
  #STEP 4: Perform Hierarchical Clustering with WARD Link
  
  HClust = cluster::agnes(Dist,diss=TRUE,method=linkage,par.method=alpha)
  
  
  #Output= list with the fused matrix and the performed clustering
  out=list(SNF_FusedM=SNF_FusedM,DistM=Dist,Clust=HClust)
  attr(out,'method')<-'SNF'
  return(out)
}

#' @title Hybrid Bipartite Graph Formulation
#'
#' @description The \code{HBGF} function implements the Hybrid Bipartite Graph
#' Formulation, a graph-based consensus clustering method. A bipartite graph is
#' constructed from the cluster ensemble (objects on one side, clusters on the
#' other) and partitioned with a spectral method based on the singular value
#' decomposition followed by k-means.
#'
#' @export
#' @param List A list of data matrices. It is assumed the rows are corresponding with the objects.
#' @param type Indicates whether the provided matrices in "List" are either data matrices, distance
#' matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
#' matrices is skipped and if type="clust" the single source clustering is skipped.
#' Type should be one of "data", "dist" or "clust".
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").
#' @param normalize Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".
#' @param nrclusters The number of clusters to divide each individual dendrogram in. Default is c(7,7) for two data sets.
#' @param gap Logical. Whether the optimal number of clusters should be determined with the gap statistic. Default is FALSE.
#' @param maxK The maximal number of clusters to investigate in the gap statistic. Default is 15.
#' @param graphPartitioning The graph partitioning method. Currently "Spec" (spectral) is supported.
#' @param optimalk An estimate of the final optimal number of clusters. Default is 7.
#' @return The returned value is a list of two elements:
#' \item{DistM}{A NULL object}
#' \item{Clust}{The resulting clustering}
#' The value has class 'Ensemble'.
#' @references Fern, X. Z. and Brodley, C. E. (2004). Solving cluster ensemble
#' problems by bipartite graph partitioning. Proceedings of the Twenty-First
#' International Conference on Machine Learning (ICML).
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' L <- mosaic_toy$List
#' HBGF_toy <- HBGF(List = L, type = "data",
#'                  distmeasure = c("euclidean", "euclidean"),
#'                  normalize = c(FALSE, FALSE), method = c(NULL, NULL),
#'                  clust = "agnes", linkage = c("ward", "ward"),
#'                  nrclusters = c(7, 7), graphPartitioning = "Spec", optimalk = 7)
#' }
HBGF<-function(List,type=c("data","dist","clust"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),clust = "agnes", linkage = c("flexible","flexible"),alpha=0.625,nrclusters=c(7,7),gap = FALSE, maxK = 15,graphPartitioning="Spec",optimalk=7){
	# Step 1: Generate several clustering results on the objects

	if(type=="data"){

		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type="data",distmeasure[i],normalize[i],method[i],clust,linkage[i],alpha,StopRange=TRUE))

		Dist=lapply(seq(length(List)),function(i) Clusterings[[i]]$DistM)

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=clusters
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))
	}

	else if(type=="dist"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,OrderNames]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type,distmeasure[i],normalize=FALSE,method=NULL,clust,linkage[i],alpha,StopRange=TRUE))

		Dist=List

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=ceiling(mean(clusters))
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))


	}
	else if(type=="clust"){

		Clusterings=List

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		if(is.null(nrclusters)){
			stop("Please specify a number of clusters")
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))

	}
	#### Step 1 Complete

	# Step 2: construct a graph G from the cluster ensemble: based on matlab code from Brodley and Fern (http://web.engr.oregonstate.edu/~xfern/)

	ClusterEnsembles=as.data.frame(Clusters)
	rownames(ClusterEnsembles)=rownames(List[[1]])
	for(i in 1:ncol(ClusterEnsembles)){
		colnames(ClusterEnsembles)[i]=paste("Solution_",i,sep="")
	}


	A=lapply(seq(length(Clusters)),function(i) stats::model.matrix(~as.factor(Clusters[[i]])-1))
	A=Reduce(cbind,A)
	rownames(A)=rownames(List[[1]])
	for(c in 1:ncol(A)){
		colnames(A)[c]=paste("Cluster ",c,sep="")
	}



	#Need W or just the connectivity matrix A? Ferns Algorthym uses A
	if(graphPartitioning=="Spec"){
		D<-diag(sqrt(colSums(A)))
		L<-A%*%solve(D)
		SingularValues=svd(L,nu=optimalk,nv=optimalk)
		U=SingularValues$u
		V=SingularValues$v

		U=U/matrix(rep(sqrt(rowSums(U^2)),optimalk),nrow=nrow(A),ncol=optimalk,byrow=FALSE)
		V=V/matrix(rep(sqrt(rowSums(V^2)),optimalk),nrow=ncol(A),ncol=optimalk,byrow=FALSE)

		permutated=gtools::permute(1:nrow(A))
		centers=U[permutated[1],,drop=FALSE]

		c=rep(0,nrow(A))
		c[permutated[1]]=2*optimalk

		#finsih this last for loop

		for(j in 2:optimalk){

			c=c+abs(U%*%t(centers[j-1,,drop=FALSE]))
			m = which.min(c)
			centers = rbind(centers,U[m,])
			c[m] = 2*optimalk

		}

		#k-means clustering

		clusterid=try(stats::kmeans(x=rbind(U,V),centers=centers,iter.max=200),silent=TRUE)
		if(inherits(clusterid,"try-error")){
			clusterid=try(stats::kmeans(x=rbind(U,V),centers=length(centers),iter.max=200),silent=TRUE)
		}
		Clusters=clusterid$cluster[1:nrow(A)]
		names(Clusters)=rownames(A)

		clusters=unique(Clusters)
		order=c()
		for(j in clusters){
			order=c(order,which(Clusters==j))
		}

		order.lab=as.character(order)

	}

	Out=list(DistM=NULL,Clust=list(order=order,order.lab=order.lab,Clusters=Clusters))
	attr(Out,"method")="Ensemble"
	return(Out)

}


#' @title Clustering aggregation
#'
#' @description The \code{ClusteringAggregation} includes the ensemble clustering methods Balls, Agglomerative (Aggl.) and Furthest which are graph-based consensus methods.
#'
#' @export
#' @param List A list of data matrices. It is assumed the rows are corresponding with the objects.
#' @param type indicates whether the provided matrices in "List" are either data matrices, distance
#' matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
#' matrices is skipped and if type="clust" the single source clustering is skipped.
#' Type should be one of "data", "dist" or "clust".
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").
#' @param normalize	Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".
#' @param nrclusters The number of clusters to divide each individual dendrogram in. Default is c(7,7) for two data sets.
#' @param gap Logical. Whether the optimal number of clusters should be determined with the gap statistic. Default is FALSE.
#' @param maxK The maximal number of clusters to investigate in the gap statistic. Default is 15.
#' @param agglMethod The method to be performed: "Balls","Aggl","Furthest" or "LocalSearch".
#' @param improve Logical. If TRUE, a local search is performed to improve the obtained results. Default is TRUE.
#' @param distThresh_B  A distance threshold for the Balls algoritme. Default is 0.5.
#' @param distThresh_A A distance threshold for the Aggl. algoritme. Default is 0.8.
#' @details Gionis, Mannila and Tsaparas (2007) propose heuristic algorithms in order to find a solution for the consensus problem. In a first step, a
#' weighted graph is built from the objects with weights between two vertices determined by the fraction of clusterings that place the two vertices
#' in different clusters. In a second step, an algorithm searches for the partition that minimizes the total number of disagreements with the given
#' partitions. The Balls algorithm is an iterative process which finds a cluster for the consensus partition in each iteration. For each object,
#' all objects at a distance of at most 0.5 are collected and the average distance of this set to the object of interest is calculated. If the
#' average distance is less or equal to a parameter the objects form a cluster; otherwise the object forms a singleton. The Agglomerative
#' (Aggl.) algorithm starts by considering every object as a singleton cluster. Next, the two closest clusters are merged if the average distance
#' between the clusters is less than 0.5. If there are no two clusters with an average distance smaller than 0.5, the algorithm stops and returns
#' the created clusters as a solution. The Furthest algorithm starts by placing all objects into a single cluster. In each iteration, the pair of
#' objects that are the furthest apart are considered as new cluster centers. The remaining objects are appointed to the center that increases the
#' cost of the partition the least and the new cost is computed. The cost is the sum of the all distances between the obtained partition and the
#' partitions in the ensemble. The iteration continues until the cost of the new partition is higher than the previous partition.
#' @return The returned value is a list of two elements:
#' \item{DistM}{A NULL object}
#' \item{Clust}{The resulting clustering}
#' The value has class 'Ensemble'.
#' @references Gionis, A., Mannila, H. and Tsaparas, P. (2007). Clustering
#' aggregation. ACM Transactions on Knowledge Discovery from Data, 1(1), 4-es.
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' L <- mosaic_toy$List
#' Aggl_toy <- ClusteringAggregation(List = L, type = "data",
#'   distmeasure = c("euclidean", "euclidean"), normalize = c(FALSE, FALSE),
#'   method = c(NULL, NULL), clust = "agnes", linkage = c("ward", "ward"),
#'   alpha = 0.625, nrclusters = c(7, 7), agglMethod = "Aggl",
#'   improve = TRUE, distThresh_B = 0.5, distThresh_A = 0.8)
#' }
ClusteringAggregation<-function(List,type=c("data","dist","clust"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),clust = "agnes", linkage = c("flexible","flexible"),alpha=0.625,nrclusters=c(7,7),gap = FALSE, maxK = 15,agglMethod=c("Balls","Aggl","Furthest","LocalSearch"),improve=TRUE,distThresh_B=0.5,distThresh_A=0.8){
	# Step 1: Generate several clustering results on the objects

	if(type=="data"){

		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type="data",distmeasure[i],normalize[i],method[i],clust,linkage[i],alpha,StopRange=TRUE))

		Dist=lapply(seq(length(List)),function(i) Clusterings[[i]]$DistM)


		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=clusters
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))
	}

	else if(type=="dist"){
		OrderNames=rownames(List[[1]])
		for(i in 1:length(List)){
			List[[i]]=List[[i]][OrderNames,OrderNames]
		}

		Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type,distmeasure[i],normalize=FALSE,method=NULL,clust,linkage[i],alpha,StopRange=TRUE))

		Dist=List

		if(is.null(nrclusters)){
			if(gap==FALSE){
				stop("Specify a number of clusters of put gap to TRUE")
			}
			else{
				clusters=sapply(seq(length(List)),function(i) Clusterings[[i]]$k$Tibs2001SEmax)
				nrclusters=ceiling(mean(clusters))
			}
		}

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))


	}
	else if(type=="clust"){

		Clusterings=List

		for(i in 1:length(Clusterings)){
			names(Clusterings)[i]=paste("Clust",i,sep=' ')
		}

		if(is.null(nrclusters)){
			stop("Please specify a number of clusters")
		}

		Clusters=lapply(seq(length(Clusterings)),function(i) stats::cutree(Clusterings[[i]]$Clust,k=nrclusters[i]))

	}

	# Step 2: Set up the distance matrix X
	H=list()
	for(i in 1:length(List)){
		Bin=matrix(0,nrow=max(Clusters[[i]]),ncol=length(Clusters[[i]]))
		for(j in 1:nrow(Bin)){
			Bin[j,which(Clusters[[i]]==j)]=1
		}
		H[[i]]=as.data.frame(Bin)
	}
	T=do.call(rbind,H)
	S=t(as.matrix(T))%*%(as.matrix(T))
	S=S/length(List)
	X=1-S
	if(type!="clust"){
		rownames(X)=rownames(List[[1]])
		colnames(X)=rownames(List[[1]])
	}
	else{
		rownames(X)=rownames(List[[1]]$DistM)
		colnames(X)=rownames(List[[1]]$DistM)
	}

	# Step 3: The specified algorithm on the distance matrix X

	if(agglMethod=="Balls"){
		AggClusters=list()
		SortedVertices=sort(rowSums(X))
		while(length(SortedVertices)>0){
			u=SortedVertices[1]
			B=which(SortedVertices[-c(1)]<=u+0.5)
			if(length(B)>0){
				AvDistBtoU=mean(X[which(rownames(X)==names(u)),which(colnames(X)%in%names(B))])

				if(AvDistBtoU<=distThresh_B){
					AggClusters[[length(AggClusters)+1]]=c(names(u),names(B))
					SortedVertices=SortedVertices[-which(names(SortedVertices)%in%c(names(u),names(B)))]
				}
				else{
					AggClusters[[length(AggClusters)+1]]=names(u)
					SortedVertices=SortedVertices[-which(names(SortedVertices)%in%c(names(u)))]
				}
			}
			else{
				AggClusters[[length(AggClusters)+1]]=names(u)
				SortedVertices=SortedVertices[-which(names(SortedVertices)%in%c(names(u)))]
			}
		}

		nrclusters=length(AggClusters)
		order.lab=unlist(AggClusters)

		if(type!="clust"){
			order=match(order.lab,rownames(List[[1]]))
		}
		else{
			order=match(order.lab,rownames(List[[1]]$DistM))
		}

		clusters=sapply(1:nrclusters, function(i) rep(i,length(AggClusters[[i]])))
		Clusters=unlist(clusters)
		names(Clusters)=order.lab

		if(type!="clust"){
			Clusters=Clusters[rownames(List[[1]])]
		}
		else{
			Clusters=Clusters[rownames(List[[1]]$DistM)]
		}

		if(improve==TRUE){agglMethod="LocalSearch"}
	}

	if(agglMethod=="Aggl"){
		AvDist<-function(DistMat, pairs,temp_clusters){
			Cluster1=names(temp_clusters)[which(temp_clusters==pairs[1])]
			Cluster2=names(temp_clusters)[which(temp_clusters==pairs[2])]
			AvDist1to2=mean(DistMat[which(rownames(DistMat)%in%Cluster2),which(colnames(DistMat)%in%Cluster1)])

			return(AvDist1to2)

		}

		temp_clusters=seq(1:length(rownames(X)))
		names(temp_clusters)=rownames(X)

		Continue=TRUE
		while(Continue==TRUE){

			Pairs=utils::combn(unique(temp_clusters),2)
			AvDistances=apply(Pairs,2,AvDist,DistMat=X,temp_clusters=temp_clusters)

			if(min(AvDistances)<distThresh_A){
				ChosenPair=Pairs[,which.min(AvDistances)]
				temp_clusters[which(temp_clusters==ChosenPair[2])]=ChosenPair[1]
			}
			else{
				Continue=FALSE
			}

			if(length(unique(temp_clusters))==1){
				Continue=FALSE # all have been put into the same cluster
			}

		}

		Clusters=as.numeric(as.factor(temp_clusters))
		names(Clusters)=names(temp_clusters)

		clusters=unique(Clusters)
		order=c()
		for(j in clusters){
			order=c(order,which(Clusters==j))
		}

		order.lab=as.character(order)
		if(improve==TRUE){agglMethod="LocalSearch"}
	}

	if(agglMethod=="Furthest"){

		clusters=rep(0,length(rownames(X)))
		names(clusters)=rownames(X)

		centers=c(rownames(X)[which(X==max(X),arr.ind = TRUE)[1,1]],colnames(X)[which(X==max(X),arr.ind = TRUE)[1,2]])
		for(i in 1:length(centers)){
			clusters[which(names(clusters)%in%centers[i])]=i
		}

		for(j in names(clusters)[-c(which(names(clusters)%in%centers))]){
			DistancesToCenters=X[which(rownames(X)==j),which(colnames(X)%in%centers)]
			AssignedCenter=names(DistancesToCenters)[which.min(DistancesToCenters)]
			clusters[which(names(clusters)==j)]=clusters[which(names(clusters)==AssignedCenter)]

		}

		Part_1=0
		Part_2=0

		Pairs=utils::combn(length(rownames(X)),2)
		for(k in 1:ncol(Pairs)){
			Pair=rownames(X)[Pairs[,k]]

			if(clusters[which(names(clusters)%in%Pair[1])]==clusters[which(names(clusters)%in%Pair[2])]){
				Part_1=Part_1+X[which(rownames(X)%in%Pair[1]),which(colnames(X)%in%Pair[2])]
			}
			else{
				Part_2=Part_2+(1-X[which(rownames(X)%in%Pair[1]),which(colnames(X)%in%Pair[2])])
			}

		}

		Cost_clusters=Part_1+Part_2
		Solution=clusters

		Continue=TRUE

		while(Continue==TRUE){

			AvDistToCenters=c()
			for(i in names(clusters)[-c(which(names(clusters)%in%centers))]){
				DistancesToCenters=X[which(rownames(X)==i),which(colnames(X)%in%centers)]
				AvDistToCenters=c(AvDistToCenters,mean(DistancesToCenters))
			}
			names(AvDistToCenters)=names(clusters)[-c(which(names(clusters)%in%centers))]

			new_center=names(AvDistToCenters)[which.max(AvDistToCenters)]

			centers<-c(centers,new_center)

			for(i in 1:length(centers)){
				clusters[which(names(clusters)%in%centers[i])]=i
			}

			for(j in names(clusters)[-c(which(names(clusters)%in%centers))]){
				DistancesToCenters=X[which(rownames(X)==j),which(colnames(X)%in%centers)]
				AssignedCenter=names(DistancesToCenters)[which.min(DistancesToCenters)]
				clusters[which(names(clusters)==j)]=clusters[which(names(clusters)==AssignedCenter)]
			}

			Part_1=0
			Part_2=0

			Pairs=utils::combn(length(rownames(X)),2)
			for(k in 1:ncol(Pairs)){
				Pair=rownames(X)[Pairs[,k]]

				if(clusters[which(names(clusters)%in%Pair[1])]==clusters[which(names(clusters)%in%Pair[2])]){
					Part_1=Part_1+X[which(rownames(X)%in%Pair[1]),which(colnames(X)%in%Pair[2])]
				}
				else{
					Part_2=Part_2+(1-X[which(rownames(X)%in%Pair[1]),which(colnames(X)%in%Pair[2])])
				}

			}

			Cost_clusters_new=Part_1+Part_2


			if(Cost_clusters_new<Cost_clusters){
				Cost_clusters=Cost_clusters_new
				Solution=clusters
			}
			else{
				Continue=FALSE
				clusters=Solution
			}


		}

		Clusters=clusters
		clusters=unique(Clusters)
		order=c()
		for(j in clusters){
			order=c(order,which(Clusters==j))
		}

		order.lab=as.character(order)

		if(improve==TRUE){
			agglMethod="LocalSearch"
		}
	}

	if(agglMethod=="LocalSearch"){
		UpdateClustering=Clusters

		Stay=rep(0,length(UpdateClustering))
		names(Stay)=names(UpdateClustering)
		Moved=c()

		while(length(which(Stay=="Stay"))!=length(UpdateClustering)){  #continue untill all nodes want to stay where they are

			for(i in names(UpdateClustering)){   #Pick a node

				PresentCluster=UpdateClustering[which(names(UpdateClustering)==i)]  # Present cluster of the node
				ClusterSizes=table(UpdateClustering)  # Current cluster sizes

				Cost_Cj=c()   #Compute the cost of moving node i to any cluster
				for(j in sort(unique(UpdateClustering))){
					Cj=names(UpdateClustering)[which(UpdateClustering==j)]
					M_v_Cj=sum(X[which(rownames(X)==i),which(colnames(X)%in%Cj)])
					Cost_Cj=c(Cost_Cj,M_v_Cj)
				}
				names(Cost_Cj)=sort(unique(UpdateClustering))

				Dist_k=c()  #Compute the cost of moving node i to any cluster or a singleton
				for(k in sort(unique(UpdateClustering))){
					ClusterToMoveTo=k
					Cost_Singl=sum(ClusterSizes[which(names(ClusterSizes)!=ClusterToMoveTo)]-Cost_Cj[which(names(Cost_Cj)!=ClusterToMoveTo)])
					Dist_k=c(Dist_k,Cost_Cj[which(names(ClusterSizes)==ClusterToMoveTo)]+Cost_Singl)
				}
				PotentialClusters=which(Dist_k==min(Dist_k)) # To which clusters does the move hae minimal cost


				if(PresentCluster%in%PotentialClusters){
					Stay[which(names(UpdateClustering)==i)]="Stay"	#If present cluster is among the potential ones: stay
				}
				else{
					UpdateClustering[which(names(UpdateClustering)==i)]=PotentialClusters[1]  #else move and if mover before stay is changed to 0 again
					if(Stay[which(names(UpdateClustering)==i)]=="Stay"){
						Stay[which(names(UpdateClustering)==i)]=0
					}
					Moved=c(Moved,i)
					Moved=unique(Moved)
				}
			}

		}

		Clusters=UpdateClustering

		clusters=unique(Clusters)
		order=c()
		for(j in clusters){
			order=c(order,which(Clusters==j))
		}

		order.lab=as.character(order)

	}

	Out=list(DistM=NULL,Clust=list(order=order,order.lab=order.lab,Clusters=Clusters))
	attr(Out,"method")="Ensemble"
	return(Out)
}

test_that("HBGF (spectral) returns a partition over all objects", {
  data(mosaic_toy)
  L <- mosaic_toy$List

  out <- HBGF(
    List = L, type = "data",
    distmeasure = c("euclidean", "euclidean"),
    normalize = c(FALSE, FALSE), method = c(NULL, NULL),
    clust = "agnes", linkage = c("ward", "ward"),
    nrclusters = c(7, 7), graphPartitioning = "Spec", optimalk = 7
  )

  expect_null(out$DistM)
  expect_equal(attr(out, "method"), "Ensemble")
  cl <- out$Clust$Clusters
  expect_length(cl, nrow(L[[1]]))
  expect_equal(names(cl), rownames(L[[1]]))
  expect_true(length(unique(cl)) >= 1)
})

test_that("ClusteringAggregation Aggl yields a named partition", {
  data(mosaic_toy)
  L <- mosaic_toy$List

  out <- ClusteringAggregation(
    List = L, type = "data",
    distmeasure = c("euclidean", "euclidean"),
    normalize = c(FALSE, FALSE), method = c(NULL, NULL),
    clust = "agnes", linkage = c("ward", "ward"), alpha = 0.625,
    nrclusters = c(7, 7), agglMethod = "Aggl",
    improve = TRUE, distThresh_B = 0.5, distThresh_A = 0.8
  )

  expect_null(out$DistM)
  expect_equal(attr(out, "method"), "Ensemble")
  cl <- out$Clust$Clusters
  expect_length(cl, nrow(L[[1]]))
  expect_setequal(names(cl), rownames(L[[1]]))
})

test_that("ClusteringAggregation Balls yields a named partition", {
  data(mosaic_toy)
  L <- mosaic_toy$List

  out <- ClusteringAggregation(
    List = L, type = "data",
    distmeasure = c("euclidean", "euclidean"),
    normalize = c(FALSE, FALSE), method = c(NULL, NULL),
    clust = "agnes", linkage = c("ward", "ward"), alpha = 0.625,
    nrclusters = c(7, 7), agglMethod = "Balls",
    improve = FALSE, distThresh_B = 0.5, distThresh_A = 0.8
  )

  cl <- out$Clust$Clusters
  expect_length(cl, nrow(L[[1]]))
  expect_setequal(names(cl), rownames(L[[1]]))
})

test_that("ClusteringAggregation Furthest yields a named partition", {
  data(mosaic_toy)
  L <- mosaic_toy$List

  out <- ClusteringAggregation(
    List = L, type = "data",
    distmeasure = c("euclidean", "euclidean"),
    normalize = c(FALSE, FALSE), method = c(NULL, NULL),
    clust = "agnes", linkage = c("ward", "ward"), alpha = 0.625,
    nrclusters = c(7, 7), agglMethod = "Furthest",
    improve = FALSE, distThresh_B = 0.5, distThresh_A = 0.8
  )

  cl <- out$Clust$Clusters
  expect_length(cl, nrow(L[[1]]))
  expect_setequal(names(cl), rownames(L[[1]]))
})

#' @title Aggregated data clustering
#'
#' @description Aggregated Data Clustering (ADC) is a direct clustering multi-source technique. The data matrices are merged into a single fused data matrix on which a dissimilarity matrix is computed. Hierarchical clustering is then performed on this dissimilarity matrix.
#'
#' @export
#' @param List A list of data matrices of the same type. It is assumed the rows are corresponding with the objects.
#' @param distmeasure Choice of metric for the dissimilarity matrix (character). Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to "tanimoto".
#' @param normalize Logical. Indicates whether to normalize the distance matrices or not, defaults to FALSE. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is NULL.
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character). Defaults to "flexible".
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".
#' @return The returned value is a list with the following three elements.
#' \item{AllData}{Fused data matrix of the data matrices}
#' \item{DistM}{The resulting distance matrix}
#' \item{Clust}{The resulting clustering}
#' The value has class 'ADC'. The Clust element will be of interest for further applications.
#' @references Fodeh, S. J., Punch, W. & Tan, P.-N. (2013). On unifying multi-source information from probabilistically generated functional data. Proteins 81:1298-1310.
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' L <- mosaic_toy$List
#' res_ADC <- ADC(List=L, distmeasure="euclidean", normalize=FALSE, method=NULL,
#'                clust="agnes", linkage="flexible", alpha=0.625)
#' }
ADC<-function(List,distmeasure="tanimoto",normalize=FALSE,method=NULL,clust="agnes",linkage="flexible",alpha=0.625){

	#Checking required data types and methods:
	if(!is.list(List)){
		stop("Data must be of type list")
	}


	#Fuse variables into 1 data Matrix

	OrderNames=rownames(List[[1]])
	for(i in 1:length(List)){
		List[[i]]=List[[i]][OrderNames,]
	}

	AllData<-NULL
	for (i in 1:length(List)){
		if(i==1){
			AllData=List[[1]]
		}
		else{
			AllData=cbind(AllData,List[[i]])
		}
	}

	#Compute Distance Matrix on AllData

	AllDataDist=Distance(AllData,distmeasure,normalize,method)

	#Perform hierarchical clustering with ward link on distance matrix

	HClust = cluster::agnes(AllDataDist,diss=TRUE,method=linkage,par.method=alpha)


	out=list(AllData=AllData,DistM=AllDataDist,Clust=HClust)
	attr(out,'method')<-'ADC'
	return(out)

}


#' @title Aggregated data ensemble clustering
#'
#' @description Aggregated Data Ensemble Clustering (ADEC) is a direct clustering multi-source technique. ADEC is an iterative procedure which starts with the merging of the data sets. In each iteration, a random sample of the features is selected and/or a resulting dendrogram is divided into k clusters for a range of values of k.
#'
#' @export
#' @param List A list of data matrices of the same type. It is assumed the rows are corresponding with the objects.
#' @param distmeasure Choice of metric for the dissimilarity matrix (character). Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to "tanimoto".
#' @param normalize	Logical. Indicates whether to normalize the distance matrices or not, defaults to FALSE. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is NULL.
#' @param t The number of iterations. Defaults to 10.
#' @param r The number of features to take for the random sample. If NULL (default), all features are considered.
#' @param nrclusters A sequence of numbers of clusters to cut the dendrogram in. If NULL (default), the function stops.
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character). Defaults to "flexible".
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".
#' @return The returned value is a list with the following three elements.
#' \item{AllData}{Fused data matrix of the data matrices}
#' \item{DistM}{The resulting co-association matrix}
#' \item{Clust}{The resulting clustering}
#' The value has class 'ADEC'. The Clust element will be of interest for further applications.
#' @details If r is specified and nrclusters is a fixed number, only a random sampling of the features will be performed for the t iterations (ADECa). If r is NULL and the nrclusters is a sequence, the clustering is performed on all features and the dendrogam is divided into clusters for the values of nrclusters (ADECb). If both r is specified and nrclusters is a sequence, the combination is performed (ADECc).
#' After every iteration, either be random sampling, multiple divisions of the dendrogram or both, an incidence matrix is set up. All incidence matrices are summed and represent the distance matrix on which a final clustering is performed.
#' @references Fodeh, S. J., Punch, W. & Tan, P.-N. (2013). On unifying multi-source information from probabilistically generated functional data. Proteins 81:1298-1310.
#' @examples
#' \dontrun{
#' data(mosaic_toy)
#' L <- mosaic_toy$List
#' res_ADEC <- ADEC(List=L, distmeasure="euclidean", normalize=FALSE, method=NULL,
#'                  t=5, r=50, nrclusters=5, clust="agnes", linkage="flexible", alpha=0.625)
#' }
ADEC<-function(List,distmeasure="tanimoto",normalize=FALSE,method=NULL,t=10,r=NULL,nrclusters=NULL,clust="agnes",linkage="flexible",alpha=0.625){

	if(!is.list(List)){
		stop("Data must be of type lists")
	}

	if(is.null(nrclusters)){
		stop("Give a number of cluters to cut the dendrogram into.")
	}

	if(!is.null(r)&length(nrclusters)==1){
		message("Performing a random sampling of the features with a fixed number of clusters.")
	}else if(is.null(r)&length(nrclusters)>1){
		message("Dividing the dendrogram in k clusters for a range of values of k.")
		t=1
	}else if(!is.null(r)&length(nrclusters)>1){
		message("Performing a random sampling of the features and dividing the dendrogram in k clusters for a range of values of k.")
	}
	else{
		stop("Specify r and/or nrclusters in order to perform an ADEC method.")
	}

	#Fuse A1 and A2 into 1 Data Matrix

	OrderNames=rownames(List[[1]])
	for(i in 1:length(List)){
		List[[i]]=List[[i]][OrderNames,]
	}

	AllData<-NULL
	for (i in 1:length(List)){
		if(i==1){
			AllData=List[[1]]
		}
		else{
			AllData=cbind(AllData,List[[i]])
		}
	}


	#take random sample of features

	nc=ncol(AllData)
	evenn=function(x){if(x%%2!=0) x=x-1 else x}
	nc=evenn(nc)


	#Put up Incidence matrix
	Incidence=matrix(0,dim(List[[i]])[1],dim(List[[i]])[1])
	rownames(Incidence)=rownames(AllData)
	colnames(Incidence)=rownames(AllData)


	#Repeat for t iterations


	for(g in 1:t){
		#message(g)

		#if r is not fixed: changes per iteration. Need 1 value for r.
		if(is.null(r)){
			r=ncol(AllData)
		}

		#take random sample:
		ZeroPresent=TRUE
		while(ZeroPresent){
			temp1=sample(ncol(AllData),r,replace=FALSE)

			A_prime=AllData[,temp1]

			if(all(rowSums(A_prime)!=0)){
				ZeroPresent=FALSE
			}

		}
		#Step 2: apply hierarchical clustering on A_prime  + cut tree into nrclusters

		DistM=Distance(A_prime,distmeasure,normalize,method)

		HClust_A_prime=cluster::agnes(DistM,diss=TRUE,method=linkage,par.method=alpha)


		for(k in 1:length(nrclusters)){
			#message(k)
			Temp=stats::cutree(HClust_A_prime,nrclusters[k])
			MembersofClust=matrix(1,dim(List[[1]])[1],dim(List[[1]])[1])

			for(l in 1:length(Temp)){
				label=Temp[l]
				sameclust=which(Temp==label)
				MembersofClust[l,sameclust]=0
			}
			Incidence=Incidence+MembersofClust
		}

	}

	Clust=cluster::agnes(Incidence,diss=TRUE,method="ward",par.method=alpha)

	out=list(AllData=AllData,DistM=Incidence,Clust=Clust)
	attr(out,'method')<-'ADEC'
	return(out)

}

test_that("ADC returns expected structure", {
  data(mosaic_toy)
  L <- mosaic_toy$List
  res <- ADC(List = L, distmeasure = "euclidean", normalize = FALSE,
             method = NULL, clust = "agnes", linkage = "flexible", alpha = 0.625)

  expect_type(res, "list")
  expect_named(res, c("AllData", "DistM", "Clust"))
  expect_equal(attr(res, "method"), "ADC")

  # Fused data has objects in rows and all features stacked in columns
  expect_equal(nrow(res$AllData), nrow(L[[1]]))
  expect_equal(ncol(res$AllData), ncol(L[[1]]) + ncol(L[[2]]))

  # Distance matrix is square over the objects
  expect_equal(dim(res$DistM), c(nrow(L[[1]]), nrow(L[[1]])))

  # Clustering result is an agnes object
  expect_s3_class(res$Clust, "agnes")
})

test_that("ADC errors on non-list input", {
  expect_error(ADC(List = matrix(1:4, 2, 2)), "list")
})

test_that("ADEC with random sampling and fixed nrclusters runs", {
  data(mosaic_toy)
  L <- mosaic_toy$List
  res <- ADEC(List = L, distmeasure = "euclidean", normalize = FALSE,
              method = NULL, t = 3, r = 50, nrclusters = 5,
              clust = "agnes", linkage = "flexible", alpha = 0.625)

  expect_type(res, "list")
  expect_named(res, c("AllData", "DistM", "Clust"))
  expect_equal(attr(res, "method"), "ADEC")

  # Incidence/co-association matrix is square over the objects
  expect_equal(dim(res$DistM), c(nrow(L[[1]]), nrow(L[[1]])))
  expect_s3_class(res$Clust, "agnes")
})

test_that("ADEC with sequence of nrclusters runs", {
  data(mosaic_toy)
  L <- mosaic_toy$List
  res <- ADEC(List = L, distmeasure = "euclidean", normalize = FALSE,
              method = NULL, t = 5, r = NULL, nrclusters = seq(2, 5, 1),
              clust = "agnes", linkage = "flexible", alpha = 0.625)

  expect_equal(attr(res, "method"), "ADEC")
  expect_equal(dim(res$DistM), c(nrow(L[[1]]), nrow(L[[1]])))
  expect_s3_class(res$Clust, "agnes")
})

test_that("ADEC errors when nrclusters is NULL", {
  data(mosaic_toy)
  expect_error(ADEC(List = mosaic_toy$List, nrclusters = NULL),
               "number of cluters")
})

#' @title Hierarchical ensemble clustering
#'
#' @description \insertCite{Zheng2014}{MosaiClusteR} proposed the Hierarchical Ensemble Clustering (HEC) algorithm. For each dendrogram, the cophenetic
#' distances between the object are calculated. The distances are aggregated across the data sets and an ultra-metric which is the closest to the
#' distance matrix is determined. A final hierarchical clustering is based on the ultra-metric values.
#' @export
#' @param List A list of data matrices. It is assumed the rows are corresponding with the objects.
#' @param type indicates whether the provided matrices in "List" are either data matrices, distance
#' matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
#' matrices is skipped and if type="clusters" the single source clustering is skipped.
#' Type should be one of "data", "dist" or"clusters".
#' @param distmeasure A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").
#' @param normalize	Logical. Indicates whether to normalize the distance matrices or not, default is FALSE. This is recommended if different distance types are used. More details on normalization in \code{Normalization}
#' @param method A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.
#' @param clust Choice of clustering function (character). Defaults to "agnes".
#' @param linkage Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.
#' @param alpha The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".
#' @return The returned value is a list of two elements:
#' \item{DistM}{The resulting distance matrix}
#' \item{Clust}{The resulting hierarchical structure}
#' The value has class 'HEC'.
#' @references \insertRef{Zheng2014}{MosaiClusteR}
HierarchicalEnsembleClustering<-function(List,type=c("data","dist","clust"),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),clust = "agnes", linkage = c("flexible","flexible"),alpha=0.625){
  
  ## Step 1: perfom aggl clustering on each data set
  
  if(type=="data"){
    
    if(is.null(rownames(List[[1]]))){
      for(i in 1:length(List)){
        rownames(List[[i]])=seq(1:nrow(List[[i]]))
      }
    }
    else{
      OrderNames=rownames(List[[1]])
      
      for(i in 1:length(List)){
        List[[i]]=List[[i]][OrderNames,,drop=FALSE]
      }
    }
    Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type="data",distmeasure[i],normalize[i],method[i],clust,linkage[i],alpha,gap=FALSE,maxK=5,StopRange=TRUE))
    #Clusterings=lapply(seq(length(List)),function(i) agnes(List[[i]],method=linkage[i],par.method=0.625))
    
    for(i in 1:length(Clusterings)){
      names(Clusterings)[i]=paste("Clust",i,sep=' ')
    }
    
  }
  
  else if(type=="dist"){
    OrderNames=rownames(List[[1]])
    for(i in 1:length(List)){
      List[[i]]=List[[i]][OrderNames,OrderNames]
    }
    
    Clusterings=lapply(seq(length(List)),function(i) Cluster(List[[i]],type,distmeasure[i],normalize=FALSE,method=NULL,clust,linkage[i],alpha,gap=FALSE,maxK=5,StopRange=TRUE))
    
    for(i in 1:length(Clusterings)){
      names(Clusterings)[i]=paste("Clust",i,sep=' ')
    }
    
  }
  else if(type=="clust"){
    
    Clusterings=List
    
    for(i in 1:length(Clusterings)){
      names(Clusterings)[i]=paste("Clust",i,sep=' ')
    }
    
    
  }
  
  ## Step 2: cophenetic distance of each object in each dendrogram
  
  CD=list()
  for(j in 1:length(Clusterings)){
    CD[[j]]=as.matrix(cophenetic(as.hclust(Clusterings[[j]]$Clust)))
  }
  
  ## Step 3: Aggregate the distance of the dendrograms and compute the matrix D
  ## ? what is meant by aggregate?
  ## Assumed: aggragete ==  added
  
  D_H=Reduce("+",CD)
  
  D=1/2*D_H
  
  ## Step 4: Find an ultra-metric distance T closest to D
  ## Apply the modified Floyd-Warshall algorithm on D
  
  Floyd_Warshall<-function(G){
    M=G
    N=nrow(G)
    for(k in c(1:N)){
      for(i in c(1:N)){
        for(j in c(1:N)){
          M[i,j]=min(M[i,j],max(M[i,k],M[k,j]))
          
        }
      }
    }
    return(M)
  }
  
  T=Floyd_Warshall(D)  # Distance matrix
  
  ## Step 5: Construct the final hierarchical clustering based on T
  ## the alpha-cut method form Meyer et al 2004
  
  ## ? defuzzify
  
  #	## Should
  #	test=cut(T, level = 1, type = c("alpha"), strict = FALSE)
  #
  #	#Testing
  #	A=matrix(c(0,1,14,18,18,18,1,0,14,18,18,18,14,14,0,18,18,18,18,18,18,0,15,15,18,18,18,15,0,2,18,18,18,15,2,0),ncol=6,nrow=6)
  #
  
  
  #try alpha cut for each value: denote cluster
  #build tree from this?
  
  alpha=sort(unique(as.vector(T)),decreasing = TRUE)
  C=list()
  #	for(i in alpha[-length(alpha)][c(1:11)]){
  #		C[[length(C)+1]]=archi(as.dist(T),i)[[1]]
  #
  #		if(length(C)>1){
  #
  #			if(max(C[[length(C)]])>max(C[[length(C)-1]])+1){
  #				Changed=which(C[[length(C)]]+ C[[length(C)-1]]<2*C[[length(C)]]-1)
  #				Groups=unique(C[[length(C)]][Changed])
  #
  #				if(length(Groups)==1){
  #					N=C[[length(C)]][-c(Changed)]
  #					O=C[[length(C)-1]][-c(Changed)]
  #					Joinedtemp=which(!as.vector(c(table(N)))==as.vector(c(table(O),0)))[c(2)]
  #					Joined=as.numeric(names(table(O))[Joinedtemp])
  #					New2=which(C[[length(C)]]==Joined[1])
  #					Groups=c(Groups,unique(C[[length(C)]][New2]))
  #					Changed=c(Changed,New2)
  #				}
  #
  #				Group1=Changed[which(C[[length(C)]][Changed]==Groups[1])]
  #				Group2=Changed[which(C[[length(C)]][Changed]==Groups[2])]
  #
  #				Clus=C[[length(C)-1]]
  #				Clusnext=C[[length(C)]]
  #				Clus1=Clusnext
  #				ChangeAfter=unique(Clus1[Group2])
  #				Clus1[Group2]=Clus[Group2]
  #				Clus1[which(Clus1>ChangeAfter)]=Clus1[which(Clus1>ChangeAfter)]-1
  #
  #				Clus2=Clusnext
  #
  #				C=C[-c(length(C))]
  #				C[[length(C)+1]]=Clus1
  #				C[[length(C)+1]]=Clus2
  #
  #			}
  #		}
  #	}
  
  for(k in c(1:nrow(List[[1]]))){
    Y=cutree(as.hclust(agnes(T,diss=TRUE)),k)
    names(Y)=NULL
    C[[length(C)+1]]=Y
  }
  #C[[length(C)+1]]=seq(1,nrow(T))
  
  Record=rep(0,length(C))
  names(Record)=seq(1,length(C))
  Merge=matrix(0,ncol=2,nrow=1)
  MergingList=list()
  
  for(i in length(C):1){
    if(i==length(C)){
      #All Leafs - Bottom of the tree
      Leafs=C[[length(C)]]
      Record=Leafs
    }
    
    else{
      
      NewMerge=C[[i]]
      if(i==(length(C)-1)){
        NewOnes=which(duplicated(NewMerge))
        for(n in 1:length(NewOnes)){
          MergedTogether=which(NewMerge==NewMerge[NewOnes][n])
          if(n==1){
            Merge[1,]=MergedTogether*(-1)
            
          }
          else{
            Merge=rbind(Merge,MergedTogether*(-1))
            
          }
          MergingList[[length(MergingList)+1]]=MergedTogether
          Record=NewMerge
        }
      }
      
      else{
        
        Changed=which(NewMerge!=Record)
        if(length(Changed)!=0){
          
          PositionChanged=Position(function(x) identical(x,Changed), MergingList, nomatch = 0)
          if(PositionChanged==0){
            
            formedafter=FALSE
            NewOnes=unique(c(which(duplicated(NewMerge)),which(duplicated(NewMerge,fromLast=TRUE))))[which(!unique(c(which(duplicated(NewMerge)),which(duplicated(NewMerge,fromLast=TRUE))))%in%unique(c(which(duplicated(Record)),which(duplicated(Record,fromLast=TRUE)))))]
            if(any(NewMerge[Changed]%in%NewMerge[NewOnes])){
              Temp=Changed[which(NewMerge[Changed]%in%NewMerge[NewOnes])]
              if(length(intersect(Temp,NewOnes))>0){
                NewOnes=Temp
                formedafter=TRUE
              }
            }
          }
          else{
            formedafter=FALSE
            NewOnes=which(NewMerge==unique(NewMerge[Changed]))[!which(NewMerge==unique(NewMerge[Changed]))%in%Changed]
          }
          if(length(NewOnes)==0){
            Joined=which(!as.vector(c(table(NewMerge),0))==as.vector(table(Record)))[c(1,2)]
            NewOnes=which(Record==Joined[1])
            
          }
          for(k in 1:length(unique(NewMerge[NewOnes]))){
            MergedTogether=which(NewMerge==unique(NewMerge[NewOnes])[k])
            
            Position1=Position(function(x) identical(x,(as.integer(MergedTogether[-which(MergedTogether%in%NewOnes)]))), MergingList, nomatch = 0)
            if(Position1==0){
              Merge=rbind(Merge,MergedTogether*(-1))  #they are both leaves
              
            }
            else{
              #First group was formed before
              if(formedafter==TRUE){
                TrulyNew=unique(c(which(duplicated(NewMerge)),which(duplicated(NewMerge,fromLast=TRUE))))[which(!unique(c(which(duplicated(NewMerge)),which(duplicated(NewMerge,fromLast=TRUE))))%in%unique(c(which(duplicated(Record)),which(duplicated(Record,fromLast=TRUE)))))]
                Group=NewOnes[-c(which(NewOnes%in%TrulyNew))]
                Set=list(TrulyNew,Group)
                for(s in Set){
                  if(length(s)>0){
                    Position2=Position(function(x) identical(x,(as.integer(MergedTogether[which(MergedTogether%in%s)]))), MergingList, nomatch = 0)
                    
                    if(Position2==0){  #first one was a group, newones is a leaf
                      Merge=rbind(Merge,c(Position1,(intersect(s,MergedTogether)*(-1))))
                      
                    }
                    else{  #both were grouped before
                      Merge=rbind(Merge,c(Position1,Position2))
                      
                    }
                  }
                }
              }
              else{
                Position2=Position(function(x) identical(x,(as.integer(MergedTogether[which(MergedTogether%in%NewOnes)]))), MergingList, nomatch = 0)
                
                if(Position2==0){  #first one was a group, newones is a leaf
                  Merge=rbind(Merge,c(Position1,(intersect(NewOnes,MergedTogether)*(-1))))
                  
                }
                else{  #both were grouped before
                  Merge=rbind(Merge,c(Position1,Position2))
                  
                }
              }
            }
            
            MergingList[[length(MergingList)+1]]=MergedTogether
          }
          Record=NewMerge
        }
      }
    }
  }
  
  
  Start=which(Merge==-1,arr.ind=TRUE)[1]
  Checked=rep(0,(nrow(Merge)))
  out=c()
  Row=Start
  while(!all(Checked==1)){
    Joined=Merge[Row,]
    #print(Row)
    if(sign(Joined[1])==-1 & sign(Joined[2])==-1){
      
      Checked[Row]=1
      
      if(Row==Start){
        
        out=abs(Joined)
        RNext=which(Merge==Row,arr.ind=TRUE)[1]
        
      }
      else{
        
        if(any(Checked>1)){
          out=c(out,abs(Joined))
          RNext=which.max(Checked)
        }
        
        else if(any(Checked==0)){
          out=c(out,abs(Joined))
          RNext=which(Merge==Row,arr.ind=TRUE)[1]
        }
      }
      
    }
    
    else if(sign(Joined[1])==1 & sign(Joined[2])==1){
      
      if(all(Checked[Joined]==1)){
        Checked[Row]=1
        
        if(any(Checked>1)){
          RNext=which.max(Checked)
        }
        else if(!(all(Checked==1))){
          RNext=which(Merge==Row,arr.ind=TRUE)[1]
        }
        
      }
      
      else{
        Checked[Row]=max(Checked)+1
        RNext=Joined[which(!Joined%in%which(Checked==1))]
        
        if(length(RNext)==2){
          Checked[RNext[2]]=max(Checked)+1
          RNext=RNext[1]
        }
      }
      
    }
    
    else if(sign(Joined[1])==1 & sign(Joined[2])==-1){
      
      Checked[Row]=1
      
      if(Checked[Joined[1]]==0){
        out=c(out,abs(Joined[2]))
        RNext=Joined[1]
      }
      else{
        if(!(abs(Joined)[2]%in%out)){
          out=c(out,abs(Joined[2]))
        }
        RNext=which(Merge==Row,arr.ind=TRUE)[1]
        
      }
    }
    
    else if(sign(Joined[1])==-1 & sign(Joined[2])==1){
      Checked[Row]=1
      
      if(Checked[Joined[2]]==0){
        out=c(out,abs(Joined[1])) #possibly to be shifted
        RNext=Joined[2]
        
      }
      else{
        if(!(abs(Joined)[1]%in%out)){
          out=c(out,abs(Joined[1]))
        }
        #					print("301")
        RNext=which(Merge==Row,arr.ind=TRUE)[1]
        
      }
      
    }
    
    Row=RNext
  }
  
  
  #	OrderObjects<-function(Row=Start,M=Merge,out=c(),Checked=rep(0,(nrow(T)-1))){
  #
  #		if(!all(Checked==1)){
  ##			print(out)
  #			print(Row)
  ##			print(Checked)
  ##			print("212")
  #
  #			Joined=M[Row,]
  #
  #			if(sign(Joined[1])==-1 & sign(Joined[2])==-1){
  #
  #				Checked[Row]=1
  #
  #				if(Row==Start){
  #
  #					out=abs(Joined)
  #					RNext=which(Merge==Row,arr.ind=TRUE)[1]
  ##					print("218")
  #					out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #
  #				}
  #				else{
  ##					print("222")
  #
  #
  #					if(any(Checked>1)){
  #						out=c(out,abs(Joined))
  #						#RNext=which(Merge==which.max(Checked),arr.ind=TRUE)[1]
  #						RNext=which.max(Checked)
  ##						print("227")
  #						#Checked[which.max(Checked)]=1
  #						out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #					}
  #
  #					else if(any(Checked==0)){
  #						out=c(out,abs(Joined))
  #						RNext=which(Merge==Row,arr.ind=TRUE)[1]
  #						out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #					}
  ##					return(out)
  #				}
  #
  #				if((all(Checked==1))){
  #					print(out)
  #					out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #				}
  #				return(out)
  #			}
  #
  #
  #			else if(sign(Joined[1])==1 & sign(Joined[2])==1){
  #
  #				if(all(Checked[Joined]==1)){
  #					Checked[Row]=1
  ##					print("239")
  #					print(Checked)
  #
  #					if(any(Checked>1)){
  #						RNext=which.max(Checked)
  ##						print("242")
  #						out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #					}
  #					else if(!(all(Checked==1))){
  #						RNext=which(Merge==Row,arr.ind=TRUE)[1]
  ##						print("247")
  #						out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #					}
  #
  #				}
  #
  #				else{
  #					Checked[Row]=max(Checked)+1
  #					RNext=Joined[which(!Joined%in%which(Checked==1))]
  ##					print("259")
  #
  #					if(length(RNext)==2){
  #						Checked[RNext[2]]=max(Checked)+1
  #						RNext=RNext[1]
  ##						print("263")
  #						out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #					}
  #					else{
  ##						print("269")
  #						out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #					}
  #				}
  #
  #				if((all(Checked==1))){
  #					print(out)
  #					out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #				}
  #
  #				return(out)
  #			}
  #
  #			else if(sign(Joined[1])==1 & sign(Joined[2])==-1){
  #
  ##				print("276")
  #				Checked[Row]=1
  #
  #
  #				if(Checked[Joined[1]]==0){
  #					out=c(out,abs(Joined[2]))
  ##					print("280")
  #					RNext=Joined[1]
  #					out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #				}
  #				else{
  ##					print("288")
  #					if(!(abs(Joined)[2]%in%out)){
  #						out=c(out,abs(Joined[2]))
  #					}
  #					RNext=which(Merge==Row,arr.ind=TRUE)[1]
  #					out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #				}
  #				if((all(Checked==1))){
  ##					print(out)
  ##					out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #			}
  #				return(out)
  #			}
  #
  #			else if(sign(Joined[1])==-1 & sign(Joined[2])==1){
  #				Checked[Row]=1
  ##				print("293")
  #
  #				if(Checked[Joined[2]]==0){
  #					out=c(out,abs(Joined[1])) #possibly to be shifted
  ##					print("296")
  #					RNext=Joined[2]
  #					out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #				}
  #				else{
  #					if(!(abs(Joined)[1]%in%out)){
  #						out=c(out,abs(Joined[1]))
  #					}
  ##					print("301")
  #					RNext=which(Merge==Row,arr.ind=TRUE)[1]
  #					out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #				}
  #
  #				if((all(Checked==1))){
  #					out=OrderObjects(Row=RNext,M=M,out=out,Checked=Checked)
  #				}
  #				return(out)
  #			}
  #
  #		}
  #
  #		else{
  ##			print("here")
  ##			print(out)
  ##			print(Checked)
  #			return(out)
  #		}
  #
  #
  #	}
  #	Order=OrderObjects(Row=Start,M=Merge,out=c(),Checked=rep(0,(nrow(T)-1)))
  #
  
  Order=out
  Heights=c()
  for(j in 1:nrow(Merge)){
    joined=Merge[j,]
    
    if(sign(joined[1])==-1){
      a=abs(joined[1])
    }
    else{
      a=MergingList[[joined[1]]]
      a=abs(a)
    }
    
    if(sign(joined[2])==-1){
      b=abs(joined[2])
    }
    else{
      b=MergingList[[joined[2]]]
      b=abs(b)
    }
    
    Heights=c(Heights,unique(as.vector(T[a,b]))) #should be put in the order of the names
    
  }
  
  
  #	HeightsObjects<-function(Row=Start,M=Merge,H=Heights,heights=c(),Checked=rep(0,(nrow(T)-1))){
  #
  #		if(!all(Checked==1)){
  #			print(heights)
  #			print(Row)
  #			print(Checked)
  #			print("212")
  #
  #			Joined=M[Row,]
  #
  #			if(sign(Joined[1])==-1 & sign(Joined[2])==-1){
  #
  #				Checked[Row]=1
  #
  #				if(Row==Start){
  #
  #					heights=c(heights,H[Row])
  #					RNext=which(Merge==Row,arr.ind=TRUE)[1]
  #					print("218")
  #					heights=HeightsObjects(Row=RNext,M=M,H=H,heights=heights,Checked=Checked)
  #
  #				}
  #				else{
  #					print("222")
  #					heights=c(heights,H[Row])
  #
  #					if(any(Checked>1)){
  #						#RNext=which(Merge==which.max(Checked),arr.ind=TRUE)[1]
  #						RNext=which.max(Checked)
  #						print("227")
  #						#Checked[which.max(Checked)]=1
  #						heights=HeightsObjects(Row=RNext,H=H,heights=heights,Checked=Checked)
  #					}
  ##					return(out)
  #				}
  #
  #				if((all(Checked==1))){
  #					print(heights)
  #					heights=HeightsObjects(Row=RNext,H=H,heights=heights,Checked=Checked)
  #				}
  #				return(heights)
  #			}
  #
  #
  #			else if(sign(Joined[1])==1 & sign(Joined[2])==1){
  #
  #				if(all(Checked[Joined]==1)){
  #					Checked[Row]=1
  #					heights=c(heights,H[Row])
  #					print("239")
  #
  #					if(any(Checked>1)){
  #						RNext=which.max(Checked)
  #						print("242")
  #						heights=HeightsObjects(Row=RNext,H=H,heights=heights,Checked=Checked)
  #					}
  #					else if(!(all(Checked==1))){
  #						RNext=which(Merge==Row,arr.ind=TRUE)[1]
  #						print("247")
  #						heights=HeightsObjects(Row=RNext,H=H,heights=heights,Checked=Checked)
  #					}
  #
  #				}
  #
  #				else{
  #					Checked[Row]=max(Checked)+1
  #					#Checked[Row]=1
  #					#heights=c(heights,H[Row])
  #					RNext=Joined[which(!Joined%in%which(Checked==1))]
  #					print("259")
  #
  #					if(length(RNext)==2){
  #						Checked[RNext[2]]=max(Checked)+1
  #						RNext=RNext[1]
  #						print("263")
  #						heights=HeightsObjects(Row=RNext,H=H,heights=heights,Checked=Checked)
  #					}
  #					else{
  #						print("269")
  #						heights=HeightsObjects(Row=RNext,H=H,heights=heights,Checked=Checked)
  #					}
  #				}
  #
  #				if((all(Checked==1))){
  #					print(heights)
  #					heights=HeightsObjects(Row=RNext,H=H,heights=heights,Checked=Checked)
  #				}
  #
  #				return(heights)
  #			}
  #
  #			else if(sign(Joined[1])==1 & sign(Joined[2])==-1){
  #
  #				print("276")
  #				Checked[Row]=1
  #				heights=c(heights,H[Row]) #possibly to be shifted
  #
  #				if(Checked[Joined[1]]==0){
  #					print("280")
  #					RNext=Joined[1]
  #					heights=HeightsObjects(Row=RNext,H=H,heights=heights,Checked=Checked)
  #				}
  #				else{
  #					print("288")
  #					RNext=which(Merge==Row,arr.ind=TRUE)[1]
  #					heights=HeightsObjects(Row=RNext,H=H,heights=heights,Checked=Checked)
  #				}
  #				if((all(Checked==1))){
  #					print(heights)
  #					heights=HeightsObjects(Row=RNext,H=H,heights=heights,Checked=Checked)
  #				}
  #				return(heights)
  #			}
  #
  #			else if(sign(Joined[1])==-1 & sign(Joined[2])==1){
  #				Checked[Row]=1
  #				print("293")
  #				heights=c(heights,H[Row]) #possibly to be shifted
  #				if(Checked[Joined[2]]==0){
  #					print("296")
  #					RNext=Joined[2]
  #					heights=HeightsObjects(Row=RNext,H=H,heights=heights,Checked=Checked)
  #				}
  #				else{
  #					print("301")
  #					RNext=which(Merge==Row,arr.ind=TRUE)[1]
  #					heights=HeightsObjects(Row=RNext,H=H,heights=heights,Checked=Checked)
  #				}
  #
  #				if((all(Checked==1))){
  #					print(heights)
  #					heights=HeightsObjects(Row=RNext,H=H,heights=heights,Checked=Checked)
  #				}
  #				return(heights)
  #			}
  #
  #		}
  #
  #		else{
  #			print("here")
  #			print(heights)
  #			return(heights)
  #		}
  #
  #
  #	}
  #	HeightsOrder<-HeightsObjects(Row=Start,M=Merge,H=Heights,heights=c(),Checked=rep(0,(nrow(T)-1)))
  
  Labels=rownames(T)
  
  out=list(height=Heights,merge=Merge,order=Order,labels=rownames(T))
  class(out)="hclust"
  plot(out)
  
  order.lab=rownames(T)[Order]
  Out=list(DistM=T,Clust=list(height=Heights,merge=Merge,order=Order,labels=rownames(T),order.lab=order.lab))
  attr(Out$Clust,"class")="hclust"
  attr(Out,"method")="HEC"
  return(Out)
  
}

# =============================================================================
# M_ABCpp.R
# Multi-source M-ABC -- the canonical, C++-accelerated wrapper (Osang'ir).
# Functions: f.clustABC.MultiSource, M_ABCpp
#
# This retains the ORIGINAL design:
#   * per source we run ABCpp.SingleInMultiple and row-stack the label matrices;
#   * the consensus co-clustering counts are computed by the C++ kernel
#     count_coclustering_cpp when it is available (exactly as in the original,
#     via an exists() guard), otherwise a pure-R double loop is used;
#   * the [0,1] dissimilarity is 1 - Sij ("abc") or sqrt(1 - Sij^2) ("mabc").
#
# In the original this kernel was compiled inline with Rcpp::cppFunction() at
# source() time. As a package, the SAME C++ code lives in
# src/mcpp_sampling.cpp (compiled once at install) and is picked up here by the
# very same exists("count_coclustering_cpp") guard -- so the behaviour and the
# pure-R fallback are unchanged.
#
# Corrections carried over:
#   B6  Sij denominator = co-selection count, not nrow(res)
#   B7  'dissimilarity' argument ("abc" | "mabc") replaces the old distmeth flag
#   B8  alpha is an explicit parameter
#   B9  NC2 / extra args are not forwarded to ABCpp.SingleInMultiple
#
# NEW in 0.1.1: weighting = "nugget" (data nuggets) is passed straight through
#   to ABCpp.SingleInMultiple; nfeat / nugget_type / nugget_args are forwarded.
# =============================================================================

#' Consensus clustering from stacked ABC label matrices
#'
#' Turns a row-stacked matrix of per-iteration cluster labels
#' (\eqn{K \cdot R} rows by \eqn{N} columns) into a co-clustering dissimilarity
#' and applies \code{cluster::agnes}. \eqn{S_{ij}} is the co-clustering
#' probability over iterations selecting both objects (B6). The co-clustering
#' counts use the compiled C++ kernel \code{count_coclustering_cpp}
#' (from \code{src/mcpp_sampling.cpp}) when available, otherwise a pure-R loop.
#'
#' @param res Integer label matrix (\code{0} = object not selected).
#' @param numclust Optional number of clusters to cut the tree into.
#' @param dissimilarity \code{"mabc"} (arc-cosine, default) or \code{"abc"} (linear).
#' @param linkage AGNES agglomeration method.
#' @param alpha Flexible-linkage parameter.
#' @param mds Logical; draw an MDS of the dissimilarities.
#' @return A list with \code{DistM}, \code{Clust} and (if \code{numclust} given)
#'   \code{cut}.
#' @references \insertCite{Amaratunga2008}{MosaiClusteR}
#' @seealso \code{\link{M_ABCpp}}
#' @export
f.clustABC.MultiSource <- function(res,
                                   numclust      = NULL,
                                   dissimilarity = "mabc",
                                   linkage       = "ward",
                                   alpha         = 0.625,
                                   mds           = FALSE) {

  res <- as.matrix(res)
  storage.mode(res) <- "integer"
  N <- ncol(res)

  # co-clustering counts: prefer the C++ kernel if present, else pure-R loop.
  # (Same exists() guard as the original; the kernel now ships in src/.)
  if (exists("count_coclustering_cpp", mode = "function", inherits = TRUE)) {
    counts       <- get("count_coclustering_cpp")(res)
    co_sel       <- counts$co_sel        # times i and j were BOTH selected
    not_co_clust <- counts$not_co_clust  # ... and placed in DIFFERENT clusters
  } else {
    co_sel       <- matrix(0.0, N, N)
    not_co_clust <- matrix(0.0, N, N)
    for (i in seq_len(N)) {
      for (j in i:N) {                    # upper triangle (symmetry)
        both <- (res[, i] != 0L) & (res[, j] != 0L)
        cs   <- sum(both)
        ncc  <- sum(both & (res[, i] != res[, j]))
        co_sel[i, j]       <- cs;  co_sel[j, i]       <- cs
        not_co_clust[i, j] <- ncc; not_co_clust[j, i] <- ncc
      }
    }
  }

  # Sij = co-clustering probability with the correct denominator          [B6]
  Sij <- matrix(0.0, N, N)
  nz  <- co_sel > 0
  Sij[nz] <- (co_sel[nz] - not_co_clust[nz]) / co_sel[nz]

  # Dissimilarity                                                          [B7]
  Dij <- switch(
    dissimilarity,
    abc  = 1.0 - Sij,
    mabc = { M <- 1.0 - Sij^2; M[M < 0] <- 0; sqrt(M) },   # shape-preserving
    stop("'dissimilarity' must be 'abc' or 'mabc'")
  )
  diag(Dij) <- 0.0
  rownames(Dij) <- colnames(Dij) <- colnames(res)

  if (isTRUE(mds)) {
    pts <- stats::cmdscale(stats::as.dist(Dij), k = 2)
    plot(pts, pch = 19, xlab = "Dim 1", ylab = "Dim 2",
         main = paste0("MDS - f.clustABC.MultiSource (", dissimilarity, ")"))
  }

  Clust <- cluster::agnes(Dij, diss = TRUE, method = linkage, par.method = alpha)
  out <- list(DistM = Dij, Clust = Clust)
  if (!is.null(numclust)) {
    nc <- as.integer(numclust)
    if (nc >= 2L && nc < N) out$cut <- stats::cutree(stats::as.hclust(Clust), k = nc)
  }
  attr(out, "method") <- "M-ABC"
  out
}

#' Multi-source Aggregating Bundles of Clusters (M-ABC), C++-accelerated
#'
#' Extends the single-source ABC algorithm of
#' \insertCite{Amaratunga2008}{MosaiClusteR} to \eqn{K \ge 2} sources: runs
#' \code{\link{ABCpp.SingleInMultiple}} per source, row-stacks the per-iteration
#' label matrices, and builds a single consensus dissimilarity via
#' \code{\link{f.clustABC.MultiSource}} (C++ kernel when available).
#'
#' @section Data-nugget weighting:
#' Set \code{weighting = "nugget"} (per source, recycled) to weight features by
#' the data-nugget between-group variance instead of the classic variance; see
#' \code{\link{ABCpp.SingleInMultiple}} and \code{\link{create_data_nuggets}}.
#'
#' @param List A list of \eqn{K} data matrices over the same objects,
#'   \strong{objects (samples) in rows} and features in columns.
#' @param distmeasure,weighting,normalize,bag,numvar,linkage Per-source vectors
#'   (recycled to \eqn{K}) forwarded to \code{\link{ABCpp.SingleInMultiple}}.
#' @param gr Unused; kept for compatibility.
#' @param numsim Iterations per source.
#' @param NC Base-cluster count per source (\code{NULL} = data-driven).
#' @param dissimilarity Consensus dissimilarity, \code{"mabc"} or \code{"abc"}.
#' @param final_linkage,alpha Final clustering controls.
#' @param mds Logical; MDS of the consensus dissimilarities.
#' @param nfeat Feature-subsample size forwarded to
#'   \code{\link{ABCpp.SingleInMultiple}} (\code{NULL} = \eqn{\lfloor\sqrt{G}\rfloor}).
#' @param nugget_type,nugget_args Forwarded to the nugget weighting.
#' @return A list of class \code{"M_ABC"} with \code{DistM} and \code{Clust}.
#' @references \insertAllCited{}
#' @seealso \code{\link{ABCpp.SingleInMultiple}}, \code{\link{f.clustABC.MultiSource}},
#'   \code{\link{create_data_nuggets}}
#' @examples
#' data(mosaic_toy)
#' fit <- M_ABCpp(mosaic_toy$List, numsim = 50, NC = 3,
#'                weighting = c("var", "nugget"))
#' table(stats::cutree(stats::as.hclust(fit$Clust), 3))
#' @export
M_ABCpp <- function(List,
                    distmeasure   = rep("euclidean", length(List)),
                    weighting     = rep("var",       length(List)),
                    normalize     = rep(FALSE,       length(List)),
                    gr            = c(),
                    bag           = rep(TRUE,        length(List)),
                    numsim        = 1000,
                    numvar        = rep(100,         length(List)),
                    linkage       = rep("ward.D2",   length(List)),
                    NC            = NULL,
                    dissimilarity = "mabc",
                    final_linkage = "ward",
                    alpha         = 0.625,
                    mds           = FALSE,
                    nfeat         = NULL,
                    nugget_type   = "between",
                    nugget_args   = list()) {

  K <- length(List)

  # recycle the per-source arguments to length K (as in the original)
  distmeasure <- rep_len(distmeasure, K)
  weighting   <- rep_len(weighting,   K)
  normalize   <- rep_len(normalize,   K)
  bag         <- rep_len(bag,         K)
  numvar      <- rep_len(numvar,      K)
  linkage     <- rep_len(linkage,     K)
  # per-source nugget args: one shared list, or a length-K list of lists
  if (length(nugget_args) && all(vapply(nugget_args, is.list, logical(1))))
    nugget_args <- rep_len(nugget_args, K)
  else
    nugget_args <- rep(list(nugget_args), K)

  Results <- NULL
  for (i in seq_len(K)) {
    message("M_ABCpp: source ", i, " of ", K, " (weighting = ", weighting[i], ") ...")
    norm_i <- if (isFALSE(normalize[[i]])) NULL else normalize[[i]]
    res <- ABCpp.SingleInMultiple(          # the original single-source engine
      data        = List[[i]],
      distmeasure = distmeasure[i],
      weighting   = weighting[i],
      normalize   = norm_i,
      gr          = gr,
      bag         = bag[i],
      numsim      = numsim,
      numvar      = numvar[i],
      linkage     = linkage[i],
      NC          = NC,                     # B9: no NC2 / extra args
      nfeat       = nfeat,
      nugget_type = nugget_type,
      nugget_args = nugget_args[[i]]
    )
    res[is.na(res)] <- 0L
    Results <- rbind(Results, res)
  }

  message("M_ABCpp: building consensus dissimilarity",
          if (exists("count_coclustering_cpp", mode = "function")) " (C++ kernel)" else " (pure R)",
          " ...")
  out <- f.clustABC.MultiSource(
    res           = Results,
    dissimilarity = dissimilarity,
    linkage       = final_linkage,
    alpha         = alpha,                  # B8
    mds           = mds
  )
  class(out) <- c("M_ABC", class(out))
  out
}

# =============================================================================
# abc-helpers.R  - internal helpers shared by the ABC / M-ABC family.
# =============================================================================

# Solve for c so the top `top_frac` of features carry `target` of the combined
# selection probability under w = 1 / (rank + c). (Amaratunga et al. 2008.)
.solve_weight_c <- function(G, top_frac = 0.01, target = 0.20) {
  n_top <- max(1L, floor(G * top_frac))
  if (n_top >= G) return(1e-6)
  obj <- function(cc) {
    w <- 1.0 / (seq_len(G) + cc)
    sum(w[seq_len(n_top)]) / sum(w) - target
  }
  lo <- 1e-6; hi <- G * 1e4
  if (obj(lo) <= 0) return(lo)
  if (obj(hi) >= 0) return(hi)
  stats::uniroot(obj, lower = lo, upper = hi, tol = 1e-10)$root
}

# Feature-subsample size per iteration. nfeat = NULL -> floor(sqrt(G)) (the ABC
# default, best for high-dimensional sparse-signal data); a value in (0,1) is a
# fraction of G; a value >= 1 is an absolute count. Raising it helps when G is
# small and most features are informative (feature subsampling would otherwise
# discard signal).
.abc_gsize <- function(nf, nfeat = NULL) {
  if (is.null(nfeat))            return(max(2L, floor(sqrt(nf))))
  if (nfeat > 0 && nfeat < 1)    return(max(2L, min(nf, as.integer(round(nfeat * nf)))))
  max(2L, min(nf, as.integer(nfeat)))
}

# Per-feature selection-probability vector for the ABC family. X is
# features-by-objects (internal layout). Implements the data-nugget weighting
# option alongside the classic variance / coefficient-of-variation schemes.
.abc_zf <- function(X, weighting, nugget_type = "between", nugget_args = list()) {
  nf <- nrow(X)
  score <- switch(as.character(weighting),
    var = matrixStats::rowVars(X, na.rm = TRUE),
    cv  = {
      mn <- matrixStats::rowMeans2(X, na.rm = TRUE)
      mn[abs(mn) < .Machine$double.eps] <- .Machine$double.eps
      matrixStats::rowSds(X, na.rm = TRUE) / abs(mn)
    },
    nugget = {
      dn <- do.call(create_data_nuggets, c(list(x = t(X)), nugget_args))
      nugget_feature_weights(dn, type = nugget_type)
    },
    NULL)
  if (is.null(score)) return(rep(1.0 / nf, nf))   # equal weights
  score[!is.finite(score)] <- 0
  cc <- .solve_weight_c(nf)
  zf <- 1.0 / (rank(-score, ties.method = "first") + cc)
  zf / sum(zf)
}

# =============================================================================
# distance.R  - distance/dissimilarity computation dispatcher
# =============================================================================

#' Compute a distance matrix
#'
#' Dispatcher computing an object-by-object distance matrix under a range of
#' measures suitable for continuous, binary and mixed-type data.
#'
#' @param Data A data matrix; rows are the objects.
#' @param distmeasure One of \code{"tanimoto"}, \code{"jaccard"},
#'   \code{"euclidean"} (Gower), \code{"hamming"}, \code{"cont tanimoto"},
#'   \code{"MCA_coord"}, \code{"gower"}, \code{"chi.squared"}, \code{"cosine"}.
#' @param normalize Optional normalisation method (see \code{\link{Normalization}}),
#'   applied for the \code{"euclidean"} / \code{"cont tanimoto"} measures.
#' @param method Ignored; retained for backward compatibility.
#' @return A symmetric numeric distance matrix.
#' @details \code{"cosine"}, \code{"chi.squared"} and \code{"MCA_coord"} require
#'   the suggested packages \pkg{lsa}, \pkg{analogue} and \pkg{FactoMineR}
#'   respectively.
#' @examples
#' m <- matrix(rbinom(40, 1, 0.5), nrow = 8)
#' D <- Distance(m, distmeasure = "tanimoto")
#' dim(D)
#' @export
Distance <- function(Data,
                     distmeasure = c("tanimoto", "jaccard", "euclidean",
                                     "hamming", "cont tanimoto", "MCA_coord",
                                     "gower", "chi.squared", "cosine"),
                     normalize = NULL, method = NULL) {
  distmeasure <- match.arg(distmeasure)

  if (distmeasure == "euclidean" && !is.null(normalize))
    Data <- Normalization(Data, method = normalize)

  tanimoto <- function(m) {
    m <- as.matrix(m); storage.mode(m) <- "numeric"; n <- nrow(m)
    if (!anyNA(m)) {
      N.C <- m %*% t(m); N.A <- m %*% (1 - t(m)); N.B <- (1 - m) %*% t(m)
      denom <- N.A + N.B + N.C
      S <- N.C / denom; S[denom == 0] <- 0
    } else {
      S <- matrix(NA_real_, n, n)
      for (i in seq_len(n)) for (j in i:n) {
        xi <- m[i, ]; xj <- m[j, ]; ok <- !is.na(xi) & !is.na(xj)
        if (!any(ok)) { S[i, j] <- S[j, i] <- 0; next }
        xi <- xi[ok]; xj <- xj[ok]
        c <- sum(xi == 1 & xj == 1); u <- sum(xi == 1 | xj == 1)
        S[i, j] <- S[j, i] <- if (u == 0) 0 else c / u
      }
    }
    D <- 1 - S; diag(D) <- 0; (D + t(D)) / 2
  }

  if (distmeasure == "jaccard") {
    dist <- as.matrix(ade4::dist.binary(Data, method = 1))
  } else if (distmeasure == "tanimoto") {
    dist <- as.matrix(tanimoto(Data)); rownames(dist) <- rownames(Data)
  } else if (distmeasure %in% c("euclidean", "gower")) {
    dist <- as.matrix(FD::gowdis(Data))
  } else if (distmeasure == "hamming") {
    dist <- as.matrix(e1071::hamming.distance(Data))
  } else if (distmeasure == "MCA_coord") {
    if (!requireNamespace("FactoMineR", quietly = TRUE))
      stop("distmeasure='MCA_coord' requires the 'FactoMineR' package.")
    Data <- as.data.frame(Data); Data <- Data[, sapply(Data, nlevels) > 1]
    MCA_result <- FactoMineR::MCA(Data, graph = FALSE)
    dist <- Distance(MCA_result$ind$coord, distmeasure = "euclidean", normalize = NULL)
  } else if (distmeasure == "chi.squared") {
    if (!requireNamespace("analogue", quietly = TRUE))
      stop("distmeasure='chi.squared' requires the 'analogue' package.")
    Temp <- ade4::acm.disjonctif(Data)
    dist <- analogue::distance(Temp, method = "chi.distance")
  } else if (distmeasure == "cosine") {
    if (!requireNamespace("lsa", quietly = TRUE))
      stop("distmeasure='cosine' requires the 'lsa' package.")
    Temp <- lsa::cosine(t(as.matrix(Data)))
    dist <- 1 - (1 - (2 * acos(Temp)) / pi)
  } else if (distmeasure == "cont tanimoto") {
    if (!is.null(normalize)) Data <- Normalization(Data, method = normalize)
    m <- as.matrix(Data); X_A_X_B <- m %*% t(m); temp <- diag(X_A_X_B)
    X_A <- matrix(rep(temp, nrow(Data)), nrow(Data), nrow(Data), byrow = FALSE)
    X_B <- t(X_A); Denom <- X_A + X_B - X_A_X_B
    dist <- 1 - (X_A_X_B / Denom)
  } else {
    stop("Incorrect choice of distmeasure.")
  }
  colnames(dist) <- rownames(dist)
  dist
}

#' @rdname Distance
#' @export
Distance_v2 <- function(Data, distmeasure = "tanimoto", normalize = NULL)
  Distance(Data, distmeasure = distmeasure, normalize = normalize)

# =============================================================================
# data-sim.R  - synthetic multi-source data generator + bundled example data
# =============================================================================

#' Simulate a multi-source data set with planted clusters
#'
#' Generates \code{K} feature-by-sample matrices that share a common set of
#' \code{n} samples drawn from \code{g} ground-truth groups. Each source has its
#' own informative features (shifted between groups) plus noise features, so the
#' clusters are only partially visible in any single source - the canonical
#' setting in which multi-source integration helps.
#'
#' @param n Number of samples (objects).
#' @param g Number of ground-truth groups.
#' @param p Per-source feature counts (recycled to length \code{K}).
#' @param informative Per-source number of group-informative features.
#' @param effect Between-group mean shift for informative features.
#' @param K Number of sources.
#' @param seed Optional RNG seed.
#' @return A list with \code{List} (the \code{K} matrices, \strong{objects in
#'   rows}, features in columns) and \code{truth} (the integer group label per
#'   object).
#' @examples
#' sim <- mosaic_sim(n = 30, g = 3, K = 2, seed = 1)
#' lengths(lapply(sim$List, dim))
#' @export
mosaic_sim <- function(n = 60, g = 3, p = c(120, 150), informative = c(20, 25),
                       effect = 2.5, K = 2, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  p           <- rep_len(p, K)
  informative <- rep_len(informative, K)
  truth   <- sort(rep_len(seq_len(g), n))
  s_names <- sprintf("obj%02d", seq_len(n))

  List <- vector("list", K)
  for (k in seq_len(K)) {
    X <- matrix(stats::rnorm(n * p[k]), nrow = n, ncol = p[k])  # objects x features
    inf <- seq_len(informative[k])
    # each informative feature separates a (cyclically chosen) group
    for (j in inf) {
      gj <- ((j - 1) %% g) + 1
      X[truth == gj, j] <- X[truth == gj, j] + effect
    }
    rownames(X) <- s_names
    colnames(X) <- sprintf("src%d_f%03d", k, seq_len(p[k]))
    List[[k]] <- X
  }
  names(List) <- sprintf("source%d", seq_len(K))
  list(List = List, truth = stats::setNames(truth, s_names))
}

#' Toy multi-omics example data
#'
#' A small two-source example produced by \code{\link{mosaic_sim}} with three
#' planted groups, bundled for examples and tests.
#'
#' @format A list with two elements:
#' \describe{
#'   \item{List}{A list of two numeric matrices with the 60 shared objects in
#'     rows and features in columns (120 and 150 features).}
#'   \item{truth}{Integer vector of the 60 ground-truth group labels.}
#' }
#' @examples
#' data(mosaic_toy)
#' str(mosaic_toy, max.level = 2)
"mosaic_toy"

# =============================================================================
# simulate-weighting.R
# Simulation designed to evaluate FEATURE-WEIGHTING schemes fairly, in
# particular the coefficient-of-variation (CV) weighting. Two regimes:
#
#   "variance" - the discriminative features carry high ABSOLUTE variance and
#                the decoys are low-variance. Ranking by variance and by CV
#                coincide, so variance / CV / nugget weighting all succeed.
#
#   "cv"       - the discriminative features have a LOW mean with moderate
#                spread (high CV) while the decoys have a HIGH mean and high
#                absolute variance but are NOT discriminative (variance grows
#                with the mean, as in count / intensity omics data). Variance
#                weighting is then dominated by the high-abundance decoys and
#                misses the signal, whereas CV weighting surfaces it.
#
# This lets the package demonstrate *when* CV weighting is advantageous, rather
# than only the variance-dominated regime in which CV appears inferior.
# =============================================================================

#' Simulate data for evaluating feature-weighting schemes (variance vs CV)
#'
#' Generates a multi-source data set with a known cluster structure in which the
#' discriminative signal is carried either by high-absolute-variance features
#' (\code{regime = "variance"}) or by high-\emph{coefficient-of-variation}
#' features buried under high-abundance, high-variance decoys
#' (\code{regime = "cv"}). It is intended for benchmarking the \code{"var"},
#' \code{"cv"} and \code{"nugget"} weightings of the ABC family (and any other
#' feature-weighting method) on the same footing.
#'
#' @param regime \code{"cv"} (default) or \code{"variance"}; see Details.
#' @param n_samples Number of objects (rows), split evenly across clusters.
#' @param n_clusters Number of clusters.
#' @param n_sources Number of data sources (independent draws of the same design).
#' @param n_signal,n_decoy,n_noise Numbers of discriminative, decoy
#'   (non-discriminative but high-variance in the CV regime) and pure-noise
#'   features per source.
#' @param effect Between-cluster mean shift on the signal features (defaults:
#'   1.4 for the variance regime, 0.9 for the cv regime).
#' @param seed Optional RNG seed.
#'
#' @return A list with \code{List} (the sources, objects in rows), \code{truth}
#'   (cluster labels), \code{signal_features} (indices of the discriminative
#'   features), \code{regime} and \code{params}.
#'
#' @seealso \code{\link{M_ABCpp}} (the \code{weighting} argument),
#'   \code{\link{mosaic_sim}}
#' @examples
#' cv  <- simulate_weighting_regime("cv", seed = 1)
#' # variance weighting is misled by high-abundance decoys, CV recovers the truth:
#' fit_var <- M_ABCpp(cv$List, numsim = 80, NC = 3, weighting = c("var", "var"))
#' fit_cv  <- M_ABCpp(cv$List, numsim = 80, NC = 3, weighting = c("cv",  "cv"))
#' c(var = cluster_agreement(mosaic_labels(fit_var, 3), cv$truth)[["ARI"]],
#'   cv  = cluster_agreement(mosaic_labels(fit_cv,  3), cv$truth)[["ARI"]])
#' @export
simulate_weighting_regime <- function(regime      = c("cv", "variance"),
                                      n_samples   = 120,
                                      n_clusters  = 3,
                                      n_sources   = 2,
                                      n_signal    = 40,
                                      n_decoy     = 500,
                                      n_noise     = 60,
                                      effect      = NULL,
                                      seed        = NULL) {
  regime <- match.arg(regime)
  if (!is.null(seed)) set.seed(seed)
  z  <- rep(seq_len(n_clusters), length.out = n_samples)   # balanced clusters
  dir <- as.integer(factor(z)) - (n_clusters + 1) / 2      # per-cluster direction
  if (is.null(effect)) effect <- if (regime == "variance") 1.4 else 0.9

  one_source <- function() {
    if (regime == "variance") {
      # signal: cluster mean-shift on unit-scale features -> high absolute variance
      Xs <- vapply(seq_len(n_signal), function(j)
        stats::rnorm(n_samples, 5 + effect * dir, 1.2), numeric(n_samples))
      Xd <- matrix(stats::rnorm(n_samples * n_decoy, 5, 0.3), n_samples)  # low-variance decoys
      Xn <- matrix(stats::rnorm(n_samples * n_noise, 5, 1.0), n_samples)
    } else {
      # signal: LOW mean, moderate spread, cluster mean-shift -> high CV, modest abs. variance
      Xs <- vapply(seq_len(n_signal), function(j)
        abs(stats::rnorm(n_samples, 3 + effect * dir, 0.7)), numeric(n_samples))
      # decoys: HIGH mean, variance proportional to mean (count/intensity-like),
      # NOT cluster-discriminative -> high absolute variance, LOW CV
      Xd <- vapply(seq_len(n_decoy), function(j) {
        mu <- stats::runif(1, 25, 60); stats::rnorm(n_samples, mu, sqrt(mu * 2)) },
        numeric(n_samples))
      Xn <- matrix(stats::rnorm(n_samples * n_noise, 8, 1.0), n_samples)
    }
    X <- cbind(Xs, Xd, Xn)
    rownames(X) <- sprintf("obj%03d", seq_len(n_samples))
    colnames(X) <- sprintf("f%04d", seq_len(ncol(X)))
    X
  }

  List <- lapply(seq_len(n_sources), function(i) one_source())
  names(List) <- sprintf("source%d", seq_len(n_sources))
  list(List = List,
       truth = stats::setNames(z, sprintf("obj%03d", seq_len(n_samples))),
       signal_features = seq_len(n_signal),
       regime = regime,
       params = list(n_samples = n_samples, n_clusters = n_clusters,
                     n_sources = n_sources, n_signal = n_signal,
                     n_decoy = n_decoy, n_noise = n_noise, effect = effect))
}

test_that("plot/feature functions are exported functions", {
  expect_true(is.function(BinFeaturesPlot_SingleData))
  expect_true(is.function(BinFeaturesPlot_MultipleData))
  expect_true(is.function(ContFeaturesPlot))
  expect_true(is.function(PlotPathways))
  expect_true(is.function(ProfilePlot))
})

test_that("BinFeaturesPlot_MultipleData needs plotrix", {
  testthat::skip_if_not_installed("plotrix")
  expect_true(is.function(BinFeaturesPlot_MultipleData))
})

test_that("PlotPathways needs MLP/biomaRt/org.Hs.eg.db", {
  testthat::skip_if_not_installed("MLP")
  testthat::skip_if_not_installed("biomaRt")
  testthat::skip_if_not_installed("org.Hs.eg.db")
  expect_true(is.function(PlotPathways))
})

test_that("ProfilePlot needs Biobase for ExpressionSet input", {
  testthat::skip_if_not_installed("Biobase")
  expect_true(is.function(ProfilePlot))
})

test_that("characterization and tracking utilities are exported functions", {
  expect_true(is.function(CharacteristicFeatures))
  expect_true(is.function(ChooseCluster))
  expect_true(is.function(FeatSelection))
  expect_true(is.function(FeaturesOfCluster))
  expect_true(is.function(TrackCluster))
})

test_that("ChooseCluster gene-expression path needs suggested packages", {
  testthat::skip_if_not_installed("limma")
  testthat::skip_if_not_installed("a4Core")
  testthat::skip_if_not_installed("a4Base")
  expect_true(is.function(ChooseCluster))
})

---
title: "MosaiClusteR: an umbrella framework for multi-source and multi-omics clustering"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{MosaiClusteR: an umbrella framework for multi-source and multi-omics clustering}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", warning = FALSE,
                      message = FALSE, fig.width = 7, fig.height = 4.5)
set.seed(1)
```

# What MosaiClusteR is for

Modern studies rarely describe their samples with a single table. A cohort may be
profiled by several assays at once — transcriptomics, proteomics, metabolomics,
methylation, microbiome — or a set of objects may be described by several
heterogeneous data sources. **MosaiClusteR** is a framework for discovering the
sample structure that these sources *agree* on: it takes a collection of data
matrices measured over the **same objects** and returns an integrated clustering.

The package is deliberately a **catalogue, not a single algorithm**. Multi-source
clustering can be approached in many ways, and no one method is best across all
data regimes, so MosaiClusteR gathers a broad set of methods behind **one
consistent interface** and adds everything you need to *choose between them*:
preprocessing, cluster-number selection, agreement metrics, visual comparison,
cluster characterisation, and — for omics — pathway and gene-set interpretation.

The methods are organised into five integration **paradigms**:

| Paradigm | Idea | Examples in this package |
|---|---|---|
| **Direct** | Concatenate the sources, cluster once | `ADC`, `ADEC` |
| **Similarity-based** | Combine per-source object similarities | `WeightedClust`, `WonM`, `SNF`, `NEMO` |
| **Graph-based** | Partition a graph built from many clusterings | `EnsembleClustering`, `HBGF`, `ClusteringAggregation` |
| **Voting / consensus** | Let many partitions vote on co-membership | `CEC`, `CVAA`, `ConsensusClustering`, `EvidenceAccumulation`, `LinkBasedClustering`, the ABC family |
| **Hierarchy-based** | Merge per-source hierarchies | `HierarchicalEnsembleClustering`, `EHC` |
| **Factor / low-rank** | Cluster a shared latent representation | `intNMF`, `spectral_clustering`, `LUCID` |

```{r setup}
library(MosaiClusteR)
```

# The data model

Every method in MosaiClusteR speaks the same language.

* **Input** is a `List` of numeric matrices, one per source, all measured over the
  **same objects in the same order**. The convention is **objects in rows**,
  features in columns (`n` objects x `m_k` features for source `k`). Sources may
  have different numbers of features; they only need to share the objects.
* **Output** is a small list with two standard components: `DistM` (an
  object-by-object dissimilarity, `n x n`) and `Clust` (a fitted hierarchical
  clustering). Some methods additionally return a `$cluster` label vector.
* Methods that work from a precomputed dissimilarity accept `type = "dist"`;
  otherwise `type = "data"` (the default) starts from the raw matrices.

Because the interface is uniform, swapping one method for another is a one-line
change, and the evaluation / visualisation tools work with any of them.

The package ships a small toy data set — two sources over 60 shared objects with
three planted groups — used throughout this vignette.

```{r data}
data(mosaic_toy)
L     <- mosaic_toy$List      # a list of two matrices, objects in rows
truth <- mosaic_toy$truth     # the known group of each object (for illustration)
S     <- length(L)            # number of sources
K     <- 3                    # number of groups
vapply(L, dim, integer(2))    # 60 objects x (120, 150) features
```

You can also simulate structured multi-source data with `mosaic_sim()`:

```{r sim, eval = FALSE}
sim <- mosaic_sim(n = 90, g = 3, seed = 1)   # n objects, g groups
str(sim$List, max.level = 1)
```

# A first end-to-end analysis

A typical analysis moves through four layers: **preprocess -> baseline ->
integrate -> evaluate**.

## Layer 1 - Preprocess

Make heterogeneous sources comparable and turn a source into a dissimilarity.

```{r preprocess}
Xn <- Normalization(L[[1]], method = "Quantile")   # optional per-source scaling
D1 <- Distance(L[[1]], distmeasure = "euclidean")   # object-by-object distance
round(as.matrix(D1)[1:4, 1:4], 2)
```

`Distance()` supports many measures — `"euclidean"`, `"manhattan"`,
`"tanimoto"`/`"jaccard"` (for binary fingerprints), and correlation-based
distances among them — and `Normalization()` offers quantile, standardisation and
range scaling. Binary sources (e.g. molecular fingerprints, presence/absence)
are handled by choosing an appropriate `distmeasure`.

## Layer 2 - Single-source baselines

Cluster each source on its own to see what it recovers in isolation. Weak or
*disagreeing* baselines are exactly what motivates integration.

```{r baseline}
b1 <- Cluster(L[[1]], distmeasure = "euclidean", normalize = FALSE, nrclusters = 3)
b2 <- Cluster(L[[2]], distmeasure = "euclidean", normalize = FALSE, nrclusters = 3)
c(source1 = cluster_agreement(mosaic_labels(b1, 3), truth)[["ARI"]],
  source2 = cluster_agreement(mosaic_labels(b2, 3), truth)[["ARI"]])
```

## Layer 3 - Integrate, and Layer 4 - Evaluate

The whole point of the uniform interface is that you can run *many* integration
methods with almost identical calls and score them the same way. Here we run one
method from each paradigm and compare how well each recovers the planted groups
with the Adjusted Rand Index (`cluster_agreement()` also returns NMI, Jaccard and
purity). `mosaic_labels(fit, k)` cuts any fitted result into `k` clusters.

```{r integrate}
DM <- rep("euclidean", S); NM <- rep(FALSE, S); ME <- rep(list(NULL), S)

methods <- list(
  "ADC (direct)"           = function() mosaic_labels(
      ADC(L, distmeasure = "euclidean", normalize = FALSE), K),
  "WeightedClust (simil.)" = function() mosaic_labels(
      WeightedClust(L, type = "data", distmeasure = DM, normalize = NM,
                    method = ME, weight = seq(1, 0, -0.25)), K),
  "WonM (similarity)"      = function() mosaic_labels(
      WonM(L, type = "data", distmeasure = DM, normalize = NM, method = ME,
           nrclusters = rep(list(2:6), S), linkage = rep("ward", S)), K),
  "SNF (graph fusion)"     = function() mosaic_labels(
      SNF(L, type = "data", distmeasure = DM, normalize = NM, method = ME,
          NN = 20, T = 20), K),
  "NEMO (neighbourhood)"   = function() NEMO(L, k = K, NN = 20)$cluster,
  "CEC (consensus)"        = function() mosaic_labels(
      CEC(L, distmeasure = DM, normalize = NM, method = ME,
          r = vapply(L, ncol, 1L) %/% 2L, nrclusters = as.list(rep(K, S)),
          linkage = rep("ward", S)), K),
  "intNMF (factor)"        = function() intNMF(L, k = K, nstart = 4)$cluster,
  "Data nuggets"           = function() nugget_cluster(
      do.call(cbind, L), k = K, max_nuggets = 40)$cluster,
  "ABC (voting)"           = function() mosaic_labels(
      M_ABCpp(L, numsim = 60, NC = K), K)
)

ari <- vapply(methods, function(f)
  tryCatch(round(cluster_agreement(f(), truth)[["ARI"]], 2),
           error = function(e) NA_real_), numeric(1))
sort(ari, decreasing = TRUE)
```

On this toy set most methods recover the planted structure; on real data they
diverge, which is why running several and comparing them is the recommended
workflow rather than trusting a single algorithm.

# The method catalogue

This section describes each integration method, grouped by paradigm. All take the
same `List` input; the calls below are complete and ready to adapt.

## Direct integration

The sources are combined into one representation and clustered once.

* **`ADC`** — Aggregated Data Clustering: concatenate the sources (after optional
  per-source normalisation), form a single distance and one hierarchy.
* **`ADEC`** — Aggregated Data Ensemble Clustering: repeatedly subsample /
  reduce the concatenated data and aggregate the resulting partitions.

```{r direct, eval = FALSE}
ADC(L, distmeasure = "euclidean", normalize = FALSE)
ADEC(L, distmeasure = DM, normalize = NM, method = ME, nrclusters = 3, t = 10)
```

## Similarity-based integration

Each source contributes an object-by-object similarity; the similarities are
combined before clustering.

* **`WeightedClust`** — convex (weighted) combination of per-source distances
  over a grid of weights; good when you want to control each source's influence.
* **`WonM`** — Weighting on Membership: consensus co-membership accumulated over
  many cut heights, so no single `k` has to be fixed up front.
* **`SNF`** — Similarity Network Fusion: iteratively cross-diffuses per-source
  similarity networks into a single fused network (needs `SNFtool`).
* **`NEMO`** — NEighborhood based Multi-Omics clustering: averages neighbourhood
  affinities across sources and tolerates partially missing samples.

```{r simil, eval = FALSE}
WeightedClust(L, type = "data", distmeasure = DM, normalize = NM, method = ME,
              weight = seq(1, 0, -0.25))
WonM(L, type = "data", distmeasure = DM, normalize = NM, method = ME,
     nrclusters = rep(list(2:6), S), linkage = rep("ward", S))
SNF(L, type = "data", distmeasure = DM, normalize = NM, method = ME, NN = 20, T = 20)
NEMO(L, k = K, NN = 20)$cluster
```

## Graph-based integration

Many base clusterings are encoded as a graph (of objects and/or clusters) that is
then partitioned.

* **`EnsembleClustering`** — CSPA / HGPA / MCLA hyper-graph consensus.
* **`HBGF`** — Hybrid Bipartite Graph Formulation over objects and clusters.
* **`ClusteringAggregation`** — aggregate many partitions by a distance between
  clusterings.

```{r graph, eval = FALSE}
HBGF(L, type = "data", distmeasure = DM, normalize = NM, method = ME, nrclusters = 3)
ClusteringAggregation(L, type = "data", distmeasure = DM, normalize = NM,
                      method = ME, nrclusters = 3)
# EnsembleClustering() / EHC() call an external graph-partitioning executable
# (e.g. METIS); install it and pass its path via 'executable' to use them.
```

## Voting / consensus integration

Many partitions "vote" on whether each pair of objects belongs together; a
consensus co-membership is then clustered.

* **`CEC`** — (Consensus) Ensemble Clustering by incidence accumulation across
  sources and cut points; weight-aware.
* **`CVAA`** — Cumulative Voting Adjusted Aggregation, aligning partitions to a
  reference before merging.
* **`ConsensusClustering`** — iterative (probabilistic) voting consensus.
* **`EvidenceAccumulation`** — co-association matrix across partitions.
* **`LinkBasedClustering`** — refines the co-association with link-based
  similarity (CTS / SRS / ASRS).
* **The ABC family** — `M_ABCpp`, `M_ABCdist` and the deep-learning variant
  `M_ABCdeep` build a consensus by repeatedly bootstrapping objects and feature
  bundles per source; `M_ABCdist.WC` fuses per-source distances with an
  optimally weighted combination.

```{r voting, eval = FALSE}
CEC(L, distmeasure = DM, normalize = NM, method = ME,
    r = vapply(L, ncol, 1L) %/% 2L, nrclusters = as.list(rep(K, S)),
    linkage = rep("ward", S))
ConsensusClustering(L, type = "data", distmeasure = DM, normalize = NM,
                    method = ME, nrclusters = 3)
EvidenceAccumulation(L, type = "data", distmeasure = DM, normalize = NM,
                     method = ME, nrclusters = 3)
LinkBasedClustering(L, type = "data", distmeasure = DM, normalize = NM,
                    method = ME, nrclusters = 3, linkBasedMethod = "SRS")
M_ABCpp(L, numsim = 200, NC = 3)                 # co-clustering consensus
M_ABCdist(L, numsim = 200, NC = 3)               # distance-accumulating variant
M_ABCdeep(L, numsim = 200, NC = 3, ae_epochs = 60)   # autoencoder embedding
```

## Hierarchy-based integration

Per-source hierarchies are merged into a single dendrogram.

* **`HierarchicalEnsembleClustering`** — combine per-source hierarchical
  clusterings into a consensus hierarchy.
* **`EHC`** — Ensemble Hierarchical Clustering (uses an external partitioner).

```{r hier, eval = FALSE}
HierarchicalEnsembleClustering(L, type = "data", distmeasure = DM,
                               normalize = NM, method = ME)
```

## Factor, low-rank and spectral methods

Cluster a shared latent representation of the sources.

* **`intNMF`** — integrative non-negative matrix factorisation: a shared basis
  plus per-source coefficients; cluster the shared factors.
* **`spectral_clustering`** — spectral clustering of an affinity matrix (build one
  from a fused similarity, e.g. via `SimilarityMeasure()` or `SNF()`).
* **`LUCID`** — a wrapper for latent-cluster models that *use an outcome*
  (supervised integration); supply exposures `G`, omics `Z` and outcome `Y`.

```{r factor, eval = FALSE}
intNMF(L, k = K, nstart = 8)$cluster
spectral_clustering(affinity = as.matrix(exp(-as.matrix(D1))), k = K)
LUCID(G = exposures, Z = L, Y = outcome, K = 3)   # supervised; needs 'LUCIDus'
```

# Feature weighting and data nuggets

Not all features are equally informative. Several methods can weight features by
their importance, and MosaiClusteR provides a robust, big-data-friendly weighting
based on **data nuggets** — a compression of the objects into weighted
representatives that resists outliers and scales to large `n`.

```{r nuggets}
dn <- create_data_nuggets(L[[1]], max_nuggets = 25)   # compress objects
dn
w  <- nugget_feature_weights(dn, type = "between")    # feature importance
head(sort(w, decreasing = TRUE))
```

Data nuggets can also be clustered directly, and used as the feature weighting in
the ABC family (`weighting = "nugget"`):

```{r nuggets2, eval = FALSE}
nugget_cluster(do.call(cbind, L), k = K, max_nuggets = 40)$cluster  # cluster nuggets
Wkmeans(dn, k = K); Whclust(dn, k = K)                              # weighted k-means / hclust
M_ABCpp(L, numsim = 200, NC = 3, weighting = c("nugget", "nugget")) # nugget-weighted
```

# Choosing the number of clusters

`SelectnrClusters()` scores a range of `k` (silhouette / gap-style criteria) so
you do not have to guess. `ChooseCluster()` helps pick a cut of a fitted tree.

```{r selectk}
sel <- SelectnrClusters(L, type = "data", distmeasure = DM, normalize = NM,
                        method = ME, nrclusters = 2:6)
sel$Optimal_Nr_of_CLusters
```

# Evaluating and comparing clusterings

| Function | Purpose |
|---|---|
| `cluster_agreement(a, b)` | ARI, NMI, Jaccard, purity between two labellings |
| `compare_clusterings(D1, D2, NC)` | agreement between two integrated results |
| `mosaic_labels(fit, k)` | cut any fitted result into `k` labels |
| `CompareSilCluster`, `CompareSvsM` | silhouette-based comparison of solutions |
| `DetermineWeight_SilClust`, `DetermineWeight_SimClust` | data-driven source weights |

```{r evaluate}
fit_a <- ADC(L, distmeasure = "euclidean", normalize = FALSE)
fit_b <- SNF(L, type = "data", distmeasure = DM, normalize = NM, method = ME)
compare_clusterings(fit_a$DistM, fit_b$DistM, NC = 3)$ARI
cluster_agreement(mosaic_labels(fit_b, 3), truth)
```

# Visualising results

MosaiClusteR includes a plotting suite for inspecting and comparing solutions.
A fitted result carries a hierarchy you can draw directly:

```{r dendro, fig.height = 4}
fit <- SNF(L, type = "data", distmeasure = DM, normalize = NM, method = ME)
hc  <- stats::as.hclust(fit$Clust)
plot(hc, labels = FALSE, main = "SNF consensus", xlab = "", sub = "")
stats::rect.hclust(hc, k = 3, border = 2:4)
```

Other visualisations (all take fitted results or label vectors):

| Function | Shows |
|---|---|
| `ComparePlot` | several solutions aligned side-by-side (needs `circlize`, `plotrix`) |
| `ClusterPlot`, `Cyclogram` | a single solution as a coloured tree / cyclogram |
| `SimilarityHeatmap`, `HeatmapPlot` | object-by-object similarity / feature heatmaps |
| `ProfilePlot`, `ContFeaturesPlot`, `BinFeaturesPlot_*` | per-cluster feature profiles |
| `BoxPlotDistance` | within- vs between-cluster distances |
| `ColorPalette`, `ColorsNames`, `ClusterCols`, `LabelCols` | consistent colour helpers |

```{r viz, eval = FALSE}
ComparePlot(list("ADC" = fit_a, "SNF" = fit_b), nrclusters = 3)
SimilarityHeatmap(fit_b$DistM)
ProfilePlot(L[[1]], mosaic_labels(fit_b, 3))
```

# Characterising and tracking clusters

Once you have clusters, describe *why* they differ and follow them across
solutions.

| Function | Purpose |
|---|---|
| `CharacteristicFeatures`, `FeatSelection` | features that define each cluster |
| `FeaturesOfCluster` | the driving features of one cluster |
| `FindCluster`, `FindElement` | locate a cluster / object in a result |
| `TrackCluster` | follow a cluster across several solutions |
| `ReorderToReference` | align labels of one solution to a reference (Gale-Shapley) |
| `SharedComps`, `SimilarityMeasure` | shared members / similarity between solutions |

```{r charac, eval = FALSE}
CharacteristicFeatures(L, mosaic_labels(fit_b, 3))
ReorderToReference(reference = truth, tocompare = mosaic_labels(fit_a, 3))
```

# Biological interpretation (omics)

For omics data, MosaiClusteR connects clusters to genes and pathways. These
functions rely on Bioconductor packages (`limma`, `MLP`, `biomaRt`,
`org.Hs.eg.db`, ...), declared under *Suggests*; install them to enable this
layer. All examples here are illustrative.

| Function | Purpose |
|---|---|
| `DiffGenes`, `DiffGenesSelection`, `FindGenes` | differential features per cluster |
| `PathwayAnalysis`, `Pathways`, `PathwaysIter`, `PathwaysSelection` | pathway enrichment per cluster |
| `PreparePathway`, `PlotPathways` | prepare / visualise pathway results |
| `Geneset.intersect`, `SharedGenesPathsFeat`, `SharedSelection*` | shared genes / pathways across solutions |

```{r bio, eval = FALSE}
dg <- DiffGenes(L, mosaic_labels(fit_b, 3), geneexpr = L[[1]])
pa <- PathwayAnalysis(L, mosaic_labels(fit_b, 3), geneexpr = L[[1]])
PlotPathways(pa)
```

# Putting it together

A complete, reproducible analysis is short because every step shares the
interface:

```{r together, eval = FALSE}
L    <- my_sources                                   # list of matrices, objects in rows
k    <- SelectnrClusters(L, type = "data", distmeasure = rep("euclidean", length(L)),
                         normalize = rep(FALSE, length(L)),
                         method = rep(list(NULL), length(L)),
                         nrclusters = 2:8)$Optimal_Nr_of_CLusters
fit  <- SNF(L, type = "data", distmeasure = rep("euclidean", length(L)),
            normalize = rep(FALSE, length(L)), method = rep(list(NULL), length(L)))
lab  <- mosaic_labels(fit, k)
CharacteristicFeatures(L, lab)                        # what defines the clusters
```

Swap `SNF` for any other method above to compare paradigms on your own data.

```{r session, echo = FALSE}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{DiffGenes}
\alias{DiffGenes}
\title{Find differentially expressed genes}
\usage{
DiffGenes(
  List,
  Selection = NULL,
  geneExpr = NULL,
  nrclusters = NULL,
  method = "limma",
  sign = 0.05,
  topG = NULL,
  fusionsLog = TRUE,
  weightclust = TRUE,
  names = NULL
)
}
\arguments{
\item{List}{A list of the clustering outputs to be compared. The first
element of the list will be used as the reference in
\code{ReorderToReference}.}

\item{Selection}{If differential gene expression should be investigated for
a specific selection of objects, this selection can be provided here.
Selection can be of the type "character" (names of the objects) or
"numeric" (the number of specific cluster). Default is NULL.}

\item{geneExpr}{The gene expression matrix or ExpressionSet of the objects.
The rows should correspond with the genes.}

\item{nrclusters}{Optional. The number of clusters to cut the dendrogram in.
Default is NULL.}

\item{method}{The method to applied to look for DE genes. For now, only the
limma method is available. Default is "limma".}

\item{sign}{The significance level to be handled. Default is 0.05.}

\item{topG}{Overrules sign. The number of top genes to be shown. Default is NULL.}

\item{fusionsLog}{Logical. To be handed to \code{ReorderToReference}: indicator for the fusion of clusters. Default is TRUE}

\item{weightclust}{Logical. To be handed to \code{ReorderToReference}: to be used for the outputs of CEC,
WeightedClust or WeightedSimClust. If TRUE, only the result of the Clust element is considered. Default is TRUE.}

\item{names}{Optional. Names of the methods. Default is NULL.}
}
\value{
The result is a list with an element per method. Each element is
again a list with for each cluster the found differentially expressed genes.
}
\description{
The \code{DiffGenes} function looks for differentially expressed
genes for each cluster of a clustering result by comparing the objects of a
cluster against all other objects. The limma method is applied to find the
differentially expressed genes.
}
\details{
This function relies on the suggested Bioconductor package 'limma'.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)
data(geneMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(MCF7_F,MCF7_T)
names=c('FP','TP')

MCF7_FT_DE = DiffGenes(List=L,geneExpr=geneMat,nrclusters=7,method="limma",sign=0.05,
topG=10,fusionsLog=TRUE,weightclust=TRUE,names=names)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-compare.r
\name{DetermineWeight_SilClust}
\alias{DetermineWeight_SilClust}
\title{Determines an optimal weight for weighted clustering by silhouettes widths.}
\usage{
DetermineWeight_SilClust(
  List,
  type = c("data", "dist", "clusters"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  weight = seq(0, 1, by = 0.01),
  nrclusters = NULL,
  names = NULL,
  nboot = 10,
  StopRange = FALSE,
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{List}{A list of matrices of the same type. It is assumed the rows are corresponding with the objects.}

\item{type}{indicates whether the provided matrices in "List" are either data matrices, distance
matrices or clustering results obtained from the data. Type should be one of "data", "dist" or "clusters".}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices. Default is c(FALSE, FALSE).}

\item{method}{A method of normalization. Default is c(NULL,NULL).}

\item{weight}{Optional. A list of different weight combinations for the data sets in List. Defaults to seq(0,1,by=0.01).}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{names}{The labels to give to the elements in List. Default is NULL.}

\item{nboot}{Number of bootstraps to be run. Default is 10.}

\item{StopRange}{Logical. Indicates whether the distance matrices with values not between zero and one
should be standardized to have so. Default is FALSE.}

\item{plottype}{Should be one of "pdf","new" or "sweave". Default is "new".}

\item{location}{If plottype is "pdf", a location should be provided here. Default is NULL.}
}
\value{
Two plots and a list with two elements: \item{Result}{A data frame with the statistic for each
weight combination} \item{Weight}{The optimal weight}.
}
\description{
The function \code{DetermineWeight_SilClust} determines an optimal weight
for weighted similarity clustering by calculating silhouettes widths. For each given
weight, a linear combination of the distance matrices of the single data sources is
obtained, medoid clustering is performed and the silhouette widths are regressed against
the cluster memberships. A statistic is derived and a p-value obtained via bootstrapping.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(MCF7_F,MCF7_T)

MC7_Weight=DetermineWeight_SilClust(List=L,type="clusters",distmeasure=
c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),
weight=seq(0,1,by=0.01),nrclusters=c(7,7),names=c("FP","TP"),nboot=10,
StopRange=FALSE,plottype="new",location=NULL)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nuggets.R
\name{create_data_nuggets}
\alias{create_data_nuggets}
\title{Create data nuggets from a data matrix}
\usage{
create_data_nuggets(
  x,
  max_nuggets = NULL,
  min_size = 1L,
  engine = c("native", "datanugget"),
  scale_features = TRUE,
  seed = NULL,
  ...
)
}
\arguments{
\item{x}{A numeric matrix or data frame with observations in the \emph{rows}
and variables in the \emph{columns}.}

\item{max_nuggets}{Target (maximum) number of nuggets \eqn{M}. Defaults to a
data-driven value (see Details). Ignored when \code{engine = "datanugget"}.}

\item{min_size}{Minimum number of observations per nugget (native engine
only). Defaults to \code{1}.}

\item{engine}{Either \code{"native"} (a self-contained k-means based
partition, no extra dependencies) or \code{"datanugget"} (delegates to the
\pkg{datanugget} package via \code{datanugget::create.DN} /
\code{datanugget::refine.DN} when it is installed).}

\item{scale_features}{Logical; if \code{TRUE} (default) the partition is
computed on standardised features so that no single high-variance variable
dominates the nugget assignment. Centres and scales are always reported on
the original scale.}

\item{seed}{Optional integer seed for reproducible partitioning.}

\item{...}{Additional arguments forwarded to
\code{datanugget::create.DN} when \code{engine = "datanugget"}.}
}
\value{
An object of class \code{"data_nugget"}: a list with elements
\item{centers}{\eqn{M \times p} matrix of nugget centres.}
\item{weights}{Length-\eqn{M} integer vector of nugget sizes \eqn{n_i}.}
\item{scales}{\eqn{M \times p} matrix of within-nugget variances.}
\item{membership}{Length-\eqn{n} integer vector mapping each observation to
its nugget.}
\item{x_scale}{Length-\eqn{M} vector of total within-nugget variability
\eqn{\mathrm{tr}(\mathrm{Cov})}.}
\item{engine}{The engine used.}
}
\description{
\code{create_data_nuggets} compresses \code{n} observations into
\code{M <= n} representative "nuggets". Each nugget stores a centre
(location), a weight (the number of member observations) and a scale
(within-nugget variability). Data nuggets preserve the covariance structure
of the data - including the periphery - far better than a random sub-sample,
which makes them a robust, memory-light surrogate for the full data set when
clustering or computing feature weights.
}
\details{
With the native engine the number of nuggets defaults to
\code{min(nrow(x), max(2, floor(nrow(x) / 10)))} when \code{n > 50}, and to
\code{n} otherwise (in which case every observation is its own nugget and the
nugget-weighted variance reduces to the ordinary variance - a safe fallback
for small samples). The partition is obtained with \code{stats::kmeans};
missing values are mean-imputed for the partition step only.
}
\examples{
set.seed(1)
x <- rbind(matrix(rnorm(200 * 5,  0), ncol = 5),
           matrix(rnorm(200 * 5,  4), ncol = 5))
dn <- create_data_nuggets(x, max_nuggets = 40)
dn
}
\references{
\insertRef{Cherasia2023}{MosaiClusteR}

\insertRef{WCluster2025}{MosaiClusteR}
}
\seealso{
\code{\link{nugget_feature_weights}}, \code{\link{M_ABCpp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-heatmap.r
\name{HeatmapPlot}
\alias{HeatmapPlot}
\title{A heatmap of the comparison of two clustering results.}
\usage{
HeatmapPlot(
  Data1,
  Data2,
  names = NULL,
  nrclusters = NULL,
  cols = NULL,
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{Data1}{The resulting clustering of method 1.}

\item{Data2}{The resulting clustering of method 2.}

\item{names}{The names of the objects in the data sets. Default is NULL.}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{cols}{A character vector with the colours for the clusters. Default is NULL.}

\item{plottype}{Should be one of "pdf","new" or "sweave". If "pdf", a
location should be provided in "location" and the figure is saved there. If
"new" a new graphic device is opened and if "sweave", the figure is made
compatible to appear in a sweave or knitr document, i.e. no new device is
opened and the plot appears in the current device or document. Default is "new".}

\item{location}{If plottype is "pdf", a location should be provided in
"location" and the figure is saved there. Default is NULL.}
}
\value{
A heatmap based on the distance matrix created by the function with
the dendrogram of method 2 on top of the plot and the one from method 1 on
the left. The names of the objects are depicted on the bottom in the order
of clustering of method 2 and on the right by the ordering of method 1.
Vertically the cluster of method 2 can be seen while horizontally those of
method 1 are portrayed.
}
\description{
A function to plot a heatmap of the agreement between two clustering
results.

These are the clusters to which a comparison is made. A matrix is set up of
which the columns are determined by the ordering of clustering of method 2
and the rows by the ordering of method 1. Every column represent one object
just as every row and every column represent the color of its cluster. A
function visits every cell of the matrix. If the objects represented by the
cell are still together in a cluster, the color of the column is passed to
the cell. This creates the distance matrix which can be given to the
HeatmapCols function to create the heatmap.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)
data(Colors1)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
clust="agnes",linkage="flexible",gap=FALSE,maxK=15)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
clust="agnes",linkage="flexible",gap=FALSE,maxK=15)


L=list(MCF7_F,MCF7_T)
names=c("FP","TP")

HeatmapPlot(Data1=MCF7_T,Data2=MCF7_F,names=rownames(fingerprintMat)
,nrclusters=7,cols=Colors1,plottype="new", location=NULL)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-heatmap.r
\name{BoxPlotDistance}
\alias{BoxPlotDistance}
\title{Box plots of one distance matrix categorized against another distance
matrix.}
\usage{
BoxPlotDistance(
  Data1,
  Data2,
  type = c("data", "dist", "clusters"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  lab1,
  lab2,
  limits1 = NULL,
  limits2 = NULL,
  plot = 1,
  StopRange = FALSE,
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{Data1}{The first data matrix, cluster outcome or distance matrix to be
plotted.}

\item{Data2}{The second data matrix, cluster outcome or distance matrix to
be plotted.}

\item{type}{indicates whether the provided matrices in "List" are either data matrices, distance
matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
matrices is skipped and if type="clusters" the single source clustering is skipped.
Type should be one of "data", "dist" or "clusters".}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.}

\item{lab1}{The label to plot for Data1.}

\item{lab2}{The label to plot for Data2.}

\item{limits1}{The limits for the categories of Data1.}

\item{limits2}{The limits for the categories of Data2.}

\item{plot}{The type of plots: 1 - Plot the values of Data1 versus the
categories of Data2. 2 - Plot the values of Data2 versus the categories of
Data1. 3 - Plot both types 1 and 2.}

\item{StopRange}{Logical. Indicates whether the distance matrices with
values not between zero and one should be standardized to have so. If FALSE
the range normalization is performed. See \code{Normalization}. If TRUE, the
distance matrices are not changed. This is recommended if different types of
data are used such that these are comparable. Default is FALSE.}

\item{plottype}{Should be one of "pdf","new" or "sweave". If "pdf", a
location should be provided in "location" and the figure is saved there. If
"new" a new graphic device is opened and if "sweave", the figure is made
compatible to appear in a sweave or knitr document, i.e. no new device is
opened and the plot appears in the current device or document. Default is "new".}

\item{location}{If plottype is "pdf", a location should be provided in
"location" and the figure is saved there. Default is NULL.}
}
\value{
One/multiple box plots.
}
\description{
Given two distance matrices, the function categorizes one distance matrix
and produces a box plot from the other distance matrix against the created
categories. The option is available to choose one of the plots or to have
both plots. The function also works on outputs from ADEC and CEC functions
which do not have distance matrices but incidence matrices.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)


BoxPlotDistance(MCF7_F,MCF7_T,type="cluster",lab1="FP",lab2="TP",limits1=c(0.3,0.7),
limits2=c(0.3,0.7),plot=1,StopRange=FALSE,plottype="new", location=NULL)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manuscript-viz.R
\name{clustering_concordance}
\alias{clustering_concordance}
\title{Compare clustering solutions to each other (no ground truth needed)}
\usage{
clustering_concordance(labels_list)
}
\arguments{
\item{labels_list}{A named list of label vectors, all of the same length.}
}
\value{
A list with \code{agreement} (pairwise ARI matrix), \code{stability}
(mean pairwise co-membership per object) and \code{changed} (objects whose
label is not constant across solutions).
}
\description{
Given several labellings of the same objects (e.g. different weightings or
methods), compute their pairwise agreement (ARI) and each object's stability
-- how consistently it is co-clustered across the solutions. Useful for real
data where no ground truth is available.
}
\examples{
data(mosaic_toy)
v <- mosaic_labels(M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3,
                           weighting = c("var", "var")), 3)
n <- mosaic_labels(M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3,
                           weighting = c("nugget", "nugget")), 3)
clustering_concordance(list(var = v, nugget = n))$agreement
}
\seealso{
\code{\link{cluster_agreement}}, \code{\link{compare_clusterings}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{Geneset.intersectSelection}
\alias{Geneset.intersectSelection}
\title{Intersection over resulting gene sets of \code{PathwaysIter} function for a selection of objects}
\usage{
Geneset.intersectSelection(
  list.output,
  sign = 0.05,
  names = NULL,
  seperatetables = FALSE,
  separatepvals = FALSE
)
}
\arguments{
\item{list.output}{The output of the \code{PathwaysIter} function.}

\item{sign}{The significance level to be handled for cutting of the
pathways. Default is 0.05.}

\item{names}{Optional. Names of the methods. Default is NULL.}

\item{seperatetables}{Logical. If TRUE, a separate element is created per
cluster containing the pathways for each iteration. Default is FALSE.}

\item{separatepvals}{Logical. If TRUE, the p-values of the each iteration of
each pathway in the intersection is given. If FALSE, only the mean p-value
is provided. Default is FALSE.}
}
\description{
Internal function of \code{Geneset.intersect}.
}
\examples{
\dontrun{
Geneset.intersectSelection(MCF7_Paths_FandT,sign=0.05)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-compare.r
\name{CompareSilCluster}
\alias{CompareSilCluster}
\title{Compares medoid clustering results based on silhouette widths}
\usage{
CompareSilCluster(
  List,
  type = c("data", "dist"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  nrclusters = NULL,
  names = NULL,
  nboot = 100,
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{List}{A list of data matrices. It is assumed the rows are corresponding with the objects.}

\item{type}{indicates whether the provided matrices in "List" are either data matrices or distance
matrices. Type should be one of "data" or "dist".}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices. Default is c(FALSE, FALSE).}

\item{method}{A method of normalization. Default is c(NULL,NULL).}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{names}{The labels to give to the elements in List. Default is NULL.}

\item{nboot}{Number of bootstraps to be run. Default is 100.}

\item{plottype}{Should be one of "pdf","new" or "sweave". Default is "new".}

\item{location}{If plottype is "pdf", a location should be provided here. Default is NULL.}
}
\value{
A plot of the density of the statistic under the null hypothesis. Further a list with two
elements: \item{Observed Statistic}{The observed statistical value} \item{P-Value}{The P-value of the
obtained statistic retrieved after bootstrapping}.
}
\description{
The function \code{CompareSilCluster} compares the results of two medoid
clusterings. The null hypothesis is that the clustering is identical. A test
statistic is calculated and a p-value obtained with bootstrapping.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)

List=list(fingerprintMat,targetMat)

Comparison=CompareSilCluster(List=List,type="data",
distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),method=c(NULL,NULL),
nrclusters=7,names=NULL,nboot=100,plottype="new",location=NULL)

Comparison
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-navigation.r
\name{SharedComps}
\alias{SharedComps}
\title{Objects shared across clusterings}
\usage{
SharedComps(
  List,
  nrclusters = NULL,
  fusionsLog = FALSE,
  weightclust = FALSE,
  names = NULL
)
}
\arguments{
\item{List}{A list of the clustering outputs to be compared. The first
element of the list will be used as the reference in
\code{ReorderToReference}.}

\item{nrclusters}{If List is the output of several clustering methods, it has
to be provided in how many clusters to cut the dendrograms in. Default is NULL.}

\item{fusionsLog}{Logical. To be handed to \code{ReorderToReference}:
indicator for the fusion of clusters. Default is FALSE.}

\item{weightclust}{Logical. To be handed to \code{ReorderToReference}: to be
used for the outputs of CEC, WeightedClust or WeightedSimClust. If TRUE, only
the result of the Clust element is considered. Default is FALSE.}

\item{names}{Names of the methods or clusters. Default is NULL.}
}
\value{
A list containing the shared objects of all listed elements per cluster.
}
\description{
\code{SharedComps} determines, per cluster, which objects are
shared across all listed clustering results. The clusterings are first
rearranged to a common reference with \code{ReorderToReference} and the
intersection of the lead objects of every method is taken per cluster.
}
\examples{
\dontrun{
data(mosaic_toy)
L <- mosaic_toy$List
res <- list(Cluster(L[[1]], type = "data", distmeasure = "euclidean"),
            Cluster(L[[2]], type = "data", distmeasure = "euclidean"))
Comps <- SharedComps(List = res, nrclusters = 5, fusionsLog = FALSE,
                     weightclust = FALSE, names = c("S1", "S2"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{SharedSelectionMLP}
\alias{SharedSelectionMLP}
\title{Intersection of pathways over multiple methods for a selection of objects.}
\usage{
SharedSelectionMLP(DataMLP = NULL, names = NULL)
}
\arguments{
\item{DataMLP}{Optional. The output of \code{Geneset.intersect} function. Default is NULL.}

\item{names}{Optional. Names of the methods or "Selection" if it only
considers a selection of objects. Default is NULL.}
}
\value{
A list with a Table and a Which element for the shared pathways.
}
\description{
Internal function of \code{SharedGenesPathsFeat}.
}
\examples{
\dontrun{
SharedSelectionMLP(DataMLP=MCF7_Paths_intersection)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manuscript-viz.R
\name{cluster_k_sweep}
\alias{cluster_k_sweep}
\title{Show the clustering at every k (see how it deteriorates)}
\usage{
cluster_k_sweep(
  x,
  k_range = 2:6,
  linkage = "ward.D2",
  embed = c("pca", "umap", "tsne"),
  ncol = 3
)
}
\arguments{
\item{x}{A fitted result (\code{$DistM}), a dissimilarity, or a \code{List}.}

\item{k_range}{Candidate cluster counts to display.}

\item{linkage}{Agglomeration method used to cut the fused dissimilarity.}

\item{embed}{Embedding for the (fixed) 2-D coordinates: \code{"pca"} (default),
\code{"umap"} or \code{"tsne"}.}

\item{ncol}{Columns in the panel grid.}
}
\value{
Invisibly a named numeric vector of the average silhouette per \eqn{k}.
}
\description{
Rather than only reporting the best \eqn{k}, project the objects to a
\emph{fixed} 2-D embedding and recolour them by the \eqn{k}-cut for each
candidate \eqn{k} in a grid, annotating the average silhouette per panel. This
makes it visible how the structure splits and degrades as \eqn{k} grows.
}
\examples{
data(mosaic_toy)
fit <- M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3)
cluster_k_sweep(fit, k_range = 2:6)
}
\seealso{
\code{\link{cluster_selection_plot}}, \code{\link{embedding_plot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregation.r
\name{ClusteringAggregation}
\alias{ClusteringAggregation}
\title{Clustering aggregation}
\usage{
ClusteringAggregation(
  List,
  type = c("data", "dist", "clust"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  clust = "agnes",
  linkage = c("flexible", "flexible"),
  alpha = 0.625,
  nrclusters = c(7, 7),
  gap = FALSE,
  maxK = 15,
  agglMethod = c("Balls", "Aggl", "Furthest", "LocalSearch"),
  improve = TRUE,
  distThresh_B = 0.5,
  distThresh_A = 0.8
)
}
\arguments{
\item{List}{A list of data matrices. It is assumed the rows are corresponding with the objects.}

\item{type}{indicates whether the provided matrices in "List" are either data matrices, distance
matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
matrices is skipped and if type="clust" the single source clustering is skipped.
Type should be one of "data", "dist" or "clust".}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".}

\item{nrclusters}{The number of clusters to divide each individual dendrogram in. Default is c(7,7) for two data sets.}

\item{gap}{Logical. Whether the optimal number of clusters should be determined with the gap statistic. Default is FALSE.}

\item{maxK}{The maximal number of clusters to investigate in the gap statistic. Default is 15.}

\item{agglMethod}{The method to be performed: "Balls","Aggl","Furthest" or "LocalSearch".}

\item{improve}{Logical. If TRUE, a local search is performed to improve the obtained results. Default is TRUE.}

\item{distThresh_B}{A distance threshold for the Balls algoritme. Default is 0.5.}

\item{distThresh_A}{A distance threshold for the Aggl. algoritme. Default is 0.8.}
}
\value{
The returned value is a list of two elements:
\item{DistM}{A NULL object}
\item{Clust}{The resulting clustering}
The value has class 'Ensemble'.
}
\description{
The \code{ClusteringAggregation} includes the ensemble clustering methods Balls, Agglomerative (Aggl.) and Furthest which are graph-based consensus methods.
}
\details{
Gionis, Mannila and Tsaparas (2007) propose heuristic algorithms in order to find a solution for the consensus problem. In a first step, a
weighted graph is built from the objects with weights between two vertices determined by the fraction of clusterings that place the two vertices
in different clusters. In a second step, an algorithm searches for the partition that minimizes the total number of disagreements with the given
partitions. The Balls algorithm is an iterative process which finds a cluster for the consensus partition in each iteration. For each object,
all objects at a distance of at most 0.5 are collected and the average distance of this set to the object of interest is calculated. If the
average distance is less or equal to a parameter the objects form a cluster; otherwise the object forms a singleton. The Agglomerative
(Aggl.) algorithm starts by considering every object as a singleton cluster. Next, the two closest clusters are merged if the average distance
between the clusters is less than 0.5. If there are no two clusters with an average distance smaller than 0.5, the algorithm stops and returns
the created clusters as a solution. The Furthest algorithm starts by placing all objects into a single cluster. In each iteration, the pair of
objects that are the furthest apart are considered as new cluster centers. The remaining objects are appointed to the center that increases the
cost of the partition the least and the new cost is computed. The cost is the sum of the all distances between the obtained partition and the
partitions in the ensemble. The iteration continues until the cost of the new partition is higher than the previous partition.
}
\examples{
\dontrun{
data(mosaic_toy)
L <- mosaic_toy$List
Aggl_toy <- ClusteringAggregation(List = L, type = "data",
  distmeasure = c("euclidean", "euclidean"), normalize = c(FALSE, FALSE),
  method = c(NULL, NULL), clust = "agnes", linkage = c("ward", "ward"),
  alpha = 0.625, nrclusters = c(7, 7), agglMethod = "Aggl",
  improve = TRUE, distThresh_B = 0.5, distThresh_A = 0.8)
}
}
\references{
Gionis, A., Mannila, H. and Tsaparas, P. (2007). Clustering
aggregation. ACM Transactions on Knowledge Discovery from Data, 1(1), 4-es.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ABCdeep.R
\name{ABCdeep.SingleInMultiple}
\alias{ABCdeep.SingleInMultiple}
\title{Single-source ABC with a deep-learning (autoencoder) Step 4 (ABCdeep)}
\usage{
ABCdeep.SingleInMultiple(
  data,
  weighting = "var",
  normalize = NULL,
  gr = c(),
  bag = TRUE,
  numsim = 1000,
  numvar = 100,
  NC = NULL,
  topk_frac = 0.2,
  latent_dim = "auto",
  hidden_units = 32L,
  ae_epochs = 80L,
  ae_lr = 0.001,
  cluster_in_latent = "ward",
  linkage = "ward.D2",
  nugget_type = "between",
  nugget_args = list(),
  min_samples = 40L,
  seed = NULL
)
}
\arguments{
\item{data}{Numeric matrix, \strong{objects (samples) in rows}, features in
columns (the MosaiClusteR convention).}

\item{weighting}{Feature weighting used to rank features for the top-K focus:
\code{"var"}, \code{"cv"}, \code{"nugget"} or equal (as in
\code{\link{ABCpp.SingleInMultiple}}).}

\item{normalize}{Optional normalisation applied before the internal min-max
scaling (see \code{\link{Normalization}}); \code{NULL}/\code{FALSE} = none.}

\item{gr}{Unused; kept for interface compatibility.}

\item{bag}{Logical; bootstrap the objects each iteration (Step 2).}

\item{numsim}{Number of iterations \eqn{R}.}

\item{numvar}{Kept for backward compatibility; unused.}

\item{NC}{Base-cluster count (\code{NULL} = \eqn{\lfloor\sqrt{N}\rfloor}).}

\item{topk_frac}{Fraction of the highest-weighted features the autoencoder is
trained/embedded on (default 0.2). Focusing on the informative features
drops the noise that would dilute a full-feature embedding on sparse signals,
while keeping enough features for dense signals; the count is floored at
\code{min(G, 20)} and capped at \code{min(G, 4000)}.}

\item{latent_dim}{Autoencoder bottleneck size: an integer, or \code{"auto"}
(default) = \code{max(NC + 2, 8)}, capped at the hidden width and \eqn{K-1}.}

\item{hidden_units}{Hidden-layer width of the encoder/decoder.}

\item{ae_epochs}{Training epochs for the one-time autoencoder fit.}

\item{ae_lr}{Adam learning rate.}

\item{cluster_in_latent}{Latent clustering (Step 4b): \code{"ward"} (default,
hierarchical) or \code{"kmeans"}.}

\item{linkage}{Agglomeration method when \code{cluster_in_latent = "ward"}.}

\item{nugget_type, nugget_args}{Passed to the nugget weighting when
\code{weighting = "nugget"}.}

\item{min_samples}{Emit a warning when \code{nrow(data)} is below this (the
autoencoder is underdetermined at very small \eqn{N}); default 40.}

\item{seed}{Optional RNG seed (controls the autoencoder init and the
bootstrap).}
}
\value{
A data frame, \code{numsim} rows by one column per object; entries are
base-cluster labels (\code{0} = object not selected that iteration) -- the
same contract as \code{\link{ABCpp.SingleInMultiple}}.
}
\description{
A deep-learning variant of \code{\link{ABCpp.SingleInMultiple}} that keeps the
ABC algorithm of \insertCite{Amaratunga2008}{MosaiClusteR} intact and replaces
\strong{only Step 4}: instead of clustering the raw feature sub-matrix, a
neural autoencoder embeds the objects into a latent space and the latent
vectors are clustered into \code{NC} base clusters. The stacked label matrix
is fused across sources by \code{\link{f.clustABC.MultiSource}}, exactly as in
\code{\link{M_ABCpp}}.
}
\section{Speed (trained once, forward pass per iteration)}{

The autoencoder is trained a single time on the source's top-\code{topk_frac}
most-informative features; each of the \code{numsim} iterations then performs
only a forward pass over the bootstrapped objects. The network is a
self-contained, vectorised R autoencoder (He initialisation, ReLU, Adam, MSE)
-- there is no Python, TensorFlow or torch dependency, and \strong{no PCA or
other fallback}: a single code path.
}

\examples{
data(mosaic_toy)
lab <- ABCdeep.SingleInMultiple(mosaic_toy$List[[1]], numsim = 40, NC = 3,
                                ae_epochs = 40, seed = 1)
dim(lab)
}
\references{
\insertAllCited{}
}
\seealso{
\code{\link{M_ABCdeep}}, \code{\link{ABCpp.SingleInMultiple}},
\code{\link{f.clustABC.MultiSource}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-navigation.r
\name{FindCluster}
\alias{FindCluster}
\title{Find a selection of objects in the output of \code{ReorderToReference}}
\usage{
FindCluster(
  List,
  nrclusters = NULL,
  select = c(1, 1),
  fusionsLog = TRUE,
  weightclust = TRUE,
  names = NULL
)
}
\arguments{
\item{List}{A list of the clustering outputs to be compared. The first
element of the list will be used as the reference in
\code{ReorderToReference}.}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{select}{The row (the method) and the number of the cluster to select. Default is c(1,1).}

\item{fusionsLog}{Logical. To be handed to \code{ReorderToReference}:
indicator for the fusion of clusters. Default is TRUE.}

\item{weightclust}{Logical. To be handed to \code{ReorderToReference}: to be
used for the outputs of CEC, WeightedClust or WeightedSimClust. If TRUE, only
the result of the Clust element is considered. Default is TRUE.}

\item{names}{Optional. Names of the methods. Default is NULL.}
}
\value{
A character vector containing the names of the objects in the
selected cluster.
}
\description{
\code{FindCluster} selects the objects belonging to a cluster
after the results of the methods have been rearranged by
\code{ReorderToReference}.
}
\examples{
\dontrun{
data(mosaic_toy)
L <- mosaic_toy$List
res <- list(Cluster(L[[1]], type = "data", distmeasure = "euclidean"),
            Cluster(L[[2]], type = "data", distmeasure = "euclidean"))
Comps <- FindCluster(List = res, nrclusters = 5, select = c(1, 2),
                     weightclust = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hec.r
\name{HierarchicalEnsembleClustering}
\alias{HierarchicalEnsembleClustering}
\title{Hierarchical ensemble clustering}
\usage{
HierarchicalEnsembleClustering(
  List,
  type = c("data", "dist", "clust"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  clust = "agnes",
  linkage = c("flexible", "flexible"),
  alpha = 0.625
)
}
\arguments{
\item{List}{A list of data matrices. It is assumed the rows are corresponding with the objects.}

\item{type}{indicates whether the provided matrices in "List" are either data matrices, distance
matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
matrices is skipped and if type="clusters" the single source clustering is skipped.
Type should be one of "data", "dist" or"clusters".}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, default is FALSE. This is recommended if different distance types are used. More details on normalization in \code{Normalization}}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".}
}
\value{
The returned value is a list of two elements:
\item{DistM}{The resulting distance matrix}
\item{Clust}{The resulting hierarchical structure}
The value has class 'HEC'.
}
\description{
\insertCite{Zheng2014}{MosaiClusteR} proposed the Hierarchical Ensemble Clustering (HEC) algorithm. For each dendrogram, the cophenetic
distances between the object are calculated. The distances are aggregated across the data sets and an ultra-metric which is the closest to the
distance matrix is determined. A final hierarchical clustering is based on the ultra-metric values.
}
\references{
\insertRef{Zheng2014}{MosaiClusteR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/M_ABCdeep.R
\name{M_ABCdeep}
\alias{M_ABCdeep}
\title{Multi-source M-ABC with a deep-learning (autoencoder) Step 4 (M-ABCdeep)}
\usage{
M_ABCdeep(
  List,
  weighting = rep("var", length(List)),
  normalize = rep(FALSE, length(List)),
  gr = c(),
  bag = rep(TRUE, length(List)),
  numsim = 500,
  numvar = rep(100, length(List)),
  NC = NULL,
  dissimilarity = "mabc",
  final_linkage = "ward",
  alpha = 0.625,
  mds = FALSE,
  topk_frac = 0.2,
  latent_dim = "auto",
  hidden_units = 32L,
  ae_epochs = 80L,
  ae_lr = 0.001,
  cluster_in_latent = "ward",
  linkage = rep("ward.D2", length(List)),
  nugget_type = "between",
  nugget_args = list(),
  min_samples = 40L,
  seed = NULL
)
}
\arguments{
\item{List}{A list of \eqn{K} data matrices over the same objects,
\strong{objects (samples) in rows} and features in columns.}

\item{weighting, normalize, bag, numvar}{Per-source vectors (recycled to \eqn{K})
forwarded to \code{\link{ABCdeep.SingleInMultiple}}.}

\item{gr}{Unused; kept for compatibility.}

\item{numsim}{Iterations per source.}

\item{NC}{Base-cluster count per source (\code{NULL} = data-driven).}

\item{dissimilarity}{Consensus dissimilarity, \code{"mabc"} or \code{"abc"}.}

\item{final_linkage, alpha}{Final clustering controls.}

\item{mds}{Logical; MDS of the consensus dissimilarities.}

\item{topk_frac}{Fraction of top-weighted features the autoencoder focuses on
(default 0.2); forwarded to \code{\link{ABCdeep.SingleInMultiple}}.}

\item{latent_dim, hidden_units, ae_epochs, ae_lr, cluster_in_latent, linkage}{Autoencoder / latent-clustering controls (scalar or length-\eqn{K}),
forwarded to \code{\link{ABCdeep.SingleInMultiple}}.}

\item{nugget_type, nugget_args}{Forwarded to the nugget weighting.}

\item{min_samples}{Warn (per source) when \eqn{N} is below this; the
autoencoder is underdetermined at very small \eqn{N} (default 40).}

\item{seed}{Optional RNG seed.}
}
\value{
A list of class \code{"M_ABC"} with \code{DistM} and \code{Clust}.
}
\description{
The deep-learning counterpart of \code{\link{M_ABCpp}}: runs
\code{\link{ABCdeep.SingleInMultiple}} per source (autoencoder embedding +
latent clustering for the base partitions), row-stacks the per-iteration label
matrices, and builds one consensus dissimilarity via
\code{\link{f.clustABC.MultiSource}}. The ABC algorithm of
\insertCite{Amaratunga2008}{MosaiClusteR} is preserved end to end; only Step 4
is deep. There is no Python/torch dependency and no PCA fallback (see
\code{\link{ABCdeep.SingleInMultiple}}).
}
\examples{
data(mosaic_toy)
fit <- M_ABCdeep(mosaic_toy$List, numsim = 40, NC = 3, ae_epochs = 40, seed = 1)
table(stats::cutree(stats::as.hclust(fit$Clust), 3))
}
\references{
\insertAllCited{}
}
\seealso{
\code{\link{ABCdeep.SingleInMultiple}}, \code{\link{M_ABCpp}},
\code{\link{f.clustABC.MultiSource}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/M_ABCdist.R
\name{M_ABCdist}
\alias{M_ABCdist}
\title{Multi-source M-ABC with distance accumulation}
\usage{
M_ABCdist(
  List,
  source_weights = NULL,
  distmeasure = rep("euclidean", length(List)),
  weighting = rep("var", length(List)),
  normalize = rep(list(NULL), length(List)),
  gr = c(),
  bag = rep(TRUE, length(List)),
  numsim = 1000L,
  numvar = rep(100L, length(List)),
  linkage = rep("ward.D", length(List)),
  NC = NULL,
  NC2 = NULL,
  final_linkage = "ward",
  alpha = 0.625,
  mds = FALSE,
  nfeat = NULL,
  nugget_type = "between",
  nugget_args = list()
)
}
\arguments{
\item{List}{A list of \eqn{K} data matrices over the same objects,
\strong{objects (samples) in rows} and features in columns.}

\item{source_weights}{Optional non-negative source weights (normalised to sum
to one); \code{NULL} = equal weights.}

\item{distmeasure, weighting, normalize, bag, numvar, linkage}{Per-source vectors
(recycled to \eqn{K}) forwarded to \code{\link{ABCpp.SingleInMultiple}}.}

\item{gr}{Unused; kept for compatibility.}

\item{numsim}{Iterations per source.}

\item{NC}{Base-cluster count per source (\code{NULL} = data-driven).}

\item{NC2}{Optional number of clusters to cut the final tree into.}

\item{final_linkage, alpha}{Final clustering controls.}

\item{mds}{Logical; MDS of the consensus dissimilarities.}

\item{nfeat}{Feature-subsample size forwarded to
\code{\link{ABCpp.SingleInMultiple}} (\code{NULL} = \eqn{\lfloor\sqrt{G}\rfloor}).}

\item{nugget_type, nugget_args}{Forwarded to the nugget weighting.}
}
\value{
A list of class \code{"M_ABCdist"} with \code{source_D}, \code{DistM},
\code{weights} and \code{Clust}.
}
\description{
Runs \code{\link{ABCdist.SingleInMultiple}} on each source to obtain a
\code{[0, 1]} dissimilarity, combines them as a weighted mean
\eqn{D = \sum_k w_k D_k} and applies a final agglomerative clustering.
}
\examples{
data(mosaic_toy)
fit <- M_ABCdist(mosaic_toy$List, numsim = 40, weighting = c("var", "nugget"))
dim(fit$DistM)
}
\references{
\insertCite{Amaratunga2008}{MosaiClusteR}
}
\seealso{
\code{\link{M_ABCpp}}, \code{\link{ABCdist.SingleInMultiple}},
\code{\link{M_ABCdist.WC}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-compare.r
\name{CompareInteractive}
\alias{CompareInteractive}
\title{Interactive comparison of single and multiple source clustering results}
\usage{
CompareInteractive(
  ListM,
  ListS,
  nrclusters = NULL,
  cols = NULL,
  fusionsLogM = FALSE,
  fusionsLogS = FALSE,
  weightclustM = FALSE,
  weightclustS = FALSE,
  namesM = NULL,
  namesS = NULL,
  marginsM = c(2, 2.5, 2, 2.5),
  marginsS = c(8, 2.5, 2, 2.5),
  Interactive = TRUE,
  n = 1
)
}
\arguments{
\item{ListM}{A list of the outputs from the multiple source clusterings to be compared.}

\item{ListS}{A list of the outputs from the single source clusterings to be compared.}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{cols}{A character vector with the names of the colours. Default is NULL.}

\item{fusionsLogM}{The fusionsLog parameter for the elements in ListM. Default is FALSE.}

\item{fusionsLogS}{The fusionsLog parameter for the elements in ListS. Default is FALSE.}

\item{weightclustM}{The weightclust parameter for the elements in ListM. Default is FALSE.}

\item{weightclustS}{The weightclust parameter for the elements in ListS. Default is FALSE.}

\item{namesM}{Optional. Names of the multiple source clusterings. Default is NULL.}

\item{namesS}{Optional. Names of the single source clusterings. Default is NULL.}

\item{marginsM}{Optional. Margins to be used for the ListM plot. Default is c(2,2.5,2,2.5).}

\item{marginsS}{Optional. Margins to be used for the ListS plot. Default is c(8,2.5,2,2.5).}

\item{Interactive}{Optional. Do you want an interactive plot? Default is TRUE.}

\item{n}{The number of methods/clusters you want to identify. Default is 1.}
}
\value{
A plot of the comparison of the elements of ListM, on which multiple clusters
and/or methods can be identified to compare them to the elements in ListS.
}
\description{
The function \code{CompareInteractive} produces an interactive plot of the
multiple source clustering results in \code{ListM}. By identifying a cluster
or a method, a comparison with the single source clustering results in
\code{ListS} is shown in a new plotting window.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)
data(Colors1)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(fingerprintMat,targetMat)

MCF7_W=WeightedClust(List=L,type="data",distmeasure=c("tanimoto","tanimoto"),
normalize=c(FALSE,FALSE),method=c(NULL,NULL),weight=seq(1,0,-0.1),weightclust=0.5,
clust="agnes",linkage="ward",StopRange=FALSE)

ListM=list(MCF7_W)
namesM=c(seq(1.0,0.0,-0.1))

ListS=list(MCF7_F,MCF7_T)
namesS=c("FP","TP")

CompareInteractive(ListM,ListS,nrclusters=7,cols=Colors1,fusionsLogM=FALSE,
fusionsLogS=FALSE,weightclustM=FALSE,weightclustS=TRUE,namesM,namesS,
marginsM=c(2,2.5,2,2.5),marginsS=c(8,2.5,2,2.5),Interactive=TRUE,n=1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manuscript-viz.R
\name{embedding_plot}
\alias{embedding_plot}
\title{Two-dimensional cluster visualisation (PCA / UMAP / t-SNE)}
\usage{
embedding_plot(
  x,
  labels,
  method = c("pca", "umap", "tsne"),
  col = NULL,
  main = NULL
)
}
\arguments{
\item{x}{A fitted result (\code{$DistM}), a dissimilarity, or a \code{List}.}

\item{labels}{Cluster label of each object.}

\item{method}{\code{"pca"} (default), \code{"umap"} or \code{"tsne"}.}

\item{col}{Optional cluster colours.}

\item{main}{Plot title.}
}
\value{
Invisibly the 2-column coordinate matrix.
}
\description{
Project the objects to two dimensions and colour them by cluster. \code{"pca"}
uses classical MDS of the fused dissimilarity (always available); \code{"umap"}
needs \pkg{uwot} and \code{"tsne"} needs \pkg{Rtsne}.
}
\examples{
data(mosaic_toy)
fit <- M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3)
embedding_plot(fit, mosaic_labels(fit, 3), method = "pca")
}
\seealso{
\code{\link{cluster_selection_plot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-heatmap.r
\name{SimilarityHeatmap}
\alias{SimilarityHeatmap}
\title{A heatmap of similarity values between objects}
\usage{
SimilarityHeatmap(
  Data,
  type = c("data", "clust", "sim", "dist"),
  distmeasure = "tanimoto",
  normalize = FALSE,
  method = NULL,
  linkage = "flexible",
  cutoff = NULL,
  percentile = FALSE,
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{Data}{The data of which a heatmap should be drawn.}

\item{type}{indicates whether the provided matrices in "List" are either data matrices, distance
matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
matrices is skipped and if type="clusters" the single source clustering is skipped.
Type should be one of "data", "dist" ,"sim" or "clusters".}

\item{distmeasure}{The distance measure. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to "tanimoto".}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is NULL.}

\item{linkage}{Choice of inter group dissimilarity (character). Defaults to "flexible".}

\item{cutoff}{Optional. If a cutoff value is specified, all values lower are
put to zero while all other values are kept. This helps to highlight the
most similar objects. Default is NULL.}

\item{percentile}{Logical. The cutoff value can be a percentile. If one want
the cutoff value to be the 90th percentile of the data, one should specify
cutoff = 0.90 and percentile = TRUE. Default is FALSE.}

\item{plottype}{Should be one of "pdf","new" or "sweave". If "pdf", a
location should be provided in "location" and the figure is saved there. If
"new" a new graphic device is opened and if "sweave", the figure is made
compatible to appear in a sweave or knitr document, i.e. no new device is
opened and the plot appears in the current device or document. Default is "new".}

\item{location}{If plottype is "pdf", a location should be provided in
"location" and the figure is saved there. Default is NULL.}
}
\value{
A heatmap with the names of the objects on the right and bottom
and a dendrogram of the clustering at the left and top.
}
\description{
The function \code{SimilarityHeatmap} plots the similarity values between
objects. The darker the shade, the more similar objects are. The option
is available to set a cutoff value to highlight the most similar objects.
}
\details{
If data is of type "clust", the distance matrix is extracted from the result
and transformed to a similarity matrix. Possibly a range normalization is
performed. If data is of type "dist", it is also transformed to a similarity
matrix and cluster is performed on the distances. If data is of type "sim",
the data is tranformed to a distance matrix on which clustering is
performed. Once the similarity mattrix is obtained, the cutoff value is
applied and a heatmap is drawn. If no cutoff value is desired, one can leave
the default NULL specification.
}
\examples{

\dontrun{
data(fingerprintMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55)

SimilarityHeatmap(Data=MCF7_F,type="clust",cutoff=0.90,percentile=TRUE)
SimilarityHeatmap(Data=MCF7_F,type="clust",cutoff=0.75,percentile=FALSE)

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-selection.r
\name{SelectnrClusters}
\alias{SelectnrClusters}
\title{Select the number of clusters via silhouette widths}
\usage{
SelectnrClusters(
  List,
  type = c("data", "dist", "pam"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  nrclusters = seq(5, 25, 1),
  names = NULL,
  StopRange = FALSE,
  plottype = "none",
  location = NULL
)
}
\arguments{
\item{List}{A list of data matrices (objects in rows), distance matrices, or
\code{cluster::pam} outputs, depending on \code{type}.}

\item{type}{One of \code{"data"} (compute a distance first via
\code{\link{Distance}}), \code{"dist"} (elements are dissimilarity
matrices) or \code{"pam"} (elements are \code{pam} objects).}

\item{distmeasure}{Distance measure(s) used when \code{type = "data"}.}

\item{normalize, method}{Normalisation controls passed to \code{\link{Distance}}.}

\item{nrclusters}{Sequence of cluster numbers to evaluate.}

\item{names}{Optional names for the sources; used in output labels.}

\item{StopRange}{Logical; if \code{FALSE} and dissimilarities fall outside
\code{[0, 1]} they are range-normalised for comparability.}

\item{plottype}{One of \code{"none"} (default; no plot, safe for headless
runs), \code{"new"}, \code{"pdf"} or \code{"sweave"}. Any value other than
\code{"none"} attempts to draw the silhouette curve; drawing is guarded so
that a missing graphics device does not abort the computation.}

\item{location}{File location (without extension) used when
\code{plottype = "pdf"}. Default \code{NULL}.}
}
\value{
A list with two elements: \code{Silhoutte_Widths} (a matrix of
average silhouette widths per source and their mean) and
\code{Optimal_Nr_of_CLusters} (a one-row data frame with the silhouette
optimal number of clusters per source and overall).
}
\description{
For each element of \code{List}, partitions around medoids
(\code{cluster::pam}) are computed over a range of cluster numbers and the
average silhouette width is recorded. The number of clusters maximising the
average silhouette width is reported per source and for the across-source
average. The analytical result (the silhouette table and the chosen number
of clusters) is returned and is testable without producing any plot.
}
\examples{
\dontrun{
data(mosaic_toy)
sel <- SelectnrClusters(mosaic_toy$List, type = "data",
                        distmeasure = c("euclidean", "euclidean"),
                        nrclusters = seq(2, 5), plottype = "none")
sel$Optimal_Nr_of_CLusters
}
}
\references{
Kaufman, L. and Rousseeuw, P. J. (1990). Finding Groups in Data: An
Introduction to Cluster Analysis. Wiley, New York.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/snf.r
\name{SNF}
\alias{SNF}
\title{Similarity network fusion}
\usage{
SNF(
  List,
  type = c("data", "dist", "clusters"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  StopRange = FALSE,
  NN = 20,
  mu = 0.5,
  T = 20,
  clust = "agnes",
  linkage = "ward",
  alpha = 0.625
)
}
\arguments{
\item{List}{A list of data matrices of the same type. It is assumed the rows are corresponding with the objects.}

\item{type}{indicates whether the provided matrices in "List" are either data matrices, distance
matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
matrices is skipped and if type="clusters" the single source clustering is skipped.
Type should be one of "data", "dist" or "clusters".}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.}

\item{StopRange}{Logical. Indicates whether the distance matrices with values not between zero and one should be standardized to have so.
If FALSE the range normalization is performed. See \code{Normalization}. If TRUE, the distance matrices are not changed.
This is recommended if different types of data are used such that these are comparable. Default is FALSE.}

\item{NN}{The number of neighbours to be used in the procedure. Defaults to 20.}

\item{mu}{The parameter epsilon. The value is recommended to be between 0.3 and 0.8. Defaults to 0.5.}

\item{T}{The number of iterations.}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character) for the final clustering. Defaults to "ward".}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible"}
}
\value{
The returned value is a list with the following three elements.
\item{FusedM }{The fused similarity matrix}
\item{DistM }{The distance matrix computed by subtracting FusedM from one}
\item{Clust}{The resulting clustering}
The value has class 'SNF'.
}
\description{
Similarity Network Fusion (SNF) is a similarity-based multi-source clustering technique. SNF consists of two steps. In the initial step a similarity network is set up for each data matrix. The network is the visualization of the similarity matrix as a weighted graph with the objects as vertices and the pairwise similarities as weights on the edges. In the network-fusion step, each network is iteratively updated with information of the other network which results in more alike networks every time. This eventually converges to a single network.
}
\details{
If r is specified and nrclusters is a fixed number, only a random sampling of the features will be performed for the t iterations (ADECa). If r is NULL and the nrclusters is a sequence, the clustering is performedon all features and the dendrogam is divided into clusters for the values of nrclusters (ADECb). If both r is specified and nrclusters is a sequence, the combination is performed (ADECc).
After every iteration, either be random sampling, multiple divisions of the dendrogram or both, an incidence matrix is set up. All incidence matrices are summed and represent the distance matrix on which a final clustering is performed.
}
\references{
\insertRef{Wang2014a}{MosaiClusteR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/M_ABCdist_WC.R
\name{M_ABCdist.WC}
\alias{M_ABCdist.WC}
\title{Multi-source M-ABC fused with WeightedClust}
\usage{
M_ABCdist.WC(
  List,
  distmeasure = rep("euclidean", length(List)),
  weighting = rep("var", length(List)),
  normalize = rep(list(NULL), length(List)),
  gr = c(),
  bag = rep(TRUE, length(List)),
  numsim = 1000L,
  numvar = rep(100L, length(List)),
  linkage = rep("ward.D", length(List)),
  NC = NULL,
  NC2 = NULL,
  StopRange = FALSE,
  wc_weight = seq(1, 0, -0.1),
  wc_weightclust = 0.5,
  wc_linkage = "ward",
  wc_alpha = 0.625,
  mds = FALSE,
  nugget_type = "between",
  nugget_args = list()
)
}
\arguments{
\item{List}{A list of \eqn{K} data matrices over the same objects,
\strong{objects (samples) in rows} and features in columns.}

\item{distmeasure, weighting, normalize, bag, numvar, linkage}{Per-source vectors
(recycled to \eqn{K}) forwarded to \code{\link{ABCpp.SingleInMultiple}}.}

\item{gr}{Unused; kept for compatibility.}

\item{numsim}{Iterations per source.}

\item{NC}{Base-cluster count per source (\code{NULL} = data-driven).}

\item{NC2}{Optional number of clusters to cut the final tree into.}

\item{StopRange}{Passed to \code{\link{WeightedClust}}.}

\item{wc_weight}{Weight grid for \code{\link{WeightedClust}}.}

\item{wc_weightclust}{Focus weight whose result is returned in \code{Clust}.}

\item{wc_linkage, wc_alpha}{Final clustering controls for \code{\link{WeightedClust}}.}

\item{mds}{Logical; MDS of the consensus dissimilarities.}

\item{nugget_type, nugget_args}{Forwarded to the nugget weighting.}
}
\value{
A list of class \code{"M_ABCdist_WC"} with \code{source_D}, the full
\code{WC} object, the focus \code{DistM} and \code{Clust}, and the
\code{weight_grid}.
}
\description{
Runs \code{\link{ABCdist.SingleInMultiple}} on each source, then feeds the
per-source dissimilarities to \code{\link{WeightedClust}} (\code{type =
"dist"}) to explore convex combinations over a weight grid and return the
focus-weight clustering.
}
\examples{
data(mosaic_toy)
fit <- M_ABCdist.WC(mosaic_toy$List, numsim = 40,
                    wc_weight = seq(1, 0, -0.25), wc_weightclust = 0.5)
dim(fit$DistM)
}
\references{
\insertCite{Amaratunga2008}{MosaiClusteR}
}
\seealso{
\code{\link{M_ABCdist}}, \code{\link{WeightedClust}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ensemble-graph.r
\name{EnsembleClustering}
\alias{EnsembleClustering}
\title{Ensemble clustering via graph partitioning (CSPA / HGPA / MCLA)}
\usage{
EnsembleClustering(
  List,
  type = c("data", "dist", "clust"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  clust = "agnes",
  linkage = c("flexible", "flexible"),
  alpha = 0.625,
  nrclusters = c(7, 7),
  gap = FALSE,
  maxK = 15,
  ensembleMethod = c("CSPA", "HGPA", "MCLA", "Best"),
  waitingtime = 300,
  file_number = 0,
  executable = FALSE
)
}
\arguments{
\item{List}{A list of data matrices. It is assumed the rows are corresponding with the objects.}

\item{type}{indicates whether the provided matrices in "List" are either data matrices, distance
matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
matrices is skipped and if type="clust" the single source clustering is skipped.
Type should be one of "data", "dist" or "clust".}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".}

\item{nrclusters}{The number of clusters to divide each individual dendrogram in. Default is c(7,7) for two data sets.}

\item{gap}{Logical. Whether the optimal number of clusters should be determined with the gap statistic. Default is FALSE.}

\item{maxK}{The maximal number of clusters to investigate in the gap statistic. Default is 15.}

\item{ensembleMethod}{A character string indicating the consensus function to use. One of "CSPA", "HGPA", "MCLA" or "Best". Defaults to "CSPA".}

\item{waitingtime}{The maximum number of seconds to wait for the external program to write its result before stopping. Defaults to 300.}

\item{file_number}{An identifier appended to the temporary files exchanged with the external program. Defaults to 00.}

\item{executable}{Either FALSE (the default) or the path/flag identifying the external graph-partitioning executable (or MATLAB) to use. When FALSE the function stops, since no consensus can be computed in pure R. Precompiled binaries are not bundled with the package.}
}
\value{
The returned value is a list of two elements:
\item{DistM}{A NULL object}
\item{Clust}{The resulting consensus clustering}
The value has class 'Ensemble'.
}
\description{
Cluster-based ensemble clustering that combines several
single-source partitions into a single consensus partition through the
graph-partitioning algorithms of Strehl and Ghosh: the Cluster-based
Similarity Partitioning Algorithm (CSPA), the HyperGraph Partitioning
Algorithm (HGPA) and the Meta-CLustering Algorithm (MCLA), as well as a
"Best" selection strategy.

\strong{External executable required.} This method does not run in pure R.
It writes the stacked label matrix to disk and delegates the actual graph
partitioning to an \strong{external graph-partitioning executable}
(e.g. a compiled CSPA/HGPA/MCLA binary backed by METIS / \code{gpmetis} /
\code{shmetis}) or to a \strong{MATLAB} session. These binaries are
\strong{not bundled} with the package; the user must supply and configure
them. The path to the executable is selected through the \code{executable}
argument. When \code{executable=FALSE} (the default), or when the configured
executable cannot be located, the function stops with an informative error
rather than attempting to launch an unavailable program.
}
\examples{
\dontrun{
data(mosaic_toy)
L <- mosaic_toy$List

# Requires an external graph-partitioning executable / MATLAB to be
# configured through the 'executable' argument.
res <- EnsembleClustering(List = L, type = "data",
  distmeasure = c("euclidean", "euclidean"), normalize = c(FALSE, FALSE),
  method = c(NULL, NULL), clust = "agnes",
  linkage = c("flexible", "flexible"), nrclusters = c(7, 7),
  gap = FALSE, maxK = 15, ensembleMethod = "CSPA",
  executable = "./EnsembleClusteringC")
}
}
\references{
Strehl A. and Ghosh J. (2002). Cluster ensembles - a knowledge reuse framework for combining multiple partitions. Journal of Machine Learning Research, 3, 583-617.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nugget-clustering.R
\name{Wkmeans}
\alias{Wkmeans}
\title{Weighted k-means (minimises WWCSS)}
\usage{
Wkmeans(x, k, weights = rep(1, nrow(x)), nstart = 10, max_iter = 100)
}
\arguments{
\item{x}{Numeric matrix, observations in rows.}

\item{k}{Number of clusters.}

\item{weights}{Non-negative weight per observation (default all 1).}

\item{nstart}{Number of random restarts (best WWCSS kept).}

\item{max_iter}{Maximum Lloyd iterations per start.}
}
\value{
A list with \code{cluster}, \code{centers} and \code{wwcss}.
}
\description{
k-means in which each observation carries a non-negative weight. Cluster
centres are weighted means and the objective is the Weighted Within-Cluster
Sum of Squares \eqn{\sum_c \sum_{i \in c} w_i \lVert x_i - \mu_c \rVert^2}.
}
\examples{
set.seed(1)
x <- rbind(matrix(rnorm(20 * 3, 0), 20), matrix(rnorm(20 * 3, 5), 20))
Wkmeans(x, k = 2, weights = runif(40, 1, 5))$cluster[1:5]
}
\references{
Dey, T., Duan, Y., Cabrera, J. & Cheng, X. (2025). WCluster:
Clustering and PCA with Weights, and Data Nuggets Clustering.
}
\seealso{
\code{\link{nugget_cluster}}, \code{\link{Whclust}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{Pathways}
\alias{Pathways}
\title{Pathway analysis for multiple clustering results}
\usage{
Pathways(
  List,
  Selection = NULL,
  geneExpr = NULL,
  nrclusters = NULL,
  method = c("limma", "MLP"),
  geneInfo = NULL,
  geneSetSource = "GOBP",
  topP = NULL,
  topG = NULL,
  GENESET = NULL,
  sign = 0.05,
  fusionsLog = TRUE,
  weightclust = TRUE,
  names = NULL
)
}
\arguments{
\item{List}{A list of clustering outputs or output of the\code{DiffGenes}
function. The first element of the list will be used as the reference in
\code{ReorderToReference}. The output of \code{ChooseFeatures} is also
accepted.}

\item{Selection}{If pathway analysis should be conducted for a specific
selection of objects, this selection can be provided here. Selection can
be of the type "character" (names of the objects) or "numeric" (the number
of specific cluster). Default is NULL.}

\item{geneExpr}{The gene expression matrix or ExpressionSet of the objects.
The rows should correspond with the genes.}

\item{nrclusters}{Optional. The number of clusters to cut the dendrogram in.
The number of clusters should not be specified if the interest lies only in
a specific selection of objects which is known by name.  Otherwise, it is
required. Default is NULL.}

\item{method}{The method to applied to look for differentially expressed genes and related pathways. For now, only the
limma method is available for gene analysis and the MLP method for pathway analysis. Default is c("limma","MLP").}

\item{geneInfo}{A data frame with at least the columns ENTREZID and SYMBOL.
This is necessary to connect the symbolic names of the genes with their
EntrezID in the correct order. The order of the gene is here not in the
order of the rownames of the gene expression matrix but in the order of
their significance. Default is NULL.}

\item{geneSetSource}{The source for the getGeneSets function, defaults to
"GOBP".}

\item{topP}{Overrules sign. The number of pathways to display for each
cluster. If not specified, only the significant genes are shown. Default is NULL.}

\item{topG}{Overrules sign. The number of top genes to be returned in the
result. If not specified, only the significant genes are shown. Defaults is NULL.}

\item{GENESET}{Optional. Can provide own candidate gene sets. Default is NULL.}

\item{sign}{The significance level to be handled. Default is 0.05.}

\item{fusionsLog}{Logical. To be handed to \code{ReorderToReference}: indicator for the fusion of clusters. Default is TRUE}

\item{weightclust}{Logical. To be handed to \code{ReorderToReference}: to be used for the outputs of CEC,
WeightedClust or WeightedSimClust. If TRUE, only the result of the Clust element is considered. Default is TRUE.}

\item{names}{Optional. Names of the methods. Default is NULL.}
}
\value{
The returned value is a list with an element per cluster per method.
}
\description{
A pathway analysis per cluster per method is conducted.
}
\details{
After finding differently expressed genes, it can be investigated whether
pathways are related to those genes. This can be done with the help of the
function \code{Pathways} which makes use of the \code{MLP} function of the
MLP package. Given the output of a method, the cutree function is performed
which results into a specific number of clusters. For each cluster, the
limma method is performed comparing this cluster to the other clusters. This
to obtain the necessary p-values of the genes. These are used as the input
for the \code{MLP} function to find interesting pathways. By default the
candidate gene sets are determined by the \code{AnnotateEntrezIDtoGO}
function. The default source will be GOBP, but this can be altered.
Further, it is also possible to provide own candidate gene sets in the form
of a list of pathway categories in which each component contains a vector of
Entrez Gene identifiers related to that particular pathway. The default
values for the minimum and maximum number of genes in a gene set for it to
be considered were used. For MLP this is respectively 5 and 100. If a list
of outputs of several methods is provided as data input, the cluster numbers
are rearranged according to a reference method. The first method is taken as
the reference and ReorderToReference is applied to get the correct ordering.
When the clusters haven been re-appointed, the pathway analysis as described
above is performed for each cluster of each method.

This function relies on the suggested Bioconductor packages 'MLP', 'biomaRt' and 'org.Hs.eg.db'.
}
\examples{

\dontrun{
data(fingerprintMat)
data(targetMat)
data(geneMat)
data(GeneInfo)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(MCF7_F,MCF7_T)
names=c('FP','TP')

MCF7_PathsFandT=Pathways(List=L, geneExpr = geneMat, nrclusters = 7, method = c("limma",
"MLP"), geneInfo = GeneInfo, geneSetSource = "GOBP", topP = NULL,
topG = NULL, GENESET = NULL, sign = 0.05,fusionsLog = TRUE, weightclust = TRUE,
 names =names)
 }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-sim.R
\docType{data}
\name{mosaic_toy}
\alias{mosaic_toy}
\title{Toy multi-omics example data}
\format{
A list with two elements:
\describe{
\item{List}{A list of two numeric matrices with the 60 shared objects in
rows and features in columns (120 and 150 features).}
\item{truth}{Integer vector of the 60 ground-truth group labels.}
}
}
\usage{
data(mosaic_toy)
}
\description{
A small two-source example produced by \code{\link{mosaic_sim}} with three
planted groups, bundled for examples and tests.
}
\examples{
data(mosaic_toy)
str(mosaic_toy, max.level = 2)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nemo.R
\name{NEMO}
\alias{NEMO}
\title{NEMO: neighborhood-based multi-omics clustering}
\usage{
NEMO(List, k = NULL, NN = 20, normalize = TRUE)
}
\arguments{
\item{List}{A list of data matrices over the same objects, \strong{objects in
rows}. Sources may cover different subsets of objects (matched by row name)
for the partial-data setting.}

\item{k}{Number of clusters (\code{NULL} = eigengap estimate).}

\item{NN}{Number of neighbours for the affinity graphs and local scaling.}

\item{normalize}{Logical; row-normalise each affinity to a transition matrix
before integration (NEMO default \code{TRUE}).}
}
\value{
A list of class \code{"NEMO"} with \code{FusedM} (integrated
affinity), \code{DistM} (\code{1 - FusedM}), \code{cluster} (labels) and
\code{Clust} (an \code{agnes} object on \code{DistM} for interoperability).
}
\description{
NEMO \insertCite{Wang2014a}{MosaiClusteR} integrates several omics by building
a locally-scaled, k-nearest-neighbour affinity network for each one and
averaging them, for every pair of objects, over only the omics in which both
objects are measured. Partial / missing modalities are therefore handled
without imputation. The integrated affinity is clustered spectrally.
}
\examples{
data(mosaic_toy)
fit <- NEMO(mosaic_toy$List, k = 3, NN = 15)
table(fit$cluster)
}
\references{
\insertCite{Wang2014a}{MosaiClusteR}; Rappoport, N. & Shamir, R.
(2019). NEMO: cancer subtyping by integration of partial multi-omic data.
Bioinformatics 35:3348-3356.
}
\seealso{
\code{\link{SNF}}, \code{\link{spectral_clustering}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-evaluation.R
\name{compare_clusterings}
\alias{compare_clusterings}
\title{Compare two clusterings derived from distance matrices}
\usage{
compare_clusterings(
  D1,
  D2,
  NC = NULL,
  linkage = "ward.D2",
  labels = c("D1", "D2"),
  verbose = TRUE
)
}
\arguments{
\item{D1, D2}{Square dissimilarity matrices over the same objects.}

\item{NC}{Number of clusters (default \code{floor(sqrt(n))}).}

\item{linkage}{Linkage method. Default \code{"ward.D2"}.}

\item{labels}{Length-2 labels for the two solutions.}

\item{verbose}{Logical; print a short report.}
}
\value{
Invisibly, a list with the two label vectors, \code{ARI},
\code{NMI}, \code{pair_Jaccard} and the \code{contingency} table.
}
\description{
Clusters two dissimilarity matrices with the same linkage and number of
clusters and reports their agreement (ARI, NMI, pairwise Jaccard) and the
contingency table.
}
\examples{
set.seed(1)
D1 <- as.matrix(dist(matrix(rnorm(60), 20)))
D2 <- as.matrix(dist(matrix(rnorm(60), 20)))
compare_clusterings(D1, D2, NC = 3, verbose = FALSE)$ARI
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-dendro.r
\name{ColorsNames}
\alias{ColorsNames}
\title{Function that annotates colors to their names}
\usage{
ColorsNames(matrixColors, cols = NULL)
}
\arguments{
\item{matrixColors}{The output of the ReorderToReference function.}

\item{cols}{A character vector with the names of the colours to be used. Default is NULL.}
}
\value{
A matrix containing the hex code of the color that corresponds to
each cell of the matrix to be colored. This function is called upon by the
\code{ComparePlot} function.
}
\description{
The \code{ColorsNames} function is used on the output of the
\code{ReorderToReference} and matches the cluster numbers indicated by the
cell with the names of the colors.  This is necessary to produce the plot of
the \code{ComparePlot} function and is therefore an internal function of
this function but can also be applied separately.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)
data(Colors1)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(MCF7_F,MCF7_T)
names=c("FP","TP")

MatrixColors=ReorderToReference(List=L,nrclusters=7,fusionsLog=TRUE,weightclust=TRUE,
names=names)

Names=ColorsNames(matrixColors=MatrixColors,cols=Colors1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-dendro.r
\name{ClusterCols}
\alias{ClusterCols}
\title{Helper that colours dendrogram leaves by cluster membership}
\usage{
ClusterCols(x, Data, nrclusters = NULL, cols = NULL, colorComps = NULL)
}
\arguments{
\item{x}{A node of a dendrogram (passed by \code{dendrapply}).}

\item{Data}{The clustering result (an \code{agnes}/\code{hclust} object) used to determine cluster membership.}

\item{nrclusters}{Optional. The number of clusters to cut the dendrogram in. Default is NULL.}

\item{cols}{The colours for the clusters if nrclusters is specified. Default is NULL.}

\item{colorComps}{Optional. A character vector of objects to highlight in red. Default is NULL.}
}
\value{
The dendrogram node \code{x} with coloured \code{nodePar} and \code{edgePar} attributes.
}
\description{
Internal helper applied via \code{dendrapply} that colours the
leaves and edges of a dendrogram according to the cluster a leaf belongs to,
or to a specific selection of objects. Used by \code{ClusterPlot}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spectral.R
\name{spectral_clustering}
\alias{spectral_clustering}
\title{Spectral clustering of an affinity matrix}
\usage{
spectral_clustering(affinity, k = NULL, max_k = 10, nstart = 10)
}
\arguments{
\item{affinity}{A symmetric \eqn{n \times n} non-negative similarity/affinity
matrix over the objects.}

\item{k}{Number of clusters. If \code{NULL}, chosen by the largest eigengap
of the normalised Laplacian (searched over \code{2:max_k}).}

\item{max_k}{Upper bound for the eigengap search when \code{k = NULL}.}

\item{nstart}{Number of k-means restarts.}
}
\value{
A list with \code{cluster} (named integer labels), \code{k} and the
spectral \code{embedding}.
}
\description{
Normalised (Ng-Jordan-Weiss) spectral clustering. Given a symmetric,
non-negative affinity matrix it forms the normalised Laplacian, embeds the
objects in the space of its leading eigenvectors and applies k-means.
}
\examples{
set.seed(1)
X <- rbind(matrix(rnorm(20 * 5, 0), 20), matrix(rnorm(20 * 5, 6), 20))
A <- exp(-as.matrix(dist(X))^2 / 2)
spectral_clustering(A, k = 2)$cluster[1:5]
}
\references{
Ng, A. Y., Jordan, M. I. & Weiss, Y. (2002). On spectral
clustering: analysis and an algorithm. NIPS 14.
}
\seealso{
\code{\link{NEMO}}, \code{\link{SNF}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-heatmap.r
\name{distanceheatmaps}
\alias{distanceheatmaps}
\title{Determine the distance in a heatmap}
\usage{
distanceheatmaps(Data1, Data2, names = NULL, nrclusters = 7)
}
\arguments{
\item{Data1}{The resulting clustering of method 1.}

\item{Data2}{The resulting clustering of method 2.}

\item{names}{The names of the objects in the data sets. Default is NULL.}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}
}
\description{
Internal function of \code{HeatmapPlot}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-evaluation.R
\name{cluster_agreement}
\alias{cluster_agreement}
\title{Agreement between two partitions}
\usage{
cluster_agreement(a, b)
}
\arguments{
\item{a, b}{Integer/character/factor label vectors of equal length.}
}
\value{
A named numeric vector \code{c(ARI, NMI, Jaccard)}.
}
\description{
Computes the adjusted Rand index (ARI), normalised mutual information (NMI)
and pairwise Jaccard between two label vectors of equal length.
}
\details{
ARI rule of thumb: \code{>0.90} excellent; \code{0.75-0.90} good;
\code{0.50-0.75} moderate; \code{<0.50} weak.
}
\examples{
cluster_agreement(c(1, 1, 2, 2), c(2, 2, 1, 1))   # identical up to relabel
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lucid.R
\name{LUCID}
\alias{LUCID}
\title{LUCID: latent unknown clustering integrating omics with an outcome}
\usage{
LUCID(
  G,
  Z,
  Y,
  K,
  lucid_model = c("early", "parallel", "serial"),
  family = c("normal", "binary"),
  ...
)
}
\arguments{
\item{G}{Exposure/predictor matrix (objects in rows).}

\item{Z}{Omics matrix, or a list of matrices for the parallel/serial models
(objects in rows).}

\item{Y}{Outcome vector (length = number of objects).}

\item{K}{Number of latent clusters.}

\item{lucid_model}{One of \code{"early"}, \code{"parallel"}, \code{"serial"}.}

\item{family}{Outcome family, \code{"normal"} or \code{"binary"}.}

\item{...}{Further arguments forwarded to \code{LUCIDus::estimate_lucid}.}
}
\value{
A list of class \code{"LUCID"} with the fitted \code{model} and the
posterior \code{cluster} assignment per object.
}
\description{
Thin wrapper around \pkg{LUCIDus}' \code{estimate_lucid}. LUCID is a
\emph{model-based, supervised} integrative clustering method: it defines a
categorical latent variable (the clusters) jointly from upstream exposures
\code{G}, multi-omics data \code{Z} and an outcome \code{Y} via an EM
algorithm (a quasi-mediation model). Unlike the unsupervised methods in
MosaiClusteR it requires an outcome and returns posterior cluster membership.
}
\examples{
\dontrun{
# requires the 'LUCIDus' package
data(mosaic_toy)
Z <- mosaic_toy$List[[1]]
G <- mosaic_toy$List[[2]][, 1:3]
Y <- as.numeric(mosaic_toy$truth)
fit <- LUCID(G = G, Z = Z, Y = Y, K = 3)
table(fit$cluster)
}
}
\references{
Zhao, Y., Jia, K., Goodrich, J. & Conti, D. V. (2024). LUCIDus:
an R package for implementing LUCID with phenotypic traits. The R Journal
16(2).
}
\seealso{
\code{\link{M_ABCpp}}, \code{\link{intNMF}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{PreparePathway}
\alias{PreparePathway}
\title{Prepare data for pathway analysis}
\usage{
PreparePathway(Object, geneExpr, topG, sign)
}
\arguments{
\item{Object}{A clustering output or DiffGenes output element.}

\item{geneExpr}{The gene expression matrix or ExpressionSet of the objects.}

\item{topG}{Overrules sign. The number of top genes. Default is NULL.}

\item{sign}{The significance level to be handled. Default is 0.05.}
}
\value{
A list with the p-values of the genes, the objects and the genes.
}
\description{
The \code{PreparePathway} function prepares the input for the
pathway analysis by computing differentially expressed genes (via limma) for
a selection of objects if these were not provided.
}
\details{
This function may rely on the suggested Bioconductor package 'limma'.
}
\examples{
\dontrun{
PreparePathway(Object=MCF7_F,geneExpr=geneMat,topG=NULL,sign=0.05)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-features.r
\name{BinFeaturesPlot_MultipleData}
\alias{BinFeaturesPlot_MultipleData}
\title{Visualization of characteristic binary features of multiple data sets}
\usage{
BinFeaturesPlot_MultipleData(
  leadCpds,
  orderLab,
  features = list(),
  data = list(),
  validate = NULL,
  colorLab = NULL,
  nrclusters = NULL,
  cols = NULL,
  name = c("Data1", "Data2"),
  colors1 = c("gray90", "blue"),
  colors2 = c("gray90", "green"),
  margins = c(5.5, 3.5, 0.5, 5.5),
  cexB = 0.8,
  cexL = 0.8,
  cexR = 0.8,
  spaceNames = 0.2,
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{leadCpds}{A character vector with the names of the objects in a first group, i.e., the group for which the specified features are characteristic. Default is NULL.}

\item{orderLab}{A character vector with the order of the objects. Default is NULL.}

\item{features}{A list with as elements character vectors with the names of the features to be visualized for each data set. Default is NULL.}

\item{data}{A list with the different data sets. Default is NULL.}

\item{validate}{Optional. A list with validation data sets. If a feature has a validation reference, these are added in a red colour. Default is NULL.}

\item{colorLab}{Optional. A clustering object if the objects are to be coloured accoring to their clustering order. Default is NULL.}

\item{nrclusters}{Optional. The number of clusters to divide the dendrogram of ColorLab. Default is NULL.}

\item{cols}{Optional. A character vector with the colours of the different clusters. Default is NULL.}

\item{name}{A character string with the names of the data sets. Default is c("Data1" ,"Data2") for two data sets.}

\item{colors1}{A character vector with the colours to indicate the presence (first element) or the absence of the features for the objects in LeadCpds. Default is c('gray90','blue').}

\item{colors2}{A character vector with the colours to indicate the presence (first element) or the absence of the features for the objects in the remaining objects. Default is c('gray90','green').}

\item{margins}{A vector with the margings of the plot. Default is c(5.5,3.5,0.5,5.5).}

\item{cexB}{The font size of the labels on the bottom: the object labels. Default is 0.80.}

\item{cexL}{The font size of the labels on the left: the data labels. Default is 0.80.}

\item{cexR}{The font size of the labels on the right: the feature labels. Default is 0.80.}

\item{spaceNames}{A percentage of the height of the figure to be reserved for the names of the objects. Default is 0.20.}

\item{plottype}{Should be one of "pdf","new" or "sweave". If "pdf", a
location should be provided in "location" and the figure is saved there. If
"new" a new graphic device is opened and if "sweave", the figure is made
compatible to appear in a sweave or knitr document, i.e. no new device is
opened and the plot appears in the current device or document. Default is "new".}

\item{location}{Optional. If plottype is "pdf", a location should be provided in
"location" and the figure is saved there. Default is NULL.}
}
\value{
A plot visualizing the presence and absence of the characteristic binary features across multiple data sets.
}
\description{
A tool to visualize characteristic binary features of a set of objects in comparison with the remaining objects for multiple data sets. The result is a matrix with coloured cells. Columns represent
objects and rows represent the specified features. A feature which is present is give a coloured cell while an absent feature is represented by a grey cell. The labels on the right indicate the names of the features while the labels on the bottom are the names of the objects.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

Comps=FindCluster(list(MCF7_F),nrclusters=10,select=c(1,8))

MCF7_Char=CharacteristicFeatures(List=NULL,Selection=Comps,binData=
list(fingerprintMat,targetMat),datanames=c("FP","TP"),nrclusters=NULL,
topC=NULL,sign=0.05,fusionsLog=TRUE,weightclust=TRUE,names=c("FP","TP"))

FeatFP=MCF7_Char$Selection$Characteristics$FP$TopFeat$Names[c(1:10)]
FeatTP=MCF7_Char$Selection$Characteristics$TP$TopFeat$Names[c(1:10)]

BinFeaturesPlot_MultipleData(leadCpds=Comps,orderLab=MCF7_Char$Selection$
objects$OrderedCpds,features=list(FeatFP,FeatTP),data=list(fingerprintMat,targetMat),
validate=NULL,colorLab=NULL,nrclusters=NULL,cols=NULL,name=c("FP","TP"),colors1=
c('gray90','blue'),colors2=c('gray90','green'),margins=c(5.5,3.5,0.5,5.5),cexB=0.80,
cexL=0.80,cexR=0.80,spaceNames=0.20,plottype="new",location=NULL)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{PathwayAnalysis}
\alias{PathwayAnalysis}
\title{Pathway analysis with intersection over iterations}
\usage{
PathwayAnalysis(
  List,
  Selection = NULL,
  geneExpr = NULL,
  nrclusters = NULL,
  method = c("limma", "MLP"),
  geneInfo = NULL,
  geneSetSource = "GOBP",
  topP = NULL,
  topG = NULL,
  GENESET = NULL,
  sign = 0.05,
  niter = 10,
  fusionsLog = TRUE,
  weightclust = TRUE,
  names = NULL,
  seperatetables = FALSE,
  separatepvals = FALSE
)
}
\arguments{
\item{List}{A list of clustering outputs or output of the\code{DiffGenes}
function.}

\item{Selection}{If pathway analysis should be conducted for a specific
selection of objects, this selection can be provided here. Default is NULL.}

\item{geneExpr}{The gene expression matrix or ExpressionSet of the objects.}

\item{nrclusters}{Optional. The number of clusters to cut the dendrogram in. Default is NULL.}

\item{method}{The methods for gene and pathway analysis. Default is c("limma","MLP").}

\item{geneInfo}{A data frame with at least the columns ENTREZID and SYMBOL. Default is NULL.}

\item{geneSetSource}{The source for the getGeneSets function, defaults to "GOBP".}

\item{topP}{Overrules sign. The number of pathways to display for each cluster. Default is NULL.}

\item{topG}{Overrules sign. The number of top genes to be returned. Default is NULL.}

\item{GENESET}{Optional. Can provide own candidate gene sets. Default is NULL.}

\item{sign}{The significance level to be handled. Default is 0.05.}

\item{niter}{The number of times to perform pathway analysis. Default is 10.}

\item{fusionsLog}{Logical. To be handed to \code{ReorderToReference}. Default is TRUE}

\item{weightclust}{Logical. To be handed to \code{ReorderToReference}. Default is TRUE.}

\item{names}{Optional. Names of the methods. Default is NULL.}

\item{seperatetables}{Logical. Default is FALSE.}

\item{separatepvals}{Logical. Default is FALSE.}
}
\value{
The intersection over the iterations of the pathway analysis.
}
\description{
The \code{PathwayAnalysis} function performs pathway analysis
multiple times via \code{PathwaysIter} and takes the intersection over the
iterations via \code{Geneset.intersect}.
}
\details{
This function relies on the suggested Bioconductor packages 'MLP', 'biomaRt' and 'org.Hs.eg.db'.
}
\examples{
\dontrun{
MCF7_PathsFandT=PathwayAnalysis(List=L, geneExpr = geneMat, nrclusters = 7,
method = c("limma","MLP"), geneInfo = GeneInfo, geneSetSource = "GOBP",niter=2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-characterize.r
\name{FeaturesOfCluster}
\alias{FeaturesOfCluster}
\title{List all features present in a selected cluster of objects}
\usage{
FeaturesOfCluster(
  leadCpds,
  data,
  threshold = 1,
  plot = TRUE,
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{leadCpds}{A character vector containing the objects one wants to
investigate in terms of features.}

\item{data}{The data matrix.}

\item{threshold}{The number of objects the features at least should be
shared amongst. Default is set to 1.}

\item{plot}{Logical. Indicates whether or not a BinFeaturesPlot should be
set up. Default is TRUE.}

\item{plottype}{Should be one of "pdf","new" or "sweave". Default is "new".}

\item{location}{If plottype is "pdf", a location should be provided. Default is NULL.}
}
\value{
A list with 2 items: the number of shared features among the objects,
and a character vector of the plotted features.
}
\description{
The function \code{FeaturesOfCluster} lists the number of features objects
of the cluster have in common. A threshold can be set selecting among how
many objects of the cluster the features should be shared. An optional
plot of the features is available.
}
\examples{
\dontrun{
data(fingerprintMat)

Lead=rownames(fingerprintMat)[1:5]

FeaturesOfCluster(leadCpds=Lead,data=fingerprintMat,
threshold=1,plot=TRUE,plottype="new",location=NULL)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manuscript-viz.R
\name{cluster_selection_plot}
\alias{cluster_selection_plot}
\title{Justify the number of clusters (silhouette + elbow / scree)}
\usage{
cluster_selection_plot(
  x,
  k_range = 2:8,
  linkage = "ward.D2",
  select = c("elbow", "silhouette"),
  plot = TRUE
)
}
\arguments{
\item{x}{A fitted result (with \code{$DistM}), a dissimilarity matrix, or a
\code{List} of data sources (objects in rows).}

\item{k_range}{Integer vector of candidate cluster counts.}

\item{linkage}{Agglomeration method used to cut the dissimilarity.}

\item{select}{How to pick \code{best_k}: \code{"elbow"} (default) uses the
knee of the within-cluster dispersion (scree) curve -- the \eqn{k} at which
the curve starts to flatten; \code{"silhouette"} uses the maximum average
silhouette width.}

\item{plot}{Logical; draw the two-panel figure.}
}
\value{
Invisibly a data frame with \code{k}, \code{silhouette} and
\code{within_dispersion}, plus attributes \code{"best_k"}, \code{"elbow_k"}
and \code{"silhouette_k"}.
}
\description{
For a fused dissimilarity (or a fitted result, or a \code{List} of sources)
compute, over a range of \eqn{k}, the average silhouette width and the total
within-cluster dispersion (an elbow / scree curve), and plot them so the
chosen \eqn{k} can be justified in the manuscript.
}
\examples{
data(mosaic_toy)
fit <- M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3)
cluster_selection_plot(fit, k_range = 2:6)
}
\seealso{
\code{\link{embedding_plot}}, \code{\link{SelectnrClusters}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-dendro.r
\name{LabelCols}
\alias{LabelCols}
\title{Helper that colours specific dendrogram leaves}
\usage{
LabelCols(x, Sel1, Sel2 = NULL, col1 = NULL, col2 = NULL)
}
\arguments{
\item{x}{A node of a dendrogram (passed by \code{dendrapply}).}

\item{Sel1}{The selection of objects to be colored.}

\item{Sel2}{An optional second selection to be colored. Default is NULL.}

\item{col1}{The color for the first selection. Default is NULL.}

\item{col2}{The color for the optional second selection. Default is NULL.}
}
\value{
The dendrogram node \code{x} with coloured \code{nodePar} and \code{edgePar} attributes.
}
\description{
Internal helper applied via \code{dendrapply} that colours the
leaves and edges of a dendrogram for one or two selections of objects. Used
by \code{LabelPlot}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ABCpp.R
\name{ABCpp.SingleInMultiple}
\alias{ABCpp.SingleInMultiple}
\title{Single-source Aggregating Bundles of Clusters (ABC), C++-accelerated engine}
\usage{
ABCpp.SingleInMultiple(
  data,
  distmeasure = "euclidean",
  weighting = "var",
  normalize = NULL,
  gr = c(),
  bag = TRUE,
  numsim = 1000,
  numvar = 100,
  linkage = "ward.D2",
  NC = NULL,
  mds = FALSE,
  nfeat = NULL,
  nugget_type = "between",
  nugget_args = list()
)
}
\arguments{
\item{data}{Numeric matrix with \strong{objects (samples) in rows} and
features in columns (the MosaiClusteR convention; transposed internally to
the features-by-objects layout of the original engine).}

\item{distmeasure}{Distance measure passed to \code{\link{Distance}}.}

\item{weighting}{\code{"var"}, \code{"cv"}, \code{"nugget"} or equal.}

\item{normalize}{Optional normalisation (see \code{\link{Normalization}}).}

\item{gr}{Unused; kept for interface compatibility.}

\item{bag}{Logical; bootstrap the objects each iteration (Step 2).}

\item{numsim}{Number of iterations \eqn{R}.}

\item{numvar}{Kept for backward compatibility; superseded by \code{nfeat}.}

\item{linkage}{Agglomeration method for \code{fastcluster::hclust}.}

\item{NC}{Base-cluster count (\code{NULL} = \eqn{\lfloor\sqrt{N}\rfloor}).}

\item{mds}{Unused placeholder.}

\item{nfeat}{Optional feature-subsample size. \code{NULL} (default) uses the
classic \eqn{g=\lfloor\sqrt{G}\rfloor}; a value \eqn{\ge 1} sets an absolute
count (raise it for low-dimensional sources where most features matter).}

\item{nugget_type, nugget_args}{Passed to the data-nugget weighting when
\code{weighting = "nugget"}.}
}
\value{
A data frame, \code{numsim} rows by one column per object; entries are
base-cluster labels (\code{0} = object not selected that iteration).
}
\description{
The canonical single-source ABC engine of
\insertCite{Amaratunga2008}{MosaiClusteR}. Over \code{numsim} iterations it
(optionally) bootstraps the objects, draws a weighted subsample of features,
clusters the sub-matrix with \code{fastcluster::hclust}, and records each
selected object's base-cluster label. \code{\link{M_ABCpp}} turns the stacked
label matrix into a consensus dissimilarity using the C++ co-clustering
kernel.
}
\section{Feature weighting (incl. data nuggets)}{

Features are subsampled with probability proportional to a rank-transformed
importance score:
\itemize{
\item \code{"var"} -- feature variance (the classic ABC choice);
\item \code{"cv"}  -- coefficient of variation;
\item \code{"nugget"} -- \emph{NEW}: the data-nugget between-group weighted
variance (\code{\link{create_data_nuggets}}), a robust, Big-Data-friendly
alternative to the raw variance;
\item anything else -- equal weights.
}
}

\examples{
data(mosaic_toy)
lab <- ABCpp.SingleInMultiple(mosaic_toy$List[[1]], numsim = 40, NC = 3,
                              weighting = "nugget")
dim(lab)
}
\references{
\insertAllCited{}
}
\seealso{
\code{\link{M_ABCpp}}, \code{\link{create_data_nuggets}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-dendro.r
\name{LabelPlot}
\alias{LabelPlot}
\title{Coloring specific leaves of a dendrogram}
\usage{
LabelPlot(Data, sel1, sel2 = NULL, col1 = NULL, col2 = NULL)
}
\arguments{
\item{Data}{The result of a method which contains the dendrogram to be
colored.}

\item{sel1}{The selection of objects to be colored. Default is NULL.}

\item{sel2}{An optional second selection to be colored. Default is NULL.}

\item{col1}{The color for the first selection. Default is NULL.}

\item{col2}{The color for the optional second selection. Default is NULL.}
}
\value{
A plot of the dendrogram of which the leaves of the selection(s) are
colored.
}
\description{
The function plots a dendrogrmam of which specific leaves are coloured.
}
\examples{
\dontrun{
data(fingerprintMat)
MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

ClustF_6=cutree(MCF7_F$Clust,6)

SelF=rownames(fingerprintMat)[ClustF_6==6]
SelF

LabelPlot(Data=MCF7_F,sel1=SelF,sel2=NULL,col1='darkorchid')
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkbased.r
\name{LinkBasedClustering}
\alias{LinkBasedClustering}
\title{Link-based cluster ensembles}
\usage{
LinkBasedClustering(
  List,
  type = c("data", "dist", "clust"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  clust = "agnes",
  linkage = c("flexible", "flexible"),
  alpha = 0.625,
  nrclusters = c(7, 7),
  gap = FALSE,
  maxK = 15,
  linkBasedMethod = c("cts", "srs", "asrs"),
  decayfactor = 0.8,
  niter = 5,
  linkBasedLinkage = "ward",
  waitingtime = 300,
  file_number = 0,
  executable = FALSE
)
}
\arguments{
\item{List}{A list of data matrices, distance matrices or clustering results.
It is assumed the rows are corresponding with the objects.}

\item{type}{Indicates whether the provided matrices in "List" are either data
matrices, distance matrices or clustering results obtained from the data.
Should be one of "data", "dist" or "clust".}

\item{distmeasure}{A vector of the distance measures to be used on each data
matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming".
Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices
or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if
different distance types are used. More details on normalization in
\code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile",
"Fisher-Yates", "standardize","Range" or any of the first letters of these
names. Default is c(NULL,NULL) for two data sets.}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character) for each data
set. Defaults to c("flexible", "flexible") for two data sets.}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the
agnes function. Defaults to 0.625 and is only used if the linkage is set to
"flexible".}

\item{nrclusters}{A vector with the number of clusters to cut each base
dendrogram into. If NULL and gap=FALSE, the function stops.}

\item{gap}{Logical. Whether to use the gap statistic to determine the number
of clusters per data set. Defaults to FALSE.}

\item{maxK}{The maximal number of clusters to consider when gap=TRUE.}

\item{linkBasedMethod}{The link-based refinement scheme. One of "cts", "srs"
or "asrs". Defaults to "cts".}

\item{decayfactor}{The decay (confidence) factor used by the refinement scheme.
Defaults to 0.8.}

\item{niter}{The number of iterations for the SimRank-based scheme ("srs").
Defaults to 5.}

\item{linkBasedLinkage}{The linkage used in the final agglomerative clustering
of the refined similarity matrix. Defaults to "ward".}

\item{waitingtime}{Retained for backward compatibility with the external
executable / MATLAB workflow; ignored by the pure-R schemes.}

\item{file_number}{Retained for backward compatibility with the external
executable / MATLAB workflow; ignored by the pure-R schemes.}

\item{executable}{Logical. If TRUE, the refined similarity matrix is computed
by an external executable / MATLAB. The "cts" and "srs" schemes are pure R
and work without an executable. The "asrs" scheme is only available through
the external executable; if executable=FALSE it stops with an informative
message.}
}
\value{
The returned value is a list of two elements:
\item{DistM}{The resulting (refined) distance matrix}
\item{Clust}{The resulting hierarchical clustering}
The value has class 'LinkBased'.
}
\description{
Link-based cluster ensembles combine multiple base clusterings
into a single, refined similarity matrix by exploiting the link structure
between clusters of the ensemble. Three refinement schemes are available:
Connected-Triple-Based Similarity ("cts"), SimRank-Based Similarity ("srs")
and Approximate SimRank-Based Similarity ("asrs"). The refined object-by-object
similarity matrix is turned into a dissimilarity matrix on which a final
agglomerative hierarchical clustering is performed.
}
\details{
Each base clustering of the ensemble is represented as a hard
partition. The clusters of all partitions form the vertices of a weighted
graph; the link weights between clusters are derived from shared objects. The
refined object-by-object similarity is obtained by propagating these
cluster-to-cluster links ("cts", "srs", "asrs"). See Iam-On et al. (2010).
}
\examples{
\dontrun{
data(mosaic_toy)
L <- mosaic_toy$List
res <- LinkBasedClustering(List = L, type = "data",
  distmeasure = c("euclidean", "euclidean"), normalize = c(FALSE, FALSE),
  linkBasedMethod = "cts", decayfactor = 0.8, niter = 5,
  linkBasedLinkage = "ward", nrclusters = c(3, 3))
}
}
\references{
Iam-On, N., Boongoen, T., Garrett, S. and Price, C. (2010). Link-based cluster
ensembles for the combination of multiple clusterings. IEEE Transactions on
Knowledge and Data Engineering, 23(12), 1839-1853.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-compare.r
\name{DetermineWeight_SimClust}
\alias{DetermineWeight_SimClust}
\title{Determines an optimal weight for weighted clustering by similarity weighted clustering.}
\usage{
DetermineWeight_SimClust(
  List,
  type = c("data", "dist", "clusters"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  weight = seq(0, 1, by = 0.01),
  nrclusters = NULL,
  clust = "agnes",
  linkage = c("flexible", "flexible"),
  linkageF = "ward",
  alpha = 0.625,
  gap = FALSE,
  maxK = 15,
  names = NULL,
  StopRange = FALSE,
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{List}{A list of matrices of the same type. It is assumed the rows are corresponding with the objects.}

\item{type}{indicates whether the provided matrices in "List" are either data matrices, distance
matrices or clustering results obtained from the data. Type should be one of "data", "dist" or "clusters".}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices. Default is c(FALSE, FALSE).}

\item{method}{A method of normalization. Default is c(NULL,NULL).}

\item{weight}{Optional. A list of different weight combinations for the data sets in List. Defaults to seq(0,1,by=0.01).}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity for the individual clusterings. Defaults to c("flexible","flexible").}

\item{linkageF}{Choice of inter group dissimilarity for the final clustering. Defaults to "ward".}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625.}

\item{gap}{Logical. Whether or not to calculate the gap statistic. Only if type="data". Default is FALSE.}

\item{maxK}{The maximal number of clusters to consider in calculating the gap statistic. Default is 15.}

\item{names}{The labels to give to the elements in List. Default is NULL.}

\item{StopRange}{Logical. Indicates whether the distance matrices with values not between zero and one
should be standardized to have so. Default is FALSE.}

\item{plottype}{Should be one of "pdf","new" or "sweave". Default is "new".}

\item{location}{If plottype is "pdf", a location should be provided here. Default is NULL.}
}
\value{
The returned value is a list with three elements: \item{ClustSep}{The result of \code{Cluster}
for each single element of List} \item{Result}{A data frame with the Jaccard coefficients and their
ratios for each weight} \item{Weight}{The optimal weight}.
}
\description{
The function \code{DetermineWeight_SimClust} determines an optimal weight
for performing weighted similarity clustering. For each given weight, each separate clustering
is compared to the clustering on a weighted dissimilarity matrix and a Jaccard coefficient is
calculated. The ratio of the Jaccard coefficients closest to one indicates an optimal weight.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",alpha=0.625,gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",alpha=0.625,gap=FALSE,maxK=55,StopRange=FALSE)

L=list(MCF7_F,MCF7_T)

MCF7_Weight=DetermineWeight_SimClust(List=L,type="clusters",weight=seq(0,1,by=0.01),
nrclusters=c(7,7),distmeasure=c("tanimoto","tanimoto"),normalize=c(FALSE,FALSE),
method=c(NULL,NULL),clust="agnes",linkage=c("flexible","flexible"),linkageF="ward",
alpha=0.625,gap=FALSE,maxK=50,names=c("FP","TP"),StopRange=FALSE,plottype="new",location=NULL)
}
}
\references{
Perualila-Tan, N. et al. (2016). Weighted similarity-based clustering of chemical
structures and bioactivity data in early drug discovery. Journal of Bioinformatics and Computational Biology.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nugget-clustering.R
\name{Whclust}
\alias{Whclust}
\title{Weighted hierarchical clustering (weighted Ward)}
\usage{
Whclust(x, weights = rep(1, nrow(x)))
}
\arguments{
\item{x}{Numeric matrix, observations (e.g. nugget centres) in rows.}

\item{weights}{Non-negative weight per observation.}
}
\value{
An \code{\link[stats]{hclust}} object.
}
\description{
Agglomerative clustering of weighted observations using the weighted Ward
merge cost. Intended for the small set of data-nugget centres.
}
\examples{
set.seed(1)
x <- rbind(matrix(rnorm(8 * 3, 0), 8), matrix(rnorm(8 * 3, 5), 8))
stats::cutree(Whclust(x, weights = rep(2, 16)), k = 2)[1:4]
}
\references{
Dey et al. (2025). WCluster.
}
\seealso{
\code{\link{Wkmeans}}, \code{\link{nugget_cluster}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate-weighting.R
\name{simulate_weighting_regime}
\alias{simulate_weighting_regime}
\title{Simulate data for evaluating feature-weighting schemes (variance vs CV)}
\usage{
simulate_weighting_regime(
  regime = c("cv", "variance"),
  n_samples = 120,
  n_clusters = 3,
  n_sources = 2,
  n_signal = 40,
  n_decoy = 500,
  n_noise = 60,
  effect = NULL,
  seed = NULL
)
}
\arguments{
\item{regime}{\code{"cv"} (default) or \code{"variance"}; see Details.}

\item{n_samples}{Number of objects (rows), split evenly across clusters.}

\item{n_clusters}{Number of clusters.}

\item{n_sources}{Number of data sources (independent draws of the same design).}

\item{n_signal, n_decoy, n_noise}{Numbers of discriminative, decoy
(non-discriminative but high-variance in the CV regime) and pure-noise
features per source.}

\item{effect}{Between-cluster mean shift on the signal features (defaults:
1.4 for the variance regime, 0.9 for the cv regime).}

\item{seed}{Optional RNG seed.}
}
\value{
A list with \code{List} (the sources, objects in rows), \code{truth}
(cluster labels), \code{signal_features} (indices of the discriminative
features), \code{regime} and \code{params}.
}
\description{
Generates a multi-source data set with a known cluster structure in which the
discriminative signal is carried either by high-absolute-variance features
(\code{regime = "variance"}) or by high-\emph{coefficient-of-variation}
features buried under high-abundance, high-variance decoys
(\code{regime = "cv"}). It is intended for benchmarking the \code{"var"},
\code{"cv"} and \code{"nugget"} weightings of the ABC family (and any other
feature-weighting method) on the same footing.
}
\examples{
cv  <- simulate_weighting_regime("cv", seed = 1)
# variance weighting is misled by high-abundance decoys, CV recovers the truth:
fit_var <- M_ABCpp(cv$List, numsim = 80, NC = 3, weighting = c("var", "var"))
fit_cv  <- M_ABCpp(cv$List, numsim = 80, NC = 3, weighting = c("cv",  "cv"))
c(var = cluster_agreement(mosaic_labels(fit_var, 3), cv$truth)[["ARI"]],
  cv  = cluster_agreement(mosaic_labels(fit_cv,  3), cv$truth)[["ARI"]])
}
\seealso{
\code{\link{M_ABCpp}} (the \code{weighting} argument),
\code{\link{mosaic_sim}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/characterize.R
\name{cluster_biology_panel}
\alias{cluster_biology_panel}
\title{Panel of cluster characterisation plots}
\usage{
cluster_biology_panel(labels, metadata, vars = NULL, ncol = 3)
}
\arguments{
\item{labels}{Cluster labels.}

\item{metadata}{Data frame of external variables (rows = objects).}

\item{vars}{Optional subset of columns to show (default: all).}

\item{ncol}{Number of columns in the panel grid.}
}
\value{
Invisibly \code{NULL}.
}
\description{
Lay out \code{\link{plot_cluster_profile}} for every external variable in a
single figure -- boxplots for continuous variables, bar plots for categorical
ones -- ready for the manuscript.
}
\seealso{
\code{\link{characterize_clusters}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adc.r
\name{ADC}
\alias{ADC}
\title{Aggregated data clustering}
\usage{
ADC(
  List,
  distmeasure = "tanimoto",
  normalize = FALSE,
  method = NULL,
  clust = "agnes",
  linkage = "flexible",
  alpha = 0.625
)
}
\arguments{
\item{List}{A list of data matrices of the same type. It is assumed the rows are corresponding with the objects.}

\item{distmeasure}{Choice of metric for the dissimilarity matrix (character). Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to "tanimoto".}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to FALSE. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is NULL.}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character). Defaults to "flexible".}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".}
}
\value{
The returned value is a list with the following three elements.
\item{AllData}{Fused data matrix of the data matrices}
\item{DistM}{The resulting distance matrix}
\item{Clust}{The resulting clustering}
The value has class 'ADC'. The Clust element will be of interest for further applications.
}
\description{
Aggregated Data Clustering (ADC) is a direct clustering multi-source technique. The data matrices are merged into a single fused data matrix on which a dissimilarity matrix is computed. Hierarchical clustering is then performed on this dissimilarity matrix.
}
\examples{
\dontrun{
data(mosaic_toy)
L <- mosaic_toy$List
res_ADC <- ADC(List=L, distmeasure="euclidean", normalize=FALSE, method=NULL,
               clust="agnes", linkage="flexible", alpha=0.625)
}
}
\references{
Fodeh, S. J., Punch, W. & Tan, P.-N. (2013). On unifying multi-source information from probabilistically generated functional data. Proteins 81:1298-1310.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-characterize.r
\name{TrackCluster}
\alias{TrackCluster}
\title{Track a cluster or a selection of objects across multiple methods}
\usage{
TrackCluster(
  List,
  Selection,
  nrclusters = NULL,
  followMaxComps = FALSE,
  followClust = TRUE,
  fusionsLog = TRUE,
  weightclust = TRUE,
  names = NULL,
  selectionPlot = FALSE,
  table = FALSE,
  completeSelectionPlot = FALSE,
  ClusterPlot = FALSE,
  cols = NULL,
  legendposx = 0.5,
  legendposy = 2.4,
  plottype = "sweave",
  location = NULL
)
}
\arguments{
\item{List}{A list of clustering outputs to be compared. The first element of
the list is used as the reference in \code{ReorderToReference}.}

\item{Selection}{Either a character vector of objects, or a numeric cluster number.}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{followMaxComps}{Logical. Whether to follow the cluster with the maximum
number of objects. Default is FALSE.}

\item{followClust}{Logical. Whether to follow the cluster of interest. Default is TRUE.}

\item{fusionsLog}{Logical. Indicator for the fusion of clusters. Default is TRUE.}

\item{weightclust}{Logical. For the outputs of CEC, WeightedClust or
WeightedSimClust, if TRUE only the result of the Clust element is considered.
Default is TRUE.}

\item{names}{Optional. Names of the methods. Default is NULL.}

\item{selectionPlot}{Logical. Whether to produce the selection plot. Default is FALSE.}

\item{table}{Logical. Whether to produce a table of the tracking. Default is FALSE.}

\item{completeSelectionPlot}{Logical. Whether to produce the complete selection
plot. Default is FALSE.}

\item{ClusterPlot}{Logical. Whether to produce the cluster plot. Default is FALSE.}

\item{cols}{The colors to use in the plots. Default is NULL.}

\item{legendposx}{The x-coordinate of the legend. Default is 0.5.}

\item{legendposy}{The y-coordinate of the legend. Default is 2.4.}

\item{plottype}{Should be one of "pdf","new" or "sweave". Default is "sweave".}

\item{location}{If plottype is "pdf", a location should be provided. Default is NULL.}
}
\value{
A list with one element per method describing how the selection is
distributed across the clusters, and optionally a tracking table.
}
\description{
\code{TrackCluster} follows a selection of objects or a specific cluster
across a sequence of clustering results, recording how the objects are
distributed over the clusters of each method and optionally producing
tracking plots.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(MCF7_F,MCF7_T)
MCF7_Track=TrackCluster(List=L,Selection=4,nrclusters=7,followMaxComps=FALSE,
followClust=TRUE,selectionPlot=TRUE,table=TRUE,cols=NULL)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-navigation.r
\name{FindElement}
\alias{FindElement}
\title{Find an element in a data structure}
\usage{
FindElement(what = NULL, object = NULL, element = list())
}
\arguments{
\item{what}{A character string indicating which object to look for. Default is NULL.}

\item{object}{The data structure to look into. Only the classes data frame
and list are supported. Default is NULL.}

\item{element}{Not to be specified by the user.}
}
\value{
The returned value is a list with an element for each object found.
The element contains everything the object contained in the original data
structure.
}
\description{
The function \code{FindElement} is used internally in the
\code{PreparePathway} function but might come in handy for other uses as well.
Given the name of an object, the function searches for that object in the data
structure and extracts it. When multiple objects have the same name, all are
extracted.
}
\examples{
\dontrun{
obj <- list(a = 1, b = list(TopDE = 42, c = list(TopDE = 7)))
FindElement(what = "TopDE", object = obj)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{Geneset.intersect}
\alias{Geneset.intersect}
\title{Intersection over resulting gene sets of \code{PathwaysIter} function}
\usage{
Geneset.intersect(
  PathwaysOutput,
  Selection = FALSE,
  sign = 0.05,
  names = NULL,
  seperatetables = FALSE,
  separatepvals = FALSE
)
}
\arguments{
\item{PathwaysOutput}{The output of the \code{PathwaysIter} function.}

\item{Selection}{Logical. Indicates whether or not the output of the
pathways function were concentrated on a specific selection of objects. If
this was the case, Selection should be put to TRUE. Otherwise, it should be
put to FALSE. Default is TRUE.}

\item{sign}{The significance level to be handled for cutting of the
pathways. Default is 0.05.}

\item{names}{Optional. Names of the methods. Default is NULL.}

\item{seperatetables}{Logical. If TRUE, a separate element is created per
cluster containing the pathways for each iteration. Default is FALSE.}

\item{separatepvals}{Logical. If TRUE, the p-values of the each iteration of
each pathway in the intersection is given. If FALSE, only the mean p-value
is provided. Default is FALSE.}
}
\value{
The output is a list with an element per method. For each method, it
is portrayed per cluster which pathways belong to the intersection over all
iterations and their corresponding mean p-values.
}
\description{
The function \code{Geneset.intersect} collects the results of the
\code{PathwaysIter} function per method for each cluster and takes the
intersection over the iterations per cluster per method. This is to see if
over the different resamplings of the data, similar pathways were
discovered.
}
\examples{

\dontrun{
data(fingerprintMat)
data(targetMat)
data(geneMat)
data(GeneInfo)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(MCF7_F,MCF7_T)

MCF7_Paths_FandT=PathwaysIter(List=L, geneExpr = geneMat, nrclusters = 7, method =
c("limma", "MLP"), geneInfo = GeneInfo, geneSetSource = "GOBP", topP = NULL,
topG = NULL, GENESET = NULL, sign = 0.05,niter=2,fusionsLog = TRUE,
weightclust = TRUE, names =names)

MCF7_Paths_intersection=Geneset.intersect(PathwaysOutput=MCF7_Paths_FandT,
sign=0.05,names=c("FP","TP"),seperatetables=FALSE,separatepvals=FALSE)

str(MCF7_Paths_intersection)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nugget-clustering.R
\name{nugget_cluster}
\alias{nugget_cluster}
\title{Data-nugget clustering}
\usage{
nugget_cluster(
  data,
  k,
  method = c("kmeans", "ward"),
  max_nuggets = NULL,
  nugget_args = list()
)
}
\arguments{
\item{data}{A numeric matrix (objects in rows) \emph{or} a precomputed
\code{\link{create_data_nuggets}{data_nugget}} object. For multi-source
data, column-bind the per-source matrices first (the direct/\code{ADC}
strategy) or pass a single fused matrix.}

\item{k}{Number of clusters.}

\item{method}{Weighted clusterer for the nugget centres: \code{"kmeans"}
(WWCSS) or \code{"ward"} (weighted Ward).}

\item{max_nuggets, nugget_args}{Passed to \code{\link{create_data_nuggets}}
when \code{data} is a raw matrix.}
}
\value{
A list of class \code{"nugget_cluster"} with \code{cluster} (labels
for the original objects), \code{nugget_cluster} (labels for the nuggets)
and the \code{nuggets} object.
}
\description{
Cluster a (potentially large) data set by first compressing its observations
into weighted data nuggets (\code{\link{create_data_nuggets}}), clustering the
nugget centres with a weighted clusterer, and mapping the nugget labels back
to the original observations. This is the clustering analogue of the
data-nugget weighting used by \code{\link{M_ABCpp}} and follows
\code{DN.Wkmeans} / \code{DN.Whclust} from the WCluster package.
}
\examples{
data(mosaic_toy)
X <- do.call(cbind, mosaic_toy$List)        # fuse sources (objects in rows)
fit <- nugget_cluster(X, k = 3, max_nuggets = 30)
table(fit$cluster)
}
\references{
Cherasia et al. (2023); Dey et al. (2025), WCluster.
}
\seealso{
\code{\link{create_data_nuggets}}, \code{\link{Wkmeans}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{SharedSelection}
\alias{SharedSelection}
\title{Intersection of genes and pathways over multiple methods for a selection of objects.}
\usage{
SharedSelection(
  DataLimma = NULL,
  DataMLP = NULL,
  DataFeat = NULL,
  names = NULL
)
}
\arguments{
\item{DataLimma}{Optional. The output of a \code{DiffGenes} function. Default is NULL.}

\item{DataMLP}{Optional. The output of \code{Geneset.intersect} function. Default is NULL.}

\item{DataFeat}{Optional. The output of \code{CharacteristicFeatures}
function. Default is NULL.}

\item{names}{Optional. Names of the methods or "Selection" if it only
considers a selection of objects. Default is NULL.}
}
\value{
A list with a Table and a Which element for the selection.
}
\description{
Internal function of \code{SharedGenesPathsFeat}.
}
\examples{
\dontrun{
SharedSelection(DataLimma=MCF7_FT_DE,names="Selection")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-features.r
\name{ProfilePlot}
\alias{ProfilePlot}
\title{Plotting gene profiles}
\usage{
ProfilePlot(
  Genes,
  Comps,
  geneExpr = NULL,
  raw = FALSE,
  orderLab = NULL,
  colorLab = NULL,
  nrclusters = NULL,
  cols = NULL,
  addLegend = TRUE,
  margins = c(8.1, 4.1, 1.1, 6.5),
  extra = 5,
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{Genes}{The genes to be plotted.}

\item{Comps}{The objects to be plotted or to be separated from the other
objects.}

\item{geneExpr}{The gene expression matrix or ExpressionSet of the objects.}

\item{raw}{Logical. Should raw p-values be plotted? Default is FALSE.}

\item{orderLab}{Optional. If the objects are to set in a specific order of
a specific method. Default is NULL.}

\item{colorLab}{The clustering result that determines the color of the
labels of the objects in the plot. Default is NULL.}

\item{nrclusters}{Optional. The number of clusters to cut the dendrogram in.}

\item{cols}{Optional. The color to use for the objects in Clusters for each
method.}

\item{addLegend}{Optional. Whether a legend of the colors should be added to
the plot.}

\item{margins}{Optional. Margins to be used for the plot. Default is margins=c(8.1,4.1,1.1,6.5).}

\item{extra}{The space between the plot and the legend. Default is 5.}

\item{plottype}{Should be one of "pdf","new" or "sweave". If "pdf", a
location should be provided in "location" and the figure is saved there. If
"new" a new graphic device is opened and if "sweave", the figure is made
compatible to appear in a sweave or knitr document, i.e. no new device is
opened and the plot appears in the current device or document. Default is "new".}

\item{location}{If plottype is "pdf", a location should be provided in
"location" and the figure is saved there. Default is NULL.}
}
\value{
A plot which contains multiple gene profiles. A distinction is made
between the values for the objects in Comps and the others.
}
\description{
In \code{ProfilePlot}, the gene profiles of the significant genes for a
specific cluster are shown on 1 plot. Therefore, each gene is normalized by
subtracting its the mean.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)
data(geneMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
	method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
	method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(MCF7_F,MCF7_T)
names=c('FP','TP')

MCF7_FT_DE = DiffGenes(List=L,geneExpr=geneMat,nrclusters=7,method="limma",sign=0.05,topG=10,
fusionsLog=TRUE,weightclust=TRUE)

Comps=SharedComps(list(MCF7_FT_DE$`Method 1`$"Cluster 1",MCF7_FT_DE$`Method 2`$"Cluster 1"))[[1]]

MCF7_SharedGenes=FindGenes(dataLimma=MCF7_FT_DE,names=c("FP","TP"))

Genes=names(MCF7_SharedGenes[[1]])[-c(2,4,5)]

colscl=ColorPalette(colors=c("red","green","purple","brown","blue","orange"),ncols=9)

ProfilePlot(Genes=Genes,Comps=Comps,geneExpr=geneMat,raw=FALSE,orderLab=MCF7_F,
colorLab=NULL,nrclusters=7,cols=colscl,addLegend=TRUE,margins=c(16.1,6.1,1.1,13.5),
extra=4,plottype="sweave",location=NULL)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{FindGenes}
\alias{FindGenes}
\title{Find the genes shared across methods}
\usage{
FindGenes(dataLimma, names = NULL)
}
\arguments{
\item{dataLimma}{The output of the \code{DiffGenes} function.}

\item{names}{Optional. Names of the methods. Default is NULL.}
}
\value{
The result is a list with for each cluster a list of the found genes
and the methods that found them.
}
\description{
The \code{FindGenes} function collects the differentially
expressed genes per cluster per method (the output of \code{DiffGenes}) and
determines which genes are found across the methods.
}
\examples{
\dontrun{
MCF7_SharedGenes=FindGenes(dataLimma=MCF7_FT_DE,names=c("FP","TP"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-dendro.r
\name{ColorPalette}
\alias{ColorPalette}
\title{Create a color palette to be used in the plots}
\usage{
ColorPalette(colors = c("red", "green"), ncols = 5)
}
\arguments{
\item{colors}{A vector containing the colors of choice}

\item{ncols}{The number of colors to be specified. If higher than the number
of colors, it specifies colors in the region between the given colors.}
}
\value{
A vector containing the hex codes of the chosen colors.
}
\description{
In order to facilitate the visualization of the influence of the different
methods on the clustering of the objects, colours can be used. The function
\code{ColorPalette} is able to pick out as many colours as there are
clusters. This is done with the help of the \code{ColorRampPalette} function
of the grDevices package
}
\examples{

Colors1<-ColorPalette(c("cadetblue2","chocolate","firebrick2",
"darkgoldenrod2", "darkgreen","blue2","darkorchid3","deeppink2"), ncols=8)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalization.R
\name{Normalization}
\alias{Normalization}
\title{Normalisation of features}
\usage{
Normalization(
  Data,
  method = c("Quantile", "Fisher-Yates", "Standardize", "Range", "Q", "q", "F", "f", "S",
    "s", "R", "r")
)
}
\arguments{
\item{Data}{A data matrix; rows are assumed to correspond to the objects.}

\item{method}{One of \code{"Quantile"}, \code{"Fisher-Yates"},
\code{"Standardize"}, \code{"Range"} (or any unambiguous first letter).}
}
\value{
The normalised matrix.
}
\description{
When data of different scales are combined it is recommended to normalise so
that the structures are comparable.
}
\details{
\code{"Quantile"} performs quantile normalisation (common in omics);
\code{"Fisher-Yates"} uses normal scores based only on the number of rows;
\code{"Standardize"} centres and scales each column (base
\code{\link[base]{scale}}); \code{"Range"} maps the matrix to \code{[0, 1]}.
}
\examples{
x <- matrix(rnorm(100), 10, 10)
Norm_x <- Normalization(x, method = "R")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weightedclust.r
\name{WeightedClust}
\alias{WeightedClust}
\title{Weighted clustering}
\usage{
WeightedClust(
  List,
  type = c("data", "dist", "clusters"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  StopRange = FALSE,
  weight = seq(1, 0, -0.1),
  weightclust = 0.5,
  clust = "agnes",
  linkage = "ward",
  alpha = 0.625
)
}
\arguments{
\item{List}{A list of data matrices. It is assumed the rows are corresponding with the objects.}

\item{type}{indicates whether the provided matrices in "List" are either data matrices, distance
matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
matrices is skipped and if type="clusters" the single source clustering is skipped.
Type should be one of "data", "dist" or "clusters".}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.}

\item{StopRange}{Logical. Indicates whether the distance matrices with values not between zero and one should be standardized to have values between zero and one.
If FALSE the range normalization is performed. See \code{Normalization}. If TRUE, the distance matrices are not changed.
This is recommended if different types of data are used such that these are comparable. Default is FALSE.}

\item{weight}{Optional. A list of different weight combinations for the data sets in List.
If NULL, the weights are determined to be equal for each data set.
It is further possible to fix weights for some data matrices and to
let it vary randomly for the remaining data sets. Defaults to seq(1,0,-0.1).  An example is provided in the details.}

\item{weightclust}{A weight for which the result will be put aside of the other results. This was done for comparative reason and easy access. Defaults to 0.5 (two data sets)}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character) for the final clustering. Defaults to "ward".}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".}
}
\value{
The returned value is a list of four elements:
\item{DistM}{A list with the distance matrix for each data structure}
\item{WeightedDist }{A list with the weighted distance matrices}
\item{Results}{The hierarchical clustering result for each element in WeightedDist}
\item{Clust}{The result for the weight specified in Clustweight}
The value has class 'Weighted'.
}
\description{
Weighted Clustering (Weighted) is a similarity-based multi-source clustering technique. Weighted clustering is performed with the function \code{WeightedClust}. Given a
list of the data matrices, a dissimilarity matrix is computed of each with
the provided distance measures. These matrices are then combined resulting
in a weighted dissimilarity matrix. Hierarchical clustering is performed
on this weighted combination with the agnes function and the ward link
}
\details{
The weight combinations should be provided as elements in a list. For three data
matrices an example could be: weights=list(c(0.5,0.2,0.3),c(0.1,0.5,0.4)). To provide
a fixed weight for some data sets and let it vary randomly for others, the element "x"
indicates a free parameter. An example is weights=list(c(0.7,"x","x")). The weight 0.7
is now fixed for the first data matrix while the remaining 0.3 weight will be divided over
the other two data sets. This implies that every combination of the sequence from 0 to 0.3
with steps of 0.1 will be reported and clustering will be performed for each.
}
\references{
\insertRef{PerualilaTan2016}{MosaiClusteR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-compare.r
\name{CompareSvsM}
\alias{CompareSvsM}
\title{Comparison of clustering results for the single and multiple source clustering.}
\usage{
CompareSvsM(
  ListS,
  ListM,
  nrclusters = NULL,
  cols = NULL,
  fusionsLogS = FALSE,
  fusionsLogM = FALSE,
  weightclustS = FALSE,
  weightclustM = FALSE,
  namesS = NULL,
  namesM = NULL,
  margins = c(8.1, 3.1, 3.1, 4.1),
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{ListS}{A list of the outputs from the single source clusterings to be compared.}

\item{ListM}{A list of the outputs from the multiple source clusterings to be compared.}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{cols}{A character vector with the names of the colours. Default is NULL.}

\item{fusionsLogS}{The fusionslog parameter for the elements in ListS. Default is FALSE.}

\item{fusionsLogM}{The fusionsLog parameter for the elements in ListM. Default is FALSE.}

\item{weightclustS}{The weightclust parameter for the elements in ListS. Default is FALSE.}

\item{weightclustM}{The weightclust parameter for the elements in ListM. Default is FALSE.}

\item{namesS}{Optional. Names of the single source clusterings. Default is NULL.}

\item{namesM}{Optional. Names of the multiple source clusterings. Default is NULL.}

\item{margins}{Optional. Margins to be used for the plot. Default is c(8.1,3.1,3.1,4.1).}

\item{plottype}{Should be one of "pdf","new" or "sweave". Default is "new".}

\item{location}{If plottype is "pdf", a location should be provided here. Default is NULL.}
}
\value{
A plot with on the left the comparison over the objects in ListS and on the right a
comparison over the objects in ListM.
}
\description{
The function \code{CompareSvsM} plots the comparison of the single source
clustering results on the left and that of the multiple source clustering results on the
right such that a visual comparison is possible.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)
data(Colors1)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(fingerprintMat,targetMat)

MCF7_W=WeightedClust(List=L,type="data", distmeasure=c("tanimoto","tanimoto"),
normalize=c(FALSE,FALSE),method=c(NULL,NULL),weight=seq(1,0,-0.1),weightclust=0.5
,clust="agnes",linkage="ward",StopRange=FALSE)

ListM=list(MCF7_W)
namesM=seq(1.0,0.0,-0.1)

ListS=list(MCF7_F,MCF7_T)
namesS=c("FP","TP")

CompareSvsM(ListS,ListM,nrclusters=7,cols=Colors1,fusionsLogS=FALSE,
fusionsLogM=FALSE,weightclustS=FALSE,weightclustM=FALSE,namesS,
namesM,plottype="new",location=NULL)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nuggets.R
\name{nugget_feature_weights}
\alias{nugget_feature_weights}
\title{Feature weights derived from data nuggets}
\usage{
nugget_feature_weights(nuggets, type = c("between", "ratio", "inv_scale"))
}
\arguments{
\item{nuggets}{A \code{"data_nugget"} object from
\code{\link{create_data_nuggets}} (built with samples in the rows).}

\item{type}{One of:
\describe{
\item{\code{"between"}}{(default) size-weighted between-nugget variance of
each feature: \eqn{\sum_i w_i (c_{ij} - \bar c_j)^2 / \sum_i w_i}.}
\item{\code{"ratio"}}{a separability / pseudo-\eqn{F} score,
\eqn{\mathrm{between}_j / (\mathrm{between}_j + \overline{\mathrm{within}}_j)}.}
\item{\code{"inv_scale"}}{inverse mean within-nugget variance, which
down-weights noisy features.}
}}
}
\value{
A named numeric vector of length \code{ncol(centers)} of
non-negative feature scores.
}
\description{
Computes a per-feature importance score from a data-nugget
(\code{\link{create_data_nuggets}}) object built over the \emph{observations}
(samples). This score is the
data-nugget analogue of the feature variance used by the ABC / M-ABC family:
it rewards features that separate the representative groups while discounting
within-nugget noise, giving a robust, Big-Data-friendly weighting.
}
\examples{
set.seed(1)
x <- cbind(signal = c(rnorm(50, 0), rnorm(50, 5)),  # separates groups
           noise  = rnorm(100))                      # pure noise
dn <- create_data_nuggets(x, max_nuggets = 20)
nugget_feature_weights(dn, type = "between")
}
\references{
\insertRef{Cherasia2023}{MosaiClusteR}
}
\seealso{
\code{\link{create_data_nuggets}}, \code{\link{M_ABCpp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-features.r
\name{ContFeaturesPlot}
\alias{ContFeaturesPlot}
\title{Plot of continuous features}
\usage{
ContFeaturesPlot(
  leadCpds,
  data,
  nrclusters = NULL,
  orderLab = NULL,
  colorLab = NULL,
  cols = NULL,
  ylab = "features",
  addLegend = TRUE,
  margins = c(5.5, 3.5, 0.5, 8.7),
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{leadCpds}{A character vector containing the objects one wants to
separate from the others.}

\item{data}{The data matrix.}

\item{nrclusters}{Optional. The number of clusters to consider if colorLab
is specified. Default is NULL.}

\item{orderLab}{Optional. If the objects are to set in a specific order of
a specific method. Default is NULL.}

\item{colorLab}{The clustering result that determines the color of the
labels of the objects in the plot. If NULL, the labels are black. Default is NULL.}

\item{cols}{The colors for the labels of the objects. Default is NULL.}

\item{ylab}{The lable of the y-axis. Default is "features".}

\item{addLegend}{Logical. Indicates whether a legend should be added to the
plot. Default is TRUE.}

\item{margins}{Optional. Margins to be used for the plot. Default is c(5.5,3.5,0.5,8.7).}

\item{plottype}{Should be one of "pdf","new" or "sweave". If "pdf", a
location should be provided in "location" and the figure is saved there. If
"new" a new graphic device is opened and if "sweave", the figure is made
compatible to appear in a sweave or knitr document, i.e. no new device is
opened and the plot appears in the current device or document. Default is "new".}

\item{location}{If plottype is "pdf", a location should be provided in
"location" and the figure is saved there. Default is NULL.}
}
\value{
A plot in which the values of the features of the leadCpds are
separeted from the others.
}
\description{
The function \code{ContFeaturesPlot} plots the values of continuous features.
It is possible to separate between objects of interest and the
other objects.
}
\examples{
\dontrun{
data(Colors1)
Comps=c("Cpd1", "Cpd2", "Cpd3", "Cpd4", "Cpd5")

Data=matrix(sample(15, size = 50*5, replace = TRUE), nrow = 50, ncol = 5)
colnames(Data)=colnames(Data, do.NULL = FALSE, prefix = "col")
rownames(Data)=rownames(Data, do.NULL = FALSE, prefix = "row")
for(i in 1:50){
	rownames(Data)[i]=paste("Cpd",i,sep="")
}

ContFeaturesPlot(leadCpds=Comps,orderLab=rownames(Data),colorLab=NULL,data=Data,
nrclusters=7,cols=Colors1,ylab="features",addLegend=TRUE,margins=c(5.5,3.5,0.5,8.7),
plottype="new",location=NULL)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-evaluation.R
\name{mosaic_labels}
\alias{mosaic_labels}
\title{Extract a flat partition from a MosaiClusteR result}
\usage{
mosaic_labels(object, k, linkage = "ward")
}
\arguments{
\item{object}{A MosaiClusteR result (e.g. from \code{\link{M_ABCpp}}), an
\code{agnes}/\code{hclust} object, or a distance matrix.}

\item{k}{Number of clusters.}

\item{linkage}{Linkage used when \code{object} is a bare distance matrix.}
}
\value{
An integer vector of cluster labels, named by object.
}
\description{
Cuts the hierarchical clustering carried in a MosaiClusteR result object into
\code{k} groups. Works with any method returning a \code{Clust} element (an
\code{agnes}/\code{hclust} object) and/or a \code{DistM}.
}
\examples{
set.seed(1)
s <- paste0("o", 1:20)
L <- list(matrix(rnorm(20 * 40), 20, dimnames = list(s, NULL)),
          matrix(rnorm(20 * 50), 20, dimnames = list(s, NULL)))
fit <- M_ABCpp(L, numsim = 30)
mosaic_labels(fit, k = 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-heatmap.r
\name{HeatmapSelection}
\alias{HeatmapSelection}
\title{A function to select a group of objects via the similarity heatmap.}
\usage{
HeatmapSelection(
  Data,
  type = c("data", "dist", "clust", "sim"),
  distmeasure = "tanimoto",
  normalize = FALSE,
  method = NULL,
  linkage = "flexible",
  cutoff = NULL,
  percentile = FALSE,
  dendrogram = NULL,
  width = 7,
  height = 7
)
}
\arguments{
\item{Data}{The data of which a heatmap should be drawn.}

\item{type}{indicates whether the provided matrices in "List" are either data matrices, distance
matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
matrices is skipped and if type="clusters" the single source clustering is skipped.
Type should be one of "data", "dist" or "clusters".}

\item{distmeasure}{The distance measure. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to "tanimoto".}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is NULL.}

\item{linkage}{Choice of inter group dissimilarity (character). Defaults to "flexible".}

\item{cutoff}{Optional. If a cutoff value is specified, all values lower are
put to zero while all other values are kept. This helps to highlight the
most similar objects. Default is NULL.}

\item{percentile}{Logical. The cutoff value can be a percentile. If one want
the cutoff value to be the 90th percentile of the data, one should specify
cutoff = 0.90 and percentile = TRUE. Default is FALSE.}

\item{dendrogram}{Optional. If the clustering results of the data is already
available and should not be recalculated, this results can be provided here.
Otherwise, it will be calculated given the data. This is necessary to have
the objects in their order of clustering on the plot. Default is NULL.}

\item{width}{The width of the plot to be made. This can be adjusted since
the default size might not show a clear picture. Default is 7.}

\item{height}{The height of the plot to be made. This can be adjusted since
the default size might not show a clear picture. Default is 7.}
}
\value{
A heatmap with the names of the objects on the right and bottom.
Once points are selected, it will return the names of the objects that are
in the selected square provided that these show similarity among each other.
}
\description{
The function \code{HeatmapSelection} plots the similarity values between
objects. The plot is similar to the one produced by
\code{SimilarityHeatmap} but without the dendrograms on the sides. The
function is rather explorative and experimental and is to be used with some
caution. By clicking in the plot, the user can select a group of objects
of interest. See more in \code{Details}.

A similarity heatmap is created in the same way as in
\code{SimilarityHeatmap}. The user is now free to select two points on the
heatmap. It is advised that these two points are in opposite corners of a
square that indicates a high similarity among the objects. The points do
not have to be the exact corners of the group of interest, a little
deviation is allowed as rows and columns of the selected subset of the
matrix with sum equal to 1 are filtered out. A sum equal to one, implies
that the compound is only similar to itself.

The function is meant to be explorative but is experimental. The goal was to
make the selection of interesting objects easier as sometimes the labels
of the dendrograms are too distorted to be read. If the figure is exported
to a pdf file with an appropriate width and height, the labels can be become
readable again.
}
\examples{

\dontrun{
data(fingerprintMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55)

HeatmapSelection(Data=MCF7_F$DistM,type="dist",cutoff=0.90,percentile=TRUE,
dendrogram=MCF7_F,width=7,height=7)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{SharedSelectionLimma}
\alias{SharedSelectionLimma}
\title{Intersection of genes over multiple methods for a selection of objects.}
\usage{
SharedSelectionLimma(DataLimma = NULL, names = NULL)
}
\arguments{
\item{DataLimma}{Optional. The output of a \code{DiffGenes} function. Default is NULL.}

\item{names}{Optional. Names of the methods or "Selection" if it only
considers a selection of objects. Default is NULL.}
}
\value{
A list with a Table and a Which element for the shared genes.
}
\description{
Internal function of \code{SharedGenesPathsFeat}.
}
\examples{
\dontrun{
SharedSelectionLimma(DataLimma=MCF7_FT_DE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/M_ABCpp.R
\name{M_ABCpp}
\alias{M_ABCpp}
\title{Multi-source Aggregating Bundles of Clusters (M-ABC), C++-accelerated}
\usage{
M_ABCpp(
  List,
  distmeasure = rep("euclidean", length(List)),
  weighting = rep("var", length(List)),
  normalize = rep(FALSE, length(List)),
  gr = c(),
  bag = rep(TRUE, length(List)),
  numsim = 1000,
  numvar = rep(100, length(List)),
  linkage = rep("ward.D2", length(List)),
  NC = NULL,
  dissimilarity = "mabc",
  final_linkage = "ward",
  alpha = 0.625,
  mds = FALSE,
  nfeat = NULL,
  nugget_type = "between",
  nugget_args = list()
)
}
\arguments{
\item{List}{A list of \eqn{K} data matrices over the same objects,
\strong{objects (samples) in rows} and features in columns.}

\item{distmeasure, weighting, normalize, bag, numvar, linkage}{Per-source vectors
(recycled to \eqn{K}) forwarded to \code{\link{ABCpp.SingleInMultiple}}.}

\item{gr}{Unused; kept for compatibility.}

\item{numsim}{Iterations per source.}

\item{NC}{Base-cluster count per source (\code{NULL} = data-driven).}

\item{dissimilarity}{Consensus dissimilarity, \code{"mabc"} or \code{"abc"}.}

\item{final_linkage, alpha}{Final clustering controls.}

\item{mds}{Logical; MDS of the consensus dissimilarities.}

\item{nfeat}{Feature-subsample size forwarded to
\code{\link{ABCpp.SingleInMultiple}} (\code{NULL} = \eqn{\lfloor\sqrt{G}\rfloor}).}

\item{nugget_type, nugget_args}{Forwarded to the nugget weighting.}
}
\value{
A list of class \code{"M_ABC"} with \code{DistM} and \code{Clust}.
}
\description{
Extends the single-source ABC algorithm of
\insertCite{Amaratunga2008}{MosaiClusteR} to \eqn{K \ge 2} sources: runs
\code{\link{ABCpp.SingleInMultiple}} per source, row-stacks the per-iteration
label matrices, and builds a single consensus dissimilarity via
\code{\link{f.clustABC.MultiSource}} (C++ kernel when available).
}
\section{Data-nugget weighting}{

Set \code{weighting = "nugget"} (per source, recycled) to weight features by
the data-nugget between-group variance instead of the classic variance; see
\code{\link{ABCpp.SingleInMultiple}} and \code{\link{create_data_nuggets}}.
}

\examples{
data(mosaic_toy)
fit <- M_ABCpp(mosaic_toy$List, numsim = 50, NC = 3,
               weighting = c("var", "nugget"))
table(stats::cutree(stats::as.hclust(fit$Clust), 3))
}
\references{
\insertAllCited{}
}
\seealso{
\code{\link{ABCpp.SingleInMultiple}}, \code{\link{f.clustABC.MultiSource}},
\code{\link{create_data_nuggets}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-features.r
\name{BinFeaturesPlot_SingleData}
\alias{BinFeaturesPlot_SingleData}
\title{Visualization of characteristic binary features of a single data set}
\usage{
BinFeaturesPlot_SingleData(
  leadCpds = c(),
  orderLab = c(),
  features = c(),
  data = NULL,
  colorLab = NULL,
  nrclusters = NULL,
  cols = NULL,
  name = c("Data"),
  colors1 = c("gray90", "blue"),
  colors2 = c("gray90", "green"),
  highlightFeat = NULL,
  margins = c(5.5, 3.5, 0.5, 5.5),
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{leadCpds}{A character vector with the names of the objects in a first group, i.e., the group for which the specified features are characteristic. Default is NULL.}

\item{orderLab}{A character vector with the order of the objects. Default is NULL.}

\item{features}{A character vector with the names of the features to be visualized. Default is NULL.}

\item{data}{The data matrix. Default is NULL.}

\item{colorLab}{Optional. A clustering object if the objects are to be coloured accoring to their clustering order. Default is NULL.}

\item{nrclusters}{Optional. The number of clusters to divide the dendrogram of ColorLab. Default is NULL.}

\item{cols}{Optional. A character vector with the colours of the different clusters. Default is NULL.}

\item{name}{A character string with the name of the data. Default is "Data".}

\item{colors1}{A character vector with the colours to indicate the presence (first element) or the absence of the features for the objects in LeadCpds. Default is c('gray90','blue').}

\item{colors2}{A character vector with the colours to indicate the presence (first element) or the absence of the features for the objects in the remaining objects. Default is c('gray90','green').}

\item{highlightFeat}{Optional. A character vector with names of features to be highlighted. The names of the features are coloured purple. Default is NULL.}

\item{margins}{A vector with the margings of the plot. Default is c(5.5,3.5,0.5,5.5).}

\item{plottype}{Should be one of "pdf","new" or "sweave". If "pdf", a
location should be provided in "location" and the figure is saved there. If
"new" a new graphic device is opened and if "sweave", the figure is made
compatible to appear in a sweave or knitr document, i.e. no new device is
opened and the plot appears in the current device or document. Default is "new".}

\item{location}{Optional. If plottype is "pdf", a location should be provided in
"location" and the figure is saved there. Default is NULL.}
}
\value{
A plot visualizing the presence and absence of the characteristic binary features.
}
\description{
A tool to visualize characteristic binary features of a set of objects in comparison with the remaining objects for a single data set. The result is a matrix with coloured cells. Columns represent
objects and rows represent the specified features. A feature which is present is give a coloured cell while an absent feature is represented by a grey cell. The labels on the right indicate the names of the features while the labels on the bottom are the names of the objects.
}
\examples{
\dontrun{
data(fingerprintMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

Comps=FindCluster(list(MCF7_F),nrclusters=10,select=c(1,8))

MCF7_Char=CharacteristicFeatures(List=list(fingerprintMat),Selection=Comps,
binData=list(fingerprintMat),datanames=c("FP"),nrclusters=NULL,topC=NULL,
sign=0.05,fusionsLog=TRUE,weightclust=TRUE,names=c("FP"))
Feat=MCF7_Char$Selection$Characteristics$FP$TopFeat$Names[c(1:10)]

BinFeaturesPlot_SingleData(leadCpds=Comps,orderLab=MCF7_Char$Selection$
objects$OrderedCpds,features=Feat,data=fingerprintMat,colorLab=NULL,
nrclusters=NULL,cols=NULL,name=c("FP"),colors1=c('gray90','blue'),colors2=
c('gray90','green'),highlightFeat=NULL,margins=c(5.5,3.5,0.5,5.5),
plottype="new",location=NULL)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-navigation.r
\name{ReorderToReference}
\alias{ReorderToReference}
\title{Relabel and reorder clusterings to a reference}
\usage{
ReorderToReference(
  List,
  nrclusters = NULL,
  fusionsLog = FALSE,
  weightclust = FALSE,
  names = NULL
)
}
\arguments{
\item{List}{A list of clustering outputs to be compared. The first element
of the list will be used as the reference.}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{fusionsLog}{Logical. Indicator for the fusion of clusters. Default is FALSE.}

\item{weightclust}{Logical. To be used for the outputs of CEC, WeightedClust
or WeightedSimClust. If TRUE, only the result of the Clust element is
considered. Default is FALSE.}

\item{names}{Optional. A character vector with the names of the methods.}
}
\value{
A matrix of which the cells indicate to what cluster the objects
belong to according to the rearranged methods.
}
\description{
\code{ReorderToReference} relabels and reorders the clusters of
a series of clustering results so that the cluster numbers of every method
are matched, via a Gale-Shapley stable-matching scheme, to those of the first
(reference) method. The result is a matrix in which the columns represent the
objects (ordered as for the reference method) and the rows represent the
methods. Each cell contains the number of the cluster the object belongs to
for that method, relabelled to the reference. It is a possibility that two or
more clusters are fused together compared to the reference; in that case the
function alerts the user and asks to set \code{fusionsLog} to \code{TRUE}.
}
\note{
The \code{ReorderToReference} function was optimized for the situations
presented by the data sets at hand. It is noted that the function might fail
in a particular situation which results in an infinite loop.
}
\examples{
\dontrun{
data(mosaic_toy)
L <- mosaic_toy$List
res <- list(Cluster(L[[1]], type = "data", distmeasure = "euclidean"),
            Cluster(L[[2]], type = "data", distmeasure = "euclidean"))
M <- ReorderToReference(res, nrclusters = 5, fusionsLog = FALSE,
                        weightclust = FALSE, names = c("S1", "S2"))
}
}
\references{
Van Moerbeke, M. (2017). MoSaiClusteR: integrative clustering of
multi-source data. Hasselt University.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-characterize.r
\name{ChooseCluster}
\alias{ChooseCluster}
\title{Interactive plot to determine DE Genes and DE features for a specific cluster}
\usage{
ChooseCluster(
  Interactive = TRUE,
  leadCpds = NULL,
  clusterResult = NULL,
  colorLab = NULL,
  binData = NULL,
  contData = NULL,
  datanames = c("FP"),
  geneExpr = NULL,
  topChar = 20,
  topG = 20,
  sign = 0.05,
  nrclusters = NULL,
  cols = NULL,
  n = 1
)
}
\arguments{
\item{Interactive}{Logical. Whether an interactive plot should be made. Defaults to TRUE.}

\item{leadCpds}{A list of the objects of the clusters of interest. Defaults to NULL.}

\item{clusterResult}{The output of one of the aggregated cluster functions. Default is NULL.}

\item{colorLab}{The clustering result the dendrogram should be colored after. Default is NULL.}

\item{binData}{A list of the binary feature data matrices. Default is NULL.}

\item{contData}{A list of continuous data sets of the objects. Default is NULL.}

\item{datanames}{A vector with the names of the data matrices. Default is c("FP").}

\item{geneExpr}{A gene expression matrix, may also be an ExpressionSet. Default is NULL.}

\item{topChar}{The number of top characteristics to return. Default is 20.}

\item{topG}{The number of top genes to return. Default is 20.}

\item{sign}{The significance level. Default is 0.05.}

\item{nrclusters}{Optional. The number of clusters to cut the dendrogram in. Default is NULL.}

\item{cols}{The colors to use in the dendrogram. Default is NULL.}

\item{n}{The number of clusters one wants to identify by a mouse click. Default is 1.}
}
\value{
A list with one element per cluster of interest with elements
objects, Characteristics and (optionally) Genes.
}
\description{
If desired, the function produces a dendrogram of a clustering result. One
or multiple clusters can be indicated by a mouse click. From these clusters
DE genes and characteristic features are determined. It is also possible to
provide the objects of interest without producing the plot.
}
\details{
This function may require the suggested packages 'a4Base', 'a4Core'
and 'limma' when a gene expression matrix is supplied.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)
data(geneMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

MCF7_Interactive=ChooseCluster(Interactive=TRUE,leadCpds=NULL,clusterResult=MCF7_T,
colorLab=MCF7_F,binData=list(fingerprintMat),datanames=c("FP"),geneExpr=geneMat,
topChar = 20, topG = 20,nrclusters=7,n=1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/voting.r
\name{CVAA}
\alias{CVAA}
\title{Cumulative Voting Aggregation (CVAA / W-CVAA)}
\usage{
CVAA(
  Reference = NULL,
  nrclustersR = 7,
  List,
  typeL = c("data", "dist", "clust"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  clust = "agnes",
  linkage = c("flexible", "flexible"),
  alpha = 0.625,
  nrclusters = c(7, 7),
  gap = FALSE,
  maxK = 15,
  votingMethod = c("CVAA", "W-CVAA"),
  optimalk = nrclustersR
)
}
\arguments{
\item{Reference}{A reference partition (a clustering result, optionally an
object produced by a weighted/CEC method). Required.}

\item{nrclustersR}{The number of clusters of the reference partition. Default is 7.}

\item{List}{A list of data matrices. It is assumed the rows correspond with the objects.}

\item{typeL}{Indicates whether the provided matrices in "List" are either data
matrices ("data"), distance matrices ("dist") or clustering results ("clust").}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE).}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL).}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible","flexible").}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625.}

\item{nrclusters}{The number of clusters to divide each individual dendrogram in. Default is c(7,7).}

\item{gap}{Logical. Whether the optimal number of clusters should be determined with the gap statistic. Defaults to FALSE.}

\item{maxK}{The maximal number of clusters to investigate in the gap statistic. Default is 15.}

\item{votingMethod}{The method to be performed: "CVAA" or "W-CVAA".}

\item{optimalk}{An estimate of the final optimal number of clusters. Defaults to nrclustersR.}
}
\value{
A list of two elements:
\item{DistM}{A NULL object}
\item{Clust}{The resulting clustering (order, order.lab and Clusters)}
The value has class 'Ensemble'.
}
\description{
\code{CVAA} performs cumulative voting consensus clustering. A
reference partition is used to align the individual source partitions, which
are then aggregated into a consensus stochastic membership matrix. Two
variants are available: cumulative voting (\code{"CVAA"}) and the
information-weighted variant (\code{"W-CVAA"}).
}
\details{
Cumulative voting aggregation aligns each source membership matrix to
a running consensus by least-squares voting, then averages cumulatively. The
weighted variant orders the partitions by average information content and
weights their contribution accordingly. When \code{optimalk} is smaller than
\code{nrclustersR} the JS-ALink agglomeration is used to merge reference
clusters.
}
\examples{
\dontrun{
data(mosaic_toy)
L <- mosaic_toy$List
Ref <- Cluster(L[[1]], type = "data", distmeasure = "euclidean")
attr(Ref, "method") <- "Single"
res <- CVAA(Reference = Ref, nrclustersR = 5, List = L, typeL = "data",
  distmeasure = c("euclidean","euclidean"), votingMethod = "CVAA", optimalk = 5)
}
}
\references{
Ayad H.G. and Kamel M.S. (2008). Cumulative voting consensus
method for partitions with variable number of clusters. IEEE Transactions on
Pattern Analysis and Machine Intelligence, 30(1), 160-173.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/consensus.r
\name{ConsensusClustering}
\alias{ConsensusClustering}
\title{Voting-based consensus clustering}
\usage{
ConsensusClustering(
  List,
  type = c("data", "dist", "clust"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  clust = "agnes",
  linkage = c("flexible", "flexible"),
  alpha = 0.625,
  nrclusters = c(7, 7),
  gap = FALSE,
  maxK = 15,
  votingMethod = c("IVC", "IPVC", "IPC"),
  optimalk = 7
)
}
\arguments{
\item{List}{A list of data matrices. It is assumed the rows are corresponding with the objects.}

\item{type}{Indicates whether the provided matrices in "List" are either data matrices ("data"),
distance matrices ("dist") or clustering results obtained from the data ("clust").}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. More details on normalization in \code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".}

\item{nrclusters}{The number of clusters to divide each individual dendrogram in. Default is c(7,7) for two data sets.}

\item{gap}{Logical. Whether the optimal number of clusters should be determined with the gap statistic. Defaults to FALSE.}

\item{maxK}{The maximal number of clusters to investigate in the gap statistic. Default is 15.}

\item{votingMethod}{The voting method to be performed: one of "IVC", "IPVC" or "IPC".}

\item{optimalk}{The number of clusters for the final consensus partition. Defaults to 7.}
}
\value{
The returned value is a list of two elements:
\item{DistM}{A NULL object}
\item{Clust}{The resulting clustering}
The value has class 'Ensemble'.
}
\description{
Consensus Clustering performs voting-based consensus on a set of
single-source partitions. Each data matrix is clustered separately and the
resulting label matrix is reconciled into a single consensus partition by an
iterative voting scheme. Three voting methods are available: Iterative Voting
Consensus (IVC), Iterative Probabilistic Voting Consensus (IPVC) and Iterative
Pairwise Consensus (IPC).
}
\examples{
\dontrun{
data(mosaic_toy)
fit <- ConsensusClustering(List = mosaic_toy$List, type = "data",
  distmeasure = c("euclidean", "euclidean"), normalize = c(FALSE, FALSE),
  method = c(NULL, NULL), clust = "agnes", linkage = c("ward", "ward"),
  nrclusters = c(3, 3), gap = FALSE, votingMethod = "IVC", optimalk = 3)
}
}
\references{
Nguyen N. and Caruana R. (2007). Consensus Clusterings. Seventh IEEE International Conference on Data Mining (ICDM 2007), 607-612.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/characterize.R
\name{plot_cluster_profile}
\alias{plot_cluster_profile}
\title{Plot one external variable by cluster}
\usage{
plot_cluster_profile(labels, x, name = deparse(substitute(x)), col = NULL)
}
\arguments{
\item{labels}{Cluster labels.}

\item{x}{The variable (numeric -> boxplot; factor/character -> barplot).}

\item{name}{Axis / title label for the variable.}

\item{col}{Optional vector of cluster colours.}
}
\value{
Invisibly \code{NULL}; called for the plot.
}
\description{
Boxplot (continuous) or grouped bar plot of column proportions (categorical)
of an external variable across clusters -- the per-cluster distribution and
gradient the manuscript should show.
}
\seealso{
\code{\link{characterize_clusters}}, \code{\link{cluster_biology_panel}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/voting.r
\name{EvidenceAccumulation}
\alias{EvidenceAccumulation}
\title{Evidence accumulation clustering (co-association)}
\usage{
EvidenceAccumulation(
  List,
  type = c("data", "dist", "clust"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  clust = "agnes",
  linkage = c("flexible", "flexible"),
  alpha = 0.625,
  nrclusters = c(7, 7),
  gap = FALSE,
  maxK = 15,
  graphPartitioning = c("MST", "SL", "SL_agnes"),
  t = NULL
)
}
\arguments{
\item{List}{A list of data matrices. It is assumed the rows correspond with the objects.}

\item{type}{Indicates whether the provided matrices in "List" are either data
matrices ("data"), distance matrices ("dist") or clustering results ("clust").}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE).}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL).}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible","flexible").}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625.}

\item{nrclusters}{The number of clusters to divide each individual dendrogram in. Default is c(7,7).}

\item{gap}{Logical. Whether the optimal number of clusters should be determined with the gap statistic. Defaults to FALSE.}

\item{maxK}{The maximal number of clusters to investigate in the gap statistic. Default is 15.}

\item{graphPartitioning}{The graph partitioning method to be performed: "MST", "SL" or "SL_agnes".}

\item{t}{A threshold used by the "MST" and "SL" partitioning procedures. Defaults to NULL.}
}
\value{
A list of two elements:
\item{DistM}{A NULL object}
\item{Clust}{The resulting clustering}
The value has class 'Ensemble' (or 'Single Clustering' for "SL_agnes").
}
\description{
\code{EvidenceAccumulation} builds a co-association
(similarity) matrix from the individual source partitions and extracts a
consensus partition by graph partitioning. Three graph-partitioning options
are available: a minimum-spanning-tree procedure (\code{"MST"}, requires the
\pkg{igraph} package), a thresholded single-link procedure (\code{"SL"}), and
an agnes single-link procedure (\code{"SL_agnes"}).
}
\details{
The co-association matrix \code{S} records the fraction of source
partitions in which two objects are co-clustered. \code{"MST"} prunes a
minimum spanning tree of the similarity graph (optionally at threshold
\code{t}); \code{"SL"} performs a thresholded single-link grouping; and
\code{"SL_agnes"} runs agnes single linkage on the dissimilarity \code{1-S}.
The \code{"MST"} path requires the suggested package \pkg{igraph}.
}
\examples{
\dontrun{
data(mosaic_toy)
L <- mosaic_toy$List
res <- EvidenceAccumulation(List = L, type = "data",
  distmeasure = c("euclidean","euclidean"), nrclusters = c(5,5),
  graphPartitioning = "SL_agnes")
}
}
\references{
Fred A.L.N. and Jain A.K. (2005). Combining multiple clusterings
using evidence accumulation. IEEE Transactions on Pattern Analysis and
Machine Intelligence, 27(6), 835-850.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intnmf.R
\name{intNMF}
\alias{intNMF}
\title{Integrative NMF clustering (intNMF)}
\usage{
intNMF(List, k, nstart = 10, max_iter = 200, seed = NULL)
}
\arguments{
\item{List}{A list of data matrices over the same objects, \strong{objects in
rows}. Matrices are shifted to be non-negative internally.}

\item{k}{Number of clusters (also the factorisation rank).}

\item{nstart}{Number of random restarts forming the consensus.}

\item{max_iter}{Maximum multiplicative-update iterations per restart.}

\item{seed}{Optional RNG seed.}
}
\value{
A list of class \code{"intNMF"} with \code{cluster}, the consensus
\code{DistM}, an \code{agnes} \code{Clust}, the best-fit basis \code{W} and
source weights \code{alpha}.
}
\description{
Joint non-negative matrix factorisation across data sources: a single shared
basis matrix \eqn{W} (objects \eqn{\times} rank) and source-specific
coefficient matrices \eqn{H_k} minimise \eqn{\sum_k \alpha_k \lVert X_k - W
H_k \rVert_F^2} with all factors non-negative. Sources are weighted by
\eqn{\alpha_k = \max_j \overline{SS}_j / \overline{SS}_k}. Object clusters are
obtained by consensus of the dominant-factor assignment across restarts.
}
\examples{
data(mosaic_toy)
fit <- intNMF(mosaic_toy$List, k = 3, nstart = 5, seed = 1)
table(fit$cluster)
}
\references{
Chalise, P. & Fridley, B. L. (2017). Integrative clustering of
multi-level omic data based on non-negative matrix factorization. PLoS ONE
12:e0176278. Reviewed in Zhang et al. (2022), WIREs Comput Stat 14:e1553.
}
\seealso{
\code{\link{M_ABCpp}}, \code{\link{SNF}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ensemble-graph.r
\name{EHC}
\alias{EHC}
\title{Ensemble hierarchical clustering via graph partitioning (METIS / MST)}
\usage{
EHC(
  List,
  type = c("data", "dist", "clust"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  clust = "agnes",
  linkage = c("flexible", "flexible"),
  alpha = 0.625,
  gap = FALSE,
  maxK = 15,
  graphPartitioning = c("METIS", "MST"),
  optimalk = 7,
  waitingtime = 300,
  file_number = 0,
  executable = FALSE
)
}
\arguments{
\item{List}{A list of data matrices. It is assumed the rows are corresponding with the objects.}

\item{type}{indicates whether the provided matrices in "List" are either data matrices, distance
matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
matrices is skipped and if type="clust" the single source clustering is skipped.
Type should be one of "data", "dist" or "clust".}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".}

\item{gap}{Logical. Whether the optimal number of clusters should be determined with the gap statistic. Default is FALSE.}

\item{maxK}{The maximal number of clusters to investigate in the gap statistic. Default is 15.}

\item{graphPartitioning}{A character string indicating the graph partitioning strategy. One of "METIS" (external executable) or "MST" (pure R, requires igraph). Defaults to "METIS".}

\item{optimalk}{An estimate of the final optimal number of clusters. Default is 7.}

\item{waitingtime}{The maximum number of seconds to wait for the external program to write its result before stopping. Defaults to 300.}

\item{file_number}{An identifier appended to the temporary files exchanged with the external program. Defaults to 00.}

\item{executable}{Either FALSE (the default) or the path/flag identifying the external graph-partitioning executable (METIS / gpmetis / shmetis) or MATLAB to use for the "METIS" strategy. When FALSE the "METIS" path stops, since no partition can be computed in pure R. Precompiled binaries are not bundled with the package.}
}
\value{
The returned value is a list of two elements:
\item{DistM}{The weighted cluster association matrix}
\item{Clust}{The resulting clustering}
The value has class 'Ensemble'.
}
\description{
Ensemble Hierarchical Clustering (EHC) builds a cluster
association graph from the agglomerative clustering results of each data
source. Strengths of association between pairs of objects are accumulated
across the dendrograms (weighted by cluster depth and intra-cluster
proximity), yielding a weighted adjacency matrix that is then partitioned.

Two partitioning strategies are available through \code{graphPartitioning}:
\describe{
\item{"METIS"}{Delegates the partitioning of the association graph to an
\strong{external graph-partitioning executable} (METIS / \code{gpmetis} /
\code{shmetis}, or the corresponding MATLAB \code{hmetis} routine). These
binaries are \strong{not bundled} with the package and must be configured
through the \code{executable} argument. When \code{executable=FALSE}
(the default), or when the configured executable cannot be located, the
function stops with an informative error.}
\item{"MST"}{A pure-R minimum spanning tree partitioning that needs no
external executable. It does require the \pkg{igraph} package; if
\pkg{igraph} is not installed the function stops with an informative
error.}
}
}
\examples{
\dontrun{
data(mosaic_toy)
L <- mosaic_toy$List

# METIS path requires an external graph-partitioning executable / MATLAB
# configured through the 'executable' argument.
res <- EHC(List = L, type = "data",
  distmeasure = c("euclidean", "euclidean"), normalize = c(FALSE, FALSE),
  method = c(NULL, NULL), clust = "agnes",
  linkage = c("flexible", "flexible"), alpha = 0.625, gap = FALSE,
  maxK = 15, graphPartitioning = "METIS", optimalk = 7,
  executable = "./MetisAlgorithm")

# MST path is pure R but needs the igraph package.
res2 <- EHC(List = L, type = "data", graphPartitioning = "MST")
}
}
\references{
Mirzaei A. and Rahmati M. (2010). A novel hierarchical-clustering-combination scheme based on fuzzy-similarity relations. IEEE Transactions on Fuzzy Systems, 18(1), 27-39.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-characterize.r
\name{FeatSelection}
\alias{FeatSelection}
\title{Determine the characteristic features of a cluster or selection}
\usage{
FeatSelection(
  List,
  Selection = NULL,
  binData = NULL,
  contData = NULL,
  datanames = NULL,
  nrclusters = NULL,
  topChar = NULL,
  sign = 0.05,
  fusionsLog = TRUE,
  weightclust = TRUE
)
}
\arguments{
\item{List}{A list of clustering outputs. Default is NULL.}

\item{Selection}{Either a character vector of objects, or a numeric cluster
number. Default is NULL.}

\item{binData}{A list of the binary feature data matrices. Default is NULL.}

\item{contData}{A list of continuous feature data matrices. Default is NULL.}

\item{datanames}{A vector with the names of the data matrices. Default is NULL.}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{topChar}{The number of top characteristics to return. Default is NULL.}

\item{sign}{The significance level. Default is 0.05.}

\item{fusionsLog}{Logical. Indicator for the fusion of clusters. Default is TRUE.}

\item{weightclust}{Logical. For the outputs of CEC, WeightedClust or
WeightedSimClust, if TRUE only the result of the Clust element is considered.
Default is TRUE.}
}
\value{
A list with the found (top) characteristics of the feature data.
}
\description{
\code{FeatSelection} determines the characteristic features of a selection of
objects or of a specific cluster across the methods. Binary features are
evaluated with Fisher's exact test, continuous features with the t-test.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(MCF7_F)
MCF7_FeatSel=FeatSelection(List=L,Selection=4,binData=list(fingerprintMat),
datanames=c("FP"),nrclusters=7,topChar=20)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregation.r
\name{HBGF}
\alias{HBGF}
\title{Hybrid Bipartite Graph Formulation}
\usage{
HBGF(
  List,
  type = c("data", "dist", "clust"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  clust = "agnes",
  linkage = c("flexible", "flexible"),
  alpha = 0.625,
  nrclusters = c(7, 7),
  gap = FALSE,
  maxK = 15,
  graphPartitioning = "Spec",
  optimalk = 7
)
}
\arguments{
\item{List}{A list of data matrices. It is assumed the rows are corresponding with the objects.}

\item{type}{Indicates whether the provided matrices in "List" are either data matrices, distance
matrices or clustering results obtained from the data. If type="dist" the calculation of the distance
matrices is skipped and if type="clust" the single source clustering is skipped.
Type should be one of "data", "dist" or "clust".}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".}

\item{nrclusters}{The number of clusters to divide each individual dendrogram in. Default is c(7,7) for two data sets.}

\item{gap}{Logical. Whether the optimal number of clusters should be determined with the gap statistic. Default is FALSE.}

\item{maxK}{The maximal number of clusters to investigate in the gap statistic. Default is 15.}

\item{graphPartitioning}{The graph partitioning method. Currently "Spec" (spectral) is supported.}

\item{optimalk}{An estimate of the final optimal number of clusters. Default is 7.}
}
\value{
The returned value is a list of two elements:
\item{DistM}{A NULL object}
\item{Clust}{The resulting clustering}
The value has class 'Ensemble'.
}
\description{
The \code{HBGF} function implements the Hybrid Bipartite Graph
Formulation, a graph-based consensus clustering method. A bipartite graph is
constructed from the cluster ensemble (objects on one side, clusters on the
other) and partitioned with a spectral method based on the singular value
decomposition followed by k-means.
}
\examples{
\dontrun{
data(mosaic_toy)
L <- mosaic_toy$List
HBGF_toy <- HBGF(List = L, type = "data",
                 distmeasure = c("euclidean", "euclidean"),
                 normalize = c(FALSE, FALSE), method = c(NULL, NULL),
                 clust = "agnes", linkage = c("ward", "ward"),
                 nrclusters = c(7, 7), graphPartitioning = "Spec", optimalk = 7)
}
}
\references{
Fern, X. Z. and Brodley, C. E. (2004). Solving cluster ensemble
problems by bipartite graph partitioning. Proceedings of the Twenty-First
International Conference on Machine Learning (ICML).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distance.R
\name{Distance}
\alias{Distance}
\alias{Distance_v2}
\title{Compute a distance matrix}
\usage{
Distance(
  Data,
  distmeasure = c("tanimoto", "jaccard", "euclidean", "hamming", "cont tanimoto",
    "MCA_coord", "gower", "chi.squared", "cosine"),
  normalize = NULL,
  method = NULL
)

Distance_v2(Data, distmeasure = "tanimoto", normalize = NULL)
}
\arguments{
\item{Data}{A data matrix; rows are the objects.}

\item{distmeasure}{One of \code{"tanimoto"}, \code{"jaccard"},
\code{"euclidean"} (Gower), \code{"hamming"}, \code{"cont tanimoto"},
\code{"MCA_coord"}, \code{"gower"}, \code{"chi.squared"}, \code{"cosine"}.}

\item{normalize}{Optional normalisation method (see \code{\link{Normalization}}),
applied for the \code{"euclidean"} / \code{"cont tanimoto"} measures.}

\item{method}{Ignored; retained for backward compatibility.}
}
\value{
A symmetric numeric distance matrix.
}
\description{
Dispatcher computing an object-by-object distance matrix under a range of
measures suitable for continuous, binary and mixed-type data.
}
\details{
\code{"cosine"}, \code{"chi.squared"} and \code{"MCA_coord"} require
the suggested packages \pkg{lsa}, \pkg{analogue} and \pkg{FactoMineR}
respectively.
}
\examples{
m <- matrix(rbinom(40, 1, 0.5), nrow = 8)
D <- Distance(m, distmeasure = "tanimoto")
dim(D)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/characterize.R
\name{characterize_clusters}
\alias{characterize_clusters}
\title{Characterise clusters with external (biological / clinical) variables}
\usage{
characterize_clusters(labels, metadata, continuous = NULL, categorical = NULL)
}
\arguments{
\item{labels}{Cluster label of each object (vector; \code{NA} allowed).}

\item{metadata}{A data frame of external variables, one row per object, in the
same order as \code{labels}.}

\item{continuous, categorical}{Optional character vectors naming the columns to
treat as continuous / categorical. If \code{NULL}, numeric columns are taken
as continuous and the rest as categorical.}
}
\value{
A list with \code{continuous} (per-cluster summary, long format),
\code{continuous_tests} (one row per variable: \code{KW_p}, \code{eta2},
\code{gradient} -- the clusters ordered by mean -- and a monotone-trend
\code{trend_rho}/\code{trend_p}), \code{categorical} (counts and column
percentages) and \code{categorical_tests} (\code{p}, \code{cramers_v}).
}
\description{
For a clustering, summarise how each external variable differs across the
clusters. Continuous variables get \eqn{n}, mean, SD, median, IQR and range
\emph{per cluster}, an overall Kruskal-Wallis test, a rank \eqn{\eta^2} effect
size, and \strong{gradient detection} (are the cluster means monotonically
ordered?). Categorical variables get per-cluster counts and column
percentages, a chi-squared / Fisher test and Cramer's V. The emphasis is on
effect sizes and descriptive summaries rather than p-values alone.
}
\examples{
data(mosaic_toy)
fit <- M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3)
cl  <- mosaic_labels(fit, 3)
meta <- data.frame(score = rnorm(length(cl)), grp = sample(c("A", "B"), length(cl), TRUE))
ch  <- characterize_clusters(cl, meta)
ch$continuous_tests
}
\seealso{
\code{\link{plot_cluster_profile}}, \code{\link{cluster_biology_panel}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/consensus.r
\name{WonM}
\alias{WonM}
\title{Weighting on Membership clustering}
\usage{
WonM(
  List,
  type = c("data", "dist", "clusters"),
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  nrclusters = list(seq(5, 25), seq(5, 25)),
  clust = "agnes",
  linkage = c("flexible", "flexible"),
  alpha = 0.625
)
}
\arguments{
\item{List}{A list of data matrices. It is assumed the rows are corresponding with the objects.}

\item{type}{Indicates whether the provided matrices in "List" are either data matrices ("data"),
distance matrices ("dist") or clustering results obtained from the data ("clusters").}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. More details on normalization in \code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.}

\item{nrclusters}{A list with, for each data matrix, the sequence of numbers of clusters to cut the dendrogram in. Defaults to list(seq(5,25), seq(5,25)).}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".}
}
\value{
The returned value is a list of two elements:
\item{DistM}{The resulting consensus dissimilarity matrix}
\item{Clust}{The resulting clustering}
The value has class 'WonM'.
}
\description{
Weighting on Membership (WonM) is a hierarchy-based consensus
technique. Each data matrix is hierarchically clustered and the resulting
dendrogram is cut into a range of K values. For every cut a co-membership
matrix is built, recording which objects fall in the same cluster. These
matrices are averaged within and across data sets to produce an overall
consensus matrix, which is transformed into a dissimilarity and clustered a
final time.
}
\examples{
\dontrun{
data(mosaic_toy)
fit <- WonM(List = mosaic_toy$List, type = "data",
  distmeasure = c("euclidean", "euclidean"), normalize = c(FALSE, FALSE),
  method = c(NULL, NULL), nrclusters = list(seq(3, 6), seq(3, 6)),
  clust = "agnes", linkage = c("ward", "ward"))
}
}
\references{
Saeed F., Salim N. and Abdo A. (2012). Voting-based consensus clustering for combining multiple clusterings of chemical structures. Journal of Cheminformatics, 4, 37.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-dendro.r
\name{ClusterPlot}
\alias{ClusterPlot}
\title{Colouring clusters in a dendrogram}
\usage{
ClusterPlot(
  Data1,
  Data2 = NULL,
  nrclusters = NULL,
  cols = NULL,
  colorComps = NULL,
  hangdend = 0.02,
  plottype = "new",
  location = NULL,
  ...
)
}
\arguments{
\item{Data1}{The resulting clustering of a method which contains the
dendrogram to be colored.}

\item{Data2}{Optional. The resulting clustering of another method , i.e. the
resulting clustering on which the colors should be based. Default is NULL.}

\item{nrclusters}{Optional. The number of clusters to cut the dendrogram in.
If not specified the dendrogram will be drawn without colours to discern the
different clusters. Default is NULL.}

\item{cols}{The colours for the clusters if nrclusters is specified. Default is NULL.}

\item{colorComps}{If only a specific set of objects needs to be
highlighted, this can be specified here. The objects should be given in a
character vector. If specified, all other compound labels will be colored
black. Default is NULL.}

\item{hangdend}{A specification for the length of the brances of the dendrogram. Default is 0.02.}

\item{plottype}{Should be one of "pdf","new" or "sweave". If "pdf", a
location should be provided in "location" and the figure is saved there. If
"new" a new graphic device is opened and if "sweave", the figure is made
compatible to appear in a sweave or knitr document. Default is "new".}

\item{location}{If plottype is "pdf", a location should be provided in
"location" and the figure is saved there. Default is NULL.}

\item{\dots}{Other options which can be given to the plot function.}
}
\value{
A plot of the dendrogram of the first clustering result with colored
leaves. If a second clustering result is given in Data2, the colors are
based on this clustering result.
}
\description{
Plot a dendrogram with leaves colored by a result of choice.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)
data(Colors1)

MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

ClusterPlot(MCF7_T ,nrclusters=7,cols=Colors1,plottype="new",location=NULL,
main="Clustering on Target Predictions: Dendrogram",ylim=c(-0.1,1.8))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{PathwaysIter}
\alias{PathwaysIter}
\title{Iterations of the pathway analysis}
\usage{
PathwaysIter(
  List,
  Selection = NULL,
  geneExpr = NULL,
  nrclusters = NULL,
  method = c("limma", "MLP"),
  geneInfo = NULL,
  geneSetSource = "GOBP",
  topP = NULL,
  topG = NULL,
  GENESET = NULL,
  sign = 0.05,
  niter = 10,
  fusionsLog = TRUE,
  weightclust = TRUE,
  names = NULL
)
}
\arguments{
\item{List}{A list of clustering outputs or output of the\code{DiffGenes}
function. The first element of the list will be used as the reference in
\code{ReorderToReference}. The output of \code{ChooseFeatures} is also
accepted.}

\item{Selection}{If pathway analysis should be conducted for a specific
selection of objects, this selection can be provided here. Selection can
be of the type "character" (names of the objects) or "numeric" (the number
of specific cluster). Default is NULL.}

\item{geneExpr}{The gene expression matrix of the objects. The rows should
correspond with the genes.}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{method}{The method to applied to look for differentially expressed genes and related pathways. For now, only the
limma method is available for gene analysis and the MLP method for pathway analysis. Default is c("limma","MLP").}

\item{geneInfo}{A data frame with at least the columns ENTREZID and SYMBOL.
This is necessary to connect the symbolic names of the genes with their
EntrezID in the correct order. The order of the gene is here not in the
order of the rownames of the gene expression matrix but in the order of
their significance. Default is NULL.}

\item{geneSetSource}{The source for the getGeneSets function ("GOBP",
"GOMF","GOCC", "KEGG" or "REACTOME"). Default is "GOBP".}

\item{topP}{Overrules sign. The number of pathways to display for each
cluster. If not specified, only the significant genes are shown. Default is NULL.}

\item{topG}{Overrules sign. The number of top genes to be returned in the
result. If not specified, only the significant genes are shown. Default is NULL.}

\item{GENESET}{Optional. Can provide own candidate gene sets. Default is NULL.}

\item{sign}{The significance level to be handled. Default is 0.05.}

\item{niter}{The number of times to perform pathway analysis. Default is 10.}

\item{fusionsLog}{Logical. To be handed to \code{ReorderToReference}: indicator for the fusion of clusters. Default is TRUE}

\item{weightclust}{Logical. To be handed to \code{ReorderToReference}: to be used for the outputs of CEC,
WeightedClust or WeightedSimClust. If TRUE, only the result of the Clust element is considered. Default is TRUE.}

\item{names}{Optional. Names of the methods. Default is NULL.}
}
\value{
A list with an element per iteration, each being the output of \code{Pathways}.
}
\description{
The MLP method to perform pathway analysis is based on resampling of the
data. Therefore it is recommended to perform the pathway analysis multiple
times to observe how much the results are influenced by a different
resample. The function \code{PathwaysIter} performs the pathway analysis as
described in \code{Pathways} a specified number of times. The input can be
one data set or a list as in \code{Pathway.2} and \code{Pathways}.
}
\details{
This function relies on the suggested Bioconductor packages 'MLP', 'biomaRt' and 'org.Hs.eg.db'.
}
\examples{

\dontrun{
data(fingerprintMat)
data(targetMat)
data(geneMat)
data(GeneInfo)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(MCF7_F,MCF7_T)
names=c('FP','TP')

MCF7_Paths_FandT=PathwaysIter(List=L, geneExpr = geneMat, nrclusters = 7, method =
c("limma", "MLP"), geneInfo = GeneInfo, geneSetSource = "GOBP", topP = NULL,
topG = NULL, GENESET = NULL, sign = 0.05,niter=2,fusionsLog = TRUE,
weightclust = TRUE, names =names)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/M_ABCpp.R
\name{f.clustABC.MultiSource}
\alias{f.clustABC.MultiSource}
\title{Consensus clustering from stacked ABC label matrices}
\usage{
f.clustABC.MultiSource(
  res,
  numclust = NULL,
  dissimilarity = "mabc",
  linkage = "ward",
  alpha = 0.625,
  mds = FALSE
)
}
\arguments{
\item{res}{Integer label matrix (\code{0} = object not selected).}

\item{numclust}{Optional number of clusters to cut the tree into.}

\item{dissimilarity}{\code{"mabc"} (arc-cosine, default) or \code{"abc"} (linear).}

\item{linkage}{AGNES agglomeration method.}

\item{alpha}{Flexible-linkage parameter.}

\item{mds}{Logical; draw an MDS of the dissimilarities.}
}
\value{
A list with \code{DistM}, \code{Clust} and (if \code{numclust} given)
\code{cut}.
}
\description{
Turns a row-stacked matrix of per-iteration cluster labels
(\eqn{K \cdot R} rows by \eqn{N} columns) into a co-clustering dissimilarity
and applies \code{cluster::agnes}. \eqn{S_{ij}} is the co-clustering
probability over iterations selecting both objects (B6). The co-clustering
counts use the compiled C++ kernel \code{count_coclustering_cpp}
(from \code{src/mcpp_sampling.cpp}) when available, otherwise a pure-R loop.
}
\references{
\insertCite{Amaratunga2008}{MosaiClusteR}
}
\seealso{
\code{\link{M_ABCpp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-dendro.r
\name{Cyclogram}
\alias{Cyclogram}
\title{Comparison of clustering results over multiple results in circular format}
\usage{
Cyclogram(
  List,
  nrclusters = NULL,
  cols = NULL,
  fusionsLog = FALSE,
  weightclust = FALSE,
  names = NULL,
  canvaslims = c(-1, 1, -1, 1),
  margins = c(8.1, 3.1, 3.1, 4.1),
  Highlight = NULL,
  cex.highlight = 1,
  substr = NULL,
  cex.labels = 0.7,
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{List}{A list of the outputs from the methods to be compared. The first
element of the list will be used as the reference in
\code{ReorderToReference}.}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{cols}{A character vector with the colours to be used. Default is NULL.}

\item{fusionsLog}{Logical. To be handed to \code{ReorderToReference}: indicator for the fusion of clusters. Default is TRUE}

\item{weightclust}{Logical. To be handed to \code{ReorderToReference}: to be used for the outputs of CEC,
WeightedClust or WeightedSimClust. If TRUE, only the result of the Clust element is considered. Default is TRUE.}

\item{names}{Optional. Names of the methods to be used as labels for the
columns. Default is NULL.}

\item{canvaslims}{The limits for the circular dendrogam. Default is c(-1.0,1.0,-1.0,1.0).}

\item{margins}{Optional. Margins to be used for the plot. Default is c(8.1,3.1,3.1,4.1).}

\item{Highlight}{Optional. A list of character vectors of objects to be highlighted. Default is NULL.}

\item{cex.highlight}{Magnification for the highlight text. Default is 1.}

\item{substr}{Optional. A vector of length two giving start and stop positions to shorten labels. Default is NULL.}

\item{cex.labels}{Magnification for the labels. Default is 0.7.}

\item{plottype}{Should be one of "pdf","new" or "sweave". If "pdf", a
location should be provided in "location" and the figure is saved there. If
"new" a new graphic device is opened and if "sweave", the figure is made
compatible to appear in a sweave or knitr document, i.e. no new device is
opened and the plot appears in the current device or document. Default is "new".}

\item{location}{Optional. If plottype is "pdf", a location should be provided in
"location" and the figure is saved there. Default is NULL.}
}
\value{
A plot which translates the matrix output of the function
\code{ReorderToReference} into a circular format.
}
\description{
A visual comparison of all methods is handy to see which objects will
always cluster together independent of the applied methods. To this aid the
function \code{Cyclogram} has been written. The function relies on methods
of the \code{circlize} package.
}
\details{
This function makes use of the functions \code{ReorderToReference} and
\code{Colorsnames}. Given a list with the outputs of several methods, the
first step is to call upon \code{ReorderToReference} and to produce a
matrix of which the columns are ordered according to the ordering of the
objects of the first method in the list. Each cell represent the number of
the cluster the object belongs to for a specific method indicated by the
rows. The clusters are arranged in such a way that these correspond to that
one cluster of the referenced method that they have the most in common with.
The \code{circlize} package is used to visualize the matrix is a circular format.
The inner element of the circle is the first element of the List parameter, the second inner
element is the second element of the list and so on. The object names are written on the
circular dendrogram portayed in the center of the circle.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-dendro.r
\name{ComparePlot}
\alias{ComparePlot}
\title{Comparison of clustering results over multiple methods}
\usage{
ComparePlot(
  List,
  nrclusters = NULL,
  cols = NULL,
  fusionsLog = FALSE,
  weightclust = FALSE,
  names = NULL,
  margins = c(8.1, 3.1, 3.1, 4.1),
  circle = FALSE,
  canvaslims = c(-1, 1, -1, 1),
  Highlight = NULL,
  substr = NULL,
  cex.highlight = 1,
  cex.labels = 0.7,
  trackheightdend = 0.5,
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{List}{A list of the outputs from the methods to be compared. The first
element of the list is used as the reference in \code{ReorderToReference}.}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{cols}{A character vector with the colours to be used. Default is NULL.}

\item{fusionsLog}{Logical. To be handed to \code{ReorderToReference}. Default is FALSE.}

\item{weightclust}{Logical. To be handed to \code{ReorderToReference}. Default is FALSE.}

\item{names}{Optional. Names of the methods to be used as labels. Default is NULL.}

\item{margins}{Optional. Margins for the plot. Default is c(8.1,3.1,3.1,4.1).}

\item{circle}{Logical. Whether the figure should be circular (TRUE) or a rectangle (FALSE). Default is FALSE.}

\item{canvaslims}{The limits for the circular dendrogram. Default is c(-1.0,1.0,-1.0,1.0).}

\item{Highlight}{Optional. A list of character vectors of objects to be highlighted. Default is NULL.}

\item{substr}{Optional. A vector of length two giving start and stop positions to shorten labels. Default is NULL.}

\item{cex.highlight}{Magnification for the highlight text. Default is 1.}

\item{cex.labels}{Magnification for the labels. Default is 0.7.}

\item{trackheightdend}{The height of the dendrogram track in the circular plot. Default is 0.5.}

\item{plottype}{Should be one of "pdf","new" or "sweave". Default is "new".}

\item{location}{Optional. If plottype is "pdf", a location should be provided. Default is NULL.}
}
\value{
A plot which translates the matrix output of \code{ReorderToReference}.
}
\description{
A visual comparison of the clustering results of several methods.
The function relies on \code{ReorderToReference} and \code{ColorsNames} and
renders the resulting matrix either as a rectangular heatmap (using the
\code{plotrix} package) or in a circular format (using the \code{circlize}
package).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adc.r
\name{ADEC}
\alias{ADEC}
\title{Aggregated data ensemble clustering}
\usage{
ADEC(
  List,
  distmeasure = "tanimoto",
  normalize = FALSE,
  method = NULL,
  t = 10,
  r = NULL,
  nrclusters = NULL,
  clust = "agnes",
  linkage = "flexible",
  alpha = 0.625
)
}
\arguments{
\item{List}{A list of data matrices of the same type. It is assumed the rows are corresponding with the objects.}

\item{distmeasure}{Choice of metric for the dissimilarity matrix (character). Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to "tanimoto".}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to FALSE. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is NULL.}

\item{t}{The number of iterations. Defaults to 10.}

\item{r}{The number of features to take for the random sample. If NULL (default), all features are considered.}

\item{nrclusters}{A sequence of numbers of clusters to cut the dendrogram in. If NULL (default), the function stops.}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character). Defaults to "flexible".}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible".}
}
\value{
The returned value is a list with the following three elements.
\item{AllData}{Fused data matrix of the data matrices}
\item{DistM}{The resulting co-association matrix}
\item{Clust}{The resulting clustering}
The value has class 'ADEC'. The Clust element will be of interest for further applications.
}
\description{
Aggregated Data Ensemble Clustering (ADEC) is a direct clustering multi-source technique. ADEC is an iterative procedure which starts with the merging of the data sets. In each iteration, a random sample of the features is selected and/or a resulting dendrogram is divided into k clusters for a range of values of k.
}
\details{
If r is specified and nrclusters is a fixed number, only a random sampling of the features will be performed for the t iterations (ADECa). If r is NULL and the nrclusters is a sequence, the clustering is performed on all features and the dendrogam is divided into clusters for the values of nrclusters (ADECb). If both r is specified and nrclusters is a sequence, the combination is performed (ADECc).
After every iteration, either be random sampling, multiple divisions of the dendrogram or both, an incidence matrix is set up. All incidence matrices are summed and represent the distance matrix on which a final clustering is performed.
}
\examples{
\dontrun{
data(mosaic_toy)
L <- mosaic_toy$List
res_ADEC <- ADEC(List=L, distmeasure="euclidean", normalize=FALSE, method=NULL,
                 t=5, r=50, nrclusters=5, clust="agnes", linkage="flexible", alpha=0.625)
}
}
\references{
Fodeh, S. J., Punch, W. & Tan, P.-N. (2013). On unifying multi-source information from probabilistically generated functional data. Proteins 81:1298-1310.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-selection.r
\name{SimilarityMeasure}
\alias{SimilarityMeasure}
\title{Similarity of a set of clusterings to a reference}
\usage{
SimilarityMeasure(
  List,
  nrclusters = NULL,
  fusionsLog = TRUE,
  weightclust = TRUE,
  names = NULL
)
}
\arguments{
\item{List}{A label matrix (rows are clusterings, columns are objects) or a
list of clustering outputs to be aligned via \code{ReorderToReference}.}

\item{nrclusters}{Number of clusters used when \code{List} is a list. Default
\code{NULL}.}

\item{fusionsLog, weightclust}{Logical controls passed to
\code{ReorderToReference} when \code{List} is a list.}

\item{names}{Optional method names passed to \code{ReorderToReference}.}
}
\value{
A numeric vector of similarities in \code{[0, 1]}, one per row, where
the first entry is the reference compared to itself (always \code{1}).
}
\description{
Given a colour/label matrix (clusterings in rows, objects in columns) or a
list of clustering outputs, the proportion of objects sharing the label of
the first (reference) row is computed for every row. When a list is supplied
it is first aligned with \code{ReorderToReference}; supplying the label
matrix directly avoids that dependency.
}
\examples{
\dontrun{
M <- rbind(c(1, 1, 2, 2, 3, 3),
           c(1, 1, 2, 3, 3, 3),
           c(2, 2, 1, 1, 3, 3))
SimilarityMeasure(M)
}
}
\references{
Fodeh, S. J., Brandt, C., Luong, T. B., Haddad, A., Schultz, M., Murphy, T.
and Krauthammer, M. (2013). Complementary ensemble clustering of biomedical
data. Journal of Biomedical Informatics, 46(3), 436-443.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster.R
\name{Cluster}
\alias{Cluster}
\title{Single-source base clustering}
\usage{
Cluster(
  Data,
  type = c("data", "dist"),
  distmeasure = "euclidean",
  normalize = FALSE,
  method = NULL,
  clust = "agnes",
  linkage = "ward",
  alpha = 0.625,
  StopRange = TRUE,
  ...
)
}
\arguments{
\item{Data}{A data matrix (objects in rows) or a precomputed distance matrix.}

\item{type}{\code{"data"} to compute a distance first, or \code{"dist"} if
\code{Data} is already a dissimilarity matrix.}

\item{distmeasure}{Distance measure (see \code{\link{Distance}}), used when
\code{type = "data"}.}

\item{normalize, method}{Normalisation controls passed to \code{\link{Distance}}.}

\item{clust}{Clustering backend; currently \code{"agnes"}.}

\item{linkage}{Agglomeration method. Default \code{"ward"}.}

\item{alpha}{Flexible-linkage parameter for \code{cluster::agnes}.}

\item{StopRange}{Logical; if \code{FALSE} and the dissimilarities fall outside
\code{[0, 1]} they are range-normalised for comparability.}

\item{...}{Ignored; accepted for backward compatibility (e.g. \code{gap},
\code{maxK}).}
}
\value{
A list with \code{DistM} (the dissimilarity matrix) and \code{Clust}
(an \code{agnes} object).
}
\description{
Cluster one data (or distance) matrix with agglomerative hierarchical
clustering. This is the Layer-2 baseline primitive of the MoSaIC framework:
run it on each tile on its own to see what each modality recovers before
integrating. It also underpins \code{\link{HierarchicalEnsembleClustering}}.
}
\examples{
data(mosaic_toy)
base1 <- Cluster(t(mosaic_toy$List[[1]]), type = "data",
                 distmeasure = "euclidean")
mosaic_labels(base1, k = 3)[1:6]
}
\seealso{
\code{\link{Distance}}, \code{\link{M_ABCpp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manuscript-viz.R
\name{compare_methods_plot}
\alias{compare_methods_plot}
\title{Visually compare clustering solutions with ComparePlot (no ground truth)}
\usage{
compare_methods_plot(
  solutions,
  nrclusters,
  cols = NULL,
  cex.labels = 0.5,
  margins = c(5, 11, 3, 2),
  ...
)
}
\arguments{
\item{solutions}{A named list of fitted results and/or label vectors, all over
the same objects in the same order.}

\item{nrclusters}{Number of clusters to cut each solution into.}

\item{cols}{Optional cluster colours; defaults to a distinct palette.}

\item{cex.labels, margins}{Passed to \code{\link{ComparePlot}}.}

\item{...}{Further arguments to \code{\link{ComparePlot}}.}
}
\value{
Invisibly \code{NULL}; called for the plot. Needs \pkg{circlize} and
\pkg{plotrix}.
}
\description{
Aligns several clustering solutions of the same objects and draws them with
\code{\link{ComparePlot}} so their agreement can be read off directly -- which
objects the methods place together and where they disagree -- without any
ground truth. Each solution may be a fitted result (with \code{$Clust} /
\code{$DistM}) or a plain label vector; label vectors are turned into a
co-membership dissimilarity so every solution is comparable.
}
\examples{
data(mosaic_toy)
sols <- list(
  var    = M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3, weighting = c("var", "var")),
  nugget = M_ABCpp(mosaic_toy$List, numsim = 40, NC = 3, weighting = c("nugget", "nugget")),
  NEMO   = NEMO(mosaic_toy$List, k = 3)$cluster)
\dontrun{compare_methods_plot(sols, nrclusters = 3)}
}
\seealso{
\code{\link{clustering_concordance}}, \code{\link{ComparePlot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-characterize.r
\name{CharacteristicFeatures}
\alias{CharacteristicFeatures}
\title{Determine the characteristic features of clusters}
\usage{
CharacteristicFeatures(
  List,
  Selection = NULL,
  binData = NULL,
  contData = NULL,
  datanames = NULL,
  nrclusters = NULL,
  sign = 0.05,
  topChar = NULL,
  fusionsLog = TRUE,
  weightclust = TRUE,
  names = NULL
)
}
\arguments{
\item{List}{A list of clustering outputs to be compared. The first element of
the list is used as the reference in \code{ReorderToReference}.}

\item{Selection}{If a selection of objects is provided, the function only
investigates the features of this selection. Default is NULL.}

\item{binData}{A list of the binary feature data matrices. Default is NULL.}

\item{contData}{A list of continuous feature data matrices. Default is NULL.}

\item{datanames}{A vector with the names of the data matrices. Default is NULL.}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{sign}{The significance level. Default is 0.05.}

\item{topChar}{The number of top characteristics to return. If NULL, only the
significant characteristics are saved. Default is NULL.}

\item{fusionsLog}{Logical. Indicator for the fusion of clusters. Default is TRUE.}

\item{weightclust}{Logical. For the outputs of CEC, WeightedClust or
WeightedSimClust, if TRUE only the result of the Clust element is considered.
Default is TRUE.}

\item{names}{Optional. Names of the methods. Default is NULL.}
}
\value{
A list with for each method the found (top) characteristics of the
feature data per cluster.
}
\description{
\code{CharacteristicFeatures} identifies the features that are characteristic
for the objects of each cluster of one or more clustering results. Binary
features are evaluated with Fisher's exact test, continuous features with the
t-test. Multiplicity correction is included.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(MCF7_F,MCF7_T)
MCF7_Char=CharacteristicFeatures(List=L,Selection=NULL,binData=list(fingerprintMat),
contData=NULL,datanames=c("FP"),nrclusters=7,topChar=20)
}
}
\references{
Perualila-Tan N. et al. (2016). Weighted similarity-based clustering
of chemical structures and bioactivity data in early drug discovery.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-features.r
\name{PlotPathways}
\alias{PlotPathways}
\title{A GO plot of a pathway analysis output.}
\usage{
PlotPathways(
  Pathways,
  nRow = 5,
  main = NULL,
  plottype = "new",
  location = NULL
)
}
\arguments{
\item{Pathways}{One element of the output list returned by
\code{PathwayAnalysis} or \code{Geneset.intersect}.}

\item{nRow}{Number of GO IDs for which to produce the plot. Default is 5.}

\item{main}{Title of the plot. Default is NULL.}

\item{plottype}{Should be one of "pdf","new" or "sweave". If "pdf", a
location should be provided in "location" and the figure is saved there. If
"new" a new graphic device is opened and if "sweave", the figure is made
compatible to appear in a sweave or knitr document. Default is "new".}

\item{location}{If plottype is "pdf", a location should be provided in
"location" and the figure is saved there. Default is NULL.}
}
\value{
The output is a GO graph.
}
\description{
The \code{PlotPathways} function takes an output of the
\code{PathwayAnalysis} function and plots a GO graph with the help of the
\code{plotGOgraph} function of the MLP package.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)
data(geneMat)
data(GeneInfo)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(MCF7_F,MCF7_T)
names=c('FP','TP')

MCF7_PathsFandT=PathwayAnalysis(List=L, geneExpr = geneMat, nrclusters = 7, method = c("limma",
"MLP"), geneInfo = GeneInfo, geneSetSource = "GOBP", topP = NULL,
topG = NULL, GENESET = NULL, sign = 0.05,niter=2,fusionsLog = TRUE, weightclust = TRUE,
 names =names,seperatetables=FALSE,separatepvals=FALSE)

PlotPathways(MCF7_PathsFandT$FP$"Cluster 1"$Pathways,nRow=5,main=NULL)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-sim.R
\name{mosaic_sim}
\alias{mosaic_sim}
\title{Simulate a multi-source data set with planted clusters}
\usage{
mosaic_sim(
  n = 60,
  g = 3,
  p = c(120, 150),
  informative = c(20, 25),
  effect = 2.5,
  K = 2,
  seed = NULL
)
}
\arguments{
\item{n}{Number of samples (objects).}

\item{g}{Number of ground-truth groups.}

\item{p}{Per-source feature counts (recycled to length \code{K}).}

\item{informative}{Per-source number of group-informative features.}

\item{effect}{Between-group mean shift for informative features.}

\item{K}{Number of sources.}

\item{seed}{Optional RNG seed.}
}
\value{
A list with \code{List} (the \code{K} matrices, \strong{objects in
rows}, features in columns) and \code{truth} (the integer group label per
object).
}
\description{
Generates \code{K} feature-by-sample matrices that share a common set of
\code{n} samples drawn from \code{g} ground-truth groups. Each source has its
own informative features (shifted between groups) plus noise features, so the
clusters are only partially visible in any single source - the canonical
setting in which multi-source integration helps.
}
\examples{
sim <- mosaic_sim(n = 30, g = 3, K = 2, seed = 1)
lengths(lapply(sim$List, dim))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{DiffGenesSelection}
\alias{DiffGenesSelection}
\title{Differential expression for a selection of objects}
\usage{
DiffGenesSelection(
  List,
  Selection,
  geneExpr = NULL,
  nrclusters = NULL,
  method = "limma",
  sign = 0.05,
  topG = NULL,
  fusionsLog = TRUE,
  weightclust = TRUE,
  names = NULL
)
}
\arguments{
\item{List}{A list of the clustering outputs to be compared. The first
element of the list will be used as the reference in
\code{ReorderToReference}.}

\item{Selection}{If differential gene expression should be investigated for
a specific selection of objects, this selection can be provided here.
Selection can be of the type "character" (names of the objects) or
"numeric" (the number of specific cluster). Default is NULL.}

\item{geneExpr}{The gene expression matrix or ExpressionSet of the objects.
The rows should correspond with the genes.}

\item{nrclusters}{Optional. The number of clusters to cut the dendrogram in.
The number of clusters should not be specified if the interest lies only in
a specific selection of objects which is known by name.  Otherwise, it is
required. Default is NULL.}

\item{method}{The method to applied to look for DE genes. For now, only the
limma method is available. Default is "limma".}

\item{sign}{The significance level to be handled. Default is 0.05.}

\item{topG}{Overrules sign. The number of top genes to be shown. Default is NULL.}

\item{fusionsLog}{Logical. To be handed to \code{ReorderToReference}: indicator for the fusion of clusters. Default is TRUE}

\item{weightclust}{Logical. To be handed to \code{ReorderToReference}: to be used for the outputs of CEC,
WeightedClust or WeightedSimClust. If TRUE, only the result of the Clust element is considered. Default is TRUE.}

\item{names}{Optional. Names of the methods. Default is NULL.}
}
\description{
Internal function of \code{DiffGenes}.
}
\examples{
\dontrun{
MCF7_FT_DE = DiffGenes(List=L,Selection=c("Cpd1","Cpd2"),geneExpr=geneMat,method="limma")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cec.r
\name{CEC}
\alias{CEC}
\title{Complementary ensemble clustering}
\usage{
CEC(
  List,
  distmeasure = c("tanimoto", "tanimoto"),
  normalize = c(FALSE, FALSE),
  method = c(NULL, NULL),
  t = 10,
  r = NULL,
  nrclusters = NULL,
  weight = NULL,
  clust = "agnes",
  linkage = c("flexible", "flexible"),
  alpha = 0.625,
  weightclust = 0.5
)
}
\arguments{
\item{List}{A list of data matrices. It is assumed the rows are corresponding with the objects.}

\item{distmeasure}{A vector of the distance measures to be used on each data matrix. Should be one of "tanimoto", "euclidean", "jaccard", "hamming". Defaults to c("tanimoto","tanimoto").}

\item{normalize}{Logical. Indicates whether to normalize the distance matrices or not, defaults to c(FALSE, FALSE) for two data sets. This is recommended if different distance types are used. More details on normalization in \code{Normalization}.}

\item{method}{A method of normalization. Should be one of "Quantile","Fisher-Yates", "standardize","Range" or any of the first letters of these names. Default is c(NULL,NULL) for two data sets.}

\item{t}{The number of iterations. Defaults to 10.}

\item{r}{A vector with the number of features to take for the random sample for each element in List. If NULL (default), all features are considered.}

\item{nrclusters}{A list with a sequence of numbers of clusters to cut the dendrogram in for each element in List. If NULL (default), the function stops.}

\item{weight}{The weights for the weighted linear combination.}

\item{clust}{Choice of clustering function (character). Defaults to "agnes".}

\item{linkage}{Choice of inter group dissimilarity (character) for each data set. Defaults to c("flexible", "flexible") for two data sets.}

\item{alpha}{The parameter alpha to be used in the "flexible" linkage of the agnes function. Defaults to 0.625 and is only used if the linkage is set to "flexible"}

\item{weightclust}{A weight for which the result will be put aside of the other results. This was done for comparative reason and easy access.}
}
\value{
The returned value is a list of four elements:
\item{DistM}{The resulting incidence matrix}
\item{Results}{The hierarchical clustering result for each element in WeightedDist}
\item{Clust}{The result for the weight specified in Clustweight}
The value has class 'CEC'.
}
\description{
Complementary Ensemble Clustering (CEC) Complementary Ensemble Clustering (CEC, \cite{Fodeh2013}) shows similarities with ADEC.
However, instead of merging the data matrices, ensemble clustering is performedon each data matrix separately. The resulting incidence
matrices for each data sets are combined in a weighted linear equation. The weighted incidence matrix is the input for the final clustering
algorithm. Similarly as ADEC, there are versions depending of the specification of the number of features to sample and the number of clusters.
}
\details{
If r is specified and nrclusters is a fixed number, only a random sampling of the features will be performed for the t iterations (CECa). If r is NULL and the nrclusters is a sequence, the clustering is performedon all features and the dendrogam is divided into clusters for the values of nrclusters (CECb). If both r is specified and nrclusters is a sequence, the combination is performed (CECc).
After every iteration, either be random sampling, multiple divisions of the dendrogram or both, an incidence matrix is set up. All incidence matrices are summed and represent the distance matrix on which a final clustering is performed.
}
\references{
\insertRef{Fodeh2013}{MosaiClusteR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ABCdist.R
\name{ABCdist.SingleInMultiple}
\alias{ABCdist.SingleInMultiple}
\title{Single-source ABC with distance accumulation}
\usage{
ABCdist.SingleInMultiple(
  data,
  distmeasure = "euclidean",
  weighting = "var",
  normalize = NULL,
  gr = c(),
  bag = TRUE,
  numsim = 1000,
  numvar = 100,
  linkage = "ward.D",
  NC = NULL,
  mds = FALSE,
  nfeat = NULL,
  nugget_type = "between",
  nugget_args = list()
)
}
\arguments{
\item{data}{Numeric matrix with \strong{objects (samples) in rows} and
features in columns (the MosaiClusteR convention; transposed internally to
the features-by-objects layout of the original engine).}

\item{distmeasure}{Distance measure passed to \code{\link{Distance}}.}

\item{weighting}{\code{"var"}, \code{"cv"}, \code{"nugget"} or equal.}

\item{normalize}{Optional normalisation (see \code{\link{Normalization}}).}

\item{gr}{Unused; kept for interface compatibility.}

\item{bag}{Logical; bootstrap the objects each iteration (Step 2).}

\item{numsim}{Number of iterations \eqn{R}.}

\item{numvar}{Kept for backward compatibility; superseded by \code{nfeat}.}

\item{linkage}{Agglomeration method for \code{fastcluster::hclust}.}

\item{NC}{Base-cluster count (\code{NULL} = \eqn{\lfloor\sqrt{N}\rfloor}).}

\item{mds}{Unused placeholder.}

\item{nfeat}{Optional feature-subsample size. \code{NULL} (default) uses the
classic \eqn{g=\lfloor\sqrt{G}\rfloor}; a value \eqn{\ge 1} sets an absolute
count (raise it for low-dimensional sources where most features matter).}

\item{nugget_type, nugget_args}{Passed to the data-nugget weighting when
\code{weighting = "nugget"}.}
}
\value{
A list with \code{D} (\eqn{N \times N} dissimilarity), \code{NC} and
\code{ng} (per-iteration feature count); if \code{mds = TRUE} also
\code{mds_coords}.
}
\description{
A variant of \code{\link{ABCpp.SingleInMultiple}} that accumulates the mean
scaled pairwise distance between co-selected objects across iterations,
yielding a \code{[0, 1]} dissimilarity matrix that \code{\link{M_ABCdist}}
fuses across sources.
}
\examples{
data(mosaic_toy)
out <- ABCdist.SingleInMultiple(mosaic_toy$List[[1]], numsim = 40)
dim(out$D)
}
\references{
\insertCite{Amaratunga2008}{MosaiClusteR}
}
\seealso{
\code{\link{M_ABCdist}}, \code{\link{ABCpp.SingleInMultiple}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{PathwaysSelection}
\alias{PathwaysSelection}
\title{Pathway analysis for a selection of objects}
\usage{
PathwaysSelection(
  List = NULL,
  Selection,
  geneExpr = NULL,
  nrclusters = NULL,
  method = c("limma", "MLP"),
  geneInfo = NULL,
  geneSetSource = "GOBP",
  topP = NULL,
  topG = NULL,
  GENESET = NULL,
  sign = 0.05,
  fusionsLog = TRUE,
  weightclust = TRUE,
  names = NULL
)
}
\arguments{
\item{List}{A list of clustering outputs or output of the\code{DiffGenes}
function. The first element of the list will be used as the reference in
\code{ReorderToReference}. The output of \code{ChooseFeatures} is also
accepted.}

\item{Selection}{If pathway analysis should be conducted for a specific
selection of objects, this selection can be provided here. Selection can
be of the type "character" (names of the objects) or "numeric" (the number
of specific cluster). Default is NULL.}

\item{geneExpr}{The gene expression matrix or ExpressionSet of the objects.
The rows should correspond with the genes.}

\item{nrclusters}{Optional. The number of clusters to cut the dendrogram in.
The number of clusters should not be specified if the interest lies only in
a specific selection of objects which is known by name.  Otherwise, it is
required. Default is NULL.}

\item{method}{The method to applied to look for differentially expressed genes and related pathways. For now, only the
limma method is available for gene analysis and the MLP method for pathway analysis. Default is c("limma","MLP").}

\item{geneInfo}{A data frame with at least the columns ENTREZID and SYMBOL.
This is necessary to connect the symbolic names of the genes with their
EntrezID in the correct order. The order of the gene is here not in the
order of the rownames of the gene expression matrix but in the order of
their significance. Default is NULL.}

\item{geneSetSource}{The source for the getGeneSets function, defaults to
"GOBP".}

\item{topP}{Overrules sign. The number of pathways to display for each
cluster. If not specified, only the significant genes are shown. Default is NULL.}

\item{topG}{Overrules sign. The number of top genes to be returned in the
result. If not specified, only the significant genes are shown. Defaults is NULL.}

\item{GENESET}{Optional. Can provide own candidate gene sets. Default is NULL.}

\item{sign}{The significance level to be handled. Default is 0.05.}

\item{fusionsLog}{Logical. To be handed to \code{ReorderToReference}: indicator for the fusion of clusters. Default is TRUE}

\item{weightclust}{Logical. To be handed to \code{ReorderToReference}: to be used for the outputs of CEC,
WeightedClust or WeightedSimClust. If TRUE, only the result of the Clust element is considered. Default is TRUE.}

\item{names}{Optional. Names of the methods. Default is NULL.}
}
\description{
Internal function of \code{Pathways}.
}
\details{
This function relies on the suggested Bioconductor packages 'MLP', 'biomaRt' and 'org.Hs.eg.db'.
}
\examples{
\dontrun{
PathwaysSelection(List=L,Selection=c("Cpd1","Cpd2"),geneExpr=geneMat,geneInfo=GeneInfo)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathways.r
\name{SharedGenesPathsFeat}
\alias{SharedGenesPathsFeat}
\title{Shared genes, pathways and features across methods}
\usage{
SharedGenesPathsFeat(
  DataLimma = NULL,
  DataMLP = NULL,
  DataFeat = NULL,
  names = NULL,
  Selection = FALSE
)
}
\arguments{
\item{DataLimma}{Optional. The output of a \code{DiffGenes} function. Default is NULL.}

\item{DataMLP}{Optional. The output of \code{Geneset.intersect} function. Default is NULL.}

\item{DataFeat}{Optional. The output of \code{CharacteristicFeatures} function. Default is NULL.}

\item{names}{Optional. Names of the methods. Default is NULL.}

\item{Selection}{Logical. Whether a selection of objects was considered. Default is FALSE.}
}
\value{
A list with a Table summarizing the shared elements and a Which
element detailing the shared genes, pathways and features per cluster.
}
\description{
The \code{SharedGenesPathsFeat} function takes the intersection
of the differentially expressed genes, the pathways and the characteristic
features over the methods per cluster.
}
\examples{
\dontrun{
SharedGenesPathsFeat(DataLimma=MCF7_FT_DE,DataMLP=MCF7_Paths_intersection,names=c("FP","TP"))
}
}
